/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/


#ifndef COMMON_H
#define	COMMON_H

#include "geometry.h"

extern char neighbor_Offset[8][2]; 

extern unsigned char C_which_bit[8];
extern unsigned char C_not_bit[8];
extern unsigned char C_start_bits[8];
extern unsigned char C_end_bits[8];

#define BUF_NON_EMPTY 0x00000001
#define BUF_VERSION_NUM_USED 0x00000002
#define BUF_2D 0x00000004
#define BUF_VALUE_FLOAT 0x00000008
#define BUF_VALUE_DOUBLE 0x00000010

#define BUF_MIN_MAX_COMPUTED 0x00000080
#define BUF_MIN_MAX_SET 0x00000100
#define BUF_DATA_VALID  0x00000200
#define BUF_DATA_GRADIENT 0x00000400
#define BUF_MEAN_VALUE_COMPUTED 0x00000800
#define BUF_MEAN_VALUE_SET 0x00001000
#define BUF_OVERWRITE 0x00002000
#define BUF_MODIFIED 0x00004000
#define BUF_VALUE_SIGNED 0x00008000
#define BUF_USTARTXY_VALID 0x01000000
#define BUF_UNITXY_VALID 0x02000000
#define BUF_MATRIX_VALID 0x10000000

#define BUF_FLUSH_MIN_MAX_STATUS 0x0000fe7f
#define BUF_FLUSH_MEAN_STATUS 0x0000e7ff
#define BUF_DATA_FLUSH_GRADIENT 0x0000fbff
#define BUF_FLUSH_DATA_STATUS 0x0000807f
#define BUF_VALUE_TYPE 0x00008018
#define BUF_PIXEL_SIGNED 0x00008018
#define BUF_PIXEL_FLOAT_OR_DOUBLE 0x00000018


struct unit_2d_rec
   {
#define UNIT_2D_IN_USE 1
    unsigned short status2;
    float ustartx,ustarty;
    float unitx,unity; /* physical unit measurement for dimx & dimy */
    unsigned long dimx,dimy;  /* [0,dimx - 1] & [0,dimy - 1] are arges of
                                x & y coordinates */
    float local_ustartx,local_ustarty;
    float local_unitx,local_unity; 
    struct matrix_2d_rec tolocal_mat,localto_mat;
   };

struct buf_2d_rec
   {
    unsigned long status4; 
            /* BUF_NON_EMPTY if buf is not NULL
               BUF_2D
                  buf is 2 dimensional....
               BUF_MIN_MAX_COMPUTED
                  if min & max are computed
               BUF_OWN_NON_CONTIGUOUS_MEMORY
                 if **buf are pointers to some rows of non-contiguous memory
                  locations that are owned by **buf
                       ...referenced by buf[0..dimy-1]...
            */
    long startoffx,startoffy;  /* actual start of x & y...that is all
                       indices of x & y are subtracted from [startx,starty],
                       while the ranges of coordiantes described immediately 
                       below are still the same.  The default initial values
                       of startx & starty are 0,
                      */
    unsigned long dimx,dimy;  /* [0,dimx - 1] & [0,dimy - 1] are arges of
                                x & y coordinates */

    unsigned long total_bytes; /* total # of bytes used */
    unsigned long pixel_depth; /* can be 0,1, 2, or 4 bytes
                                if pixel_depth == 0 then **buf is a bitmap */
    double minval,maxval;
    double minval_set,maxval_set;
    double meanval,meanval_set;

    unsigned short version_num;
    float ustartx,ustarty; /* physical unit measurement for startx & starty */
    float unitx,unity; /* physical unit measurement for dimx & dimy */
    struct matrix_2d_rec matrix;

#define USE_MIN_MAX_SET 0x0001
#define USE_MIN_MAX_COMPUTED 0x0002
#define USE_MIN_MAX_RESET 0xfffc
#define BUF_DATA_NEW 0x0004
#define BUF_DATA_MMAP 0x0008
#define BUF_OWN_CONTIGUOUS_MEMORY 0x0040
#define BUF_OWN_NON_CONTIGUOUS_MEMORY 0x0080
#define BUF_FLUSH_MEMORY_STATUS 0xff0f
#define BUF_AREA_STATUS 0x00c0

    unsigned short prog_status2; 
    unsigned char **buf; 
    double (*GetVal)();
    int (*SetVal)();
   };

struct mask_rec
   {
    unsigned long total_pixels;   /* total # of pixels whose values are not 0 */
    struct buf_2d_rec _2d;
   };

#define BUF_2D_VERSION_NUM 1
#endif COMMON_H
