/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

#ifndef INPUT_H
#define	INPUT_H

#include <stdio.h>
#include "slice_roi_rec.h"
#include "buf_3d_rec.h"
#include "geometry.h"
#include "common.h"
#include "list_rec.h"
#include "ia_hdr.h"
#include "str_def.h"
#include "text_format.h"
#include "count_analysis_rec.h"
#include "pixel_desc.h"
#include "output.h"
#include "intensity_rec.h"
#include "display_win.h"

#include <xview/xview.h>

#define METHOD_BLEND_SUPERIMPOSE 1 
#define METHOD_BLEND_3_BY_3 2
#define METHOD_BLEND_6_BY_6 4
#define METHOD_BLEND_9_BY_9 8
#define METHOD_BLEND_AVE_OF_ADD 0x00000010
#define METHOD_UNSELECTED_IS_DEST 0x00000020
#define METHOD_BY_EDGE_DETECTOR 0x00000040
#define METHOD_BLEND_SUB 0x00000080
#define METHOD_USE_BITMAP 0x00000200
#define FUSION_MUST_RECOMPUTE 0x00000400
#define FUSION_SHOW_ONLY_SRC 0x00000800
#define FUSION_DYNAMIC 0x00001000
#define FUSION_SRC_MINMAX_DEFINED 0x00002000
#define FUSION_DST_MINMAX_DEFINED 0x00004000
#define FUSION_ACCUMULATED 0x00008000
#define FUSION_MOVE_UPDATE 0x00010000
#define FUSION_MOVE_WHOLE 0x00020000
#define FUSION_MOVE_CLEARED 0x00040000
#define FUSION_MOVE_PERFORMED 0x00080000
#define FUSION_MOVE_IGNORED_EVENT_UP 0x00100000
#define FUSION_LAST_LEFT_TOP_VALID 0x00200000
#define FUSION_SYNC 0x00400000
#define FUSION_FORWARD_READY 0x00800000
#define FUSION_BACKWARD_READY 0x01000000
#define METHOD_SRC_DEST_EQUAL_SIZE 0x02000000
#define FUSION_JUST_PERFORMED 0x04000000

#define FUSION_MOVE_SELF 0x20000000

#define FUSION_METHOD 0x01ff
#define FUSION_NEW_CLEAR_STATUS 0x01df0fff
#define FUSION_NEXT_CLEAR_STATUS 0x01d60C00
#define FUSION_CLEAR_SYNC 0x01c00000
struct sel_buf_2d_rec
   {
    unsigned long status;
    struct int_bound_rec src_bound,dst_bound,src_screen_bound;
    long last_dst_left,last_dst_top,def_dst_left,def_dst_top;
    unsigned long dstmin,dstmax;
    unsigned long srcmin,srcmax;
    struct pixel_desc_rec pixel;
    XImage *im;
    struct unit_2d_rec src_trans_unit;
    struct color_screen_rec src,trans_src;
    struct buf_2d_rec bitmask,screen_2d,_2d,srcindst_2d;
   };

/*
 possible values for img->status
*/
#define IMAGE_USE_SLICE_MIN_MAX 0x00000001
#define IMAGE_USE_SET_MIN_MAX 0x00000002
#define IMAGE_NEED_DISPLAY 0x00000004
#define IMAGE_3D_SUPERIMPOSED 0x00000008
#define IMAGE_3D_SUPERIMPOSED_READY 0x00000010
#define IMG_REDEFINE_FUSION_PARAMS 0x00000020
#define IMAGE_2D_SUPERIMPOSED_READY 0x00000040
#define IMAGE_2D_SUPERIMPOSED_READY_SAVED 0x00000080
#define IMAGE_2D_CHANGED 0x00000100
#define IMAGE_POS_ACTIVATION 0x00000200
#define IMAGE_NEG_ACTIVATION 0x00000400
#define IMAGE_3D_SUPERIMPOSED_FILLED 0x00000800

#define IMG_HISTO_1D 0x00002000
#define IMG_HISTO_2D 0x00004000
#define IMG_HISTO_AUTO_SET_X_BACKGROUND 0x00008000
#define IMG_HISTO_AUTO_SET_Y_BACKGROUND 0x00010000

#define IMG_HISTO_INITIALIZE_BUF 0x00020000
#define IMG_HISTO_1D_DO_NOT_SETUP_HISTOGRAM 0x00040000
#define IMG_HISTO_1D_SHOW_EACH_HISTOGRAM 0x00080000
#define IMG_HISTO_1D_DISABLE_AUTO_SET 0x00100000

#define IMG_MOVE_2D_VALID 0x00400000

#define IMAGE_3D_SUPERIMPOSED_OPRS 0x00000008
#define IMAGE_USE_MIN_MAX 0x00000003

#define IMG_HISTO_AUTO_SET 0x00018000
#define IMAGE_POS_NEG_ACTIVATION 0x00000600

struct img_buf_rec
   {
    unsigned long status4;
    struct input_file_rec *file;
    struct hdr_scan_entry_rec *previous_slice,*last_slice,*cur_slice;

    struct unit_2d_rec last_unit,unit;
    struct buf_2d_rec _2d,move_2d;

    unsigned long total_dst_wids,num_dst_wids;
    char **dst_wids,*src_wid;
    struct sel_buf_2d_rec _2d_fusion;
    struct sel_buf_2d_rec _3d_fusion;

    struct win_rec *win;
    struct img_buf_rec *partner;
   };

struct img_rec
   {
    struct input_file_rec *file;
    unsigned int num_data;
    struct img_buf_rec **data;
   };

/* each image which may consist of more than 1 slice is assumed to have a
      slice_roi_rec for outlining the rois.  Its associated slices share
      the same slice_roi_rec... */
struct slice_roi_win_rec
   {
    struct slice_roi_rec slice_roi;
    unsigned int num_wins;
    struct image_display_window **wins;
   };

struct slice_attr_rec
   {
    struct slice_info_rec *info;
    double min_pixel,max_pixel; /* if min > max then min & max haven't been
                              computed yet */
    struct slice_roi_win_rec *slice_roi_win;
    struct buf_2d_rec _2d;
    struct buf_2d_rec roi_2d;
    struct view_rec *view;
   };

struct data_slice_rec
   {
    struct hdr_scan_entry_rec slice;
    struct slice_attr_rec attr;
    struct slice_roi_win_rec slice_roi_win;
   };

struct co_register_param_rec
   {
    struct vector_3d_rec base;
    struct vector_3d_rec normal;
    struct vector_3d_rec rotation;
    struct matrix_3d_rec planetoxyz,xyztoplane;
   };

#define LOCKED 1
#define CO_DONT_PROCESSED 2
#define CO_REG_PARAM_IN_USE 4
#define CO_REG_PARAM_DEFAULT 8
#define CO_REG_PARAM_REG 0x0010
#define CO_REG_ADJUST 0x0020
#define CO_REG_PARAM_REG_IN_USE 0x0014

#define CO_PLANE 0x0100
#define CO_ALREADY_PROCESSED 0x0200

struct co_register_rec
   {
    unsigned short status;
    char *wid,*coplane_of_wid;
    struct co_register_param_rec reg,coreg;
   };

#define PROJECT_PLANE_DATA 1
#define PLANE_DATA_PROJECTED 2
#define PLANE_SETUP_OK 4
#define PLANE_DATA_READY 8
#define PLANE_PROJECT_ROIS 0x00000010
#define PLANE_PROJECT_PIXEL_TYPES 0x00000020
#define PLANE_ROI_BUF_EXIST 0x00000040
#define PLANE_PIXEL_TYPE_BUF_EXIST 0x00000080

#define PLANE_3D_ROIS_READY 0x00000100
#define PLANE_3D_PIXEL_TYPES_READY 0x00000200
#define PLANE_3D_PLANE_READY 0x00000400
#define PLANE_3D_LOAD_DATA_ERROR 0x00000800

#define PLANE_SLICE_ROI_NEED_SETUP 0x00001000
#define PLANE_SLICE_ROI_SETUP 0x00002000
#define PLANE_SLICE_ROI_UPDATED 0x00004000
#define PLANE_OPACITY_DEFINED 0x00008000

#define PLANE_OPACITY_READY 0x00010000
#define PLANE_SEE_THROUGH_PRIOR 0x00020000
#define PLANE_DATA_MODIFIED 0x00040000
#define PLANE_DATA_MUST_DISPLAY 0x00080000

#define PLANE_LAST_PLANE_VALID 0x00100000
#define PLANE_3D_ROIS_NEED_UPDATED 0x00200000
#define PLANE_3D_PIXEL_TYPES_NEED_UPDATED 0x00400000
#define PLANE_3D_SUB_OBJ 0x00800000

#define PLANE_CO_REGISTERED 0x01000000
#define PLANE_CO_PROCESSED 0x02000000
#define PLANE_CROSS_REGISTERED 0x04000000
#define PLANE_LOCKED 0x08000000
#define PLANE_CROSS_REGISTERED_MAJOR 0x10000000
#define PLANE_LOCKED_PROCESSED 0x20000000
#define PLANE_DELETED 0x40000000
#define PLANE_CO_REG_ADJUST 0x80000000

#define PLANE_REGISTERED_STATUS 0xbf000000

#define PLANE_CO_REGISTERED_PROCESSED 0x22000000

#define PROJECT_THE_PLANE_DATA 5
#define FLUSH_PLANE_DATA 0xfff5
/*
for more_status
*/
#define DP_XYZ 0x00000001
#define VIEW_3D_APPROX 0x00000002
#define VIEW_MORE_STATUS_ADJUSTED 0x00000004
#define VIEW_MORE_STATUS_PROCESSED_STATUS 0x00000004
#define VIEW_MORE_STATUS_PLANE_UNCHANGED 0x00000100
#define VIEW_MORE_STATUS_CO_REG_XYZ_VALID 0x00000200
struct view_rec
   {
    unsigned long status,more_status;
    struct input_file_rec *file;
    int group_num;
    char *id,*wid;
    struct data_slice_rec sl;
    struct list_rec _3d_list;
    struct vector_3d_rec unit_ratio;
    struct plane_3d_rec gplane;
    struct plane_3d_rec last_gplane;
    struct img_buf_rec *img;
    struct strs_rec all_rois,rois;
    struct strs_rec roi_pixel_types,pixel_types;
    unsigned total_roi_2ds,num_roi_2ds;
    struct buf_2d_rec **roi_2ds;
    unsigned total_type_2ds,num_type_2ds;
    struct buf_2d_rec **type_2ds;
    struct strs_rec mask_rois;
    long opacity_min,opacity_max;
    struct ary_3d_xyz_rec xyz;
    unsigned long total_cos,num_cos;
    struct co_register_rec *cos;
    char *dp_xyz_co_reg_wid;
    struct vector_3d_rec co_reg_xyz;
   };

/*
select_3d_gravity = 0  = default
select_3d_gravity = 8  = bottom to top
select_3d_gravity is currently disabled for plane_3d_rec.
When select_3d_gravity = 8, the slices in the buf_3d_recs are reversed...
*/
#define _3D_SLICE_DEFAULT_ORDER 0
#define _3D_SLICE_BOTTOM_TO_TOP 8
#define _3D_ORIENTATION_KEY_ENABLED 16
#define _3D_ABS_FLIP_KEY_ENABLED 0x0020
#define _3D_KP_MODE 0x0030

#define RD_ALL_IMAGES 1
struct slice_info_rec
   {
    unsigned short status;

    unsigned long max_sroi_wins; /* max # of slice_roi_recs read in */
    unsigned long num_sroi_wins; /* # of individual images */
    struct slice_roi_win_rec **sroi_wins;

    unsigned long num_slices;
    struct slice_attr_rec *slices;
    double min_pixel,max_pixel; /* image pixel min & image pixel max
                              use to keep track of the global &
                              user defined global min & max,
                              if (min > max) then not set */

    unsigned long total_3ds,num_3ds;
    struct buf_3d_rec **_3ds;

    int select_3d_gravity;
    struct vector_3d_rec translate,rotate;
    int group_num;
    long num_3d_sampled_pts;
    struct vector_3d_rec sampled_pts[3],sampled_pt0_base;

    unsigned long total_views,num_views;
    struct view_rec **views;

    unsigned long total_data_slices,num_data_slices;
    struct data_slice_rec **data_slices;
   };

#define HISTO_START 7
/*
 histo status2
*/

struct histo_bkg_rec
   {
    long num_bkgs;
    long *bkgs;
    struct roi_rec roi;
   };

#define HISTO_1D 0x0001
#define HISTO_MULTI_D 0x0002
#define HISTO_TYPE 0x0003
#define HISTO_CNT_1D_VALID 0x0400
/*
 histo prog_status2
*/
#define HISTO_READY 0x0003
#define HISTO_1D_USED 0x0004
#define HISTO_MULTI_D_USED 0x0008
#define SET_X_AXIS 0x0100
#define SET_Y_AXIS 0x0200
struct histo_params_rec
   {
    struct input_file_rec *file;
    unsigned short status2,prog_status2;
    struct slice_roi_win_rec slice_roi_win_2d;
    struct slice_roi_win_rec slice_roi_win_1d;
    struct histo_bkg_rec bkg;

    unsigned int total_centers;
    unsigned int cur_center,num_centers;
    struct pixel_center_rec *centers;

    int mid,range;
    unsigned int total_types;
    unsigned int cur_type,num_types;
    struct pixel_tissue_type_rec *types;

    struct strs_rec pixel_type_order; /* consists of the names of the edited  
                and partitioned histogram rois (of type HISTO_PIXEL_TYPE_ROI) */

/*   the dimension of _2d are determined by the preset dimension size of the
        histogram;  they are histo_dimx and histo_dimy found in the header
       */
    unsigned short _2d_status;
    struct buf_2d_rec _2d;
    struct buf_2d_rec map;

    unsigned short _1d_status;
    struct buf_2d_rec _2d_1d;
    struct buf_2d_rec cnt_1d;
    struct buf_2d_rec histo_1d;
   };

struct offset_rec
   {unsigned long start,size;
   };

#define PIXEL_TYPE_SETUP_USER 0x0400
#define PIXEL_TYPE_SETUP_2D_PARTITION 0x0800
#define PIXEL_TYPE_SETUP_METHOD 0x0C00
struct pixmap_hdr_rec
   {
    char *id;
    struct strs_rec order;
    unsigned short status;
    unsigned long dimx,dimy;
    unsigned long max_slices;
    unsigned long num_pixmaps;
    unsigned long pixmap_start,*pixmap_offsets;
   };

#define PIXEL_TYPE_COMPUTED 0x0001
#define PIXEL_TYPE_2D_PARTITION_SETUP 0x0004
#define PIXEL_TYPE_ALL_ROIS_COMPUTED 0x0008
/* the following 2 flags are used to relate the histogram, pixel types, and
   analysis of pixel type counts */

#define PIXEL_TYPE_DEFINED 0x0010
#define ANALYSIS_PROCEEDING 0x0020
#define PIXEL_TYPE_3D_REDEFINED 0x0040
#define PIXEL_TYPE_PIXMAP_HDR_VALID 0x0080

#define PIXEL_TYPE_2D_PARTITION 0x0100
#define PIXEL_TYPE_GROW_REGION 0x0200
#define INCOMPATIBLE_TYPE_BY_NEIGHBOR 0x0400
#define INCOMPATIBLE_TYPE_METHOD_SET 0x0800

#define HISTO_AND_PIXEL_TYPE_RELATED 0x1000
#define PIXEL_TYPE_AND_ANALYSIS_RELATED 0x2000

#define PIXEL_TYPE_GROW_REGION_SETUP 0x4000
#define PIXEL_TYPE_USER_SETUP 0x8000

#define PIXEL_TYPE_SEGMENTED 0xc304
#define PIXMAP_METHOD_DEFINED 0xc204
struct pixel_type_rec
   {
    unsigned short status;
    struct strs_rec order;
    struct buf_2d_rec map;

    struct slice_roi_win_rec *slice_roi_win; /* this slice_roi_win is used
                   to keep track of the slice_roi_rec of the current slice
                    so that a recomputation of pixel types of the same slice
                    is not needed to be done twice if the slice_map below
                    is still being used for the current slice */
    char *slice_map_data_creation_date;
    struct buf_2d_rec slice_map;

    struct strs_rec grow_order;
    unsigned long total_slice_maps,num_slice_maps;
    struct buf_2d_rec **slice_maps;
    struct pixmap_hdr_rec pixmap_hdr;
   };

struct file_register_rec
   {
    struct input_file_rec *regfile;
    struct matrix_2d_rec tolocal,localto;
    struct matrix_2d_rec coregxyztoregxyz;
    struct co_register_rec co;
   };

struct file_co_register_rec
   {
    unsigned long total_cos,num_cos;
    struct file_register_rec **cos;
   };

#define FILE_OPEN 0x0001
#define FILE_BATCH 0x0002
#define FILE_DISPLAY 0x0004
#define FILE_OPEN_ERROR 0x1000
#define FILE_SAVE_PIXEL_TYPE 0x0008
#define FILE_READ 0x0010
#define FILE_WRITE 0x0020
#define FILE_READ_WRITE 0x0030

#define DEF_GA_MENU 0x0040
#define DEF_TA_MENU 0x0080
#define DEF_CA_MENU 0x0100
#define DEF_ANAL_MENU 0x01c0
struct input_file_rec
   {
    struct prog_rec *prog;

    unsigned short status;
               /* FILE_OPEN --- The file is open.
                  FILE_BATCH --- the file is to be batched.
                  FILE_DISPLAY --- the file is to be displayed.
               */
    char *dirfn,*fn;         /* directory fn & fn */
    char *hdr_fn_suffix,*img_fn_suffix;
    struct hdr_rec hdr; /* store in *.hdr file */
    FILE *img_fid;
    struct img_rec img;
             /* each input file is assumed to have an image data area 
                which may have more than 1 buffer */

    struct output_file_rec output;
             /* each input file is assumed to have an output file rec */

    struct slice_info_rec slice_info;
    struct histo_params_rec histo_params;
    struct pixel_type_rec pixel_type;
    struct intensity_rec *color_guide;
    char *edit_begin;
    char *txt_format_fn;
    char *count_format_fn;
    struct text_output_rec text;
    struct count_analysis_rec count;
    struct file_co_register_rec fco;
   };

#endif INPUT_H
