/****************************************************************************
nria_lib.h - Prototypes for functions defined in the NRIA library

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


This header file should be included after all other NRIA header files
(which are needed to define the structures passed as arguments).

The word (COMPLETE) below means that, as far as I know, the list of
prototypes includes all those defined in that source file and used
elsewhere.  

The remark (Automatically generated) means that the prototypes were 
generated using cproto -e.  It also implies that some of the functions 
listed might properly be static instead; this seems a reasonable 
tradeoff for getting prototypes up quickly.

FIXME: This header file is almost certainly not yet complete.

FIXME:  Since this header file contains everything indiscriminately,
it is often the case that it attempts to prototype functions that
depend on a type declaration not in scope.  To temporarily ignore
such problems we use the ugly kluge of skipping such prototypes
in the absense of the flag showing that the header file defining
the type has been loaded.  Such instances are marked as "Ugly kluge".
****************************************************************************/


#ifndef NRIA_LIB_H
#define NRIA_LIB_H

#include <stdio.h>       /* For size_t */

#include <sys/types.h>   /* Needed to define off_t */


/* Defined in 2d_and_mask.c */
#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
#ifdef RECT_H     /* Ugly kluge for struct int_bound_rec */
extern int _2dToBitMask(struct buf_2d_rec *mask, struct buf_2d_rec *_2d, 
    double lb, double ub);
extern int assign_buf_2d_rec_by_not_mask(struct buf_2d_rec *dest, 
    struct buf_2d_rec *src, long startleft, long starttop, 
    struct buf_2d_rec *mask, struct int_bound_rec *bound);
extern int map_buf_2d_rec_by_bound_mask_and_ratio(struct buf_2d_rec *result, 
    long startleft, long starttop, double xratio, double yratio, 
    struct buf_2d_rec *_2d, struct buf_2d_rec *mask, 
    struct int_bound_rec *bound, unsigned long *map_values, 
    unsigned long num_map_values, unsigned long const_val);
extern int assign_buf_2d_rec_by_mask(struct buf_2d_rec *dest, 
    struct buf_2d_rec *src, long startleft, long starttop, 
    struct buf_2d_rec *mask, struct int_bound_rec *bound);
extern int PickAssign2d(struct buf_2d_rec *dest, struct buf_2d_rec *src, 
    struct buf_2d_rec *type_2d, long startleft, long starttop, 
    struct buf_2d_rec *mask, struct int_bound_rec *bound, char *pick_types, 
    unsigned long num_pick_types);
extern int process_buf_2d_rec_by_mask(struct buf_2d_rec *_2d,
    long startleft, long starttop, struct buf_2d_rec *mask,
    struct int_bound_rec *bound, int (*proc)(), void *arg1,
    void *arg2, void *arg3, void *arg4, void *arg5, void *arg6, 
    void *arg7, void *arg8, void *arg9);
extern int Set2dByNotMask(struct buf_2d_rec *result, long startleft, 
    long starttop, struct buf_2d_rec *mask, struct int_bound_rec *bound, 
    double value);
extern int Set2d(struct buf_2d_rec *result, long startleft, long starttop, 
    struct buf_2d_rec *mask, struct int_bound_rec *bound, double value);
extern int SelectSet2d(struct buf_2d_rec *result, long startleft, 
    long starttop, struct buf_2d_rec *mask, struct int_bound_rec *bound, 
    double lb, double ub, double setval, int isSetNotInRange, 
    double NotInRangeVal);
extern int Smooth2dByMask(struct buf_2d_rec *_2d, long startleft, 
    long starttop, struct buf_2d_rec *mask, struct int_bound_rec *bound, 
    long num_neighbors);
extern int Minus2dsAndScaleByMask(struct buf_2d_rec *dest, 
    struct buf_2d_rec *src, long startleft, long starttop, 
    struct buf_2d_rec *mask, struct int_bound_rec *bound, int is_abs);
extern int Add2dsAndScaleByMask(struct buf_2d_rec *dest, 
    struct buf_2d_rec *src, long startleft, long starttop, 
    struct buf_2d_rec *mask, struct int_bound_rec *bound, int is_abs);
extern int Minus2dsAndScale(struct buf_2d_rec *dest, struct buf_2d_rec *src, 
    int is_abs);
extern int Add2dsAndScale(struct buf_2d_rec *dest, struct buf_2d_rec *src, 
    int is_abs);
extern int Map2dPixelValues(struct buf_2d_rec *in_2d, 
    struct buf_2d_rec *out_2d, unsigned long *map_indx, 
    unsigned long num_indxs, unsigned long indx_depth, 
    unsigned long first_indx_value);
#endif
#endif


/* Defined in 2d_buf_opr.c */
#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
extern void init_buf_2d_rec(struct buf_2d_rec *_2d);
extern void free_buf_2d_rec(struct buf_2d_rec *_2d);
extern int copy_buf_2d_rec(struct buf_2d_rec *dst, struct buf_2d_rec *src);
extern int copy_buf_2d_rec_content(struct buf_2d_rec *dst, 
    struct buf_2d_rec *src);
extern int copy_buf_2d_rec_hdr_only(struct buf_2d_rec *dst, 
    struct buf_2d_rec *src);
extern int CopyToNonContiguos2d(struct buf_2d_rec *result, 
    struct buf_2d_rec *src);
extern unsigned long get_1d_buf_value(unsigned char *buf, unsigned long col, 
    unsigned long pixel_depth);
extern int SetupBuf2dRec(struct buf_2d_rec *_2d, unsigned long dimx, 
    unsigned long dimy, unsigned long pixel_depth, unsigned long flags);
extern int SetupSameDimxyDst2d(struct buf_2d_rec *dst, 
    struct buf_2d_rec *src1, struct buf_2d_rec *src2, unsigned long select);
extern int ValidDstAndSrcBound(long dleft, long dtop, long dright, 
    long dbottom, long dx, long dy, long sleft, long stop, long sright, 
    long sbottom, long *vdleft, long *vdtop, long *vdright, long *vdbottom, 
    long *vsleft, long *vstop, long *vsright, long *vsbottom);
extern int DefineDstDepthAndType(struct buf_2d_rec *src1, 
    struct buf_2d_rec *src2, unsigned long *depth, unsigned long *type4, 
    unsigned long select);
extern int SetupBuf2dRecIfSmall(struct buf_2d_rec *_2d, unsigned long dimx, 
    unsigned long dimy, unsigned long pixel_depth, int flags);
extern int Cp2dByBound(struct buf_2d_rec *dest, long dleft, long dtop, 
    long dright, long dbottom, long dx, long dy, struct buf_2d_rec *src, 
    long sleft, long stop, long sright, long sbottom, int option, double min, 
    double max, int is_unselected_to_0);
extern void Set2dStartOffsetXY(struct buf_2d_rec *_2d, long startoffx, 
    long startoffy);
extern int is_same_buf_2d_rec_data_size(struct buf_2d_rec *_2d1, 
    struct buf_2d_rec *_2d2);
extern int InitBuf2dRecData(struct buf_2d_rec *_2d, double val);
extern int CpBuf2dRecByBound(struct buf_2d_rec *dest, long dleft, long dtop, 
    long dright, long dbottom, long dx, long dy, struct buf_2d_rec *src, 
    long sleft, long stop, long sright, long sbottom);
extern int GetUnitXY(struct buf_2d_rec *_2d,
    float *unitx, float *unity);
extern void set_2d_ustart_xy(struct buf_2d_rec *_2d, double ustartx, 
    double ustarty);
extern int is_same_buf_2d_rec_dimxy(struct buf_2d_rec *_2d1, 
    struct buf_2d_rec *_2d2);
extern void Get2dUnitInfo(struct buf_2d_rec *_2d, float *ustartx, 
    float *ustarty, float *unitx, float *unity, unsigned long *dimx, 
    unsigned long *dimy, double def_ustartx, double def_ustarty, 
    double def_unitx, double def_unity, unsigned long def_dimx, 
    unsigned long def_dimy);
extern int CntNonValBuf2dRec(struct buf_2d_rec *dest, double val);
extern int SetupNoncontiguosBuf2dRec(struct buf_2d_rec *_2d, 
    unsigned long dimx, unsigned long dimy, unsigned long pixel_depth, 
    unsigned long select);
extern int Get1dBitMapFirstNon0Posi(unsigned char *buf, unsigned long dim, 
    unsigned long pixel_depth, unsigned long *posi);
extern int Get1dBitMapLastNon0Posi(unsigned char *buf, unsigned long dim, 
    unsigned long pixel_depth, unsigned long *posi);
extern void get_range_byte_offsets(unsigned long left, unsigned long right, 
    unsigned long pixel_depth, unsigned char *first_and_byte, 
    unsigned long *start, unsigned long *width, unsigned char *last_and_byte);
extern int get_buf_2d_rec_bound(struct buf_2d_rec *_2d, unsigned long *left, 
    unsigned long *top, unsigned long *right, unsigned long *bottom);
#endif


/* Defined in 2d_value.c */
extern unsigned long GetDepthNeeded(double min, double max, 
    unsigned long *type4);
extern void *get_1d_buf_addr(unsigned char *buf, unsigned long col, 
    unsigned long pixel_depth, unsigned long *start_bit);
extern int set_1d_buf_value(unsigned char *buf, unsigned long col, 
    unsigned long pixel_depth, unsigned long value);
extern double DoubleMax4ByteBufValue(unsigned char *buf, unsigned long col, 
    unsigned long pixel_depth, unsigned long select);

#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
extern int AddConstantTo2d(struct buf_2d_rec *_2d, double value, 
    int is_non0_elems);
extern int Set2dByArea(struct buf_2d_rec *_2d, int left, int top, int right, 
    int bottom, double value);
extern int Get2dValProc(struct buf_2d_rec *_2d, double (**getproc)());
extern int Set2dValProc(struct buf_2d_rec *_2d, int (**setproc)());
extern int Set2dValueType(struct buf_2d_rec *_2d, unsigned long type4);
extern int GetMinMaxValues(struct buf_2d_rec *_2d, double *min, double *max);
extern int Add2ds(struct buf_2d_rec *result, struct buf_2d_rec *_2d1, 
    struct buf_2d_rec *_2d2);
extern int Abs2d(struct buf_2d_rec *_2d);
extern int Assign2d(struct buf_2d_rec *dst, struct buf_2d_rec *src);
extern int Sub2ds(struct buf_2d_rec *result, struct buf_2d_rec *_2d1, 
    struct buf_2d_rec *_2d2);
extern int Add2dVal(struct buf_2d_rec *_2d, unsigned long row, 
    unsigned long col, double val);
extern int SelectAssign2d(struct buf_2d_rec *result, struct buf_2d_rec *_2d, 
    unsigned long select, double min, double max, double setval, double addval,
    unsigned long *map_2d_indxs, unsigned long num_map_2d_indxs, 
    unsigned long first_2d_indx);
extern int ScaleAssign2d(struct buf_2d_rec *_2d, struct buf_2d_rec *src);
extern int SelectAssign2dProc(struct buf_2d_rec *result, unsigned long row, 
    unsigned long col, struct buf_2d_rec *_2d, unsigned long select, 
    double *min, double *max, double *setval, double *addval, 
    unsigned long *map_2d_indxs, unsigned long *num_map_2d_indxs, 
    unsigned long *first_2d_indx);
extern int Set2dByIndexValueRange(struct buf_2d_rec *out_2d, 
    struct buf_2d_rec *in_2d, struct buf_2d_rec *mask, unsigned long *indxs, 
    unsigned long num_indxs, unsigned long first, unsigned long findxval, 
    unsigned long lindxval, double setval);
extern int Assign2dByGeneralMask(struct buf_2d_rec *result, 
    struct buf_2d_rec *src, struct buf_2d_rec *mask, unsigned long select);
extern int Set2dByValue(struct buf_2d_rec *_2d, double value);
extern int Div2dByConstant(struct buf_2d_rec *result, struct buf_2d_rec *_2d1,
    double constant, int is_div_0_equal_0);
extern int pr_buf_2d_rec(struct buf_2d_rec *_2d);
extern int Div2ds(struct buf_2d_rec *result, struct buf_2d_rec *_2d1, 
    struct buf_2d_rec *_2d2, int is_div_0_equal_0);
extern int Sqrt2d(struct buf_2d_rec *result, struct buf_2d_rec *_2d);
extern int Mult2dByConstant(struct buf_2d_rec *result, struct buf_2d_rec *_2d1,
    double constant);
extern int Add2dArys(struct buf_2d_rec *result, struct buf_2d_rec **_2ds, 
    unsigned long num_2ds, int select);
extern double Total2dValues(struct buf_2d_rec **_2d1, 
    struct buf_2d_rec **_2d2, unsigned long num_2ds, unsigned long select);
extern int Get2dAreavalues(struct buf_2d_rec *_2d, long left, long top, 
    long right, long bottom, double *vals, long max_vals, long *num_vals);
#endif

#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
#ifdef RECT_H     /* Ugly kluge for struct int_bound_rec */
extern int SelectSet2dValue(struct buf_2d_rec *_2d, 
    struct int_bound_rec *bound, double lb, double ub, double setval, 
    int isSetNotInRange, double NotInRangeVal);
extern int FindBitmapBound(struct buf_2d_rec *_2d, 
    struct int_bound_rec *bound, int *is_empty);
#endif
#endif


/* Defined in 2dbufmoreopr.c */
#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
extern int HistogramEqualization(struct buf_2d_rec **_2ds, 
    unsigned long num_2ds, double min, double max, int use_newminmax, 
    double newmin, double newmax, float **vals, unsigned long *num_vals);
extern int AutoSet2dBackground(struct buf_2d_rec *_2d, 
    unsigned long histo_width, unsigned long min_bkg_value, 
    unsigned long max_bkg_value);
extern int Histogram2d(struct buf_2d_rec *_2d, struct buf_2d_rec *src, 
    unsigned long src_row, unsigned long src_col_start, int use_value, 
    double value, int is_clear_src);
extern int AutoSetHisto1DBuf(struct buf_2d_rec *histo, unsigned long row_indx,
    unsigned long min_bkg_value, unsigned long max_bkg_value, 
    unsigned long *cutoff_start, int is_clear_histo);
extern int CombinePosNeg2d(struct buf_2d_rec *result, struct buf_2d_rec *pos, 
    struct buf_2d_rec *neg);
extern int Gradient2d(struct buf_2d_rec *result,
    struct buf_2d_rec *src, double edge_threshold, int set_edge,
    double edge_value, int set_background, double background_value);
extern int _2dPtMeanAndSd(struct buf_2d_rec *_2d, unsigned long row, 
    unsigned long col, double *mean, double *sd);
extern int PtsMeanAndSd(double *vals, unsigned long num_vals,
    double *mean, double *sd);
extern int Count1dOccurrences(struct buf_2d_rec *result, 
    unsigned long result_row_indx, struct buf_2d_rec *src, 
    struct buf_2d_rec *bit_mask, int init_result, unsigned long *total_pixels,
    int re_init_total_pixels);
extern int Count2dOccurrences(struct buf_2d_rec *result, 
    struct buf_2d_rec *echo1, struct buf_2d_rec *echo2, 
    struct buf_2d_rec *bit_mask, int init_result, unsigned long *total_pixels,
    int re_init_total_pixels);
#endif

#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
#ifdef RECT_H     /* Ugly kluge for struct int_bound_rec */
extern int GrowRegion(struct buf_2d_rec *_2d, unsigned long starty, 
    unsigned long startx, double lb, double ub, struct buf_2d_rec *mask, 
    struct int_bound_rec *bound_area);
extern int Gradient2dByMask(struct buf_2d_rec *_2d, unsigned long startleft, 
    unsigned long starttop, struct buf_2d_rec *mask, 
    struct int_bound_rec *bound, struct buf_2d_rec *result, 
    double edge_threshold, int set_edge, double edge_value, 
    int set_background, double background_value);
extern int sobel_2d_by_mask(struct buf_2d_rec *_2d,
    unsigned long startleft, unsigned long starttop, struct buf_2d_rec *mask, 
    struct int_bound_rec *bound, struct buf_2d_rec *result);
extern int sharpen_2d_by_mask(struct buf_2d_rec *_2d, 
    unsigned long startleft, unsigned long starttop, struct buf_2d_rec *mask, 
    struct int_bound_rec *bound, struct buf_2d_rec *result);
extern int sharpen_2d(struct buf_2d_rec *_2d, struct buf_2d_rec *result);
extern int sobel_2d(struct buf_2d_rec *_2d, struct buf_2d_rec *result);
extern int _2dMeanAndSd(struct buf_2d_rec *_2d, struct buf_2d_rec *mask,
    struct int_bound_rec *bound_area, double *mean, double *sd);
#endif
#endif


/* Defined in 2dbufsystem.c */
#ifdef COMMON_H   /* Ugly kluge for struct buf_2d_rec */
extern int mmap_buf_2d_rec(struct buf_2d_rec *_2d, int prot, 
    FILE *fid, off_t off);
extern int munmap_buf_2d_rec(struct buf_2d_rec *_2d);
#endif


/* Defined in 2dbufvalrec.c */
#ifdef BUF_2D_VAL_REC_H  /* Ugly kluge for struct buf_2d_val_rec and val_rec */
extern int find_val_recs(struct buf_2d_val_rec *pixels, 
    unsigned long start_value, unsigned long end_value, struct val_rec **vals,
    unsigned long *num_vals);
extern int setup_buf_2d_val_rec(struct buf_2d_val_rec *pixels, 
    struct buf_2d_rec *_2d, unsigned long min, unsigned long max);
extern void init_buf_2d_val_rec(struct buf_2d_val_rec *pixels);
extern void free_buf_2d_val_rec(struct buf_2d_val_rec *pixels);
#endif


/* Defined in 2dbufxtraopr.c */
#ifdef COMMON_H     /* Ugly kluge for struct buf_2d_rec & mask_rec */
extern double Smooth2dValue(struct buf_2d_rec *_2d, unsigned long row, 
    unsigned long col, long num_neighbors);
extern int ResizeBuf2dRec(struct buf_2d_rec *dst, struct buf_2d_rec *src,
    unsigned long ddimx, unsigned long ddimy, struct matrix_2d_rec *to_src);
extern void init_mask_rec(struct mask_rec *mask);
extern void free_mask_rec1(struct mask_rec *mask);
extern int cp_mask_rec(struct mask_rec *dst, struct mask_rec *src);
extern int Smooth2d(struct buf_2d_rec *_2d, long num_neighbors);
#endif


/* Defined in 2dminmaxscal.c */
extern double ScaleVal(double old_val, double old_min, double old_max,
    double new_min, double new_max);


#ifdef COMMON_H      /* Ugly kluge for struct buf_2d_rec */
extern void Default2dMinMax(struct buf_2d_rec *_2d, double *minval,
    double *maxval, int exclude_range, double range_min, double range_max);
extern int Scale2dPixels(struct buf_2d_rec *out_2d, struct buf_2d_rec *in_2d,
    double min, double max, int exclude_range, double range_min,
    double range_max, int use_which_min_max);
extern int MinMax2dValues(struct buf_2d_rec *_2d, int exclude_range,
    double range_min, double range_max, double *min, double *max);
extern int MinMax2d(struct buf_2d_rec *_2d, int exclude_range,
    double range_min, double range_max);
extern int Assin2dMinMaxSetVals(struct buf_2d_rec *dst,
    struct buf_2d_rec *src);
extern int Scale2dValues(struct buf_2d_rec *result, struct buf_2d_rec *_2d,
    double min, double max, int exclude_range, double range_min,
    double range_max, double newmin, double newmax);
extern int MinMax2dByMaskProc(struct buf_2d_rec *_2d, unsigned long row,
    unsigned long col, double *min, double *max);
#endif

#ifdef COMMON_H  /* Ugly kluge for struct buf_2d_rec */
#ifdef RECT_H    /* Ugly kluge for struct int_bound_rec */
extern int MinMax2dByMask(struct buf_2d_rec *_2d, unsigned long startleft,
    unsigned long starttop, struct buf_2d_rec *mask,
    struct int_bound_rec *bound, double *min, double *max);
extern int Scale2dByMask(struct buf_2d_rec *_2d, struct buf_2d_rec *dest,
    unsigned long startleft, unsigned long starttop, struct buf_2d_rec *mask,
    struct int_bound_rec *bound, double old_min, double old_max,
    double new_min, double new_max);
#endif
#endif


/* Defined in 2dtran.c (Automatically generated) */
extern void side_and_dir_indx_of_center(double centerx, double centery,
    double firstx, double firsty, double nextx, double nexty, int *which_side,
    int *dir_indx);
extern int scale_xy_ratio(double centerx, double centery, double firstx,
    double firsty, double nextx, double nexty, double width, double height,
    float *sx, float *sy);
extern int rotate_2d_rad(double centerx, double centery, double firstx,
    double firsty, double nextx, double nexty, float *rad);

#ifdef GEOMETRY_H /* Ugly kluge for struct matrix_2d_rec */
extern int combine_2d_matrices(struct matrix_2d_rec *dest, ...);
extern void identity_2d_matrix(struct matrix_2d_rec *matrix);
extern void setup_2d_matrix(struct matrix_2d_rec *matrix, double e00,
    double e01, double e02, double e10, double e11, double e12, double e20,
    double e21, double e22);
extern void mult_2d_matrix(struct matrix_2d_rec *dest,
    struct matrix_2d_rec *src1, struct matrix_2d_rec *src2);
extern void check_and_mult_2d_matrix(struct matrix_2d_rec *dest,
    struct matrix_2d_rec *src1, struct matrix_2d_rec *src2);
extern void assign_2d_translation(struct matrix_2d_rec *matrix, double tx,
    double ty);
extern void setup_translate_2d_matrix(struct matrix_2d_rec *matrix, double tx,
    double ty);
extern void translate_2d_matrix(struct matrix_2d_rec *matrix, double tx,
    double ty);
extern void drop_2d_matrix_translation(struct matrix_2d_rec *matrix);
extern void setup_scale_2d_matrix(struct matrix_2d_rec *matrix, double sx,
    double sy);
extern void scale_2d_matrix(struct matrix_2d_rec *matrix, double sx,
    double sy);
extern void setup_scale_2d_matrix_from_xy(struct matrix_2d_rec *matrix,
    double sx, double sy, double fx, double fy);
extern void scale_2d_matrix_from_xy(struct matrix_2d_rec *matrix, double sx,
    double sy, double fx, double fy);
extern void setup_rotate_2d_matrix(struct matrix_2d_rec *matrix,
    double radian_angle);
extern void rotate_2d_matrix(struct matrix_2d_rec *matrix,
    double radian_angle);
extern void setup_rotate_2d_matrix_from_xy(struct matrix_2d_rec *matrix,
    double radian_angle, double fx, double fy);
extern void rotate_2d_matrix_from_xy(struct matrix_2d_rec *matrix,
    double radian_angle, double fx, double fy);
extern float det_2d_matrix(struct matrix_2d_rec *matrix);
extern int inverse_2d_matrix(struct matrix_2d_rec *inv,
    struct matrix_2d_rec *matrix);
extern void setup_xreflect_2d_matrix(struct matrix_2d_rec *matrix);
extern void setup_yreflect_2d_matrix(struct matrix_2d_rec *matrix);
extern void setup_xyreflect_2d_matrix(struct matrix_2d_rec *matrix);
extern void setup_xshear_2d_matrix(struct matrix_2d_rec *matrix,
    double shear_factor);
extern void setup_yshear_2d_matrix(struct matrix_2d_rec *matrix,
    double shear_factor);
extern void pr_2d_matrix(struct matrix_2d_rec *matrix);
extern void trans_2d_pt(struct matrix_2d_rec *matrix, double x, double y,
    float *newx, float *newy, int is_result_rounded);
extern int is_same_matrix_2d_rec(struct matrix_2d_rec *mat,
    struct matrix_2d_rec *mat1);
#endif


/* Defined in 3d_data.c (Automatically generated) */
#ifdef COMMON_H       /* Ugly kluge for struct buf_2d_rec */
#ifdef BUF_3D_REC_H   /* Ugly kluge for struct buf_3d_rec etc. */
extern int Get3dValApprox(struct buf_3d_rec *_3d, double x, double y, double z,
    double *value);
extern int Get3dVal(struct buf_3d_rec *_3d, double x, double y, double z,
    double *value);
extern int _3dVal(struct plane_3d_rec *plane, struct buf_3d_rec *_3d,
    double x, double y, double z, double *value);
extern double AbsXYZVal(struct buf_3d_rec *_3d, long tx, long ty, long tz);
extern int PlaneData_00_10Approx(struct plane_3d_rec *plane,
    struct buf_3d_rec *_3d, struct buf_2d_rec *result_2d,
    struct matrix_3d_rec *the_planetoxyz);
extern int PlaneData_00_10(struct plane_3d_rec *plane, struct buf_3d_rec *_3d,
    struct buf_2d_rec *result_2d, struct matrix_3d_rec *the_planetoxyz);
extern int Plane2ds_10Approx(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds,
    struct matrix_3d_rec *the_planetoxyz);
extern int Plane2ds_10(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds,
    struct matrix_3d_rec *the_planetoxyz);
extern int PlaneData_11Approx(struct plane_3d_rec *plane,
    struct buf_3d_rec *_3d, struct buf_2d_rec *result_2d,
    struct matrix_3d_rec *the_planetoxyz);
extern int PlaneData_11(struct plane_3d_rec *plane, struct buf_3d_rec *_3d,
    struct buf_2d_rec *result_2d, struct matrix_3d_rec *the_planetoxyz);
extern int Plane2ds_11Approx(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds,
    struct matrix_3d_rec *the_planetoxyz);
extern int Plane2ds_11(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds,
    struct matrix_3d_rec *the_planetoxyz);
extern int PlaneCoords_11(struct plane_3d_rec *plane, struct buf_3d_rec *_3d,
    struct ary_pt_3d_rec *xyz, struct matrix_3d_rec *the_planetoxyz);
extern int _3d_plane_coords(struct plane_3d_rec *plane,
    struct buf_3d_rec *_3d, struct ary_pt_3d_rec *xyz);
extern int xyz_xy_status(struct _3d_plane_2d_rec *_3d_2ds,
    unsigned long num_3d_2ds);
extern int Get3dPlaneDataApprox(struct plane_3d_rec *plane,
    struct buf_3d_rec *_3d, struct buf_2d_rec *result_2d);
extern int Get3dPlaneData(struct plane_3d_rec *plane, struct buf_3d_rec *_3d,
    struct buf_2d_rec *result_2d);
extern int Get3dPlane2dsApprox(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds);
extern int Get3dPlane2ds(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds);
extern int _3dPlaneData(struct plane_3d_rec *plane, struct buf_3d_rec *_3d,
    struct buf_2d_rec *result_2d, unsigned long is_approx);
extern int _3dPlane2ds(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds);
extern int paste_plane_2d_data(struct plane_3d_rec *plane,
    struct buf_2d_rec *_2d, struct buf_2d_rec *result_2d);
extern int trans_plane_2d_data(struct plane_3d_rec *plane,
    struct buf_2d_rec *_2d, struct buf_2d_rec *result_2d);
extern int paste_plane_2ds(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds);
extern int trans_plane_2ds(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds);
extern int transform_plane_2d_data(struct plane_3d_rec *plane,
    struct buf_2d_rec *_2d, struct buf_2d_rec *result_2d);
extern int transform_plane_2ds(struct plane_3d_rec *plane,
    struct _3d_plane_2d_rec *_3d_2ds, unsigned long num_3d_2ds);
#endif
#endif


/* Defined in 3dtran.c (Automatically generated) */
#ifdef GEOMETRY_H   /* Ugly kluge for struct matrix_3d_rec etc. */
extern int combine_3d_matrices(struct matrix_3d_rec *dest, ...);
extern void identity_3d_matrix(struct matrix_3d_rec *matrix);
extern void setup_3d_matrix(struct matrix_3d_rec *matrix, double e00,
    double e01, double e02, double e10, double e11, double e12, double e20,
    double e21, double e22);
extern void setup_4d_matrix(struct matrix_3d_rec *matrix, double e00,
    double e01, double e02, double e03, double e10, double e11, double e12,
    double e13, double e20, double e21, double e22, double e23, double e30,
    double e31, double e32, double e33);
extern int is_same_matrix_3d_rec(struct matrix_3d_rec *mat1,
    struct matrix_3d_rec *mat2);
extern void setup_3d_xy_reflection_matrix(struct matrix_3d_rec *matrix);
extern void setup_3d_default_xy_axes(struct matrix_3d_rec *matrix);
extern void mult_3d_matrix(struct matrix_3d_rec *dest,
    struct matrix_3d_rec *src1, struct matrix_3d_rec *src2);
extern void assign_translation(struct matrix_3d_rec *matrix,
    double tx, double ty, double tz);
extern void setup_translate_3d_matrix(struct matrix_3d_rec *matrix,
    double tx, double ty, double tz);
extern void translate_3d_matrix(struct matrix_3d_rec *matrix,
    double tx, double ty, double tz);
extern void drop_3d_matrix_translation(struct matrix_3d_rec *matrix);
extern void setup_scale_3d_matrix(struct matrix_3d_rec *matrix,
    double sx, double sy, double sz);
extern void scale_3d_matrix(struct matrix_3d_rec *matrix,
    double sx, double sy, double sz);
extern void setup_scale_3d_matrix_from_xyz(struct matrix_3d_rec *matrix,
    double sx, double sy, double sz, double fx, double fy, double fz);
extern void scale_3d_matrix_from_xyz(struct matrix_3d_rec *matrix,
    double sx, double sy, double sz, double fx, double fy, double fz);
extern void setup_xrotate_3d_matrix(struct matrix_3d_rec *matrix,
    double radian_angle);
extern void xrotate_3d_matrix(struct matrix_3d_rec *matrix,
    double radian_angle);
extern void setup_yrotate_3d_matrix(struct matrix_3d_rec *matrix,
    double radian_angle);
extern void yrotate_3d_matrix(struct matrix_3d_rec *matrix,
    double radian_angle);
extern void setup_zrotate_3d_matrix(struct matrix_3d_rec *matrix,
    double radian_angle);
extern void zrotate_3d_matrix(struct matrix_3d_rec *matrix,
    double radian_angle);
extern void Rotate3dMatrix(struct matrix_3d_rec *matrix, double radian_angle);
extern void Rotate3dMatrixFromAxis(struct matrix_3d_rec *xyztoplane,
    struct matrix_3d_rec *planetoxyz, struct vector_3d_rec *base,
    struct vector_3d_rec *dire, double radian_angle);
extern void rotate_pt_fr_line(struct vector_3d_rec *base,
    struct vector_3d_rec *dire, double rad_deg, struct vector_3d_rec *pt,
    struct vector_3d_rec *transformed_pt);
extern float det_3d_matrix(struct matrix_3d_rec *matrix);
extern int inverse_3d_matrix(struct matrix_3d_rec *inv,
    struct matrix_3d_rec *matrix);
extern void setup_3d_cofactor(struct matrix_3d_rec *result,
    struct matrix_3d_rec *matrix, int row, int col);
extern void transpose_4d_matrix(struct matrix_3d_rec *result,
    struct matrix_3d_rec *matrix);
extern float det_4d_matrix(struct matrix_3d_rec *matrix);
extern int inverse_4d_matrix(struct matrix_3d_rec *inv,
    struct matrix_3d_rec *matrix);
extern void pr_3d_matrix(struct matrix_3d_rec *matrix);
extern void FindBoundTLTR1(struct vector_3d_rec bounds[4], int *tl, int *tr,
    struct vector_3d_rec *normal);
extern void PlaneCoordMatrix(struct matrix_3d_rec *xyztoplane,
    struct matrix_3d_rec *planetoxyz, struct vector_3d_rec *base,
    struct vector_3d_rec *normal, double zrotating_rad);
extern void DefPlaneOrientation(struct matrix_3d_rec *xyztoplane,
    struct matrix_3d_rec *planetoxyz, struct matrix_3d_rec *plane_trans,
    struct vector_3d_rec *base, struct vector_3d_rec *normal,
    int *is_plane_trans);
extern void trans_3d_xyz(struct matrix_3d_rec *matrix,
    double x, double y, double z, float *newx, float *newy, float *newz,
    int is_result_rounded);
extern void trans_3d_pt(struct matrix_3d_rec *matrix, struct vector_3d_rec *pt,
    struct vector_3d_rec *map_pt, int is_result_rounded);
extern void proj_pt_vector_3d_rec(struct matrix_3d_rec *matrix,
    struct vector_3d_rec *coord_ratio, struct vector_3d_rec *pt,
    struct vector_3d_rec *transformed_pt);
extern int rotate_3d_rad(double centerx, double centery, double centerz,
    double firstx, double firsty, double firstz,
    double nextx, double nexty, double nextz, float *rad);
extern void matrix_3d_to_2d(struct matrix_2d_rec *mat2d,
    struct matrix_3d_rec *mat3d);
extern void matrix_2d_to_3d(struct matrix_3d_rec *mat3d,
    struct matrix_2d_rec *mat2d);
#endif


/* Defined in buf_3d_rec.c (Automatically generated) */
#ifdef COMMON_H            /* For struct buf_2d_rec */
#ifdef BUF_3D_REC_H        /* For struct slice_3d_rec and friends */
extern void init_slice_3d_rec(struct slice_3d_rec *slice);
extern void init_buf_3d_rec(struct buf_3d_rec *_3d);
extern void set_3d_start_xyz(struct buf_3d_rec *_3d,
    int startx, int starty, int startz);
extern void free_buf_3d_rec_slices(struct buf_3d_rec *_3d);
extern void free_buf_3d_rec(struct buf_3d_rec *_3d);
extern void init_ary_pt_3d_rec(struct ary_pt_3d_rec *xyz);
extern void free_ary_pt_3d_rec(struct ary_pt_3d_rec *xyz);
extern int sort_pt_3d_rec_proc(struct pt_3d_rec *xyz1, struct pt_3d_rec *xyz2);
extern void sort_ary_pt_3d_rec(struct ary_pt_3d_rec *xyz);
extern void init_ary_3d_xyz_rec(struct ary_3d_xyz_rec *xyz);
extern void free_ary_3d_xyz_rec(struct ary_3d_xyz_rec *xyz);
extern void total_zslices(double slice_thickness, double pixel_size,
    unsigned long total_slices, unsigned long *total_zslice,
    float *last_slice_thickness);
extern int get_zindx(double zpixel_size, double slice_thickness,
    long actual_sl_num, unsigned long startz, unsigned long dimz, long *indx);
extern int setup_buf_3d_rec(struct buf_3d_rec *_3d, char *id,
    unsigned long dimx, unsigned long dimy, unsigned long total_slices,
    double xpixel_size, double ypixel_size, double zpixel_size,
    double slice_thickness, unsigned long startx, unsigned long starty,
    long start_slice_num);
extern void default_buf_3d_rec_2d_dims(struct buf_3d_rec *_3d,
    unsigned long *dimx, unsigned long *dimy);
extern int InitBuf3dRecData(struct buf_3d_rec *_3d, double val);
extern void setup_3d_plane_2d_rec(struct _3d_plane_2d_rec *_3d_2d,
    struct buf_3d_rec *_3d, struct buf_2d_rec *plane_2d,
    struct buf_2d_rec *transform_2d);
extern void pr_buf_3d_rec(struct buf_3d_rec *_3d);
extern int add_3d_slice_infect(struct buf_3d_rec *_3d, struct buf_2d_rec *_2d,
    struct hdr_scan_entry_rec *_2d_info, unsigned long indx,
    double slice_thickness, int is_share_border_pixel);
extern int add_3d_slice_own(struct buf_3d_rec *_3d, struct buf_2d_rec *_2d,
    struct hdr_scan_entry_rec *_2d_info, unsigned long indx,
    double slice_thickness);
extern int setup_3d_interpolation(struct buf_3d_rec *_3d);
extern int find_slice_in_3d(struct buf_3d_rec *_3d, struct buf_2d_rec *_2d,
    unsigned long *indx);
extern int SortBuf3dRec(void *ptr1, void *ptr2);
extern int reverse_3d_slice_order(struct buf_3d_rec *_3d);
#endif
#endif


/* Defined in build_hdr.c (Automatically generated) */
#ifdef IA_HDR_H         /* For struct hdr_rec */
extern int slice_size(struct hdr_rec *hdr);
extern int hdr_image_labels(struct hdr_rec *hdr, char **labels[],
    unsigned int *member_img_label_indxs);
extern int Build1EchoHdr(struct hdr_rec *hdr, unsigned long skip_start_offset);
extern int Build2EchoHdr(struct hdr_rec *hdr, unsigned long skip_start_offset);
extern int BuildRiaHdr(struct hdr_rec *hdr, unsigned long skip_start_offset);
#endif


/* Defined in coord_rec.c (Automatically generated) */
#ifdef COORD_REC_H       /* For struct coord_rec */
extern void init_coord_rec(struct coord_rec *coords);
extern int get_coord_rec_memory(struct coord_rec *coords,
    unsigned int add_num_pts);
extern void free_coord_rec(struct coord_rec *coords);
extern int find_coord_rec_min_max(struct coord_rec *coords,
    unsigned int *minx, unsigned int *maxx, unsigned int *miny,
    unsigned int *maxy);
extern void pr_coord_rec(struct coord_rec *coords);
extern int coord_rec_sort_proc(struct coord_pt_rec *pt1,
    struct coord_pt_rec *pt2);
extern void sort_coord_rec(struct coord_rec *coords);
extern void set_buf_by_coord_rec(struct buf_2d_rec *_2d,
    struct coord_rec *coords, long val);
extern int xor_set_buf_by_sorted_coord_rec(struct buf_2d_rec *_2d,
    struct coord_rec *coords);
extern int sort_coord_pt_rec_proc(struct coord_pt_rec *pt1,
    struct coord_pt_rec *pt2);
extern int get_common_coords(struct coord_rec *dest, struct coord_rec *src);
extern int exclude_common_coords(struct coord_rec *dest,
    struct coord_rec *src);
extern int have_common_coords(struct coord_rec *dest, struct coord_rec *src);
#endif


/* Defined in debug_opr.c */
#ifdef MRI_FORMAT_H          /* For all sorts of NRIA internals */
void dump_output_file_rec(FILE *out, struct output_file_rec *rec);
void dump_input_file_rec(FILE *out, struct input_file_rec *rec);
void dump_output_hdr_rec(FILE *out, struct output_hdr_rec *rec);
void dump_hdr_rec(FILE *out, struct hdr_rec *rec);
void dump_slice_info_rec(FILE *out, struct slice_info_rec *rec);
void dump_slice_info_rec_tree(FILE *out, struct slice_info_rec *rec);
void dump_data_slice_rec(FILE *out, struct data_slice_rec *rec);
void dump_histo_params_rec(FILE *out, struct histo_params_rec *rec);
void dump_slice_roi_win_rec_tree(FILE *out, struct slice_roi_win_rec *rec);
void dump_slice_roi_win_rec(FILE *out, struct slice_roi_win_rec *rec);
void dump_slice_roi_rec(FILE *out, struct slice_roi_rec *rec);
void dump_roi_rec(FILE *out, struct roi_rec *rec);
void dump_curve_2d_rec(FILE *out, struct curve_2d_rec *rec);
void dump_curve_rec(FILE *out, struct curve_rec *rec);
void dump_float_bound_rec(FILE *out, struct float_bound_rec *rec);
void dump_matrix_2d_rec(FILE *out, struct matrix_2d_rec *rec);
#endif


/*  Defined in dir_fns.c (Automatically generated) */
extern int valid_dir(char *dir);
extern char *default_valid_dir(char *dir_fn, char *default_dir_fn);
extern char *parent_dir(char *dir);
extern char *complete_fn(char *fn);
extern char *cur_obsolute_fn(char *dir_fn, char *fn, int status);
extern int father_dir(char *dir, char **fatherdir);
extern int sort_strs_proc(char **str1, char **str2);
extern int get_fns(char *dir, char ***dirfns, unsigned int *num_dirfns,
    int select_dirfns, char *patt_dirfn, char ***fns, unsigned int *num_fns,
    int select_fns, char *patt_fn, char ***otherfns,
    unsigned int *num_otherfns, int select_otherfns, char *patt_otherfn);

#ifdef  STR_DEF_H     /* For struct strs_rec */
extern int son_dirs(char *dir, char *patt_dirfn, struct strs_rec **sondir);
extern int brother_dirs(char *dir, char *patt_dirfn,
    struct strs_rec **brotherdir);
#endif


/* Defined in fft.c (Automatically generated) */
#ifdef FFT_H     /* For struct complex_rec */
#ifdef COMMON_H  /* For struct buf_2d_rec */
extern void multiply_ft_by_val(struct complex_rec *ary,
    unsigned long num_elems, double val);
extern void divide_ft_by_val(struct complex_rec *ary, unsigned long num_elems,
    double val);
extern void multiply_complex(struct complex_rec *dest,
    struct complex_rec *src1, struct complex_rec *src2);
extern int mult_complex_mat(struct buf_2d_rec *dst, struct buf_2d_rec *src1,
    struct buf_2d_rec *src2);
extern int shift_fft_to_center(struct complex_rec *ary,
    unsigned long num_elems);
extern int shift_fft_back_to_origin(struct complex_rec *ary,
    unsigned long num_elems);
extern int perform_fft(struct complex_rec *ary, unsigned long num_elems,
    unsigned long num_bits);
extern int magnitude_fft(struct complex_rec *ary, unsigned long num_elems);
extern void conjugate_1d_ft(struct complex_rec *ary, unsigned long num_elems);
extern void inverse_fft(struct complex_rec *ary, unsigned long num_elems,
    unsigned long num_bits);
extern int setup_fft_2d(struct buf_2d_rec *fft_2d, struct buf_2d_rec *_2d);
extern int shift_fft_2d_to_center(struct buf_2d_rec *fft);
extern void get_fft_2d_col(struct buf_2d_rec *fft_col,
    struct buf_2d_rec *fft_2d, unsigned long col);
extern void save_fft_2d_col(struct buf_2d_rec *fft_col,
    struct buf_2d_rec *fft_2d, unsigned long col);
extern int fft_2d(struct buf_2d_rec *fft);
extern int ConjugateFftReal(struct buf_2d_rec *fft);
extern int ConjugateFftImaginary(struct buf_2d_rec *fft);
extern int ConjugateFft(struct buf_2d_rec *fft);
extern int inverse_fft_2d(struct buf_2d_rec *fft);
extern int MagnitudeFftTo2d(struct buf_2d_rec *_2d, struct buf_2d_rec *fft);
extern int get_fft_2d_min_max(struct buf_2d_rec *fft, float *min, float *max);
extern int AssignFftTo2d(struct buf_2d_rec *_2d, struct buf_2d_rec *fft);
extern int AssignMagnitudeFftTo2d(struct buf_2d_rec *_2d,
    struct buf_2d_rec *fft, double *min, double *max);
extern void pr_complex_rec(struct complex_rec *ary, unsigned long num_elems);
extern void pr_fft_2d(struct buf_2d_rec *fft);
extern int precondition_ideal_filter_fft_2d(struct buf_2d_rec *fft,
    double cut_off_freq, int is_lowpass);
extern int ideal_filter_fft_2d(struct complex_rec *comp, double cut_off_freq,
    int is_lowpass, double radius);
extern int butterworth_filter_fft_2d(struct complex_rec *comp,
    double cut_off_freq, int is_lowpass, double radius, int *order);
extern int exponential_filter_fft_2d(struct complex_rec *comp,
    double cut_off_freq, int is_lowpass, double radius, int *order);
extern int trapezoidal_filter_fft_2d(struct complex_rec *comp,
    double cut_off_freq, int is_lowpass, double radius,
    float *lb_cut_off_freq);
extern int filter_fft_2d(struct buf_2d_rec *fft, double cut_off_freq,
    int is_lowpass, int (*precond_proc)(), int (*filter_proc)(), void *arg1);
#endif
#endif


/* Defined in file_io.c  (Automatically generated) */
extern int file_set_size(FILE *fid, unsigned long size);
extern unsigned long get_file_size_by_fn(char *fn, int *err);
extern unsigned long get_file_size_by_fileno(int fnum, int *err);
extern unsigned long file_size(FILE *fid, int *err);
extern unsigned long the_file_size(char *dir, char *fn, int *err);
extern int frd(unsigned char *buf, size_t item_size, size_t num_items,
    FILE *fid, size_t *amt_read);
extern int rd_file_labels(FILE *fid, char ***strs, unsigned long num_strs);
extern int rd_file_strs(FILE *fid, char **strs, unsigned long num_strs);
extern int rd_file_ints(FILE *fid, unsigned long *ints,
    unsigned long num_ints);
extern int fwr(unsigned char *buf, size_t item_size, size_t num_items,
    FILE *fid, size_t *amt_written);
extern int wr_file_strs(FILE *fid, char **strs, unsigned long num_strs);
extern int wr_file_ints(FILE *fid, unsigned long *ints,
    unsigned long num_ints);
extern void file_access_err_proc(char **reason, void (*error_display_proc)(),
    char *err_str, int is_dynamic_str);
extern int file_access(char *fn, int mode, void (*error_display_proc)(),
    char **reason);
extern char *UniqueFn(char *dir, char *fn, char *suffix);
extern char *LastUniqueFn(char *dir, char *fn, char *suffix);
extern int delete_file(char *fn);
extern int delete_the_file(char *dir, char *fn, char *suffix);
extern void beep(void);
extern int reopen_file_as_read_update(FILE **fid, char *dirfn, char *fn,
    char *suffix, char **err_reason);


/* Defined in fn.c (Automatically generated) */
extern int locate_next_str_num(char *str, unsigned int *indx,
    int is_locate_first);
extern int build_fn(char **str, char *dirfn, char *fn, char *suffix);
extern char *locate_suffix(char *str);
extern int drop_suffix(char *str);
extern int drop_general_suffix(char *str, int suffix_start_ch);
extern char *locate_dirfn(char *str);
extern int drop_dirfn(char *str);
extern int break_fn(char *the_fn, char **dirfn, char **fn, char **suffix);
extern int GetNewFn(char **newfn, char *fn, char *newdir, char *prefix,
    char *newsuffix, unsigned long select);
/* FIXME!!  The following prototype is commented out because there are
multiple inconsistent definitions of get_mhcrc_num().  -PWH 19990706
extern int get_mhcrc_num(char *fn, char *mhcrc_num, int *mhcrc_start);
*/


/* Defined in geometry.c  (Automatically generated) */
#ifdef GEOMETRY_H      /* For struct line_rec */
extern void line_gradient(struct line_rec *line, float *gradient,
    int *is_infinity);
extern void pr_line_rec(struct line_rec *line);
extern void setup_line_rec(struct line_rec *line, double x1, double y1,
    double x2, double y2);
extern void setup_line_rec_by_gradient(struct line_rec *line, double x1,
    double y1, double gradient);
extern int apply_line_equation(struct line_rec *line, double x, double y);
extern void get_tangent_line(struct line_rec *tangent, double x, double y,
    struct line_rec *line);
extern void get_xy_intercept(struct line_rec *line, int *is_x_valid,
    float *xintercept, int *is_y_valid, float *yintercept);
extern int lines_parallel(struct line_rec *line1, struct line_rec *line2);
extern void get_lines_intersection(struct line_rec *line1,
    struct line_rec *line2, float *xintersect, float *yintersect,
    int *is_line_parallel);
extern void pt_distance_fr_line(struct line_rec *line, double x, double y,
    float *dist);
extern void angle_of_lines_intersection(struct line_rec *line1,
    struct line_rec *line2, float *angle, int *is_line_parallel);
extern void angle_between_line_and_y_axis(struct line_rec *line, float *angle,
    int *is_line_parallel);
extern void setup_line_segment_rec(struct line_segment_rec *segment,
    int x1, int y1, int x2, int y2);
extern int are_pts_divided(struct line_rec *line, double x1, double y1,
    double x2, double y2);
#endif


/* Defined in hdr_opr.c  (Automatically generated) */
#ifdef IA_HDR_H        /* For lots of different structs */
extern int setup_hdr_analysis_rec(struct hdr_analysis_rec *analysis,
    unsigned int num_img_labels, ...);
extern void init_color_control_rec(struct color_control_rec *color_control);
extern void init_extra_file_info_rec(struct extra_file_info_rec *extra_info);
extern void init_hdr_scan_entry_rec(struct hdr_scan_entry_rec *slice);
extern void free_hdr_scan_entry_rec(struct hdr_scan_entry_rec *slice);
extern void init_patient_rec(struct patient_rec *patient);
extern void free_patient_rec(struct patient_rec *patient);
extern void init_data_info_rec(struct data_info_rec *info);
extern void free_data_info_rec(struct data_info_rec *info);
extern void init_analysis_group_rec(struct analysis_group_rec *group);
extern void init_hdr_analysis_rec(struct hdr_analysis_rec *analysis);
extern void init_hdr_rec(struct hdr_rec *hdr);
extern void free_analysis_group_rec(struct analysis_group_rec *groups,
    unsigned int num_analysis_groups);
extern void free_hdr_analysis_rec(struct hdr_analysis_rec *analysis);
extern void free_hdr_scan_entry_recs(struct hdr_scan_entry_rec *scans,
    unsigned int num_scans);
extern void free_hdr_rec(struct hdr_rec *hdr);
extern void free_extra_file_info_rec(struct extra_file_info_rec *extra_info);
extern char get_plane_type_indicator(int plane_type);
extern char *get_plane_type(int plane_type);
extern char *get_patient_id(struct hdr_rec *hdr);
extern int build_default_datafn(struct hdr_rec *hdr);
extern void slice_entry_index_to_addr(struct hdr_scan_entry_rec *scans,
    unsigned int num_scans);
extern void slice_entry_addr_to_index(struct hdr_scan_entry_rec *scans,
    unsigned int num_scans);
extern int wr_extra_file_info_rec(FILE *fid,
    struct extra_file_info_rec *extra_info);
extern int wr_hdr(FILE *fid, struct hdr_rec *hdr);
extern int wr_the_hdr(char *fn, struct hdr_rec *hdr);
extern int wr_patient_rec(FILE *fid, struct patient_rec *patient);
extern int WrDataInfoRec(FILE *fid, struct data_info_rec *info);
extern int wr_analysis_group(FILE *fid, struct analysis_group_rec *group,
    unsigned int num_groups);
extern int wr_analysis_rec(FILE *fid, struct hdr_analysis_rec *analysis);
extern int wr_slice_entries(FILE *fid, struct hdr_scan_entry_rec *scans,
    unsigned int num_scans);
extern int rd_extra_file_info_rec(FILE *fid,
    struct extra_file_info_rec *extra_info);
extern int rd_patient_rec(FILE *fid, struct patient_rec *patient);
extern int RdDataInfoRec(FILE *fid, struct data_info_rec *info, char *verid);
extern int rd_analysis_group(FILE *fid, struct analysis_group_rec **group,
    unsigned int num_groups);
extern int rd_analysis_rec(FILE *fid, struct hdr_analysis_rec *analysis);
extern int RdSliceEntries(FILE *fid, struct hdr_scan_entry_rec **scans,
    unsigned int *num_scans, struct hdr_rec *hdr);
extern int rd_hdr(FILE *fid, struct hdr_rec *hdr);
extern int rd_the_hdr(char *fn, struct hdr_rec *hdr);
extern int rd_ria_hdr_params(char *fn, unsigned long *dimx,
    unsigned long *dimy, unsigned long *num_slices, unsigned long *num_echoes,
    float *xpixel_size, float *ypixel_size, float *zpixel_size,
    unsigned long *data_type, unsigned long *pixel_depth, double *min_pixel,
    double *max_pixel);
extern unsigned long NumberOfSliceScans(struct data_info_rec *info);
extern unsigned long NumberOfSliceCuts(struct data_info_rec *info);
extern int ValidateHdrParams(struct hdr_rec *hdr, unsigned long maxdimx,
    unsigned long maxdimy, unsigned long maxscans, unsigned long maxechoes);
extern int LocateScanEntry(struct hdr_rec *hdr, unsigned long slice_num,
    unsigned long scan_num, unsigned long echo_num,
    struct hdr_scan_entry_rec **scan_entry);
extern int FileHdrScan(struct hdr_rec *hdr, struct hdr_scan_entry_rec *scan,
    unsigned long *indx);
extern void ComputeScanEntryExtraInfo(struct hdr_rec *hdr);
extern int RedefineHdrNumBySliceNum(struct hdr_rec *hdr);
extern int ReverseHdrScans(struct hdr_rec *hdr);
extern unsigned long SetHdrScanEntryStatus(struct hdr_rec *hdr,
    unsigned long fsnum, unsigned long lsnum, unsigned long fecho,
    unsigned long lecho, unsigned long fscan, unsigned long lscan,
    int set_select, int status);
extern int cp_patient_rec(struct patient_rec *dest, struct patient_rec *src,
    int is_dest_new);
extern int cp_data_info_rec(struct data_info_rec *dest,
    struct data_info_rec *src, int is_dest_new);
extern int cp_analysis_group_rec(struct analysis_group_rec *dest,
    struct analysis_group_rec *src, int is_dest_new);
extern int cp_hdr_analysis_rec(struct hdr_analysis_rec *dest,
    struct hdr_analysis_rec *src, int is_dest_new);
extern int cp_extra_file_info_rec(struct extra_file_info_rec *dest,
    struct extra_file_info_rec *src, int is_dest_new);
extern int cp_hdr_scan_entry_rec(struct hdr_scan_entry_rec *dest,
    struct hdr_scan_entry_rec *src, int is_dest_new);
extern int cp_hdr_rec(struct hdr_rec *dest, struct hdr_rec *src,
    int is_dest_new);
extern unsigned long ImageFileSize(struct hdr_rec *hdr);
extern unsigned long VarInputDataTypeToBufType(unsigned long var_type);
extern unsigned long InputDataTypeToBufType(struct data_info_rec *info);
extern unsigned long BufTypeToInputDataType(unsigned long buf_type4);
extern unsigned long BufTypeToDepth(unsigned long buf_type4,
    unsigned long pixel_depth);
#endif


/* Defined in line.c  (Automatically generated) */
#ifdef COMMON_H         /* For struct buf_2d_rec */
#ifdef GEOMETRY_H       /* For matrix_2d_rec */
#ifdef SLICE_ROI_REC_H  /* For struct curve_rec */
#ifdef COORD_REC_H      /* For struct coord_rec */
extern void show_buf_pixels(struct buf_2d_rec *dest);
extern int DrawBufPts(unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, struct float_xy_rec *pts, long num_pts,
    long value, struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawBufPtsByMatrix(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, struct float_xy_rec *pts,
    long num_pts, struct matrix_2d_rec *matrix, long value,
    struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int draw_buf_xline(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, long x0, long y0, long x1,
    long y1, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int draw_buf_yline(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, long x0, long y0, long x1,
    long y1, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int DrawLineInBuf(unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, long x0, long y0, long x1, long y1,
    long value, struct coord_rec *coords, unsigned long *total_pts_drawn,
    int is_init_total_pts_drawn);
extern int DrawBufLines(unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, struct float_xy_rec *pts, long num_pts,
    long value, struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawBufLinesByMatrix(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, struct float_xy_rec *pts,
    long num_pts, struct matrix_2d_rec *matrix, long value,
    struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawCircleInBuf(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, long centerx, long centery,
    long radius, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int GetOctant(int coD, int coE);
extern void next_params(int oct, int *incd, int *incu, int *incv, int *k1,
    int *k2, int *k3, int coA, int coB, int coC, int coD, int coE, int coF,
    int x, int y);
extern int DrawConic(long centerx, long centery, long startx, long starty,
    long endx, long endy, long coA, long coB, long coC, long coD, long coE,
    long coF, unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int Conjugate(int centerx, int centery, int xp, int yp, int xq, int yq,
    int mode, unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int DrawCircleOrEllipse(unsigned char **buf, long rleft, long rtop,
    long rright, long rbottom, long pixel_depth, struct float_xy_rec *pts,
    long num_pts, unsigned long curve_type, long value,
    struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawCircleOrEllipseByMatrix(unsigned char **buf, long rleft,
    long rtop, long rright, long rbottom, long pixel_depth, struct float_xy_rec *pts, long num_pts, unsigned long curve_type, struct matrix_2d_rec *matrix, long value, struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int draw_rect_in_buf(unsigned char **buf, long rleft, long rtop,
    long rright, long rbottom, long pixel_depth, long left1, long top1,
    long right1, long bottom1, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int DrawBufRect(unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, struct float_xy_rec *pts, long num_pts,
    long value, struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawBufRectByMatrix(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, struct float_xy_rec *pts,
    long num_pts, struct matrix_2d_rec *matrix, long value,
    struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawInBuffer(unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, struct float_xy_rec *pts, long num_pts,
    unsigned long curve_type, long value, struct coord_rec *coords,
    unsigned long *total_pts_drawn);
extern int DrawInBufByMatrix(unsigned char **buf, long left, long top,
    long right, long bottom, long pixel_depth, struct float_xy_rec *pts,
    long num_pts, unsigned long curve_type, struct matrix_2d_rec *matrix,
    long value, struct coord_rec *coords, unsigned long *total_pts_drawn);
extern int DrawInBuf(unsigned char **buf, long left, long top, long right,
    long bottom, long pixel_depth, struct float_xy_rec *pts, long num_pts,
    unsigned long curve_type, struct matrix_2d_rec *matrix, int use_matrix,
    long value, struct coord_rec *coords, unsigned long *total_pts_drawn,
    struct matrix_2d_rec *sroi_inv);
extern int XorBitmapByCurvePts(struct buf_2d_rec *dest,
    struct curve_rec **curves, unsigned int num_curves,
    struct matrix_2d_rec *sroi_inv);
extern int XorBitmapByCurveLines(struct buf_2d_rec *dest,
    struct curve_rec **curves, unsigned int num_curves,
    struct matrix_2d_rec *sroi_inv);
extern int XorBitmapByCurves(struct buf_2d_rec *dest,
    struct curve_rec **curves, unsigned int num_curves,
    struct matrix_2d_rec *sroi_inv, int is_line);
#endif
#endif
#endif
#endif


/* Defined in mark_hdr_opr.c  (Automatically generated) */
#ifdef MARK_HDR_H    /* For struct mark_hdr and friends */
extern void init_makr_hdr_key(struct mark_hdr_key *key);
extern void init_mark_patient_rec(struct mark_patient_rec *patient);
extern void init_mark_hdr_history(struct mark_hdr_history *history);
extern void init_mark_hdr_dimension(struct mark_hdr_dimension *dimen);
extern void init_mark_hdr(struct mark_hdr *hdr);
extern int wr_the_mark_hdr(char *fn, struct mark_hdr *mark);
extern int load_hdr_file(char *fn, struct mark_hdr *db);
#endif


/* Defined in marktonhdr.c (Automatically generated) */
#ifdef IA_HDR_H          /* For struct extra_file_info_rec */
extern int build_extra_file_info_rec(struct extra_file_info_rec *extra_info);
#endif

#ifdef IA_HDR_H          /* For struct hdr_rec */
#ifdef MARK_HDR_H        /* For struct mark_hdr */
extern int assign_hdr_to_nhdr(struct mark_hdr *mark, struct hdr_rec *hdr);
extern int build_pr_t2_hdr(struct hdr_rec *hdr, struct mark_hdr *mark_hdr);
extern int build_t1_hdr(struct hdr_rec *hdr, struct mark_hdr *mark_hdr);
extern int build_hdr_from_mark_hdr(struct hdr_rec *hdr,
    struct mark_hdr *mark_hdr, int num_echoes);
extern int SetupHdrFromMarkHdr(struct hdr_rec *hdr, struct mark_hdr *mark_hdr);
#endif
#endif


/* Defined in match.c  (Automatically generated) */
extern int fb_is_match_vars(char *str);
extern int fb_compatible_vars(char *stra, char *strb);
extern char *locate_val(char *str, size_t len);
extern int fb_match(char *patt, char *str);


/* Defined in math_opr.c (Automatically generated) */
extern int is_float_0(double fl);
extern int is_double_0(double dou);
extern void DoubleMinMax(double *min, double *max);
extern double float_precision(double fl, int num_decimals);
extern unsigned long find_num_bytes_needed(unsigned long num);
extern unsigned long num_bit_values(unsigned long pixel_depth);
extern unsigned long find_num_bits_needed(unsigned long num);
extern unsigned long NumBitsNeeded(double min, double max,
    unsigned long *type4);
extern void MinMaxValues(unsigned long select4, unsigned long pixel_depth,
    double *min, double *max);
extern double degree(double radian);
extern double degree_1(double radian, int num_decimals);
extern double radian(double degree);
extern double radian_1(double degree, int num_decimals);
extern double DegCosRadian(double degree);
extern double acos_to_degree(double val);
extern double cos_degree(double degree);
extern double distance(double x1, double y1, double x2, double y2);
extern void perpendicular_pt(double x1, double y1, double x2, double y2,
    double x3, double y3, double *ptx, double *pty);
extern long LongRound(double num);
extern unsigned long ULongRound(double num);
extern double Sqrt(double num);
extern double Atan2(double y, double x);
extern void get_equation(float eq[3], double x1, double y1, double x2
   , double y2);
extern float fn_xy(float eq[3], double x, double y);
extern short get_2d_member_sign(float eq[3], double x, double y);


/* Defined in misc.c (COMPLETE) */
extern void make_int_in_range(int *num, int min, int max);


/* Defined in more_str_opr.c (Automatically generated) */
#ifdef STR_DEF_H      /* For struct strs_rec and friends */
extern int add_sort_no_dup_strs_rec(struct strs_rec *result,
    int is_sort_and_no_dup, unsigned long num_srcs, ...);
extern int build_strs_rec(struct strs_rec *ptr, unsigned long cur_str,
    unsigned long total_strs, ...);
extern void init_strs_rec(struct strs_rec *ptr);
extern void free_strs_rec(struct strs_rec *ptr);
extern void free_strs_rec_ptrs(struct strs_rec *ptr);
extern int add_strs_rec_entries(struct strs_rec *ptr,
    unsigned int add_num_entries);
extern int add_str_ptr_to_strs_rec(struct strs_rec *ptr, char *str);
extern int AddStrPtrToSortedStrsRec(struct strs_rec *ptr, char *str,
    int set_cur, int found_is_noerror);
extern int add_str_to_strs_rec(struct strs_rec *ptr, char *str);
extern int AddStrToSortedStrsRec(struct strs_rec *ptr, char *str, int set_cur,
    int found_is_noerror);
extern int add_str_ary_to_strs_rec(struct strs_rec *ptr, char **strs,
    long num_strs);
extern int add_str_ptr_ary_to_strs_rec(struct strs_rec *ptr, char **strs,
    long num_strs);
extern int add_str_tokens_to_strs_rec(struct strs_rec *ptr, char *str);
extern int add_str_ptr_ary_to_sorted_strs_rec(struct strs_rec *sorted,
    char **strs, long num_strs, int set_cur);
extern int cp_strs_rec_1(struct strs_rec *dest, struct strs_rec *src);
extern int cp_strs_rec(struct strs_rec *dest, struct strs_rec *src);
extern int append_strs_rec(struct strs_rec *dest, struct strs_rec *src);
extern int cp_strs_rec_ptrs_1(struct strs_rec *dest, struct strs_rec *src);
extern int cp_strs_rec_ptrs(struct strs_rec *dest, struct strs_rec *src);
extern int append_strs_rec_ptrs(struct strs_rec *dest, struct strs_rec *src);
extern int common_ptrs_in_sorted_strs_rec(struct strs_rec *result,
    struct strs_rec *sorted, struct strs_rec *src);
extern int diff_ptrs_in_sorted_strs_rec(struct strs_rec *result,
    struct strs_rec *sorted, struct strs_rec *src);
extern int sort_strs_ascendingly_proc(char **str1, char **str2);
extern void sort_strs_rec_ascendingly(struct strs_rec *ptr);
extern void sort_strs_rec(struct strs_rec *ptr, int (*sort_proc)());
extern int position_str_in_strs_rec(struct strs_rec *ptr, char *str,
    unsigned long posi);
extern int find_str_in_strs_rec(struct strs_rec *ptr, char *str,
    unsigned long *indx);
extern int FindStrInSortedStrsRec(struct strs_rec *sorted, char *str,
    unsigned long *indx);
extern int drop_indx_str_ptr_from_strs_rec(struct strs_rec *ptr,
    unsigned long indx);
extern int drop_str_ptr_from_strs_rec(struct strs_rec *ptr, char *str);
extern int drop_str_from_strs_rec(struct strs_rec *ptr, char *str);
extern int DropStrPtrInSortedStrsRec(struct strs_rec *sorted, char *str);
extern int DropStrInSortedStrsRec(struct strs_rec *sorted, char *str);
extern int drop_dup_str_from_strs_rec(struct strs_rec *ptr);
extern int setup_strs_rec(struct strs_rec **ptr, unsigned int num_strs,
    unsigned int cur_str);
extern void drop_strs_rec_suffix(struct strs_rec *labels);
extern int get_strs_rec_max_str_len(struct strs_rec *labels);
extern int get_strs_rec_max_some_str_len(struct strs_rec *labels,
    unsigned int *selected);
extern void pr_strs_rec(struct strs_rec *ptr);
extern void init_str_value_rec(struct str_value_rec *ptr);
extern void free_str_value_rec(struct str_value_rec *ptr);
extern int sort_str_value_rec_proc(struct str_value_rec *val1,
    struct str_value_rec *val2);
extern int setup_str_value_rec(struct str_value_rec *ptr, char *str,
    long value);
extern int setup_str_value_rec_ary(struct str_value_rec **ary, long *num_elems,
    char *str, long value);
extern void free_str_value_rec_ary(struct str_value_rec *ary, long num_elems);
extern int selected_strs(struct strs_rec *set, struct strs_rec *selected_set,
    unsigned char **selected);
extern int cmp_str_arys_no_order(char **ary1, long num_elems1, char **ary2,
    long num_elems2, int *result);
extern int cmp_str_arys(char **ary1, long num_elems1, char **ary2,
    long num_elems2, int *result);
extern int rd_strs_rec(FILE *fid, struct strs_rec *ptr);
extern int rd_ascii_file_strs_to_strs_rec(char *label_fn,
    struct strs_rec *labels, int is_sort);
extern int add_double_to_strs_rec(struct strs_rec *ptr, double fl_num,
    int num_decimals);
extern int add_long_to_strs_rec(struct strs_rec *ptr, long long_num);
extern int longest_str_len(struct strs_rec *ptr, long *longest_indx);
extern int wr_strs_rec(FILE *fid, struct strs_rec *ptr);
#endif


/* Defined in ndat_v3.c  (Automatically generated) */
#ifdef MRI_FORMAT_H       /* For all sorts of stuff */
extern int rd_output_slice_nums(FILE *fid, unsigned int **slice_nums,
    unsigned int *num_slice_rois);
extern int rd_output_field_offset_rec(FILE *fid,
    struct output_field_offset_rec *offset);
extern int rd_output_hdr_rec(FILE *fid, struct output_hdr_rec *hdr,
    int (*hdr_error_proc)());
extern int rd_output_slice_rois(FILE *fid, struct slice_roi_rec **slice_rois,
    unsigned int actual_total_slice_rois, unsigned int *slice_nums,
    unsigned int num_slice_rois);
extern int rd_histo_pixel_center_recs(FILE *fid,
    struct pixel_center_rec **centers, unsigned int *num_centers,
    struct slice_roi_rec *slice_roi);
extern int rd_histo_bkg_rec(FILE *fid, struct histo_bkg_rec *bkg);
extern int rd_histo_params(FILE *fid, struct histo_params_rec *histo);
extern int rd_pixel_type_cnt_rec(FILE *fid,
    struct pixel_type_cnt_rec *pixel_cnt);
extern int rd_roi_pixel_type_cnts(FILE *fid, struct slice_roi_rec *slice_rois,
    unsigned int num_slice_rois);
extern int rd_pixel_type_params(FILE *fid, struct pixel_type_rec *pixel_type);
extern int rd_text_output_analysis_group_rec(FILE *fid,
    struct text_output_analysis_group_rec *group);
extern int rd_input_text_file_format(FILE *fid,
    struct text_output_analysis_group_rec **groups, unsigned int *num_groups);
extern int rd_output_field_rec(FILE *fid, struct output_field_rec *group);
extern int rd_output_count_field_rec(FILE *fid,
    struct output_count_field_rec *group);
extern int rd_output_count_group_rec(FILE *fid,
    struct output_count_group_rec *count);
extern int rd_output_text_file_format(FILE *fid,
    struct output_text_format_rec *format);
extern int rd_roi_count_anal_rec(FILE *fid, struct roi_count_anal_rec *roi,
    int is_new_format);
extern int rd_slice_count_anal_rec(FILE *fid,
    struct slice_count_anal_rec *slice);
extern int rd_slice_count_anal_recs(FILE *fid,
    struct slice_count_anal_rec **slices, unsigned long *total_slices,
    unsigned long *num_slices);
extern int rd_count_analysis_rec(FILE *fid, struct count_analysis_rec *count);
extern int cp_dat_output_hdr_rec(struct output_hdr_rec *hdr,
    struct output_file_rec *output);
extern int cp_dat_slice_roi_recs(struct slice_roi_rec *slice_rois,
    unsigned int *slice_nums, unsigned int num_slice_rois,
    unsigned int actual_total_slice_rois, struct input_file_rec *file);
extern int cp_nth_slice_roi_rec(struct slice_roi_rec *slice_rois,
    unsigned int *slice_nums, unsigned int num_slice_rois,
    struct input_file_rec *file, long slice_num);
extern int cp_dat_histo_params_rec(struct histo_params_rec *histo,
    struct input_file_rec *file);
extern int cp_dat_pixel_type_rec(struct pixel_type_rec *pixel_type,
    struct input_file_rec *file);
extern int cp_text_output_analysis_group_rec(
   struct text_output_analysis_group_rec *groups, unsigned int num_groups,
    struct input_file_rec *file);
extern void cp_output_text_format_rec(struct output_text_format_rec *format,
    struct input_file_rec *file);
extern void cp_count_analysis_rec(struct count_analysis_rec *count,
    struct input_file_rec *file);
extern void hdr_error_proc(void);
extern int RdNdatFields(FILE *fid, struct output_hdr_rec *hdr,
    struct slice_roi_rec **slice_rois, long *total_slice_rois,
    struct histo_params_rec *histo, struct pixel_type_rec *pixel_type,
    struct text_output_analysis_group_rec **groups, unsigned int *num_groups,
    struct output_text_format_rec *format, struct count_analysis_rec *count);
extern void free_ndat_fields(struct output_hdr_rec *hdr,
    struct slice_roi_rec *slice_rois, long total_slice_rois,
    struct histo_params_rec *histo, struct pixel_type_rec *pixel_type,
    struct text_output_analysis_group_rec *groups, unsigned int num_groups,
    struct output_text_format_rec *format, struct count_analysis_rec *count);
extern int RdTheNdatFields(char *fn, struct output_hdr_rec *hdr,
    struct slice_roi_rec **slice_rois, long *total_slice_rois,
    struct histo_params_rec *histo, struct pixel_type_rec *pixel_type,
    struct text_output_analysis_group_rec **groups, unsigned int *num_groups,
    struct output_text_format_rec *format, struct count_analysis_rec *count);
extern int rd_dat_file(char *fn, struct output_file_rec *output);
extern int rd_dat_file_slice_rois(char *fn, struct input_file_rec *file);
extern int rd_file_slice_roi_rec(char *fn, struct input_file_rec *file,
    long slice_num);
extern int save_slice_roi_rec(char *dir, char *fn,
    struct slice_roi_rec *slice_roi);
extern int Wr2dImage(char *fn, struct buf_2d_rec *_2d, char *img_label,
    char **err_reason, struct hdr_rec *hdr, double xpix, double ypix,
    double zpix);
extern int init_default_output_hdr_info_rec(struct output_hdr_info_rec *info,
    struct input_file_rec *file);
extern void init_output_file_rec(struct output_file_rec *output,
    struct input_file_rec *file);
extern int setup_output_fn(struct output_file_rec *output, char *dirfn,
    char *fn, char *dat_fn_suffix, char *bin_fn_suffix);
extern int change_output_fn(struct output_file_rec *output, char *output_fn,
    char *bin_fn_suffix);
extern int rd_dat_slice_rois(struct output_file_rec *output, int is_rd_all,
    long slice_num);
extern void close_output_file_rec(struct output_file_rec *output);
extern int get_output_fn(struct output_file_rec *output, char **fn);
extern int wr_dat_file(char *fn, struct output_file_rec *output);
extern int check_file_save(struct output_file_rec *output);
extern void set_dat_file_changed(struct input_file_rec *file);
extern void set_dat_file_saved(struct input_file_rec *file, int is_save);
extern void flush_output_save_status(struct output_file_rec *output);
extern void init_pixmap_hdr_rec(struct pixmap_hdr_rec *hdr);
extern void free_pixmap_hdr_rec(struct pixmap_hdr_rec *hdr);
extern void pr_pixmap_hdr_rec(struct pixmap_hdr_rec *hdr);
extern int RdPixmapHdr(char *fn, struct pixmap_hdr_rec *hdr);
extern int RdThePixmapHdr(char *dir, char *fn, char *suffix,
    struct pixmap_hdr_rec *pixmap_hdr);
extern int rd_pixmap_slices(char *fn, struct pixmap_hdr_rec *hdr,
    struct buf_2d_rec **slice_maps, unsigned long num_slice_maps);
extern int rd_pixmap_for_slice_num(char *fn, struct pixmap_hdr_rec *hdr,
    struct buf_2d_rec *slice_map, int slice_num);
extern int wr_updated_pixmap_params(FILE *fid, struct pixmap_hdr_rec *hdr);
extern int wr_pixmap_hdr(char *fn, struct pixmap_hdr_rec *hdr);
extern int WrPixmap(char *fn, struct strs_rec *order,
    struct buf_2d_rec **slice_maps, unsigned long num_slice_maps,
    struct pixmap_hdr_rec *hdr);
extern int is_compatible_pixmap(struct pixel_type_rec *pixel_type, long dimx,
    long dimy);
extern int append_wr_file_pixmap(char *dir, char *fn, char *suffix,
    struct pixel_type_rec *pixel_type, int dimx, int dimy, char **err_reason);
extern int wr_file_pixmap_for_slice_num(char *dir, char *fn, char *suffix,
    struct pixel_type_rec *pixel_type, int dimx, int dimy, long num_slices,
    int slice_map, int slice_num, char **err_reason);
extern int wr_file_pixmap(char *dir, char *fn, char *suffix,
    struct pixel_type_rec *pixel_type, char **err_reason);
extern int v3_rd_curve_rec(FILE *fid, struct curve_rec *curve);
extern int v3_rd_curve_2d_rec(FILE *fid, struct curve_2d_rec *pen);
extern int v3_rd_roi_rec(FILE *fid, struct roi_rec *roi);
extern int v3_rd_slice_roi_rec(FILE *fid, struct slice_roi_rec *slice_roi);
extern int v3_wr_curve_rec(FILE *fid, struct curve_rec *curve);
extern int v3_wr_curve_2d_rec(FILE *fid, struct curve_2d_rec *pen);
extern int v3_wr_roi_rec(FILE *fid, struct roi_rec *roi);
extern int v3_wr_slice_roi_rec(FILE *fid, struct slice_roi_rec *slice_roi);
extern void init_output_hdr_info_rec(struct output_hdr_info_rec *info);
extern void init_output_hdr_rec(struct output_hdr_rec *hdr);
extern void free_output_hdr_info_rec(struct output_hdr_info_rec *info);
extern void free_output_hdr_rec(struct output_hdr_rec *hdr);
extern int wr_output_field_offset_rec(FILE *fid,
    struct output_field_offset_rec *offset);
extern int wr_output_hdr_rec(FILE *fid, struct output_hdr_rec *hdr,
    unsigned long *offset);
extern int wr_output_slice_nums(FILE *fid, unsigned long *slice_nums,
    unsigned long num_slice_rois);
extern int wr_histo_bkg_rec(FILE *fid, struct histo_bkg_rec *bkg);
extern int wr_histo_params(FILE *fid, struct histo_params_rec *histo);
extern int wr_text_output_analysis_group_rec(FILE *fid,
    struct text_output_analysis_group_rec *group);
extern int wr_pixel_type_cnt_rec(FILE *fid,
    struct pixel_type_cnt_rec *pixel_cnt);
extern int wr_pixel_type_params(FILE *fid, struct pixel_type_rec *pixel_type);
extern int wr_input_text_file_format(FILE *fid,
    struct text_output_analysis_group_rec *groups, unsigned long num_groups);
extern int wr_output_field_rec(FILE *fid, struct output_field_rec *group);
extern int wr_output_count_field_rec(FILE *fid,
    struct output_count_field_rec *group);
extern int wr_output_count_group_rec(FILE *fid,
    struct output_count_group_rec *count);
extern int wr_output_text_file_format(FILE *fid,
    struct output_text_format_rec *format);
extern int wr_text_file_format(FILE *fid, struct text_output_rec *text);
extern int wr_roi_count_anal_rec(FILE *fid, struct roi_count_anal_rec *roi);
extern int wr_slice_count_anal_rec(FILE *fid,
    struct slice_count_anal_rec *slice);
extern int wr_slice_count_anal_recs(FILE *fid,
    struct slice_count_anal_rec *slices, unsigned long num_slices);
extern int wr_count_analysis_rec(FILE *fid, struct count_analysis_rec *count);
extern int wr_ria_slice_rois(FILE *fid, struct slice_num_and_roi_rec *slices,
    long num_slices);
extern int setup_slice_nums(struct output_hdr_rec *hdr,
    struct slice_num_and_roi_rec *slices, long num_slices, long total_slices);
extern int wr_ria_dat_file(char *fn, struct output_hdr_rec *hdr,
    struct slice_num_and_roi_rec *slices, long num_slices, long total_slices,
    struct histo_params_rec *histo, struct pixel_type_rec *pixel_type,
    struct text_output_rec *text, struct count_analysis_rec *count);
extern int setup_slice_num_and_roi_rec(struct slice_roi_rec *slice_rois,
    long total_slices, struct slice_num_and_roi_rec **slices,
    long *num_slices);
extern int WrRiaNdatFile(char *fn, struct output_hdr_rec *hdr,
    struct slice_roi_rec *slice_rois, long total_slices,
    struct histo_params_rec *histo, struct pixel_type_rec *pixel_type,
    struct text_output_analysis_group_rec *groups, unsigned long num_groups,
    struct output_text_format_rec *format, struct count_analysis_rec *count);
#endif


/* Defined in num_v1.c  (Automatically generated) */
extern char hex(unsigned long num);
extern char *byte_hex(unsigned long num);
extern char *short_hex(int num);
extern char *long_hex(unsigned long num);
extern char *buf_hex(unsigned char *buf, unsigned long num_elems);


/* Defined in plane.c (Automatically generated) */
#ifdef GEOMETRY_H     /* For struct plane_d_rec and friends */
extern void init_plane_3d_rec(struct plane_3d_rec *plane);
extern void calc_plane_const(struct plane_3d_rec *plane);
extern int closer_to_plane_origin(struct vector_3d_rec *pt1,
    struct vector_3d_rec *pt2);
extern void get_3d_rect_base(struct vector_3d_rec bounds[4], int *indx);
extern void closest_pt_to_origin(struct vector_3d_rec bounds[4], int *indx);
extern int GlobalXYZ(struct plane_3d_rec *plane, double localx, double localy,
    double localz, float *globalx, float *globaly, float *globalz);
extern int GlobalPt(struct plane_3d_rec *plane, struct vector_3d_rec *localpt,
    struct vector_3d_rec *globalpt);
extern int LocalXYZ(struct plane_3d_rec *plane, double globalx, double globaly,
    double globalz, float *localx, float *localy, float *localz);
extern int LocalPt(struct plane_3d_rec *plane, struct vector_3d_rec *globalpt,
    struct vector_3d_rec *localpt);
extern int LocalToGlobalXYZ(struct matrix_3d_rec *coord_matrix,
    struct vector_3d_rec *coord_ratio, double x, double y, double z,
    float *proj_ptx, float *proj_pty, float *proj_ptz, int is_result_rounded);
extern int GlobalToLocalXYZ(struct matrix_3d_rec *coord_matrix,
    struct vector_3d_rec *coord_ratio, double x, double y, double z,
    float *proj_ptx, float *proj_pty, float *proj_ptz, int is_result_rounded);
extern int define_plane_orientation(struct plane_3d_rec *plane, int select);
extern void redefine_plane_orientation(struct plane_3d_rec *plane,
    long last_proj_dimx, long last_proj_dimy);
extern int define_default_plane_orientation(struct plane_3d_rec *plane);
extern int define_bound_vertices(struct plane_3d_rec *plane,
    struct vector_3d_rec pts[4]);
extern void DefinePlaneDim(struct plane_3d_rec *plane);
extern int DefinePlaneTransformation(struct plane_3d_rec *plane);
extern void set_plane_orientation(struct plane_3d_rec *plane, int select);
extern void define_obl_matrix(struct plane_3d_rec *plane, double obl_deg,
    double obj_proj_deg);
extern void DefinePlaneAttrs(struct vector_3d_rec *base,
    struct vector_3d_rec *normal_cosine, double startx, double endx,
    double starty, double endy, double startz, double endz, long *tot_planes,
    long *plane_num, struct vector_3d_rec *plane_0_vector,
    struct vector_3d_rec *plane_max_vector);
extern void PlaneAttrs(struct vector_3d_rec *base,
    struct vector_3d_rec *center, struct vector_3d_rec *normal_cosine,
    double startx, double endx, double starty, double endy, double startz,
    double endz, long *tot_planes, long *plane_num,
    struct vector_3d_rec *plane_0_vector,
    struct vector_3d_rec *plane_max_vector, int is_adjust_to_center);
extern void define_plane_trans(struct plane_3d_rec *plane,
    int is_init_plane_trans);
extern void DefPlaneRotation(struct plane_3d_rec *plane);
extern int PtNormalPlane(struct plane_3d_rec *plane, double x, double y,
    double z, double normalx, double normaly, double normalz, double startx,
    double endx, double starty, double endy, double startz, double endz,
    long buf_dimx, long buf_dimy, struct vector_3d_rec *translate,
    struct vector_3d_rec *scale, struct vector_3d_rec *rotate,
    struct vector_3d_rec *center, double obl_deg, double obj_proj_deg,
    double xratio, double yratio, double zratio,
    struct matrix_3d_rec *global_matrix, int is_def_trans,
    int is_adjust_to_center, unsigned long is_approx_value);
extern int ChangePlane(struct plane_3d_rec *plane, double x, double y,
    double z, double normalx, double normaly, double normalz,
    struct vector_3d_rec *center, struct vector_3d_rec *rotate,
    int sel_adjust_to_center, struct matrix_3d_rec *xyztoplane);
extern int is_same_plane(struct plane_3d_rec *gplane1,
    struct plane_3d_rec *gplane2);
extern int adjust_to_center(struct plane_3d_rec *plane, int adjust);
#endif


/* Defined in plane_opr.c  (Automatically generated) */
#ifdef GEOMETRY_H   /* For struct vecctor_3d_rec and friends */
extern float DistOfPtFromPlane(struct vector_3d_rec *base,
    struct vector_3d_rec *normal, struct vector_3d_rec *pt, int *sign);
extern long max_plane_dims(struct plane_3d_rec *plane);
extern void set_plane_move_params(struct plane_3d_rec *plane,
    double x, double y);
extern void pr_plane_corner_pts(struct plane_3d_rec *plane);
extern void pr_plane_3d_rec(struct plane_3d_rec *plane);
extern int RdPlaneParams(FILE *fid, struct vector_3d_rec *base,
    struct vector_3d_rec *normal, struct vector_3d_rec *center,
    struct vector_3d_rec *coord_ratio, struct vector_3d_rec *rotate,
    int *which_format, unsigned long *adjust_to_center,
    struct matrix_3d_rec *xyztoplane);
extern int WrPlaneParams(FILE *fid, struct vector_3d_rec *base,
    struct vector_3d_rec *normal, struct vector_3d_rec *center,
    struct vector_3d_rec *coord_ratio, struct vector_3d_rec *rotate,
    int which_format, unsigned long adjust_to_center,
    struct matrix_3d_rec *xyztoplane);
extern char *PlaneLabel(struct vector_3d_rec *base,
    struct vector_3d_rec *normal, struct vector_3d_rec *center,
    struct vector_3d_rec *coord_ratio, struct vector_3d_rec *rotate,
    int which_format, unsigned long adjust_to_center);
extern char *get_plane_img_label(struct plane_3d_rec *plane);
extern int DecodePlaneLabel(char *label, struct vector_3d_rec *base,
    struct vector_3d_rec *normal, struct vector_3d_rec *center,
    struct vector_3d_rec *coord_ratio, struct vector_3d_rec *rotate,
    int *which_format, unsigned long *adjust_to_center);
extern void CenterOf2Lines(struct vector_3d_rec *base1,
    struct vector_3d_rec *dire1, struct vector_3d_rec *base2,
    struct vector_3d_rec *dire2, struct vector_3d_rec *center);
extern int plane_and_1(struct vector_3d_rec *plane_normal1, double plane_k1,
    struct vector_3d_rec *plane_normal2, double plane_k2,
    struct vector_3d_rec *and_base, struct vector_3d_rec *and_direction);
extern int plane_lines_intersection(struct vector_3d_rec *base1,
    struct vector_3d_rec *direction1, struct vector_3d_rec *base2,
    struct vector_3d_rec *direction2, struct vector_3d_rec *and);
extern int plane_line_and_bound_intersection(struct vector_3d_rec bounds[4],
    struct vector_3d_rec *line_base, struct vector_3d_rec *line_direction,
    struct vector_3d_rec *startpt, struct vector_3d_rec *endpt,
    int is_result_rounded);
extern int PlaneLineAndBoundIntersection(struct plane_3d_rec *plane,
    struct vector_3d_rec *line_base, struct vector_3d_rec *line_direction,
    struct vector_3d_rec *and1, struct vector_3d_rec *and2,
    int is_result_rounded);
extern int OnPlane(struct vector_3d_rec *plane_coord,
    struct vector_3d_rec *plane_normal, struct vector_3d_rec *pt);
extern void PlaneAndLineIntersection(struct vector_3d_rec *plane_coord,
    struct vector_3d_rec *plane_normal, struct vector_3d_rec *line_base,
    struct vector_3d_rec *line_direction, struct vector_3d_rec *and_pt,
    float *line_move_unit_ratio, int *is_and_status);
extern void max_normal_extents(struct vector_3d_rec *plane_coord,
    struct vector_3d_rec *plane_normal, double startx, double endx,
    double starty, double endy, double startz, double endz,
    struct vector_3d_rec *startpt, struct vector_3d_rec *endpt);
extern int one_valid_plane_pt(struct vector_3d_rec *base,
    struct vector_3d_rec *normal, struct vector_3d_rec *local_start,
    struct vector_3d_rec *local_end, struct vector_3d_rec *pt);
extern int locate_plane(struct plane_3d_rec *plane, long new_plane_num,
    int is_plane_num_relative);
extern int PlaneXYZToLocalXYZ(struct plane_3d_rec *plane, double planex,
    double planey, double planez, float *localx, float *localy, float *localz,
    int is_round);
extern int LocalXYZToPlaneXYZ(struct plane_3d_rec *plane, double localx,
    double localy, double localz, float *planex, float *planey, float *planez);
extern int base_normal_after_angle_cut(struct plane_3d_rec *plane, double x1,
    double y1, double x2, double y2, double cut_deg,
    struct vector_3d_rec *base, struct vector_3d_rec *normal);
extern int base_normal_after_axis_rotation(struct plane_3d_rec *plane,
    double x1, double y1, double x2, double y2, double cut_deg,
    struct vector_3d_rec *base, struct vector_3d_rec *normal);
extern int natural_zplane(struct plane_3d_rec *plane);
extern int pt_in_volume(struct vector_3d_rec *local_start,
    struct vector_3d_rec *local_end, double x, double y, double z);
extern int plane_pt_in_3d(struct plane_3d_rec *plane, double x, double y,
    double z);
extern int largest_plane(struct plane_3d_rec *plane, long *firstx,
    long *lastx, long *firsty, long *lasty, int direction);
extern int PlaneIntersection(struct plane_3d_rec *plane1,
    struct plane_3d_rec *plane2, struct vector_3d_rec *and_base,
    struct vector_3d_rec *and_direction);
extern int IsSameNormalAndOverlapped(struct plane_3d_rec *plane1,
    struct plane_3d_rec *plane2);
extern void reverse_plane(struct plane_3d_rec *plane);
extern void move_center(struct plane_3d_rec *plane,
    struct vector_3d_rec *new_rel_globalcenter,
    struct vector_3d_rec *old_rel_globalcenter, int is_rel);
extern int move_center_by_planexyz(struct plane_3d_rec *plane, double planex,
    double planey, double planez, struct vector_3d_rec *center);
extern void RotatePlaneLine(struct plane_3d_rec *plane, double planex1,
    double planey1, double planex2, double planey2, double rot_deg,
    float *rplanex1, float *rplaney1, float *rplanex2, float *rplaney2);
#endif



/* Defined in ptr_and_dim.c  (Automatically generated) */
extern void init_ptr_ary(void **ptr, unsigned long num_elems,
    void *init_ptr_val);
extern void deallocate_2d_ary(char **ary, unsigned long elems);
extern unsigned long get_handle_memory(char ***ptr,
    unsigned long *total_items, unsigned long add_num_items, size_t item_size,
    int *err);
extern int get_handle(char ***ptr, unsigned long *total_items,
    unsigned long add_num_items, size_t item_size);
extern unsigned long get_ptr_memory(void **ptr, unsigned long *total_items,
    unsigned long add_num_items, size_t item_size, int *err);
extern int get_ptr(void **ptr, unsigned long *total_items,
    unsigned long add_num_items, size_t item_size);
extern void free_handle_memory(char **ptr, unsigned long total_items);
extern void free_handle_elements(char **ptr, unsigned long total_items);
extern int add_handle_element(void ***handle, unsigned long *num_elems,
    void *ptr);
extern unsigned long find_ptr_element1(char *ptr, unsigned long num_elems,
    size_t elem_size, char *element_ptr, int *err);
extern unsigned long find_handle_element(char **handle,
    unsigned long num_elems, char *ptr, int *err);
extern int FindPtrInHandleAry(void **ary, unsigned long num_elems, void *ptr,
    unsigned long *ptr_posi);
extern unsigned long FindStrInHandleAryByContent(char **handle,
    unsigned long num_elems, char *ptr, int *err);
extern int DropPtrAryPtr(char **root, unsigned int *num_elems,
    size_t elem_size, char *ptr);
extern int DropHandleAryPtr(char ***ary, unsigned long *num_elems, char *ptr);
extern int DropPtrFromHandleAry(void **ary, unsigned long *num_elems,
    void *ptr);
extern void c_free(void *ptr);
extern void free_ptr(unsigned char **ptr);
extern int cp_ptrs(unsigned char ***new_handle, unsigned char **old_handle,
    unsigned long num_ptrs);
extern int cp_unsigned_int_ary(unsigned int **new, unsigned int *old,
    unsigned long num_elems);
extern unsigned long GetPixelBytes(unsigned long pixel_depth);
extern unsigned long GetRowBytes(unsigned long dimx,
    unsigned long pixel_depth);
extern int XFirstAndLastByteOffset(unsigned long xposi,
    unsigned long pixel_depth, unsigned long *first, unsigned long *last);
extern unsigned long GetImageSize(unsigned long dimx, unsigned long dimy,
    unsigned long pixel_depth);
extern int Setup2dOn1dAry(unsigned char *ary_1d, unsigned long dimx,
    unsigned long dimy, unsigned long pixel_depth, unsigned long leftx,
    unsigned long lowery, unsigned long rightx, unsigned long uppery,
    unsigned char ***ary_2d);


/* Defined in rd_text.c (Automatically generated) */
extern void rd_text_text(FILE *fid, char *ary, unsigned int *tot, int *ok,
    char str[256], unsigned int *indx, int is_one_line);
extern int rd_text_line(FILE *fid, char *str, unsigned int *len,
    unsigned int maxlen);
extern void def_rd_signed_int(FILE *fid, int *num, int lb, int ub, int defnum);
extern void range_rd_signed_int(FILE *fid, int *num, int lb, int ub,
    int defnum);
extern void def_rd_float(FILE *fid, float *num, double lb, double ub,
    double defnum);
extern void range_rd_float(FILE *fid, float *num, double lb, double ub,
    double defnum);
extern void get_text_int(char *str, unsigned int *indx, int *ary,
    unsigned int tot, unsigned int *cnt, int *ok);
extern void rd_text_int(FILE *fid, int *ary, unsigned int *tot, int *ok,
    char str[256], unsigned int *indx);
extern void get_text_float(char *str, unsigned int *indx, float *ary,
    unsigned int tot, unsigned int *cnt, int *ok);
extern void rd_text_float(FILE *fid, float *ary, unsigned int *tot, int *ok,
    char str[256], unsigned int *indx);
extern int RdText(FILE *fid, char *ary, unsigned int *tot, int select,
    int is_flush_input_before);
extern int rd_all_texts(FILE *fid, char ***strs, unsigned int *num_strs);
extern int rd_all_file_texts(char *fn, char ***strs, unsigned int *num_strs);
extern int rd_text_fields(FILE *fid, char ***labels, unsigned int *num_labels);
extern int wr_all_texts(FILE *fid, char **strs, unsigned int num_strs);
extern int wr_all_file_texts(char *fn, char **strs, unsigned int num_strs);


/* Defined in rd_token.c  (Automatically generated) */
extern char *next_token(char *str, int is_new_str, unsigned long next_num);
extern int token_in_line(char *str, int is_new_str, char *patt,
    unsigned long *pat_start, unsigned long *pat_stop,
    unsigned long *token_num);
extern int input_line(FILE *fid, char **line, unsigned long *linelen);
extern int RdUntil(FILE *fid, char **line, unsigned long *line_len, char *patt,
    unsigned long *pat_start, unsigned long *pat_stop,
    unsigned long *token_num);
extern int RdUntilSubstr(FILE *fid, char **line, unsigned long *line_len,
    char *substr, unsigned long *start_indx);
extern int RdWrLineUntil(FILE *fid, FILE *fout, char **line,
    unsigned long *line_len, char *patt, unsigned long *pat_start,
    unsigned long *pat_stop, unsigned long *token_num);
extern int rd_until(FILE *fid, char *line, unsigned long line_len, char *patt,
    unsigned long *pat_start, unsigned long *pat_stop,
    unsigned long *token_num);
extern int rd_until_substr(FILE *fid, char *line, unsigned long line_len,
    char *substr, unsigned long *start_indx);
extern int rd_wr_line_until(FILE *fid, FILE *fout, char *line,
    unsigned long line_len, char *patt, unsigned long *pat_start,
    unsigned long *pat_stop, unsigned long *token_num);


/* Defined in rect_extra.c  (Automatically generated) */
#ifdef RECT_H      /* For struct int_rect_rec */
extern int same_rect(struct int_rect_rec *rr1, struct int_rect_rec *rr2);
extern int can_fit_rect(struct int_rect_rec *assumed_bigger,
    struct int_rect_rec *assumed_smaller);
extern int can_fit_rect_height(struct int_rect_rec *assumed_bigger,
    struct int_rect_rec *assumed_smaller);
extern int can_fit_rect_width(struct int_rect_rec *assumed_bigger,
    struct int_rect_rec *assumed_smaller);
extern int fit_rect(struct int_rect_rec *assumed_bigger,
    struct int_rect_rec *assumed_smaller);
extern void centerx_rect(struct int_rect_rec *rr, int centerx);
extern void centery_rect(struct int_rect_rec *rr, int centery);
extern void centerxy_rect(struct int_rect_rec *rr,
    int centerx, int centery);
extern void center_rect(struct int_rect_rec *center, struct int_rect_rec *rr);
extern int place_adjacent_rect(struct int_rect_rec *limitrect,
    struct int_rect_rec *rr1, struct int_rect_rec *rr2);
extern void rect_pts_in(struct int_rect_rec *rr1, struct int_rect_rec *rr2,
    int *in_rr1_pts, int *num_in_rr1_pts);
extern int get_intersected_rect_1(struct int_rect_rec *rr1,
    struct int_rect_rec *rr2, struct int_rect_rec *intersected,
    int in_rr2_pts);
extern int get_intersected_rect(struct int_rect_rec *rr1,
    struct int_rect_rec *rr2, struct int_rect_rec *intersected);
extern void make_int_rect_rec(struct int_rect_rec *rr, int x1, int y1,
    int x2, int y2);
extern void combine_rects(struct int_rect_rec *rr1, struct int_rect_rec *rr2);
extern void combine_int_bound_recs(struct int_bound_rec *rr1,
    struct int_bound_rec *rr2);
#endif


/* Defined in rect_opr.c (COMPLETE) */
#ifdef RECT_H /* Ugly kluge for int_rect_rec, int_bound_rec, float_bound_rec */
extern int build_2d_int_rects(struct int_rect_rec ***rr,
    unsigned int num_rows, unsigned int num_cols, unsigned int total_rects,
    int x, int y, int width, int height, int x_gap, int y_gap);
extern void float_to_int_bound_rec(struct int_bound_rec *int_bound,
    struct float_bound_rec *fl_bound);
extern void float_to_int_bound_rec(struct int_bound_rec *int_bound,
    struct float_bound_rec *fl_bound);
extern void init_float_bound_rec(struct float_bound_rec *bound);
extern void move_int_bound_rec(struct int_bound_rec *bound,
    int addx, int addy);
extern void pr_float_bound_rec(struct float_bound_rec *bound);
extern void pr_int_rect_rec(struct int_rect_rec *rr);
extern int pt_in_2d_int_rect(struct int_rect_rec **rr, unsigned int num_rows,
    unsigned int num_cols, unsigned int total_rects, int x, int y,
    unsigned int *row_num, unsigned int *col_num, unsigned int *rect_num);
extern int pt_in_int_rect(struct int_rect_rec *rr, int x, int y);
extern void setup_int_bound_rec(struct int_bound_rec *bound,
    int left, int top, int right, int bottom);
extern void setup_int_rect_rec(struct int_rect_rec *rr,
    int left, int top, int width, int height);
extern void valid_bound(struct int_bound_rec *checked,
    struct int_bound_rec *valid,
    int *valid_left, int *valid_right, int *valid_top, int *valid_bottom);
#endif

extern void get_valid_value(int *result, int lb, int ub, int value);
extern int is_between(int dot1, int dot2, int dot3);
extern int is_float_between(float dot1, float dot2, float dot3);
extern int is_pt_in_area(int x1, int y1, int x2, int y2, int ptx, int pty);
extern void move_area(int *x1, int *y1, int *x2, int *y2,
    int refx, int refy, int newx, int newy);


/* Defined in region.c  (Automatically generated) */
#ifdef RECT_H     /* For struct int_rect_rec */
extern void init_dimension(struct int_region_rect *dim,
    struct int_rect_rec *limitrr, struct int_rect_rec *rr,
    struct int_rect_rec *magrect, struct int_rect_rec *limitboundrect,
    struct int_bound_rec *bound);
extern void set_bound_by_int_rect(struct int_bound_rec *bound,
    struct int_rect_rec *rr);
extern int bound_centerx(struct int_bound_rec *bound);
extern int bound_centery(struct int_bound_rec *bound);
extern void move_bound_in_xdim(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int leftoffset, int rightoffset);
extern void move_bound_in_ydim(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int topoffset, int bottomoffset);
extern void in_bound(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect);
extern void expand_bound_in_xdim(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int width);
extern void expand_bound_in_ydim(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int height);
extern void center_bound_in_xdim(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int use_center, int centerx);
extern void center_bound_in_ydim(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int use_center, int centery);
extern void center_bound(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int use_center,
    int centerx, int centery);
extern void validate_boundx(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int invalidate_reset);
extern void validate_boundy(struct int_bound_rec *bound,
    struct int_rect_rec *limitboundrect, int invalidate_reset);
extern void define_dimension_by_fixed_magx(struct int_region_rect *dim);
extern void define_dimension_by_fixed_magy(struct int_region_rect *dim);
extern void define_dimension_by_fixed_boundx(struct int_region_rect *dim,
    int opr, int CanChangeBound);
extern void define_dimension_by_fixed_boundy(struct int_region_rect *dim,
    int opr, int CanChangeBound);
extern void define_dimension_by_fixed_bound(struct int_region_rect *dim,
    int opr, int CanChangeBound);
extern void define_dimension_by_max_areax(struct int_region_rect *dim);
extern void define_dimension_by_max_areay(struct int_region_rect *dim);
extern void get_same_mag_factor(struct int_region_rect *dim, int opr);
extern void define_dimension_by_max_area(struct int_region_rect *dim, int opr);
extern void define_dimension(struct int_region_rect *dim, int HowToDefine,
    int opr, int centerx, int centery);
extern void valid_dimension(struct int_region_rect *dim1,
    struct int_region_rect *dim2,
    int *left, int *right, int *top, int *bottom);
extern int is_pt_in_int_region_rect(struct int_region_rect *r1,
    int ptx, int pty);
extern int is_int_region_rect_intersect(struct int_region_rect *r1,
    struct int_region_rect *r2);
extern void pr_int_region_rect(struct int_region_rect *region);
extern void pr_int_bound_rec(struct int_bound_rec *bound);
extern void pt_as_center_or_within_int_region_rect(struct int_region_rect *r1,
    int x, int y);
extern void int_rect_to_bound(struct int_bound_rec *bound,
    struct int_rect_rec *rect);
#endif


/* Defined in ria2mrvbrhdr.c  (Automatically generated) */
#ifdef MARK_HDR_H    /* For struct mark_hdr */
#ifdef IA_HDR_H      /* For struct hdr_rec */
extern int assign_nhdr_to_hdr(struct mark_hdr *mark, struct hdr_rec *hdr);
extern int wr_hdr_file(char *fn, struct mark_hdr *db);
#endif
#endif


/* Defined in scale.c (Automatically generated) */
extern int scale_1d_byte_values(unsigned char *out_1d, unsigned char *in_1d,
    unsigned long dimx, unsigned long dimy, unsigned long min,
    unsigned long max, unsigned long new_min, unsigned long new_max);
extern int scale_1d_short_values(unsigned short *out_1d, unsigned short *in_1d,
    unsigned long dimx, unsigned long dimy, unsigned long min,
    unsigned long max, unsigned long new_min, unsigned long new_max);
extern int scale_1d_int_values(unsigned int *out_1d, unsigned int *in_1d,
    unsigned long dimx, unsigned long dimy, unsigned long min,
    unsigned long max, unsigned long new_min, unsigned long new_max);
extern int scale_values(unsigned char *out_1d, unsigned char *in_1d,
    unsigned long dimx, unsigned long dimy, unsigned long min,
    unsigned long max, unsigned long new_min, unsigned long new_max,
    unsigned long elem_bytes);
extern int assign_values(unsigned char *dest, unsigned long dest_elem_len,
    unsigned char *src, unsigned long src_elem_len, unsigned long num_elems);
extern int get_byte_ary_min_max(unsigned char *in_1d, unsigned long num_elems,
    unsigned long *min, unsigned long *max);
extern int get_short_ary_min_max(unsigned short *in_1d,
    unsigned long num_elems, unsigned long *min, unsigned long *max);
extern int get_int_ary_min_max(unsigned short *in_1d, unsigned long num_elems,
    unsigned long *min, unsigned long *max);
extern int get_1d_ary_min_max(unsigned short *in_1d, unsigned long num_elems,
    unsigned long *min, unsigned long *max, int elem_bytes);


/* Defined in show_spm_hdr.c */
#ifdef  SPM_HDR_H       /* For struct spm_hdr and friends */
extern void StrNcpy(char *dst, char *src, long len);
extern void pr_spm_hdr_key(struct spm_hdr_key *key);
extern void pr_spm_hdr_dimension(struct spm_hdr_dimension *dim);
extern void pr_spm_hdr_history(struct spm_hdr_history *history);
extern void pr_spm_hdr(struct spm_hdr *hdr);
#endif


/* Defined in showmarkhdr.c */
#ifdef MARK_HDR_H      /* For struct mark_hdr and friends */
extern void pr_makr_hdr_key(struct mark_hdr_key *key1);
extern void pr_mark_hdr_dimension(struct mark_hdr_dimension *dim);
extern void pr_mark_hdr_history(struct mark_hdr_history *history);
extern void pr_mark_hdr(struct mark_hdr *hdr);
#endif


/* Defined in sort.c (COMPLETE) */
extern int sort_asc_int_proc(int *num1, int *num2);


/* Defined in spm_hdr_opr.c (Automatically generated) */
#ifdef SPM_HDR_H        /* For struct spm_hdr and friends */
#ifdef MARK_HDR_H       /* For struct mark_hdr and friends */
extern void init_spm_hdr_key(struct spm_hdr_key *key);
extern void init_spm_hdr_history(struct spm_hdr_history *history);
extern void init_spm_hdr_dimension(struct spm_hdr_dimension *dimen);
extern void init_spm_hdr(struct spm_hdr *hdr);
extern int rd_the_spm_hdr(char *fn, struct spm_hdr *spm);
extern int wr_the_spm_hdr(char *fn, struct spm_hdr *spm);
extern void mark_to_spm_hdr_history(struct mark_hdr_history *mark,
    struct spm_hdr_history *spm);
extern void def_spm_hdr_dimension(struct spm_hdr_dimension *spm);
#endif
#endif


/* Defined in str.c (Automatically generated) */
extern int concat_strs(char **str_ptr, int num_strs, ...);
extern int append_strs(char **str_ptr, int num_strs, ...);
extern int sort_str_proc(char **str1, char **str2);
extern int sort_no_case_str_proc(char **str1, char **str2);
extern int obj_binary_search(unsigned char *objs, unsigned long num_objs,
    unsigned long obj_size, void *search_obj, int (*search_proc)(),
    unsigned long *insert_posi);
extern size_t get_strlen(char *str);
extern int center_str(char *str, unsigned long max_str_size);
extern int blank_in_str(char *str, unsigned long *blank_posi);
extern void no_blank_in_str(char *str);
extern unsigned long strlen_max(char *str, unsigned long max_len);
extern void set_str_len(char *str, int len);
extern void TrimLeadingStrCh(char *str, int ch);
extern void TrimTrailingStrCh(char *str, int ch);
extern void trim_str_blanks(char *str);
extern void drop_last_element(char *str, int separator);
extern void blank_str(char *str, unsigned long elems);
extern void left_pad_str(char *str, unsigned long max_len, int ch);
extern void right_pad_str(char *str, unsigned long max_len, int ch);
extern int copy_str(char **output_str_ptr, char *input_str, size_t output_len);
extern int cp_str(char **output_str_ptr, char *input_str);
extern int str_ncpy(char *output_str, int output_len, char *input_str);
extern void str_cpy1(char *str1, char *str2, size_t size);
extern int str_dealloc_and_cpy(char **str1, char *str2);
extern int con_str_fields(char **out_str, char **strs, unsigned long num_strs,
    int con_ch);
extern int copy_strs(char ***outstrs, unsigned long *num_outstrs,
    char **instrs, unsigned long num_instrs);
extern int delete_strs(char ***ary, int *elems, char *strs[],
    unsigned long num_strs, int deallocate_str, unsigned char *status);
extern void no_duplicate_strs(char ***ary, int *elems, int deallocate_str,
    unsigned char *status);
extern void insert_strs(char **strs, unsigned long total_slots,
    unsigned long *num_strs, char *str, unsigned long insert_posi);
extern int insert_str_to_sorted_list(char ***sorted_strs,
    unsigned long *num_sorted_strs, char *str, unsigned long *insert_posi);
extern int str_cmp(char *str1, char *str2);
extern int upper_case_ch(int ch);
extern int lower_case_ch(int ch);
extern int lower_case_str(char *str);
extern int no_case_str_cmp(char *str1, char *str2);
extern int string_found(char *ary[], int elems, char *str);


/* Defined in str_extra.c  (Automatically generated) */
extern unsigned long skip_str_first_ch_in_patt(char *str, char *patt);
extern unsigned long skip_str_until_ch_in_patt(char *str, char *patt);
extern unsigned long skip_str_until_ch_not_in_patt(char *str, char *patt);
extern int find_sub_str(char *str, char *substr, unsigned long *start_indx);
extern int replace_ch_by_str(char **result, char *str, int replace_ch,
    char *replacement_str);
extern int replace_ch_without_duplicate(char *str, unsigned int str_len,
    int replace_ch, int new_ch);
extern int replace_substr_by_str(char *str, char *substr, char *repl_str);
extern int is_blank_in_str(char *str);
extern void non_ascii_to_ch(char *str, int ch);


/* Defined in system.c (COMPLETE) */
extern int setup_batch_file(char *cmd1, ...);


/* Defined in systemspecif.c */
void swapUShorts(unsigned short *ushort_array, size_t len);
void swapULongs(unsigned long *ulong_array, size_t len);
unsigned short to_dec_short(int num);
unsigned long to_dec_long(unsigned long num);
void dec2sun_word(short buf[], int len);
void dec2sun_long(unsigned long buf[], int len);
float get_float_data(unsigned char fl_buf[4]);
void sun2dec_float(unsigned char *inbuf, unsigned char *outbuf);
void dec2sun_float(unsigned char *inbuf, unsigned char *outbuf);
void decfl2sun_float(double fl, unsigned char *outbuf);
void dec2sun_float_ary(float *in_ary, float *out_ary, unsigned long num_elems);
void pr_hex(unsigned char *hex_ary, unsigned long len);


/* Defined in time.c  (Automatically generated) */
extern char *current_asctime(void);
extern int timer_wait(unsigned long secs);
extern int timer_delete_fn(char *fn, unsigned long mask, int sec, int min,
    int hour, int monthday, int month, int year, int weekday, int yearday);

#ifdef TIME_H     /* Only if mytime.h has been read */
extern struct tm *current_time(void);
extern int timer_compare(struct tm *time, unsigned long mask, int sec,
    int min, int hour, int monthday, int month, int year);
#endif


/* Defined in unit_2d_rec.c  (Automatically generated) */
#ifdef COMMON_H        /* For struct unit_2d)rec */
extern void initial_unit_2d_rec(struct unit_2d_rec *unit);
extern int setup_unit_2d_rec(struct unit_2d_rec *unit,
    double ustartx, double ustarty, double unitx, double unity,
    unsigned long dimx, unsigned long dimy,
    double local_ustartx, double local_ustarty,
    double local_unitx, double local_unity);
extern int unit_2d_rec_extra_trans_matrix(struct matrix_2d_rec *trans,
    struct unit_2d_rec *unit);
extern void pr_unit_2d_rec(struct unit_2d_rec *unit);
#endif


/* Defined in v3rwbuf2drec.c  (Automatically generated) */
#ifdef COMMON_H    /* For struct buf_2d_rec */
extern int v3_rd_buf_2d_rec_hdr(FILE *fid, struct buf_2d_rec *_2d);
extern int v3_rd_buf_2d_rec(FILE *fid, struct buf_2d_rec *_2d);
extern int v3_rd_non_contiguos_buf_2d_rec(FILE *fid, struct buf_2d_rec *_2d);
extern int v3_wr_buf_2d_rec_hdr(FILE *fid, struct buf_2d_rec *_2d);
extern int v3_wr_buf_2d_rec(FILE *fid, struct buf_2d_rec *_2d);
extern int v3_wr_non_contiguos_buf_2d_rec(FILE *fid, struct buf_2d_rec *_2d);
extern int v3_rd_2d(char *fn, struct buf_2d_rec *_2d);
extern int v3_wr_2d(char *fn, struct buf_2d_rec *_2d);
#endif


/* Defined in vector.c  (Automatically generated) */
#ifdef GEOMETRY_H       /* For struct vector_3d_rec */
extern void add_xyz_to_vector_3d(struct vector_3d_rec *v1,
    double x, double y, double z);
extern void add_vector_3d(struct vector_3d_rec *v1, struct vector_3d_rec *v2);
extern void sub_vector_3d(struct vector_3d_rec *v1, struct vector_3d_rec *v2);
extern void sub_vectors(struct vector_3d_rec *result, struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern void add_vectors(struct vector_3d_rec *result, struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern void mult_vector_with_const(struct vector_3d_rec *v1, double fl_const);
extern void cross_product(float *x, float *y, float *z,
    double x1, double y1, double z1, double x2, double y2, double z2);
extern void vector_cross_product(struct vector_3d_rec *result,
    struct vector_3d_rec *v1, struct vector_3d_rec *v2);
extern float dot_product(double x1, double y1, double z1,
    double x2, double y2, double z2);
extern float vector_dot_product(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern void setup_vector_3d_rec(struct vector_3d_rec *vector,
    double x, double y, double z);
extern float length_vector(struct vector_3d_rec *vector);
extern float degree_angle_vectors(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern float vector_distance(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern float vector_distance_from_abs(struct vector_3d_rec *v1);
extern void mid_vectors(struct vector_3d_rec *result, struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern void unit_vector(float *ux, float *uy, float *uz,
    double x, double y, double z, int *is_infinity);
extern void direction_cosine(struct vector_3d_rec *vector,
    float *cosx, float *cosy, float *cosz, int *is_infinity);
extern void direction_cosine_vector(struct vector_3d_rec *vector);
extern void floor_vector_3d_rec(struct vector_3d_rec *vector);
extern void truncate_vector_3d_rec(struct vector_3d_rec *vector);
extern void round_vector_3d_rec(struct vector_3d_rec *vector);
extern void pos_vector_3d_rec(struct vector_3d_rec *vector);
extern void reverse_vector_3d_rec(struct vector_3d_rec *vector);
extern int same_direction_vectors(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern int same_dimension_vectors(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern int is_reverse_vector(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern int same_3d_pt(struct vector_3d_rec *pt1, struct vector_3d_rec *pt2);
extern int fl_in_range(double fl, double range1, double range2);
extern int pt_in_range(struct vector_3d_rec *pt, struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern void num_pt_coords_in_range(struct vector_3d_rec *pt,
    struct vector_3d_rec *v1, struct vector_3d_rec *v2,
    int *num_coords_in_range);
extern int rounded_pt_in_range(struct vector_3d_rec *pt,
    struct vector_3d_rec *v1, struct vector_3d_rec *v2);
extern int vector_colinear(struct vector_3d_rec *vector1,
    struct vector_3d_rec *vector2);
extern int colinear_3_pts(double x1, double y1, double z1,
    double x2, double y2, double z2, double x3, double y3, double z3);
extern float pt_line_dist(struct vector_3d_rec *base_vector,
    struct vector_3d_rec *direction_vector, struct vector_3d_rec *pt);
extern int get_vector_plane_type(struct vector_3d_rec *vector);
extern int is_vectors_ordered(struct vector_3d_rec **vectors,
    unsigned long num_vectors);
extern int arrange_vectors(struct vector_3d_rec *vectors,
    unsigned long num_vectors);
extern void pr_vector_3d_rec(struct vector_3d_rec *vector);
extern void cuboid_vertices(struct vector_3d_rec vertices[8],
    double startx, double endx, double starty, double endy,
    double startz, double endz);
extern void cuboid_edges(struct vector_3d_rec vertices[24],
    double startx, double endx, double starty, double endy, double startz,
    double endz);
extern void cuboid_plane_vertices(struct vector_3d_rec bounds[4],
    struct vector_3d_rec vertices[8], int indx_v1, int indx_v2,
    int indx_v3, int indx_v4);
extern void cuboid_planes(struct vector_3d_rec bases[6],
    struct vector_3d_rec normals[6], struct vector_3d_rec vertices[6][4],
    double startx, double endx, double starty, double endy, double startz,
    double endz);
extern void closest_2_vertices(struct vector_3d_rec *base_vector,
    struct vector_3d_rec *direction_vector, double startx, double endx,
    double starty, double endy, double startz, double endz,
    struct vector_3d_rec *closest1, struct vector_3d_rec *closest2);
extern int get_line_move_ratio(struct vector_3d_rec *base,
    struct vector_3d_rec *direction, struct vector_3d_rec *last_pt,
    float *ratio);
extern char *get_vector_angle_str(struct vector_3d_rec *vector);
extern char *get_vector_angle_str1(struct vector_3d_rec *vector);
extern void cos_degree_vector(struct vector_3d_rec *vector);
extern int vector_angle_more_than_90(struct vector_3d_rec *vector);
extern int direction_3d_index(double num);
extern int vectors_direction(struct vector_3d_rec *v1,
    struct vector_3d_rec *v2);
extern int is_rect_vertices_clockwise(struct vector_3d_rec pts[4]);
extern int find_pts_cuboid(struct vector_3d_rec *pts, unsigned long num_pts,
    struct vector_3d_rec *min, struct vector_3d_rec *max);
extern int closest_pt_from(struct vector_3d_rec *ref_pt,
    struct vector_3d_rec *pts, unsigned long num_pts, unsigned long *indx);
extern int lines_intersection_2(double px1, double py1, double dx1, double dy1,
    double px2, double py2, double dx2, double dy2, double *ratio1,
    double *ratio2);
extern int lines_intersection(struct vector_3d_rec *base1,
    struct vector_3d_rec *direction1, struct vector_3d_rec *base2,
    struct vector_3d_rec *direction2, struct vector_3d_rec *and);
#endif


/* Defined in wr_hdr_bin.c  (Automatically generated) */
#ifdef  IA_HDR_H         /* For struct hdr_rec */
extern void assign_float_defaults(struct hdr_rec *hdr, int select,
    double value);
extern void assign_int_defaults(struct hdr_rec *hdr, int select, int value);
extern int assign_text_defaults(struct hdr_rec *hdr, int select, char *str);
extern void assign_text(struct hdr_rec *dest, struct hdr_rec *src, int select);
extern void assign_int(struct hdr_rec *dest, struct hdr_rec *src, int select);
extern void assign_float(struct hdr_rec *dest, struct hdr_rec *src,
    int select);
extern int assign_num_kind_of_images(struct hdr_rec *dest,
    struct hdr_rec *src);
extern int rd_num_kind_of_images(FILE *fid, struct hdr_rec *hdr, int is_new);
extern int rd_the_hdr_file(char *in_outfn, struct hdr_rec *hdr,
    int *is_all_supplied);
extern int construct_hdr_from_text(char *inp_textfn, char *out_hdrfn,
    char *inp_hdrfn);
#endif

#ifdef  IA_HDR_H             /* For struct hdr_rec */
#ifdef  MRI_HDR_CONSTANTS_H  /* For struct hdr_field_rec and friends */
extern int assign_hdrs(struct hdr_rec *dest, struct hdr_rec *src,
    struct hdr_field_rec fields[TOTAL_FIELDS]);
extern int must_be_supplied(struct hdr_rec *hdr,
    struct hdr_field_rec fields[TOTAL_FIELDS]);
extern int build_hdr_from_text(FILE *fid, struct hdr_rec *hdr, int is_new,
    struct hdr_field_rec fields[TOTAL_FIELDS]);
#endif
#endif


/* Defined in wr_hdr_text.c  (Automatically generated) */
#ifdef  IA_HDR_H             /* For struct hdr_rec */
#ifdef  MRI_HDR_CONSTANTS_H  /* For struct hdr_field_rec and friends */
extern void WrTextField(FILE *fid, struct hdr_field_rec *field,
    char *field_value);
extern void WrCharField(FILE *fid, struct hdr_field_rec *field,
    int field_value);
extern void WrIntField(FILE *fid, struct hdr_field_rec *field,
    int field_value, int is_not_commented);
extern void WrHexIntParam(FILE *fid, struct hdr_field_rec *field,
    int field_value, int is_not_commented);
extern void WrShortParam(FILE *fid, struct hdr_field_rec *field,
    int field_value, int is_not_commented);
extern void WrFloatField(FILE *fid, struct hdr_field_rec *field,
    double field_value, int is_not_commented);
extern void WrDoubleField(FILE *fid, struct hdr_field_rec *field,
    double field_value, int is_not_commented);
extern void WrTransformationField(FILE *fid, struct hdr_field_rec *field,
    float field_value[4][4], int is_not_commented);
extern void WrNumKndOfImagesField(FILE *fid, struct hdr_field_rec *field,
    struct hdr_rec *hdr);
extern int WrTextHdrSpec(FILE *fid, char *in_outfn, struct hdr_rec *hdr,
    struct hdr_field_rec fields[TOTAL_FIELDS]);
#endif
#endif


#endif  /* NRIA_LIB_H */
