/****************************************************************************
Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/


#ifndef RIA_MAIN_H
#define	RIA_MAIN_H

#include "typedef.h"
#include "common.h"
#include "error_constants.h"
#include "special_procs.h"
#include "common_constants.h"
#include "command_keys.h"
#include "menu_items.h"
#include "list_rec.h"

#include "rect.h"
#include "str_def.h"
#include "slice_roi_rec.h"
#include "output.h"
#include "histo.h"

#include "list_rec.h"

#include "ia_hdr.h"
#include "pixel_desc.h"
#include "coord_rec.h"
#include "input.h"

#include "display_win.h"
#include "win.h"
#include "analysis.h"


struct input_rec
   {
    struct prog_rec *prog;
    unsigned int total_files,num_files;
    struct input_file_rec **files;
   };

#define PROG_ID_V1 "P1.0"
#define HDR_ID_V1 "H1.0"
#define DAT_ID_V1 "D1.0"
#define DAT_ID_V2 "D2.0"
struct defaults_fn_rec
   {
    char *color_fn,*roi_labels_fn,*pixel_types_fn;
    char *txt_format_fn,*batch_prog,*batch_status_fn;
    char *hdr_dir,*hdr_fn,*dat_dir,*dat_fn,*rater,*startup_pixel_mag_factor;
    char *bin_ratio_row_col;
   };

#define TOTAL_DEFAULT_FNS 20
#define NUM_DEFAULT_FNS 13
#define DEFAULT_COLOR_FN "color.dat"
#define DEFAULT_ROI_LABELS_FN "roi_labels.dat"
#define DEFAULT_PIXEL_TYPES_FN "pixel_types.dat"
#define DEFAULT_TXT_FORMAT_FN "txt_format.dat"
#define DEFAULT_BATCH_PROG_FN "batch_main"
#define DEFAULT_BATCH_STATUS_FN "batch.status"
#define DEFAULT_HDR_FN "*.nhdr"
#define DEFAULT_DAT_FN "*.ndat"
#define DEFAULT_BIN_RATIO_ROW_COL ".5/3/3"

#define VALUE_DEFAULT_HDR_DIR 6
#define VALUE_DEFAULT_DAT_DIR 8
#define VALUE_DEFAULT_RATER 10
#define VALUE_MAG_FACTOR 11

/*
# identifier       function            status              default value
#      0          color fn          read-write file          color.dat
#      1        roi labels fn       read-write file          roi_labels.dat
#      2        pixel types fn      read-write file          pixel_types.dat
#      3        text output
#                  format fn        read-write file          txt_format.dat
#      4        batch program       execute-only file        batch_main
#      5        batch log file      write-only file          batch.status
#      6        .hdr directory      directory specification  user directory
#                                                            from where the
#                                                            program is run
#      7        .hdr fns            fn specification         *.hdr
#      8        .dat directory      directory specification  user directory
#                                                            from where the
#                                                            program is run
#      9        .dat fns            fn specification         *.dat
#     10        rater's name        text                     login user
#     11        default image       1 <= factor <= 20        2
#               magnifying factor
#     12        bin image_ratio/num_rows/num_cols
#                            .25 <= ratio <= 1.0            .5/3/3
*/
union defaults_rec
   {
    char *fns[TOTAL_DEFAULT_FNS];
    struct defaults_fn_rec whch;
   };

struct prog_rec
   {
    char *prog_id,*hdr_id,*dat_id;
    char *prog_name;
    char *defaults_fn;
    union defaults_rec defaults;
    struct input_rec input;
    struct all_win_rec win;
   };

#endif RIA_MAIN_H
