/****************************************************************************
3dtran.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: 3dtran.c,v 1.6 1999/07/07 18:01:05 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <math.h>
#include <stdarg.h>
#include <stdio.h>

#include <sys/types.h>

#include "geometry.h"
#include "error_constants.h"
#include "typedef.h"
#include "nria_lib.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT2(x) x; fflush(stdout);
#define PRT4(x) x; fflush(stdout);
#define PRT3(x) x; fflush(stdout);
#define PRT5(x) x; fflush(stdout);
#define PRT6(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRT2(x)
#define PRT4(x)
#define PRT3(x)
#define PRT5(x)
#define PRT6(x)



/*-----------------------------------------------------------------------
identity_3d_matrix - ???
-----------------------------------------------------------------------*/

void
identity_3d_matrix(matrix)
 struct matrix_3d_rec *matrix;
{
 int row,col;

 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++)
       if (row == col) matrix->buf[row][col] = 1.0;
         else matrix->buf[row][col] = .0;
 matrix->status = MATRIX_INITIALIZED;
}


/*-----------------------------------------------------------------------
setup_3d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_3d_matrix(matrix,e00,e01,e02,e10,e11,e12,e20,e21,e22)
 struct matrix_3d_rec *matrix;
 float e00,e01,e02,e10,e11,e12,e20,e21,e22;
{
 matrix->buf[0][0] = e00;
 matrix->buf[0][1] = e01;
 matrix->buf[0][2] = e02;
 matrix->buf[1][0] = e10;
 matrix->buf[1][1] = e11;
 matrix->buf[1][2] = e12;
 matrix->buf[2][0] = e20;
 matrix->buf[2][1] = e21;
 matrix->buf[2][2] = e22;

 matrix->buf[0][3] = .0;
 matrix->buf[1][3] = .0;
 matrix->buf[2][3] = .0;
 matrix->buf[3][3] = 1.0;
 matrix->buf[3][0] = .0;
 matrix->buf[3][1] = .0;
 matrix->buf[3][2] = .0;

 matrix->status = MATRIX_INITIALIZED;
}


/*-----------------------------------------------------------------------
setup_4d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_4d_matrix(matrix,e00,e01,e02,e03,e10,e11,e12,e13,
                       e20,e21,e22,e23,e30,e31,e32,e33)
 struct matrix_3d_rec *matrix;
 float e00,e01,e02,e03,e10,e11,e12,e13,e20,e21,e22,e23,e30,e31,e32,e33;
{
 matrix->buf[0][0] = e00;
 matrix->buf[0][1] = e01;
 matrix->buf[0][2] = e02;
 matrix->buf[0][3] = e03;
 matrix->buf[1][0] = e10;
 matrix->buf[1][1] = e11;
 matrix->buf[1][2] = e12;
 matrix->buf[1][3] = e13;
 matrix->buf[2][0] = e20;
 matrix->buf[2][1] = e21;
 matrix->buf[2][2] = e22;
 matrix->buf[2][3] = e23;

 matrix->buf[3][0] = e30;
 matrix->buf[3][1] = e31;
 matrix->buf[3][2] = e32;
 matrix->buf[3][3] = e33;

 matrix->status = MATRIX_INITIALIZED;
}


/*-----------------------------------------------------------------------
is_same_matrix_3d_rec - ???
-----------------------------------------------------------------------*/

int
is_same_matrix_3d_rec(mat1,mat2)
 struct matrix_3d_rec *mat1,*mat2;
{
 int row,col;

 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++)
       if (mat1->buf[row][col] != mat2->buf[row][col]) return 0;
 return 1;
}


/*-----------------------------------------------------------------------
setup_3d_xy_reflection_matrix - ???
-----------------------------------------------------------------------*/

void
setup_3d_xy_reflection_matrix(matrix)
 struct matrix_3d_rec *matrix;
{
 identity_3d_matrix(matrix);
 matrix->buf[0][0] = 0;
 matrix->buf[1][1] = 0;
 matrix->buf[0][1] = 1;
 matrix->buf[1][0] = 1;
}


/*-----------------------------------------------------------------------
setup_3d_default_xy_axes - ???
-----------------------------------------------------------------------*/

void
setup_3d_default_xy_axes(matrix)
 struct matrix_3d_rec *matrix;
{
 setup_zrotate_3d_matrix(matrix,1.5 * RADIAN);
}



/*-----------------------------------------------------------------------
mult_3d_matrix - ???

Dest = Src1 times Src2

D(i,j) = sum( S1(i,m) * S2(m,j) ), m = 1..4
-----------------------------------------------------------------------*/

void
mult_3d_matrix(dest,src1,src2)
 struct matrix_3d_rec *dest,*src1,*src2;
{
 unsigned long row,col;
 float temp[4][4];

 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++)
       temp[row][col] = src1->buf[row][0] * src2->buf[0][col] +
                        src1->buf[row][1] * src2->buf[1][col] +
                        src1->buf[row][2] * src2->buf[2][col] +
                        src1->buf[row][3] * src2->buf[3][col];
 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++)
       dest->buf[row][col] = temp[row][col];
 dest->status |= src1->status | src2->status;
}


/*-----------------------------------------------------------------------
assign_translation - ???
-----------------------------------------------------------------------*/

void
assign_translation(matrix,tx,ty,tz)
 struct matrix_3d_rec *matrix;
 float tx,ty,tz;
{
 matrix->buf[0][3] = tx;
 matrix->buf[1][3] = ty;
 matrix->buf[2][3] = tz;
 matrix->status = MATRIX_TRANSLATED;
}


/*-----------------------------------------------------------------------
setup_translate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_translate_3d_matrix(matrix,tx,ty,tz)
 struct matrix_3d_rec *matrix;
 float tx,ty,tz;
{
 identity_3d_matrix(matrix);
 assign_translation(matrix,tx,ty,tz);
}


/*-----------------------------------------------------------------------
translate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
translate_3d_matrix(matrix,tx,ty,tz)
 struct matrix_3d_rec *matrix;
 float tx,ty,tz;
{
 struct matrix_3d_rec temp;

 setup_translate_3d_matrix(&temp,tx,ty,tz);
 mult_3d_matrix(matrix,&temp,matrix);
}


/*-----------------------------------------------------------------------
drop_3d_matrix_translation - ???
-----------------------------------------------------------------------*/

void
drop_3d_matrix_translation(matrix)
 struct matrix_3d_rec *matrix;
{
 matrix->buf[0][3] = .0;
 matrix->buf[1][3] = .0;
 matrix->buf[2][3] = .0;
}


/*-----------------------------------------------------------------------
setup_scale_3d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_scale_3d_matrix(matrix,sx,sy,sz)
 struct matrix_3d_rec *matrix;
 float sx,sy,sz;
{
 identity_3d_matrix(matrix);
 matrix->buf[0][0] = sx;
 matrix->buf[1][1] = sy;
 matrix->buf[2][2] = sz;
 matrix->status = MATRIX_SCALED;
}


/*-----------------------------------------------------------------------
scale_3d_matrix - ???
-----------------------------------------------------------------------*/

void
scale_3d_matrix(matrix,sx,sy,sz)
 struct matrix_3d_rec *matrix;
 float sx,sy,sz;
{
 struct matrix_3d_rec temp;

 setup_scale_3d_matrix(&temp,sx,sy,sz);
 mult_3d_matrix(matrix,&temp,matrix);
}


/*-----------------------------------------------------------------------
setup_scale_3d_matrix_from_xyz - ???
-----------------------------------------------------------------------*/

void
setup_scale_3d_matrix_from_xyz(matrix,sx,sy,sz,fx,fy,fz)
 struct matrix_3d_rec *matrix;
 float sx,sy,sz,fx,fy,fz;
{
 identity_3d_matrix(matrix);
 matrix->buf[0][0] = sx;
 matrix->buf[1][1] = sy;
 matrix->buf[2][2] = sz;
 matrix->buf[3][0] = (1 - sx) * fx;
 matrix->buf[3][1] = (1 - sy) * fy;
 matrix->buf[3][2] = (1 - sz) * fz;
 matrix->status |= MATRIX_SCALED | MATRIX_TRANSLATED;
}


/*-----------------------------------------------------------------------
scale_3d_matrix_from_xyz - ???
-----------------------------------------------------------------------*/

void
scale_3d_matrix_from_xyz(matrix,sx,sy,sz,fx,fy,fz)
 struct matrix_3d_rec *matrix;
 float sx,sy,sz,fx,fy,fz;
{
 struct matrix_3d_rec temp;

 scale_3d_matrix_from_xyz(&temp,sx,sy,sz,fx,fy,fz);
 mult_3d_matrix(matrix,&temp,matrix);
}


/*-----------------------------------------------------------------------
setup_xrotate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_xrotate_3d_matrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
 identity_3d_matrix(matrix);
 matrix->buf[1][1] = cos(radian_angle);
 matrix->buf[1][2] = sin(radian_angle);
 matrix->buf[2][1] = -matrix->buf[1][2];
 matrix->buf[2][2] = matrix->buf[1][1];
 matrix->status |= MATRIX_ROTATED;
}


/*-----------------------------------------------------------------------
xrotate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
xrotate_3d_matrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
 struct matrix_3d_rec temp;

 setup_xrotate_3d_matrix(&temp,radian_angle);
 mult_3d_matrix(matrix,&temp,matrix);
}


/*-----------------------------------------------------------------------
setup_yrotate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_yrotate_3d_matrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
 identity_3d_matrix(matrix);
 matrix->buf[0][0] = cos(radian_angle);
 matrix->buf[2][0] = sin(radian_angle);
 matrix->buf[0][2] = -matrix->buf[2][0];
 matrix->buf[2][2] = matrix->buf[0][0];
 matrix->status |= MATRIX_ROTATED;
}


/*-----------------------------------------------------------------------
yrotate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
yrotate_3d_matrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
 struct matrix_3d_rec temp;

 setup_yrotate_3d_matrix(&temp,radian_angle);
 mult_3d_matrix(matrix,&temp,matrix);
}


/*-----------------------------------------------------------------------
setup_zrotate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
setup_zrotate_3d_matrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
 identity_3d_matrix(matrix);
 matrix->buf[0][0] = cos(radian_angle);
 matrix->buf[0][1] = sin(radian_angle);
 matrix->buf[1][0] = -matrix->buf[0][1];
 matrix->buf[1][1] = matrix->buf[0][0];
 matrix->status |= MATRIX_ROTATED;
}


/*-----------------------------------------------------------------------
zrotate_3d_matrix - ???
-----------------------------------------------------------------------*/

void
zrotate_3d_matrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
 struct matrix_3d_rec temp;

 setup_zrotate_3d_matrix(&temp,radian_angle);
 mult_3d_matrix(matrix,&temp,matrix);
}


/*-----------------------------------------------------------------------
Rotate3dMatrix - ???
-----------------------------------------------------------------------*/

void
Rotate3dMatrix(matrix,radian_angle)
 struct matrix_3d_rec *matrix;
 float radian_angle;
{
/*
 struct matrix_3d_rec temp,temp1;

 temp = *matrix;
 zrotate_3d_matrix(&temp,radian_angle);

 identity_3d_matrix(&temp);
 identity_3d_matrix(&temp1);
 temp.buf[0][1] = -(float) tan((double) (radian_angle / 2.0));
 temp1.buf[1][0] = sin(radian_angle);
 mult_3d_matrix(&temp1,&temp1,&temp);
 mult_3d_matrix(&temp1,&temp,&temp1);
 mult_3d_matrix(matrix,&temp1,matrix);
*/
 zrotate_3d_matrix(matrix,radian_angle);
}


/*-----------------------------------------------------------------------
combine_3d_matrices - ???

combine_matrices(dest, mat1, ..., matn, 0);
-----------------------------------------------------------------------*/

int
combine_3d_matrices(
    struct matrix_3d_rec *dest,
    ...)
{
 va_list args;
 struct matrix_3d_rec *matrix;

 va_start(args,dest);
 for (;(matrix = va_arg(args,struct matrix_3d_rec *)) != 0;)
    mult_3d_matrix(dest,matrix,dest);
 va_end(args);

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
Rotate3dMatrixFromAxis - ???
-----------------------------------------------------------------------*/

void
Rotate3dMatrixFromAxis(xyztoplane,planetoxyz,base,dire,radian_angle)
 struct matrix_3d_rec *xyztoplane,*planetoxyz;
 struct vector_3d_rec *base,*dire;
 float radian_angle;
{
 struct matrix_3d_rec tran,tran1,tran2,inv,inv1,inv2;
 float direction_rad,direction_rad1,dist;

 setup_translate_3d_matrix(&tran,-(base->x),-(base->y),-(base->z));
 inverse_4d_matrix(&inv,&tran);
 PRT5(printf("Rotate3dMatrixFromAxis dire...\n");pr_vector_3d_rec(dire);)
 direction_rad = (float) Atan2((double) dire->y,(double) dire->x);

 PRT6(printf("deg1 = %f rad1 = %f\n",(float) degree(direction_rad),
      direction_rad))
 setup_zrotate_3d_matrix(&tran1,direction_rad);
 inverse_4d_matrix(&inv1,&tran1);
 mult_3d_matrix(&tran,&tran1,&tran);

 dist = (float) dire->x * (float) dire->x +
            (float) dire->y * (float) dire->y;
 dist = sqrt((double) dist);
 direction_rad1 = (float) Atan2((double) dist,(double) dire->z);

 PRT6(printf("==dist = %f deg1 = %f rad1 = %f\n",dist,(float) degree(direction_rad1),
      direction_rad1))
 setup_yrotate_3d_matrix(&tran2,direction_rad1);
 inverse_4d_matrix(&inv2,&tran2);
 mult_3d_matrix(&tran,&tran2,&tran);
 if (radian_angle != .0)
    zrotate_3d_matrix(&tran,-radian_angle);

 mult_3d_matrix(xyztoplane,&tran,xyztoplane);

 mult_3d_matrix(&inv2,&inv1,&inv2);
 mult_3d_matrix(&inv2,&inv,&inv2);
 mult_3d_matrix(planetoxyz,&inv2,planetoxyz);
}


/*-----------------------------------------------------------------------
rotate_pt_fr_line - ???
-----------------------------------------------------------------------*/

void
rotate_pt_fr_line(base,dire,rad_deg,pt,transformed_pt)
 struct vector_3d_rec *base,*dire,*pt,*transformed_pt;
 float rad_deg;
{
 struct matrix_3d_rec planetoxyz,xyztoplane;
 struct vector_3d_rec v1,origin,*base1;

 identity_3d_matrix(&planetoxyz);
 identity_3d_matrix(&xyztoplane);
 if (base == NULL)
    {setup_vector_3d_rec(&origin,.0,.0,.0);
     base1 = &origin;}
   else base1 = base;
 PRT3(printf("rotate_pt_fr_line...base");pr_vector_3d_rec(base1);
      printf("---dire...");pr_vector_3d_rec(dire);
      printf("---rad_deg = %f\n",rad_deg);
      printf("---pt...");pr_vector_3d_rec(pt);
     )
 Rotate3dMatrixFromAxis(&xyztoplane,&planetoxyz,base1,dire,rad_deg);
 trans_3d_pt(&xyztoplane,pt,&v1,0);
 trans_3d_pt(&planetoxyz,&v1,transformed_pt,0);
 PRT3(printf("---transformed_pt...");pr_vector_3d_rec(transformed_pt);)
}


/*-----------------------------------------------------------------------
det_3d_matrix - ???
-----------------------------------------------------------------------*/

float 
det_3d_matrix(matrix)
 struct matrix_3d_rec *matrix;
{
 float det,sub;
 int row,col,cnt;

#define NUM_ELEMS 3
 det = .0;
 for (row = 0; row < NUM_ELEMS; row++)
    {
     cnt = row;
     sub = matrix->buf[0][cnt];
     for (col = 1; col < NUM_ELEMS; col++)
        {cnt = (cnt + 1) % NUM_ELEMS;
         sub *= matrix->buf[col][cnt];
        }
     det += sub;
    }
 for (row = 0; row < NUM_ELEMS; row++)
    {
     cnt = row;
     sub = matrix->buf[NUM_ELEMS - 1][cnt];
     for (col = NUM_ELEMS - 2; ; col--)
        {cnt = (cnt + 1) % NUM_ELEMS;
         sub *= matrix->buf[col][cnt];
         if (col == 0) break;
        }
     det -= sub;
    }
 return det;
}


/*-----------------------------------------------------------------------
inverse_3d_matrix - ???
-----------------------------------------------------------------------*/

int
inverse_3d_matrix(inv,matrix)
 struct matrix_3d_rec *inv,*matrix;
{
 float det;
 int row,col,nextrow,nextcol,nextrow1,nextcol1;
 double temp1,temp2;

 det = det_3d_matrix(matrix);
 if (det == .0) return 1;
 for (row = 0; row < 3; row++)
    {nextrow = (row + 1) % 3;
     nextrow1 = (nextrow + 1) % 3;
     for (col = 0; col < 3; col++)
        {nextcol = (col + 1) % 3;
         nextcol1 = (nextcol + 1) % 3;
         temp1 = matrix->buf[nextcol][nextrow] * 
             matrix->buf[nextcol1][nextrow1];
         temp2 = matrix->buf[nextcol1][nextrow] * 
             matrix->buf[nextcol][nextrow1];

         temp1 = (temp1 - temp2) / (double) det;
         inv->buf[row][col] = (float) temp1;
        }
    }
 inv->buf[0][3] = .0;
 inv->buf[1][3] = .0;
 inv->buf[2][3] = .0;
 inv->buf[3][3] = 1.0;
 inv->buf[3][0] = .0;
 inv->buf[3][1] = .0;
 inv->buf[3][2] = .0;
 inv->status = MATRIX_INITIALIZED;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
setup_3d_cofactor - ???
-----------------------------------------------------------------------*/

void
setup_3d_cofactor(result,matrix,row,col)
 struct matrix_3d_rec *result,*matrix;
 int row,col;
{
 int lcv,cnt,y,x;

 identity_3d_matrix(result);
 for (y = lcv = 0; lcv < 4; lcv++)
    {if (lcv == row) continue;
     for (x = cnt = 0; cnt < 4; cnt++)
       if (cnt != col) result->buf[y][x++] = matrix->buf[lcv][cnt];
     y++;
    }
}


/*-----------------------------------------------------------------------
transpose_4d_matrix - ???
-----------------------------------------------------------------------*/

void
transpose_4d_matrix(result,matrix)
 struct matrix_3d_rec *result,*matrix;
{
 int row,col;
 struct matrix_3d_rec m1,*ptr;

 if (result == matrix) ptr = &m1;
   else ptr = result;
 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++)
       if (row == col)
          ptr->buf[row][col] = matrix->buf[row][col];
         else ptr->buf[col][row] = matrix->buf[row][col];
 if (result == matrix)
    {for (row = 0; row < 4; row++)
        for (col = 0; col < 4; col++)
           result->buf[row][col] = m1.buf[row][col];
    }
}


/*-----------------------------------------------------------------------
det_4d_matrix - ???
-----------------------------------------------------------------------*/

float 
det_4d_matrix(matrix)
 struct matrix_3d_rec *matrix;
{
 float det,temp;
 struct matrix_3d_rec m1;
 int max,maxrow,row,col,num_0s[4];

 for (row = 0; row < 4; row++) num_0s[row] = 0;
 for (row = 0; row < 4; row++) 
    for (col = 0; col < 4; col++)
       if (matrix->buf[row][col] == .0) num_0s[row] += 1;

 for (max = num_0s[0],maxrow = 0,row = 1; row < 4; row++) 
    if (num_0s[row] > max)
       {max = num_0s[row];
        maxrow = row;}

 for (det = .0,col = 0; col < 4; col++)
    {if (matrix->buf[maxrow][col] != .0)
        {setup_3d_cofactor(&m1,matrix,maxrow,col);
         temp = matrix->buf[maxrow][col] * det_3d_matrix(&m1);
         if ((maxrow + col) % 2) temp = -temp;
         det += temp;}
    }
 return det;
}


/*-----------------------------------------------------------------------
inverse_4d_matrix - ???
-----------------------------------------------------------------------*/

int
inverse_4d_matrix(inv,matrix)
 struct matrix_3d_rec *inv,*matrix;
{
 float det,cofactor;
 struct matrix_3d_rec m1,inv1;
 int row,col;

 for (det = .0,row = 0; row < 4; row++)
    {
     for (col = 0; col < 4; col++)
        {setup_3d_cofactor(&m1,matrix,row,col);
         cofactor = det_3d_matrix(&m1);
         if ((row + col) % 2) cofactor = -cofactor;
         if (row == 0) det += matrix->buf[row][col] * cofactor;
         inv1.buf[row][col] = cofactor;
        }
    }
 transpose_4d_matrix(inv,&inv1);
 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++) inv->buf[row][col] /= det;
 inv->status = MATRIX_INITIALIZED;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
pr_3d_matrix - ???
-----------------------------------------------------------------------*/

void
pr_3d_matrix(matrix)
 struct matrix_3d_rec *matrix;
{
 int row,col;

 printf("pr_3d_matrix\n");
 for (row = 0; row < 4; row++)
    {for (col  = 0; col < 4; col++)
        printf("%d/%d) %f   ",row,col,matrix->buf[row][col]);
     printf("\n");}
 printf("\n");
}


/***************** Start chop
find_bound_tl_tr(bounds,tl,tr)
 struct vector_3d_rec bounds[4];
 int *tl,*tr;
{
 struct vector_3d_rec *pts,*pvs,vs[2],min,max,temp;
 unsigned long lcv,indx;

 pts = bounds;
 find_pts_cuboid(pts,4,&min,&max);
 closest_pt_from(&min,pts,4,tl);
 setup_vector_3d_rec(&temp,max.x,(min.y + max.y) / 2.0,min.z);
 PRT1(printf("midftr = "));
 PRT1(pr_vector_3d_rec(&temp))

 vs[0] = pts[(*tl + 1) % 4];
 vs[1] = pts[(*tl + 3) % 4];
 pvs = vs;
 closest_pt_from(&temp,pvs,2,tr);
 if (*tr == 0) *tr = (*tl + 1) % 4;
   else *tr = (*tl + 3) % 4;

 PRT1(printf("tl = %d  tr = %d   midz = %f\n",*tl,*tr,(min.z + max.z) / 2.0))
 if ((pts[*tl].z > ((min.z + max.z) / 2.0)) && (pts[*tl].z > pts[*tr].z))
    {PRT1(printf("redefine...tl tr...\n"))
     *tl = *tr;
     *tr = (*tl + 1) % 4;
     PRT1(printf("final tl = %d  tr = %d\n",*tl,*tr))
    }
}
End chop *****************/



/*-----------------------------------------------------------------------
FindBoundTLTR1 - ???
-----------------------------------------------------------------------*/

void
FindBoundTLTR1(bounds,tl,tr,normal)
 struct vector_3d_rec bounds[4];
 int *tl,*tr;
 struct vector_3d_rec *normal;
{
 struct vector_3d_rec axial,*pts,*pvs,vs[2],min,max;
 float temp1,temp2,temp3,temp4;
 long deg;
 unsigned long indx;
 int is_axial_ori,status;

 pts = bounds;
 find_pts_cuboid(pts,4,&min,&max);
 closest_pt_from(&min,pts,4,tl);
 *tr = (*tl + 3) % 4;
 vs[0] = pts[(*tl + 1) % 4];
 vs[1] = pts[*tr];
 pvs = vs;
 PRT6(printf("FindBoundTLTR1...\n");pr_vector_3d_rec(normal))
 is_axial_ori = 0;
 if (((normal->x == .0) && (normal->y == .0)) ||
     (((normal->x == .0) || (normal->y == .0)) && (normal->z >= 0.939793)))
    {PRT6(printf("FindBoundTLTR1...1\n"))
     is_axial_ori = 1;}
   else
    {setup_vector_3d_rec(&axial,.0,.0,1.0);
     deg = (long) degree_angle_vectors(&axial,normal);
     if (deg < 0) deg = -deg;
     deg = deg % 180;
     if (deg > 90) deg = 180 - 90;
     if (deg < 22) is_axial_ori = 1;
    }
 PRT6(printf("is_axial_ori = %d\n",is_axial_ori))
 if (is_axial_ori)
    {temp1 = vs[0].y;
     if (temp1 < .0) temp1 = -temp1;
     temp2 = vs[1].y;
     if (temp2 < .0) temp2 = -temp2;
     if (temp1 <= temp2) *tr = (*tl + 1) % 4;
    }
   else
    {temp1 = vs[0].y;
     if (temp1 < .0) temp1 = -temp1;
     temp2 = vs[1].y;
     if (temp2 < .0) temp2 = -temp2;
     if (temp1 <= temp2) status = 1;
       else status = 0;
     temp3 = vs[0].z;
     if (temp3 < .0) temp3 = -temp3;
     temp4 = vs[1].z;
     if (temp4 < .0) temp4 = -temp4;
     if (temp3 <= temp4) status |= 2;
     if (status == 3) *tr = (*tl + 1) % 4;
       else if (status & 2) *tr = (*tl + 1) % 4;
     indx = (*tl + 2) % 4;
     vs[0] = pts[indx];
     vs[1] = pts[*tl];
     temp3 = vs[0].z;
     if (temp3 < .0) temp3 = -temp3;
     temp4 = vs[1].z;
     if (temp4 < .0) temp4 = -temp4;
     if (temp3 <= temp4)
        {PRT6(printf("... tl = %d  tr = %d\n",*tl,*tr))
         *tl = *tr;
         *tr = indx;}
    }
 PRT6(printf("final tl = %d  tr = %d\n",*tl,*tr))
/*
   else
    {setup_vector_3d_rec(&temp,max.x,(min.y + max.y) / 2.0,min.z);
     PRT1(printf("midftr = ")) PRT1(pr_vector_3d_rec(&temp))
     closest_pt_from(&temp,pvs,2,tr);
     if (*tr == 0) *tr = (*tl + 1) % 4;
       else *tr = (*tl + 3) % 4;

     PRT6(printf("tl = %d  tr = %d   midz = %f\n",*tl,*tr,(min.z + max.z) / 2.0))
     if ((pts[*tl].z > ((min.z + max.z) / 2.0)) && (pts[*tl].z > pts[*tr].z))
        {
         *tl = *tr;
         *tr = (*tl + 1) % 4;
         PRT6(printf("redefine final tl = %d  tr = %d\n",*tl,*tr))
        }
    }
*/
}


/*-----------------------------------------------------------------------
PlaneCoordMatrix - ???
-----------------------------------------------------------------------*/

void
PlaneCoordMatrix(xyztoplane,planetoxyz,base,normal,zrotating_rad)
 struct matrix_3d_rec *xyztoplane,*planetoxyz;
 struct vector_3d_rec *base,*normal;
 float zrotating_rad;
{
 identity_3d_matrix(planetoxyz);
 identity_3d_matrix(xyztoplane);
 PRT5(printf("..rad = %f base..",zrotating_rad);pr_vector_3d_rec(base);
      printf("..normal..");pr_vector_3d_rec(normal);)
 Rotate3dMatrixFromAxis(xyztoplane,planetoxyz,base,normal,zrotating_rad);
}


/***************** Start chop
def_axis_orientation(xyztoplane,planetoxyz,plane_trans,normal,is_plane_trans)
 struct matrix_3d_rec *xyztoplane,*planetoxyz,*plane_trans;
 struct vector_3d_rec *normal;
 int *is_plane_trans;
{
 struct vector_3d_rec zbounds[4],pts[3],vx,vy,dire;
 struct matrix_3d_rec ma1;
 int lcv;

 *is_plane_trans = 0;
 setup_vector_3d_rec(&(pts[0]),.0,0,.0);
 setup_vector_3d_rec(&(pts[1]),1.0,.0,.0);
 setup_vector_3d_rec(&(pts[2]),0.0,1.0,.0);
 for (lcv = 0; lcv < 3; lcv++)
    {
     trans_3d_pt(planetoxyz,&(pts[lcv]),&(zbounds[lcv]),0);
     PRT5(printf("z[%d]...",lcv)) 
     PRT5(pr_vector_3d_rec(&(zbounds[lcv])))
    }

 sub_vectors(&vx,&(zbounds[1]),&(zbounds[0]));
 sub_vectors(&vy,&(zbounds[2]),&(zbounds[0]));
 PRT5(printf("vx ...");pr_vector_3d_rec(&vx))
 PRT5(printf("vy ...");pr_vector_3d_rec(&vy))
 vector_cross_product(&dire,&vx,&vy);
 direction_cosine_vector(&dire);
 PRT5(printf("...normal...\n");pr_vector_3d_rec(normal);
      printf("vx * vy...");pr_vector_3d_rec(&dire);)
 if (!same_direction_vectors(&dire,normal))
    {PRT5(printf("GYX_REFLECTION...\n"))
     identity_3d_matrix(&ma1);
     ma1.buf[0][0] = 0;
     ma1.buf[1][1] = 0;
     ma1.buf[0][1] = 1;
     ma1.buf[1][0] = 1;
     mult_3d_matrix(xyztoplane,&ma1,xyztoplane);
     mult_3d_matrix(plane_trans,&ma1,plane_trans);
     inverse_4d_matrix(planetoxyz,xyztoplane);
     *is_plane_trans = 1;
    }
}
End chop ******************/



/*-----------------------------------------------------------------------
DefPlaneOrientation - ???
-----------------------------------------------------------------------*/

void
DefPlaneOrientation(xyztoplane,planetoxyz,plane_trans,base,normal,is_plane_trans)
 struct matrix_3d_rec *xyztoplane,*planetoxyz,*plane_trans;
 struct vector_3d_rec *base,*normal;
 int *is_plane_trans;
{
 struct vector_3d_rec zbounds[4];
 unsigned long lcv,tr,tl,next;
 float rot;
 struct matrix_3d_rec ma1;
 int is_inverse;

 PRT6(printf("base...\n");pr_vector_3d_rec(base);)
 PRT6(setup_vector_3d_rec(&dire1,0.0,1.0,.0);
      trans_3d_pt(planetoxyz,&dire1,&pt,0);
      printf("[0,1]...");pr_vector_3d_rec(&pt);
      sub_vectors(&dire,&pt,base);
      direction_cosine_vector(&dire);
      printf("dire...");pr_vector_3d_rec(&dire);
      printf("ang = %s\n",get_vector_angle_str1(&dire));)

 setup_vector_3d_rec(&(zbounds[0]),-128.0,-128.0,.0);
 setup_vector_3d_rec(&(zbounds[1]),128.0,-128.0,.0);
 setup_vector_3d_rec(&(zbounds[2]),128.0,128.0,.0);
 setup_vector_3d_rec(&(zbounds[3]),-128.0,128.0,.0);
 *is_plane_trans = 0;
 for (lcv = 0; lcv < 4; lcv++)
    {
     trans_3d_pt(planetoxyz,&(zbounds[lcv]),&(zbounds[lcv]),0);
     PRT6(printf("%d)...",lcv)) 
     PRT6(pr_vector_3d_rec(&(zbounds[lcv])))
    }
 FindBoundTLTR1(zbounds,&tl,&tr,normal);
 PRT6(printf("tl = %d  tr = %d\n",tl,tr))
 if (tl != 0)
    {if (tl == 1) rot = RADIAN / 2;
       else if (tl == 2) rot = RADIAN;
       else if (tl == 3) rot = 1.5 * RADIAN;
     PRT6(printf("==1 setup_zrotate_3d_matrix rot = %f\n",rot))
     setup_zrotate_3d_matrix(&ma1,rot);
     mult_3d_matrix(xyztoplane,&ma1,xyztoplane);
     mult_3d_matrix(plane_trans,&ma1,plane_trans);
     *is_plane_trans = 1;

     is_inverse = 1;
    }
   else is_inverse = 0;

 next = (tl + 1) % 4;
 if (next != tr)
    {PRT6(printf("GYX_REFLECTION...\n"))
     identity_3d_matrix(&ma1);
     ma1.buf[0][0] = 0;
     ma1.buf[1][1] = 0;
     ma1.buf[0][1] = 1;
     ma1.buf[1][0] = 1;
     mult_3d_matrix(xyztoplane,&ma1,xyztoplane);
     mult_3d_matrix(plane_trans,&ma1,plane_trans);
     *is_plane_trans = 1;
     is_inverse = 1;
    }

 if (is_inverse)
    {inverse_4d_matrix(planetoxyz,xyztoplane);
     PRT6(printf("is_inverse center...\n");
         setup_vector_3d_rec(&(zbounds[0]),-128.0,-128.0,.0);
         setup_vector_3d_rec(&(zbounds[1]),128.0,-128.0,.0);
         setup_vector_3d_rec(&(zbounds[2]),128.0,128.0,.0);
         setup_vector_3d_rec(&(zbounds[3]),-128.0,128.0,.0);

         for (lcv = 0; lcv < 4; lcv++)
           {
            trans_3d_pt(planetoxyz,&(zbounds[lcv]),&(zbounds[lcv]),0);
            printf("f[%d]...",lcv); pr_vector_3d_rec(&(zbounds[lcv]));
           }
         )
    }
 PRT6(
      printf("DefPlaneOrientation   end...\n\n");)
}


/*************** Start chop
orthographic_observer(result,observe_x,observe_y,observe_z)
 struct matrix_3d_rec *result;
 float observe_x,observe_y,observe_z;
{
 float direction_angle,dist;

 PRT1(printf("observe...\n"))
 if (is_float_0(observe_x)) direction_angle = RADIAN / 2.0;
   else direction_angle = atan2((double) observe_y,(double) observe_x);

 zrotate_3d_matrix(result,direction_angle);
 PRT1(printf("observe D...\n"))
 PRT1(pr_3d_matrix(result))

 dist = (float) observe_x * (float) observe_x +
        (float) observe_y * (float) observe_y;
 dist = sqrt((double) dist);
 if (is_float_0(observe_z)) direction_angle = RADIAN / 2.0;
   else direction_angle = atan2((double) dist,(double) observe_z);
 direction_angle += RADIAN;

 yrotate_3d_matrix(result,direction_angle);
 PRT1(printf("observe Q...\n"))
 PRT1(pr_3d_matrix(result))

 direction_angle = atan2((double) result->buf[1][1],(double) result->buf[0][1]);
 zrotate_3d_matrix(result,-direction_angle);
}

orthographic_proj(result,base_vector,normal,rotating_rad,
                  observe_x,observe_y,observe_z)
 struct matrix_3d_rec *result;
 struct vector_3d_rec *base_vector,*normal;
 float rotating_rad;
 float observe_x,observe_y,observe_z;
{
 struct matrix_3d_rec actual,observe;

 identity_3d_matrix(&actual);
 actual_coord(&actual,base_vector,normal,rotating_rad);
 orthographic_observer(&observe,observe_x,observe_y,observe_z);
 mult_3d_matrix(result,&observe,&actual);
}
End chop ****************/




/*-----------------------------------------------------------------------
trans_3d_xyz - ???
-----------------------------------------------------------------------*/

void
trans_3d_xyz(matrix,x,y,z,newx,newy,newz,is_result_rounded)
 struct matrix_3d_rec *matrix;
 float x,y,z,*newx,*newy,*newz;
 int is_result_rounded;
{
 float x1,y1,z1;

 TRACE(printf("	trans_3d_xyz\n"););

 x1 = matrix->buf[0][0] * x + matrix->buf[0][1] * y +
         matrix->buf[0][2] * z + matrix->buf[0][3];
 y1 = matrix->buf[1][0] * x + matrix->buf[1][1] * y +
         matrix->buf[1][2] * z + matrix->buf[1][3];
 z1 = matrix->buf[2][0] * x + matrix->buf[2][1] * y +
         matrix->buf[2][2] * z + matrix->buf[2][3];
 
 if (is_result_rounded)
    {*newx = (float) ((double) (x1 + .5));
     *newy = (float) ((double) (y1 + .5));
     *newz = (float) ((double) (z1 + .5));
    }
   else
    {*newx = x1;
     *newy = y1;
     *newz = z1;}
}


/*-----------------------------------------------------------------------
trans_3d_pt - ???
-----------------------------------------------------------------------*/

void
trans_3d_pt(matrix,pt,map_pt,is_result_rounded)
 struct matrix_3d_rec *matrix;
 struct vector_3d_rec *pt,*map_pt;
 int is_result_rounded;
{
 trans_3d_xyz(matrix,pt->x,pt->y,pt->z,
      &(map_pt->x),&(map_pt->y),&(map_pt->z),is_result_rounded);
}


/*************** Start chop
transform_vector_3d_rec(matrix,pt,transformed_pt)
 struct matrix_3d_rec *matrix;
 struct vector_3d_rec *pt,*transformed_pt;
{
 float x,y,z;

 x = matrix->buf[0][0] * pt->x + matrix->buf[0][1] * pt->y +
         matrix->buf[0][2] * pt->z + matrix->buf[0][3];
 y = matrix->buf[1][0] * pt->x + matrix->buf[1][1] * pt->y +
         matrix->buf[1][2] * pt->z + matrix->buf[1][3];
 z = matrix->buf[2][0] * pt->x + matrix->buf[2][1] * pt->y +
         matrix->buf[2][2] * pt->z + matrix->buf[2][3];
 transformed_pt->x = x;
 transformed_pt->y = y;
 transformed_pt->z = z;
}
End chop **********************/



/*-----------------------------------------------------------------------
proj_pt_vector_3d_rec - ???
-----------------------------------------------------------------------*/

void
proj_pt_vector_3d_rec(matrix,coord_ratio,pt,transformed_pt)
 struct matrix_3d_rec *matrix;
 struct vector_3d_rec *coord_ratio;
 struct vector_3d_rec *pt,*transformed_pt;
{
 float x,y,z,x1,y1,z1;

 x1 = matrix->buf[0][0] * pt->x + matrix->buf[0][1] * pt->y +
      matrix->buf[0][2] * pt->z + matrix->buf[0][3] * coord_ratio->x;
 y1 = matrix->buf[1][0] * pt->x + matrix->buf[1][1] * pt->y +
      matrix->buf[1][2] * pt->z + matrix->buf[1][3] * coord_ratio->y;
 z1 = matrix->buf[2][0] * pt->x + matrix->buf[2][1] * pt->y +
      matrix->buf[2][2] * pt->z + matrix->buf[2][3] * coord_ratio->z;

 if (coord_ratio->x != 1.0) x1 = (float) (x1 / coord_ratio->x);
 if (coord_ratio->y != 1.0) y1 = (float) (y1 / coord_ratio->y);
 if (coord_ratio->z != 1.0) z1 = (float) (z1 / coord_ratio->z);
 transformed_pt->x = x;
 transformed_pt->y = y;
 transformed_pt->z = z;
}


/*-----------------------------------------------------------------------
rotate_3d_rad - ???
-----------------------------------------------------------------------*/

int
rotate_3d_rad(centerx,centery,centerz,firstx,firsty,firstz,
              nextx,nexty,nextz,rad)
 float centerx,centery,centerz,firstx,firsty,firstz,nextx,nexty,nextz,*rad;
{
 struct vector_3d_rec vector,vector1,vector2,v1,v2;
 float dot,angle;

 setup_vector_3d_rec(&vector,centerx,centery,centerz);
 setup_vector_3d_rec(&vector1,firstx,firsty,firstz);
 setup_vector_3d_rec(&vector2,nextx,nexty,nextz);

 sub_vectors(&v1,&vector1,&vector);
 sub_vectors(&v2,&vector2,&vector);

 angle = degree_angle_vectors(&v1,&v2);
 dot = (float) radian(angle);
 *rad = dot;

 return NO_ERROR;
}


/*-----------------------------------------------------------------------
matrix_3d_to_2d - ???
-----------------------------------------------------------------------*/

void
matrix_3d_to_2d(mat2d,mat3d)
 struct matrix_2d_rec *mat2d;
 struct matrix_3d_rec *mat3d;
{
 mat2d->buf[0][0] = mat3d->buf[0][0];
 mat2d->buf[0][1] = mat3d->buf[0][1];
 mat2d->buf[0][2] = mat3d->buf[0][3];
 mat2d->buf[1][0] = mat3d->buf[1][0];
 mat2d->buf[1][1] = mat3d->buf[1][1];
 mat2d->buf[1][2] = mat3d->buf[1][3];
 mat2d->buf[2][0] = mat3d->buf[3][0];
 mat2d->buf[2][1] = mat3d->buf[3][1];
 mat2d->buf[2][2] = mat3d->buf[3][3];

}


/*-----------------------------------------------------------------------
 matrix_2d_to_3d- ???
-----------------------------------------------------------------------*/

void
matrix_2d_to_3d(mat3d,mat2d)
 struct matrix_3d_rec *mat3d;
 struct matrix_2d_rec *mat2d;
{
 identity_3d_matrix(mat3d);
 mat3d->buf[0][0] = mat2d->buf[0][0];
 mat3d->buf[0][1] = mat2d->buf[0][1];
 mat3d->buf[0][3] = mat2d->buf[0][2];

 mat3d->buf[1][0] = mat2d->buf[1][0];
 mat3d->buf[1][1] = mat2d->buf[1][1];
 mat3d->buf[1][3] = mat2d->buf[1][2];

 mat3d->buf[3][0] = mat2d->buf[2][0];
 mat3d->buf[3][1] = mat2d->buf[2][1];
 mat3d->buf[3][3] = mat2d->buf[2][2];

}
