/****************************************************************************
buf_3d_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: buf_3d_rec.c,v 1.7 1999/07/07 18:01:05 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <sys/types.h>

/* #include <sys/timeb.h> */

#include "geometry.h"
#include "buf_3d_rec.h"
#include "common.h"
#include "error_constants.h"
#include "ia_hdr.h"
#include "rect.h"
#include "nria_lib.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT1(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRT1(x)



/*------------------------------------------------------------------------
init_slice_3d_rec - ???
------------------------------------------------------------------------*/

void
init_slice_3d_rec(slice)
 struct slice_3d_rec *slice;
{
 slice->status = 0;
 slice->s1 = NULL;
 slice->s2 = NULL;
 slice->s1_info = NULL;
 slice->s2_info = NULL;
 slice->slice_thickness = .0;
 slice->slice_thickness = .0;
}


/*------------------------------------------------------------------------
init_buf_3d_rec - ???
------------------------------------------------------------------------*/

void
init_buf_3d_rec(_3d)
 struct buf_3d_rec *_3d;
{
 _3d->status = 0;
 _3d->startx = _3d->starty = _3d->startz = 0;
 _3d->id = NULL;
 _3d->total_slices = _3d->num_slices = 0;
 _3d->sls = NULL;
}


/*------------------------------------------------------------------------
set_3d_start_xyz - ???
------------------------------------------------------------------------*/

void
set_3d_start_xyz(_3d,startx,starty,startz)
 struct buf_3d_rec *_3d;
{
 _3d->startx = startx;
 _3d->starty = starty;
 _3d->startz = startz;

}


/*------------------------------------------------------------------------
free_buf_3d_rec_slices - ???
------------------------------------------------------------------------*/

void
free_buf_3d_rec_slices(_3d)
 struct buf_3d_rec *_3d;
{
 unsigned long lcv;

 if (_3d->status & BUF_NON_EMPTY)
    {if (_3d->total_slices > 0)
        {if (!(_3d->status & _3D_BUF_DO_NOT_DEALLOCATE_SLICES))
            {
             for (lcv = 0; lcv < _3d->num_slices; lcv++)
               {if (_3d->sls[lcv].status & SLICE_3D_ORIGINAL)
                   free_buf_2d_rec(_3d->sls[lcv].s1);
                c_free(_3d->sls[lcv].s1);}
            }
        }
    }
}


/*------------------------------------------------------------------------
free_buf_3d_rec - ???
------------------------------------------------------------------------*/

void
free_buf_3d_rec(_3d)
 struct buf_3d_rec *_3d;
{
 if (_3d->status & BUF_NON_EMPTY)
    {
     c_free(_3d->id);
     if (_3d->total_slices > 0)
        {free_buf_3d_rec_slices(_3d);
         c_free(_3d->sls);}
    }
 init_buf_3d_rec(_3d);
}


/*------------------------------------------------------------------------
init_ary_pt_3d_rec - ???
------------------------------------------------------------------------*/

void
init_ary_pt_3d_rec(xyz)
 struct ary_pt_3d_rec *xyz;
{
 xyz->total_coords = 0;
 xyz->num_coords = 0;
 xyz->coords = NULL;
}


/*------------------------------------------------------------------------
free_ary_pt_3d_rec - ???
------------------------------------------------------------------------*/

void
free_ary_pt_3d_rec(xyz)
 struct ary_pt_3d_rec *xyz;
{
 if (xyz->total_coords > 0)
    c_free(xyz->coords);
 init_ary_pt_3d_rec(xyz);
}


/*------------------------------------------------------------------------
sort_pt_3d_rec_proc - ???
------------------------------------------------------------------------*/

int
sort_pt_3d_rec_proc(xyz1,xyz2)
 struct pt_3d_rec *xyz1,*xyz2;
{
 if (xyz1->z > xyz2->z) return 1;
 if (xyz1->z < xyz2->z) return -1;
 if (xyz1->y > xyz2->y) return 1;
 if (xyz1->y < xyz2->y) return -1;
 if (xyz1->x > xyz2->x) return 1;
 if (xyz1->x < xyz2->x) return -1;
 return 0;
}


/*------------------------------------------------------------------------
sort_ary_pt_3d_rec - ???
------------------------------------------------------------------------*/

void
sort_ary_pt_3d_rec(xyz)
 struct ary_pt_3d_rec *xyz;
{
 if (xyz->num_coords > 1)
    qsort(xyz->coords,xyz->num_coords,
        (size_t) sizeof(*(xyz->coords)),sort_pt_3d_rec_proc);
}


/*------------------------------------------------------------------------
init_ary_3d_xyz_rec - ???
------------------------------------------------------------------------*/

void
init_ary_3d_xyz_rec(xyz)
 struct ary_3d_xyz_rec *xyz;
{
 xyz->status = 0;
 xyz->total_coords = 0;
 xyz->num_coords = 0;
 xyz->coords = NULL;
}


/*------------------------------------------------------------------------
free_ary_3d_xyz_rec - ???
------------------------------------------------------------------------*/

void
free_ary_3d_xyz_rec(xyz)
 struct ary_3d_xyz_rec *xyz;
{
 if (xyz->total_coords > 0)
    c_free(xyz->coords);
 init_ary_3d_xyz_rec(xyz);
}


/*------------------------------------------------------------------------
total_zslices - ???
------------------------------------------------------------------------*/

void
total_zslices(slice_thickness,pixel_size,total_slices,total_zslice,last_slice_thickness)
 float slice_thickness,pixel_size;
 unsigned long total_slices,*total_zslice;
 float *last_slice_thickness;
{
 double zslices,rem;

 zslices = slice_thickness * total_slices / pixel_size;
 rem = fmod(zslices,(double) pixel_size);
 *total_zslice = (float) floor((double) zslices);
 if (rem > .0)
    {(*total_zslice) += 1;
     *last_slice_thickness = (float) rem;}
   else *last_slice_thickness = pixel_size;
}


/*------------------------------------------------------------------------
get_zindx - ???
------------------------------------------------------------------------*/

int
get_zindx(zpixel_size,slice_thickness,actual_sl_num,startz,dimz,indx)
 float zpixel_size,slice_thickness;
 unsigned long startz,dimz;
 long actual_sl_num,*indx;
{
 float sl_num,half;

 half = zpixel_size / 2.0;
 sl_num = actual_sl_num * slice_thickness + slice_thickness / 2.0;
 sl_num -= half;
 sl_num = (sl_num / zpixel_size);
 sl_num = (float) ceil((double) sl_num);
 *indx = (long) sl_num - startz;
 if (*indx >= dimz)
    {TRACE(printf("get_zindx *indx = %d >= dimz = %d  startz = %d actual_sl_num = %d  thk = %f\n",*indx,dimz,startz,actual_sl_num,slice_thickness))
     return CANCEL;}
 return NO_ERROR;
}


/*------------------------------------------------------------------------
setup_buf_3d_rec - ???
------------------------------------------------------------------------*/

int
setup_buf_3d_rec(_3d,id,dimx,dimy,total_slices,xpixel_size,ypixel_size,
                 zpixel_size,slice_thickness,startx,starty,start_slice_num)
 struct buf_3d_rec *_3d;
 char *id;
 unsigned long dimx,dimy,total_slices;
 float xpixel_size,ypixel_size,zpixel_size,slice_thickness;
 unsigned long startx,starty;
 long start_slice_num;
{
 int err;
 long lcv,cur;
 float last_slice_thickness,lb,last;

 free_buf_3d_rec(_3d);
 total_zslices(slice_thickness,zpixel_size,total_slices,
             &(_3d->dimz),&last_slice_thickness);

 if ((err = get_ptr(&(_3d->sls),&(_3d->total_slices),_3d->dimz,
           (size_t) sizeof(*(_3d->sls))))  != NO_ERROR)
    return err;
 _3d->status = BUF_NON_EMPTY | BUF_OWN_CONTIGUOUS_MEMORY | _3D_VALID;

 for (lcv = 0; lcv < _3d->total_slices; lcv++)
    {init_slice_3d_rec(&(_3d->sls[lcv]));
     _3d->sls[lcv].slice_thickness = zpixel_size;
    }
 _3d->num_slices = _3d->total_slices;

 _3d->sls[lcv - 1].slice_thickness = last_slice_thickness;

 if ((err = cp_str(&(_3d->id),id)) != NO_ERROR) return err;
 _3d->dimx = dimx;
 _3d->dimy = dimy;

 _3d->xpixel_size = xpixel_size;
 _3d->ypixel_size = ypixel_size;
 _3d->zpixel_size = zpixel_size;
 _3d->slice_thickness = slice_thickness;
 _3d->startx = startx;
 _3d->starty = starty;
 setup_vector_3d_rec(&(_3d->center),
     (float) _3d->dimx * _3d->xpixel_size / 2.0,
     (float) _3d->dimy * _3d->ypixel_size / 2.0,
     (float) _3d->dimz * _3d->zpixel_size / 2.0);

 if (start_slice_num == 0) _3d->startz = 0;
   else
    {
     if ((err = get_zindx(_3d->zpixel_size,slice_thickness,start_slice_num,
         0,_3d->dimz,&cur)) != NO_ERROR)
        return err;
     TRACE(printf("start original = %d\n",cur))
     lb = cur * _3d->zpixel_size;
     last = start_slice_num * slice_thickness;
     for (;lb > last; cur -= 1)
        lb -= _3d->zpixel_size;
     TRACE(printf("first startz = %d\n",cur))
     _3d->startz = cur;}


 return err;
}


/*------------------------------------------------------------------------
default_buf_3d_rec_2d_dims - ???
------------------------------------------------------------------------*/

void
default_buf_3d_rec_2d_dims(_3d,dimx,dimy)
 struct buf_3d_rec *_3d;
 unsigned long *dimx,*dimy;
{
 unsigned long max;

 if (_3d->dimx >= _3d->dimy) max = _3d->dimx;
   else max = _3d->dimy;
 if (max < _3d->dimz) max = _3d->dimz;

 *dimx = max;
 *dimy = max;
}


/********************* chop
setup_3d_mask(_3d,dimx,dimy,total_slices,zero_if_new)
 struct buf_3d_rec *_3d;
 unsigned long dimx,dimy,total_slices;
 int zero_if_new;
{
 int err;
 unsigned long lcv;
 struct buf_2d_rec *_2d;

 free_buf_3d_rec(_3d);
 if ((err = get_ptr(&(_3d->slices),&(_3d->total_slices),total_slices,
           (size_t) sizeof(*(_3d->slices)))) == NO_ERROR)
    {
     for (lcv = 0; lcv < _3d->total_slices; lcv++)
        _3d->slices[lcv] = NULL;
     for (lcv = 0; lcv < _3d->total_slices; lcv++)
        {
         if ((err = get_ptr(&_2d,NULL,1,(size_t) sizeof(*_2d))) != NO_ERROR) 
            return err;
         init_buf_2d_rec(_2d);
         if ((err = setup_buf_2d_rec(_2d,dimx,dimy,1,zero_if_new)) != NO_ERROR)
            {free_buf_2d_rec(_2d);
             c_free(_2d);
             return err;}
         _3d->slices[lcv] = _2d;
        }

     _3d->status = BUF_NON_EMPTY | BUF_OWN_CONTIGUOUS_MEMORY | 
                   _3D_MASK | _3D_VALID;
     _3d->num_slices = 0;
     _3d->dimx = dimx;
     _3d->dimy = dimy;
     _3d->dimz = total_slices;
     _3d->xpixel_size = .0;
     _3d->ypixel_size = .0;
     _3d->zpixel_size = .0;
     _3d->slice_thickness = .0;

    }
 return err;
}
unchop **********************/


/*------------------------------------------------------------------------
InitBuf3dRecData - ???
------------------------------------------------------------------------*/

int
InitBuf3dRecData(_3d,val)
 struct buf_3d_rec *_3d;
 double val;
{
 unsigned long lcv;
 int err;

 for (lcv = 0; lcv < _3d->num_slices; lcv++)
    if (_3d->sls[lcv].status & SLICE_3D_ORIGINAL)
       if ((err = InitBuf2dRecData(_3d->sls[lcv].s1,val)) != NO_ERROR)
          return err;
 return NO_ERROR;
}


/*------------------------------------------------------------------------
setup_3d_plane_2d_rec - ???
------------------------------------------------------------------------*/

void
setup_3d_plane_2d_rec(_3d_2d,_3d,plane_2d,transform_2d)
 struct _3d_plane_2d_rec *_3d_2d;
 struct buf_3d_rec *_3d;
 struct buf_2d_rec *plane_2d,*transform_2d;
{
 _3d_2d->_3d = _3d;
 _3d_2d->plane_2d = plane_2d;
 _3d_2d->transform_2d = transform_2d;
}


/*------------------------------------------------------------------------
pr_buf_3d_rec - ???
------------------------------------------------------------------------*/

void
pr_buf_3d_rec(_3d)
 struct buf_3d_rec *_3d;
{
 long lcv;

 printf("pr_buf_3d_rec status = %lx xpix = %f y = %f z = %f\n",
  (unsigned long) _3d->status,_3d->xpixel_size,_3d->ypixel_size,_3d->zpixel_size);
 printf("start  x = %ld  y = %ld  z = %ld\n",
	_3d->startx,_3d->starty,_3d->startz);
 printf("  dimx = %ld   dimy = %ld   dimz = %ld  slice_thickness = %f\n",
    _3d->dimx,_3d->dimy,_3d->dimz,_3d->slice_thickness);
 printf("  total_slices = %ld   num_slices = %ld   sls = %p\n",
    _3d->total_slices,_3d->num_slices,_3d->sls);
 for (lcv = 0; lcv < _3d->num_slices; lcv++)
    {printf("slice %ld) s1 = %p  s2 = %p  thck = %f  ratio = %f",
          lcv,_3d->sls[lcv].s1,_3d->sls[lcv].s2,
          _3d->sls[lcv].slice_thickness,
          _3d->sls[lcv].ratio);
     if (_3d->sls[lcv].s1 != NULL) 
        printf("status = %lx dimx = %ld dimy = %ld..s1num = %d",
          _3d->sls[lcv].s1->status4,_3d->sls[lcv].s1->dimx,_3d->sls[lcv].s1->dimy,
          (int) _3d->sls[lcv].s1_info->num);
     if (_3d->sls[lcv].s2 != NULL) 
        printf("status = %lx dimx = %ld dimy = %ld..s2num = %d",
          _3d->sls[lcv].s2->status4,_3d->sls[lcv].s2->dimx,_3d->sls[lcv].s2->dimy,
          (int) _3d->sls[lcv].s2_info->num);

     if (_3d->sls[lcv].status & SLICE_3D_ORIGINAL)
        {printf("...ORIGINAL\n");
         if (_3d->sls[lcv].s1 != NULL) 
            printf("s1...startoffx = %ld   startoffy = %ld\n",
                 _3d->sls[lcv].s1->startoffx,_3d->sls[lcv].s1->startoffy);
        }
       else printf("\n");
    }
 printf("pr_buf_3d_rec end...\n\n");
}


/*------------------------------------------------------------------------
add_3d_slice_infect - ???
------------------------------------------------------------------------*/

int
add_3d_slice_infect(_3d,_2d,_2d_info,indx,slice_thickness,is_share_border_pixel)
 struct buf_3d_rec *_3d;
 struct buf_2d_rec *_2d;
 struct hdr_scan_entry_rec *_2d_info;
 unsigned long indx;
 float slice_thickness;
 int is_share_border_pixel;
{
 long cur,cnt,last,last1;
 float up_center,sl_num,half,start,end;
 int err;

 if ((err = get_zindx(_3d->zpixel_size,slice_thickness,indx,
             _3d->startz,_3d->dimz,&cur)) != NO_ERROR)
    return err;
 if (get_zindx(_3d->zpixel_size,slice_thickness,indx + 1,
             _3d->startz,_3d->dimz,&last1) != NO_ERROR)
    last1 = _3d->dimz - 1;

 _3d->sls[cur].s1 = _2d;
 _3d->sls[cur].s1_info = _2d_info;
 _3d->sls[cur].status |= SLICE_3D_ORIGINAL;
 if (_2d == NULL) return NO_ERROR;
 half = _3d->zpixel_size / 2.0;
 if (indx == 0)
    {up_center = half;
     cnt = 0;}
   else
    {
     start = indx * slice_thickness;
     sl_num = (start / _3d->zpixel_size);
     sl_num = (float) ceil((double) sl_num);
     if ((sl_num * _3d->zpixel_size) < start) sl_num = sl_num + 1.0;
     cnt = (long) sl_num - _3d->startz;
    }

 end = (indx + 1) * slice_thickness;

 sl_num = (end / _3d->zpixel_size);
 sl_num = (float) ceil((double) sl_num);
 if (sl_num >= (float) (_3d->dimz - 1)) sl_num = (float) (_3d->dimz - 1);
   else if ((sl_num * _3d->zpixel_size) >= end) sl_num = sl_num - 1.0;

 last = (long) sl_num - _3d->startz;
 if (last >= _3d->dimz) last = _3d->dimz - 1;

 PRT1(printf("startz = %d indx = %d start = %.4f end = %.4f sl_num = %.4f cnt = %d %.4f last = %d %.4f\n",
              _3d->startz,indx,start,end,sl_num,
              cnt,cnt * _3d->zpixel_size,
              last,last * _3d->zpixel_size))

/*  
 if (is_share_border_pixel)
    {
     up_center = (cnt + _3d->startz) * _3d->zpixel_size + half;
     sl_num = indx * slice_thickness;
     if (up_center < sl_num)
        if ((sl_num - up_center) > half) cnt += 1;

     sl_num += slice_thickness;
     bot_center = (last + _3d->startz) * _3d->zpixel_size + half;
     if (bot_center > sl_num)
        if ((bot_center - sl_num) > half) last -= 1;
    }
*/
 PRT1(printf("..._3d->zpixel_size = %.4f slice_thickness = %.4f cur = %d cnt = %d last = %d\n",
              _3d->zpixel_size,slice_thickness,cur,cnt,last))
 for (; cnt <= last; cnt++)
    if (cnt != cur)
       {_3d->sls[cnt].s1 = _2d;
        _3d->sls[cnt].s1_info = _2d_info;}
 return NO_ERROR;
}


/*------------------------------------------------------------------------
add_3d_slice_own - ???
------------------------------------------------------------------------*/

int
add_3d_slice_own(_3d,_2d,_2d_info,indx,slice_thickness)
 struct buf_3d_rec *_3d;
 struct buf_2d_rec *_2d;
 struct hdr_scan_entry_rec *_2d_info;
 unsigned long indx;
 float slice_thickness;
{
 long cur;
 int err;

 if ((err = get_zindx(_3d->zpixel_size,slice_thickness,indx,
                _3d->startz,_3d->dimz,&cur)) != NO_ERROR)
    return err;

 _3d->sls[cur].s1 = _2d;
 _3d->sls[cur].s1_info = _2d_info;
 _3d->sls[cur].status |= SLICE_3D_ORIGINAL;

 return NO_ERROR;
}


/*------------------------------------------------------------------------
setup_3d_interpolation - ???
------------------------------------------------------------------------*/

int
setup_3d_interpolation(_3d)
 struct buf_3d_rec *_3d;
{
 unsigned long lcv,up,down;
 float bottom_lb,top_ub,dist,cur_dist;

 for (lcv = 0; lcv < _3d->total_slices; lcv++)
    {
     if (!(_3d->sls[lcv].status & SLICE_3D_ORIGINAL))
        {if (lcv == 0) down = 0;
           else
            {for (down = lcv - 1;!(_3d->sls[down].status & SLICE_3D_ORIGINAL); down--)
                if (down == 0) break;
            }
         for (up = lcv+1; up < _3d->total_slices; up++)
            if (_3d->sls[up].status & SLICE_3D_ORIGINAL) break;
         if (up == _3d->total_slices)
            {if ((lcv != down) && (_3d->sls[down].status & SLICE_3D_ORIGINAL))
                {_3d->sls[lcv].s1 = _3d->sls[down].s1;
                 _3d->sls[lcv].s1_info = _3d->sls[down].s1_info;}
            }
           else if ((lcv == down) || !(_3d->sls[down].status & SLICE_3D_ORIGINAL))
                {_3d->sls[lcv].s1 = _3d->sls[up].s1;
                 _3d->sls[lcv].s1_info = _3d->sls[up].s1_info;}
           else
            {
             top_ub = (down + 1) * _3d->zpixel_size;
             bottom_lb = up * _3d->zpixel_size;
             dist = bottom_lb - top_ub;
             cur_dist = lcv * _3d->zpixel_size + _3d->zpixel_size / 2.0;
             _3d->sls[lcv].ratio = (cur_dist - top_ub) / dist;
             if (_3d->sls[down].s1 == NULL)
                {if (_3d->sls[lcv].ratio > .5)
                    {_3d->sls[lcv].s1_info = _3d->sls[up].s1_info;
                     _3d->sls[lcv].s1 = _3d->sls[up].s1;
                    }
                   else _3d->sls[lcv].s1_info = _3d->sls[down].s1_info;
                }
               else if (_3d->sls[up].s1 == NULL)
                {if (_3d->sls[lcv].ratio <= .5)
                    {_3d->sls[lcv].s1_info = _3d->sls[down].s1_info;
                     _3d->sls[lcv].s1 = _3d->sls[down].s1;
                    }
                   else _3d->sls[lcv].s1_info = _3d->sls[up].s1_info;
                }
               else
                 {_3d->sls[lcv].s1_info = _3d->sls[down].s1_info;
                  _3d->sls[lcv].s1 = _3d->sls[down].s1;
                  _3d->sls[lcv].s2_info = _3d->sls[up].s1_info;
                  _3d->sls[lcv].s2 = _3d->sls[up].s1;}
            }
        }
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------------
find_slice_in_3d - ???
------------------------------------------------------------------------*/

int
find_slice_in_3d(_3d,_2d,indx)
 struct buf_3d_rec *_3d;
 struct buf_2d_rec *_2d;
 unsigned long *indx;
{
 unsigned long lcv;

 *indx = 0;
 for (lcv = 0; lcv < _3d->num_slices; lcv++)
    if (_3d->sls[lcv].s1 == _2d)
       {*indx = lcv;
        return NO_ERROR;}
 return NOT_FOUND;
}


/*------------------------------------------------------------------------
SortBuf3dRec - ???
------------------------------------------------------------------------*/

int 
SortBuf3dRec(ptr1,ptr2)
 void *ptr1,*ptr2;
{
 return str_cmp((*((struct buf_3d_rec **) ptr1))->id,
                (*((struct buf_3d_rec **) ptr2))->id);
}


/*------------------------------------------------------------------------
reverse_3d_slice_order - ???
------------------------------------------------------------------------*/

int
reverse_3d_slice_order(_3d)
 struct buf_3d_rec *_3d;
{
 long lcv,indx;
 struct slice_3d_rec slice;

 if (_3d->status & _3D_DEFAULT_ORDER) 
    {_3d->status &= ~_3D_DEFAULT_ORDER;
     _3d->status |= _3D_BOTTOM_TO_TOP;}
   else
    {_3d->status &= ~_3D_BOTTOM_TO_TOP;
     _3d->status |= _3D_DEFAULT_ORDER;}

 if (_3d->num_slices <= 1) return NO_ERROR;
 for (lcv = 0,indx = _3d->num_slices - 1; lcv < _3d->num_slices / 2; lcv++,indx--)
    {slice = _3d->sls[lcv];
     _3d->sls[lcv] = _3d->sls[indx];
     _3d->sls[indx] = slice;}
 return NO_ERROR;   
}


/************************** chop

float resampled_val(_3d,dx,dy,dz)
 struct buf_3d_rec *_3d;
 float dx,dy,dz;
{
 float fx,fy,fz,disx,disy,disz,fval,fval1,ndisz;
 float ndisx_ndisy,disx_ndisy,ndisx_disy,disx_disy;
 long tx,ty,tz,nx,ny,nz,val;

 fz = floor((double) dz);
 tz = (long) fz;
 fy = floor((double) dy);
 ty = (long) fy;
 fx = floor((double) dx);
 tx = (long) fx;

 disx = dx - fx;
 disy = dy - fy;
 disz = dz - fz;
 ndisz = 1.0 - disz;
 disx_disy = disx * disy;
 disx_ndisy = disx - disx_disy;
 ndisx_disy = disy - disx_disy;
 ndisx_ndisy = 1 - disx - ndisx_disy;

 nx = tx + 1;
 ny = ty + 1;
 if (ndisz > ALMOST_0)
    {if ((val = abs_xyz_val(_3d,tx,ty,tz)) == 0) fval = .0;
       else fval = ndisx_ndisy * (float) val;
     if ((val = abs_xyz_val(_3d,nx,ty,tz)) != 0)
        fval += disx_ndisy * (float) val;
     if ((val = abs_xyz_val(_3d,tx,ny,tz)) != 0)
        fval += ndisx_disy * (float) val;
     if ((val = abs_xyz_val(_3d,nx,ny,tz)) != 0)
        fval += disx_disy * (float) val;
     if (fval != 0) fval *= ndisz;
    }
   else fval = .0;
 if ((disz > ALMOST_0) && ((nz = tz + 1) < _3d->dimz))
    {if ((val = abs_xyz_val(_3d,tx,ty,nz)) == 0) fval1 = .0;
       else fval1 = ndisx_ndisy * (float) val;
     if ((val = abs_xyz_val(_3d,nx,ty,nz)) != 0)
        fval1 += disx_ndisy * (float) val;
     if ((val = abs_xyz_val(_3d,nx,ny,nz)) != 0)
        fval1 += disx_disy * (float) val;
     if ((val = abs_xyz_val(_3d,tx,ny,nz)) != 0)
        fval1 += ndisx_disy * (float) val;
     if (fval1 != 0) fval += fval1 * disz;
    }
 return fval;
}

get_buf_3d_rec_pixel_depth(src)
 struct buf_3d_rec *src;
{
 long pixel_depth,lcv;

 for (pixel_depth = 0,lcv = 0; lcv < src->num_slices; lcv++)
    if (src->sls[lcv].s1 != NULL)
       if (src->sls[lcv].s1->pixel_depth > pixel_depth)
          pixel_depth = src->sls[lcv].s1->pixel_depth;
 return pixel_depth;
}

resize_buf_3d_rec(dst,src,xpixel_size,ypixel_size,zpixel_size,id,is_err_reason)
 struct buf_3d_rec *dst,*src;
 float xpixel_size,ypixel_size,zpixel_size;
 char *id;
 int *is_err_reason;
{
 double fval;
 float temp,xratio,yratio,zratio,dx,dy,dz,sx,sy,sz,temp1,nsx,nsy,nsz;
 long pixel_depth,dimx,dimy,dimz,x,y,z,num,val,lcv,total,nrx,nry,nrz,rx,ry,rz;
 struct buf_2d_rec **_2ds,*s1;
 int err;
 struct timeb the_time;

 PRT(printf("resize_buf_3d_rec...dst = %u src = %u\n",dst,src))
 free_buf_3d_rec(dst);
 xratio = xpixel_size / src->xpixel_size;
 yratio = ypixel_size / src->ypixel_size;
 zratio = zpixel_size / src->zpixel_size;

 *is_err_reason = 0;
 temp1 = (float) floor((double) xratio);
 nrx = (long) temp1;
 if ((xratio - temp1) > ALMOST_0) nrx += 1;
 temp1 = (float) floor((double) yratio);
 nry = (long) temp1;
 if ((yratio - temp1) > ALMOST_0) nry += 1;
 temp1 = (float) floor((double) zratio);
 nrz = (long) temp1;
 if ((zratio - temp1) > ALMOST_0) nrz += 1;
 if (nrx == 0) *is_err_reason |= 1;
 if (nry == 0) *is_err_reason |= 2;
 if (nrz == 0) *is_err_reason |= 4;
 PRT(printf("resize_buf_3d_rec...*is_err_reason = %d nrx = %d nry = %d nrz = %d\n",
          *is_err_reason,nrx,nry,nrz))
 if (*is_err_reason) return CANCEL;

 temp = (float) src->dimx * src->xpixel_size / xpixel_size;
 dimx = (long) floor((double) temp);
 temp = (float) src->dimy * src->ypixel_size / ypixel_size;
 dimy = (long) floor((double) temp);
 temp = (float) src->dimz * src->zpixel_size /zpixel_size;
 dimz = (long) floor((double) temp);
 if (dimx == 0) *is_err_reason |= 8;
 if (dimy == 0) *is_err_reason |= 16;
 if (dimz == 0) *is_err_reason |= 32;
 PRT(printf("resize_buf_3d_rec...dimx = %d dimy = %d dimz = %d\n",
          dimx,dimy,dimz))
 if (*is_err_reason) return CANCEL;
 pixel_depth = get_buf_3d_rec_pixel_depth(src);
 PRT(printf("resize_buf_3d_rec...pixel_depth = %d\n",pixel_depth))

 if ((err = get_ptr(&(dst->sls),&(dst->total_slices),dimz,
           (size_t) sizeof(*(dst->sls)))) != NO_ERROR)
    return err;
 for (lcv = 0; lcv < dst->total_slices; lcv++)
    init_slice_3d_rec(&(dst->sls[lcv]));
 total = 0;
 if ((err = get_handle(&_2ds,&total,dimz,
           (size_t) sizeof(**_2ds))) != NO_ERROR)
    {free_buf_3d_rec(dst);
     return err;}
 for (lcv = 0; lcv < total; lcv++)
    init_buf_2d_rec(_2ds[lcv]);
 PRT(printf("resize_buf_3d_rec...xratio = %f y = %f z = %f\n",xratio,
             yratio,zratio))
 for (lcv = 0; lcv < total; lcv++)
    {if ((err = setup_buf_2d_rec(_2ds[lcv],dimx,dimy,pixel_depth,0)) != NO_ERROR)
        {free_buf_3d_rec(dst);
         for (lcv = 0; lcv < total; lcv++)
            {free_buf_2d_rec(_2ds[lcv]);
             c_free(_2ds[lcv]);}
         c_free(_2ds);
         return err;}
    }
 for (lcv = 0; lcv < total; lcv++)
    {dst->sls[lcv].s1 = _2ds[lcv];
     dst->sls[lcv].slice_thickness = zpixel_size;
     dst->sls[lcv].status |= SLICE_3D_ORIGINAL;}

 c_free(_2ds);
 dst->status = BUF_NON_EMPTY | BUF_OWN_CONTIGUOUS_MEMORY | _3D_VALID;
 dst->num_slices = dst->total_slices;
 dst->startx = src->startx;
 dst->starty = src->starty;
 dst->startz = src->startz;
 dst->xpixel_size = xpixel_size;
 dst->ypixel_size = ypixel_size;
 dst->slice_thickness = dst->zpixel_size = zpixel_size;
 dst->dimx = dimx;
 dst->dimy = dimy;
 dst->dimz = dimz;
 if ((err = cp_str(&(dst->id),id)) != NO_ERROR)
    {free_buf_3d_rec(dst);
     return err;}
 PRT(ftime(&the_time))
 PRT(printf("resize_buf_3d_rec sec = %d  msecs = %d\n",
         (int) the_time.time,the_time.millitm))
 for (sz = .0, nsz = zratio,z = 0; z < dimz; z++,sz += zratio,nsz += zratio)
    {s1 = dst->sls[z].s1;
     for (sy = .0, nsx = yratio,y = 0; y < dimy; y++,sy += yratio,nsy += yratio)
        {for (sx = .0,nsx = xratio,x = 0;x < dimx; x++,sx += xratio,nsx += xratio)
            {
             for (fval = .0, num = 0,dz = sz,rz = 0; ;)
                {
                 for (dy = sy,ry = 0; ;)
                    {
                     for (dx = sx,rx = 0; ;)
                        {
                         fval += resampled_val(src,dx,dy,dz);
                         num += 1;
                         rx += 1;
                         if (rx >= nrx) break;
                         dx += 1.0;
                         if (dx > nsx) dx = nsx;
                        }
                     ry += 1;
                     if (ry >= nry) break;
                     dy += 1.0;
                     if (dy > nsy) dy = nsy;
                    }
                 rz += 1;
                 if (rz >= nrz) break;
                 dz += 1.0;
                 if (dz > nsz) dz = nsz;
                }
             fval = fval / (float) num;
             s1->SetVal(s1,y,x,fval);
            }
        }
 PRT(ftime(&the_time))
 PRT(printf("z = %d sec = %d  msecs = %d  end\n\n",z,
         (int) the_time.time,the_time.millitm))
    }
 PRT(ftime(&the_time))
 PRT(printf("resize_buf_3d_rec sec = %d  msecs = %d  end\n\n",
         (int) the_time.time,the_time.millitm))
 return NO_ERROR;
}
unchop *************************/



