/****************************************************************************
build_hdr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: build_hdr.c,v 1.5 1999/07/07 18:01:05 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <sys/types.h>

#include "typedef.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "nria_lib.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)


unsigned long NumberOfSliceCuts();
unsigned long NumberOfSliceScans();


/*--------------------------------------------------------------------------
slice_size - ???
--------------------------------------------------------------------------*/

int
slice_size(hdr)
 struct hdr_rec *hdr;
{
 return GetImageSize(hdr->data_info.dimx,
      hdr->data_info.dimy,hdr->data_info.pixel_depth_bits);
}


/*--------------------------------------------------------------------------
hdr_image_labels - ???
--------------------------------------------------------------------------*/

int
hdr_image_labels(hdr,labels,member_img_label_indxs)
 struct hdr_rec *hdr;
 char **labels[];
 unsigned int *member_img_label_indxs;
{
 int err;
 unsigned long lcv;

 err = NO_ERROR;
 if ((hdr->analysis.img_labels == NULL) || (hdr->analysis.num_img_labels == 0))
    {hdr->analysis.num_img_labels = 0;
     hdr->analysis.img_labels = NULL;
     if ((err = get_ptr(&(hdr->analysis.img_labels),&(hdr->analysis.num_img_labels),
            (size_t) hdr->data_info.num_echoes,
            (size_t) sizeof(*(hdr->analysis.img_labels)))) != NO_ERROR)
        return err;
     for (lcv = 0; lcv < hdr->analysis.num_img_labels; lcv++)
        if ((err = cp_str(&(hdr->analysis.img_labels[lcv]),labels[lcv])) !=
                   NO_ERROR) return err;
    }
 if ((hdr->analysis.groups == NULL) ||
     (hdr->analysis.num_analysis_groups == 0))
    {hdr->analysis.num_analysis_groups = 0;
     hdr->analysis.groups = NULL;
     if ((err = get_ptr(&(hdr->analysis.groups),
                    &(hdr->analysis.num_analysis_groups),
            (size_t) 1,
            (size_t) sizeof(*(hdr->analysis.groups)))) != NO_ERROR)
        return err;
     hdr->analysis.groups[0].member_img_label_indx = NULL;
     hdr->analysis.groups[0].num_members = 0;
     if ((err = get_ptr(&(hdr->analysis.groups[0].member_img_label_indx),
            &(hdr->analysis.groups[0].num_members),
            (size_t) 1,
        (size_t)sizeof(*(hdr->analysis.groups[0].member_img_label_indx)))) != NO_ERROR)
        return err;

     for (lcv = 0; lcv < hdr->analysis.groups[0].num_members; lcv++)
        hdr->analysis.groups[0].member_img_label_indx[lcv] = 
                 member_img_label_indxs[lcv];
    }
 return err;
}


/*--------------------------------------------------------------------------
Build1EchoHdr - ???
--------------------------------------------------------------------------*/

int
Build1EchoHdr(hdr,skip_start_offset)
 struct hdr_rec *hdr;
 unsigned long skip_start_offset;
{
 int err;
 static char *labels[] = {"ECHO 1"};
 char **the_labels;
 static unsigned long member_img_label_indxs[] = {0};
 unsigned long lcv,*indxs,img_size,max_snum,img_addr,total_items,snum;
 struct hdr_scan_entry_rec *head,*last;

 TRACE(printf("Build1EchoHdr...num_img_labels = %d\n",
             hdr->analysis.num_img_labels))
 if ((err = cp_str(&(hdr->id),HDR_ID_V1)) != NO_ERROR)
    return err;
 the_labels = labels;
 indxs = member_img_label_indxs;
 if ((err = hdr_image_labels(hdr,the_labels,indxs)) != NO_ERROR)
    return err;

 hdr->num_scans = hdr->data_info.max_scans;

 total_items = 0;
 if ((err = get_ptr(&(hdr->scans),&total_items,(size_t) hdr->num_scans,
               (size_t) sizeof(struct hdr_scan_entry_rec))) != NO_ERROR)
    {hdr->num_scans = 0;
     return err;}
 img_size = GetImageSize(hdr->data_info.dimx,hdr->data_info.dimy,
                           hdr->data_info.pixel_depth_bits);
 max_snum = NumberOfSliceCuts(&(hdr->data_info));
 snum = 1;
 img_addr = skip_start_offset;
 for (lcv = 0; lcv < hdr->num_scans; lcv += 1,img_addr += img_size)
    {
     if ((err = cp_str(&(hdr->scans[lcv].img_label),
            hdr->analysis.img_labels[0])) != NO_ERROR) return err;
     hdr->scans[lcv].plane.vector.x = 1.0;
     hdr->scans[lcv].plane.vector.y = 1.0;
     hdr->scans[lcv].plane.vector.z = 1.0;
     hdr->scans[lcv].plane.constant = lcv + .5;
     hdr->scans[lcv].status = SLICE_ORIGINAL | SLICE_FIRST_ECHO;
     hdr->scans[lcv].img_addr = img_addr;
     hdr->scans[lcv].num = snum;
     hdr->scans[lcv].partner_slice = &(hdr->scans[lcv]);

     if (snum == 1)
        {head = &(hdr->scans[lcv]);
         last = NULL;
         hdr->scans[lcv].status |= SLICE_FIRST;}
     hdr->scans[lcv].prior_slice = last;
     if (last != NULL) last->next_slice = &(hdr->scans[lcv]);
     if (snum == max_snum)
        {head->prior_slice = &(hdr->scans[lcv]);
         hdr->scans[lcv].next_slice = head;
         snum = 1;}
       else
        {
         hdr->scans[lcv].next_slice = NULL;
         snum += 1;
         last = &(hdr->scans[lcv]);}
    }
 PRT(pr_scan_entry_info(hdr->scans,hdr->num_scans))
 build_default_datafn(hdr);
 TRACE(printf("Build1EchoHdr...end\n"))

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
Build2EchoHdr - ???
--------------------------------------------------------------------------*/

int
Build2EchoHdr(hdr,skip_start_offset)
 struct hdr_rec *hdr;
 unsigned long skip_start_offset;
{
 int err;
 static char *labels[] = {"ECHO 1","ECHO 2"};
 char **the_labels;
 static unsigned long member_img_label_indxs[] = {0};
 unsigned long lcv,*indxs,img_size,max_snum,img_addr,total_items,snum;
 struct hdr_scan_entry_rec *head,*last,*head1,*last1;

 TRACE(printf("Build2EchoHdr...\n"))
 if ((err = cp_str(&(hdr->id),HDR_ID_V1)) != NO_ERROR)
    return err;
 the_labels = labels;
 indxs = member_img_label_indxs;
 if ((err = hdr_image_labels(hdr,the_labels,indxs)) != NO_ERROR)
    return err;
 TRACE(printf("Build2EchoHdr...1\n"))

 hdr->num_scans = hdr->data_info.max_scans;
 total_items = 0;
 if ((err = get_ptr(&(hdr->scans),&total_items,(size_t) hdr->num_scans,
               (size_t) sizeof(struct hdr_scan_entry_rec))) != NO_ERROR)
    {hdr->num_scans = 0;
     return err;}
 TRACE(printf("max_scans = %d...pixel_depth_bits = %d\n",
          hdr->data_info.max_scans,hdr->data_info.pixel_depth_bits))
 TRACE(printf("...dimx = %d dimy = %d pixel_depth_bits = %d\n",
        hdr->data_info.dimx,hdr->data_info.dimy,hdr->data_info.pixel_depth_bits))
 img_size = GetImageSize(hdr->data_info.dimx,hdr->data_info.dimy,
                           hdr->data_info.pixel_depth_bits);
 TRACE(printf("Build2EchoHdr...img_size = %d\n",img_size))
 TRACE(printf("var_use_status = %x...num_echoes = %d\n",
         hdr->data_info.var_use_status,hdr->data_info.num_echoes))
 max_snum = NumberOfSliceCuts(&(hdr->data_info));
 snum = 1;
 TRACE(printf("num_scans = %d...max_snum = %d\n",hdr->num_scans,max_snum))
 img_addr = skip_start_offset;
 for (lcv = 0; lcv < hdr->num_scans; )
    {
     if ((err = cp_str(&(hdr->scans[lcv].img_label),
            hdr->analysis.img_labels[0])) != NO_ERROR) return err;
     if ((err = cp_str(&(hdr->scans[lcv+1].img_label),
            hdr->analysis.img_labels[1])) != NO_ERROR) return err;
     hdr->scans[lcv].plane.vector.x = 1.0;
     hdr->scans[lcv].plane.vector.y = 1.0;
     hdr->scans[lcv].plane.vector.z = 1.0;
     hdr->scans[lcv].plane.constant = lcv + .5;
     hdr->scans[lcv].status = SLICE_ORIGINAL | SLICE_FIRST_ECHO;
     hdr->scans[lcv+1].status = SLICE_ORIGINAL;
     hdr->scans[lcv+1].plane = hdr->scans[lcv].plane;

     hdr->scans[lcv].img_addr = img_addr;
     img_addr += img_size;
     hdr->scans[lcv+1].img_addr = img_addr;
     img_addr += img_size;
     TRACE(printf("%d) img_addr = %u\n",lcv,img_addr))
     hdr->scans[lcv+1].num = hdr->scans[lcv].num = snum;
     hdr->scans[lcv].partner_slice = &(hdr->scans[lcv+1]);
     hdr->scans[lcv+1].partner_slice = &(hdr->scans[lcv]);

     if (snum == 1)
        {head = &(hdr->scans[lcv]);
         last = NULL;
         hdr->scans[lcv].status |= SLICE_FIRST;}
     hdr->scans[lcv].prior_slice = last;
     if (last != NULL) last->next_slice = &(hdr->scans[lcv]);
     if (snum == max_snum)
        {head->prior_slice = &(hdr->scans[lcv]);
         hdr->scans[lcv].next_slice = head;
        }
       else
        {
         hdr->scans[lcv].next_slice = NULL;
         last = &(hdr->scans[lcv]);}

     lcv += 1;
     if (snum == 1)
        {head1 = &(hdr->scans[lcv]);
         last1 = NULL;
         hdr->scans[lcv].status |= SLICE_FIRST;}
     hdr->scans[lcv].prior_slice = last1;
     if (last1 != NULL) last1->next_slice = &(hdr->scans[lcv]);
     if (snum == max_snum)
        {head1->prior_slice = &(hdr->scans[lcv]);
         hdr->scans[lcv].next_slice = head1;
         snum = 1;}
       else
        {
         hdr->scans[lcv].next_slice = NULL;
         snum += 1;
         last1 = &(hdr->scans[lcv]);}
     lcv += 1;
    }
 TRACE(printf("Build2EchoHdr...4\n"))
 PRT(pr_scan_entry_info(hdr->scans,hdr->num_scans))
 build_default_datafn(hdr);
 TRACE(printf("Build2EchoHdr...end\n"))

 return NO_ERROR;

}


/*--------------------------------------------------------------------------
BuildRiaHdr - ???
--------------------------------------------------------------------------*/

int
BuildRiaHdr(hdr,skip_start_offset)
 struct hdr_rec *hdr;
 unsigned long skip_start_offset;
{
 int err;
 unsigned long num_cuts,num_scans;

 if (hdr->data_info.num_echoes == 1)
    {
     err = Build1EchoHdr(hdr,skip_start_offset);}
   else if (hdr->data_info.num_echoes == 2)
    {num_cuts = NumberOfSliceCuts(&(hdr->data_info));
     num_scans = NumberOfSliceScans(&(hdr->data_info));
     hdr->data_info.max_scans = num_cuts * hdr->data_info.num_echoes * num_scans;
     err = Build2EchoHdr(hdr,skip_start_offset);}
   else return CANCEL;
 if (err == NO_ERROR)
    ComputeScanEntryExtraInfo(hdr);
 return err;
}

