/****************************************************************************
file_io.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: file_io.c,v 1.7 1999/07/07 18:01:06 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <unistd.h>

#include "error_constants.h"
#include "typedef.h"
#include "nria_lib.h"


/*
cc -c file_io.c
cc -o file_io file_io.c str.o ptr_and_dim.o
cc -o file_io file_io.c str.o ptr_and_dim.o -lsuntool -lsunwindow -lpixrect -lm
*/


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)

/*
 *  file_set_size(fid,size)
 *  unsigned long get_file_size_by_fn(fn,err)
 *  unsigned long get_file_size_by_fileno(fnum,err)
 *  unsigned long file_size(fid,err)
 *  unsigned long the_file_size(dir,fn,err)
 *  int frd(buf,item_size,num_items,fid,amt_read)
 *  rd_file_labels(fid,strs,num_strs)
 *  rd_file_strs(fid,strs,num_strs)
 *  rd_file_ints(fid,ints,num_ints)
 *  int fwr(buf,item_size,num_items,fid,amt_written)
 *  wr_file_strs(fid,strs,num_strs)
 *  wr_file_ints(fid,ints,num_ints)
 *
 *  int file_access(fn,mode,error_display_proc,err_reason)
 *  char *UniqueFn(dir,fn,suffix)
 *  char *LastUniqueFn(dir,fn,suffix)
 *  delete_file(fn)
 *  delete_the_file(dir,fn,suffix)
 *  beep()
 *  reopen_file_as_read_update(fid,dirfn,fn,suffix)
 *
 *
 */



/*------------------------------------------------------------------------
 - ???

 if size < FILE_SIZE THEN SET SIZE
------------------------------------------------------------------------*/

int
file_set_size(fid,size)
 FILE *fid;
 unsigned long size;
{
 int cur_offset;

 if (((cur_offset = ftell(fid)) == EOF) ||
     (fseek(fid,size,0) == EOF) ||
     (fseek(fid,cur_offset,0) == EOF))

#ifdef SUN
    return -1;
#else
    return NOT_ENOUGH_DISK_MEMORY;
#endif

 return NO_ERROR;
}


/*------------------------------------------------------------------------
get_file_size_by_fn - ???
------------------------------------------------------------------------*/

unsigned long
get_file_size_by_fn(fn,err)
 char *fn;
 int *err;
{
 struct stat buf;

 *err = stat(fn,&buf);
 if (*err == 0) return (unsigned long) buf.st_size;
   else return (unsigned long) 0;
}


/*------------------------------------------------------------------------
get_file_size_by_fileno - ???
------------------------------------------------------------------------*/

unsigned long
get_file_size_by_fileno(fnum,err)
 int fnum;
 int *err;
{
 struct stat buf;

 *err = fstat( fnum, &buf );
 if (*err == 0)
   return (unsigned long) buf.st_size;
 else {
   fprintf( stderr, "fstat() error: filedes = %d\n", fnum );
   return (unsigned long) 0;
 }
}


/*------------------------------------------------------------------------
file_size - ???
------------------------------------------------------------------------*/

unsigned long
file_size( fid, err )
     FILE *fid;
     int *err;
{
  int fnum;
  
  fflush( fid );
  fnum = fileno( fid );
  return get_file_size_by_fileno( fnum, err );
}


/*------------------------------------------------------------------------
the_file_size - ???
------------------------------------------------------------------------*/

unsigned long
the_file_size(dir,fn,err)
 char *dir,*fn;
 int *err;
{
 char str[256];

 str[255] = '\0';
 strncpy(str,dir,255);
 strncat(str,fn,255);
 return get_file_size_by_fn(str,err);
}



/*------------------------------------------------------------------------
 - ???

  frd = no error
       READ_ERROR
       END_OF_FILE
------------------------------------------------------------------------*/

int 
frd(buf,item_size,num_items,fid,amt_read)
 unsigned char *buf;
 size_t item_size;
 size_t num_items;
 FILE *fid;
 size_t *amt_read;
{
 size_t len1,len,size;
 char enter;

 len1 = num_items * item_size;
 *amt_read = 0;
 if (len1 == 0) return NO_ERROR;
 for (enter = 0,len = len1; ;len -= size)
    {size = fread(&(buf[*amt_read]),(size_t) 1,len,fid);
     *amt_read += size;
     if ((size == 0) || (size == len)) break;
     enter = 1;}
 if (size == 0)
    {if (enter > 0) return END_OF_FILE;
     return READ_ERROR;
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------------
rd_file_labels - ???
------------------------------------------------------------------------*/

int
rd_file_labels(fid,strs,num_strs)
 FILE *fid;
 char ***strs;
 unsigned long num_strs;
{
 unsigned long total_items;
 int err;

 total_items = 0;
 get_handle_memory(strs,&total_items,(size_t) num_strs,
               (size_t) sizeof(char *), &err);
 if (err) return NOT_ENOUGH_MEMORY;
 return rd_file_strs(fid,*strs,num_strs);

}


/*------------------------------------------------------------------------
rd_file_strs - ???
------------------------------------------------------------------------*/

int
rd_file_strs(fid,strs,num_strs)
 FILE *fid;
 char **strs;
 unsigned long num_strs;
{
 unsigned long lcv,len,total_items;
 int err;

 TRACE(start =  ftell(fid))
 TRACE(printf("rd_file_strs...start = %ld...%o\n",start,start))
 init_ptr_ary(strs,num_strs,NULL);
 for (lcv =0; lcv < num_strs; lcv++)
    {
     if (fread(&len,sizeof(len),(size_t) 1,fid) != 1)
        {
         return READ_ERROR;}

     total_items = 0;
     get_ptr_memory(&(strs[lcv]),&total_items,(size_t) (len + 1),
               (size_t) 1, &err);
     if (err) return NOT_ENOUGH_MEMORY;
     strs[lcv][len] = '\0';
     if (len > 0)
        if (fread(strs[lcv],(size_t) len,(size_t) 1,fid) != 1)
           {
            return READ_ERROR;}
    }
 TRACE(printf("rd_file_strs...end\n"))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
rd_file_ints - ???
------------------------------------------------------------------------*/

int
rd_file_ints(fid,ints,num_ints)
 FILE *fid;
 unsigned long *ints;
 unsigned long num_ints;
{
 unsigned long lcv;

 for (lcv = 0; lcv < num_ints; lcv++)
    if (fread(&(ints[lcv]),sizeof(ints[lcv]),1,fid) != 1) return READ_ERROR;

 return NO_ERROR;
}



/*------------------------------------------------------------------------
fwr - ???

fwr = NO_ERROR ...........no error
      WRITE_ERROR ...........error
------------------------------------------------------------------------*/

int 
fwr(buf,item_size,num_items,fid,amt_written)
 unsigned char *buf;
 size_t item_size;
 size_t num_items;
 FILE *fid;
 size_t *amt_written;
{
 size_t len1,len,size;

 len1 = num_items * item_size;
 *amt_written = 0;
 if (len1 == 0) return NO_ERROR;
 for (len = len1; ;len -= size)
    {size = fwrite(&(buf[*amt_written]),(size_t) 1,len,fid);
     *amt_written += size;
     if ((size == 0) || (size == len)) break;
    }
 if (size == 0) return WRITE_ERROR;
 return NO_ERROR;
}


/*------------------------------------------------------------------------
wr_file_strs - ???
------------------------------------------------------------------------*/

int
wr_file_strs(fid,strs,num_strs)
 FILE *fid;
 char **strs;
 unsigned long num_strs;
{
 unsigned long lcv,len;

 for (lcv =0; lcv < num_strs; lcv++)
    {len = get_strlen(strs[lcv]);
     if (fwrite(&len,(size_t) sizeof(len),(size_t) 1,fid) != 1)
        return WRITE_ERROR;
       else if (len > 0)
        if (fwrite(strs[lcv],(size_t) len,(size_t) 1,fid) != 1)
           return WRITE_ERROR;
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------------
wr_file_ints - ???
------------------------------------------------------------------------*/

int
wr_file_ints(fid,ints,num_ints)
 FILE *fid;
 unsigned long *ints;
 unsigned long num_ints;
{
 unsigned long lcv;

 for (lcv = 0; lcv < num_ints; lcv++)
    if (fwrite(&(ints[lcv]),sizeof(ints[lcv]),1,fid) != 1) return WRITE_ERROR;

 return NO_ERROR;
}



/*------------------------------------------------------------------------
file_access_err_proc - ???
------------------------------------------------------------------------*/

void
file_access_err_proc(reason,error_display_proc,err_str,is_dynamic_str)
 char **reason,*err_str;
 void (*error_display_proc)();
 int is_dynamic_str;
{
 if (error_display_proc != NULL) 
    error_display_proc(err_str);
 if (reason == NULL)
    {if (is_dynamic_str) c_free(err_str);}
   else
    {if (is_dynamic_str) *reason = err_str;
       else cp_str(reason,err_str);
    }
}


/*------------------------------------------------------------------------
file_access - ???
------------------------------------------------------------------------*/

/*
#define WORLD_EXECUTE_MODE 1
#define WORLD_WRITE_MODE 2
#define WORLD_READ_MODE 4
#define WORLD_FILE_EXIST 8
*/

int 
file_access(fn,mode,error_display_proc,reason)
 char *fn;
 int mode;
 void (*error_display_proc)();
 char **reason;
{
 int err;
 char *str,*temp,*temp1;
 unsigned char execute_err,read_err,write_err;
 FILE *fid=0;
 int is_close;

 if (reason != NULL) *reason = NULL;
 is_close = 0;
 if ((err = access(fn,F_OK)) != 0)
    {if ((mode & 8) || !(mode & 7))
        {
         if (concat_strs(&str,2,fn," doesn't exist...\n") == NO_ERROR)
            file_access_err_proc(reason,error_display_proc,str,1);
         return FILE_ACCESS_ERROR;
        }
     if ((mode & 1) || (mode & 4))
        {if (mode & 2)
            {if ((fid = fopen(fn,"r+")) == NULL)
                {if ((err = concat_strs(&str,2,
                      "READ/WRITE access not allowed on ",fn)) == NO_ERROR)
                    file_access_err_proc(reason,error_display_proc,str,1);
                 return FILE_ACCESS_ERROR;
                }
            }
           else
            {if ((fid = fopen(fn,"r")) == NULL)
                {if (mode & 1)
                    {if ((err = concat_strs(&str,
                         2,"EXECUTE access not allowed on ",fn)) == NO_ERROR)
                        file_access_err_proc(reason,error_display_proc,str,1);
                    }
                   else if ((err = concat_strs(&str,2,
                             "READ access not allowed on ",fn)) == NO_ERROR)
                    file_access_err_proc(reason,error_display_proc,str,1);
                 return FILE_ACCESS_ERROR;
                }
            }
         }
        else if (mode & 2)
         {if ((fid = fopen(fn,"w")) == NULL)
             {if ((err = concat_strs(&str,2,"WRITE access not allowed on ",
                                fn)) == NO_ERROR)
                 file_access_err_proc(reason,error_display_proc,str,1);
              return FILE_ACCESS_ERROR;
             }
         }
     is_close = 1;
    }
 TRACE(printf("mode = %d ",mode))
 if ((mode & 1) && ((err = access(fn,X_OK)) != 0))
    execute_err = 1;
   else execute_err = 0;
 if ((mode & 2) && ((err = access(fn,W_OK)) != 0))
    write_err = 1;
   else write_err = 0;
 if ((mode & 4) && ((err = access(fn,R_OK)) != 0))
    read_err = 1;
   else read_err = 0;
 TRACE(printf(" is_close = %d xerr = %d w = %d r = %d\n",is_close,
      execute_err,write_err,read_err))
 if (is_close)
    {fclose(fid);
     unlink(fn);}
 TRACE(printf("file access...1\n");fflush(stdout))
 if (execute_err || write_err || read_err)
    {temp = NULL;
     if (execute_err) cp_str(&temp,"EXECUTE");
     if (write_err)
        {if (temp != NULL)
            {temp1 = temp;
             concat_strs(&temp,3,temp1,"/","WRITE");
             c_free(temp1);
            }
           else cp_str(&temp,"WRITE");
        }
  TRACE(printf("file access...2\n");fflush(stdout))
    if (read_err)
        {if (temp != NULL)
            {temp1 = temp;
             concat_strs(&temp,3,temp1,"/","READ");
             c_free(temp1);
            }
           else cp_str(&temp,"READ");
        }
     TRACE(printf("file access...3\n");fflush(stdout))

     concat_strs(&str,4,temp," access on ",fn," not allowed !!!");
     if (error_display_proc != NULL) error_display_proc(str);
     c_free(temp);
     if (reason == NULL) c_free(str);
       else *reason = str;
     TRACE(printf("file access...4\n");fflush(stdout))
     return FILE_ACCESS_ERROR;
    }
 return NO_ERROR;
}


/*------------------------------------------------------------------------
UniqueFn - ???
------------------------------------------------------------------------*/

char *
UniqueFn(dir,fn,suffix)
 char *dir,*fn,*suffix;
{
 char *str2,*str,str1[10];
 int err,cnt;

 for (cnt = 1; cnt < 10000; cnt++)
    {sprintf(str1,"%-d",cnt);
     if ((err = concat_strs(&str2,2,fn,str1)) != NO_ERROR)
        return NULL;
     err = build_fn(&str,dir,str2,suffix);
     c_free(str2);
     if (err != NO_ERROR) return NULL;
     if ((err = access(str,F_OK)) != 0)
        {PRT(printf("UniqueFn = %s\n",str))
         return str;}
     c_free(str);
    }
 return NULL;
}


/*------------------------------------------------------------------------
LastUniqueFn - ???
------------------------------------------------------------------------*/

char *
LastUniqueFn(dir,fn,suffix)
 char *dir,*fn,*suffix;
{
 long cnt;
 char *str2,*str,*last,str1[10];
 int err;

 last = NULL;
 for (cnt = 1; cnt < 10000; cnt++)
    {sprintf(str1,"%-ld",cnt);
     if ((err = concat_strs(&str2,2,fn,str1)) != NO_ERROR)
        break;
     err = build_fn(&str,dir,str2,suffix);
     c_free(str2);
     if (err != NO_ERROR) break;
     if ((err = access(str,F_OK)) != 0)
        {PRT(printf("LastUniqueFn = %s\n",str))
         if (cnt == 1) return str;
         c_free(str);
         return last;
        }
     c_free(last);
     last = str;
    }
 c_free(last);
 if (str != last) c_free(str);
 return NULL;
}


/*------------------------------------------------------------------------
delete_file - ???
------------------------------------------------------------------------*/

int
delete_file(fn)
 char *fn;
{
#ifdef __STDC__
 return remove(fn);
#elif defined(SUN)
 return unlink(fn);
#else
 return delete(fn);
#endif
}


/*------------------------------------------------------------------------
delete_the_file - ???
------------------------------------------------------------------------*/

int
delete_the_file(dir,fn,suffix)
 char *dir,*fn,*suffix;
{
 char *str;
 int err;

 if ((err = build_fn(&str,dir,fn,suffix)) == NO_ERROR)
    {if (access(str,F_OK) == 0) unlink(str);
     c_free(str);}
 return err;
}


/*------------------------------------------------------------------------
beep - ???
------------------------------------------------------------------------*/

void
beep()
{
 printf("%c",(char) 7);
 fflush(stdout);
}


/*------------------------------------------------------------------------
reopen_file_as_read_update - ???
------------------------------------------------------------------------*/

int
reopen_file_as_read_update(fid,dirfn,fn,suffix,err_reason)
 FILE **fid;
 char *dirfn,*fn,*suffix,**err_reason;
{
 char *the_fn;
 int err;

 if (err_reason != NULL) *err_reason = NULL;

 if ((err = build_fn(&the_fn,dirfn,fn,suffix)) == NO_ERROR)
    {
     if (file_access(the_fn,6,NULL,err_reason) == NO_ERROR)
        {fclose(*fid);
         if ((*fid = fopen(the_fn,"r+")) == NULL)
            {*fid = fopen(the_fn,"r");
             err = FILE_ACCESS_ERROR;
             cp_str(err_reason,"FILE ACCESS ERROR...");
            }
        }
     c_free(the_fn);
    }
   else if (err_reason != NULL)
    cp_str(err_reason,"NOT ENOUGH MEMORY...");
 return err;
}

/*******************
file_access_error_reason(fn,mode,reason)
 char *fn,**reason;
{
 int err;

 if (fn != NULL)
    {if ((err = access(fn,F_OK)) != 0)
        err = concat_strs(reason,2,fn,"...not exist !!!");
       else if ((err = access(fn,X_OK)) != 0)
        err = concat_strs(reason,2,fn,"...not exist !!!");
     
    }
   else err = cp_str(reason,"...NO FILE NAME !!!");
 return err;
}
************************/


/***********************
unix_flush(fid)
 int fid;
{
 FILE *id;

 if ((id = fdopen(fid,"a+"

}
*************************/
