/****************************************************************************
wr_hdr_text.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: wr_hdr_text.c,v 1.6 1999/07/07 18:01:07 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <sys/types.h>

#include "typedef.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "common_constants.h"
#include "mri_hdr_constants.h"
#include "nria_lib.h"



/*--------------------------------------------------------------------------
WrTextField - ???
--------------------------------------------------------------------------*/

void
WrTextField(fid,field,field_value)
 FILE *fid;
 struct hdr_field_rec *field;
 char *field_value;
{

 if (get_strlen(field_value) == 0)
    fprintf(fid,"%ld \"\" # %s\n",field->id,field->label);
   else
    {trim_str_blanks(field_value);
     if (is_blank_in_str(field_value))
        fprintf(fid,"%ld \"%s\" # %s\n",field->id,field_value,field->label);
       else fprintf(fid,"%ld %s # %s\n",field->id,field_value,field->label);
    }
}


/*--------------------------------------------------------------------------
WrCharField - ???
--------------------------------------------------------------------------*/

void
WrCharField(fid,field,field_value)
 FILE *fid;
 struct hdr_field_rec *field;
 char field_value;
{
 if (field_value == ' ')
    fprintf(fid,"%ld \"\" # %s\n",field->id,field->label);
   else fprintf(fid,"%ld %c # %s\n",field->id,field_value,field->label);
}


/*--------------------------------------------------------------------------
WrIntField - ???
--------------------------------------------------------------------------*/

void
WrIntField(fid,field,field_value,is_not_commented)
 FILE *fid;
 struct hdr_field_rec *field;
 int field_value,is_not_commented;
{
 if (is_not_commented)
    fprintf(fid,"%ld %d # %s\n",field->id,field_value,field->label);
   else fprintf(fid,"# %ld %d # %s\n",field->id,field_value,field->label);
}


/*--------------------------------------------------------------------------
WrHexIntParam - ???
--------------------------------------------------------------------------*/

void
WrHexIntParam(fid,field,field_value,is_not_commented)
 FILE *fid;
 struct hdr_field_rec *field;
 int field_value,is_not_commented;
{
 if (is_not_commented)
    fprintf(fid,"%ld %lx # %s\n",field->id,(unsigned long) field_value,
             field->label);
   else fprintf(fid,"# %ld %lx # %s\n",field->id,(unsigned long) field_value,
                field->label);
}


/*--------------------------------------------------------------------------
WrShortParam - ???
--------------------------------------------------------------------------*/

void
WrShortParam(fid,field,field_value,is_not_commented)
 FILE *fid;
 struct hdr_field_rec *field;
 short field_value;
 int is_not_commented;
{
 if (is_not_commented)
    fprintf(fid,"%ld %d # %s\n",field->id,(int) field_value,field->label);
   else fprintf(fid,"# %ld %d # %s\n",field->id,(int) field_value,field->label);
}


/*--------------------------------------------------------------------------
WrFloatField - ???
--------------------------------------------------------------------------*/

void
WrFloatField(fid,field,field_value,is_not_commented)
 FILE *fid;
 struct hdr_field_rec *field;
 float field_value;
 int is_not_commented;
{
 if (is_not_commented)
    fprintf(fid,"%ld %.4f # %s\n",field->id,field_value,field->label);
   else fprintf(fid,"# %ld %.4f # %s\n",field->id,field_value,field->label);
}


/*--------------------------------------------------------------------------
WrDoubleField - ???
--------------------------------------------------------------------------*/

void
WrDoubleField(fid,field,field_value,is_not_commented)
 FILE *fid;
 struct hdr_field_rec *field;
 double field_value;
 int is_not_commented;
{
 if (is_not_commented)
    fprintf(fid,"%ld %.4f # %s\n",field->id,field_value,field->label);
   else fprintf(fid,"# %ld %.4f # %s\n",field->id,field_value,field->label);
}


/*--------------------------------------------------------------------------
WrTransformationField - ???
--------------------------------------------------------------------------*/

void
WrTransformationField(fid,field,field_value,is_not_commented)
 FILE *fid;
 struct hdr_field_rec *field;
 float field_value[4][4];
 int is_not_commented;
{
 int row,col;

 if (is_not_commented) fprintf(fid,"%ld",field->id);
   else fprintf(fid,"# %ld",field->id);
 for (row = 0; row < 4; row++)
    for (col = 0; col < 4; col++)
       fprintf(fid,"  %.4f",field_value[row][col]);
 fprintf(fid," # %s\n",field->label);

}


/*--------------------------------------------------------------------------
WrNumKndOfImagesField - ???
--------------------------------------------------------------------------*/

void
WrNumKndOfImagesField(fid,field,hdr)
 FILE *fid;
 struct hdr_field_rec *field;
 struct hdr_rec *hdr;
{
 int lcv;

 fprintf(fid,"# %s\n",field->label);
 fprintf(fid,"%ld %ld\n",field->id,hdr->analysis.num_img_labels);
 for (lcv = 0; lcv < hdr->analysis.num_img_labels; lcv++)
    {trim_str_blanks(hdr->analysis.img_labels[lcv]);
     if (is_blank_in_str(hdr->analysis.img_labels[lcv]))
        fprintf(fid,"\"%s\"   ",hdr->analysis.img_labels[lcv]);
       else fprintf(fid,"%s   ",hdr->analysis.img_labels[lcv]);
    }
 fprintf(fid,"\n");
}


/*--------------------------------------------------------------------------
WrTextHdrSpec - ???

FIXME: This function takes fields as an argument but then assumes
exactly the order and contents defined in mri_header fields.
--------------------------------------------------------------------------*/

int
WrTextHdrSpec(fid,in_outfn,hdr,fields)
 FILE *fid;
 char *in_outfn;
 struct hdr_rec *hdr;
 struct hdr_field_rec fields[TOTAL_FIELDS];
{
 int err,is_not_commented;
 static int first_time = 1;
 struct data_info_rec *info;

 clearerr(fid);
 if (first_time)
    {fprintf(fid,"# Assumptions...\n");
     fprintf(fid,"# 1) All measurements of distance are in (mm) unit.\n");
     fprintf(fid,"# 2) All measurements of weight are in (gram) unit.\n");
     fprintf(fid,"# 3) All measurements of angle are in (degree) unit.\n");
     fprintf(fid,"# 4) All input fields are preceded by an identifier.\n");
     fprintf(fid,"# 5) The input of character sequences which consists of \n");
     fprintf(fid,"#    blanks must be enclosed in double quotes.\n");
     fprintf(fid,"\n\n");
     first_time = 0;
    }
 info = &(hdr->data_info);

#if 0   /*****************************/
 WrTextField(fid,&(fields[INPUT_OUTPUT_FN]),in_outfn);

 /*  DRW 960325: This is confidential information.  Do not place in a text
  *    file.  */
 WrTextField(fid,&(fields[NAME]),info->patient.name);
 WrTextField(fid,&(fields[SSNUM]),info->patient.ssnum);
 WrCharField(fid,&(fields[SEX]),info->patient.sex);
 WrIntField(fid,&(fields[AGE]),info->patient.age,1);
 WrIntField(fid,&(fields[HEIGHT]),info->patient.height,1);
 WrIntField(fid,&(fields[WEIGHT]),info->patient.weight,1);
#endif   /**************************/

 WrTextField(fid,&(fields[NAME]),  "X");
 WrTextField(fid,&(fields[SSNUM]), "X");
 WrCharField(fid,&(fields[SEX]),   'X');  /* Note that this is a char, not
					   *   a string.  */
 WrIntField(fid,&(fields[AGE]),    0, 1);
 WrIntField(fid,&(fields[HEIGHT]), 0, 1);
 WrIntField(fid,&(fields[WEIGHT]), 0, 1);

 WrTextField(fid,&(fields[DEFAULT_DATA_FN]),info->default_datafn);
 WrIntField(fid,&(fields[DIMX]),info->dimx,1);
 WrIntField(fid,&(fields[DIMY]),info->dimy,1);
 WrIntField(fid,&(fields[PIXEL_DEPTH_BITS]),info->pixel_depth_bits,1);
 WrIntField(fid,&(fields[MAX_SLICES]),info->max_scans,1);
 is_not_commented = info->var_use_status & NUM_SCANS_USED;
 WrIntField(fid,&(fields[NUM_SCANS]),info->num_scans,is_not_commented);
 WrFloatField(fid,&(fields[MIN_PIXEL]),info->min_pixel,1);
 WrFloatField(fid,&(fields[MAX_PIXEL]),info->max_pixel,1);

 WrFloatField(fid,&(fields[XPIXEL_SIZE]),info->xpixel_size,1);
 WrFloatField(fid,&(fields[YPIXEL_SIZE]),info->ypixel_size,1);
 WrFloatField(fid,&(fields[PIXEL_THICKNESS]),info->pixel_thickness,1);

 is_not_commented = info->var_use_status & TRANS_MATRIX_USED;
 WrTransformationField(fid,&(fields[TRANSFORMATION_MATRIX_4_BY_4]),
              info->trans_matrix,is_not_commented);

 WrNumKndOfImagesField(fid,&(fields[NUM_KIND_OF_IMAGES]),hdr);
 WrTextField(fid,&(fields[IMAGE_TYPE]),info->image_id);
 WrShortParam(fid,&(fields[PLANE_TYPE]),info->plane_type,1);
 WrTextField(fid,&(fields[COMMENT]),hdr->comment);
 WrTextField(fid,&(fields[DATE_OF_STUDY]),info->date_of_study);
 WrTextField(fid,&(fields[TIME_OF_STUDY]),info->time_of_study);
 WrTextField(fid,&(fields[PATIENT_ID]),info->patient.patient_id);
 WrTextField(fid,&(fields[STUDY_NUM]),info->patient.study_num);
 WrTextField(fid,&(fields[PATIENT_NUM]),info->patient.patient_num);
 WrTextField(fid,&(fields[REFERED_BY]),info->refered_by);
 WrTextField(fid,&(fields[DESCRIPTION]),info->description);
 WrTextField(fid,&(fields[HOSP_NAME]),info->hosp_name);
 WrFloatField(fid,&(fields[SPACING]),info->spacing,1);
 WrFloatField(fid,&(fields[TR]),info->tr,1);
 WrTextField(fid,&(fields[TE]),info->te);

 WrHexIntParam(fid,&(fields[VAR_USE_STATUS]),info->var_use_status,0);

 is_not_commented = info->var_use_status & UNUSED1_USED;
 WrFloatField(fid,&(fields[UNUSED1]),info->unused1,is_not_commented);
 is_not_commented = info->var_use_status & UNUSED2_USED;
 WrShortParam(fid,&(fields[UNUSED2]),info->unused2,is_not_commented);
 is_not_commented = info->var_use_status & UNUSED3_USED;
 WrShortParam(fid,&(fields[UNUSED3]),info->unused3,is_not_commented); 
 is_not_commented = info->var_use_status & UNUSED4_USED;
 WrShortParam(fid,&(fields[UNUSED4]),info->unused4,is_not_commented);
 is_not_commented = info->var_use_status & UNUSED5_USED;
 WrShortParam(fid,&(fields[UNUSED5]),info->unused5,is_not_commented);

 is_not_commented = info->var_use_status & UNUSED6_USED;
 WrFloatField(fid,&(fields[UNUSED6]),info->unused6,is_not_commented);
 is_not_commented = info->var_use_status & UNUSED7_USED;
 WrFloatField(fid,&(fields[UNUSED7]),info->unused7,is_not_commented);

 WrTextField(fid,&(fields[END_OF_INPUT]),"");

 if ((err = ferror(fid)) != NO_ERROR) return err;
 return NO_ERROR;
}
