#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include "mri_format.h"
#include "segment.h"
#include "debug.h"

int debugFlag = 0;

/*
  csh lntxt_batch.com
  cc -c ntxt_batch.c
  ntxt_batch "*.nhdr" . "" fmtfns
*/

/*
  RdCurSlice(fid,cur_slice,_2d)
  AnalyzePixelTypes(anal,slice_roi,pixel_type,format,slice_map)
  IsAllPixelTypeMapReady_1(pixel_type,hdr)
  DefinePartitionedPixelMap(slice_map,map,_2d1,_2d2)
  setup_pixel_type_maps_1(pixel_type,num_slice_maps)
  SetupPartitionedSliceMaps(pixel_type,hdr,hdrfn)
  setup_analysis(anal,hdr,pixel_type)
  process_ntxt(fn,ndatfn,fmtfns,prog_name)
  prog_usage(argc,argv)
  get_params(argc,argv,dirfn,fn_patt,ndatdir,prefix,fmtfns);
  main(argc,argv)
*/

extern char *locate_dirfn();
extern int exit_pixel_type_cnt_rec();
extern struct roi_rec *get_roi();
extern unsigned long NumberOfSliceCuts();


int
RdCurSlice(fid,cur_slice,_2d)
     FILE *fid;
     struct hdr_scan_entry_rec *cur_slice;
     struct buf_2d_rec *_2d;
{
  TRACE(printf("rd_cur_slice_data...\n"))
    _2d->status4 &= ~BUF_DATA_VALID;
  if (fseek(fid,(size_t) cur_slice->img_addr,0) != 0)
  {
    return ILLEGAL_FILE_ADDRESS;
  }
  if (fread(_2d->buf[0],(size_t) _2d->total_bytes,(size_t) 1,fid) != 1)
  {
    return READ_ERROR;
  }

  _2d->status4 |= BUF_DATA_VALID;
  return NO_ERROR;
}


int
AnalyzePixelTypes(anal,slice_roi,pixel_type,format,slice_map)
     struct analysis_rec *anal;
     struct slice_roi_rec *slice_roi;
     struct pixel_type_rec *pixel_type;
     struct output_text_format_rec *format;
     struct buf_2d_rec *slice_map;
{
  struct int_bound_rec affected;
  unsigned_int cnt,indx,roi_indx,lcv;
  struct pixel_type_cnt_rec *ptr;
  char *str;
  struct roi_rec *roi;
 
  TRACE(printf("AnalyzePixelTypes...begin\n"););

  if (slice_roi->num_rois == 0)
    return NO_ERROR;
  init_common_pixel_rec_matrix(&(anal->common),0);

  anal->_2d_ptr = slice_map;
  for (cnt = 0; cnt < format->roi_labels.num_strs; cnt++)
  {
    roi_indx = format->roi_nums[cnt];
    if (roi_indx == 0)
      continue;
    if ((roi = get_roi(slice_roi,format->roi_labels.strs[cnt])) == NULL)
      continue;
    TRACE(printf("%d) label = %s roi_indx = %d\n",
		 cnt,roi->label,roi_indx));

    if (roi->user.ptr == NULL)
    {
      roi->user.exit = exit_pixel_type_cnt_rec;
      if ((anal->err = setup_pixel_type_cnt_rec(&(roi->user.ptr),
						pixel_type)) != NO_ERROR)
	break;
    }
    ptr = (struct pixel_type_cnt_rec *) roi->user.ptr;
    TRACE(printf("...anal->_2d_ptr = %u num_pixel_type_cnts = %d\n",
		 anal->_2d_ptr,
		 ptr->num_pixel_type_cnts));
    if (((anal->err =
	  GetFilledRoiMask( roi, NULL,
			    &(anal->mask),&affected)) != NO_ERROR)
	|| ((anal->err =
	     cnt_pixel_types_by_mask( anal->_2d_ptr,
				      &(anal->mask.mask._2d),
				      &affected,
				      ptr->pixel_type_cnts,
				      ptr->num_pixel_type_cnts)) != NO_ERROR)
	|| ((anal->err =
	     add_global_pixel_rec_matrix_by_mask( &(anal->common),
						  &(anal->mask.mask._2d),
						  &affected,
						  anal->_2d_ptr,roi_indx))
	    != NO_ERROR))
      break;

    ptr->status |= PIXEL_TYPE_CNTS_COMPUTED;
    TRACE(printf("next...err = %d\n",anal->err));
  }
  TRACE(printf("set_common_pixel_rec_matrix...err = %d\n",anal->err));

  if (anal->err == NO_ERROR)
  {
    anal->err = correct_slice_counts(anal,format,slice_roi);
    TRACE(printf("after correct_slice_counts...err = %d\n",anal->err));
  }
  if (anal->err != NO_ERROR)
    unset_slice_counts(slice_roi,format,cnt);
  TRACE(printf("AnalyzePixelTypes*****err = %d\n",anal->err));

  return anal->err;
}


int
IsAllPixelTypeMapReady_1(pixel_type,hdr)
     struct pixel_type_rec *pixel_type;
     struct hdr_rec *hdr;
{
  long lcv;
  struct pixmap_hdr_rec *pixhdr;
  long num_slices;
  int yes;


  TRACE(printf("IsAllPixelTypeMapReady_1 num_slice_maps = %d\n",
	       pixel_type->num_slice_maps););

  if (pixel_type->order.num_strs == 0)
    return 0;

  num_slices = NumberOfSliceCuts(&(hdr->data_info));
  TRACE(printf("IsAllPixelTypeMapReady_1 num_slices = %d\n", num_slices););

  if ((hdr->data_info.num_echoes == 2) &&
      !(pixel_type->status & PIXEL_TYPE_GROW_REGION))
  {
    TRACE(printf("IsAllPixelTypeMapReady_1 partitioned\n"););
    if (pixel_type->map.status4 & BUF_DATA_VALID)
      return 2;
  }

  else if (pixel_type->num_slice_maps == num_slices)
  {
    pixhdr = &(pixel_type->pixmap_hdr);
    TRACE(if (pixel_type->status & PIXEL_TYPE_PIXMAP_HDR_VALID)
	  printf("PIXEL_TYPE_PIXMAP_HDR_VALID...\n"););
    TRACE(printf("IsAllPixelTypeMapReady_1 pixmap\n"););
    for (yes = 0,lcv = 0; lcv < pixel_type->num_slice_maps; lcv++)
    {
      if (!(pixel_type->slice_maps[lcv]->status4 & BUF_DATA_VALID)) 
      {
	TRACE(printf("lcv = %d\n",lcv););
	if ((pixel_type->status & PIXEL_TYPE_PIXMAP_HDR_VALID) &&
	    (lcv < pixhdr->max_slices) &&
	    (pixhdr->pixmap_offsets[lcv] != 0))
	{
	  yes = 1;
	  continue;
	}
	break;
      }
      else
	yes = 1;
    }

    if (lcv == pixel_type->num_slice_maps)
      return 2;
    TRACE(printf("IsAllPixelTypeMapReady_1 lcv = %d\n",lcv););
    if (yes)
      return 1;
    for (lcv++; lcv < pixel_type->num_slice_maps; lcv++)
      if (!(pixel_type->slice_maps[lcv]->status4 & BUF_DATA_VALID)) 
      {
	if ((pixel_type->status & PIXEL_TYPE_PIXMAP_HDR_VALID) &&
	    (lcv < pixhdr->max_slices) &&
	    (pixhdr->pixmap_offsets[lcv] != 0))
	  return 1;
      }
      else
	return 1;
  }
  return 0;
}


int
DefinePartitionedPixelMap(slice_map,map,_2d1,_2d2)
     struct buf_2d_rec *slice_map,*map,*_2d1,*_2d2;
{
  int err;
  
  unsigned long row,col,indx,val1,val2;

  for (row = 0; row < map->dimy; row++)
  {
    for (col = 0; col < map->dimx; col++)
    {
      val1 = get_1d_buf_value(_2d1->buf[row],col,_2d1->pixel_depth);
      val2 = get_1d_buf_value(_2d2->buf[row],col,_2d2->pixel_depth);
      indx = get_1d_buf_value(map->buf[val2],val1,map->pixel_depth);
      set_1d_buf_value(slice_map->buf[row],col,slice_map->pixel_depth,indx);
    }
  }
}


int
setup_pixel_type_maps_1(pixel_type,num_slice_maps)
     struct pixel_type_rec *pixel_type; 
     unsigned long num_slice_maps;
{
  int err;
  long lcv;

  if (pixel_type->num_slice_maps == 0)
  {
    if ((err = get_handle(&(pixel_type->slice_maps),
			  &(pixel_type->num_slice_maps),
			  (unsigned long) num_slice_maps,
			  (size_t) sizeof(**(pixel_type->slice_maps))))
	!= NO_ERROR)
      return err;
    for (lcv = 0; lcv < pixel_type->num_slice_maps; lcv++)
      init_buf_2d_rec(pixel_type->slice_maps[lcv]);
  }
  return NO_ERROR;
}


int
SetupPartitionedSliceMaps(pixel_type,hdr,hdrfn)
     struct pixel_type_rec *pixel_type;
     struct hdr_rec *hdr;
     char *hdrfn;
{
  FILE *fid;
  char *reason,*imgfn,*temp;
  struct buf_2d_rec _2d1,_2d2;
  long lcv;
  long num_slices;
  int err;

  num_slices = NumberOfSliceCuts(&(hdr->data_info));
  if ((err = setup_pixel_type_maps_1(pixel_type,num_slices)) != NO_ERROR)
    return err;
  temp = imgfn = NULL;
  if (((err = cp_str(&temp,hdrfn)) != NO_ERROR) ||
      ((err = drop_suffix(temp)) != NO_ERROR) ||
      ((err = concat_strs(&imgfn,2,temp,".img")) != NO_ERROR))
  {
    printf("Error constructing imgfn = %s...err = %d\n",hdrfn,err);
    c_free(temp);
    c_free(imgfn);
    return err;
  }
  c_free(temp);
  if ((fid = fopen(imgfn,"r")) == NULL)
  {
    file_access(imgfn,4,NULL,&reason);
    printf("Error opening the img file = %s...reason = %s\n",imgfn,reason);
    c_free(reason);
    c_free(imgfn);
    return FILE_ACCESS_ERROR;
  }
  TRACE(printf("SetupPartitionedSliceMaps...imgfn = %s\n",imgfn));

  init_buf_2d_rec(&_2d1);
  init_buf_2d_rec(&_2d2);

  if (((err = SetupBuf2dRec(&_2d1,hdr->data_info.dimx,hdr->data_info.dimy,
			    hdr->data_info.pixel_depth_bits,0)) == NO_ERROR) &&
      ((err = SetupBuf2dRec(&_2d2,hdr->data_info.dimx,hdr->data_info.dimy,
			    hdr->data_info.pixel_depth_bits,0)) == NO_ERROR))
  {

    for (lcv = 0; lcv < num_slices; lcv++)
    {

      if ((err = SetupBuf2dRec(pixel_type->slice_maps[lcv],
			       hdr->data_info.dimx,hdr->data_info.dimy,
			       pixel_type->map.pixel_depth,0)) != NO_ERROR)
	break;

      if ((fread(_2d1.buf[0],(size_t) _2d1.total_bytes,(size_t) 1,fid) != 1) ||
	  (fread(_2d2.buf[0],(size_t) _2d2.total_bytes,(size_t) 1,fid) != 1))
      {
	err = READ_ERROR;
	printf("Error reading Slice#%d from %s\n",lcv+1,imgfn);
	break;
      }

      _2d1.minval_set = _2d2.minval_set = hdr->data_info.min_pixel;
      _2d1.maxval_set = _2d2.maxval_set = hdr->data_info.max_pixel;
      _2d1.status4 |= BUF_MIN_MAX_SET;
      _2d2.status4 |= BUF_MIN_MAX_SET;

      if (((err = Scale2dPixels(&_2d1,&_2d1,(double) .0,
				(double) (pixel_type->map.dimx - 1),
				0,(double) .0,(double) .0,1)) != NO_ERROR) ||
	  ((err = Scale2dPixels(&_2d2,&_2d2,(double) .0,
				(double) (pixel_type->map.dimy - 1),
				0,(double) .0,(double) .0,1)) != NO_ERROR))
	break;

      DefinePartitionedPixelMap(pixel_type->slice_maps[lcv],
				&(pixel_type->map),&_2d1,&_2d2);

    }

  }

  free_buf_2d_rec(&_2d1);
  free_buf_2d_rec(&_2d2);
  c_free(imgfn);
  return err;
}


int
setup_analysis(anal,hdr,pixel_type)
     struct analysis_rec *anal;
     struct hdr_rec *hdr;
     struct pixel_type_rec *pixel_type;
{
  int err;
  
  if (((err = get_draw_mask_rec(&(anal->mask),
				hdr->data_info.dimx,hdr->data_info.dimy))
       != NO_ERROR) ||
      ((err = SetupBuf2dRec(&(anal->common),
			    hdr->data_info.dimx,hdr->data_info.dimy,
			    8 * (size_t) sizeof(struct common_pixel_rec),0))
       != NO_ERROR))
  {
    return err;
  }
  init_common_pixel_rec_matrix(&(anal->common),1);
  TRACE(printf("setup_analysis...end\n"););
  return NO_ERROR;
}


int
process_ntxt( nhdrfn, ndatfn, fmtfns, prog_name )
     char *nhdrfn,*ndatfn;
     struct strs_rec *fmtfns;
     char *prog_name;
{
  struct hdr_rec hdr;
  struct output_hdr_rec ndathdr;
  int err,define;
  struct histo_params_rec histo;
  struct pixel_type_rec pixel_type;
  struct slice_roi_rec *slice_rois;
  long total_slice_rois;
  struct text_output_rec text;
  struct analysis_rec anal;
  char *dir,*fn1,*suffix,*temp,*txtfn,str[20];
  long num_slices,cnt,lcv;
  struct count_analysis_rec count;
  char *reason;


  TRACE(printf("process_ntxt...nhdrfn = %s\n",nhdrfn));

  /*
   *  Read in slice and pixel info.
   */
  init_hdr_rec(&hdr);
  if ((err = rd_the_hdr(nhdrfn,&hdr)) != NO_ERROR)
  {
    printf("Error reading the hdr file = %s...err = %d\n",nhdrfn,err);
    return err;
  }

  num_slices = NumberOfSliceCuts(&(hdr.data_info));
  /* printf("process_ntxt...2\n");*/
  init_text_output_rec(&text,NULL);
  dir = fn1 = suffix = NULL;

  /*
   *  Read in:
   *	Regions Of Interest (for each slice)
   *	Pixel Histogram (if Thresholding Analysis used)
   *	Pixel Type List (if Grow-Region / segmentation pixmap used)
   *	Text Output Format
   *	Pixel Counts (if Count Analysis used)
   */
  if ((err = RdTheNdatFields( ndatfn,
			      &ndathdr, &slice_rois, &total_slice_rois,
			      &histo, &pixel_type, &(text.groups),
			      &(text.num_groups),
			      &(text.format),
			      &count)) != NO_ERROR)
  {
    if (err == FILE_ACCESS_ERROR)
    {
      file_access(ndatfn,14,NULL,&reason);
      printf("Error opening the ndat file = %s...reason = %s\n",
	     ndatfn,reason);
    }
    else
      printf("Error reading the ndat file...ndatfn = %s err = %d\n",
	     ndatfn,err);
  }
  else if ((err = break_fn(ndatfn,&dir,&fn1,&suffix)) == NO_ERROR)
  {
    TRACE(printf("pixel_type.status = %x...\n",
		 (unsigned long) pixel_type.status););
    if (pixel_type.status & PIXEL_TYPE_GROW_REGION)
    {
      TRACE(printf("AutoRdFilePixmapHdr1...\n"););
      if ((err = AutoRdFilePixmapHdr1(dir,fn1,".pixmap",&pixel_type))
	  == NO_ERROR)
      {
	TRACE(printf("rd_file_pixmap_slices...\n"););
	if ((err = rd_file_pixmap_slices(dir,fn1,".pixmap",
					 &pixel_type,num_slices)) == NO_ERROR)
	  pixel_type.status &= ~PIXEL_TYPE_2D_PARTITION;
      }
    }
    if (err == NO_ERROR)
      if ((define = IsAllPixelTypeMapReady_1(&pixel_type,&hdr)) != 2)
      {
	err = CANCEL;
	if (define == 0)
	  printf("Cancelled: pixel type map undefined !!!\n");
	else
	  printf("Cancelled: not all slice pixel type maps are defined !!!\n");
      }
      else if (pixel_type.status & PIXEL_TYPE_2D_PARTITION)
      {
	TRACE(printf("process_ntxt...PIXEL_TYPE_2D_PARTITION\n"));
	if ((err = SetupPartitionedSliceMaps(&pixel_type,&hdr,
					     ndatfn)) != NO_ERROR)
	  printf("ERROR setting up the slice pixel maps !!!\n");
      }
  }
  c_free(dir);
  c_free(fn1);
  c_free(suffix);

  init_analysis_rec(&anal);
  if (err == NO_ERROR)
    err = setup_analysis(&anal);
  if (err != NO_ERROR)
  {
    free_analysis_rec(&anal);
    free_ndat_fields(&ndathdr,slice_rois,total_slice_rois,
		     &histo,&pixel_type,NULL,NULL,NULL,&count);
    free_text_output_rec(&text);
    free_hdr_rec(&hdr);
    return err;
  }
  /* printf("process_ntxt...5\n");*/

  sprintf(str,"");
  for (cnt = 0; cnt < fmtfns->num_strs; cnt++)
  {
    printf("...processing %s with text format %s\n",ndatfn,fmtfns->strs[cnt]);
    free_output_text_format_rec(&(text.format));
    if ((err = RdTextFormat(fmtfns->strs[cnt],
			    &(text.groups),&(text.num_groups),&(text.format)))
	!= NO_ERROR)
      break;
    if ((err = setup_output_text_format_rec(&(text.format),&hdr,
					    text.groups,text.num_groups))
	!= NO_ERROR)
      break;

    /*
      pr_output_text_format_rec(&(text.format));
      */

    if ((err = setup_analysis_params(&anal,&(text.format),&pixel_type))
	!= NO_ERROR)
      return err;

    for (lcv = 0; lcv < num_slices; lcv++)
    {
      anal.cur_slice = &(hdr.scans[lcv * hdr.data_info.num_echoes]);
      anal.next_slice_num = lcv;
      TRACE(printf("anal.next_slice_num = %d\n",anal.next_slice_num););
      if ((err = AnalyzePixelTypes(&anal,&(slice_rois[lcv]),
				   &pixel_type,&(text.format),
				   pixel_type.slice_maps[lcv])) != NO_ERROR)
	break;
      TRACE(printf("anal.next_slice_num = %d; done\n",anal.next_slice_num););
    }
    TRACE(printf("process_ntxt...5\n"));

    if (err != NO_ERROR)
      break;

    temp = txtfn = NULL;
    if (fmtfns->num_strs > 1)
      sprintf(str,"-%-d",cnt+1);
    if (((err = cp_str(&temp,ndatfn)) != NO_ERROR) ||
	((err = drop_suffix(temp)) != NO_ERROR) ||
	((err = concat_strs(&txtfn,3,temp,str,".ntxt")) != NO_ERROR) ||
	((err = SavePixelTypeTextToFile(txtfn,&text,&hdr,slice_rois,
					&pixel_type,&ndathdr,prog_name,
					"batch1.0",
					"",nhdrfn,
					"",".img",
					"",temp,
					".ndat")) != NO_ERROR))
    {
      c_free(temp);
      c_free(txtfn);
      printf("Error saving the *.ntxt file...err = %d\n",err);
      break;
    }
    else
      printf("ntxt = %s created...\n",txtfn);

    flush_the_text_output(&text);
    c_free(temp);
    c_free(txtfn);
  }
  if (err == NO_ERROR)
  {
    pixel_type.status |= PIXEL_TYPE_ALL_ROIS_COMPUTED;
    pixel_type.status |= PIXEL_TYPE_COMPUTED |
      PIXEL_TYPE_AND_ANALYSIS_RELATED;
    printf("pixel_type.status = %x...\n",
	   (unsigned long) pixel_type.status);

    if (WrRiaNdatFile(ndatfn,
		      &ndathdr,slice_rois,total_slice_rois,
		      &histo,&pixel_type,
		      text.groups,text.num_groups,&(text.format),&count)
	!= NO_ERROR)
      printf("Error saving %s !!!\n",ndatfn);
    else
      printf("...%s updated!!!\n",ndatfn);
  }

  free_text_output_rec(&text);
  free_analysis_rec(&anal);
  free_ndat_fields(&ndathdr,slice_rois,total_slice_rois,
		   &histo,&pixel_type,NULL,NULL,NULL,&count);
  TRACE(printf("process_ntxt...end...err = %d\n",err));
  free_hdr_rec(&hdr);
  return err;
}


int
prog_usage(argc,argv)
     int argc;
     char *argv[];
{
  if (argc != 5)
  {
    printf("\nUsage : %s nhdr-fn ndat-dir ndat-prefix fmtfns\n",argv[0]);
    printf("Example 1 : %s mri10257a.nhdr /omri/ndat abc fmtfns\n",argv[0]);
    printf("   Input : mri10257a.nhdr\n\n");
    printf("         : /omri/ndat/abcmri10257a.ndat\n\n");
    printf("         : fmtfns\n\n");
    printf("   Output : /omri/ndat/abcmri10257a.pixmap\n\n");
    printf("\n");
    printf("Example 2 : %s \"/amri/*.nhdr\" /omri/ndat abc fmtfns\n",argv[0]);
    printf("   Input : /amri/*.nhdr\n\n");
    printf("         : /omri/ndat/abc*.ndat\n\n");
    printf("         : fmtfns\n\n");
    printf("   Output : /omri/ndat/abc*.pixmap\n\n");
    printf("\n");
    printf("Example 3 : %s \"/amri/*.nhdr\" /omri/ndat \"\" fmtfns\n",argv[0]);
    printf("   Input : /amri/*.nhdr\n\n");
    printf("         : /omri/ndat/*.ndat\n\n");
    printf("         : fmtfns\n\n");
    printf("   Output : /omri/ndat/*.pixmap\n\n");
    printf("\n");
    printf("Note : if you use '*' in the file specification, please\n");
    printf("       enclose that file specification by double quotes.\n");
    exit(1);
  }
}


int
get_params(argc,argv,dirfn,fn_patt,ndatdir,prefix,fmtfns)
     int argc;
     char *argv[];
     char **dirfn,**fn_patt,**ndatdir,**prefix;
     struct strs_rec *fmtfns;
{
  int err = NO_ERROR;

  init_strs_rec(fmtfns);
  if (argc != 2)
  {
    *dirfn = (char *) locate_dirfn(argv[1]);
    if (*dirfn == NULL) 
      if ((err = cp_str(dirfn,".")) != NO_ERROR)
	return err;
    drop_dirfn(argv[1]);

    if (((err = cp_str(fn_patt,argv[1])) != NO_ERROR) ||
	((err = cp_str(ndatdir,argv[2])) != NO_ERROR) ||
	((err = cp_str(prefix,argv[3])) != NO_ERROR))
      return err;

    if ((err = rd_ascii_file_strs_to_strs_rec(argv[4],fmtfns,0)) != NO_ERROR)
    {
      printf("Error reading the format file names = %s...err = %d\n",
	     argv[4],err);
      return err;
    }

  }
  else
  {
    exit(1);
  }

  return NO_ERROR;
}


int
main( argc,argv )
     int argc;
     char *argv[];
{
  int err;
  char **fns,*dirfn,*fn_patt,*fn,*temp,*ndatfn,*ndatdir,*prefix;
  unsigned int num_fns;
  unsigned long lcv;
  struct strs_rec fmtfns,errlog;

  prog_usage(argc,argv);
  if ((err = get_params(argc,argv,&dirfn,&fn_patt,&ndatdir,&prefix,&fmtfns))
      != NO_ERROR)
  {
    printf("Error getting parameters...err = %d\n",err);
    exit(err);
  }
  if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,fn_patt,
		     NULL,0,0,NULL)) != NO_ERROR)
    exit(err);

  err = CANCEL;
  if (num_fns == 0)
  {
    printf("Sorry...No file names...\n");
  }
  else
  {
    printf("\n");
    init_strs_rec(&errlog);
    for (lcv = 0; lcv < num_fns; lcv++)
    {
      fn = temp = ndatfn = NULL;
      if (((err = concat_strs( &fn, 3,
			       dirfn, "/", fns[lcv] )) == NO_ERROR)
	  && ((err = cp_str( &temp, fn )) == NO_ERROR)
	  && ((err = drop_suffix( temp )) == NO_ERROR)
	  && ((err = drop_dirfn( temp )) == NO_ERROR)
	  && ((err = concat_strs( &ndatfn, 5,
				  ndatdir, "/", prefix, temp, ".ndat" ))
	      == NO_ERROR))
      {
	printf("%d) processing %s & %s\n",lcv,fn,ndatfn);
	if ((err = process_ntxt( fn, ndatfn, &fmtfns, argv[0] )) != NO_ERROR)
	{
	  printf("...%d) ERROR processing %s & %s...err = %d\n",
		 lcv,fn,ndatfn,err);
	  add_str_to_strs_rec(&errlog,fn);
	  err = NO_ERROR;
	}
      }
      c_free(fn);
      c_free(temp);
      c_free(ndatfn);
      if (err != NO_ERROR)
      {
	printf("...%d) ERROR processing %s/%s\n",lcv,dirfn,fns[lcv]);
	printf("Quiting...\n");
	break;
      }
    }
    if (errlog.num_strs > 0)
    {
      printf("\nErrors occurred when processing the following files :-\n");
      for (lcv = 0; lcv < errlog.num_strs; lcv++)
	printf("%d) %s\n",lcv,errlog.strs[lcv]);
      printf("\n");
    }
    free_strs_rec(&errlog);
  }
  c_free(dirfn);
  c_free(fn_patt);
  c_free(prefix);
  c_free(ndatdir);
  free_strs_rec(&fmtfns);
  exit(err);
}
