/****************************************************************************
color.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: color.c,v 1.9 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <ctype.h>
#include <string.h>

#include <xview/xview.h>
#include <xview/cms.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"

/*
  cc -o color color.c ptr_and_dim.o rd_text.o str.o
*/



/* Functions defined in this file */
void init_color_scale_rec(struct color_scale_rec *scale);
void free_color_scale_rec(struct color_scale_rec *scale);
int cp_color_scale_rec(struct color_scale_rec *dest,
    struct color_scale_rec *src);
void init_color_rec(struct color_rec *color);
void free_color_rec(struct color_rec *color);
void init_intensity_rec(struct intensity_rec *intensity);
void free_intensity_rec(struct intensity_rec *intensity);
void init_color_indx(struct color_rec *color);
void init_pallette_rec(struct pallette_rec *def_color);
void free_pallette_rec(struct pallette_rec *def_color);
int get_rgb_ptr_memory(struct color_rec *color, unsigned long total_colors);
int copy_color_rec(struct color_rec *dest, struct color_rec *src, int status);
int copy_intensity_rec(struct intensity_rec *dest, struct intensity_rec *src,
    int status);
int copy_pallette_rec(struct pallette_rec *dest, struct pallette_rec *src,
    int status);
int add_intensity_rec(struct pallette_rec *def_color,
    struct intensity_rec *intensity, int status, int is_create_new,
    long *indx);
int delete_cur_intensity_rec(struct pallette_rec *def_color);
int def_color_scale_rec_control(struct color_scale_rec *scale);
void set_color_scale_rec_control(struct color_scale_rec *scale, long first,
    long num_colors, int lb, int ub, int mid_value, int range);
int define_first_visible(struct color_rec *color,
    struct color_scale_rec *scale);
void pr_color_rec(struct color_rec *color);
void pr_color_scale_rec(struct color_scale_rec *scale);
void pr_intensity_rec(struct intensity_rec *intensity);
void pr_pallette_rec(struct pallette_rec *def_color);
void map_all_scale_rgb(struct color_rec *color);
void unmap_all_scale_rgb(struct color_rec *color);
void map_scale_rgb(struct color_rec *color, struct color_scale_rec *scale);
void unmap_scale_rgb(struct color_rec *color, struct color_scale_rec *scale);
void default_map_intensity_rgb(struct intensity_rec *intensity);
void set_scale_range(struct color_scale_rec *scale, int value);
int GlobalToLocalColorIndx(struct color_scale_rec *scale,
    unsigned long global_indx, unsigned long *color_indxs);
int define_actual_color_scale(unsigned long *color_indxs,
    struct color_scale_rec *scale, int is_GT_ub_background);
int define_actual_intensity_scale(struct intensity_rec *intensity,
    int is_GT_ub_background, long cur_scale);
int define_actual_scale(struct color_rec *color, int is_GT_ub_background,
    struct color_scale_rec *scale);
int rd_intensity_rec(FILE *fid, struct intensity_rec *intensity, int ordinal);
void init_program_fore_and_background_colors(struct color_rec *color);
void init_program_colors(struct color_rec *color);
int setup_default_colors(struct intensity_rec *intensity,
    Xv_singlecolor *system_colors, unsigned long num_system_colors);
int rd_pallette_rec(char *fn, struct pallette_rec *def_color);
int setup_default_intensity(struct pallette_rec *def_color,
    Xv_singlecolor *system_colors, unsigned long num_system_colors);
int setup_pallette_rec(char *fn, struct pallette_rec *def_color,
    Xv_singlecolor *system_colors, unsigned long num_system_colors);
int check_save_color(struct pallette_rec *def_color);
void set_cur_color_scale_save_status(struct intensity_rec *intensity,
    int status, int is_and);
int wr_intensity_rec(FILE *fid, struct intensity_rec *intensity);
int wr_pallette_rec(char *fn, struct pallette_rec *def_color);
struct color_scale_rec *find_scale(struct intensity_rec *intensity,
    char *label, int set_cur);
struct intensity_rec *find_intensity_rec(struct pallette_rec *def_color,
    char *intensity_label, int set_cur);
struct intensity_rec *get_intensity_rec(struct pallette_rec *def_color,
    char *intensity_label, int set_cur);



/*----------------------------------------------------------------------
init_color_scale_rec - ???
----------------------------------------------------------------------*/

void
init_color_scale_rec(scale)
     struct color_scale_rec *scale;
{
  scale->label = NULL;
  scale->status = 0;
  scale->first = scale->first_visible = scale->num_colors = 0;
}



/*----------------------------------------------------------------------
free_color_scale_rec - ???
----------------------------------------------------------------------*/

void
free_color_scale_rec(scale)
     struct color_scale_rec *scale;
{
  c_free(scale->label);
  scale->status = 0;
}



/*----------------------------------------------------------------------
cp_color_scale_rec - ???
----------------------------------------------------------------------*/

int
cp_color_scale_rec(dest,src)
     struct color_scale_rec *dest,*src;
{
  char *str;
  int err;

  if (str_cmp(dest->label,src->label) == 0)
    str = dest->label;
  else
  {
    c_free(dest->label);
    str = NULL;
  }
  *dest = *src;
  if (str != NULL)
  {
    err = NO_ERROR;
    dest->label = str;
  }
  else
  {
    err = cp_str(&(dest->label),src->label);
  }
  return err;
}



/*----------------------------------------------------------------------
init_color_rec - ???
----------------------------------------------------------------------*/

void
init_color_rec(color)
     struct color_rec *color;
{
  color->status = 0;
  color->total_colors = color->num_system_colors = 0;
  color->use_system_colors = 0;
  color->ori_red = color->red = NULL;
  color->color_indxs = NULL;
  color->color_status = NULL;
}



/*----------------------------------------------------------------------
free_color_rec - ???
----------------------------------------------------------------------*/

void
free_color_rec(color)
     struct color_rec *color;
{
  if (color->status & RGB_FREE_BUFFERS)
  {
    c_free(color->ori_red);
    TRACE(printf("free color memory..color->status = %d\n",
		 (int) color->status));
  }
  init_color_rec(color);
}



/*----------------------------------------------------------------------
init_intensity_rec - ???
----------------------------------------------------------------------*/

void
init_intensity_rec(intensity)
     struct intensity_rec *intensity;
{
  intensity->label = NULL;
  intensity->cur_scale = intensity->num_scales = intensity->total_scales = 0;
  intensity->scales = NULL;
  init_color_rec(&(intensity->color));
}



/*----------------------------------------------------------------------
free_intensity_rec - ???
----------------------------------------------------------------------*/

void
free_intensity_rec(intensity)
     struct intensity_rec *intensity;
{
  unsigned long lcv;

  c_free(intensity->label);
  if (intensity->total_scales > 0)
  {
    for (lcv = 0; lcv < intensity->num_scales; lcv++)
      free_color_scale_rec(&(intensity->scales[lcv]));
    c_free(intensity->scales);
  }
  free_color_rec(&(intensity->color));
  init_intensity_rec(intensity);
}



/*----------------------------------------------------------------------
init_color_indx - ???
----------------------------------------------------------------------*/

void
init_color_indx(color)
     struct color_rec *color;
{
  unsigned long lcv;

  for (lcv = 0; lcv < color->total_colors; lcv++)
    color->color_indxs[lcv] = lcv;
}



/*----------------------------------------------------------------------
init_pallette_rec - ???
----------------------------------------------------------------------*/

void
init_pallette_rec(def_color)
     struct pallette_rec *def_color;
{
  def_color->color_fn = NULL;
  def_color->total_pallettes = def_color->num_pallettes = def_color->cur_pallette = 0;
  def_color->pallettes = NULL;
}



/*----------------------------------------------------------------------
free_pallette_rec - ???
----------------------------------------------------------------------*/

void
free_pallette_rec(def_color)
     struct pallette_rec *def_color;
{
  unsigned long lcv;

  c_free(def_color->color_fn);
  if (def_color->total_pallettes > 0)
  {
    for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
    {
      free_intensity_rec(def_color->pallettes[lcv]);
      c_free(def_color->pallettes[lcv]);
    }
    c_free(def_color->pallettes);
  }
  init_pallette_rec(def_color);
}



/*----------------------------------------------------------------------
get_rgb_ptr_memory - ???
----------------------------------------------------------------------*/

int
get_rgb_ptr_memory(color,total_colors)
     struct color_rec *color;
     unsigned long total_colors;
{
  unsigned long total_items,size,lcv;
  unsigned short *ptr;
  int err;


  total_items = 0;
  size =
    sizeof(*(color->ori_red)) +	/* Original RGB values */
    sizeof(*(color->ori_green)) + 
    sizeof(*(color->ori_blue)) +
    sizeof(*(color->red)) +	/* Current (?) RGB values */
    sizeof(*(color->green)) +
    sizeof(*(color->blue)) + 
    sizeof(*(color->color_indxs)) +
    sizeof(*(color->color_status)); /* Color index status */

  PRT(printf("	get_rgb_ptr_memory()\n");
      printf("		total_colors = %d size = %d\n", total_colors, size););
  if ((err = get_ptr(&ptr,&total_items,total_colors * size,1)) != NO_ERROR)
  {
    free_color_rec(color);
    return err;
  }
  color->ori_red = ptr;
  /*
   *  Set other pointers as offsets into main memory chunk
   */
  color->ori_green = (void *) ((unsigned char *) color->ori_red + 
			       total_colors * sizeof(*(color->ori_red)));
  color->ori_blue = (void *) ((unsigned char *) color->ori_green + 
			      total_colors * sizeof(*(color->ori_green)));
  color->red = (void *) ((unsigned char *) color->ori_blue + 
			 total_colors * sizeof(*(color->ori_blue)));
  color->green = (void *) ((unsigned char *) color->red + 
			   total_colors * sizeof(*(color->red)));
  color->blue = (void *) ((unsigned char *) color->green + 
			  total_colors * sizeof(*(color->green)));
  color->color_indxs = (void *) ((unsigned char *) color->blue + 
				 total_colors * sizeof(*(color->blue)));
  color->color_status = (void *) ((unsigned char *) color->color_indxs + 
				  total_colors * sizeof(*(color->color_indxs)));

  PRT(printf("		ori_red=%p, ori_green=%p, ori_blue=%p\n",
	     (void *) color->ori_red,
	     (void *) color->ori_green,
	     (void *) color->ori_blue));
  PRT(printf("		red=%p, green=%p, blue=%p\n",
	     (void *) color->red,
	     (void *) color->green,color->blue));
  PRT(printf("		color_indxs=%p, color_status=%p\n",
	     (void *) color->color_indxs,
	     (void *) color->color_status));
  color->status |= RGB_FREE_BUFFERS;
  for (lcv = 0; lcv < total_colors; lcv++)
    color->color_indxs[lcv] = lcv;
  /* Next line was: 
  bzero( color->color_status, total_colors * sizeof(*(color->color_status)) );
  */
  memset( color->color_status, 0,
	  total_colors * sizeof(*(color->color_status)) );
  return err;
}



/*----------------------------------------------------------------------
copy_color_rec - ???
----------------------------------------------------------------------*/

int
copy_color_rec( dest, src, status )
     struct color_rec *dest, *src;
     unsigned short status;
{
  int err;


  free_color_rec( dest );
  memcpy( dest, src, (size_t) sizeof( *dest ) );

  if (status != RGB_FREE_BUFFERS) 
  {
    dest->status &= RGB_DONT_FREE_BUFFERS;
    return CANCEL;
  }

  if ((err = get_rgb_ptr_memory(dest,dest->total_colors)) != NO_ERROR)
    return err;

  memcpy( dest->ori_red,
	  src->ori_red,
	  ((sizeof(*(dest->ori_red)) +
	    sizeof(*(dest->ori_green)) + 
	    sizeof(*(dest->ori_blue)) +
	    sizeof(*(dest->red)) +
	    sizeof(*(dest->green)) +
	    sizeof(*(dest->blue)) +
	    sizeof(*(dest->color_indxs)) +
	    sizeof(*(dest->color_status)))
	   * dest->total_colors) );
  dest->status |= RGB_COLORS_VALID;

  return NO_ERROR;
}



/*----------------------------------------------------------------------
copy_intensity_rec - ???
----------------------------------------------------------------------*/

int
copy_intensity_rec(dest,src,status)
     struct intensity_rec *dest,*src;
     unsigned short status;
{
 int err;
 long lcv;
 struct intensity_rec temp;

 init_intensity_rec(&temp);
 if (((err = cp_str(&(temp.label),src->label))  != NO_ERROR) ||
     ((err = get_ptr(&(temp.scales),&(temp.total_scales),src->num_scales,
        sizeof(*(temp.scales)))) != NO_ERROR))
    {free_intensity_rec(&temp);
     return err;}

 temp.num_scales = temp.total_scales;
 temp.cur_scale = src->cur_scale;
 for (lcv = 0; lcv < temp.num_scales; lcv++)
    init_color_scale_rec(&(temp.scales[lcv]));
 for (lcv = 0; lcv < temp.num_scales; lcv++)
    {temp.scales[lcv] = src->scales[lcv];
     temp.scales[lcv].label = NULL;
     if ((err = cp_str(&(temp.scales[lcv].label),src->scales[lcv].label))
           != NO_ERROR)
        {free_intensity_rec(&temp);
         return err;}
    }

 if ((err = copy_color_rec(&(temp.color),&(src->color),status)) == NO_ERROR)
    {free_intensity_rec(dest);
     *dest = temp;}
   else free_intensity_rec(&temp);

 return err;
}



/*----------------------------------------------------------------------
copy_pallette_rec - ???
----------------------------------------------------------------------*/

int
copy_pallette_rec(dest,src,status)
 struct pallette_rec *dest,*src;
 unsigned short status;
{
 int err;
 unsigned long lcv;

 PRT(printf("copy_pallette_rec...1\n"))
 free_pallette_rec(dest);
 /* FIXME:  Memory leak below; *(dest->color_fn) is not freed. */
 if ((err = copy_str(&(dest->color_fn),src->color_fn,
         get_strlen(src->color_fn))) != NO_ERROR)
    {free_pallette_rec(dest);
     return err;}
 PRT(printf("copy_pallette_rec...2\n"))
 dest->cur_pallette = src->cur_pallette;
 if ((err = get_handle(&(dest->pallettes),&(dest->total_pallettes),
        src->num_pallettes,(size_t) sizeof(**(dest->pallettes)))) != NO_ERROR)
    {free_pallette_rec(dest);
     return err;}
 dest->num_pallettes = dest->total_pallettes;
 dest->cur_pallette = src->cur_pallette;
 for (lcv = 0; lcv < dest->num_pallettes; lcv++)
    init_intensity_rec(dest->pallettes[lcv]);

 PRT(printf("copy_pallette_rec...3\n"))
 for (lcv = 0; lcv < src->num_pallettes; lcv++)
    if ((err = copy_intensity_rec(dest->pallettes[lcv],
                   src->pallettes[lcv],status)) != NO_ERROR)
       {free_pallette_rec(dest);
        return err;}
 PRT(printf("copy_pallette_rec...4\n"))
 return NO_ERROR;
}



/*----------------------------------------------------------------------
add_intensity_rec - ???
----------------------------------------------------------------------*/

int
add_intensity_rec(def_color,intensity,status,is_create_new,indx)
 struct pallette_rec *def_color;
 struct intensity_rec *intensity;
 unsigned short status;
 int is_create_new;
 long *indx;
{
 unsigned long total;
 struct intensity_rec *intensity1;
 int err;

 if (is_create_new)
    {total = 0;
     if ((err = get_ptr(&intensity1,&total,1,(size_t) sizeof(*intensity1))) != NO_ERROR)
        return err;
     init_intensity_rec(intensity1);
     if ((err = copy_intensity_rec(intensity1,intensity,status)) != NO_ERROR)
        {free_intensity_rec(intensity1);
         c_free(intensity1);
         return err;}
    }
   else intensity1 = intensity;
 if (def_color->num_pallettes >= def_color->total_pallettes)
    if ((err = get_ptr(&(def_color->pallettes),&(def_color->total_pallettes),1,
          (size_t) sizeof(*(def_color->pallettes)))) != NO_ERROR)
        {if (is_create_new)
            {free_intensity_rec(intensity1);
             c_free(intensity1);}
         return err;}
 *indx = def_color->num_pallettes;
 def_color->pallettes[def_color->num_pallettes] = intensity1;
 def_color->num_pallettes += 1;
 return NO_ERROR;
}



/*----------------------------------------------------------------------
delete_cur_intensity_rec - ???
----------------------------------------------------------------------*/

int
delete_cur_intensity_rec(def_color)
 struct pallette_rec *def_color;
{
 long lcv;

 free_intensity_rec(def_color->pallettes[def_color->cur_pallette]);
 c_free(def_color->pallettes[def_color->cur_pallette]);
 def_color->num_pallettes -= 1;
 for (lcv = def_color->cur_pallette; lcv < def_color->num_pallettes; lcv++)
    def_color->pallettes[lcv] = def_color->pallettes[lcv + 1];
 if (def_color->cur_pallette >= def_color->num_pallettes) def_color->cur_pallette = 0;
 return NO_ERROR;
}



/*----------------------------------------------------------------------
def_color_scale_rec_control - ???
----------------------------------------------------------------------*/

int
def_color_scale_rec_control(scale)
 struct color_scale_rec *scale;
{
 scale->status |= COLOR_CONTROL_VALID;
 scale->lb = scale->ub = scale->num_colors / 4;
 scale->mid_value = scale->first + scale->num_colors / 2;
 scale->range = scale->num_colors / 2;
 return NO_ERROR;
}


/*
 value = color->num_colors;
 control->min_value = control->mid_value - value / 2;
 if (value % 2)  value = value / 2 + 1;
   else value /= 2;
 control->max_value = control->mid_value + value;
*/



/*----------------------------------------------------------------------
set_color_scale_rec_control - ???
----------------------------------------------------------------------*/

void
set_color_scale_rec_control(scale,first,num_colors,lb,ub,mid_value,range)
     struct color_scale_rec *scale;
     long first,num_colors;
     int lb, ub;
     int mid_value,range;
{
  int height, old_height;
 

  scale->lb = lb;
  scale->ub = ub;

  if ((scale->status & COLOR_CONTROL_VALID) &&
      !(scale->status & COLOR_CONTROL_OVERWRITE) &&
      (scale->num_colors > 0))
  {
    old_height = scale->mid_value - scale->first;
    if (old_height < 0)
      old_height = 0;
    height = (int) ULongRound((double) (num_colors * old_height) /
			      (double) scale->num_colors);
    if (height > num_colors)
      height = num_colors;

    scale->mid_value = first + height / 2;

    scale->range = (int) ULongRound((double) (num_colors * scale->range) /
				    (double) scale->num_colors);
  }

  else				/* Initialize structures */
  {
    scale->mid_value = mid_value;
    scale->range = (range < 0 ? 0 : range);
  }

  scale->first = first;
  scale->num_colors = num_colors;
  if ((scale->mid_value < first) ||
      (scale->mid_value > (first + num_colors - 1)))
    scale->mid_value = first + num_colors / 2;
  scale->status |= COLOR_CONTROL_VALID;
}



/*----------------------------------------------------------------------
define_first_visible - ???
----------------------------------------------------------------------*/

int
define_first_visible(color,scale)
 struct color_rec *color;
 struct color_scale_rec *scale;
{
 unsigned long lcv,last;

 last = scale->first + scale->num_colors;
 scale->first_visible = scale->first;
 for (lcv = scale->first; lcv < last;lcv++)
    if (!(color->color_status[lcv] & BACKGROUND_COLOR))
       {scale->first_visible = lcv;
        break;}
 return NO_ERROR;
}


/*----------------------------------------------------------------------
pr_color_rec - ???
----------------------------------------------------------------------*/

void
pr_color_rec(color)
 struct color_rec *color;
{
 unsigned long lcv;

 printf("pr_color_rec status = %x total_colors = %ld\n",
  (int) color->status,color->total_colors);

 for (lcv = 0; lcv < color->total_colors;lcv++)
     printf("lcv = %ld index# = %d r = %d/%d g = %d/%d b = %d/%d\n",lcv,
      (int) color->color_indxs[lcv],
          (int) color->ori_red[lcv],(int) color->red[lcv],
          (int) color->ori_green[lcv],(int) color->green[lcv],
          (int) color->ori_blue[lcv],(int) color->blue[lcv]);

}



/*----------------------------------------------------------------------
pr_color_scale_rec - ???
----------------------------------------------------------------------*/

void
pr_color_scale_rec(scale)
     struct color_scale_rec *scale;
{
  printf("	pr_color_scale_rec()\n");
  printf("		scale = %p\n", (void *) scale);
  printf("		label = %s first = %ld num_colors = %ld status = %lx\n",
	 scale->label,scale->first,scale->num_colors,
	 (unsigned long) scale->status);
  printf("		lb = %d ub = %d mid_value = %d range = %d\n",
	 scale->lb,scale->ub,scale->mid_value,scale->range);
}



/*----------------------------------------------------------------------
pr_intensity_rec - ???
----------------------------------------------------------------------*/

void
pr_intensity_rec(intensity)
     struct intensity_rec *intensity;
{
  unsigned long lcv;

  printf("	pr_intensity_rec()\n");
  printf("		intensity=%p, label=\"%s\", cur=%ld, num=%ld, total=%ld\n",
	 (void *) intensity,
	 intensity->label,
	 intensity->cur_scale,
	 intensity->num_scales,
	 intensity->total_scales);
  for (lcv = 0; lcv < intensity->num_scales;lcv++)
    pr_color_scale_rec(&(intensity->scales[lcv]));

  /*  printf("	pr_intensity_rec...end\n"); */
}


/*----------------------------------------------------------------------
pr_pallette_rec - ???
----------------------------------------------------------------------*/

void
pr_pallette_rec(def_color)
 struct pallette_rec *def_color;
{
 unsigned long lcv;

 printf("pr_pallette_rec...def_color = %p cur = %ld num = %ld total = %ld\n",
	def_color,def_color->cur_pallette,def_color->num_pallettes,
	def_color->total_pallettes);
 for (lcv = 0; lcv < def_color->num_pallettes;lcv++)
    pr_intensity_rec(def_color->pallettes[lcv]);

 printf("pr_pallette_rec...end\n");

}


/*----------------------------------------------------------------------
map_all_scale_rgb - ???
----------------------------------------------------------------------*/

void
map_all_scale_rgb(color)
 struct color_rec *color;
{
 unsigned long lcv;
/*
 TRACE(printf("map_all_scale_rgb...ori_red = %u ori_green = %u ori_blue = %u\n",
      color->ori_red,color->ori_green,color->ori_blue))
 TRACE(printf("...red = %u green = %u  blue = %u color_indxs = %u\n",
        color->red,color->green,color->blue,color->color_indxs))
*/
 for (lcv = 0; lcv < color->total_colors; lcv++)
    {/*TRACE(printf("%d) color_indxs = %d ",lcv,color->color_indxs[lcv]))
     TRACE(printf("red = %d g = %d b = %d\n",
         (unsigned long) color->ori_red[color->color_indxs[lcv]],
         (unsigned long) color->ori_green[color->color_indxs[lcv]],
         (unsigned long) color->ori_blue[color->color_indxs[lcv]]))
     */
     color->red[lcv] = color->ori_red[color->color_indxs[lcv]];
     color->green[lcv] = color->ori_green[color->color_indxs[lcv]];
     color->blue[lcv] = color->ori_blue[color->color_indxs[lcv]];
    }
/*
 TRACE(printf("map_all_scale_rgb...end\n"))
*/
}


/*----------------------------------------------------------------------
unmap_all_scale_rgb - ???
----------------------------------------------------------------------*/

void
unmap_all_scale_rgb(color)
 struct color_rec *color;
{
 init_color_indx(color);
 map_all_scale_rgb(color);
}


/*----------------------------------------------------------------------
map_scale_rgb - ???
----------------------------------------------------------------------*/

void
map_scale_rgb(color,scale)
 struct color_rec *color;
 struct color_scale_rec *scale;
{
 unsigned long lcv,last;

 last = scale->first + scale->num_colors;
 for (lcv = scale->first; lcv < last; lcv++)
    {
     color->red[lcv] = color->ori_red[color->color_indxs[lcv]];
     color->green[lcv] = color->ori_green[color->color_indxs[lcv]];
     color->blue[lcv] = color->ori_blue[color->color_indxs[lcv]];
    }
}


/*----------------------------------------------------------------------
unmap_scale_rgb - ???
----------------------------------------------------------------------*/

void
unmap_scale_rgb(color,scale)
 struct color_rec *color;
 struct color_scale_rec *scale;
{
 unsigned long lcv,last;

 last = scale->first + scale->num_colors;
 for (lcv = scale->first; lcv < last; lcv++)
    {
     color->red[lcv] = color->ori_red[lcv];
     color->green[lcv] = color->ori_green[lcv];
     color->blue[lcv] = color->ori_blue[lcv];
    }
}


/*----------------------------------------------------------------------
default_map_intensity_rgb - ???
----------------------------------------------------------------------*/

void
default_map_intensity_rgb(intensity)
 struct intensity_rec *intensity;
{
 unsigned long lcv;

 for (lcv = 0; lcv < intensity->num_scales;lcv++)
    if (intensity->scales[lcv].status & COLOR_LOCAL)
       unmap_scale_rgb(&(intensity->color),&(intensity->scales[lcv]));
      else map_scale_rgb(&(intensity->color),&(intensity->scales[lcv]));
}


/*----------------------------------------------------------------------
set_scale_range - ???
----------------------------------------------------------------------*/

void
set_scale_range(scale,value)
 struct color_scale_rec *scale;
 int value;
{
 scale->range = value;
 scale->lb = value / 2;
 if (value % 2) scale->ub = scale->lb + 1;
   else scale->ub = scale->lb;
}


/*----------------------------------------------------------------------
GlobalToLocalColorIndx - ???
----------------------------------------------------------------------*/

int
GlobalToLocalColorIndx(scale,global_indx,color_indxs)
 struct color_scale_rec *scale;
 unsigned long global_indx,*color_indxs;
{
 float step,step_value;
 unsigned long indx,highest_indx;
 long min_value;

 if (global_indx <= scale->first)
    {PRT1(printf("global_indx = %d first = %d\n",
         global_indx,scale->first);)
     return scale->first;}
 highest_indx = scale->first + scale->num_colors - 1;
 if (global_indx >= highest_indx) 
    {PRT1(printf("global_indx = %d highest_indx = %d ind = %d\n",
            global_indx,highest_indx,color_indxs[scale->num_colors - 1]);)
     return highest_indx;}

 step = (float) scale->num_colors / (float) (scale->lb + scale->ub + 1);
 min_value = scale->mid_value - scale->lb;

 if (min_value >= (long) scale->first)
    {
     if (global_indx < min_value) return global_indx;
     step_value = (float) (global_indx - scale->first) / step;
     indx = min_value + (unsigned long) step_value;
     if (indx > highest_indx) return highest_indx;
     return indx;
    }
   else
    {step_value = (float) (global_indx - min_value) / step;
     indx = min_value + (unsigned long) step_value;
     if (indx < (int) scale->first) return scale->first;
     if (indx > highest_indx) return highest_indx;
     return indx;
    }
}

/*****************
 PRT1(if (indx == highest_indx)
    printf("GL = %d step = %f stv = %f first = %d num = %d mid = %d lb = %d ub = %d indx = %d ind = %d\n",
     global_indx,step,step_value,scale->first,scale->num_colors,scale->mid_value,
     scale->lb,scale->ub,indx,color_indxs[indx]);
   else 
    printf("GL = %d step = %f stv = %f first = %d num = %d mid = %d lb = %d ub = %d indx = %d ind = %d %d\n",
     global_indx,step,step_value,scale->first,scale->num_colors,scale->mid_value,
     scale->lb,scale->ub,indx,color_indxs[indx],color_indxs[indx + 1]);)



 PRT1(if (indx == highest_indx)
    printf("global = %d step = %f stv = %f first = %d num = %d mid = %d lb = %d ub = %d indx = %d ind = %d\n",
     global_indx,step,step_value,scale->first,scale->num_colors,scale->mid_value,
     scale->lb,scale->ub,indx,color_indxs[indx]);
   else 
    printf("global = %d step = %f stv = %f first = %d num = %d mid = %d lb = %d ub = %d indx = %d ind = %d %d\n",
     global_indx,step,step_value,scale->first,scale->num_colors,scale->mid_value,
     scale->lb,scale->ub,indx,color_indxs[indx],color_indxs[indx + 1]);)
*******************/



/*----------------------------------------------------------------------
define_actual_color_scale - ???
----------------------------------------------------------------------*/

int
define_actual_color_scale(color_indxs,scale,is_GT_ub_background)
 unsigned long *color_indxs;
 struct color_scale_rec *scale;
 int is_GT_ub_background;
{
 long lcv,color_range,highest_indx,min_value,max_value;
 float step,step_value,highest;
 unsigned long first,num_colors;

 if (is_GT_ub_background) scale->status |= COLOR_CONTROL_EXCLUDE_BKG;
   else scale->status &= ~COLOR_CONTROL_EXCLUDE_BKG;
 color_range = scale->lb + scale->ub + 1;
 min_value = scale->mid_value - scale->lb;
 max_value = scale->mid_value + scale->ub;
 first = scale->first;
 num_colors = scale->num_colors;
/*
 printf("min_value = %d first = %d\n",min_value,first);
*/
 if ((int) min_value > (int) first)
    {
     for (lcv = first; lcv < min_value; lcv++)
        color_indxs[lcv - first] = first;
    }
 step = (float) num_colors / (float) color_range;
 highest_indx = first + num_colors - 1;
 step_value = (float) first;

 highest = (float) highest_indx;

 TRACE(printf("mid = %d min_value = %d  max_value = %d highest_indx = %d\n",
       scale->mid_value,min_value,max_value,highest_indx))
 for (lcv = min_value; lcv <= max_value; lcv++)
    {	
     if ( (step_value += step) > highest) step_value = highest;
     if (lcv >= (int) first)
        {if (lcv > (int) highest_indx) break;
         color_indxs[lcv - first] =
                   (int) ULongRound((double) step_value);}
    }

 if (max_value < highest_indx)
    {if (is_GT_ub_background)
        for (lcv = max_value + 1; lcv <= highest_indx; lcv++)
           color_indxs[lcv - first] = first;
       else
        for (lcv = max_value + 1; lcv <= highest_indx; lcv++)
           color_indxs[lcv - first] = highest_indx;
    }
/*
 for (lcv = first; lcv < first + num_colors; lcv++)
    printf("%d) indx = %d\n",lcv,color_indxs[lcv - first]);
*/
 TRACE(printf("define_actual_color_scale mid = %d lb = %d  ub = %d range = %d\n",
      (int) scale->mid_value,scale->lb,
      scale->ub,scale->range))
 return NO_ERROR;
}


/*----------------------------------------------------------------------
define_actual_intensity_scale - ???
----------------------------------------------------------------------*/

int
define_actual_intensity_scale(intensity,is_GT_ub_background,cur_scale)
 struct intensity_rec *intensity;
 int is_GT_ub_background;
 long cur_scale;
{
 define_actual_color_scale(
     &(intensity->color.color_indxs[intensity->scales[cur_scale].first]),
     &(intensity->scales[cur_scale]),is_GT_ub_background);
 TRACE(printf("define_actual_intensity_scale map_all_scale_rgb...\n"))
 map_scale_rgb(&(intensity->color),&(intensity->scales[cur_scale]));
 TRACE(printf("define_actual_intensity_scale 1...\n"))
 return NO_ERROR;
}


/*----------------------------------------------------------------------
define_actual_scale - ???
----------------------------------------------------------------------*/

int
define_actual_scale(color,is_GT_ub_background,scale)
 struct color_rec *color;
 int is_GT_ub_background;
 struct color_scale_rec *scale;
{
 define_actual_color_scale(&(color->color_indxs[scale->first]),
     scale,is_GT_ub_background);
 TRACE(printf("define_actual_intensity_scale map_all_scale_rgb...\n"))
 map_scale_rgb(color,scale);
 TRACE(printf("define_actual_scale 1...\n"))
 return NO_ERROR;
}



/*----------------------------------------------------------------------
rd_intensity_rec - ???

Returns error code, 0 = OK.
----------------------------------------------------------------------*/

int
rd_intensity_rec(fid,intensity, ordinal)
     FILE *fid;
     struct intensity_rec *intensity;
     int ordinal;
{
  int err;
  char str[80];
  int ary[7],*ints;
  unsigned long total_items,lcv,indx,num;
  struct color_rec *color;


  TRACE(printf("rd_intensity_rec()\n"););

  color = &(intensity->color);
  color->status = RGB_INVALID_COLOR_FILE;

  /*
   *  For each color intensity bundle, read in:
   *
   *  1) Label,			 	str
   *  2) Total number of colors,	ary[0]
   *  3) Flag: Use system colors?,	ary[1]
   *  4) Number of system colors,	ary[2]
   *  5) Reserve color start (?)	ary[3] 
   */

  total_items = sizeof(str) - 1;
  if ((err = RdText(fid,str,&total_items,RD_TEXT,0)) != NO_ERROR)
    return err;
  trim_str_blanks(str);
  if ((err = cp_str(&(intensity->label),str)) != NO_ERROR)
    return err;

  total_items = 4;
  if ( (err = RdText(fid,ary,&total_items,RD_INT,0)) )
    return err;
  if (total_items != 4 || ary[0] <= 2) /* At least 3 colors */
    return ~NO_ERROR;
  if (ary[2] >= ary[0])
    return CANCEL;
  if (ary[3] >= ary[0])
    return CANCEL;

  color->total_colors = ary[0];
  color->use_system_colors = ary[1];
  color->num_system_colors = ary[2];
  color->reserve_color_start = ary[3];

  PRT(printf( "	label = \"%s\"\n",intensity->label );
      printf( "	total colors: %d\n", color->total_colors );
      printf( "	use system colors? %c\n",
	      (color->use_system_colors ? 'y' : 'n') );
      printf( "	number of system colors: %d\n", color->num_system_colors );
      printf( "	reserve color start: %d\n", color->reserve_color_start );
      );

  NOPRT(printf("rd_intensity_rec 1 err = %d\n",err));

  total_items = 0;
  num = 4;			/* # of fields to read per color */
  if ((err = get_ptr( &ints, &total_items,
		      (unsigned long) (color->total_colors * num),
		      (size_t) sizeof(int))) != NO_ERROR)
    return err;
  /* total_items now should contain (color->total_colors * num) */

  if ((err = RdText( fid, ints, &total_items, RD_INT, 0 )) != NO_ERROR)
  {
    c_free(ints);
    return err;
  }
  /* total_items now should contain actual number of colors read */

  if (total_items != (color->total_colors * num))
  {
    c_free(ints);
    return BAD_DATA;
  }

  color->red = color->green = color->blue = NULL;
  total_items /= num;

  if ((err = get_rgb_ptr_memory( color, (unsigned long) total_items))
      != NO_ERROR)
  {
    c_free(ints);
    return err;
  }

  if (color->use_system_colors)
  {
    PRT1(printf("	Using system colors.\n"););
    for (lcv = color->num_system_colors,
	   indx = color->num_system_colors * num;
	 lcv < total_items;
	 lcv++)
    {
      color->ori_red[lcv] = ints[indx++];
      color->ori_green[lcv] = ints[indx++];
      color->ori_blue[lcv] = ints[indx++];
      color->color_status[lcv] = ints[indx++];
    }
  }
  else
  {
    PRT1(printf("	Overriding system colors.\n"););
    for (lcv = 0,indx = 0; lcv < total_items; lcv++)
    {
      color->ori_red[lcv] = ints[indx++];
      color->ori_green[lcv] = ints[indx++];
      color->ori_blue[lcv] = ints[indx++];
      color->color_status[lcv] = ints[indx++];
    }
  }


  c_free(ints);



  total_items = 2;
  err = RdText(fid,ary,&total_items,RD_INT,0);
  if (err != NO_ERROR)
    return err;

  TRACE(printf("	Number of scales: %d\n", ary[0]);
	printf("	Current scale: %d\n", ary[1]););
  if ((total_items != 2) || (ary[0] <= 0))
    return CANCEL;

  if ((ary[1] > ary[0]) || (ary[1] < 1))
    ary[1] = 1;
  /*  Human readable starts at 1.  Computer internal starts at 0.  */
  ary[1] -= ordinal;

  if ((err = get_ptr(&(intensity->scales),&(intensity->total_scales),ary[0],
		     sizeof(*(intensity->scales)))) != NO_ERROR)
    return err;
    

  intensity->num_scales = intensity->total_scales;
  intensity->cur_scale = ary[1];

  for (lcv = 0; lcv < intensity->num_scales; lcv++)
  {
    init_color_scale_rec(&(intensity->scales[lcv]));

    total_items = sizeof(str) - 1;
    NOPRT(printf("rd_intensity_rec...5\n"));
    if ((err = RdText(fid,str,&total_items,RD_TEXT,0)) != NO_ERROR) 
      return err;
    trim_str_blanks(str);
    if ((err = cp_str(&(intensity->scales[lcv].label),str)) != NO_ERROR)
      return err;
    PRT(printf("	scale label: \"%s\"\n", intensity->scales[lcv].label));

    /*
     *  For each color scale, read in:
     *
     *  1) Label (above),
     *  2) First color index,
     *  3) Number of colors used, starting from the first color index, (?)
     *  4) Lower bound of range,
     *  5) Upper bound of range,
     *  6) Current (?) medium color (?) index,
     *  7) Range of values to be shown starting from mid-value as mid-point
     */
    total_items = 6;
    if ((err = RdText(fid,ary,&total_items,RD_INT,0)) != NO_ERROR)
      return err;
    PRT(printf("	First color index: %d, ", ary[0]);
	printf("	Number of colors used, starting from 1st indx: %d\n",
	       ary[1]););

    if (total_items != 6)
      return CANCEL;

    /*  Human readable starts at 1.  Computer internal starts at 0.  */
    ary[0] -= ordinal;

    if (ary[0] >= color->total_colors)
      return CANCEL;
    if ((ary[0] + ary[1]) > color->total_colors)
      return CANCEL;

    set_color_scale_rec_control(&(intensity->scales[lcv]),
				ary[0],	/* First color index */
				ary[1],	/* Number of colors */
				ary[2],	/* Lower bound of range */
				ary[3],	/* Upper bound of range */
				ary[4],	/* Middle color index */
				ary[5]); /* Range of values */
    NOPRT(printf("rd_intensity_rec...8\n"));
    define_first_visible(&(intensity->color),&(intensity->scales[lcv]));
  }
  map_all_scale_rgb(&(intensity->color));
  for (lcv = 0; lcv < intensity->num_scales; lcv++)
    define_actual_intensity_scale(intensity,0,lcv);
  color->status &= ~RGB_INVALID_COLOR_FILE;
  color->status |= RGB_COLORS_VALID;

  TRACE(pr_intensity_rec(intensity););
  NOPRT(printf("rd_intensity_rec err = %d\n",err));
  return err;
}



/*----------------------------------------------------------------------
init_program_fore_and_background_colors - ???

FIXME:  Body is empty!
----------------------------------------------------------------------*/

void
init_program_fore_and_background_colors(color)
     struct color_rec *color;
{
#if 0
  /* workspace color */
  color->ori_red[0] = 0;
  color->ori_green[0] = 255; 
  color->ori_blue[0] = 255; 

  /* window background color */
  color->ori_red[color->num_system_colors] = 64;
  color->ori_green[color->num_system_colors] = 160; 
  color->ori_blue[color->num_system_colors] = 192; 

  /* window foreground color */
  color->ori_red[color->num_system_colors + 1] = 255;
  color->ori_green[color->num_system_colors + 1] = 255; 
  color->ori_blue[color->num_system_colors + 1] = 0;
#endif
}


/*----------------------------------------------------------------------
init_program_colors - ???
----------------------------------------------------------------------*/

/*
#define TOTAL_PROGRAM_COLORS 7
*/

void
init_program_colors(color)
     struct color_rec *color;
{
  int lcv;


  /*
   * colors that are used internally to draw lines...
   */

  lcv = color->total_colors - TOTAL_PROGRAM_COLORS;

  /*
   *
   *  Orange (?)
   */
  color->ori_red[lcv] = 250;
  color->ori_green[lcv] = 177; 
  color->ori_blue[lcv] = 0;
  lcv++;

  /*
   *
   *  Light aqua (?)
   */
  color->ori_red[lcv] = 155;
  color->ori_green[lcv] = 199;
  color->ori_blue[lcv] = 244;
  lcv++;

  /*
   *
   *  Yellow
   */
  color->ori_red[lcv] = 255;
  color->ori_green[lcv] = 255; 
  color->ori_blue[lcv] = 0; 
  lcv++;

  /*
   *  Non-current regions and ROI's
   *  Red
   */
  color->ori_red[lcv] = 250;
  color->ori_green[lcv] = 0;
  color->ori_blue[lcv] = 0; 
  lcv++;

  /*
   *  Current ROI, non-current region
   *  Blue
   */
  color->ori_red[lcv] = 0;
  color->ori_green[lcv] = 0;
  color->ori_blue[lcv] = 200;
  lcv++;

  /*
   *  Current ROI, current region
   *  Green
   */
  color->ori_red[lcv] = 0;
  color->ori_green[lcv] = 255;
  color->ori_blue[lcv] = 0;
  lcv++;

  /*
   *
   *  Purple
   */
  color->ori_red[lcv] = 255;
  color->ori_green[lcv] = 0;
  color->ori_blue[lcv] = 255;
}



/*----------------------------------------------------------------------
setup_default_colors - ???
----------------------------------------------------------------------*/

int
setup_default_colors(intensity,system_colors,num_system_colors)
     struct intensity_rec *intensity;
     Xv_singlecolor *system_colors;
     unsigned long num_system_colors;
{
  struct color_rec *color;
  unsigned long total_items;
  int lcv,err,indx;

  color = &(intensity->color);
  TRACE(printf("setup_default_colors...\n"));
  if ((err = cp_str(&(intensity->label),"GRAY SCALE")) != NO_ERROR)
    return err;
  TRACE(printf("setup_default_colors...1\n"));
  
  if ((err = get_rgb_ptr_memory(color,(unsigned long) 256)) != NO_ERROR)
    return err;
  TRACE(printf("setup_default_colors...total_scales = %d\n",intensity->total_scales));
  if ((err = get_ptr(&(intensity->scales),&(intensity->total_scales),1,
		     sizeof(*(intensity->scales)))) != NO_ERROR)
    return err;
  intensity->num_scales = intensity->total_scales;
  init_color_scale_rec(&(intensity->scales[0]));
  TRACE(printf("setup_default_colors...0..1\n"));
  if ((err = cp_str(&(intensity->scales[0].label),"GRAY SCALE")) != NO_ERROR)
    return err;

  color->use_system_colors = 1;
  color->num_system_colors = num_system_colors;
  color->total_colors = 236;
  init_program_fore_and_background_colors(color);
  TRACE(printf("setup_default_colors...1\n"));

  for (lcv = 0;lcv < color->num_system_colors; lcv++)
  {
    color->ori_red[lcv] = system_colors[lcv].red;
    color->ori_green[lcv] = system_colors[lcv].green;
    color->ori_blue[lcv] = system_colors[lcv].blue;
  }

  intensity->scales[0].first = color->num_system_colors + 3;
  intensity->scales[0].first_visible = intensity->scales[0].first;
  lcv = intensity->scales[0].first + TOTAL_PROGRAM_COLORS;
  if (color->total_colors > lcv) 
    intensity->scales[0].num_colors = color->total_colors - lcv;
  else
  {
    perror("Not enough colors...");
    TRACE(printf("Not enough colors...total_colors = %d lcv = %d\n",
		 color->total_colors,lcv););
    exit(0);
  }
  TRACE(printf("setup_default_colors...first = %d\n",
	       intensity->scales[0].first));
  total_items = intensity->scales[0].first + intensity->scales[0].num_colors;
  for (indx = 0, lcv = intensity->scales[0].first; lcv < total_items; lcv++)
    color->ori_red[lcv] =
      color->ori_green[lcv] =
      color->ori_blue[lcv] =
      indx++;

  init_program_colors(color);
  TRACE(printf("setup_default_colors...3\n"));
  def_color_scale_rec_control(&(intensity->scales[0]));

  TRACE(printf("setup_default_colors...4\n"));
  color->status |= RGB_COLORS_VALID | RGB_DEFAULT_COLORS;
  TRACE(printf("setup_default_colors...5\n"));
  map_all_scale_rgb(&(intensity->color));
  define_actual_intensity_scale(intensity,0,0);
  TRACE(printf("setup_default_colors...6\n"));
  return NO_ERROR;
}



/*----------------------------------------------------------------------
rd_pallette_rec - ???

Attempt to read in a colormap file (default color.dat) and set
a pallette to the assigned values.
----------------------------------------------------------------------*/

int
rd_pallette_rec( fn, def_color )
     char *fn;
     struct pallette_rec *def_color;
{
  int ary[2];
  unsigned long total_items, lcv;
  FILE *fid;
  int err;
  int ordinal = 0;


  free_pallette_rec(def_color);

  if (fn == NULL)
    return FILE_NAME_ERROR;

  if ((fid = fopen( fn, "r" )) == NULL)
  {
    fprintf( stderr, "NRIA: Could not open the color file \"%s\"\n", fn );
    return RGB_INVALID_COLOR_FILE;
  }


  /*
   *  Old style: numbering system begins with zero, which is confusing
   *    for humans.  In old style color.dat files, the first character
   *    is a digit.
   *
   *  Current style: numbering system begins with one, which is a more
   *    natural way to represent ordinals.  In current style color.dat
   *    files, the first character is always a pound sign ('#').
   */
  if ((ordinal = fgetc( fid )) != EOF) {
    ungetc( ordinal, fid );
    if (isdigit(ordinal))
      ordinal = 0;
    else
      ordinal = 1;
  }


  /*
   *  The first two integers are
   *  1)  Number of 'bundles' or pallettes
   *  2)  Current pallette (at start of program session)
   */

  total_items = 2;
  err = RdText(fid,ary,&total_items,RD_INT,0);

  if ((total_items == 2) && !err && (ary[0] > 0))
  {
    TRACE(printf("rd_pallette_rec(): # bundles = %d\n",ary[0]));
    lcv = 0;

    /*
     *  Get memory to store ary[0] number of bundles
     */
    if ((err = get_handle(&(def_color->pallettes),
			  &lcv,
			  (unsigned long) ary[0],
			  (size_t) sizeof(**(def_color->pallettes))))
	!= NO_ERROR)
    {
      fclose(fid);
      return err;
    }
    if ((err = cp_str(&(def_color->color_fn),fn)) != NO_ERROR)
    {
      free_pallette_rec(def_color);
      fclose(fid);
      return err;
    }
    NOPRT(printf("rd_pallette_rec ready...\n"));

    def_color->total_pallettes = def_color->num_pallettes = ary[0];

    /*  Human readable starts at 1.  Computer internal starts at 0.  */
    def_color->cur_pallette = ary[1] - ordinal;

    /* Sanity check */
    if (def_color->cur_pallette >= def_color->num_pallettes)
      def_color->cur_pallette = 0;

    for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
      init_intensity_rec(def_color->pallettes[lcv]);

    for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
    {
      if ((err = rd_intensity_rec( fid, def_color->pallettes[lcv],
				   ordinal )) !=
	  NO_ERROR)
      {
	free_pallette_rec(def_color);
	break;
      }
    }

  }
  else
    err = BAD_DATA;
  fclose(fid);
  NOPRT(printf("rd_pallette_rec end...err = %d\n",err));

  return err;
}



/*----------------------------------------------------------------------
setup_default_intensity - ???
----------------------------------------------------------------------*/

int
setup_default_intensity(def_color,system_colors,num_system_colors)
     struct pallette_rec *def_color;
     Xv_singlecolor *system_colors;
     unsigned long num_system_colors;
{
  int err;

  free_pallette_rec(def_color);
  if ((err = get_handle(&(def_color->pallettes),
			&(def_color->total_pallettes),1,
			(size_t) sizeof(**(def_color->pallettes))))
      != NO_ERROR)
    return err;
  def_color->num_pallettes = 1;
  init_intensity_rec(def_color->pallettes[0]);
  if ((err = setup_default_colors(def_color->pallettes[0],
				  system_colors,num_system_colors))
      != NO_ERROR)
  {
    free_pallette_rec(def_color);
  }
  TRACE(printf("setup_default_intensity...err = %d\n",err));
  return err;
}



/*----------------------------------------------------------------------
setup_pallette_rec - ???
----------------------------------------------------------------------*/

int
setup_pallette_rec( fn, def_color, system_colors, num_system_colors )
     char *fn;
     struct pallette_rec *def_color;
     Xv_singlecolor *system_colors;
     unsigned long num_system_colors;
{
  int err;

  TRACE(printf("setup_pallette_rec()\n"););

  if ((err = rd_pallette_rec( fn, def_color )) != NO_ERROR)
  {
    printf("Error reading the color file \"%s\"\n",fn);
    printf("Proceed with defining the default intensity...\n");
    if ((err = setup_default_intensity( def_color,
					system_colors,
					num_system_colors )) != NO_ERROR)
      printf("Error defining the default intensity...err = %d\n",err);
  }
  else
  {
    TRACE(printf("	Read in color file \"%s\"\n", fn););
  }

  return err;
}



/*----------------------------------------------------------------------
check_save_color - ???
----------------------------------------------------------------------*/

int
check_save_color(def_color)
     struct pallette_rec *def_color;
{
  unsigned long lcv,cnt;
  struct intensity_rec *intensity;

  for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
  {
    intensity = def_color->pallettes[lcv];
    for (cnt = 0; cnt < intensity->num_scales; cnt++)
      if (intensity->scales[cnt].status & COLOR_CONTROL_UPDATED)
	return 1;
  }
  return 0;
}



/*----------------------------------------------------------------------
set_cur_color_scale_save_status - ???
----------------------------------------------------------------------*/

void
set_cur_color_scale_save_status(intensity,status,is_and)
     struct intensity_rec *intensity;
     unsigned short status;
     int is_and;
{
  if (is_and)
    intensity->scales[intensity->cur_scale].status &= status;
  else
    intensity->scales[intensity->cur_scale].status |= status;
}



/*----------------------------------------------------------------------
wr_intensity_rec - ???
----------------------------------------------------------------------*/

int
wr_intensity_rec(fid,intensity)
     FILE *fid;
     struct intensity_rec *intensity;
{
  unsigned long lcv;
  struct color_rec *color;
  char descrip[200];
#define numdescrips 2
  char *descrips[numdescrips] = {
    "Background",
    "Foreground"
  };
#define numenddescrips 4
  char *enddescrips[numenddescrips] = {
    "Inactive ROIs, Inactive Regions",
    "Active ROI, Inactive Regions",
    "Active ROI, Active Region",
    "???"
  };

  color = &(intensity->color);
  fprintf( fid, "#  Name\n" );
  fprintf( fid, "\"%s\"\n", intensity->label );

  fprintf( fid, "#  Total number of colors\n" );
  fprintf( fid, "%ld\n", color->total_colors );

  fprintf( fid, "#  Use system colors?  1 = yes, 0 = no\n" );
  fprintf( fid, "%d\n", color->use_system_colors );

  fprintf( fid, "#  Number of system colors\n" );
  fprintf( fid, "%ld\n", color->num_system_colors );

  fprintf( fid, "#  Reserve color start (?)\n" );
  fprintf( fid, "%ld\n",color->reserve_color_start );

  fprintf( fid, "\n##########\n#  Colormap\n#\n" );
  fprintf( fid, "#  Field 1, 2, 3: Red, Green, Blue\n" );
  fprintf( fid, "#  Field 4: Status (?)\n" );
  fprintf( fid, "#  Status & 1 == BACKGROUND_COLOR\n" );
  fprintf( fid, "#  Status & 2 == FOREGROUND_COLOR\n\n" );

  for (lcv = 0; lcv < color->total_colors;lcv++)
  {
    if (lcv < numdescrips)
      sprintf( descrip, "\t#  %s", descrips[lcv] );
    else if (lcv >= color->total_colors - numenddescrips)
      sprintf( descrip, "\t#  %s",
	       enddescrips[lcv - (color->total_colors - numenddescrips)] );
    else
      sprintf( descrip, "\t#  Color %ld", lcv + 1 );
    fprintf( fid, "%03ld	%03ld	%03ld	%1ld%s\n",
	     (unsigned long) color->ori_red[lcv],
	     (unsigned long) color->ori_green[lcv],
	     (unsigned long) color->ori_blue[lcv],
	     (unsigned long) color->color_status[lcv],
	     descrip );
  }
  fprintf( fid, "\n" );

  fprintf( fid, "###########\n#  Individual scales within this pallette\n#\n" );
  fprintf( fid, "#  Number of scales\n" );
  fprintf( fid, "%ld\n", intensity->num_scales );
  fprintf( fid, "#  Default scale index\n" );
  fprintf( fid, "%ld\n", intensity->cur_scale + 1 );
  fprintf( fid, "\n" );

  fprintf( fid, "# LOWER BOUND <= MEDIUM <= UPPER BOUND\n" );
  fprintf( fid, "# FIRST COLOR INDEX <= LOWER BOUND <= UPPER BOUND\n" );
  fprintf( fid, "# LOWER BOUND <= UPPER BOUND < FIRST COLOR INDEX + # OF COLORS USED\n" );
  fprintf( fid, "\n" );


  for (lcv = 0; lcv < intensity->num_scales; lcv++)
  {
    fprintf( fid, "######\n#  Scale definition no. %ld\n#\n", lcv + 1 );
    fprintf( fid, "#  Name of scale\n" );
    fprintf( fid, "\"%s\"\n", intensity->scales[lcv].label );
    
    fprintf( fid, "#  First color index (F.C.I.)\n" );
    fprintf( fid, "%ld\n" ,intensity->scales[lcv].first + 1);

    fprintf( fid, "#  Number of colors used starting from F.C.I.\n" );
    fprintf( fid, "%ld\n", intensity->scales[lcv].num_colors );

    fprintf( fid, "#  Range: lower and upper bounds\n" );
    fprintf( fid, "%d\n", intensity->scales[lcv].lb );
    fprintf( fid, "%d\n", intensity->scales[lcv].ub );

    fprintf( fid, "#  Current medium color index\n" );
    fprintf( fid, "%d\n", intensity->scales[lcv].mid_value );

    fprintf( fid, "#  Range of values to be shown starting from\n" );
    fprintf( fid, "#    mid_value as the mid-point\n" );
    fprintf( fid, "%d\n",intensity->scales[lcv].range );

    fprintf( fid, "#\n#  End of scale no. %ld\n#####\n\n", lcv + 1 );
  }

  return NO_ERROR;
}



/*----------------------------------------------------------------------
wr_pallette_rec - ???
----------------------------------------------------------------------*/

int
wr_pallette_rec(fn,def_color)
     char *fn;
     struct pallette_rec *def_color;
{
  unsigned long lcv;
  FILE *fid;
  int err;
  time_t currtime;

  if (fn == NULL)
    return FILE_NAME_ERROR;

  if ((fid = fopen(fn,"w")) == NULL)
  {
    TRACE(printf("\nCould not open \"%s\" for writing color info.\n",fn) );
    return RGB_INVALID_COLOR_FILE;
  }
  clearerr(fid);

  fprintf( fid, "#  This file is written out automatically by NRIA.\n" );
  fprintf( fid, "#  Do not edit this file while NRIA is running.\n" );
  time( &currtime );
  fprintf( fid, "#  Last saved on: %s\n", ctime( &currtime ) );

  fprintf( fid, "#  Number of different pallettes in this file\n" );
  fprintf( fid, "%ld\n\n", def_color->num_pallettes );

  fprintf( fid, "#  Default pallette ID\n" );
  fprintf( fid, "%ld\n\n", def_color->cur_pallette + 1);

  for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
  {
    fprintf( fid, "##################################################\n" );
    fprintf( fid, "#  Pallette no. %ld\n#\n", lcv + 1 );
    if ((err = wr_intensity_rec( fid, def_color->pallettes[lcv] )) !=
	NO_ERROR)
    {
      fclose(fid);
      return err;
    }
    fprintf( fid, "#\n#  End of pallette no. %ld\n", lcv + 1 );
    fprintf( fid, "##################################################\n\n\n" );
  }
  err = ferror(fid);
  fclose(fid);
  return err;
}



/*----------------------------------------------------------------------
find_scale - ???
----------------------------------------------------------------------*/

struct color_scale_rec *
find_scale(intensity,label,set_cur)
     struct intensity_rec *intensity;
     char *label;
     int set_cur;
{
  unsigned long lcv;

  for (lcv = 0; lcv < intensity->num_scales; lcv++)
  {
    if (str_cmp(intensity->scales[lcv].label,label) == 0)
    {
      if (set_cur)
	intensity->cur_scale = lcv;
      return &(intensity->scales[lcv]);
    }
  }
  return NULL;
}



/*----------------------------------------------------------------------
find_intensity_rec - ???
----------------------------------------------------------------------*/

struct intensity_rec *
find_intensity_rec(def_color,intensity_label,set_cur)
     struct pallette_rec *def_color;
     char *intensity_label;
     int set_cur;
{
  unsigned long lcv;
  struct intensity_rec *intensity;

  for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
  {
    intensity = def_color->pallettes[lcv];
    if (no_case_str_cmp(intensity->label,intensity_label) == 0)
    {
      if (set_cur)
	def_color->cur_pallette = lcv;
      return intensity;
    }
  }
  return NULL;
}



/*----------------------------------------------------------------------
get_intensity_rec - ???
----------------------------------------------------------------------*/

struct intensity_rec *
get_intensity_rec(def_color,intensity_label,set_cur)
     struct pallette_rec *def_color;
     char *intensity_label;
     int set_cur;
{
  struct intensity_rec *intensity;
  unsigned long cur;

  cur = def_color->cur_pallette;
  intensity = find_intensity_rec(def_color,intensity_label,1);
  if (intensity  == NULL)
  {
    if (def_color->num_pallettes == 0)
      return NULL;
    else
    {
      if (def_color->cur_pallette >= def_color->num_pallettes)
	def_color->cur_pallette = 0;
      intensity = def_color->pallettes[def_color->cur_pallette];
    }
  }
  else if (!set_cur)
    def_color->cur_pallette = cur;

  return intensity;
}


/********************
main(argc,argv)
 int argc;
 char *argv[];
{
 struct pallette_rec def_color;
 struct intensity_rec intensity;
 int err,lcv;

 if (setup_pallette_rec(argv[1],&def_color) == NO_ERROR)
    {
     for (lcv = 0; lcv < def_color.num_pallettes; lcv++)
        {printf("color #%d\n",lcv);
         copy_intensity_rec(&intensity,def_color.pallettes[lcv],
                  RGB_FREE_BUFFERS);
         pr_color_rec(&(intensity.color));
         free_color_rec(&(intensity.color));
        }
     wr_pallette_rec("color.dat1",&def_color);
     free_pallette_rec(&def_color);

    }

}
******************/


#if 0
init_color_scale_control_rec(control)
 struct color_scale_control_rec *control;
{
 control->status = 0;
}
#endif
