/****************************************************************************
color_opr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: color_opr.c,v 1.8 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"



/* Functions defined in this file */
int WinFindLocalAndGlocalScale(struct image_display_window *win,
    struct color_scale_rec **local, struct color_scale_rec **global);
struct color_scale_rec *win_cur_scale(struct image_display_window *win);
int win_first_and_num_colors(struct image_display_window *win,
    unsigned long *first, unsigned long *num_colors);
int CurColorIndxs(struct intensity_rec *intensity,
    struct color_data_rec *color_data, unsigned long **color_indxs,
    unsigned long *num_color_indxs);
int WinCurScaleAndColorIndxs(struct image_display_window *win,
    struct color_data_rec *color_data, struct color_scale_rec **scale,
    unsigned long **color_indxs, unsigned long *num_color_indxs);
int WinSetDefaultScaleLabels(struct image_display_window *win,
    struct image_display_window *def_win);
int WinDefaultColorScaleLabels(struct image_display_window *win);
int WinDefaultColorScale(struct image_display_window *win,
    struct color_data_rec *color_data);
int WinEnableDisableAllLocalContrast(struct image_display_window *win,
    int is_enable, int is_setup);
int WinSelectScale(struct image_display_window *win,
    unsigned long color_data_indx);
int WinEnableGlobalLocalContrast(struct image_display_window *win, int how);
int win_set_cur_scale(struct image_display_window *win,
    unsigned long cur_scale);
int WinSetCurColorDataByScaleLabel(struct image_display_window *win,
    char *scale_label, int option);
int WinSetDefaultColorDataScale(struct image_display_window *win,
    char *scale_label);
int win_cur_scale_is_default(struct image_display_window *win);
int WinSetupScaleLabels(struct image_display_window *win,
    struct strs_rec *labels);
void win_update_color_opr(struct image_display_window *win);
int win_set_color_map(struct image_display_window *win, int use_original);
int win_set_color_by_intensity_rec(struct image_display_window *win,
    struct intensity_rec *intensity, int is_to_all_window);
void win_change_color_scale(struct image_display_window *win,
    char *intensity_label, char *scale_label);
int win_set_color_by_new_intensity_rec(struct image_display_window *win,
    struct intensity_rec *intensity, int is_create_new, int is_to_all_window);
int win_set_color_by_intensity_label(struct image_display_window *win,
    char *intensity_label, int is_to_all_window);
int win_define_color_map(struct image_display_window *win,
    int is_GT_ub_background);
int win_set_related_color_maps(struct image_display_window *win,
    unsigned long *color_indxs);
int win_image_enhancement(struct image_display_window *win);
int WinSetMidContrast(struct image_display_window *win, char *scale_label,
    unsigned long scale_indx, int value, int is_GT_ub_background);
int WinSetRangeContrast(struct image_display_window *win, char *scale_label,
    unsigned long scale_indx, int value, int is_GT_ub_background);
int WinSetContrastLbUb(struct image_display_window *win, char *scale_label,
    unsigned long scale_indx, long lb, long ub, long is_GT_ub_background);
int win_local_color_indxs(struct image_display_window *win,
    unsigned long *color_indxs);
int WinDefaultLocalColorIndxs(struct image_display_window *win,
    struct image_display_window *def_win, struct color_data_rec *color_data);
int win_setup_default_local_color_indxs(struct image_display_window *win,
    struct image_display_window *def_win);
int win_actual_color_indx(struct image_display_window *win,
    unsigned long color_indx);
struct image_display_window *WinFindFirstWinOfThisScale(
   struct image_display_window *win, char *scale_label);
int win_same_local_contrast(struct image_display_window *win,
    struct image_display_window *def_win);
int WinSet2dByColorValueRange(struct image_display_window *win,
    struct buf_2d_rec *_2d, struct buf_2d_rec *mask,
    unsigned long first_color_val, unsigned long last_color_val);
int Set2dBackgroundColor(struct buf_2d_rec *result, struct buf_2d_rec *_2d,
    struct buf_2d_rec *mask, struct color_rec *color,
    unsigned long *color_indxs, unsigned long num_indxs,
    unsigned long first_indx, double setval, unsigned long select);
int WinActualColor2d(struct image_display_window *win, struct buf_2d_rec *_2d);
int WinMapActualColorData(struct image_display_window *win,
    struct color_data_rec *cur_color_data, struct buf_2d_rec *color_data_2d);



/*----------------------------------------------------------------------
WinFindLocalAndGlocalScale - ???
----------------------------------------------------------------------*/

int
WinFindLocalAndGlocalScale(win,local,global)
     struct image_display_window *win;
     struct color_scale_rec **local,**global;
{
  if (win->data.num_color_datas == 0) 
  {
    *global = *local = &(win->intensity->scales[win->intensity->cur_scale]);
  }
  else
  {
    *local = &(win->data.color_datas[win->data.cur_color_data]->scale);
    if ((*global = find_scale(win->intensity,(*local)->label,0)) == NULL)
      return CANCEL;
  }
  PRT3(printf("WinFindLocalAndGlocalScale cur_color_data = %d\n",
	       win->data.cur_color_data));
  return NO_ERROR;
}




/*----------------------------------------------------------------------
win_cur_scale - ???
----------------------------------------------------------------------*/

struct color_scale_rec *
win_cur_scale(win)
     struct image_display_window *win;
{
  struct color_scale_rec *local,*global;

  if (WinFindLocalAndGlocalScale(win,&local,&global) != NO_ERROR)
    return NULL;
  if (global->status & COLOR_LOCAL)
    return local;
  else
    return global;
}




/*----------------------------------------------------------------------
win_first_and_num_colors - ???
----------------------------------------------------------------------*/

int
win_first_and_num_colors(win,first,num_colors)
     struct image_display_window *win;
     unsigned long *first,*num_colors;
{
  struct color_scale_rec *scale;

  if ((scale = win_cur_scale(win)) == NULL)
    return CANCEL;
  *first = scale->first;
  *num_colors = scale->num_colors;
  return NO_ERROR;
}



/*----------------------------------------------------------------------
CurColorIndxs - ???
----------------------------------------------------------------------*/

int
CurColorIndxs(intensity,color_data,color_indxs,num_color_indxs)
     struct intensity_rec *intensity;
     struct color_data_rec *color_data;
     unsigned long **color_indxs,*num_color_indxs;
{
  PRT2(printf("CurColorIndxs...status = %x num_color_indxs = %d %d\n",
	       (unsigned long) color_data->scale.status,
	       color_data->num_color_indxs,color_data->scale.num_colors));
  if ((color_data->scale.status & COLOR_LOCAL) &&
      (color_data->num_color_indxs == color_data->scale.num_colors))
  {
    PRT2(printf("CurColorIndxs...local\n"));
    *color_indxs = color_data->color_indxs;
    *num_color_indxs = color_data->num_color_indxs;
  }
  else 
  {
    PRT2(printf("CurColorIndxs...global\n"));
    *color_indxs = &(intensity->color.color_indxs[color_data->scale.first]);
    *num_color_indxs = color_data->scale.num_colors;
  }
  return NO_ERROR;
}




/*----------------------------------------------------------------------
WinCurScaleAndColorIndxs - ???
----------------------------------------------------------------------*/

int
WinCurScaleAndColorIndxs(win,color_data,scale,color_indxs,num_color_indxs)
     struct image_display_window *win;
     struct color_data_rec *color_data;
     struct color_scale_rec **scale;
     unsigned long **color_indxs,*num_color_indxs;
{
  struct color_data_rec *color_data1;

  if ((*scale = win_cur_scale(win)) == NULL) return CANCEL;
  if ((*scale)->status & COLOR_LOCAL)
  {
    if (color_data != NULL)
      color_data1 = color_data;
    else
      color_data1 = win->data.color_datas[win->data.cur_color_data];
    *scale = &(color_data1->scale);
    *color_indxs = color_data1->color_indxs;
    *num_color_indxs = color_data1->num_color_indxs;
    if (*num_color_indxs != (*scale)->num_colors)
      return CANCEL;
  }
  else 
  {
    if (color_data != NULL)
      *scale = &(color_data->scale);

    *color_indxs = &(win->intensity->color.color_indxs[(*scale)->first]);
    *num_color_indxs = (*scale)->num_colors;
  }
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinSetDefaultScaleLabels - ???
----------------------------------------------------------------------*/

int
WinSetDefaultScaleLabels(win,def_win)
     struct image_display_window *win,*def_win;
{
  struct color_data_rec *color_data;
  unsigned long indx;

  PRT(printf("WinSetDefaultScaleLabels...\n"));
  if ((WinFindPosNegColorDataRec(def_win,&color_data,&indx,1) == NO_ERROR) &&
      (color_data->scale.label != NULL))
  {
    c_free(win->data.def_pos_scale_label);
    PRT(printf("WinSetDefaultScaleLabels...pos = %s\n",color_data->scale.label));
    cp_str(&(win->data.def_pos_scale_label),color_data->scale.label);
  }

  if (def_win->status4 & WIN_NEG_ACTIVATION)
  {
    if ((WinFindPosNegColorDataRec(def_win,&color_data,&indx,0) == NO_ERROR) &&
	(color_data->scale.label != NULL))
    {
      c_free(win->data.def_neg_scale_label);
      PRT(printf("WinSetDefaultScaleLabels...neg = %s\n",color_data->scale.label));
      cp_str(&(win->data.def_neg_scale_label),color_data->scale.label);
    }
  }
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinDefaultColorScaleLabels - ???
----------------------------------------------------------------------*/

int
WinDefaultColorScaleLabels(win)
     struct image_display_window *win;
{
  c_free(win->data.def_pos_scale_label);
  c_free(win->data.def_neg_scale_label);
  if (win->status4 & WIN_POS_NEG_ACTIVATION)
  {
    cp_str(&(win->data.def_pos_scale_label),"HOT METAL");
    cp_str(&(win->data.def_neg_scale_label),"COLD METAL");
  }
  else
    cp_str(&(win->data.def_pos_scale_label),"GRAY SCALE");
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinDefaultColorScale - ???
----------------------------------------------------------------------*/

int
WinDefaultColorScale(win,color_data)
     struct image_display_window *win;
     struct color_data_rec *color_data;
{
  struct color_scale_rec *scale;
  int err;

  PRT1(printf("WinDefaultColorScale\n"));
  scale = NULL;

  if ((color_data->status4 & COLOR_DATA_NEG_ACTIVATION) &&
      (win->data.def_neg_scale_label != NULL))
    scale = find_scale(win->intensity,win->data.def_neg_scale_label,0);

  else if (win->data.def_pos_scale_label != NULL)
    scale = find_scale(win->intensity,win->data.def_pos_scale_label,0);

  if (scale == NULL)
    scale = &(win->intensity->scales[win->intensity->cur_scale]);

  if ((err = cp_color_scale_rec(&(color_data->scale),scale)) == NO_ERROR)
    if (win->intensity->scales[win->intensity->cur_scale].status & COLOR_LOCAL)
      err = WinDefaultLocalColorIndxs(win,NULL,color_data);

  PRT1(printf("WinDefaultColorScale...err = %d status = %x label = %s num_color_indxs = %d\n",err,
	       (unsigned long) color_data->scale.status,
	       color_data->scale.label,
	       color_data->num_color_indxs));
  return err;
}



/*----------------------------------------------------------------------
WinEnableDisableAllLocalContrast - ???
----------------------------------------------------------------------*/

int
WinEnableDisableAllLocalContrast(win,is_enable,is_setup)
     struct image_display_window *win;
     int is_enable,is_setup;
{
  unsigned long lcv,*color_indxs;
  struct color_scale_rec *local,*global;
  int err;

  PRT(printf("win_enable_local_contrast wid = %s is_enable = %d is_setup = %d\n",
	     win->owner->wid,is_enable,is_setup));
  if (is_setup)
  {
    if ((global = &(win->intensity->scales[win->intensity->cur_scale])) == NULL)
      return CANCEL;
  }
  else if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return CANCEL;

  color_indxs = &(win->intensity->color.color_indxs[global->first]);
  for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
  {
    local = &(win->data.color_datas[lcv]->scale);
    PRT(printf("%d) is_setup = %d local->label = %s global->label = %s\n",lcv,
	       is_setup,local->label,global->label));
    if (is_setup || (str_cmp(local->label,global->label) == 0))
    {
      if (is_enable)
      {
	local->status |= COLOR_LOCAL;
	if ((err = SetupLocalScale(win->data.color_datas[lcv],global,color_indxs,
				   global->num_colors)) != NO_ERROR)
	  return err;
	PRT(printf("%d) enabled\n",lcv));
      }
      else
      {
	if (is_setup)
	  if ((err = cp_color_scale_rec(local,global)) != NO_ERROR)
	    return err;
	local->status &= ~COLOR_LOCAL;
	win->data.color_datas[lcv]->num_color_indxs = 0;
	PRT(printf("%d) disabled\n",lcv));
      }
    }
  }
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinSelectScale - ???
----------------------------------------------------------------------*/

int
WinSelectScale(win,color_data_indx)
     struct image_display_window *win;
     unsigned long color_data_indx;
{
  struct color_scale_rec *scale;
  struct color_data_rec *color_data;
  unsigned long *color_indxs;
  int err;

  PRT(printf("WinSelectScale wid = %s color_data_indx = %d\n",win->owner->wid,color_data_indx));
  if (color_data_indx < win->data.num_color_datas)
  {
    color_data = win->data.color_datas[color_data_indx];
    scale = &(color_data->scale);
    if (scale->status & COLOR_LOCAL)
    {
      if ((color_data->total_color_indxs < scale->num_colors) ||
	  (color_data->num_color_indxs < scale->num_colors))
      {
	color_indxs = &(win->intensity->color.color_indxs[scale->first]);
	if ((err = SetupLocalScale(color_data,scale,color_indxs,
				   scale->num_colors)) != NO_ERROR)
	  return err;
      }
      PRT(printf("WinSelectScale enabled\n"));
    }
    else
    {
      color_data->num_color_indxs = 0;
    }
    win->data.cur_color_data = color_data_indx;
    c_free(win->data.scale_label);
    cp_str(&(win->data.scale_label),color_data->scale.label);
    PRT(pr_color_scale_rec(&(color_data->scale)););
  }
  PRT(printf("WinSelectScale end\n"));
  win_update_cur_contrast_settings(win);
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinEnableGlobalLocalContrast - ???
----------------------------------------------------------------------*/

int
WinEnableGlobalLocalContrast(win,how)
     struct image_display_window *win;
     int how;
{
  int redisplay,err;
  struct color_scale_rec *local,*global;

  PRT(printf("\nWinEnableGlobalLocalContrast begin\n"));
  if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return err;
  if (how != 0)
  {
    global->status |= COLOR_LOCAL;   
    if ((err =
	 win_run_proc_to_all_file_wins( win, WinEnableDisableAllLocalContrast,
					0x000000A3,1,0,0)) == NO_ERROR)
    {
      unmap_scale_rgb(&(win->intensity->color),global);
      ChangeExistingColorSegment(win->dwin->frame,win->intensity->label,
				 &(win->intensity->color),0,&redisplay);
    }
    PRT(printf("after win_enable_local_contrast err = %d\n",err));
  }
  if ((how == 0) || (err != NO_ERROR))
  {
    global->status &= ~COLOR_LOCAL;   
    win_run_proc_to_all_file_wins( win, WinEnableDisableAllLocalContrast,
				   0x000000A3,0,0,0);
    PRT(printf("after win_disable_local_contrast\n"));
    win_define_color_map(win,-1);
  }
  
  win_run_proc_to_all_file_wins(win,WinMultiScreenCanvas,0x000000A3,0,0,0);

  win_update_color_opr(win);
  PRT(printf("WinEnableGlobalLocalContrast end\n\n"));
  return NO_ERROR;
}



/*----------------------------------------------------------------------
win_set_cur_scale - ???
----------------------------------------------------------------------*/

int
win_set_cur_scale(win,cur_scale)
     struct image_display_window *win;
     unsigned long cur_scale;
{
  int how,err;
  struct color_scale_rec *local,*global;
  
  PRT(printf("win_set_cur_scale...\n"));
  if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return err;
  PRT(printf("old global = %s   status = %x\n",global->label,
	       (unsigned long) global->status));
  PRT(printf("old local = %s   status = %x\n",local->label,
	       (unsigned long) local->status));

  global = &(win->intensity->scales[win->intensity->cur_scale]);
  if ((err = cp_color_scale_rec(local,global)) != NO_ERROR)
    return err;
  PRT3(printf("new global = %u  = %s   status = %x\n",global,global->label,
	       (unsigned long) global->status));
  PRT3(printf("new local = %u = %s   status = %x\n",local,local->label,
	       (unsigned long) local->status));
  how = global->status & COLOR_LOCAL;
  if (how == 0)
    WinEnableDisableAllLocalContrast(win,0,0);
  else
    WinEnableDisableAllLocalContrast(win,1,0);
  WinNewColorMultiCanvas(win);
  PRT(printf("win_set_cur_scale...win_update_cur_contrast\n"));
  win_update_cur_contrast(win);
  PRT3(printf("win_set_cur_scale...end\n"));
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinSetCurColorDataByScaleLabel - ???
----------------------------------------------------------------------*/

int
WinSetCurColorDataByScaleLabel(win,scale_label,option)
     struct image_display_window *win;
     char *scale_label;
     int option;
{
  unsigned long indx;
  
  PRT(printf("WinSetCurColorDataByScaleLabel scale_label = %s option = %d\n",
	     scale_label,option));
  if (scale_label != NULL)
  {
    if (sscanf(scale_label,"IMAGE#%ld",&indx) == 1)
    {
      indx -= 1;
      PRT(printf("WinSetCurColorDataByScaleLabel..ok\n"));
      if (indx < win->data.num_color_datas)
	WinSelectScale(win,indx);
    }
    PRT(printf("WinSetCurColorDataByScaleLabel indx = %d\n",indx));
  }
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinSetDefaultColorDataScale - ???
----------------------------------------------------------------------*/

int
WinSetDefaultColorDataScale(win,scale_label)
     struct image_display_window *win;
     char *scale_label;
{
  struct color_scale_rec *scale;
  struct color_data_rec *color_data;
  int err;
  
  PRT(printf("WinSetDefaultColorDataScale cur_color_data = %d\n",win->data.cur_color_data));
  err = NO_ERROR;
  if (win->data.cur_color_data < win->data.num_color_datas)
  {
    color_data = win->data.color_datas[win->data.cur_color_data];
    PRT(printf("WinSetDefaultColorDataScale scale_label = %s  clabel = %s\n",
	       scale_label,color_data->scale.label));
    if ((str_cmp(scale_label,color_data->scale.label) != 0) &&
	((scale = find_scale(win->intensity,scale_label,1)) != NULL) &&
	((err = cp_color_scale_rec(&(color_data->scale),scale)) == NO_ERROR))
    {
      err = WinSelectScale(win,win->data.cur_color_data);
      WinNewColorCanvas(win);
    }
  }
  return err;
}



/*----------------------------------------------------------------------
win_cur_scale_is_default - ???
----------------------------------------------------------------------*/

int
win_cur_scale_is_default(win)
     struct image_display_window *win;
{
  struct color_scale_rec *scale;

  if ((scale = win_cur_scale(win)) != NULL)
    find_scale(win->intensity,scale->label,1);
  return NO_ERROR;
}



/*----------------------------------------------------------------------
WinSetupScaleLabels - ???
----------------------------------------------------------------------*/

int
WinSetupScaleLabels(win,labels)
     struct image_display_window *win;
     struct strs_rec *labels;
{
  unsigned long lcv;
  char *str,str1[30];
  int err;

  PRT(printf("WinSetupScaleLabels...\n"));
  for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
  {
    sprintf(str1,"IMAGE#%ld) ",lcv + 1);
    if (((err = concat_strs(&str,4,str1,win->intensity->label,":",
			    win->data.color_datas[lcv]->scale.label))
	 != NO_ERROR) ||
	((err = add_str_ptr_to_strs_rec(labels,str)) != NO_ERROR))
    {
      c_free(str);
      return err;
    }
  }
  PRT(printf("WinSetupScaleLabels...end\n"));
  return NO_ERROR;
}



/*----------------------------------------------------------------------
win_update_color_opr - ???
----------------------------------------------------------------------*/

void
win_update_color_opr(win)
     struct image_display_window *win;
{
  PRT(printf("win_update_color_opr...win_update_contrast\n"));
  win_update_contrast(win);
  PRT(printf("win_update_color_opr...win_modify_update_color_opr\n"));
  win_modify_update_color_opr(win);
  PRT(printf("win_update_color_opr...end\n"));
}



/*----------------------------------------------------------------------
win_set_color_map - ???
----------------------------------------------------------------------*/

int
win_set_color_map(win,use_original)
     struct image_display_window *win;
     int use_original;
{
  int redisplay,err;
  
  err = ChangeExistingColorSegment(win->dwin->frame,win->intensity->label,
				   &(win->intensity->color),use_original,
				   &redisplay);
  if (redisplay && (err == NO_ERROR))
    WinNewColorMultiCanvas(win);
  return err;
}




/*----------------------------------------------------------------------
win_set_color_by_intensity_rec - ???

apply to = 0 -- apply the current scale to windows using the same scale of
                the same pallette.  For windows using other pallettes,
  	        their scale labels are searched and if they are found in
 		the current pallette, they will be changed to that scale,
 		othewise to the current scale.
apply to = 1 -- apply the current scale to all windows
apply to = 2 -- apply the current scale to windows using the same scale of
                the same pallette only.
----------------------------------------------------------------------*/

int
win_set_color_by_intensity_rec(win,intensity,is_to_all_window)
     struct image_display_window *win;
     struct intensity_rec *intensity;
     int is_to_all_window;
{
  unsigned long lcv;
  struct color_rec *color;
  struct image_display_window *win1;
  Frame *frame;
  int first,num_colors,first1,num_colors1;
  struct color_scale_rec *scale,*local,*global;
  struct intensity_rec *old_intensity;
  int err,how;
  struct img_buf_rec *img;
  struct win_rec **rwins;

  PRT(printf("win_set_color_by_intensity_rec...0\n"););
  frame = (Frame *) main_display(2,NULL,0,0);
  if (*frame == 0)
    return CANCEL;
  color = &(intensity->color);
  default_map_intensity_rgb(intensity);
  if ((err = ChangeColorSegment(*frame,intensity->label,color,0)) != NO_ERROR)
    return err;
  PRT(printf("win_set_color_by_intensity_rec...1\n"););
  xv_set(*frame,
	 WIN_BACKGROUND_COLOR,1 + color->num_system_colors,
	 WIN_FOREGROUND_COLOR,2 + color->num_system_colors,
	 NULL );
  set_win_foreground_to_default_gc(*frame);
  set_win_background_to_default_gc(*frame);
  PRT(printf("win_set_color_by_intensity_rec...2\n"););

  first1 = intensity->scales[intensity->cur_scale].first;
  num_colors1 = intensity->scales[intensity->cur_scale].num_colors;
  old_intensity = win->intensity;
  how = intensity->scales[intensity->cur_scale].status & COLOR_LOCAL;
  rwins = win->owner->root->wins;
  for (lcv = 0; lcv < win->owner->root->total_wins; lcv++)
  {
    if ((rwins[lcv] != NULL) &&
	((win1 = rwins[lcv]->image_display) != NULL) &&
	(win1->intensity != NULL))
    {
      if (is_to_all_window == 2)
	if (win1->intensity != old_intensity)
	  continue;
      if ((err = WinFindLocalAndGlocalScale(win1,&local,&global)) != NO_ERROR)
	continue;

      first = global->first;
      num_colors = global->num_colors;
      TRACE(printf("%d) %s first = %d num = %d first1 = %d num1 = %d\n",
		   lcv,global->label,
		   first,num_colors,first1,num_colors1););
      if (is_to_all_window == 0)
      {
	if (win1->intensity != old_intensity)
	  scale = find_scale(intensity,global->label,1);
      }
         
      win1->intensity = intensity;
      if ((img = win_rec_cur_img_buf(win1->owner)) != NULL)
	img->file->color_guide = intensity;
      if (how == 0)
	WinEnableDisableAllLocalContrast(win1,0,1);
      else
	WinEnableDisableAllLocalContrast(win1,1,1);

      TRACE(printf("wid = %s global->label =  %s\n",
		   win1->owner->wid,global->label););
      if ((first1 != first) || (num_colors1 != num_colors))
      {
	WinNewColorMultiCanvas(win1);
      }
    }
  }
  set_cur_color_scale_save_status(intensity,
				  (unsigned short) COLOR_CONTROL_UPDATED,0);
  win_update_color_opr(win);
  PRT(printf("win_set_color_by_intensity_rec...end\n"););
  return NO_ERROR;
}



/*----------------------------------------------------------------------
win_change_color_scale - ???
----------------------------------------------------------------------*/

void
win_change_color_scale(win,intensity_label,scale_label)
     struct image_display_window *win;
     char *intensity_label,*scale_label;
{
  struct pallette_rec *def_color;
  long cur_scale,cur_pallette;
  struct intensity_rec *intensity;
  int redisplay;
  int how;

  def_color = &(win->owner->root->def_color);
  cur_pallette = def_color->cur_pallette;
  PRT(printf("win_change_color_scale intensity_label = %s scale_label = %s\n",
	     intensity_label,scale_label));
  if ((intensity = find_intensity_rec(def_color,intensity_label,1)) != NULL)
  {
    PRT(printf("found = %s cur_pallette = %d %d\n",intensity_label,
	       cur_pallette,def_color->cur_pallette));
    if (find_scale(intensity,scale_label,1) != NULL)
    {
      PRT(printf("found intensity_label = %s %d\n",intensity_label,
		 intensity->cur_scale););
      if (cur_pallette == def_color->cur_pallette)
      {
	cur_scale = intensity->cur_scale;
	how = intensity->scales[cur_scale].status & COLOR_LOCAL;
	if (how)
	{
	  unmap_scale_rgb(&(intensity->color),&(intensity->scales[cur_scale]));
	  ChangeExistingColorSegment(win->dwin->frame,win->intensity->label,
				     &(win->intensity->color),0,&redisplay);
	}
	WinRunProcInList(WIN_LIST,win,win_set_cur_scale,cur_scale,0,0,0,0,0);
	set_cur_color_scale_save_status(intensity,
					(unsigned short) COLOR_CONTROL_UPDATED,
					0);
      }
      else 
	ShowMsgStrs(win->dwin->frame,1,NO_ERROR,0,2,
		    "Changing the color map for all windows...",
		    PLEASE_WAIT,
		    win_set_color_by_intensity_rec,3,win,intensity,0);
    }
  }
}



/*----------------------------------------------------------------------
win_set_color_by_new_intensity_rec - ???
----------------------------------------------------------------------*/

int
win_set_color_by_new_intensity_rec(win,intensity,is_create_new,is_to_all_window)
 struct image_display_window *win;
 struct intensity_rec *intensity;
 int is_create_new,is_to_all_window;
{
 struct intensity_rec *intensity1;
 int err;
 long lcv,indx;
 struct pallette_rec *def_color;

 TRACE(printf("win_set_color_by_new_intensity_rec...0\n");)
 def_color = &(win->owner->root->def_color);
 PRT(for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
    printf("%d) %u %s\n",lcv,def_color->pallettes[lcv],
           def_color->pallettes[lcv]->label);)
 if ((err = add_intensity_rec(def_color,intensity,(unsigned short) RGB_FREE_BUFFERS,
       is_create_new,&indx)) != NO_ERROR)
    return err;
 intensity = def_color->pallettes[indx];
 for (lcv = 0; lcv < intensity->num_scales; lcv++)
    if (intensity->scales[lcv].status & COLOR_CONTROL_EXCLUDE_BKG)
       define_actual_intensity_scale(intensity,1,lcv);
      else define_actual_intensity_scale(intensity,0,lcv);

 TRACE(printf("num_pallettes = %d...indx = %d\n",def_color->num_pallettes,indx);)
 if ((err = win_set_color_by_intensity_rec(win,intensity,is_to_all_window))!= NO_ERROR)
    return err;
 TRACE(printf("num_pallettes = %d...indx = %d\n",def_color->num_pallettes,indx);)

 if ((intensity1 = find_intensity_rec(def_color,intensity->label,1)) != NULL)
    {TRACE(printf("cur_pallette = %d indx = %d\n",def_color->cur_pallette,indx);)
     if (def_color->cur_pallette != indx)
        {def_color->pallettes[def_color->cur_pallette] = intensity;
         def_color->pallettes[indx] = intensity1;
         def_color->cur_pallette = indx;
         delete_cur_intensity_rec(def_color);
         find_intensity_rec(def_color,intensity->label,1);}
    }

 PRT(printf("win_set_color_by_new_intensity_rec...intensity = %u cur_pallette = %d pal = %u\n",
         intensity,def_color->cur_pallette,def_color->pallettes[def_color->cur_pallette]);)
 PRT(for (lcv = 0; lcv < def_color->num_pallettes; lcv++)
    printf("final %d) %u %s\n",lcv,def_color->pallettes[lcv],
             def_color->pallettes[lcv]->label);)
 return err;
}


/*----------------------------------------------------------------------
win_set_color_by_intensity_label - ???
----------------------------------------------------------------------*/

int
win_set_color_by_intensity_label(win,intensity_label,is_to_all_window)
 struct image_display_window *win;
 char *intensity_label;
 int is_to_all_window;
{
 struct intensity_rec *intensity;
 struct pallette_rec *def_color;

 TRACE(printf("win_set_color_by_intensity_label...\n");)
  def_color = &(win->owner->root->def_color);
if ((intensity = find_intensity_rec(def_color,intensity_label,1)) != NULL)
    return win_set_color_by_intensity_rec(win,intensity,is_to_all_window);
 return CANCEL;
}



/*----------------------------------------------------------------------
win_define_color_map - ???
----------------------------------------------------------------------*/

int
win_define_color_map(win,is_GT_ub_background)
     struct image_display_window *win;
     int is_GT_ub_background;
{
  struct color_scale_rec *local,*global;
  struct color_data_rec *color_data;
  unsigned long lcv;

  PRT3(printf("\nwin_define_color_map win = %d\n",win));

  if (WinFindLocalAndGlocalScale(win,&local,&global) != NO_ERROR)
    return 0;
  if (global->status & COLOR_LOCAL)
  {
    PRT3(printf("win_define_color_map COLOR_LOCAL wnum_colors = %d\n",
		 win->data.num_color_datas));

    for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
    {
      color_data = win->data.color_datas[lcv];
      local = &(color_data->scale);
      PRT3(printf("%d) num_color_indxs = %d num_colors = %d\n",lcv,
		   color_data->num_color_indxs,local->num_colors));
      if (color_data->num_color_indxs == local->num_colors)
      {
	if (local->status & COLOR_CONTROL_EXCLUDE_BKG)
	  is_GT_ub_background = 1;
	else
	  is_GT_ub_background = 0;
	define_actual_color_scale(color_data->color_indxs,
				  local,is_GT_ub_background);
      }
    }
    PRT3(printf("win_define_color_map..WinMultiScreen\n"));
    WinMultiScreenCanvas(win);
  }
  else
  {
    PRT(printf("win_define_color_map win = COLOR_GLOBAL\n"));
    if (global->status & COLOR_CONTROL_EXCLUDE_BKG)
      is_GT_ub_background = 1;
    else
      is_GT_ub_background = 0;
    define_actual_scale(&(win->intensity->color),is_GT_ub_background,global);
    global->status |= COLOR_CONTROL_UPDATED;
    win_set_color_map(win,0);
  }
  PRT(printf("\nwin_define_color_map win....end\n"));
  return NO_ERROR;
}


/*----------------------------------------------------------------------
win_set_related_color_maps - ???
----------------------------------------------------------------------*/

int
win_set_related_color_maps(win,color_indxs)
 struct image_display_window *win;
 unsigned long *color_indxs;
{
 struct color_scale_rec *local,*global;
 unsigned long lcv,cnt,last_color_index,first,num_colors;
 struct color_rec *color;

 if (WinFindLocalAndGlocalScale(win,&local,&global) != NO_ERROR)
    return 0;
 if (global->status & COLOR_LOCAL)
    {
     return win_local_color_indxs(win,color_indxs);
    }
   else
    {first = global->first;
     num_colors = global->num_colors;
     last_color_index = first + num_colors - 1;
     color = &(win->intensity->color);
     for (lcv = first,cnt = 0;  lcv <= last_color_index; lcv++,cnt++)
        {color->color_indxs[lcv] = color_indxs[cnt];
         color->red[lcv] = color->ori_red[color_indxs[cnt]];
         color->green[lcv] = color->ori_green[color_indxs[cnt]];
         color->blue[lcv] = color->ori_blue[color_indxs[cnt]];
        }
     win_set_color_map(win,0);
    }
 return NO_ERROR;
}


/*----------------------------------------------------------------------
win_image_enhancement - ???
----------------------------------------------------------------------*/

int
win_image_enhancement(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 struct buf_2d_rec **images,*the_image[1];
 long first,num_colors;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 the_image[0] = &(img->_2d);
 images = the_image;
 win_first_and_num_colors(win,&first,&num_colors);
 HistogramEqualization(images,1,img->file->hdr.data_info.min_pixel,
        img->file->hdr.data_info.max_pixel,0,
        img->file->hdr.data_info.min_pixel,
        img->file->hdr.data_info.max_pixel,NULL,NULL);
 WinNewColorCanvas(win);
 return NO_ERROR;
}



/*----------------------------------------------------------------------
WinSetMidContrast - ???
----------------------------------------------------------------------*/

int
WinSetMidContrast(win,scale_label,scale_indx,value,is_GT_ub_background)
     struct image_display_window *win;
     char *scale_label;
     unsigned long scale_indx;
     int value,is_GT_ub_background;
{
  struct color_data_rec *color_data;
  struct color_scale_rec *local,*global;
  int err;

  PRT3(printf("WinSetMidContrast...wid = %s scale_label = %s value = %d\n",
	       win->owner->wid,scale_label,value););
  if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return CANCEL;
  PRT3( printf("mid global->label = %s status = %x\n",
		global->label,
		(unsigned long) global->status); );
  if (global->status & COLOR_LOCAL)
  {
    PRT3(printf("WinSetMidContrast..COLOR_LOCAL\n"));
    if (scale_indx < win->data.num_color_datas)
    {
      color_data = win->data.color_datas[scale_indx];
      if (str_cmp(scale_label,color_data->scale.label) == 0)
	color_data->scale.mid_value = value;
    }
  }
  else if (str_cmp(scale_label,global->label) == 0)
  {
    PRT3(printf("..COLOR_GLOBAL mid_value= %d\n",global->mid_value));
    global->mid_value = value;
  }
  else
    return NO_ERROR;
  PRT3(printf("WinSetMidContrast..1\n"));
  
  win_define_color_map(win,is_GT_ub_background);
  win_update_cur_contrast(win);
  return NO_ERROR;
}

/*
     for (lcv = 0; lcv < win->data.num_colors; lcv++)
        {color_data = win->data.colors[lcv];
         PRT3(printf("%d) %s = %s\n",lcv,scale_label,color_data->scale.label))
         if (str_cmp(scale_label,color_data->scale.label) == 0)
            color_data->scale.mid_value = value;
        }
*/





/*----------------------------------------------------------------------
WinSetRangeContrast - ???
----------------------------------------------------------------------*/

int
WinSetRangeContrast(win,scale_label,scale_indx,value,is_GT_ub_background)
     struct image_display_window *win;
     char *scale_label;
     unsigned long scale_indx;
     int value,is_GT_ub_background;
{
  struct color_scale_rec *local,*global;
  struct color_data_rec *color_data;
  int err;

  PRT3(printf("WinSetRangeContrast...wid = %s scale_label = %s value = %d\n",
	       win->owner->wid,scale_label,value));
  if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return CANCEL;
  PRT(printf("global->label = %s status = %x\n",global->label,
	     (unsigned long) global->status););
  
  if (global->status & COLOR_LOCAL)
  {
    PRT3(printf("WinSetRangeContrast..COLOR_LOCAL\n"));
    if (scale_indx < win->data.num_color_datas)
    {
      color_data = win->data.color_datas[scale_indx];
      if (str_cmp(scale_label,color_data->scale.label) == 0)
	set_scale_range(&(color_data->scale),value);
    }
  }
  else if (str_cmp(scale_label,global->label) == 0)
    set_scale_range(global,value);

  win_define_color_map(win,is_GT_ub_background);
  win_update_cur_contrast(win);
  PRT(printf("WinSetRangeContrast end\n"););
  return NO_ERROR;
}




/*----------------------------------------------------------------------
WinSetContrastLbUb - ???
----------------------------------------------------------------------*/

int
WinSetContrastLbUb(win,scale_label,scale_indx,lb,ub,is_GT_ub_background)
     struct image_display_window *win;
     char *scale_label;
     unsigned long scale_indx;
     long lb,ub,is_GT_ub_background;
{
  struct color_data_rec *color_data;
  struct color_scale_rec *local,*global;
  int err;

  PRT(printf("WinSetContrastLbUb...wid = %s scale_label = %s lb = %d ub = %d\n",
	     win->owner->wid,scale_label,lb,ub));
  if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return CANCEL;
  PRT(printf("global->label = %s status = %x\n",global->label,
	     (unsigned long) global->status););
  if (global->status & COLOR_LOCAL)
  {
    if (scale_indx < win->data.num_color_datas)
    {
      color_data = win->data.color_datas[scale_indx];
      if (str_cmp(scale_label,color_data->scale.label) == 0)
      {
	color_data->scale.lb = lb;
	color_data->scale.ub = ub;
      }
    }
  }
  else if (str_cmp(scale_label,global->label) == 0)
  {
    global->lb = lb;
    global->ub = ub;
  }
  
  PRT(printf("WinSetContrastLbUb win_update_cur_contrast\n"););
  win_define_color_map(win,is_GT_ub_background);
  win_update_cur_contrast(win);
  PRT(printf("WinSetContrastLbUb end\n"););
  return NO_ERROR;
}




/*----------------------------------------------------------------------
win_local_color_indxs - ???
----------------------------------------------------------------------*/

int
win_local_color_indxs(win,color_indxs)
     struct image_display_window *win;
     unsigned long *color_indxs;
{
  long cnt,lcv;
  struct color_data_rec *color_data;
  struct color_scale_rec *local,*global;
  int err;

  PRT(printf("win_local_color_indxs...color_indxs = %u\n",color_indxs));
  if ((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR)
    return err;
  for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
  {
    color_data = win->data.color_datas[lcv];
    if ((str_cmp(color_data->scale.label,global->label) == 0) &&
	(global->num_colors <= color_data->total_color_indxs))
      for (cnt = 0; cnt < global->num_colors; cnt++)
	color_data->color_indxs[cnt] = color_indxs[cnt];
  }
  return NO_ERROR;
}




/*----------------------------------------------------------------------
WinDefaultLocalColorIndxs - ???
----------------------------------------------------------------------*/

int
WinDefaultLocalColorIndxs(win,def_win,color_data)
     struct image_display_window *win,*def_win;
     struct color_data_rec *color_data;
{
  struct color_scale_rec *local,*global;
  struct color_data_rec *color_data1;
  unsigned long *color_indxs;
  int err;

  PRT1(printf("WinDefaultLocalColorIndxs def_win = %u wid = %s\n",
	      def_win,win->owner->wid););
  if ((def_win != NULL) &&
      ((color_data1 = WinCurColorData(def_win)) != NULL) &&
      ((local = &(color_data1->scale)) != NULL) &&
      (ValidateLocalColorIndxs(color_data1,local) == NO_ERROR))
  {
    PRT1(printf("WinDefaultLocalColorIndxs def_win wid = %s\n",
		def_win->owner->wid););
    if (color_data != NULL) 
      global = find_scale(win->intensity,color_data->scale.label,0);
    else
      global = NULL;
    PRT1(printf("WinDefaultLocalColorIndxs global = %u\n",global));
    if ((global == NULL) &&
	((err = WinFindLocalAndGlocalScale(def_win,&local,&global))
	 != NO_ERROR))
      return err;
    PRT1(printf("local status = %x %u\n",(unsigned long) local->status,local));
    PRT1(printf("local status = %x label = %u = %s mid = %d lb = %d ub = %d\n",
		(unsigned long) local->status,
		local->label,local->label,
		local->mid_value,local->lb,
		local->ub););

    PRT1(printf("global status = %x %u\n",
		(unsigned long) global->status,global));
    if (!(global->status & COLOR_LOCAL))
      return NO_ERROR;
    PRT1(printf("glbl status = %x label = %u = %s mid = %d lb = %d ub = %d\n",
		(unsigned long) global->status,
		global->label,global->label,
		global->mid_value,global->lb,
		global->ub););
    global = local;
    PRT1(printf("global =  %u\n",global));
    color_indxs = color_data1->color_indxs;
  }
  else
  {
    if (color_data != NULL) 
      global = find_scale(win->intensity,color_data->scale.label,0);
    else
      global = NULL;
    if ((global == NULL) &&
	((err = WinFindLocalAndGlocalScale(win,&local,&global)) != NO_ERROR))
      return err;
    PRT1(printf("WinDefaultLocalColorIndxs global->status = %x\n",
		(unsigned long) global->status));
    if (!(global->status & COLOR_LOCAL))
      return NO_ERROR;
    color_indxs = &(win->intensity->color.color_indxs[global->first]);
  }
  PRT1(printf("=== global =  %u\n",global));

  if (color_data != NULL)
    color_data1 = color_data;
  else if ((color_data1 = WinCurColorData(win)) == NULL) 
  {
    PRT1(printf("WinDefaultLocalColorIndxs WinCurColorData(win)) == NULL\n"););
    return CANCEL;
  }

  PRT1(printf("WinDefaultLocalColorIndxs glbl=%u, glbl->label=%s loc = %s\n",
	      global,global->label,color_data1->scale.label));

#if 0
  if (str_cmp(color_data1->scale.label,global->label) != 0) {}
#endif

  if ((err = cp_color_scale_rec(&(color_data1->scale),global)) != NO_ERROR)
    return err;

  PRT1(printf("win = %u first = %d num_colors = %d total_color_indxs = %d\n",
	      win,global->first,global->num_colors,
	      color_data1->total_color_indxs));
  err = SetupLocalScale(color_data1,global,color_indxs,global->num_colors);
  
  PRT1(printf("status = %x label = %u = %s mid = %d lb = %d ub = %d\n",
	      (unsigned long) color_data1->scale.status,
	      color_data1->scale.label,color_data1->scale.label,
	      color_data1->scale.mid_value,color_data1->scale.lb,
	      color_data1->scale.ub););
  PRT1(printf("num_color_indxs = %d total_color_indxs = %d\n\n",
	      color_data1->num_color_indxs,color_data1->total_color_indxs));

  return err;
}



/*----------------------------------------------------------------------
win_setup_default_local_color_indxs - ???
----------------------------------------------------------------------*/

int
win_setup_default_local_color_indxs(win,def_win)
 struct image_display_window *win,*def_win;
{
 struct color_scale_rec *local,*global;

 PRT1(printf("win_setup_default_local_color_indxs num_scales = %d\n",
       win->intensity->num_scales))
 if (WinFindLocalAndGlocalScale(win,&local,&global) == NO_ERROR)
    {PRT1(printf("win_setup_default_local_color_indxs...1\n"))
     if (global->status & COLOR_LOCAL)
        {PRT1(printf("win_setup_default_local_color_indxs...COLOR_LOCAL\n"))
         return WinDefaultLocalColorIndxs(win,def_win,NULL);}
    }
 return NO_ERROR;
}


/*----------------------------------------------------------------------
win_actual_color_indx - ???
----------------------------------------------------------------------*/

int
win_actual_color_indx(win,color_indx)
 struct image_display_window *win;
 unsigned long color_indx;
{
 unsigned long actual;
 struct color_scale_rec *local,*global;
 struct color_data_rec *color_data;

 actual = 0;
 if ((WinFindLocalAndGlocalScale(win,&local,&global) == NO_ERROR) &&
     ((color_data = WinCurColorData(win)) != NULL))
    {if ((global->status & COLOR_LOCAL) &&
         (color_indx < color_data->num_color_indxs))
        actual = color_data->color_indxs[color_indx];
      else if (color_indx < win->intensity->color.total_colors)
       actual = win->intensity->color.color_indxs[color_indx];
    }
 return actual;
}


/*----------------------------------------------------------------------
WinFindFirstWinOfThisScale - ???
----------------------------------------------------------------------*/

struct image_display_window *
WinFindFirstWinOfThisScale(win,scale_label)
 struct image_display_window *win;
 char *scale_label;
{
 struct image_display_window *win1;
 struct win_rec **rwins;
 unsigned long lcv,total_wins;
 struct color_scale_rec *scale;

 rwins = win->owner->root->wins;
 total_wins = win->owner->root->total_wins;
 for (lcv = 0; lcv < total_wins; lcv++)
    {if (rwins[lcv] != NULL)
        {if (((win1 = rwins[lcv]->image_display) != NULL) &&
             ((scale = win_cur_scale(win1)) != NULL) &&
             (str_cmp(scale->label,scale_label) == 0))
            return win1;
         if (((win1 = rwins[lcv]->histogram) != NULL) &&
             ((scale = win_cur_scale(win1)) != NULL) &&
             (str_cmp(scale->label,scale_label) == 0))
            return win1;
        }
    }
 return NULL;
}


/*----------------------------------------------------------------------
win_same_local_contrast - ???
----------------------------------------------------------------------*/

int
win_same_local_contrast(win,def_win)
 struct image_display_window *win,*def_win;
{
 unsigned long lcv;
 struct color_data_rec *color_data;
 struct color_scale_rec *scale,*scale1;

 PRT(printf("win_same_local_contrast wid = %s wid1 = %s...\n",
         win->owner->wid,def_win->owner->wid));
 if ((scale1 = win_cur_scale(def_win)) == NULL)
    return CANCEL;
 for (lcv = 0; lcv < win->data.num_color_datas; lcv++)
    {color_data = win->data.color_datas[lcv];
     scale = &(color_data->scale);
     if (str_cmp(scale->label,scale1->label) == 0)
        {PRT(printf("%d) label = %s %s\n",lcv,color_data->scale.label,scale1->label))
         cp_color_scale_rec(&(color_data->scale),scale);}
    }
 PRT(printf("win_same_local_contrast done...\n"));
 return NO_ERROR;
}


/*----------------------------------------------------------------------
WinSet2dByColorValueRange - ???
----------------------------------------------------------------------*/

int
WinSet2dByColorValueRange(win,_2d,mask,first_color_val,last_color_val)
 struct image_display_window *win;
 struct buf_2d_rec *_2d,*mask;
 unsigned long first_color_val,last_color_val;
{
 unsigned long *color_indxs,first,num_color_indxs;
 struct color_scale_rec *scale;
 struct color_data_rec *color_data;
 int err;

 PRT(printf("WinSet2dByColorValueRange---first_color_val = %d last_color_val = %d \n",
           first_color_val,last_color_val))
 if ((err = WinCurScaleAndColorIndxs(win,NULL,&scale,&color_indxs,
          &num_color_indxs)) != NO_ERROR)
    return err;
 if ((color_data = WinCurColorData(win)) == NULL) return CANCEL;
 first = scale->first;

 PRT(printf("1---first = %d num_color_indxs = %d\n",first,num_color_indxs))
 PRT(printf("color_data->_2d cnt = %d \n",
         CntValBuf2dRec(&(color_data->_2d),(double) first_color_val)))
 PRT(printf("1---_2d cnt = %d \n",CntValBuf2dRec(_2d,(double) 0)))
 Set2dByIndexValueRange(_2d,&(color_data->_2d),mask,
             color_indxs,num_color_indxs,first,
             first_color_val,last_color_val,(double) .0);
 PRT(printf("end---_2d cnt = %d \n",CntValBuf2dRec(_2d,(double) 0)))
 return NO_ERROR;
}


/*----------------------------------------------------------------------
 - ???

 initial val = value(_2d);
 if (select & 1) if (color_indxs[num] valid) val = color_indxs[num];
 if (select & 2) result = val;
----------------------------------------------------------------------*/

int
Set2dBackgroundColor(result,_2d,mask,color,color_indxs,num_indxs,
    first_indx,setval,select)
 struct buf_2d_rec *result,*_2d,*mask;
 struct color_rec *color;
 unsigned long *color_indxs,num_indxs,first_indx;
 double setval;
 unsigned long select;
{
 unsigned long row,col,indx;
 double val,val1;
 unsigned long num,dimx,dimy,cnt;

 if (result->dimy > _2d->dimy) dimy = _2d->dimy;
   else dimy = result->dimy;
 if (result->dimx > _2d->dimx) dimx = _2d->dimx;
   else dimx = result->dimx;
 PRT(printf("Set2dBackgroundColor num_indxs = %d first_indx = %d setval = %f total_colors = %d\n",num_indxs,first_indx,setval,color->total_colors))
 cnt = 0;
 for (row = 0; row < dimy; row++)
    {for (col = 0; col < dimx; col++)
        {if (mask != NULL)
            if (mask->GetVal(mask,row,col) == .0)
               continue;
         val1 = val = _2d->GetVal(_2d,row,col);
         indx = (unsigned long) rint(val);
         if ((num_indxs > 0) && (indx >= first_indx) &&
             ((num = (indx - first_indx)) < num_indxs))
            indx = color_indxs[num];
         if ((indx < color->total_colors) &&
             ((indx == first_indx) ||
              (color->color_status[indx] & BACKGROUND_COLOR)))
            {result->SetVal(result,row,col,setval);
             cnt++;
            }
           else if ((result != _2d) && (select & 3))
            {if (select & 1) val1 = (double) indx;
             if (select & 2) 
                result->SetVal(result,row,col,val1);
            }
        }
    }
 PRT(printf("Set2dBackgroundColor...cnt = %d end\n",cnt))
 return NO_ERROR;
}


/*----------------------------------------------------------------------
WinActualColor2d - ???

FIXME:  This function is never used and contains a serious bug (see
below).  I have removed it.  -PWH 19990331
----------------------------------------------------------------------*/

#if 0
int
WinActualColor2d(win,_2d)
 struct image_display_window *win;
 struct buf_2d_rec *_2d;
{
 int err;
 struct buf_2d_rec *_2d1;
 struct color_scale_rec *scale;
 unsigned long *color_indxs,num_color_indxs,first,num_colors;
 struct color_data_rec *color_data;

 if ((color_data = WinCurColorData(win)) == NULL) return CANCEL;
 _2d1 = &(color_data->_2d);

 PRT(printf("WinActualColor2d...\n"))
 if ((err = SetupBuf2dRec(_2d,_2d1->dimx,_2d1->dimy,_2d1->pixel_depth,
         _2d1->status4 & BUF_VALUE_TYPE)) != NO_ERROR)
    return err;
 win_first_and_num_colors(win,&first,&num_colors);
 PRT(printf("WinActualColor2d...1\n"))
 WinCurScaleAndColorIndxs(win,NULL,&scale,&color_indxs,&num_color_indxs);
 PRT(printf("WinActualColor2d...2\n"));
 /* FIXME:  Argument 4 below is completely the wrong type.  Fortunately,
    this function is never used.  -PWH 19990331 */
 Set2dBackgroundColor(_2d,_2d1,NULL,(win->intensity->color),
           color_indxs,num_color_indxs,first,(double) .0,3);
 PRT(printf("WinActualColor2d...3\n"))
 return NO_ERROR;
}
#endif


/*----------------------------------------------------------------------
WinMapActualColorData - ???
----------------------------------------------------------------------*/

int
WinMapActualColorData(win,cur_color_data,color_data_2d)
 struct image_display_window *win;
 struct color_data_rec *cur_color_data;
 struct buf_2d_rec *color_data_2d;
{
 unsigned long row,col,value;
 struct color_scale_rec *scale;
 unsigned long *color_indxs,num_color_indxs;
 int err;

 if ((err = WinCurScaleAndColorIndxs(win,cur_color_data,&scale,
         &color_indxs,&num_color_indxs)) != NO_ERROR)
    return CANCEL;
 for (row = 0; row < color_data_2d->dimy; row++)
    for (col = 0; col < color_data_2d->dimx; col++)
       {value = get_1d_buf_value(color_data_2d->buf[row],col,
                        color_data_2d->pixel_depth);
        if (value < num_color_indxs)
           set_1d_buf_value(color_data_2d->buf[row],col,
                 color_data_2d->pixel_depth,
                 (unsigned long) color_indxs[value]);
       }
 return NO_ERROR;
}


