/****************************************************************************
contrast.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: contrast.c,v 1.10 1999/07/07 17:41:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"



#define CLEAN_UP_IMAGE_CONSTRAST 0
#define SETUP_IMAGE_CONSTRAST 1
#define GET_CONSTRAST_PTR 2
#define BRING_UP_IMAGE_CONSTRAST 3

struct contrast_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Panel_item label,percen,mid,range,lb,ub,GT_ub_background,
               min_max,apply_to,how,how_to,min,max;
   };

struct all_contrast_rec_recs
   {
    long num_contrasts;
    struct contrast_rec **contrasts;
   };



/* Functions defined in this file */
void init_contrast_rec(struct contrast_rec *contrast);
void free_contrast_rec(struct contrast_rec *contrast);
void init_all_contrast_rec_recs(struct all_contrast_rec_recs *all_contrasts);
void contrast_global_local_proc(Panel_item item, Event *event);
void enable_global_local_contrast_proc(Menu menu, Menu_item item, void *arg,
    Xv_Window window, Event *last_event);
void cur_contrast_settings(struct contrast_rec *contrast);
int contrast_setup_scale_labels(struct contrast_rec *contrast);
int ContrastScaleLabel(struct contrast_rec *contrast, char **label,
    struct color_scale_rec **scale, unsigned long *scale_indx,
    int is_exc_gscale);
void contrast_color_proc(Panel_item item, Event *event);
void contrast_label_proc(Panel_item item, int value, Event *event);
void notify_mid_contrast_proc(Panel_item item, int value, Event *event);
int notify_range_contrast_proc(Panel_item item, int value, Event *event);
int contrast_lb_ub_proc(Panel_item item, int value, Event *event);
void more_commands_proc(Panel_item item, Event *event);
void notify_minmax_proc(Panel_item item, int value, Event *event);
void contrast_done_proc(Frame frame);
void global_min_max(struct contrast_rec *contrast, struct img_buf_rec *img);
void slice_min_max(struct contrast_rec *contrast, struct img_buf_rec *img);
void set_min_max(struct contrast_rec *contrast, struct img_buf_rec *img);
int win_set_slice_min_max(struct image_display_window *win, int select);
void min_max_proc(Panel_item item, int value, Event *event);
void set_contrast_item_values(struct contrast_rec *contrast);
int setup_contrast_proc(struct contrast_rec *contrast,
    struct image_display_window *win);
int pick_non_background_pixels(struct image_display_window *win);
int map_non_background_pixels(struct image_display_window *win);
int contrast_mouse_proc(struct image_display_window *win, Event *event);
struct contrast_rec *find_contrast_rec(
   struct all_contrast_rec_recs *all_contrasts,
    struct image_display_window *win);
void win_update_cur_contrast_settings(struct image_display_window *win);
void win_update_cur_contrast(struct image_display_window *win);
void win_update_contrast(struct image_display_window *win);
int enter_image_contrast(struct image_display_window *win, int status);



/*------------------------------------------------------------------------
init_contrast_rec - ???
------------------------------------------------------------------------*/

void
init_contrast_rec(contrast)
 struct contrast_rec *contrast;
{
 contrast->frame = 0;
 contrast->win = NULL;
 contrast->status = 0;
}


/*------------------------------------------------------------------------
free_contrast_rec - ???
------------------------------------------------------------------------*/

void
free_contrast_rec(contrast)
 struct contrast_rec *contrast;
{
 struct all_contrast_rec_recs *all_contrasts;

 if (contrast->frame != 0) xv_destroy_safe(contrast->frame);
 c_free(contrast); 
 all_contrasts = (struct all_contrast_rec_recs *) 
           enter_image_contrast(NULL,GET_CONSTRAST_PTR);
 DropHandleAryPtr(&(all_contrasts->contrasts),
                     &(all_contrasts->num_contrasts),contrast);
 PRT2(printf("free_contrast_rec...num_contrasts = %d\n",all_contrasts->num_contrasts))
}


/*------------------------------------------------------------------------
init_all_contrast_rec_recs - ???
------------------------------------------------------------------------*/

void
init_all_contrast_rec_recs(all_contrasts)
 struct all_contrast_rec_recs *all_contrasts;
{
 all_contrasts->num_contrasts = 0;
 all_contrasts->contrasts = NULL;
}


/*------------------------------------------------------------------------
contrast_global_local_proc - ???
------------------------------------------------------------------------*/

void
contrast_global_local_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct contrast_rec *contrast;
 int how;

 contrast = (struct contrast_rec *) xv_get(item,PANEL_CLIENT_DATA);
 how = (int) xv_get(contrast->how,PANEL_VALUE);
 WinEnableGlobalLocalContrast(contrast->win,how);
}


/*------------------------------------------------------------------------
enable_global_local_contrast_proc - ???
------------------------------------------------------------------------*/

void
enable_global_local_contrast_proc(menu,item,arg,window,last_event)
 Menu menu;
 Menu_item item;
 void *arg;
 Xv_Window window;
 Event *last_event;
{
 struct display_win_rec *dwin;
 struct image_display_window *win;
 struct color_scale_rec *scale;
 int how;

 dwin = (struct display_win_rec *) arg;
 PRT1(printf("enable_global_local_contrast_proc dwin = %u\n",dwin))
 win = dwin->owner;
 win->dwin = dwin;
 PRT1(printf("wid = %s\n",win->owner->wid))
 if ((scale = win_cur_scale(win)) == NULL) return;
 if (scale->status & COLOR_LOCAL)
    how = 0;
   else how = 1;
 WinEnableGlobalLocalContrast(win,how);
 return;
}




/*------------------------------------------------------------------------
cur_contrast_settings - ???
------------------------------------------------------------------------*/

void
cur_contrast_settings(contrast)
     struct contrast_rec *contrast;
{
  struct color_scale_rec *scale;
  long last,fnum,lnum,scale_indx;
  char str[50];
  float lb_percent,ub_percent;
  char *label;

  xv_set( contrast->label,
	  PANEL_VALUE, contrast->win->data.cur_color_data,
	  0 );
  ContrastScaleLabel(contrast,&label,&scale,&scale_indx,0);
  PRT3(printf("cur_contrast_settings scale = %u status = %x scale_indx = %d\n",
	      scale,(unsigned long) scale->status,scale_indx));
  PRT1(pr_color_scale_rec(scale));

  if (!(scale->status & COLOR_LOCAL))
  {
    PRT1(printf("not COLOR_LOCAL...\n"));
    xv_set(contrast->how_to,PANEL_INACTIVE,TRUE,0);
    xv_set(contrast->how,PANEL_VALUE,0,0);
  }
  else
  {
    PRT1(printf(" COLOR_LOCAL...\n"))
      xv_set(contrast->how_to,PANEL_INACTIVE,FALSE,0);
    xv_set(contrast->how,PANEL_VALUE,1,0);
  }

  if (scale->status & COLOR_CONTROL_EXCLUDE_BKG)
    xv_set(contrast->GT_ub_background,PANEL_VALUE,1,0);
  else
    xv_set(contrast->GT_ub_background,PANEL_VALUE,0,0);
  last = scale->first + scale->num_colors - 1;

  fnum = scale->mid_value - scale->lb;
  if (fnum < (long) scale->first)
    fnum = 1;
  else
    fnum = fnum - (long) scale->first + 1;
 
  if (scale->status & COLOR_CONTROL_EXCLUDE_BKG)
  {
    lnum = scale->mid_value + scale->ub;
    if (lnum > last)
      lnum = last;
    lnum = lnum - scale->first + 1;
  }
  else
    lnum = scale->num_colors;
  lb_percent = (float) (fnum - 1) * 100.0 / (float) scale->num_colors;
  ub_percent = (float) lnum * 100.0 / (float) scale->num_colors;
  sprintf(str,"%.0f..%.0f%%/%-ld..%-ld",lb_percent,ub_percent,fnum,lnum);
  xv_set(contrast->percen,PANEL_LABEL_STRING,str,0);

  PRT2(printf(" cur_contrast_settings...1\n"));
  xv_set(contrast->mid,PANEL_VALUE,scale->mid_value,
	 PANEL_MIN_VALUE,scale->first,
	 PANEL_MAX_VALUE,last,
	 0);
  xv_set(contrast->range,PANEL_VALUE,scale->range,
	 PANEL_MIN_VALUE,0,
	 PANEL_MAX_VALUE,scale->num_colors - 1,
	 0);
  xv_set(contrast->lb,PANEL_VALUE,scale->lb,
	 PANEL_MIN_VALUE,0,
	 PANEL_MAX_VALUE,(int) (scale->num_colors - 1),
	 0);
  xv_set(contrast->ub,PANEL_VALUE,scale->ub,
	 PANEL_MIN_VALUE,0,
	 PANEL_MAX_VALUE,(int) (scale->num_colors - 1),
	 0);
}




/*------------------------------------------------------------------------
contrast_setup_scale_labels - ???
------------------------------------------------------------------------*/

int
contrast_setup_scale_labels(contrast)
 struct contrast_rec *contrast;
{
 unsigned long lcv;
 char *str,str1[30];

 PRT(printf("contrast_setup_scale_labels...\n"))
 for (lcv = 0; lcv < contrast->win->data.num_color_datas; lcv++)
    {sprintf(str1,"IMAGE#%ld) ",lcv + 1);
     if (concat_strs(&str,4,str1,contrast->win->intensity->label,":",
           contrast->win->data.color_datas[lcv]->scale.label) == NO_ERROR)
        {PRT2(printf("%d) %s\n",lcv,str))
         xv_set(contrast->label,PANEL_CHOICE_STRING,lcv,str,0);
         c_free(str);
        }
    }
 xv_set(contrast->label,PANEL_VALUE,contrast->win->data.cur_color_data,0);
 PRT(printf("contrast_setup_scale_labels...end\n"))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
ContrastScaleLabel - ???
------------------------------------------------------------------------*/

int
ContrastScaleLabel(contrast,label,scale,scale_indx,is_exc_gscale)
 struct contrast_rec *contrast;
 struct color_scale_rec **scale;
 char **label;
 unsigned long *scale_indx;
 int is_exc_gscale;
{
 char *label1,*str,str1[30];
 struct color_scale_rec *gscale,*lscale;
 struct color_data_rec *color_data;
 int err;

 PRT(printf("ContrastScaleLabel...\n"))
 if ((err = WinFindLocalAndGlocalScale(contrast->win,&lscale,
       &gscale)) != NO_ERROR)
    return err;
 *scale_indx = xv_get(contrast->label,PANEL_VALUE);
 PRT(printf("ContrastScaleLabel...*scale_indx = %d\n",*scale_indx))
 PRT(if (*scale_indx < contrast->win->data.num_color_datas)
         printf("scale.status = %x\n",(unsigned long) 
          contrast->win->data.color_datas[*scale_indx]->scale.status))
 str = NULL;
 if (!is_exc_gscale && !(gscale->status & COLOR_LOCAL))
    *scale = gscale;
   else
    {if (*scale_indx < contrast->win->data.num_color_datas)
        color_data = contrast->win->data.color_datas[*scale_indx];
     sprintf(str1,"IMAGE#%ld) ",*scale_indx + 1);
     if ((*scale_indx < contrast->win->data.num_color_datas) &&
         (color_data->scale.status & COLOR_CONTROL_VALID) &&
         ((label1 = (char *) xv_get(contrast->label,
                   PANEL_CHOICE_STRING,*scale_indx)) != NULL) &&
         (concat_strs(&str,4,str1,contrast->win->intensity->label,":",
               color_data->scale.label) == NO_ERROR) &&
         (str_cmp(str,label1) == 0))
        {PRT(printf("label1 = %s\n",label1))
         *scale = &(contrast->win->data.color_datas[*scale_indx]->scale);}
       else
        {contrast_setup_scale_labels(contrast);
         PRT(printf("ContrastScaleLabel...1\n"))
         *scale = &(contrast->win->data.color_datas[0]->scale);
         contrast->win->data.cur_color_data = 0;
         c_free(contrast->win->data.scale_label);
         cp_str(&(contrast->win->data.scale_label),(*scale)->label);
         xv_set(contrast->label,PANEL_VALUE,0,0);}
    }
 *label = (*scale)->label;
 PRT(printf("ContrastScaleLabel *scale_indx = %d label = %s\n",*scale_indx,*label))
 c_free(str);
 return NO_ERROR;
}


/*------------------------------------------------------------------------
contrast_color_proc - ???
------------------------------------------------------------------------*/

void
contrast_color_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct contrast_rec *contrast;
 Menu colormenu;

 contrast = (struct contrast_rec *)
                       xv_get(item,PANEL_CLIENT_DATA);
 if ((colormenu = setup_color_pallette_menu(2,NULL,0)) != 0)
    {PRT1(printf("contrast_color_proc...colormenu = %u  dwin = R%u\n",
          colormenu,contrast->win->dwin))
     xv_set(colormenu,MENU_CLIENT_DATA,contrast->win->dwin,0);
     xv_set(colormenu,XV_KEY_DATA,1000,contrast->win->dwin,0);
     PRT1(printf("event_window... = %u\n",event_window(event)))
     menu_show(colormenu,contrast->frame,event,0);}
   else ShowMsgStrs(contrast->frame,1,NO_ERROR,1,2,
                   "Sorry...The Color Pallette Menu is accessible !!!",
                   "Please invoke it from the FILE menu",NULL,0);
}


/*------------------------------------------------------------------------
contrast_label_proc - ???
------------------------------------------------------------------------*/

void
contrast_label_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct contrast_rec *contrast;
 struct color_scale_rec *scale;
 char *label;
 unsigned long scale_indx;

 contrast = (struct contrast_rec *) xv_get(item,PANEL_CLIENT_DATA);
 ContrastScaleLabel(contrast,&label,&scale,&scale_indx,1);
 WinSelectScale(contrast->win,scale_indx);
}


/*------------------------------------------------------------------------
notify_mid_contrast_proc - ???
------------------------------------------------------------------------*/

void
notify_mid_contrast_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct image_display_window *win;
 struct contrast_rec *contrast;
 int how_to;
 char *label;
 struct color_scale_rec *scale;
 unsigned long select,scale_indx;
 int err;

 contrast = (struct contrast_rec *)
                       xv_get(item,PANEL_CLIENT_DATA);
 win = contrast->win;
 PRT2(printf("\nnotify_mid_contrast_proc...win = %u\n",win))
 if ((err = ContrastScaleLabel(contrast,&label,&scale,&scale_indx,0)) != NO_ERROR)
     return /*err*/;
 if (scale->status & COLOR_LOCAL)
    how_to = (int) xv_get(contrast->how_to,PANEL_VALUE,0);
   else how_to = 0;

 PRT2(printf("notify_mid_contrast_proc label = %s value = %d how_to = %d\n",
     label,value,how_to))
 PRT2(printf("scale = %u label = %s status = %x\n",
     scale,scale->label,(unsigned long) scale->status))
 if (how_to)
    {if (how_to == 1) select = 0x00000083;
       else select = 0x000000A3;
     PRT2(printf("notify_mid_contrast_proc win_run_proc_to_all_file_wins\n\n"))
     win_run_proc_to_all_file_wins(win,win_same_local_contrast,select,
				   win,0,0);
     /* FIXME!! The function call below has more arguments than are
	permitted by the definition.  Commented out the extras to
	placate the compiler.  -PWH 19990603 */
     win_run_proc_to_all_file_wins(win,WinSetMidContrast,select,
              label,scale_indx,value /* ,
	      (int) xv_get(contrast->GT_ub_background,PANEL_VALUE) */ );}
   else
    {PRT2(printf("notify_mid_contrast_proc 1\n\n"))
     WinSetMidContrast(contrast->win,label,scale_indx,value,
       (int) xv_get(contrast->GT_ub_background,PANEL_VALUE));}
}


/*------------------------------------------------------------------------
notify_range_contrast_proc - ???
------------------------------------------------------------------------*/

int
notify_range_contrast_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct image_display_window *win;
 struct contrast_rec *contrast;
 int how_to,err;
 unsigned long select,scale_indx;
 char *label;
 struct color_scale_rec *scale;

 contrast = (struct contrast_rec *) xv_get(item,PANEL_CLIENT_DATA);
 win = contrast->win;
 if ((err = ContrastScaleLabel(contrast,&label,&scale,&scale_indx,0)) != NO_ERROR)
    return err;
 if (scale->status & COLOR_LOCAL)
    how_to = (int) xv_get(contrast->how_to,PANEL_VALUE,0);
   else how_to = 0;
 PRT3(printf("notify_range_contrast_proc how_to = %d\n",how_to))
 if (how_to)
    {PRT3(printf("notify_range_contrast_proc win_run_proc_to_all_file_wins\n"))
     if (how_to == 1) select = 0x00000083;
       else select = 0x000000A3;
     win_run_proc_to_all_file_wins(win,win_same_local_contrast,select,win,0,0);
     /* FIXME!! The function call below has more arguments than are
	permitted by the definition.  I commented out the extras to
	placate the compiler.  -PWH 19990603 */
     win_run_proc_to_all_file_wins(win,WinSetRangeContrast,select,
             label,scale_indx,value /* ,
	     (int) xv_get(contrast->GT_ub_background,PANEL_VALUE)*/ );}
 else
    {PRT3(printf("notify_range_contrast_proc 1\n"))
     WinSetRangeContrast(contrast->win,
        label,scale_indx,value,
       (int) xv_get(contrast->GT_ub_background,PANEL_VALUE));}
 return NO_ERROR;
}


/*------------------------------------------------------------------------
contrast_lb_ub_proc - ???
------------------------------------------------------------------------*/

int
contrast_lb_ub_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct image_display_window *win;
 struct contrast_rec *contrast;
 int how_to;
 char *label;
 struct color_scale_rec *scale;
 unsigned long select,scale_indx;
 int err;

 contrast = (struct contrast_rec *) xv_get(item,PANEL_CLIENT_DATA);
 win = contrast->win;
 if ((err = ContrastScaleLabel(contrast,&label,&scale,&scale_indx,0)) != NO_ERROR)
    return err;
 if (scale->status & COLOR_LOCAL)
    how_to = (int) xv_get(contrast->how_to,PANEL_VALUE,0);
   else how_to = 0;
 if (how_to)
    {PRT3(printf("contrast_lb_ub_proc win_run_proc_to_all_file_wins\n"))
     if (how_to == 1) select = 0x00000083;
       else select = 0x000000A3;
     win_run_proc_to_all_file_wins(win,win_same_local_contrast,select,win,0,0);
     /* FIXME!!  The following function call has more arguments than are
	allowed by the definition.  Temporarily removed the extras to
	placate the compiler.  -PWH 19990603 */
     win_run_proc_to_all_file_wins(win,WinSetContrastLbUb,select,
				   label,scale_indx,
             (int) xv_get(contrast->lb,PANEL_VALUE) /*,
             (int) xv_get(contrast->ub,PANEL_VALUE),
             (int) xv_get(contrast->GT_ub_background,PANEL_VALUE) */ );}
   else
    {PRT3(printf("contrast_lb_ub_proc 1\n"))
     WinSetContrastLbUb(contrast->win,label,scale_indx,
             (int) xv_get(contrast->lb,PANEL_VALUE),
             (int) xv_get(contrast->ub,PANEL_VALUE),
             (int) xv_get(contrast->GT_ub_background,PANEL_VALUE));}
 return NO_ERROR;
}


/*------------------------------------------------------------------------
more_commands_proc - ???
------------------------------------------------------------------------*/

void
more_commands_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct contrast_rec *contrast;

 contrast = (struct contrast_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(contrast->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "Ctrl Sh <>LEFT - set contrast mid = color(x,y) only",
               "Ctrl Sh <>MIDDLE - \"non-background\" ROI = non-background pixels",
               "Ctrl Sh <>RIGHT - Map \"non-background\" ROI TO HISTO-ROI or",
               "..............Map non-background pixels if ROI not exists",
               "",
               "Sh U - use slice min max for the cur slice",
               "Ctrl U - use slice min max for the cur file",
               "Sh Ctrl U - use slice min max for all files",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
}


/*------------------------------------------------------------------------
notify_minmax_proc - ???
------------------------------------------------------------------------*/

void
notify_minmax_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct contrast_rec *contrast;
 int value1;

 contrast = (struct contrast_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (item == contrast->min)
    {value1 = (int) xv_get(contrast->max,PANEL_VALUE);
     if (value1 < value) xv_set(contrast->min,PANEL_VALUE,value1,0);
    }
   else if (item == contrast->max)
    {value1 = (int) xv_get(contrast->min,PANEL_VALUE);
     if (value1 > value) xv_set(contrast->max,PANEL_VALUE,value1,0);
    }

}


/*------------------------------------------------------------------------
contrast_done_proc - ???
------------------------------------------------------------------------*/

void
contrast_done_proc(frame)
 Frame frame;
{
 struct contrast_rec *contrast;

 contrast = (struct contrast_rec *)
                     xv_get(frame,WIN_CLIENT_DATA);
 free_contrast_rec(contrast);
}




/*------------------------------------------------------------------------
global_min_max - ???
------------------------------------------------------------------------*/

void
global_min_max(contrast,img)
 struct contrast_rec *contrast;
 struct img_buf_rec *img;
{
  TRACE(printf("global_min_max...\n"));
  if (!(img->status4 & IMAGE_USE_MIN_MAX))
    return;
  img->status4 &= ~IMAGE_USE_MIN_MAX;
  img->_2d.status4 |= BUF_MIN_MAX_SET;
  img->_2d.minval_set = img->file->hdr.data_info.min_pixel;
  img->_2d.maxval_set = img->file->hdr.data_info.max_pixel;
  if ((img->win != NULL) && (img->win->image_display != NULL))
    WinNewColorMultiCanvas(img->win->image_display);
}




/*------------------------------------------------------------------------
slice_min_max - ???
------------------------------------------------------------------------*/

void
slice_min_max(contrast,img)
     struct contrast_rec *contrast;
     struct img_buf_rec *img;
{
  TRACE(printf("slice_min_max...\n"));

  if (img->status4 & IMAGE_USE_SLICE_MIN_MAX)
      return /*NO_ERROR*/;
  img->status4 &= ~IMAGE_USE_MIN_MAX;
  img->status4 |= IMAGE_USE_SLICE_MIN_MAX;
  img->_2d.status4 &= ~BUF_MIN_MAX_SET;
  GetImgSliceMinMax(img,NULL);
  if ((img->win != NULL) && (img->win->image_display != NULL))
    WinNewColorMultiCanvas(img->win->image_display);
}



/*------------------------------------------------------------------------
set_min_max - ???
------------------------------------------------------------------------*/

void
set_min_max(contrast,img)
 struct contrast_rec *contrast;
 struct img_buf_rec *img;
{
 TRACE(printf("set_min_max...\n"))
 img->status4 &= ~IMAGE_USE_MIN_MAX;
 img->status4 |= IMAGE_USE_SET_MIN_MAX;
 img->_2d.status4 |= BUF_MIN_MAX_SET;
 img->file->slice_info.min_pixel = (double) ((unsigned long) xv_get(contrast->min,PANEL_VALUE));
 img->file->slice_info.max_pixel = (double) ((unsigned long) xv_get(contrast->max,PANEL_VALUE));
 if ((img->_2d.minval_set != img->file->slice_info.min_pixel) ||
     (img->_2d.maxval_set != img->file->slice_info.max_pixel)) 
    {img->_2d.minval_set = img->file->slice_info.min_pixel;
     img->_2d.maxval_set = img->file->slice_info.max_pixel;
     if ((img->win != NULL) && (img->win->image_display != NULL))
        WinNewColorMultiCanvas(img->win->image_display);
    }
}



/*------------------------------------------------------------------------
win_set_slice_min_max - ???
------------------------------------------------------------------------*/

int
win_set_slice_min_max(win,select)
     struct image_display_window *win;
     int select;
{
  struct img_buf_rec *img;
  struct input_file_rec *file;
  unsigned long lcv,cnt;


  if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
    return CANCEL;
  if (select == 3)
    for (cnt = 0; cnt < img->file->prog->input.num_files; cnt++)
    {
      file = img->file->prog->input.files[cnt];
      for (lcv = 0; lcv < file->img.num_data; lcv++)
	slice_min_max(NULL,file->img.data[lcv]);
    }
  else if (select == 2)
    for (lcv = 0; lcv < img->file->img.num_data; lcv++)
      slice_min_max(NULL,img->file->img.data[lcv]);
  else
    slice_min_max(NULL,img);
  return NO_ERROR;
}


/*------------------------------------------------------------------------
min_max_proc - ???
------------------------------------------------------------------------*/

void
min_max_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct contrast_rec *contrast;
 struct img_buf_rec *img;
 struct input_file_rec *file;
 unsigned long lcv,cnt,apply_to;
 void (*proc)();       /* Was int.  -PWH 19990401 */

 contrast = (struct contrast_rec *)
                       xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(contrast->win->owner)) == NULL) return;
 apply_to = (int) xv_get(contrast->apply_to,PANEL_VALUE);

 if (value == 0) proc = global_min_max;
   else if (value == 1) proc = slice_min_max;
   else proc = set_min_max;
 if (apply_to == 0)
    proc(contrast,img);
   else if (apply_to == 1)
    {
     for (lcv = 0; lcv < img->file->img.num_data; lcv++)
        proc(contrast,img->file->img.data[lcv]);
    }
   else
    {
     for (cnt = 0; cnt < img->file->prog->input.num_files; cnt++)
        {file = img->file->prog->input.files[cnt];
         for (lcv = 0; lcv < file->img.num_data; lcv++)
            proc(contrast,file->img.data[lcv]);
        }
    }
 set_contrast_item_values(contrast);
}


/*------------------------------------------------------------------------
set_contrast_item_values - ???
------------------------------------------------------------------------*/

void
set_contrast_item_values(contrast)
 struct contrast_rec *contrast;
{
 unsigned long min,max;
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(contrast->win->owner)) != NULL)
    {if (img->status4 & IMAGE_USE_SLICE_MIN_MAX)
        {min = (long) img->_2d.minval;
         max = (long) img->_2d.maxval;
         xv_set(contrast->min_max,PANEL_VALUE,1,0);
        }
       else
        {min = (long) img->_2d.minval_set;
         max = (long) img->_2d.maxval_set;
         if (img->status4 & IMAGE_USE_SET_MIN_MAX)
            xv_set(contrast->min_max,PANEL_VALUE,2,0);
           else xv_set(contrast->min_max,PANEL_VALUE,0,0);
        }
     xv_set(contrast->min,PANEL_VALUE,min,0);
     xv_set(contrast->max,PANEL_VALUE,max,0);
    }
}



/*------------------------------------------------------------------------
setup_contrast_proc - ???
------------------------------------------------------------------------*/

int
setup_contrast_proc(contrast,win)
     struct contrast_rec *contrast;
     struct image_display_window *win;
{
  Panel_item item;
  int row,offset,temp;
  double min,max;
  struct img_buf_rec *img;
  char *str;
  Rect *rr;
  unsigned long first,num_colors;
  struct color_scale_rec *scale;
  Cms cms;			/* DRW 19970811 */


  if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
    return CANCEL;

  if ((scale = win_cur_scale(win)) == NULL)
    return CANCEL;

  cms = (Cms) xv_get( win->dwin->frame, WIN_CMS ); /* DRW 19970811 */
  PRT(printf("setup_contrast_proc(): cms = %p, parent frame = %p\n",
	     (void *) cms, (void *) win->dwin->frame ););
  contrast->frame = xv_create(win->dwin->frame, FRAME,
			      FRAME_INHERIT_COLORS,TRUE,
			      WIN_CLIENT_DATA,contrast,
			      FRAME_DONE_PROC,contrast_done_proc,
			      0);
  if (contrast->frame == 0)
      /* FIXME:  Guessed at return value below.  -PWH 19990401 */
      return CANCEL;
  contrast->win = win;
  contrast->dwin = win->dwin;
  str = win_rec_win_id_title(win->owner,IMAGE_CONTRAST);
  xv_set(contrast->frame,
	 WIN_CMS, cms,		/* DRW 19970811 */
	 FRAME_SHOW_LABEL, TRUE,
	 XV_LABEL,str,
	 FRAME_NO_CONFIRM,TRUE,
	 0);
  c_free(str);
  xv_set(contrast->frame,FRAME_CLOSED,FALSE,0);
  
  contrast->panel = xv_create(contrast->frame,PANEL,
			      WIN_CMS, cms,
			      FRAME_INHERIT_COLORS,TRUE,
			      0);
  if (contrast->panel == 0)
  {
    free_contrast_rec(contrast);
    return 0;
  }

  row = 2;
  win_first_and_num_colors(win,&first,&num_colors);

  offset = xv_rows(contrast->panel,1) - xv_rows(contrast->panel,0) - 10;
  
  contrast->how = xv_create(contrast->panel,PANEL_CHOICE_STACK,
			    PANEL_LAYOUT,PANEL_VERTICAL,
			    PANEL_CHOICE_STRINGS,"GLOBAL CONTRAST-Ctrl F4",
			    "LOCAL CONTRAST - Ctrl F4",NULL,
			    PANEL_NOTIFY_PROC,contrast_global_local_proc,
			    PANEL_VALUE,0,
			    XV_X,5,XV_Y,row,
			    PANEL_CLIENT_DATA,contrast,
			    0);

  rr = (Rect *) xv_get(contrast->how,XV_RECT);
  contrast->how_to = xv_create(contrast->panel,PANEL_CHOICE_STACK,
			       PANEL_LAYOUT,PANEL_VERTICAL,
			       PANEL_CHOICE_STRINGS,"TO CUR WINDOW",
			       "TO CUR FILE","TO ALL FILES",NULL,
			       PANEL_VALUE,0,
			       XV_X,rr->r_left + rr->r_width,XV_Y,row,
			       PANEL_CLIENT_DATA,contrast,
			       0);
  PRT3(printf("setup_contrast_proc...\n"));
  row += offset;
  item = xv_create(contrast->panel,PANEL_BUTTON,
		   PANEL_LABEL_STRING,"COLOR",
		   PANEL_NOTIFY_PROC,contrast_color_proc,
		   PANEL_CLIENT_DATA,contrast,
		   XV_X,5,XV_Y,row,0);
  rr = (Rect *) xv_get(item,XV_RECT);
  temp = rr->r_left + rr->r_width + 1;
  contrast->label = xv_create(contrast->panel,PANEL_CHOICE_STACK,
			      PANEL_LAYOUT,PANEL_HORIZONTAL,
			      PANEL_LABEL_STRING,"CUR",
			      XV_X,temp,XV_Y,row,
			      PANEL_NOTIFY_PROC,contrast_label_proc,
			      PANEL_CLIENT_DATA,contrast,
			      XV_KEY_DATA,1,0,
			      PANEL_VALUE,0,
			      0);
  contrast_setup_scale_labels(contrast);
  PRT3(printf("setup_contrast_proc...1\n"));
  row += offset;
  contrast->percen = xv_create(contrast->panel,PANEL_MESSAGE,
			       XV_X,5,XV_Y,row,
			       0);

 contrast->mid = xv_create(contrast->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"MID",
	PANEL_VALUE,
          (int) scale->mid_value,
	PANEL_MIN_VALUE,first,
	PANEL_MAX_VALUE,(int) (first + num_colors - 1),
	PANEL_SLIDER_WIDTH,85,
	PANEL_NOTIFY_PROC,notify_mid_contrast_proc,
        XV_X,130,XV_Y,row,
        PANEL_CLIENT_DATA,contrast,
	0);
 row += offset - 3;
 contrast->range = xv_create(contrast->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"RANGE",
	PANEL_VALUE,(int) (scale->range),
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,(int) (num_colors - 1),
	PANEL_SLIDER_WIDTH,100,
        XV_X,5,XV_Y,row,
	PANEL_NOTIFY_PROC,notify_range_contrast_proc,
        PANEL_CLIENT_DATA,contrast,
	0);
 row += offset;
 contrast->lb = xv_create(contrast->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"LB",
	PANEL_SHOW_RANGE,FALSE,
	PANEL_VALUE,scale->lb,
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,(int) (num_colors - 1),
	PANEL_SLIDER_WIDTH,50,
        XV_X,15,XV_Y,row,
	PANEL_NOTIFY_PROC,contrast_lb_ub_proc,
        PANEL_CLIENT_DATA,contrast,
	0);
 PRT3(printf("setup_contrast_proc...2\n"))

 contrast->ub = xv_create(contrast->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"UB",
	PANEL_SHOW_RANGE,FALSE,
	PANEL_VALUE,scale->ub,
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,num_colors - 1,
	PANEL_SLIDER_WIDTH,50,
        XV_Y,row,
	PANEL_NOTIFY_PROC,contrast_lb_ub_proc,
        PANEL_CLIENT_DATA,contrast,
	0);
 row += offset - 7;
 contrast->GT_ub_background = xv_create(contrast->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,"(> MID + UB) = BACKGROUND",NULL,
	PANEL_NOTIFY_PROC,contrast_lb_ub_proc,
        PANEL_CLIENT_DATA,contrast,
        PANEL_VALUE,0,
        XV_X,15,XV_Y,row,
        0);
 if (scale->status & COLOR_CONTROL_EXCLUDE_BKG)
    xv_set(contrast->GT_ub_background,PANEL_VALUE,1,0);


 row += offset;
 contrast->min_max = xv_create(contrast->panel,PANEL_CHOICE_STACK,
                  PANEL_LAYOUT,PANEL_VERTICAL,
                  PANEL_CHOICE_STRINGS,"USE GLOBAL MIN MAX",
                           "USE SLICE MIN MAX",
                           "USE SET MIN MAX",NULL,
                  XV_X,5,XV_Y,row,
                  PANEL_NOTIFY_PROC,min_max_proc,
                  PANEL_CLIENT_DATA,contrast,
                  0);
 rr = (Rect *) xv_get(contrast->min_max,XV_RECT);
 contrast->apply_to = xv_create(contrast->panel,PANEL_CHOICE_STACK,
                  PANEL_LAYOUT,PANEL_VERTICAL,
                  PANEL_CHOICE_STRINGS,"CUR WINDOW",
                           "CUR FILE",
                           "ALL FILES",NULL,
                  PANEL_VALUE,0,
                  XV_X,rr->r_left + rr->r_width + 10,XV_Y,row,
                  PANEL_CLIENT_DATA,contrast,
                  0);
 row += offset;
 GetMinMaxValues(&(img->_2d),&min,&max);
 PRT2(printf("min = %f  max = %f\n",min,max))

 contrast->min = xv_create(contrast->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"SET MIN",
	PANEL_MIN_VALUE,(long) min,PANEL_MAX_VALUE,(long) max,
	PANEL_NOTIFY_PROC,notify_minmax_proc,
	PANEL_SLIDER_WIDTH,80,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,contrast,
	0);

 row += offset;
 contrast->max = xv_create(contrast->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"MAX",
	PANEL_MIN_VALUE,(long) min,PANEL_MAX_VALUE,(long) max,
	PANEL_NOTIFY_PROC,notify_minmax_proc,
	PANEL_SLIDER_WIDTH,80,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,contrast,
	0);
 rr = (Rect *) xv_get(contrast->max,XV_RECT);
 xv_create(contrast->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"MORE...",
                   PANEL_NOTIFY_PROC,more_commands_proc,
                   XV_X,rr->r_left + rr->r_width + 10,XV_Y,row,
                   PANEL_CLIENT_DATA,contrast,
                   0);
 cur_contrast_settings(contrast);
 set_contrast_item_values(contrast);

 window_fit(contrast->panel);
 xv_set(contrast->panel,XV_HEIGHT,rr->r_top + rr->r_height + 5,0);
 window_fit(contrast->frame);
 make_2_wins_adjacent(contrast->frame,win->dwin->frame,0);
 win_set_mouse_at_center(contrast->frame);
 xv_set(contrast->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 PRT3(printf("setup_contrast_proc...4\n"))
 return NO_ERROR;
}


/*------------------------------------------------------------------------
pick_non_background_pixels - ???
------------------------------------------------------------------------*/

int
pick_non_background_pixels(win)
 struct image_display_window *win;
{
 unsigned long row,col;
 struct buf_2d_rec *_2d;
 double val;
 unsigned long num,indx,first,num_indxs,*color_indxs;
 struct color_scale_rec *scale;
 struct color_rec *color;
 unsigned short status;
 int err;
 struct roi_rec *roi;
 struct curve_rec *pt_curve;
 struct color_data_rec *color_data;

 TRACE(printf("pick_non_background_pixels...1\n"))
 if ((color_data = WinCurColorData(win)) == NULL) return CANCEL;
 pt_curve = NULL;
 if (((err = setup_curve_rec_ptr(&pt_curve,DRAW_POINT)) != NO_ERROR) ||
     ((err = WinCurScaleAndColorIndxs(win,NULL,&scale,&color_indxs,
            &num_indxs)) != NO_ERROR))
    {c_free(pt_curve);
     return err;}
 first = scale->first;
 _2d = &(color_data->_2d);
 color = &(win->intensity->color);
 for (row = 0; row < _2d->dimy; row++)
    for (col = 0; col < _2d->dimx; col++)
       {val = _2d->GetVal(_2d,row,col);
        indx = (unsigned long) rint((double) val);
        if ((num_indxs > 0) && (indx >= first) &&
            ((num = (indx - first)) < num_indxs))
           indx = color_indxs[num];
         if ((indx < color->total_colors) &&
             ((indx == first) ||
              (color->color_status[indx] & BACKGROUND_COLOR)))
           {}
          else if ((err = EnterCurveXY(pt_curve,
                 (float) col,(float) row, &status,NULL,0,0,0)) != NO_ERROR)
              break;
       }
 TRACE(printf("pick_non_background_pixels...2\n");)
 if (err == NO_ERROR)
    if ((err = setup_more_rois(&(win->control.sroi.sroi_win->slice_roi),
                    NON_BACKGROUND,1)) == NO_ERROR)
       {if ((roi = get_roi(&(win->control.sroi.sroi_win->slice_roi),
                       NON_BACKGROUND)) != NULL) 
           {pt_curve->status |= CURVE_OR;
            err = append_curve_to_roi_rec(roi,pt_curve,
                  (unsigned short) DRAW_POINT,1,
             &(win->control.sroi.sroi_win->slice_roi.status));
            if (err == NO_ERROR) 
               win_execute_proc_to_related_wins(win,win_redisplay_all_curves,
						0,0);
           }
       }
 TRACE(printf("pick_non_background_pixels...err = %d\n",err);)
 if (err != NO_ERROR)
    {free_curve_rec(pt_curve);
     c_free(pt_curve);}
 return err;
}


/*------------------------------------------------------------------------
map_non_background_pixels - ???
------------------------------------------------------------------------*/

int
map_non_background_pixels(win)
 struct image_display_window *win;
{
 struct roi_rec *roi;
 unsigned long row,col;
 struct buf_2d_rec *_2d,_2d1;
 double val;
 unsigned long num,indx,first,num_indxs,*color_indxs;
 struct color_scale_rec *scale;
 struct color_rec *color;
 int err,yes;
 struct int_bound_rec affected;
 struct color_data_rec *color_data;

 TRACE(printf("map_non_background_pixels...1\n"))
 if ((color_data = WinCurColorData(win)) == NULL) return CANCEL;
 if ((err = WinCurScaleAndColorIndxs(win,NULL,&scale,&color_indxs,
            &num_indxs)) != NO_ERROR)
    return err;
 if ((roi = next_cur_roi(
       &(win->control.sroi.sroi_win->slice_roi),NON_BACKGROUND))
                   != NULL)
    {if (is_roi_non_empty(roi))
        return roi_to_histo_roi(win,NULL,HISTO_ROI_POINTS,HISTO_NON_BACKGROUND,NULL);
    }
 init_buf_2d_rec(&_2d1);
 if ((err = SetupBuf2dRec(&_2d1,color_data->_2d.dimx,
                 color_data->_2d.dimy,1,1)) != NO_ERROR)
    return err;
 TRACE(printf("map_non_background_pixels...2\n");)
 _2d = &(color_data->_2d);
 color = &(win->intensity->color);
 first = scale->first;

 affected.left = affected.top = 1;
 affected.right = affected.bottom = 0;
 for (yes = 0,row = 0; row < _2d->dimy; row++)
    for (col = 0; col < _2d->dimx; col++)
       {val = _2d->GetVal(_2d,row,col);
        indx = (unsigned long) rint((double) val);
        if ((num_indxs > 0) && (indx >= first) &&
            ((num = (indx - first)) < num_indxs))
           indx = color_indxs[num];
        if ((indx < color->total_colors) &&
            ((indx == first) ||
             (color->color_status[indx] & BACKGROUND_COLOR)))
           {}
          else
           {_2d1.SetVal(&_2d1,row,col,(double) 1.0);
            if (affected.left > col) affected.left = col;
            if (affected.right < col) affected.right = col;
            if (affected.top > row) affected.top = row;
            if (affected.bottom < row) affected.bottom = row;
            yes = 1;
          }
       }
 TRACE(printf("map_non_background_pixels...yes = %d\n",yes))
 if (yes)
    err = win_mask_to_histo_roi(win,&_2d1,&affected,
               HISTO_NON_BACKGROUND,HISTO_ROI_POINTS,NULL);
   else ShowMsgStrs(win->dwin->frame,0,NO_ERROR,0,2,
               "All pixels are background pixels",
               "No pixels to map !!!",NULL,0);
 TRACE(printf("map_non_background_pixels...err = %d\n",err);)
 free_buf_2d_rec(&_2d1);
 return err;
}


/*------------------------------------------------------------------------
contrast_mouse_proc - ???
------------------------------------------------------------------------*/

int
contrast_mouse_proc(win,event)
 struct image_display_window *win;
 Event *event;
{
 struct all_contrast_rec_recs *all_contrasts;
 struct contrast_rec *contrast;
 struct color_scale_rec *scale;
 double val;
 long x,y;
 int err;
 struct color_data_rec *color_data;

 if ((color_data = WinCurColorData(win)) == NULL) return CANCEL;
 if ((scale = win_cur_scale(win)) == NULL) return CANCEL;
 set_win_busy(win);
 err = NO_ERROR;
 if (event_left_is_down(event))
    {TRACE(printf("contrast_mouse_proc...event_left_is_down\n");)
     if (dwin_get_coord_xy_from_event(win->dwin,event,&x,&y) != NO_ERROR)
        return CANCEL;
     if (color_data->_2d.GetVal(&(color_data->_2d),y,x,&val) == NO_ERROR)
        {scale->mid_value = (unsigned long) rint((double) val);
         scale->range = 0;
         scale->lb = 0;
         scale->ub = 0;
         win_define_color_map(win,1);
        }
     if (((all_contrasts = (struct all_contrast_rec_recs *) 
               enter_image_contrast(win,GET_CONSTRAST_PTR)) != NULL) &&
         ((contrast != find_contrast_rec(all_contrasts,win)) != 0))

        {xv_set(contrast->mid,PANEL_VALUE,scale->mid_value,0);
         xv_set(contrast->range,PANEL_VALUE,0,0);
         xv_set(contrast->lb,PANEL_VALUE,0,0);
         xv_set(contrast->ub,PANEL_VALUE,0,0);
         xv_set(contrast->GT_ub_background,PANEL_VALUE,1,0);
        }
    }
   else if (event_middle_is_down(event))
    {TRACE(printf("contrast_mouse_proc...event_middle_is_down\n");)
     pick_non_background_pixels(win);
    }
   else if (event_right_is_down(event))
    {TRACE(printf("contrast_mouse_proc...event_right_is_down\n");)
     map_non_background_pixels(win);
    }
   else err = CANCEL;
 set_win_not_busy(win);
 return err;
}



/*------------------------------------------------------------------------
find_contrast_rec - ???
------------------------------------------------------------------------*/

struct contrast_rec *
find_contrast_rec(all_contrasts,win)
     struct all_contrast_rec_recs *all_contrasts;
     struct image_display_window *win;
{
  int lcv;

  for (lcv = 0; lcv < all_contrasts->num_contrasts; lcv++)
    if (all_contrasts->contrasts[lcv]->win == win)
      return all_contrasts->contrasts[lcv];
  return NULL;
}




/*------------------------------------------------------------------------
win_update_cur_contrast_settings - ???
------------------------------------------------------------------------*/

void
win_update_cur_contrast_settings(win)
 struct image_display_window *win;
{
 struct all_contrast_rec_recs *all_contrasts;
 struct contrast_rec *contrast;

 PRT2(printf("win_update_cur_contrast_settings...1\n"))
 if (((all_contrasts = (struct all_contrast_rec_recs *) 
               enter_image_contrast(win,GET_CONSTRAST_PTR)) != NULL) &&
     ((contrast = find_contrast_rec(all_contrasts,win)) != NULL))
    {PRT2(printf("win_update_cur_contrast_settings...contrast = %u\n",contrast))
     cur_contrast_settings(contrast);}
 PRT2(printf("win_update_cur_contrast_settings...end\n"))
}


/*------------------------------------------------------------------------
win_update_cur_contrast - ???
------------------------------------------------------------------------*/

void
win_update_cur_contrast(win)
 struct image_display_window *win;
{
 struct all_contrast_rec_recs *all_contrasts;
 struct contrast_rec *contrast;
 struct color_scale_rec *scale;

 if ((scale = win_cur_scale(win)) == NULL) return /*CANCEL*/;
 if (!(scale->status & COLOR_LOCAL))
    win_update_color_opr(win);
 else if (((all_contrasts = (struct all_contrast_rec_recs *) 
               enter_image_contrast(win,GET_CONSTRAST_PTR)) != NULL) &&
     ((contrast = find_contrast_rec(all_contrasts,win)) != NULL))
    {contrast_setup_scale_labels(contrast);
     cur_contrast_settings(contrast);}
}


/*------------------------------------------------------------------------
win_update_contrast - ???
------------------------------------------------------------------------*/

void
win_update_contrast(win)
 struct image_display_window *win;
{
 struct all_contrast_rec_recs *all_contrasts;
 int lcv;

 PRT2(printf("win_update_contrast...\n"))
 if ((all_contrasts = (struct all_contrast_rec_recs *) 
               enter_image_contrast(win,GET_CONSTRAST_PTR)) != NULL)
    {PRT2(printf("win_update_contrast...all_contrasts->num_contrasts = %d\n",
             all_contrasts->num_contrasts))
     for (lcv = 0; lcv < all_contrasts->num_contrasts; lcv++)
        {contrast_setup_scale_labels(all_contrasts->contrasts[lcv]);
         cur_contrast_settings(all_contrasts->contrasts[lcv]);}
    }
 PRT2(printf("win_update_contrast...end\n"))
}




/*------------------------------------------------------------------------
enter_image_contrast - ???

Called from image_contrast_proc callback in img_menu.c
------------------------------------------------------------------------*/

int
enter_image_contrast( win, status )
     struct image_display_window *win;
     int status;
{
  static struct all_contrast_rec_recs all_contrasts;
  struct contrast_rec *contrast;
  static char first_time = 1;
  int err;

  err = NO_ERROR;
  if (first_time)
  {
    init_all_contrast_rec_recs(&all_contrasts);
    first_time = 0;
  }
  TRACE(printf("enter_image_contrast num_contrasts = %d\n",
	       all_contrasts.num_contrasts));

  if (status == GET_CONSTRAST_PTR)
  {
    return (unsigned long) &all_contrasts;     /* FIXME:  ptr as long */
  }

  else if (status == CLEAN_UP_IMAGE_CONSTRAST)
  {
    if (!first_time)
      if ((contrast = find_contrast_rec(&all_contrasts,win)) != NULL)
	free_contrast_rec(contrast);
    return NO_ERROR;
  }

  else if (status == BRING_UP_IMAGE_CONSTRAST)
  {
    if (!first_time)
      if ((contrast = find_contrast_rec(&all_contrasts,win)) != NULL)
	cur_contrast_settings(contrast);
    return NO_ERROR;
  }

  if ((contrast = find_contrast_rec(&all_contrasts,win)) != NULL)
  {
    TRACE(printf("contrast = %u\n",contrast));
    xv_set(contrast->frame,
	   XV_SHOW,TRUE,WIN_ALARM,FRAME_CLOSED,FALSE,0);
    win_set_mouse_at_center(contrast->frame);
    wmgr_top(contrast->frame);
    return NO_ERROR;
  }
  TRACE(printf("contrast = NOT FOUND\n"));

  get_handle_memory(&(all_contrasts.contrasts),
		    &(all_contrasts.num_contrasts),1,
		    (size_t) sizeof(**(all_contrasts.contrasts)),&err);
  if (err != NO_ERROR)
    return err;
  contrast = all_contrasts.contrasts[all_contrasts.num_contrasts - 1];
  init_contrast_rec(contrast);
  TRACE(printf("contrast...num_contrasts = %d\n",all_contrasts.num_contrasts));
  if ((err = setup_contrast_proc(contrast,win)) != NO_ERROR)
    ShowMsgStrs(win->dwin->frame,0,err,0,2,
		"An error occured when setting up",
		"   the contrast window !!!",NULL,0);
  TRACE(printf("contrast...2\n"));
  return err;
}
