/****************************************************************************
cursor.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: cursor.c,v 1.9 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/cursor.h>
#include <xview/svrimage.h>

#include "mri_format.h"
#include "nria_procedures.h"


#if 0
#  define TRACE(x) x; fflush(stdout);
#  define PRT(x) x; fflush(stdout);
#  define PRT1(x) x; fflush(stdout);
#  define PRT2(x) x; fflush(stdout);
#  define PRT3(x) x; fflush(stdout);
#  define PRT4(x) x; fflush(stdout);
#else
#  define TRACE(x)
#  define PRT(x)
#  define PRT1(x)
#  define PRT2(x)
#  define PRT3(x)
#  define PRT4(x)
#endif


#define DRAW_LABEL_DISABLED 1
#define DRAW_LABEL_ENABLED 2
#define DRAW_LABEL_UPDATE 4
#define LAST_CURSOR 8
#define ENABLE_3D_DRAW_MODE 0x0010

#define DRAW_LABEL_STATUS 0x0007
#define DISABLED_AND_LAST 0x0009
#define ENABLED_AND_LAST 0x000a
#define DISABLED_LAST_3D 0x0019
#define ENABLED_LAST_3D 0x001a


struct cursor_action_rec
{
  Xv_Cursor (*cursor)();	/* Pointer to routine which returns
				 *  pointer to actual cursor data.  */

  int			(*last_check_proc)();
  int			(*check_proc)();
  int			(*setup_proc)();
  int			(*action)();

  unsigned short	mode;
  struct int_bound_rec	bound;
};



/* Public functions defined in this file */
int win_set_3d_pt_cursor_proc(struct image_display_window *win,
    int last_which_cursor);
int win_check_disable_pt_cursor(struct image_display_window *win,
    int last_which_cursor);
int get_cursor(struct image_display_window *win, unsigned long select,
    Xv_Cursor *cursor, void **action);
int dwin_default_cursor(struct display_win_rec *win);
int win_set_cursor(struct image_display_window *win, Xv_Cursor cursor);
int win_select_cursor(struct image_display_window *win, unsigned long select);
int win_set_def_cursor(struct image_display_window *win);
int win_get_cursor_xy_and_bound(struct image_display_window *win,
    Event *event, long *x, long *y, int *use_this_xybound,
    struct int_bound_rec *xybound);


/* Private functions */
static Xv_Cursor cursor_rotate(void);
static Xv_Cursor cursor_rotate_vertical(void);
static Xv_Cursor cursor_rotate_horizontal(void);
static Xv_Cursor cursor_pen(void);
static Xv_Cursor cursor_cut(void);
static Xv_Cursor cursor_angle_cut(void);
static Xv_Cursor cursor_0(void);
static Xv_Cursor cursor_90(void);
static Xv_Cursor cursor_180(void);
static Xv_Cursor cursor_270(void);
static Xv_Cursor cursor_R90(void);
static Xv_Cursor cursor_R180(void);
static Xv_Cursor cursor_R270(void);
static Xv_Cursor cursor_x_reflection(void);
static Xv_Cursor cursor_y_reflection(void);
static Xv_Cursor cursor_yx_reflection(void);
static Xv_Cursor cursor_y_x_reflection(void);
static Xv_Cursor cursor_no_reflection(void);
static Xv_Cursor cursor_clockwise(void);
static Xv_Cursor cursor_anticlockwise(void);
static Xv_Cursor cursor_delete_roi(void);
static Xv_Cursor cursor_move_roi(void);
static Xv_Cursor cursor_resize_roi(void);
static Xv_Cursor cursor_rotate_roi(void);
static Xv_Cursor cursor_magnify_plus_2(void);
static Xv_Cursor cursor_magnify_minus_2(void);
static Xv_Cursor cursor_pt_scan(void);
static Xv_Cursor cursor_locate_plane(void);
static Xv_Cursor cursor_move_superimpose(void);
static Xv_Cursor cursor_3d_pt1(void);
static Xv_Cursor cursor_3d_pt2(void);
static Xv_Cursor cursor_3d_pt3(void);
static Xv_Cursor cursor_same_angle_cut(void);




/* Table of cursors */
static struct cursor_action_rec
nriaCursors[] =
{

  /*  Default cursor  */
  {NULL,
   NULL, NULL, NULL, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  /*  Pen cursor  */
  {cursor_pen,
   NULL, NULL, NULL, NULL,
   ENABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_cut,
   NULL, NULL, NULL, NULL,
   ENABLED_LAST_3D,
   {0,0,15,15}},
  
  /*  Curved arrow cursor  */
  {cursor_rotate,
   NULL, NULL, NULL, NULL,
   ENABLED_LAST_3D,
   {0,0,14,15}},
  
  {cursor_rotate_vertical,
   NULL, NULL, NULL, NULL,
   ENABLED_LAST_3D,
   {0,0,14,15}},
  
  {cursor_rotate_horizontal,
   NULL, NULL, NULL, NULL,
   ENABLED_AND_LAST,
   {0,0,14,15}},
  
  /*  "0" Text cursor  */
  {cursor_0,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  /*  "90" Text cursor  */
  {cursor_90,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  /*  "180" Text cursor  */
  {cursor_180,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  /*  "270" Text cursor  */
  {cursor_270,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_R90,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_R180,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_R270,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  /*  Two-way arrows, horizontal cursor  */
  {cursor_x_reflection,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  /*  Two-way arrows, vertical cursor  */
  {cursor_y_reflection,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  /*  Two-way arrows, diagonal (  )  cursor  */
  {cursor_yx_reflection,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_y_x_reflection,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_no_reflection,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_clockwise,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_anticlockwise,
   NULL, NULL, NULL, NULL,
   0,
   {0,0,15,15}},
  
  {cursor_angle_cut,
   NULL, NULL, NULL, NULL,
   ENABLED_AND_LAST,
   {0,0,15,15}},
  
  /*  X-cross cursor  */
  {cursor_delete_roi,
   NULL, NULL, NULL, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  /*  cursor  */
  {cursor_move_roi,
   NULL, NULL,setup_trans_roi_rec, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_resize_roi,
   NULL, NULL,setup_trans_roi_rec, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_rotate_roi,
   NULL, NULL,setup_trans_roi_rec, NULL,
   DISABLED_AND_LAST,
   {0,0,14,15}},
  
  {cursor_magnify_plus_2,
   NULL, NULL, NULL, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_magnify_minus_2,
   NULL, NULL, NULL, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_pt_scan,
   NULL, NULL, NULL, NULL,
   DISABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_locate_plane,
   NULL, NULL, NULL, NULL,
   ENABLED_AND_LAST,
   {0,1,15,13}},
  
  {cursor_move_superimpose,
   NULL, NULL, NULL, NULL,
   DRAW_LABEL_DISABLED,
   {0,0,15,15}},
  
  {cursor_3d_pt1,
   win_check_disable_pt_cursor, win_set_3d_pt_cursor_proc, NULL, NULL,
   ENABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_3d_pt2,
   win_check_disable_pt_cursor, win_set_3d_pt_cursor_proc, NULL, NULL,
   ENABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_3d_pt3,
   win_check_disable_pt_cursor, win_set_3d_pt_cursor_proc, NULL, NULL,
   ENABLED_AND_LAST,
   {0,0,15,15}},
  
  {cursor_same_angle_cut,
   NULL, NULL, NULL, NULL,
   ENABLED_AND_LAST,
   {0,5,15,11}},
};


/*------------------------------------------------------------------------
cursor_rotate - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_rotate()
{
  static unsigned short cursor_bits[] = {
    0x3000, 0x7000, 0xFFFC, 0x7FFE, 0x3006, 0x3006,
    0x0006, 0x6006, 0x6006, 0x6006, 0x6006, 0x6006,
    0x6006, 0x6006, 0x7FFE, 0x3FFC
  };
  static Server_image img = 0;
  static Xv_Cursor cursor = 0;

  if (img == 0)
  {
    img = (Server_image)
      xv_create( XV_NULL, SERVER_IMAGE,
		 XV_WIDTH, 16,
		 XV_HEIGHT, 16,
		 SERVER_IMAGE_BITS, cursor_bits,
		 NULL );
    cursor = (Xv_Cursor)
      xv_create( XV_NULL, CURSOR,
		 CURSOR_IMAGE, img,
		 CURSOR_XHOT, 0,
		 CURSOR_YHOT, 2,
		 CURSOR_OP, PIX_SRC ^ PIX_DST,
		 NULL );
  }

  TRACE(printf("cursor_rotate...\n"));
  return cursor;
}



/*------------------------------------------------------------------------
cursor_rotate_vertical - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_rotate_vertical()
{
 static unsigned short cursor_bits[] = {
   0x3000, 0x7000, 0xFFFC, 0x7FFE, 0x3006, 0x3006,
   0x0186, 0x6186, 0x6186, 0x6186, 0x6186, 0x6186,
   0x6006, 0x6006, 0x7FFE, 0x3FFC};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_XHOT,0,CURSOR_YHOT,2,
                  CURSOR_OP,PIX_SRC ^ PIX_DST,0);}
 TRACE(printf("cursor_rotate_vertical...\n"))
 return cursor;
}



/*------------------------------------------------------------------------
cursor_rotate_horizontal - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_rotate_horizontal()
{
 static unsigned short cursor_bits[] = {
   0x3000, 0x7000, 0xFFFC, 0x7FFE, 0x3006, 0x3006,
   0x0006, 0x6006, 0x67E6, 0x67E6, 0x6006, 0x6006,
   0x6006, 0x6006, 0x7FFE, 0x3FFC};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_XHOT,0,CURSOR_YHOT,2,
                  CURSOR_OP,PIX_SRC ^ PIX_DST,0);}
 TRACE(printf("cursor_rotate_horizontal...\n"))
 return cursor;
}



/*------------------------------------------------------------------------
cursor_pen - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_pen()
{
 static unsigned short cursor_bits[] = {
    0x8000,0x6000,0x7800,0x3C00,0x3E00,0x1F00,0x0F80,0x07C0,
    0x03E0,0x01F0,0x00F8,0x007C,0x003C,0x001E,0x0007,0x0003};
/*
    0xFE00,0xFF00,0xFF80,0xFFC0,0xF060,0xF030,0x7018,0x300C,
    0x1806,0x0C03,0x0603,0x0303,0x0187,0x00CE,0x007C,0x0038};
*/
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 TRACE(printf("cursor_pen...\n"))
 return cursor;
}



/*------------------------------------------------------------------------
cursor_cut - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_cut()
{
 static unsigned short cursor_bits[] = {
   0xE000, 0xF000, 0xD800, 0xCC00, 0xC600, 0xE300,
   0x7180, 0x38C0, 0x1CE0, 0x0FF0, 0x0778, 0x007C,
   0x003E, 0x001F, 0x000F, 0x0007};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,0,CURSOR_YHOT,0,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 TRACE(printf("cursor_cut...\n"))
 return cursor;
}



/*------------------------------------------------------------------------
cursor_angle_cut - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_angle_cut()
{
 static unsigned short cursor_bits[] = {
   0x0007, 0x000F, 0x001F, 0x003E, 0x007C, 0x00F8,
   0x01F0, 0x03C0, 0x0660, 0x0C60, 0x18E0, 0x31C0,
   0x6380, 0xC700, 0xFE00, 0xFC00};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,0,CURSOR_YHOT,15,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 TRACE(printf("cursor_angle_cut...\n"))
 return cursor;
}



/*------------------------------------------------------------------------
cursor_0 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_0()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0000, 0x03C0, 0x0420, 0x0420, 0x0420,
   0x0420, 0x0420, 0x0420, 0x0420, 0x0420, 0x0420,
   0x0420, 0x03C0, 0x0000, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_90 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_90()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0000, 0x1C38, 0x2244, 0x2244, 0x2244,
   0x2244, 0x1E44, 0x0244, 0x0244, 0x2244, 0x2244,
   0x2244, 0x1C38, 0x0000, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_180 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_180()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0000, 0x470C, 0x4892, 0x4892, 0x4892,
   0x4892, 0x4712, 0x4712, 0x4892, 0x4892, 0x4892,
   0x4892, 0x470C, 0x0000, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_270 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_270()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0000, 0x31CC, 0x4A52, 0x4A52, 0x4A52,
   0x0A52, 0x1052, 0x2052, 0x4052, 0x4052, 0x4052,
   0x4052, 0x784C, 0x0000, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_R90 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_R90()
{
 static unsigned short cursor_bits[] = {
   0x4002, 0xFFFF, 0x4002, 0x1C38, 0x2244, 0x2244,
   0x2244, 0x2244, 0x1E44, 0x0244, 0x0244, 0x2244,
   0x2244, 0x2244, 0x1C38, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_R180 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_R180()
{
 static unsigned short cursor_bits[] = {
   0x4002, 0xFFFF, 0x4002, 0x0000, 0x470C, 0x4892,
   0x4892, 0x4892, 0x4892, 0x4712, 0x4712, 0x4892,
   0x4892, 0x4892, 0x4892, 0x470C};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_R270 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_R270()
{
 static unsigned short cursor_bits[] = {
   0x4002, 0xFFFF, 0x4002, 0x0000, 0x31CC, 0x4A52,
   0x4A52, 0x4A52, 0x0A52, 0x1052, 0x2052, 0x4052,
   0x4052, 0x4052, 0x4052, 0x784C};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
 cursor_x_reflection- ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_x_reflection()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0810, 0x0838, 0x087C, 0x0810, 0x0810,
   0x0810, 0x7FFE, 0x7FFE, 0x0810, 0x0810, 0x0810,
   0x3E10, 0x1C10, 0x0810, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_y_reflection - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_y_reflection()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0180, 0x1180, 0x3180, 0x7FFE, 0x3180,
   0x1180, 0x0180, 0x0180, 0x0188, 0x018C, 0x7FFE,
   0x018C, 0x0188, 0x0180, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_yx_reflection - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_yx_reflection()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x6010, 0x7020, 0x3840, 0x1C8E, 0x0F06,
   0x070A, 0x0790, 0x09E0, 0x50F0, 0x60F0, 0x7138,
   0x021C, 0x040E, 0x0806, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_y_x_reflection - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_y_x_reflection()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x0E06, 0x0C0E, 0x0A1C, 0x4178, 0x20F0,
   0x10E0, 0x09E0, 0x0790, 0x0708, 0x0F04, 0x1C82,
   0x3850, 0x7030, 0x6070, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_no_reflection - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_no_reflection()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x7FFE, 0x4002, 0x4002, 0x4002, 0x4002,
   0x4002, 0x4002, 0x4002, 0x4002, 0x4002, 0x4002,
   0x4002, 0x4002, 0x7FFE, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_clockwise - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_clockwise()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x41F0, 0x2378, 0x1618, 0x0C18, 0x1C7E,
   0x323C, 0x3118, 0x3080, 0x6046, 0x602E, 0x301C,
   0x3078, 0x3FE4, 0x0F82, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_anticlockwise - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_anticlockwise()
{
 static unsigned short cursor_bits[] = {
   0x0000, 0x41F0, 0x23F8, 0x160C, 0x0C0C, 0x1C0C,
   0x320C, 0x310C, 0x308C, 0x304C, 0xFC2C, 0x7818,
   0x3068, 0x01E4, 0x0182, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_delete_roi - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_delete_roi()
{
 static unsigned short cursor_bits[] = {
   0xE007, 0xF00F, 0xF81F, 0x7C3E, 0x3E7C, 0x1FF8,
   0x0C30, 0x05A0, 0x05A0, 0x0C30, 0x1FF8, 0x3E7C,
   0x7C3E, 0xF81F, 0xF00F, 0xE007};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,7,CURSOR_YHOT,7,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_move_roi - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_move_roi()
{
 static unsigned short cursor_bits[] = {
   0xC000, 0xF000, 0x7C00, 0x7F80, 0x3F80, 0x3E3C,
   0x1F7C, 0x1BDC, 0x199C, 0x031C, 0x061C, 0x07FC,
   0x07FC, 0x07FC, 0x0000, 0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_resize_roi - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_resize_roi()
{
 static unsigned short cursor_bits[] = {
   0xFFF8, 0x8248, 0x8048, 0x8248, 0x8048, 0x8248,
   0xD648, 0x8048, 0x8048, 0xFFC8, 0x8008, 0x8008,
   0xFFF8, 0x0005, 0x0003, 0x0007};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_rotate_roi - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_rotate_roi()
{
 static unsigned short cursor_bits[] = {
   0xFFFE, 0x8002, 0x8402, 0x8C02, 0x9FE2, 0xBFF2,
   0x9C3A, 0x8C1A, 0x841A, 0x801A, 0x981A, 0x9C3A,
   0x8FF2, 0x87E2, 0x8002, 0xFFFE};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_magnify_plus_2 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_magnify_plus_2()
{
 static unsigned short cursor_bits[] = {
	0x0FC0,0x3FF0,0x7038,0xE01C,0xC30E,0xC306,
        0xCFC6,0xCFC6,0xC306,0xE30E,0x601C,0x7878,
	0x3FFC,0x0F9E,0x000F,0x0007};
/*
	0x0FC0,0x3FF0,0x7038,0xE49C,0xC5CE,0xC426,
	0xDF26,0xC446,0xC486,0xE5EE,0x601C,0x7878,
	0x3FFC,0x0F9E,0x000F,0x0007};
*/
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_magnify_minus_2 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_magnify_minus_2()
{
 static unsigned short cursor_bits[] = {
	0x0FC0,0x3FF0,0x7038,0xE01C,0xC00E,0xC006,
	0xCFC6,0xCFC6,0xC006,0xE00E,0x601C,0x7878,
	0x3FFC,0x0F9E,0x000F,0x0007};
/*
	0x0FC0,0x3FF0,0x7038,0xE09C,0xC1CE,0xC026,
	0xDF26,0xC046,0xC086,0xE1EE,0x601C,0x7878,
	0x3FFC,0x0F9E,0x000F,0x0007};
*/
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_pt_scan - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_pt_scan()
{
 static unsigned short cursor_bits[] = {
	0x0002,0x0007,0x001E,0x003C,0x007C,0x00F8,
	0x01F0,0x03E0,0x07C0,0x3F80,0x4F00,0x0E00,
	0x1200,0x2200,0x4200,0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,1,CURSOR_YHOT,14,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_locate_plane - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_locate_plane()
{
 static unsigned short cursor_bits[] = {
	0x0000,0x0180,0x03C0,0x0FF0,0x1188,0x0180,
	0x7FFE,0xFFFF,0xC183,0xC993,0xCDB3,0xC3C3,
	0xFFFF,0x7FFE,0x0000,0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,7,CURSOR_YHOT,1,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_move_superimpose - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_move_superimpose()
{
 static unsigned short cursor_bits[] = {
	0x0180,0x01B0,0x0DB0,0x0DB6,0x0DB6,0x0DB6,
	0xCDB6,0xEDB6,0xEFF6,0xEFFE,0x7FFF,0x7FFF,
	0x3FFF,0x3FFE,0x1FFC,0x0FF8};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,0,CURSOR_YHOT,6,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_3d_pt1 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_3d_pt1()
{
 static unsigned short cursor_bits[] = {
	0xC000,0xF000,0x7800,0x7C00,0x7C00,0x7E03,
	0x6F07,0x678F,0x63DE,0x01FC,0x00F8,0x00F0,
	0x01E0,0x03C0,0x0380,0x0300};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,0,CURSOR_YHOT,0,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_3d_pt2 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_3d_pt2()
{
 static unsigned short cursor_bits[] = {
	0xC000,0xFF00,0x7F80,0x7380,0x6180,0x6181,
	0x6183,0x6187,0x618E,0x619C,0x01F8,0x01F0,
	0x01E0,0x01C0,0x0180,0x0100};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,0,CURSOR_YHOT,0,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_3d_pt3 - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_3d_pt3()
{
 static unsigned short cursor_bits[] = {
	0xC000,0xFF00,0x7F00,0x7300,0x633C,0x63FE,
	0x63C7,0x6787,0x060E,0x001C,0x0038,0x0070,
	0x00E0,0x01C0,0x0180,0x0100};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,0,CURSOR_YHOT,0,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
cursor_same_angle_cut - ???
------------------------------------------------------------------------*/

static Xv_Cursor
cursor_same_angle_cut()
{
 static unsigned short cursor_bits[] = {
	0x0000,0x0000,0x0000,0x0000,0x0000,0x7FFF,
	0xFFFF,0xC03F,0xC030,0xE070,0x7FE0,0x1FC0,
	0x0000,0x0000,0x0000,0x0000};
 static Server_image img = 0;
 static Xv_Cursor cursor = 0;

 if (img == 0)
    {img = (Server_image) xv_create(XV_NULL,SERVER_IMAGE,
             XV_WIDTH,16,XV_HEIGHT,16,SERVER_IMAGE_BITS,cursor_bits,0);
     cursor = (Xv_Cursor) xv_create(XV_NULL,CURSOR,
                  CURSOR_XHOT,3,CURSOR_YHOT,11,
                  CURSOR_IMAGE,img,CURSOR_OP,PIX_SRC ^ PIX_DST,0);
    }
 return cursor;
}



/*------------------------------------------------------------------------
win_set_3d_pt_cursor_proc - ???
------------------------------------------------------------------------*/

int
win_set_3d_pt_cursor_proc(win,last_which_cursor)
 struct image_display_window *win;
 int last_which_cursor;
{
 struct img_buf_rec *img;
 int yes,last_yes;

 PRT(printf("win_set_3d_pt_cursor_proc...whic = %d\n",win->control.which_cursor);)
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((last_which_cursor == CURSOR_3D_PT1) ||
     (last_which_cursor == CURSOR_3D_PT2) ||
     (last_which_cursor == CURSOR_3D_PT3))
    last_yes = 1;
   else last_yes = 0;
 PRT(printf("last_which_cursor = %d which_cursor = %d last_yes = %d\n",
          last_which_cursor,win->control.which_cursor,last_yes))
 if (img->file->slice_info.num_3d_sampled_pts == 0)
    {win->control.which_cursor = CURSOR_3D_PT1;
     yes = 1;}
   else if (img->file->slice_info.num_3d_sampled_pts == 1)
    {win->control.which_cursor = CURSOR_3D_PT2;
     yes = 1;}
   else if (img->file->slice_info.num_3d_sampled_pts == 2)
    {win->control.which_cursor = CURSOR_3D_PT3;
     yes = 1;}
   else
    {yes = 0;
     if (last_yes)
        {
         win_next_plane_roi(win);
         win_enable_3d_draw_mode(win);
         win->control.which_cursor = CURSOR_CUT;
        }
       else 
        win->control.which_cursor = last_which_cursor;
     win_alarm(win->dwin->frame,2);
    }
 if (yes)
    if (!last_yes) win_enable_3d_pt_mode(win);
 return NO_ERROR;   /* FIXME: Guessed at return value.  -PWH 19990401 */
  
}



/*------------------------------------------------------------------------
win_check_disable_pt_cursor - ???
------------------------------------------------------------------------*/

int
win_check_disable_pt_cursor(win,last_which_cursor)
 struct image_display_window *win;
 int last_which_cursor;
{
 struct img_buf_rec *img;
 int last_yes;

 PRT(printf("win_check_disable_pt_cursor...whic = %d last_which_cursor = %d\n",
        win->control.which_cursor,last_which_cursor);)
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((last_which_cursor == CURSOR_3D_PT1) ||
     (last_which_cursor == CURSOR_3D_PT2) ||
     (last_which_cursor == CURSOR_3D_PT3))
    last_yes = 1;
   else last_yes = 0;
 if ((win->control.which_cursor != CURSOR_3D_PT1) &&
     (win->control.which_cursor != CURSOR_3D_PT2) &&
     (win->control.which_cursor != CURSOR_3D_PT3))
    {win_next_plane_roi(win);
     win_enable_3d_draw_mode(win);}
 return NO_ERROR;   /* FIXME: Guessed at return value.  -PWH 19990401 */
}



/*------------------------------------------------------------------------
get_cursor - ???
------------------------------------------------------------------------*/

int
get_cursor(win,select,cursor,action)
     struct image_display_window *win;
     unsigned long select;
     Xv_Cursor *cursor;
     void **action;
{
  struct cursor_action_rec *cursors;
  int is_clear;
  long last;

  cursors = nriaCursors;

  if (select >= TOTAL_CURSORS)
  {
    *cursor = 0;
    *action = NULL;
    return NO_ERROR;
  }
  last = win->control.which_cursor;
  win->control.which_cursor = select;
  if (cursors[last].last_check_proc != NULL)
  {
    cursors[last].last_check_proc(win,last);
  }

  PRT2(printf("1 last = %d select = %d\n", last,select));
  if (win->control.which_cursor != win->control.last_which_cursor)
  {
    if (win->control.proc_upon_cursor_change != NULL)
    {
      win->control.proc_upon_cursor_change(win->control.proc_upon_cursor_change_arg);
      set_proc_upon_cursor_change(&(win->control), NULL, NULL);
    }
  }
  PRT2(printf("2 win->control.which_cursor = %d\n",win->control.which_cursor));

  if (cursors[select].check_proc != NULL)
  {
    cursors[select].check_proc(win,last);
    select = win->control.which_cursor;
  }
  PRT2(printf("3 last = %d select = %d\n",last,select));

  if (cursors[select].cursor == NULL)
    *cursor = 0;
  else
    *cursor = cursors[select].cursor();
  *action = (void *) cursors[select].action;
  if (cursors[select].setup_proc != NULL)
    cursors[select].setup_proc(win);
  if ((cursors[select].mode & ENABLE_3D_DRAW_MODE) &&
      (win->type2 & DISPLAY_WINDOW_3D) &&
      !(win->control.status & _3D_DRAW_MODE))
    win_enable_3d_draw_mode(win); 
  if (cursors[select].mode & DRAW_LABEL_STATUS)
  {
    is_clear = 1;
    if (cursors[select].mode & DRAW_LABEL_DISABLED)
      win->control.status &= ~DRAW_ENABLED;
    else if (cursors[select].mode & DRAW_LABEL_ENABLED)
    {
      PRT3(printf("DRAW_ENABLED...2\n"));
      win->control.status |= DRAW_ENABLED;
    }

    win_execute_proc_to_multiple_wins(win, win_draw_enabled_label, &is_clear,
				      0,0,0,0);
  }

  PRT3(printf("4 select = %d which_cursor = %d\n",select,
	      win->control.which_cursor));
  return NO_ERROR;
}



/*------------------------------------------------------------------------
dwin_default_cursor - ???
------------------------------------------------------------------------*/

int
dwin_default_cursor(win)
 struct display_win_rec *win;
{
 Xv_Cursor cursor;
 int (*action)();
 int is_get_cursor;

 TRACE(printf("===dwin_default_cursor..which_cursor = %x\n",
         win->owner->control.which_cursor))

 if (win->owner->type2 & HISTOGRAM_1D_WINDOW_DISPLAY_ONLY) return NO_ERROR;
 if (win->default_cursor == 0)
    win->default_cursor = (Xv_Cursor) xv_get(
           canvas_paint_window(win->frame_canvas.canvas),WIN_CURSOR);
 if (win->owner->control.status & DRAW_ENABLED)
    {TRACE(printf("dwin_default_cursor DRAW_ENABLED...\n"))
     if (!(win->owner->control.which_cursor))
        {
         if (win->owner->type2 & DISPLAY_WINDOW_3D)
            {
             win->owner->control.which_cursor = CURSOR_CUT;}
           else win->owner->control.which_cursor = CURSOR_PEN;
        }
     is_get_cursor = 1;
    }
   else if (!(win->owner->control.which_cursor))
    {TRACE(printf("dwin_default_cursor default_cursor...\n"))
     xv_set(canvas_paint_window(win->frame_canvas.canvas),
                    WIN_CURSOR,win->default_cursor,0);
     is_get_cursor = 0;}
   else is_get_cursor = 1;

 if (is_get_cursor)
    {get_cursor(win->owner,win->owner->control.which_cursor,&cursor,&action);
     if (cursor == 0) cursor = win->default_cursor;
     xv_set(canvas_paint_window(win->frame_canvas.canvas),
                    WIN_CURSOR,cursor,0);
     if (action != NULL) action(win->owner);}


 return NO_ERROR;
}


/*------------------------------------------------------------------------
win_set_cursor - ???
------------------------------------------------------------------------*/

int
win_set_cursor(win,cursor)
 struct image_display_window *win;
 Xv_Cursor cursor;
{
 PRT4(printf("win_set_cursor...win = %u dwin = %u cursor = %u %u %u\n",
           win,win->dwin,cursor,cursor_cut,
     cursor_move_superimpose))
 if (cursor == 0) 
    xv_set(canvas_paint_window(win->dwin->frame_canvas.canvas),
         WIN_CURSOR,win->dwin->default_cursor,0);
   else
    {PRT4(printf("canvas = %u\n",
           canvas_paint_window(win->dwin->frame_canvas.canvas)))
     xv_set(canvas_paint_window(win->dwin->frame_canvas.canvas),
         WIN_CURSOR,cursor,0);}
 return NO_ERROR;
}



/*------------------------------------------------------------------------
win_select_cursor - ???
------------------------------------------------------------------------*/

int
win_select_cursor(win,select)
     struct image_display_window *win;
     unsigned long select;
{
  Xv_Cursor cursor;
  struct cursor_action_rec *cursors;
  int (*action)();
  int err;

  if (win->control.last_which_cursor != win->control.which_cursor)
  {
    if ((win->control.which_cursor >= 0) && 
	(win->control.which_cursor <= TOTAL_CURSORS))
    {
      cursors = nriaCursors;
      if (cursors[win->control.which_cursor].mode & LAST_CURSOR)
	win->control.last_which_cursor = win->control.which_cursor;
    }
  }
  get_cursor(win,select,&cursor,&action);
  PRT4(printf("win_select_cursor...select = %d cursor = %u action = %u\n",
	      select,cursor,action));
  if ((err = win_execute_proc_to_multiple_wins(win,win_set_cursor,cursor,
					       0,0,0,0))
      == NO_ERROR)
  {
    PRT4(printf("win_select_cursor...action = %u\n",action));
    if (action != NULL)
      err = win_execute_proc_to_multiple_wins(win,action,0,0,0,0,0);
  }
  
  return err;
}



/*------------------------------------------------------------------------
win_set_def_cursor - ???
------------------------------------------------------------------------*/

int
win_set_def_cursor(win)
     struct image_display_window *win;
{
  return dwin_default_cursor(win->dwin);
}



/*------------------------------------------------------------------------
win_get_cursor_xy_and_bound - ???
------------------------------------------------------------------------*/

int
win_get_cursor_xy_and_bound( win, event, x, y, use_this_xybound, xybound )
     struct image_display_window *win;
     Event *event;
     long *x,*y;
     int *use_this_xybound;
     struct int_bound_rec *xybound;
{
  struct cursor_action_rec *cursors = NULL;
  Rect *rr = NULL;
  int err = 0;

  /* printf("win 1: %p\n", win); */
  if (event != NULL)
  {
    if ((err = dwin_get_coord_xy_from_event( win->dwin, event, x, y )) != 
	NO_ERROR)
      /*      printf("win 2: %p\n", win); */
      return err;
  }
  else
  {
    rr = (Rect *) xv_get( canvas_paint_window(win->dwin->frame_canvas.canvas),
			  WIN_MOUSE_XY );
    *x = rr->r_left;
    *y = rr->r_top;
  }
  if (xybound != NULL)
  {
    if ((win->control.which_cursor >= 0) &&
	(win->control.which_cursor <= TOTAL_CURSORS))
    {
      cursors = nriaCursors;
      *use_this_xybound = 1;
      *xybound = cursors[win->control.which_cursor].bound;
         
      xybound->left += *x; 
      xybound->right += *x; 
      xybound->top += *y; 
      xybound->bottom += *y; 
    }
    else
      *use_this_xybound = 0;
  }
  else
    *use_this_xybound = 0;

#if 0
  if (*use_this_xybound)
  {
    printf("cursor x = %d y = %d\n",*x,*y);
    pr_int_bound_rec(xybound);
  }
  else
    printf("x = %d y = %d\n",*x,*y);
#endif

  return NO_ERROR;
}
