/****************************************************************************
display_bin.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: display_bin.c,v 1.10 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";

/* FIXME:  Not needed?
#include <time.h>
#include <sys/timeb.h>
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>         /* For XDestroyImage */

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/xv_xrect.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define TRACE(x) x; fflush(stdout);
*/
#define PRINT(x)
#define TRACE(x)



#define CLEAN_UP_BIN 0
#define SETUP_BIN 1
#define GET_ALL_BIN_PTR 2
#define CLOSE 10
#define OPEN 11

#define CLOSE_UPON_OWNER_REQUEST 1


struct bin_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Frame canvas_frame;
    Canvas canvas;
    Menu menu;
    struct strs_rec pixel_types,rois;
    unsigned int total_slices,num_display_slices,lb_slice_num,ub_slice_num,
                 last_displayed_slice_num,already_displayed;
    unsigned int *slice_nums,total_slice_nums,cnt_displayed_slice;
    unsigned int num_rows,num_cols,total_rows,total_cols;
    unsigned int buf_dimx,buf_dimy,new_dimx,new_dimy;
    struct img_rec img;
    float pixel_ratio;
    unsigned int win_dimx,win_dimy;

    Frame subframe;
    Panel panel;
    Panel_item start_item,ratio_item,num_col_items,num_row_items,
               pixel_type_item,roi_item;

    struct buf_2d_rec all_slice_map;
   };


struct the_bin_rec
   {
    int num_bins;
    struct bin_rec **bins;
   };


/* Functions defined in the file */
int default_init_bin_rec(struct bin_rec *bin);
void init_bin_rec(struct bin_rec *bin);
void free_bin_rec(struct bin_rec *bin);
void init_the_bin_rec(struct the_bin_rec *the_bin);
void dp_pixel_type_repaint_proc(Canvas canvas, Xv_Window xvwin, Display *dp,
    Window xwin, Xv_xrectlist *xrects);
void bin_done_proc(Frame frame);
void display_pixel_types_proc(Panel_item item, Event *event);
void bin_next_group_proc(Panel_item item, Event *event);
void bin_ratio_proc(Panel_item item, int value, Event *event);
int bin_ready_show(struct bin_rec *bin);
void bin_ready_proc(Panel_item item, Event *event);
void bin_cancel_proc(Panel_item item, Event *event);
void bin_subframe_done_proc(Frame frame);
void bin_settings_proc(Panel_item item, Event *event);
void bin_help_proc(Panel_item item, Event *event);
void bin_closed_proc(Panel_item item, Event *event);
void get_total_cols_and_rows(struct bin_rec *bin,
    double xratio, double yratio,
    unsigned int *num_cols, unsigned int *num_rows,
    unsigned int *new_dimx, unsigned int *new_dimy);
int setup_bin_labels(struct bin_rec *bin);
void bin_dp_pixel_types(struct bin_rec *bin, int which);
int setup_bin_rec(struct bin_rec *bin, struct image_display_window *win);
void dp_pixel_type_to_screen(struct bin_rec *bin);
int dp_pixel_types(struct bin_rec *bin, struct img_buf_rec *img,
    struct buf_2d_rec *pixel_map, struct draw_mask_rec *mask1,
    struct buf_2d_rec *roi_map, struct buf_2d_rec *slice_map,
    unsigned long *real_color_indxs, unsigned int num_real_color_indxs);
int display_pixel_types(struct bin_rec *bin);
int bin_display_pixel_types(struct bin_rec *bin);
struct bin_rec *find_bin_rec(struct the_bin_rec *the_bin,
    struct image_display_window *win, struct display_win_rec *dwin);
struct bin_rec *find_bin_same_input_file_rec(struct the_bin_rec *the_bin,
    struct image_display_window *win);
int enter_bin(struct image_display_window *win, int status);
int win_setup_bin(struct image_display_window *win);



/*-------------------------------------------------------------------------
default_init_bin_rec - ???
-------------------------------------------------------------------------*/

int
default_init_bin_rec(bin)
 struct bin_rec *bin;
{
 Rect *rr;
 Frame *frame;
 struct prog_rec *prog;
 struct img_buf_rec *img;
 struct hdr_scan_entry_rec *slice_info;
 int err;

 if ((img = win_rec_cur_img_buf(bin->win->owner)) == NULL)
    return CANCEL;
 bin->total_slices = img->file->slice_info.num_sroi_wins;
 bin->buf_dimy = img->_2d.dimy;
 bin->buf_dimx = img->_2d.dimx;

 frame = (Frame *) main_display(2,NULL,0,0);
 if (frame != NULL)
    {prog = (struct prog_rec *) xv_get(*frame,WIN_CLIENT_DATA);
     sscanf(prog->defaults.whch.bin_ratio_row_col,"%f/%d/%d",
        &(bin->pixel_ratio),&(bin->num_rows),&(bin->num_cols));
     get_total_cols_and_rows(bin,bin->pixel_ratio,bin->pixel_ratio,
          &(bin->total_cols),&(bin->total_rows),&(bin->new_dimx),
          &(bin->new_dimy));
     if (bin->num_cols > bin->total_cols) bin->num_cols = bin->total_cols;
     if (bin->num_rows > bin->total_rows) bin->num_rows = bin->total_rows;
     bin->win_dimx = bin->num_cols * bin->new_dimx;
     bin->win_dimy = bin->num_rows * bin->new_dimx;    
    }
   else
    {bin->pixel_ratio = .5;
     rr = (Rect *) xv_get(bin->frame,WIN_SCREEN_RECT);
     bin->win_dimx = rr->r_width;
     bin->win_dimy = rr->r_height;
     get_total_cols_and_rows(bin,bin->pixel_ratio,bin->pixel_ratio,
          &(bin->total_cols),&(bin->total_rows),&(bin->new_dimx),
          &(bin->new_dimy));
     bin->num_cols = bin->win_dimx / bin->new_dimx;
     bin->num_rows = bin->win_dimy / bin->new_dimy;

     if ((bin->new_dimy > bin->win_dimy) || (bin->new_dimx > bin->win_dimx))
        {
         bin->num_rows = bin->num_cols = 1;
        }
    }
 bin->num_display_slices = bin->num_rows * bin->num_cols;

 bin->lb_slice_num = get_img_slice_num(img);
 bin->ub_slice_num = bin->lb_slice_num + bin->num_display_slices - 1;
 bin->ub_slice_num %= bin->total_slices;

 bin->win_dimx = bin->num_cols * bin->new_dimx;
 bin->win_dimy = bin->num_rows * bin->new_dimx;
 /* FIXME!! Added a final argument "0" to the next line so that the
    function call matches the number of arguments in the definition.
    I make no promises that this is correct.  -PWH 19990603 */
 get_slice_info("a",img,bin->lb_slice_num,&slice_info,0);
 bin->img.file = img->file;
 if (NewImgBuf(&(bin->img),slice_info,&err,0,0,0,0,0,0,1) == NULL)
    return err;
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
init_bin_rec - ???
-------------------------------------------------------------------------*/

void
init_bin_rec(bin)
 struct bin_rec *bin;
{
 bin->status = 0;
 bin->frame = 0;
 bin->win = NULL;
 init_strs_rec(&(bin->pixel_types));
 init_strs_rec(&(bin->rois));
 init_img_rec(&(bin->img),NULL);
 init_buf_2d_rec(&(bin->all_slice_map));

 bin->total_slice_nums = 0;
 bin->slice_nums = NULL;
 bin->already_displayed = 0;
 bin->subframe = 0;
}


/*-------------------------------------------------------------------------
free_bin_rec - ???
-------------------------------------------------------------------------*/

void
free_bin_rec(bin)
 struct bin_rec *bin;
{
 struct the_bin_rec *the_bin;

 TRACE(("free_bin_rec...\n"))
 if (bin->frame != 0) xv_destroy_safe(bin->frame);
 free_strs_rec_ptrs(&(bin->pixel_types));
 free_strs_rec_ptrs(&(bin->rois));
 free_img_rec(&(bin->img));
 free_buf_2d_rec(&(bin->all_slice_map));
 c_free(bin->slice_nums);

 the_bin = (struct the_bin_rec *) enter_bin(NULL,GET_ALL_BIN_PTR);
 c_free(bin);
 DropHandleAryPtr(&(the_bin->bins),&(the_bin->num_bins),bin);

}


/*-------------------------------------------------------------------------
init_the_bin_rec - ???
-------------------------------------------------------------------------*/

void
init_the_bin_rec(the_bin)
 struct the_bin_rec *the_bin;
{
 the_bin->num_bins = 0;
 the_bin->bins = NULL;
}


/*-------------------------------------------------------------------------
dp_pixel_type_repaint_proc - ???
-------------------------------------------------------------------------*/

void
dp_pixel_type_repaint_proc(canvas,xvwin,dp,xwin,xrects)
 Canvas canvas;
 Xv_Window xvwin;
 Display *dp;
 Window xwin;
 Xv_xrectlist *xrects;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 dp_pixel_type_to_screen(bin);

}


/*-------------------------------------------------------------------------
bin_done_proc - ???
-------------------------------------------------------------------------*/

void
bin_done_proc(frame)
 Frame frame;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_bin_rec(bin);

}



/*-------------------------------------------------------------------------
display_pixel_types_proc - ???
-------------------------------------------------------------------------*/

void
display_pixel_types_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bin_rec *bin;

 TRACE(printf("display_pixel_types_proc...\n"))
 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (!xv_get(bin->canvas_frame,XV_SHOW) && 
     !xv_get(bin->canvas_frame,FRAME_CLOSED))
    make_2_wins_adjacent(bin->canvas_frame,bin->frame,0);
 bin_display_pixel_types(bin);

}

/************************
bin_event_proc(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 if (event_is_ascii(event) || event_is_key_top(event) ||
     event_is_key_left(event) || event_is_meta(event) || event_is_button(event))
    menu_show(bin->menu,canvas,event,0);
   else return NOTIFY_DONE;
}
********************/


/*-------------------------------------------------------------------------
bin_next_group_proc - ???
-------------------------------------------------------------------------*/

void
bin_next_group_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bin_rec *bin;

 TRACE(printf("display_pixel_types_proc...\n"))
 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (bin->already_displayed)
    bin->lb_slice_num = bin->last_displayed_slice_num + 1;
   else bin->lb_slice_num += bin->num_display_slices;
 bin->lb_slice_num %= bin->total_slices;
 bin->ub_slice_num = bin->lb_slice_num + bin->num_display_slices - 1;
 bin->ub_slice_num %= bin->total_slices;
 if (!xv_get(bin->canvas_frame,XV_SHOW) && 
     !xv_get(bin->canvas_frame,FRAME_CLOSED))
    make_2_wins_adjacent(bin->canvas_frame,bin->frame,0);
 bin_display_pixel_types(bin);
}


/*-------------------------------------------------------------------------
bin_ratio_proc - ???
-------------------------------------------------------------------------*/

void
bin_ratio_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct bin_rec *bin;
 float ratio;
 unsigned int num_cols,num_rows,new_dimx,new_dimy;

 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 ratio = (float) value / 100.0;
 get_total_cols_and_rows(bin,ratio,ratio,&num_cols,&num_rows,
                         &new_dimx,&new_dimy);
 if (num_cols * num_rows)
    {num_rows = bin->total_slices / num_cols;
     if (bin->total_slices % num_cols) num_rows++;}

 xv_set(bin->num_col_items,PANEL_MAX_VALUE,num_cols,0);
 xv_set(bin->num_row_items,PANEL_MAX_VALUE,num_rows,0);
}


/*-------------------------------------------------------------------------
bin_ready_show - ???
-------------------------------------------------------------------------*/

int
bin_ready_show(bin)
 struct bin_rec *bin;
{
 int num_cols,num_rows,start;
 float ratio;

 start = (int) xv_get(bin->start_item,PANEL_VALUE);
 num_cols = (int) xv_get(bin->num_col_items,PANEL_VALUE);
 num_rows = (int) xv_get(bin->num_row_items,PANEL_VALUE);
 ratio = (int) xv_get(bin->ratio_item,PANEL_VALUE) / 100.0;
 xv_destroy_safe(bin->subframe);
 bin->subframe = 0;

 if ((start != (bin->lb_slice_num + 1)) ||
     (num_cols != bin->num_cols) || (num_rows != bin->num_rows) ||
     (bin->pixel_ratio != ratio))
    {
     bin->pixel_ratio = ratio;
     get_total_cols_and_rows(bin,bin->pixel_ratio,bin->pixel_ratio,
          &(bin->total_cols),&(bin->total_rows),&(bin->new_dimx),
          &(bin->new_dimy));
     bin->num_cols = num_cols;
     bin->num_rows = num_rows;
     bin->num_display_slices = bin->num_rows * bin->num_cols;
     if (bin->num_display_slices > bin->total_slices)
        {bin->num_display_slices = bin->total_slices;}
     bin->lb_slice_num = start - 1;
     bin->ub_slice_num = (bin->lb_slice_num + bin->num_display_slices - 1) %
                      bin->total_slices;
     TRACE(printf("lb = %d ub = %d num = %d num_rows = %d num_cols = %d\n",bin->lb_slice_num,bin->ub_slice_num,
              bin->num_display_slices,bin->num_rows,bin->num_cols))

     return 1;
    }
 return 0;
}


/*-------------------------------------------------------------------------
bin_ready_proc - ???
-------------------------------------------------------------------------*/

void
bin_ready_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (bin_ready_show(bin))
    {
     bin_display_pixel_types(bin);}

}


/*-------------------------------------------------------------------------
bin_cancel_proc - ???
-------------------------------------------------------------------------*/

void
bin_cancel_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 xv_destroy_safe(bin->subframe);
 bin->subframe = 0;
}


/*-------------------------------------------------------------------------
bin_subframe_done_proc - ???
-------------------------------------------------------------------------*/

void
bin_subframe_done_proc(frame)
 Frame frame;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(frame,WIN_CLIENT_DATA);
 xv_destroy_safe(bin->subframe);
 bin->subframe = 0;
}


/*-------------------------------------------------------------------------
bin_settings_proc - ???
-------------------------------------------------------------------------*/

void
bin_settings_proc(item,event)
 Panel_item item;
 Event *event;
{
 int row,offset;
 struct bin_rec *bin;
 int ratio;
 char *str;
/*
 Cms cms;
 struct color_rec *color;
*/
 TRACE(printf("bin_settings_proc...\n"))
 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (bin->subframe != 0)
    {xv_set(bin->subframe,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
     return;}
 bin->subframe = xv_create(bin->frame,FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              /*WIN_DYNAMIC_VISUAL,TRUE,*/
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,"ALL ROI PIXEL TYPES",
              FRAME_NO_CONFIRM,TRUE,
              WIN_CLIENT_DATA,bin,
              FRAME_DONE_PROC,bin_subframe_done_proc,
              WIN_EVENT_PROC,main_event_proc,
              0);
 if (bin->subframe == 0) return;
 bin->panel = xv_create(bin->subframe,PANEL,/* WIN_CMS,cms,*/
                         FRAME_INHERIT_COLORS,TRUE,
                         0);
 if (bin->panel == 0) 
    {xv_destroy_safe(bin->subframe);
     bin->subframe = 0;
     return;}
 row = 5;
 offset = xv_rows(bin->panel,1) - xv_rows(bin->panel,0);
 bin->start_item = xv_create(bin->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"1ST SLICE# :",
	PANEL_VALUE,(int) bin->lb_slice_num,
	PANEL_MIN_VALUE,1,
	PANEL_MAX_VALUE,(int) bin->total_slices,
        XV_X,5,XV_Y,row,
	PANEL_SLIDER_WIDTH,100,
	0);
 row += offset;
 ratio = (int) ULongRound(bin->pixel_ratio * 100);
 bin->ratio_item = xv_create(bin->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"MAG RATIO / 100 :",
	PANEL_VALUE,ratio,
	PANEL_MIN_VALUE,25,
	PANEL_MAX_VALUE,100,
	PANEL_SLIDER_WIDTH,100,
        XV_X,5,XV_Y,row,
	PANEL_NOTIFY_PROC,bin_ratio_proc,
        PANEL_CLIENT_DATA,bin,
	0);
 row += offset;
 bin->num_col_items = xv_create(bin->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"TOTAL COLS :",
	PANEL_VALUE,(int) bin->num_cols,
	PANEL_MIN_VALUE,1,
	PANEL_MAX_VALUE,bin->total_cols,
	PANEL_SLIDER_WIDTH,60,
        XV_X,5,XV_Y,row,
	0);
 row += offset;
 bin->num_row_items = xv_create(bin->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"TOTAL ROWS :",
	PANEL_VALUE,(int) bin->num_rows,
	PANEL_MIN_VALUE,1,
	PANEL_MAX_VALUE,bin->total_rows,
	PANEL_SLIDER_WIDTH,60,
        XV_X,5,XV_Y,row,
	0);
 row += offset;
 str = "READY";
 xv_create(bin->panel,PANEL_BUTTON,
 	PANEL_LABEL_STRING,str,
        PANEL_NOTIFY_PROC,bin_ready_proc,
        PANEL_CLIENT_DATA,bin,
        XV_X,5,XV_Y,row,
        0);
 str = "CANCEL";
 xv_create(bin->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,str,
                   PANEL_NOTIFY_PROC,bin_cancel_proc,
                   PANEL_CLIENT_DATA,bin,
                   0);
 window_fit(bin->panel);
 window_fit(bin->subframe);
 make_2_wins_adjacent(bin->subframe,bin->frame,0);
 win_set_mouse_at_center(bin->subframe);
 xv_set(bin->subframe,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 TRACE(printf("bin_settings_proc...end\n"))

}


/*-------------------------------------------------------------------------
bin_help_proc - ???
-------------------------------------------------------------------------*/

void
bin_help_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 ShowMsgStrs(bin->frame,0,NO_ERROR,0,1,
       "Help menu will be available in the future !!!",
       NULL,0);

}


/*-------------------------------------------------------------------------
bin_closed_proc - ???
-------------------------------------------------------------------------*/

void
bin_closed_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bin_rec *bin;

 bin = (struct bin_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (bin != NULL) bin_done_proc(bin->frame);

}


/*-------------------------------------------------------------------------
get_total_cols_and_rows - ???
-------------------------------------------------------------------------*/

void
get_total_cols_and_rows(bin,xratio,yratio,num_cols,num_rows,new_dimx,new_dimy)
 struct bin_rec *bin;
 float xratio,yratio;
 unsigned int *num_cols,*num_rows,*new_dimx,*new_dimy;
{
 Rect *rr;

 get_shrink_dim(bin->buf_dimx,bin->buf_dimy,xratio,yratio,new_dimx,new_dimy);
 rr = (Rect *) xv_get(bin->frame,WIN_SCREEN_RECT);
 if ((rr->r_width - 5) <= *new_dimx) *num_cols = 1;
   else *num_cols = (rr->r_width - 5) / (*new_dimx);
 if ((rr->r_height - 5) <= *new_dimy) *num_rows = 1;
   else *num_rows = (rr->r_height - 5) / (*new_dimy);
}


/*-------------------------------------------------------------------------
setup_bin_labels - ???
-------------------------------------------------------------------------*/

int
setup_bin_labels(bin)
 struct bin_rec *bin;
{
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(bin->win->owner)) == NULL)
    return CANCEL;
 free_strs_rec_ptrs(&(bin->pixel_types));
 free_strs_rec_ptrs(&(bin->rois));
 cp_strs_rec_ptrs(&(bin->pixel_types),&(img->file->pixel_type.order));
 WinGetAllRoiLabelPtrs(bin->win,&(bin->rois),0,1);
 if ((bin->pixel_types.num_strs == 0) ||
     (bin->rois.num_strs == 0)) return CANCEL;
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
bin_dp_pixel_types - ???
-------------------------------------------------------------------------*/

void
bin_dp_pixel_types(bin,which)
 struct bin_rec *bin;
 int which;
{
 (void) notify_set_itimer_func((Notify_client) bin,NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 bin_display_pixel_types(bin);
}


/*-------------------------------------------------------------------------
setup_bin_rec - ???
-------------------------------------------------------------------------*/

int
setup_bin_rec(bin,win)
 struct bin_rec *bin;
 struct image_display_window *win;
{
 int err,width,height;
 int row,offset;
 char *str;
/*
 Cms cms;
 struct color_rec *color;
*/
 Panel panel;
 Panel_item item;
 Rect *rr;

 TRACE(printf("setup_bin_rec...\n"))
 bin->win = win;
 bin->dwin = win->dwin;
 if ((err = setup_bin_labels(bin)) != NO_ERROR)
    {free_bin_rec(bin);
     return err;}
 TRACE(printf("setup_bin_rec...%%%\n"))
 str = win_rec_win_id_title(win->owner,"PIXEL TYPES");
 bin->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              /*WIN_DYNAMIC_VISUAL,TRUE,*/
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              WIN_CLIENT_DATA,bin,
              FRAME_DONE_PROC,bin_done_proc,
              0);
 c_free(str);
 if (bin->frame == 0)
    {free_bin_rec(bin);
     return CANCEL;}
 TRACE(printf("setup_bin_rec...0\n"))
 if ((err = default_init_bin_rec(bin)) != NO_ERROR)
    {free_bin_rec(bin);
     return err;}

 panel = xv_create(bin->frame,PANEL,
              0);

 if (panel == 0)
    {free_bin_rec(bin);
     return CANCEL;}

 str = win_rec_win_id_title(win->owner,"PIXEL TYPES");
 bin->canvas_frame = xv_create(bin->frame,FRAME,FRAME_INHERIT_COLORS,TRUE,
              /*WIN_DYNAMIC_VISUAL,TRUE,*/
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              WIN_CLIENT_DATA,bin,
              FRAME_DONE_PROC,bin_done_proc,
              0);
 c_free(str);
 bin->canvas = xv_create(bin->canvas_frame,CANVAS,CANVAS_RETAINED,FALSE,
                     /*WIN_DYNAMIC_VISUAL,TRUE,*/
                     CANVAS_X_PAINT_WINDOW,TRUE,
                     CANVAS_REPAINT_PROC,dp_pixel_type_repaint_proc,
                     0);
 if (bin->canvas == 0)
    {free_bin_rec(bin);
     return CANCEL;}

 width = bin->win_dimx + 1;
 height = bin->win_dimy + 1;
 xv_set(bin->canvas,
        CANVAS_WIDTH,bin->win_dimx,
        CANVAS_HEIGHT,bin->win_dimy,
        XV_WIDTH,width,XV_HEIGHT,height,
        CANVAS_FIXED_IMAGE,TRUE,
        CANVAS_AUTO_SHRINK,FALSE,
        CANVAS_AUTO_EXPAND,FALSE,
        WIN_CLIENT_DATA,bin,
        0);

 xv_set(canvas_paint_window(bin->canvas),WIN_CLIENT_DATA,bin,0);

 row = 5;
 offset = xv_rows(panel,1) - xv_rows(panel,0) - 10;
 xv_create(panel,PANEL_BUTTON,
           PANEL_CLIENT_DATA,bin,
           PANEL_LABEL_STRING,"HELP",
           PANEL_NOTIFY_PROC,bin_help_proc,0);
 xv_create(panel,PANEL_BUTTON,
           PANEL_CLIENT_DATA,bin,
           PANEL_LABEL_STRING,"VIEW MORE",
           PANEL_NOTIFY_PROC,bin_next_group_proc,0);

 bin->pixel_type_item = xv_create(panel,PANEL_LIST,
        PANEL_LAYOUT,PANEL_VERTICAL,
        PANEL_LABEL_STRING,"PIXEL TYPES",
        PANEL_CHOOSE_ONE,FALSE,
        PANEL_LIST_DISPLAY_ROWS,2,
        PANEL_LIST_WIDTH,100,XV_Y,row,
        PANEL_CLIENT_DATA,bin,0);
 setup_list_items(&(bin->pixel_types),bin->pixel_type_item,1);
 rr = (Rect *) xv_get(bin->pixel_type_item,XV_RECT);

 bin->roi_item = xv_create(panel,PANEL_LIST,
        PANEL_LAYOUT,PANEL_VERTICAL,
        PANEL_LABEL_STRING,"   ROIS",
        PANEL_CHOOSE_ONE,FALSE,
        PANEL_LIST_DISPLAY_ROWS,2,
        PANEL_LIST_WIDTH,100,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,row,
        PANEL_CLIENT_DATA,bin,0);
 setup_list_items(&(bin->rois),bin->roi_item,1);

 xv_create(panel,PANEL_BUTTON,
           PANEL_CLIENT_DATA,bin,
           PANEL_LABEL_STRING,"DISPLAY CURRENT",
           XV_X,5,XV_Y,xv_rows(panel,1),
           PANEL_NOTIFY_PROC,display_pixel_types_proc,0);

 item = (Panel_item) xv_create(panel,PANEL_BUTTON,
           PANEL_CLIENT_DATA,bin,
           PANEL_LABEL_STRING,"SETUP...",
           XV_X,0,XV_Y,xv_rows(panel,2),
           PANEL_NOTIFY_PROC,bin_settings_proc,0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(panel,PANEL_BUTTON,
           PANEL_CLIENT_DATA,bin,
           PANEL_LABEL_STRING,"DONE",
           XV_X,rr->r_left + rr->r_width + 10,XV_Y,xv_rows(panel,2),
           PANEL_NOTIFY_PROC,bin_closed_proc,0);


 window_fit(panel);
 window_fit(bin->canvas_frame);
 window_fit(bin->frame);

 make_2_wins_adjacent(bin->frame,win->dwin->frame,0);
 xv_set(bin->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 wmgr_top(bin->frame);
/*
 TRACE(printf("setup_bin_rec...end\n"))
 (void) notify_set_itimer_func((Notify_client) bin,
             (Notify_func) bin_dp_pixel_types,
             ITIMER_REAL,&NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
*/
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
dp_pixel_type_to_screen - ???
-------------------------------------------------------------------------*/

void
dp_pixel_type_to_screen(bin)
 struct bin_rec *bin;
{
 XImage *im;
 char str[30];
 unsigned int lcv,row,col,win_dimx,win_dimy;

 if (!(bin->all_slice_map.status4 & BUF_NON_EMPTY)) return /*CANCEL*/;
 create_image(&im,bin->win->dwin->dp.dp,bin->win->dwin->dp.screen_num,
           &(bin->all_slice_map),
           bin->win->dwin->screen._2d.pixel_depth);
/*
 bytes = GetRowBytes(bin->all_slice_map.dimx,
                    bin->all_slice_map.pixel_depth);
 visual = DefaultVisual(bin->win->dwin->dp.dp,bin->win->dwin->dp.screen_num);
 im = XCreateImage(bin->win->dwin->dp.dp,
            visual,bin->all_slice_map.pixel_depth,
            ZPixmap,0,bin->all_slice_map.buf[0],bin->all_slice_map.dimx,
            bin->all_slice_map.dimy,bin->win->dwin->screen._2d.pixel_depth,
            bytes);
*/
 if (im == NULL) return /*CANCEL*/;

 get_shrink_dim(bin->buf_dimx,bin->buf_dimy,bin->pixel_ratio,bin->pixel_ratio,
            &win_dimx,&win_dimy);
 win_dimx = bin->num_cols * win_dimx;
 win_dimy = bin->num_rows * win_dimy;
 if ((win_dimx != bin->win_dimx) || (win_dimy != bin->win_dimy))
    {
     bin->win_dimx = win_dimx;
     bin->win_dimy = win_dimy;
    
     xv_set(bin->canvas,CANVAS_WIDTH,bin->win_dimx,
                    CANVAS_HEIGHT,bin->win_dimy,0);
     xv_set(bin->canvas,XV_WIDTH,bin->win_dimx + 1,
                    XV_HEIGHT,bin->win_dimy + 1,0);
     window_fit(bin->canvas_frame);
     }

 user_write(bin->canvas,0,0,bin->all_slice_map.dimx,bin->all_slice_map.dimy,GXcopy,
                      im,0,0);
 for (row = col = lcv = 0; lcv < bin->cnt_displayed_slice; lcv++)
    {sprintf(str,"#%d",bin->slice_nums[lcv]);
     user_text(bin->canvas,NULL,1,
               col * bin->new_dimx,row * bin->new_dimy + 14,
                   GXcopy,NULL,str);
     col = (col + 1) % bin->num_cols;
     if (col == 0) row += 1;
    }

 if (im != NULL)
    {im->data = NULL;
     XDestroyImage(im);}
}


/*-------------------------------------------------------------------------
dp_pixel_types - ???
-------------------------------------------------------------------------*/

int
dp_pixel_types(bin,img,pixel_map,mask1,roi_map,slice_map,
                    real_color_indxs,num_real_color_indxs)
 struct bin_rec *bin;
 struct img_buf_rec *img;
 struct draw_mask_rec *mask1;
 struct buf_2d_rec *pixel_map,*slice_map,*roi_map;
 unsigned long *real_color_indxs;
 unsigned int num_real_color_indxs;
{
 unsigned int lcv,cnt,row,col,pixel_depth,cnt_slice,dimx,dimy;
 int start_color;
 int err,yes;
 struct roi_rec *roi;
 unsigned int left,top,right,bottom;
 struct int_bound_rec bound,all_bound;

 /* FIXME!!  Added a final argument "0" in the next line so that the
    call matches the number of arguments specified in the definition.
    I make no promises that this is correct.  -PWH 19990603 */
 get_next_slice("a",img,bin->lb_slice_num,0);

 pixel_depth = pixel_map->pixel_depth / 8;
 if (pixel_map->pixel_depth % 8) pixel_depth++;
 pixel_depth *= 8;

 bin->cnt_displayed_slice = 0;
 if (bin->total_slice_nums < bin->num_display_slices)
    {c_free(bin->slice_nums);
     bin->total_slice_nums = 0;
     get_ptr_memory(&(bin->slice_nums),&(bin->total_slice_nums),
              bin->num_display_slices,(size_t) sizeof(*(bin->slice_nums)),&err);
     if (err != NO_ERROR) return err;
    }
 get_shrink_dim(bin->buf_dimx,bin->buf_dimy,bin->pixel_ratio,bin->pixel_ratio,
            &dimx,&dimy);

 TRACE(printf("all_slice_map...dimx = %d dimx = %d\n",dimx,dimy))

 if (((err = get_draw_mask_rec(mask1,img->_2d.dimx,img->_2d.dimy))
          != NO_ERROR) ||
     ((err = SetupBuf2dRec(roi_map,img->_2d.dimx,img->_2d.dimy,1,0))
          != NO_ERROR) ||
     ((err = SetupBuf2dRec(slice_map,img->_2d.dimx,img->_2d.dimy,pixel_depth,0))
          != NO_ERROR) ||
     ((err = SetupBuf2dRec(&(bin->all_slice_map),
              dimx * bin->num_cols,dimy * bin->num_rows,
              pixel_depth,0)) != NO_ERROR))
     return err;

 start_color = bin->win->intensity->color.reserve_color_start;

 err = NO_ERROR;
 left = top = 0;
 right = dimx - 1;
 bottom = dimy - 1;
 InitBuf2dRecData(&(bin->all_slice_map),(double) real_color_indxs[0]);

 for (row = col = 0,cnt_slice = lcv = bin->lb_slice_num; ; )
    {yes = 0;
     setup_int_bound_rec(&all_bound,img->_2d.dimx,img->_2d.dimy,0,0);
     if (is_slice_roi_non_empty(
             &(img->cur_slice->slice->slice_roi_win->slice_roi)))
        {
         for (cnt = 0; cnt < bin->rois.num_strs; cnt++)
            {
             if (xv_get(bin->roi_item,PANEL_LIST_SELECTED,cnt))
                {
                 if (((roi =
                     get_roi(&(img->cur_slice->slice->slice_roi_win->slice_roi),
                        bin->rois.strs[cnt])) != NULL) &&
                     is_roi_non_empty(roi))
                   {
                    FillDrawRoiRec(mask1,NULL,roi,NULL);
                    FindRoiLongMinMax(roi,NULL,&(bound.left),&(bound.right),
                                  &(bound.top),&(bound.bottom),0,0,0);
                    if (yes == 0) 
                       copy_buf_2d_rec_content(roi_map,&(mask1->mask._2d));
                      else
                       superimpose_bitmap_region(roi_map,&(mask1->mask._2d),
                                   &bound);
                    combine_int_bound_recs(&all_bound,&bound);
                    yes = 1;
                   }
                }
            }
         if (yes)
            {
             bin->last_displayed_slice_num = lcv;
             bin->already_displayed = 1;
             get_map(img,slice_map,roi_map,&all_bound,pixel_map);
             map_buf_2d_rec_by_bound_mask_and_ratio(&(bin->all_slice_map),
                          left,top,bin->pixel_ratio,bin->pixel_ratio,
                          slice_map,roi_map,&all_bound,
                          real_color_indxs,num_real_color_indxs,start_color);

             bin->slice_nums[bin->cnt_displayed_slice++] = img->cur_slice->num;
             col = (col + 1) % bin->num_cols;
             if (col == 0)
                {left = 0;
                 right = dimx - 1;
                 top = bottom + 1;
                 bottom += dimy;
                 row += 1;}
               else
                {left = right + 1;
                 right += dimx;}

             if (cnt_slice == bin->ub_slice_num) break;
             cnt_slice = (cnt_slice + 1) % bin->total_slices;
            }
         }
     lcv = (lcv + 1) % bin->total_slices;
     if (lcv == bin->lb_slice_num) break;
     /* FIXME!!  Added a final argument "0" on the next line so that
	the number of arguments matches the definition; I hve absolutely
	no idea if this is the correct value.  -PWH 19990603 */
     get_next_slice("",img,1,0);
    }
 dp_pixel_type_to_screen(bin);
 xv_set(bin->canvas_frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 wmgr_top(bin->canvas_frame);

 TRACE(printf("all_slice_map...\n"))

 return err;
}


/*-------------------------------------------------------------------------
display_pixel_types - ???
-------------------------------------------------------------------------*/

int
display_pixel_types(bin)
 struct bin_rec *bin;
{
 struct draw_mask_rec mask1;
 struct buf_2d_rec *pixel_type_map,pixel_map,slice_map,roi_map;
 struct img_buf_rec *img;
 Cms cms;
 unsigned long *temp_real_color_indxs,*real_color_indxs;
 unsigned int total,num_real_color_indxs;
 unsigned int cnt,lb;
 int err,yes,select;

 if ((err = setup_bin_labels(bin)) != NO_ERROR) return err;
 for (cnt = 0; cnt < bin->rois.num_strs; cnt++)
    {if (xv_get(bin->roi_item,PANEL_LIST_SELECTED,cnt))
        break;
    }
 if (cnt == bin->rois.num_strs) return CANCEL;
 if ((pixel_type_map = get_img_pixel_type_map(img)) == NULL) return CANCEL;

 img = bin->img.data[0];
 if (!(pixel_type_map->status4 & BUF_NON_EMPTY))
    return CANCEL;

 for (select = yes = cnt = 0; cnt < bin->pixel_types.num_strs; cnt++)
    {
     if (xv_get(bin->pixel_type_item,PANEL_LIST_SELECTED,cnt))
        {
         if (!yes)
            {init_buf_2d_rec(&pixel_map);
             if ((err =  SetupBuf2dRec(&pixel_map,
                 pixel_type_map->dimx,
                 pixel_type_map->dimy,
                 pixel_type_map->pixel_depth,1)) != NO_ERROR)
                return ABORT;
            }
         yes = 1;
         if (select == 0) lb = cnt + 1;
         select += 1;
        }
       else if (select)
        {
/*
         SelectAssign2d(&pixel_map,pixel_type_map,0x00004048,
                 (double) lb,(double) (lb + select - 1),
                 (double) .0,(double) .0,
                 NULL,0,0);
*/
         Select2d(pixel_type_map,
              (double) lb,(double) (lb + select - 1),
              &pixel_map,0,(double) 0,0,(double) 0);
         select = 0;}
    }
 if (!yes) return CANCEL;
 if (select)
    {
/*
     SelectAssign2d(&pixel_map,pixel_type_map,0x00004048,
                 (double) lb,(double) (lb + select - 1),
                 (double) .0,(double) .0,
                 NULL,0,0);
*/
     Select2d(pixel_type_map,
           (double) lb,(double) (lb + select - 1),
           &pixel_map,0,(double) 0,0,(double) 0);
    }

 init_draw_mask_rec(&mask1);
 init_buf_2d_rec(&roi_map);
 init_buf_2d_rec(&slice_map);

 cms = (Cms) xv_get(bin->canvas,WIN_CMS);
 temp_real_color_indxs = (unsigned long *) xv_get(cms,CMS_INDEX_TABLE);
 num_real_color_indxs = xv_get(cms,CMS_SIZE);
 total = 0;
 get_ptr_memory(&real_color_indxs,&total,num_real_color_indxs,
        (size_t) sizeof(unsigned long),&err);
 if (err == NO_ERROR)
    {for (cnt = 0; cnt < num_real_color_indxs; cnt++)
        real_color_indxs[cnt] = temp_real_color_indxs[cnt];
     real_color_indxs[0] = temp_real_color_indxs[
                    bin->win->intensity->color.num_system_colors + 3];

     err = dp_pixel_types(bin,img,&pixel_map,&mask1,
            &roi_map,&slice_map,real_color_indxs,num_real_color_indxs);
     c_free(real_color_indxs);
    }

 free_buf_2d_rec(&pixel_map);
 free_buf_2d_rec(&slice_map);
 free_buf_2d_rec(&roi_map);
 free_draw_mask_rec(&mask1);
 return err;
}


/*-------------------------------------------------------------------------
bin_display_pixel_types - ???
-------------------------------------------------------------------------*/

int
bin_display_pixel_types(bin)
 struct bin_rec *bin;
{
 int err;

 if ((err = display_pixel_types(bin)) != NO_ERROR)
    {if (err == CANCEL)
        {if (bin->frame != 0)
            ShowMsgStrs(bin->frame,0,err,0,2,
               "Nothing needs to be displayed because",
               "   no ROIs/PIXEL TYPES options are selected !!!",NULL,0);
         return NO_ERROR;}
       else
        {if (bin->frame != 0) ShowMsgStrs(bin->frame,0,err,0,2,
               "An error occured when displaying",
               "   the pixel types for the roi regions !!!",NULL,0);
        }
     free_bin_rec(bin);

    }
 return err;
}


/*-------------------------------------------------------------------------
find_bin_rec - ???
-------------------------------------------------------------------------*/

struct bin_rec *
find_bin_rec(the_bin,win,dwin)
 struct the_bin_rec *the_bin;
 struct image_display_window *win;
 struct display_win_rec *dwin;
{
 int lcv;

 for (lcv = 0; lcv < the_bin->num_bins; lcv++)
    if ((the_bin->bins[lcv]->win == win) &&
        (the_bin->bins[lcv]->dwin == dwin))
       return the_bin->bins[lcv];
 return NULL;
}


/*-------------------------------------------------------------------------
find_bin_same_input_file_rec - ???
-------------------------------------------------------------------------*/

struct bin_rec *
find_bin_same_input_file_rec(the_bin,win)
 struct the_bin_rec *the_bin;
 struct image_display_window *win;
{
 int lcv;
 struct input_file_rec *f1;
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NULL;
 f1 = img->file;
 for (lcv = 0; lcv < the_bin->num_bins; lcv++)
    {if ((img = win_rec_cur_img_buf(the_bin->bins[lcv]->win->owner)) != NULL)
        if (img->file == f1)
           return the_bin->bins[lcv];
    }
 return NULL;
}


/*-------------------------------------------------------------------------
enter_bin - ???
-------------------------------------------------------------------------*/

int
enter_bin(win,status)
 struct image_display_window *win;
 int status;
{
 static struct the_bin_rec the_bin;
 struct bin_rec *bin;
 static char first_time = 1;
 int err;

 if (status == GET_ALL_BIN_PTR)
    {if (first_time) return 0;
       else return (unsigned long) &the_bin;}     /* FIXME:  ptr as long */
 if (status == CLEAN_UP_BIN)
   {if (!first_time)
       if ((bin = find_bin_rec(&the_bin,win,win->dwin)) != NULL)
          free_bin_rec(bin);
    return NO_ERROR;}
  else if ((status == CLOSE) || (status == OPEN))
   {if (first_time) return NO_ERROR;
    if (((bin = find_bin_rec(&the_bin,win,win->dwin)) == NULL) ||
        (bin->frame == 0)) 
       return NO_ERROR;
    if (status == CLOSE)
       {bin->status |= CLOSE_UPON_OWNER_REQUEST;
        xv_set(bin->frame,FRAME_CLOSED,TRUE,0);}
    if (status == OPEN)
       if (bin->status & CLOSE_UPON_OWNER_REQUEST)
          {bin->status &= ~CLOSE_UPON_OWNER_REQUEST;
           xv_set(bin->frame,FRAME_CLOSED,FALSE,0);}
    return NO_ERROR;}
 if (first_time)
    {
     init_the_bin_rec(&the_bin);
     first_time = 0;}
   else
    {

     if ((bin = find_bin_rec(&the_bin,win,win->dwin)) != NULL)
        {xv_set(bin->canvas_frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         wmgr_top(bin->canvas_frame);
         win_set_mouse_at_center(bin->frame);
         bin->status &= ~CLOSE_UPON_OWNER_REQUEST;
         return NO_ERROR;}
     if ((bin = find_bin_same_input_file_rec(&the_bin,win)) != NULL)
        {bin->win = win;
         bin->dwin = win->dwin;
         xv_set(bin->canvas_frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         bin->status &= ~CLOSE_UPON_OWNER_REQUEST;
         wmgr_top(bin->frame);
         win_set_mouse_at_center(bin->frame);
         return NO_ERROR;
        }
    }

 get_handle_memory(&(the_bin.bins),&(the_bin.num_bins),1,
     (size_t) sizeof(struct bin_rec),&err);
 if (err != NO_ERROR) return err;
 bin = the_bin.bins[the_bin.num_bins - 1];
 init_bin_rec(bin);
 if ((err = setup_bin_rec(bin,win)) == NO_ERROR)
    xv_set(bin->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
   else 
    {free_bin_rec(bin);
     ShowMsgStrs(win->dwin->frame,0,err,0,2,
               "An error occured when setting up",
               "   the pixel type display window !!!",NULL,0);}

 return err;
}


/*-------------------------------------------------------------------------
win_setup_bin - ???
-------------------------------------------------------------------------*/

int
win_setup_bin(win)
 struct image_display_window *win;
{
 return enter_bin(win,1);
}
