/*****************************************************************************
display_main.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


Defined options in arg list:
    -d  Print verbose debugging output to stdout (if compiled with DEBUG)
    -D  Print verbose output and abort on error.
    -h  Print "Help!".

If one more argument exists besides the options, use it as the
init file (.set file).  

DRW 960910 Added code to find our real executable path, so we can
find configuration files no matter how we are called.
*****************************************************************************/

static const char rcsid[] = "$Id: display_main.c,v 1.8 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <unistd.h>

#include <xview/cms.h>
#include <xview/notify.h>

#include "mri_format.h"
#include "debug.h"



#ifdef EXTERN_CONSTANTS           /* FIXME:  What is this?! */
#include "ria_constants.h"
#endif EXTERN_CONSTANTS

#include "nria_procedures.h"



/* Global variables defined here */
int abortFlag = 0;         /* Signal ABORT on an error */
int debugFlag = 0;         /* Produce verbose debugging output */

/* Global variables defined elsewhere to control Xview operation */
extern int fullscreendebug;  /* 1 => don't grab X server in FULLSCREEN */

/*  FIXME: Must match up with #def's in display_main_extra.c;
    ought to be in .h file.  */
#define SETUP_MAIN_DISPLAY 1
#define GET_MAIN_DISPLAY_FRAME 2
#define SHOW_MAIN_DISPLAY 3
#define CLEAN_UP_MAIN_DISPLAY 4


/* Functions defined in this file */
int main(int argc, char *argv[]);
static void parseArgs(int argc, char *argv[], struct prog_rec *progptr);



/*--------------------------------------------------------------------------
parseArgs - ???
---------------------------------------------------------------------------*/

static void
parseArgs( argc, argv, progptr )
     int argc;
     char *argv[];
     struct prog_rec *progptr;
{
  extern char *optarg;
  extern int optind, opterr;
  int c;
  char *fn;

  debugFlag = 0;
  while ((c = getopt( argc, argv, "Ddh")) != -1)
  {
    switch (c)
    {

    case 'D':
      abortFlag = 1;
      /* Fall through */

    case 'd':
      debugFlag = 1;
      fullscreendebug = 1;
      printf( "\n\n***************************************************\n\n" );
      printf( "Debugging turned ON.\n" ); fflush( stdout );
      break;

    case 'h':
      printf( "Help!\n" ); fflush( stdout );
      exit( 0 );
      break;
    }
  } /* While more options exist */

  /*  If one more argument exists besides the options, use it as the
   *  init file (.set file).  */
  if (optind == (argc - 1))
  {
    setup_prog_rec( progptr, argv[optind], 1 );
    return;
  }

  /*  If there are no more arguments, use the default .set file.  */
  if (optind == argc)
  {
    fn = (char *) get_prog_def_fn( progptr, "default.set" );
    setup_prog_rec( progptr, fn, 0 );
    return;
  }

  /*  If there are more arguments (more than one), then the user
   *  typed in too many.  */
  fprintf( stderr, "Sorry, too many arguments given.\n" );
  exit( 1 );
}



/*--------------------------------------------------------------------------
main - ???
---------------------------------------------------------------------------*/

int
main( argc, argv )
     int argc;
     char *argv[];
{
  struct prog_rec	prog;
  Xv_singlecolor	*system_colors;
  unsigned int		num_system_colors;
  int			err;
  Xv_Server		server;

  /*  DRW 960910: for finding actual executable pathname  */
  char			execpath[MAXPATHLEN * 30];
  char			currpath[MAXPATHLEN];
  char			*tempstr;
  struct stat		fstatbuf;


  /* Initialize XV package, and handle all XV-ish arguments (removing
     them from argv[] simultaneously).  */
  server = xv_init( XV_INIT_ARGC_PTR_ARGV, &argc, argv,
		   XV_ERROR_PROC, xv_error_handler,
		   NULL );

  TRACE( printf( "Program name: %s\n", argv[0] ); );

  /* Find out our base location... currently, the support files need
      to either be in the same directory as the executable, or else
      they need to be pointed to by a supplementary info. file named
      "ria_init.dat", the path to which must be supplied as argv[1].  */
  if (strrchr( argv[0], '/' ) == NULL)
  {

    /* Since there is no / in our pathname, the shell used the PATH
       variable to find the executable.  If we can't find the PATH
       variable, we'll have to wing it.  */
    if ((tempstr = getenv( "PATH" )) != NULL)
    {

      /* strtok() modifies its search string, so let's make a copy
         first. */
      strncpy( execpath, tempstr, MAXPATHLEN * 30 );
      
      /* If initial strtok() returns NULL, then the PATH environment
	 variable was empty!  We'll just have to wing it then.
	 pathnames in the PATH environment variable are delimited by a
	 colon (:).  We extract each pathname, add a / and the
	 executable's name, and test to see if the resultant filename
	 exists and is a proper executable file.

	 This should not take much time at all since we're only
	 stat()'ing one filename per pathname in PATH. */

      if (tempstr = strtok( execpath, ":" ))
      {
	strcpy( currpath, tempstr );
	strcat( currpath, "/" );
	strcat( currpath, argv[0] );
	TRACE(printf( "Checking \"%s\"...\n", currpath ));
	if (stat( currpath, &fstatbuf ) != 0 ||	/* exists? */
	    !(fstatbuf.st_mode & S_IFREG) || /* regular file? */
	    !(fstatbuf.st_mode & (S_IXGRP | S_IXOTH))) /* executable
							  by group or
							  world? */
	{

#if 0
	  printf("Not in \"%s\":", currpath);
	  if (stat(currpath,&fstatbuf)!=0)
	    printf(" doesn't exist.\n");
	  else if (!(fstatbuf.st_mode & S_IFREG))
	    printf(" not regular file.\n");
	  else if (!(fstatbuf.st_mode & (S_IXGRP | S_IXOTH)))
	    printf(" not executable.\n");
#endif

	  /* Try searching next pathname... */
	  while (tempstr = strtok( NULL, ":" ))
	  {
	    strcpy( currpath, tempstr );
	    strcat( currpath, "/" );
	    strcat( currpath, argv[0] );
	    TRACE(printf( "Checking \"%s\"...\n", currpath ));
	    if (stat( currpath, &fstatbuf ) == 0 && /* exists? */
		(fstatbuf.st_mode & S_IFREG) &&	/* regular file? */
		(fstatbuf.st_mode & (S_IXGRP | S_IXOTH))) /* executable
							     by group
							     or world?  */
	    {
	      break;
	      /* If we break, currpath holds the path to our
		  executable, which was found in the current directory
		  in the PATH variable. */
	    }
#if 0
	    printf("Not in \"%s\":", currpath);
	    if (stat(currpath,&fstatbuf)!=0)
	      printf(" doesn't exist.\n");
	    else if (!(fstatbuf.st_mode & S_IFREG))
	      printf(" not regular file.\n");
	    else if (!(fstatbuf.st_mode & (S_IXGRP | S_IXOTH)))
	      printf(" not executable.\n");
#endif

	  }

	  /*  If none of the paths in the PATH variable yields a
	      proper executable path, we'll have to... you guessed
	      it, WING IT!  */
	  if (tempstr == NULL)
	  {
	    fprintf( stderr, "nria: Can not find \"%s\" in PATH", argv[0] );
	    fprintf( stderr, " environment variable.\n" );
	    strcpy( currpath, argv[0] );
	  }

	}
	/* At this point, currpath holds the path to our executable,
	   which was found in the first directory in the PATH
	   variable. */

      }
      else
      {
	/* Could not find pathname in PATH environment variable */
	fprintf( stderr, "nria: Improper PATH environment variable.\n" );
	strcpy( currpath, argv[0] );
      }

    }
    else
    {
      /* Could not find PATH environment variable */
      fprintf( stderr, "nria: Can not find PATH environment variable.\n" );
      strcpy( currpath, argv[0] );
    }
      
  }
  else
  {
    /* Already have a full / relative pathname, so don't bother
        searching.  */
    TRACE(printf( "nria: Already have full pathname, \"%s\"\n",
		  argv[0] ););
    strcpy( currpath, argv[0] );
  }

  TRACE(printf( "nria: main(), currpath = \"%s\"\n", currpath ));

  init_prog_rec( &prog, currpath );
  if ((err = get_system_colors( server, 0, &system_colors,
			       &num_system_colors)) != NO_ERROR)
  {
    fprintf( stderr, "Error...Unable to get system colors...\n" );
    exit(1);
  }
  TRACE(printf( "nria: main() BP#1\n" ););


  exit_handler( 1, &prog, bad_exit_called_proc );
  TRACE(printf( "nria: main() BP#2\n" ););

  parseArgs( argc, argv, &prog );
  TRACE(printf( "nria: main() BP#3\n" ););

  setup_all_win_rec( &(prog.win), &prog,
		    prog.defaults.whch.color_fn,
		    prog.defaults.whch.roi_labels_fn,
		    prog.defaults.whch.sec_roi_labels_fn,
		    prog.defaults.whch.cv_roi_labels_fn,
		    prog.defaults.whch.cv_sec_roi_labels_fn,
		    prog.defaults.whch.pixel_types_fn,
		    prog.defaults.whch.sec_pixel_types_fn,
		    prog.defaults.whch.pixel_type_priority_fn,
		    system_colors, num_system_colors );
  TRACE(printf( "nria: main() BP#4\n" ););

  (void) abort_program( &prog, 0 );
  main_display( SETUP_MAIN_DISPLAY, &prog, server, 0 );
  TRACE(printf( "nria: main() BP#5\n" ););

  /*  This indirectly turns control over to event loop of Notifier  */
  main_display( SHOW_MAIN_DISPLAY, &prog, server, 0 );

  TRACE(printf( "nria: main() end.\n" ););

  return 0;
}
