/****************************************************************************
dp_delete_roi.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: dp_delete_roi.c,v 1.10 1999/07/07 17:41:15 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRINT(x)
#define PRT(x)



#define CLEANUP_DP 0
#define SETUP_DELETE_ROI 1
#define GET_ALL_DP_PTR 2

/* for status */
#define SAVE_MAIN_LABELS 1
#define SAVE_SUB_LABELS 2
struct delete_roi_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame owner_frame;
    Frame frame;
    Panel panel;
    Panel_item rois,for_all_slices;
    int is_slice;
   };

struct all_delete_roi_recs
   {
    int num_dps;
    struct delete_roi_rec **dps;
   };


/* Functions defined in this file */
void init_delete_roi_rec(struct delete_roi_rec *dp);
void free_delete_roi_rec(struct delete_roi_rec *dp);
void init_all_delete_roi_recs(struct all_delete_roi_recs *all_dps);
void dp_delete_roi_done_proc(Frame frame);
void dp_slice_rois(struct delete_roi_rec *dp);
void dp_slice_rois_proc(Panel_item item, Event event);
void dp_file_rois_proc(Panel_item item, Event event);
int delete_slice_rois(struct slice_roi_rec *slice_roi,
    struct strs_rec *roi_labels, int *is_deleted);
int win_delete_slice_rois(struct image_display_window *win,
    struct strs_rec *roi_labels);
int win_delete_file_rois(struct image_display_window *win,
    struct strs_rec *roi_labels);
int dp_delete_rois_proc(Panel_item item, Event event);
int setup_delete_roi_rec(struct delete_roi_rec *dp,
    struct image_display_window *win);
struct delete_roi_rec *find_delete_roi_rec(struct all_delete_roi_recs *all_dps,
    struct image_display_window *win);
void update_dp_delete_roi_menu(struct image_display_window *win,
    int is_slice_rois);
int enter_dp_delete_roi(struct image_display_window *win, int status);



/*------------------------------------------------------------------------
init_delete_roi_rec - ???
------------------------------------------------------------------------*/

void
init_delete_roi_rec(dp)
 struct delete_roi_rec *dp;
{
 dp->status = 0;
 dp->frame = 0;
 dp->win = NULL;
 dp->is_slice = 1;
}


/*------------------------------------------------------------------------
free_delete_roi_rec - ???
------------------------------------------------------------------------*/

void
free_delete_roi_rec(dp)
 struct delete_roi_rec *dp;
{
 struct all_delete_roi_recs *all_dps;

 if (dp->frame != 0)
    {
     xv_destroy_safe(dp->frame);
    }
 all_dps = (struct all_delete_roi_recs *) enter_dp_delete_roi(NULL,
            GET_ALL_DP_PTR);
 c_free(dp);
 DropHandleAryPtr(&(all_dps->dps),&(all_dps->num_dps),dp);
}


/*------------------------------------------------------------------------
init_all_delete_roi_recs - ???
------------------------------------------------------------------------*/

void
init_all_delete_roi_recs(all_dps)
 struct all_delete_roi_recs *all_dps;
{
 all_dps->num_dps = 0;
 all_dps->dps = NULL;
}


/*------------------------------------------------------------------------
dp_delete_roi_done_proc - ???
------------------------------------------------------------------------*/

void
dp_delete_roi_done_proc(frame)
 Frame frame;
{
 struct delete_roi_rec *dp;

 dp = (struct delete_roi_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_delete_roi_rec(dp);
}


/*------------------------------------------------------------------------
dp_slice_rois - ???
------------------------------------------------------------------------*/

void
dp_slice_rois(dp)
 struct delete_roi_rec *dp;
{
 struct strs_rec labels;

 delete_all_list_items(dp->rois);
 dp->is_slice = 1;
 init_strs_rec(&labels);
 WinGetRoiLabelPtrs(dp->win,&labels,0,1);
 setup_list_items(&labels,dp->rois,0);
 free_strs_rec_ptrs(&labels);
}


/*------------------------------------------------------------------------
dp_slice_rois_proc - ???
------------------------------------------------------------------------*/

void
dp_slice_rois_proc(item,event)
 Panel_item item;
 Event event;
{
 struct delete_roi_rec *dp;

 dp = (struct delete_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 dp_slice_rois(dp);
}


/*------------------------------------------------------------------------
dp_file_rois_proc - ???
------------------------------------------------------------------------*/

void
dp_file_rois_proc(item,event)
 Panel_item item;
 Event event;
{
 struct delete_roi_rec *dp;
 struct strs_rec labels;

 dp = (struct delete_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 delete_all_list_items(dp->rois);
 dp->is_slice = 0;
 init_strs_rec(&labels);
 WinGetAllRoiLabelPtrs(dp->win,&labels,0,1);
 setup_list_items(&labels,dp->rois,0);
 free_strs_rec_ptrs(&labels);
}


/*------------------------------------------------------------------------
delete_slice_rois - ???
------------------------------------------------------------------------*/

int
delete_slice_rois(slice_roi,roi_labels,is_deleted)
 struct slice_roi_rec *slice_roi;
 struct strs_rec *roi_labels;
 int *is_deleted;
{
 long lcv;
 struct roi_rec *roi;
 int err;

 for (*is_deleted = 0,lcv = 0; lcv < roi_labels->num_strs; lcv++)
    if ((roi = get_roi(slice_roi,roi_labels->strs[lcv])) != NULL)
       {if ((err = delete_roi(slice_roi,roi)) != NO_ERROR)
           return err;
        *is_deleted = 1;}
 return NO_ERROR;
}

/************************
win_delete_histogram_rois(win,roi_labels)
 struct image_display_window *win;
 struct strs_rec *roi_labels;
{
 struct img_buf_rec *img;
 struct slice_roi_win_rec *slice_roi_win;
 int err,yes;

 if (((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
     ((slice_roi_win = get_histo_slice_roi_win_rec(&(img->file->histo_params)))
               == NULL))
    return CANCEL;
 err = delete_slice_rois(&(slice_roi_win->slice_roi),
          roi_labels,&yes);
 if (yes) win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,2,0,0,0);
 return err;
}
*****************/


/*------------------------------------------------------------------------
win_delete_slice_rois - ???
------------------------------------------------------------------------*/

int
win_delete_slice_rois(win,roi_labels)
 struct image_display_window *win;
 struct strs_rec *roi_labels;
{
 int err,yes;

 if (roi_labels->num_strs <= 0) return NO_ERROR;
/*
 if (win->type2 & HISTOGRAM_WINDOW)
    return win_delete_histogram_rois(win,roi_labels);
*/
 err = delete_slice_rois(&(win->control.sroi.sroi_win->slice_roi),
          roi_labels,&yes);
 if (yes) win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,1,0,0,0);
 return err;
}


/*------------------------------------------------------------------------
win_delete_file_rois - ???
------------------------------------------------------------------------*/

int
win_delete_file_rois(win,roi_labels)
 struct image_display_window *win;
 struct strs_rec *roi_labels;
{
 int err;
 struct img_buf_rec *img;
 struct slice_roi_win_rec **sroi_wins;
 int is_deleted,yes;
 long lcv;

 if (roi_labels->num_strs <= 0) return NO_ERROR;
 if (win->type2 & HISTOGRAM_WINDOW)
    {err = delete_slice_rois(&(win->control.sroi.sroi_win->slice_roi),
               roi_labels,&yes);
     if (yes)
	 win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,2,0,0,0);
     return err;}

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 sroi_wins = img->file->slice_info.sroi_wins;
 err = NO_ERROR;
 for (yes = 0,lcv = 0; lcv < img->file->slice_info.num_sroi_wins; lcv++)
    {if ((err = delete_slice_rois(&(sroi_wins[lcv]->slice_roi),roi_labels,
          &is_deleted)) != NO_ERROR)
        break;
     yes |= is_deleted;}
 if (yes)
    win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,1,0,0,0);
 return err;
}


/*------------------------------------------------------------------------
dp_delete_rois_proc - ???
------------------------------------------------------------------------*/

int
dp_delete_rois_proc(item,event)
 Panel_item item;
 Event event;
{
 struct delete_roi_rec *dp;
 struct strs_rec roi_labels;
 int err;

 PRT(printf("dp_delete_rois_proc...\n"))
 dp = (struct delete_roi_rec *) xv_get(item,PANEL_CLIENT_DATA);
 init_strs_rec(&roi_labels);
 if (((err = selected_list_item_ptrs(&roi_labels,dp->rois)) == NO_ERROR) &&
     (roi_labels.num_strs > 0))
    if (xv_get(dp->for_all_slices,PANEL_VALUE))
       {
        PRT(printf("dp_delete_rois_proc...win_delete_file_rois\n"))
        err = win_delete_file_rois(dp->win,&roi_labels);}
      else err = win_delete_slice_rois(dp->win,&roi_labels);
 free_strs_rec_ptrs(&roi_labels);
 if (dp->is_slice) dp_slice_rois_proc(item,event);
   else dp_file_rois_proc(item,event);
 return err;
}


/*------------------------------------------------------------------------
setup_delete_roi_rec - ???
------------------------------------------------------------------------*/

int
setup_delete_roi_rec(dp,win)
 struct delete_roi_rec *dp;
 struct image_display_window *win;
{
 char *str;
 int row,offset;
 Rect *rr;
 Panel_item item;
 struct strs_rec labels;

 dp->win = win;
 dp->dwin = win->dwin;
 dp->frame = xv_create(win->dwin->frame, FRAME,
              FRAME_INHERIT_COLORS,TRUE,
              0);
 TRACE(printf("setup_delete_roi_rec...1\n"))
 if (dp->frame == 0)
    {free_delete_roi_rec(dp);
     return CANCEL;}
 str = win_rec_win_id_title(win->owner,DELETE_ROI);
 xv_set(dp->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,dp_delete_roi_done_proc,
              WIN_CLIENT_DATA,dp,
              0);
 c_free(str);
 TRACE(printf("ok111.,..."))
 dp->panel = xv_create(dp->frame,PANEL,
                         FRAME_INHERIT_COLORS,TRUE,
                         0);
 if (dp->panel == 0)
    {free_delete_roi_rec(dp);
     return CANCEL;}
 row = 0;
 offset = xv_rows(dp->panel,1) - xv_rows(dp->panel,0) - 10;
 item = xv_create(dp->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"SLICE ROIS",
                   PANEL_NOTIFY_PROC,dp_slice_rois_proc,
                   XV_X,5,XV_Y,row,
                   PANEL_CLIENT_DATA,dp,
                   0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(dp->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"FILE ROIS",
                   PANEL_NOTIFY_PROC,dp_file_rois_proc,
                   XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
                   PANEL_CLIENT_DATA,dp,
                   0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(dp->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"ALL",
                   PANEL_NOTIFY_PROC,all_list_proc,
                   XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
                   XV_KEY_DATA,1,&(dp->rois),
                   0);
 rr = (Rect *) xv_get(item,XV_RECT);
 item = xv_create(dp->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"NONE",
                   PANEL_NOTIFY_PROC,none_list_proc,
                   XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
                   XV_KEY_DATA,1,&(dp->rois),
                   0);
 row += offset;

 dp->rois = xv_create(dp->panel,PANEL_LIST,
        PANEL_LAYOUT,PANEL_VERTICAL,
        PANEL_CHOOSE_ONE,FALSE,
        PANEL_LIST_DISPLAY_ROWS,6,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,dp,0);
 dp->is_slice = 1;
 init_strs_rec(&labels);
 WinGetRoiLabelPtrs(dp->win,&labels,0,1);
 setup_list_items(&labels,dp->rois,0);
 free_strs_rec_ptrs(&labels);

 rr = (Rect *) xv_get(dp->rois,XV_RECT);
 item = xv_create(dp->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING,"DELETE ROIS",
                   PANEL_NOTIFY_PROC,dp_delete_rois_proc,
                   XV_X,5,XV_Y,rr->r_top + rr->r_height + 5,
                   PANEL_CLIENT_DATA,dp,
                   0);
 rr = (Rect *) xv_get(item,XV_RECT);
 dp->for_all_slices = item = xv_create(dp->panel,PANEL_CHECK_BOX,
	PANEL_CHOICE_STRINGS,"FOR ALL SLICES",NULL,
        XV_X,rr->r_left + rr->r_width + 5,XV_Y,rr->r_top,
        PANEL_VALUE,0,
        PANEL_CLIENT_DATA,dp,
	0);

 window_fit(dp->panel);
 rr = (Rect *) xv_get(dp->panel,XV_RECT);
 xv_set(dp->panel,XV_HEIGHT,rr->r_height - 13,XV_WIDTH,rr->r_width - 15,0);
 window_fit(dp->frame);
 make_2_wins_adjacent(dp->frame,win->dwin->frame,0);
 win_set_mouse_at_center(dp->frame);
 xv_set(dp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);

 return NO_ERROR;
}


/*------------------------------------------------------------------------
find_delete_roi_rec - ???
------------------------------------------------------------------------*/

struct delete_roi_rec *
find_delete_roi_rec(all_dps,win)
 struct all_delete_roi_recs *all_dps;
 struct image_display_window *win;
{
 int lcv;

 for (lcv = 0; lcv < all_dps->num_dps; lcv++)
    if (all_dps->dps[lcv]->win == win) return all_dps->dps[lcv];
 return NULL;
}


/*------------------------------------------------------------------------
update_dp_delete_roi_menu - ???
------------------------------------------------------------------------*/

void
update_dp_delete_roi_menu(win,is_slice_rois)
 struct image_display_window *win;
 int is_slice_rois;
{
 struct all_delete_roi_recs *all_dps;
 struct delete_roi_rec *dp;

 if ((all_dps = (struct all_delete_roi_recs *) enter_dp_delete_roi(NULL,
            GET_ALL_DP_PTR)) != NULL)
    {if ((dp = find_delete_roi_rec(all_dps,win)) != NULL)
        if (is_slice_rois || dp->is_slice) dp_slice_rois(dp);
    }
}


/*------------------------------------------------------------------------
enter_dp_delete_roi - ???
------------------------------------------------------------------------*/

int
enter_dp_delete_roi(win,status)
 struct image_display_window *win;
 int status;
{
 static struct all_delete_roi_recs all_dps;
 struct delete_roi_rec *dp;
 static char first_time = 1;
 int err;

 TRACE(printf("enter_dp_delete_roi...0\n"))
 err = NO_ERROR;
 if (first_time)
    {init_all_delete_roi_recs(&all_dps);
     first_time = 0;}
 if (status == GET_ALL_DP_PTR)
    {return (unsigned long) &all_dps;}     /* FIXME:  ptr as long */
  else if (status == CLEANUP_DP)
   {if ((dp = find_delete_roi_rec(&all_dps,win)) != NULL)
       free_delete_roi_rec(dp);
    return NO_ERROR;}
 TRACE(printf("enter_dp_delete_roi...1\n"))
 if ((dp = find_delete_roi_rec(&all_dps,win)) != NULL)
    {xv_set(dp->frame,
               XV_SHOW,TRUE,WIN_ALARM,FRAME_CLOSED,FALSE,0);
     win_set_mouse_at_center(dp->frame);
     wmgr_top(dp->frame);
     return NO_ERROR;}

 TRACE(printf("enter_dp_delete_roi...2\n"))
 get_handle_memory(&(all_dps.dps),&(all_dps.num_dps),1,
     (size_t) sizeof(struct delete_roi_rec),&err);
 if (err != NO_ERROR) return err;
 dp = all_dps.dps[all_dps.num_dps - 1];
 init_delete_roi_rec(dp);
 TRACE(printf("enter_dp_delete_roi...2\n"))
 if ((err = setup_delete_roi_rec(dp,win)) == NO_ERROR)
    {TRACE(printf("setup_delete_roi_rec...\n"))
    }
   else
    ShowMsgStrs(win->dwin->frame,0,err,0,2,
               "An error occured when setting up the window for",
               TRANS_ROI,NULL,0);
 return err;

}

