/****************************************************************************
edit_and_display.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: edit_and_display.c,v 1.10 1999/07/07 17:41:16 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <unistd.h>

#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/textsw.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "nria_procedures.h"


/* DRW 970301: Turn off editing so files aren't accidentally changed.  */

#if 0
#  define TRACE(x) x; fflush(stdout);
#  define PRT(x) x; fflush(stdout);
#else
#  define TRACE(x)
#  define PRT(x)
#endif



#define CLEANUP_EDIT_FILE 0
#define SETUP_EDIT_FILE 1

#define CLEAN_UP_DISPLAY_TEXT_FILE 0
#define SETUP_OR_CREATE_DISPLAY_TEXT_FILE 1
#define SETUP_DISPLAY_TEXT_FILE 2


struct edit_text_rec
{
  struct the_edit_text_rec *the_edit;
  Frame frame,last_owner;
  void *client_unique_handle;
  Textsw text_win;
  char *fn;
  int (*exit_proc)();
  void *arg;
};

struct the_edit_text_rec
{
  unsigned long total_text_wins,num_text_wins;
  struct edit_text_rec **text_wins;
};


/* Functions defined in this file */
void init_edit_text_rec(struct edit_text_rec *edit,
    struct the_edit_text_rec *the_edit);
void free_edit_text_rec(struct edit_text_rec *edit);
void init_the_edit_text_rec(struct the_edit_text_rec *the_edit);
void free_the_edit_text_rec(struct the_edit_text_rec *the_edit);
struct edit_text_rec *find_edit_text_rec(struct the_edit_text_rec *the_edit,
    void *client_unique_handle);
void edit_text_done_proc(Frame frame);
int setup_edit_file(struct edit_text_rec *edit, Frame win_owner,
    void *client_unique_handle, char *win_label, char *fn, int can_edit,
    int (*exit_proc)(), void *arg);
int enter_edit_file(Frame win_owner, void *client_unique_handle,
    char *win_label, char *fn, int can_edit, int (*exit_proc)(), void *arg,
    int option);
void text_done_proc(Frame frame);
void enter_display_text_file(Frame owner, int status, char *frame_label,
    char *fn);



/*--------------------------------------------------------------------------
init_edit_text_rec - ???
--------------------------------------------------------------------------*/

void
init_edit_text_rec( edit, the_edit )
     struct edit_text_rec *edit;
     struct the_edit_text_rec *the_edit;
{
  edit->the_edit = the_edit;
  edit->frame = 0;
  edit->last_owner = 0;
  edit->client_unique_handle = NULL;
  edit->text_win = 0;
  edit->fn = NULL;
  edit->exit_proc = NULL;
  edit->arg = NULL;
}



/*--------------------------------------------------------------------------
free_edit_text_rec - ???
--------------------------------------------------------------------------*/

void
free_edit_text_rec( edit )
     struct edit_text_rec *edit;
{
  TRACE(printf("free_edit_text_rec...edit = %u exit_proc = %u\n",
	       edit,edit->exit_proc));
  if (edit->exit_proc != NULL)
    edit->exit_proc(edit->arg,edit->fn,1);
  TRACE(printf("frame = %u\n",edit->frame));

  if (edit->frame != 0)
    xv_destroy_safe(edit->frame);
  c_free(edit->fn);
  c_free(edit);
  DropPtrFromHandleAry(edit->the_edit->text_wins,
		       &(edit->the_edit->num_text_wins),edit);
}



/*--------------------------------------------------------------------------
init_the_edit_text_rec - ???
--------------------------------------------------------------------------*/

void
init_the_edit_text_rec( the_edit )
     struct the_edit_text_rec *the_edit;
{
  the_edit->total_text_wins = the_edit->num_text_wins = 0;
  the_edit->text_wins = NULL;
}



/*--------------------------------------------------------------------------
free_the_edit_text_rec - ???
--------------------------------------------------------------------------*/

void
free_the_edit_text_rec( the_edit )
     struct the_edit_text_rec *the_edit;
{
  unsigned long lcv;

  if (the_edit->total_text_wins > 0)
  {
    for (lcv = 0; lcv < the_edit->num_text_wins; lcv++)
    {
      free_edit_text_rec(the_edit->text_wins[lcv]);
      c_free(the_edit->text_wins[lcv]);
    }
    c_free(the_edit->text_wins);
  }
  init_the_edit_text_rec(the_edit);
}



/*--------------------------------------------------------------------------
find_edit_text_rec - ???
--------------------------------------------------------------------------*/

struct edit_text_rec *
find_edit_text_rec( the_edit, client_unique_handle )
     struct the_edit_text_rec *the_edit;
     void *client_unique_handle;
{
  unsigned long lcv;

  for (lcv = 0; lcv < the_edit->num_text_wins; lcv++)
  {
    if (the_edit->text_wins[lcv]->client_unique_handle == client_unique_handle)
    {
      return the_edit->text_wins[lcv];
    }
  }

  return NULL;
}



/*--------------------------------------------------------------------------
edit_text_done_proc - ???
--------------------------------------------------------------------------*/

void
edit_text_done_proc( frame )
     Frame frame;
{
  struct edit_text_rec *edit;
  
  edit = (struct edit_text_rec *) xv_get(frame,WIN_CLIENT_DATA);
  TRACE(printf("edit_text_done_proc...edit = %u frame = %u\n", edit, frame));
  xv_set( frame,
	  FRAME_NO_CONFIRM, TRUE,
	  NULL );
  TRACE(printf("edit_text_done_proc...text_win = %u\n", edit->text_win));

  if (xv_get(edit->text_win,TEXTSW_MODIFIED))
  {
    TRACE(printf("edit_text_done_proc TEXTSW_MODIFIED\n"));
    textsw_save(edit->text_win,0,0);
  }

  TRACE(printf("edit_text_done_proc...\n"));
  free_edit_text_rec(edit);
}



/*--------------------------------------------------------------------------
setup_edit_file - ???
--------------------------------------------------------------------------*/

int
setup_edit_file( edit, win_owner, client_unique_handle, win_label,
		 fn, can_edit, exit_proc, arg )
     struct edit_text_rec *edit;
     Frame win_owner;
     void *client_unique_handle;
     char *win_label,*fn;
     int can_edit;
     int (*exit_proc)();
     void *arg;
{
  long height,width;
  int err;

  PRT(printf("setup_edit_file...win_label = %s\n",win_label));
  if ((err = cp_str(&(edit->fn),fn)) != NO_ERROR)
  {
    free_edit_text_rec(edit);
    return err;
  }
  edit->frame = xv_create( win_owner, FRAME,
			   FRAME_SHOW_LABEL,TRUE,
			   FRAME_DONE_PROC,edit_text_done_proc,
			   XV_LABEL,win_label,
			   WIN_CLIENT_DATA,edit,
			   FRAME_NO_CONFIRM,TRUE,
			   NULL );
  PRT(printf("setup_edit_file...1\n"));

  if (edit->frame == 0)
  {
    free_edit_text_rec(edit);
    return CANCEL;
  }

  PRT(printf("edit = %u frame = %u\n",edit,edit->frame));
  err = TEXTSW_STATUS_OKAY;

  if (((edit->text_win = (Textsw) xv_create(edit->frame,TEXTSW,
					    TEXTSW_DISABLE_LOAD,TRUE,
					    TEXTSW_DISABLE_CD,TRUE,
					    TEXTSW_STORE_CHANGES_FILE,FALSE,
					    TEXTSW_FILE,fn,TEXTSW_STATUS,&err,
					    NULL)) == 0) ||
      (err != TEXTSW_STATUS_OKAY))
  {
    PRT(printf("setup_edit_file...2\n"));
    free_edit_text_rec(edit);
    return CANCEL;
  }
  if (!can_edit)
  {
    xv_set( edit->text_win,
	    TEXTSW_READ_ONLY, TRUE,
	    NULL );
  }
  PRT(printf("setup_edit_file...text_win = %u\n",edit->text_win));
  
  edit->client_unique_handle = client_unique_handle;
  edit->exit_proc = exit_proc;
  edit->arg = arg;
  edit->last_owner = win_owner;

  window_fit(edit->text_win);
  width = xv_cols(edit->text_win,1) - 
    xv_cols(edit->text_win,0);
  height = xv_rows(edit->text_win,1) - xv_rows(edit->text_win,0);
  xv_set( edit->text_win,
	  XV_HEIGHT, height * 45,
	  XV_WIDTH, width * 85,
	  NULL );
  xv_set( edit->text_win,
	  TEXTSW_FIRST_LINE, 16,
	  NULL );

  window_fit(edit->frame);
  make_2_wins_adjacent(edit->frame,win_owner,0);
  win_set_mouse_at_center(edit->frame);
  wmgr_top(edit->frame);
  xv_set( edit->frame,
	  FRAME_CLOSED, FALSE,
	  XV_SHOW, TRUE,
	  NULL );
  return NO_ERROR;
}



/*--------------------------------------------------------------------------
enter_edit_file - ???
--------------------------------------------------------------------------*/

int
enter_edit_file( win_owner, client_unique_handle, win_label,
                 fn, can_edit, exit_proc, arg, option )
     Frame win_owner;
     void *client_unique_handle;
     char *win_label,*fn;
     int can_edit;
     int (*exit_proc)();
     void *arg;
     int option;
{
  static struct the_edit_text_rec the_edit;
  struct edit_text_rec *edit;
  static int first_time = 1;
  unsigned long total;
  int err;

  if (first_time)
  {
    first_time = 0;
    init_the_edit_text_rec(&the_edit);
  }
  edit = find_edit_text_rec(&the_edit,client_unique_handle);

  PRT(printf("enter_edit_file total_text_wins = %d num_text_wins = %d\n",
	     the_edit.total_text_wins,the_edit.num_text_wins));
  PRT(printf("client_unique_handle = %u edit = %u\n",client_unique_handle,
	     edit));

  if (option == CLEANUP_EDIT_FILE)
  {
    TRACE(printf("CLEANUP_EDIT_FILE...\n"));
    if ((edit != NULL) && (edit->last_owner == win_owner))
      free_edit_text_rec(edit);
    TRACE(printf("CLEANUP_EDIT_FILE...of\n"));
    return NO_ERROR;
  }

  if (edit == NULL)
  {
    total = 0;
    if ((err = get_ptr(&edit,&total,1,
		       (size_t) sizeof(struct edit_text_rec))) != NO_ERROR)
      return err;
    init_edit_text_rec(edit,&the_edit);
  }
  else if (edit->frame != 0)
  {
    PRT(printf("edit->frame != 0...\n"););
    if (win_owner != 0)
      if (edit->last_owner != win_owner)
      {
	xv_set(edit->frame,XV_OWNER,win_owner,0);
	edit->last_owner = win_owner;
      }
    xv_set(edit->text_win,TEXTSW_DISABLE_LOAD,FALSE,0);
    if (!can_edit)
      xv_set(edit->text_win,TEXTSW_READ_ONLY,TRUE,0);

    xv_set(edit->text_win,TEXTSW_FILE,fn,TEXTSW_STATUS,&err,0);
    c_free(edit->fn);
    if ((err != TEXTSW_STATUS_OKAY) ||
	((err = cp_str(&(edit->fn),fn)) != NO_ERROR))
    {
      free_edit_text_rec(edit);
      return CANCEL;
    }
    xv_set(edit->text_win,TEXTSW_DISABLE_LOAD,TRUE,0);
    edit->client_unique_handle = client_unique_handle;
    edit->exit_proc = exit_proc;
    edit->arg = arg;
    edit->last_owner = win_owner;
    wmgr_top(edit->frame);
    xv_set(edit->frame,XV_LABEL,win_label,
	   XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
    
    return NO_ERROR;
  }
  if (the_edit.num_text_wins >= the_edit.total_text_wins)
  {
    if ((err = add_handle_element(&(the_edit.text_wins),
				  &(the_edit.total_text_wins),edit))
	!= NO_ERROR)
    {
      c_free(edit);
      return err;
    }
  }
  the_edit.text_wins[the_edit.num_text_wins++] = edit;

  PRT(printf("...2...num_text_wins = %d total_text_wins = %d\n",
	     the_edit.num_text_wins,the_edit.total_text_wins));
  if ((err = setup_edit_file(edit,win_owner,client_unique_handle,win_label,
			     fn,can_edit,exit_proc,arg)) != NO_ERROR)
  {
    ShowMsgStrs(win_owner,0,err,0,2,
		"An error occured when setting up",
		"   the text window !!!",NULL,0);
  }
  TRACE(printf("enter_edit_file...end\n"));
  return err;
}



/*--------------------------------------------------------------------------
text_done_proc - ???
--------------------------------------------------------------------------*/

void
text_done_proc(frame)
     Frame frame;
{
  Frame *frame1;

  frame1 = (Frame *) xv_get(frame,WIN_CLIENT_DATA);
  xv_set(frame,FRAME_NO_CONFIRM,TRUE,0);
  xv_destroy_safe(frame);
  *frame1 = 0;
}



/*--------------------------------------------------------------------------
enter_display_text_file - ???
--------------------------------------------------------------------------*/

void
enter_display_text_file(owner,status,frame_label,fn)
     Frame owner;
     int status;
     char *frame_label,*fn;
{
  static Frame frame = 0, root_frame = 0;
  static Textsw text_win = 0;
  int err;
  char *str;
  FILE *fid;

  if (status == CLEAN_UP_DISPLAY_TEXT_FILE)
  {
    if ((owner == root_frame) && (frame != 0))
    {
      text_win = 0;
      frame = root_frame = 0;
    }
    return;
  }

  if (status == SETUP_OR_CREATE_DISPLAY_TEXT_FILE)
  {
    if (access(fn,F_OK) != 0)
      if ((fid = fopen(fn,"w")) != NULL)
	fclose(fid);
  }

  if ((frame != 0) && (root_frame != owner))
  {
    xv_destroy_safe(frame);
    text_win = 0;
    frame = root_frame = 0;
  }

  if (frame == 0)
  {
    if ((frame = xv_create(owner,FRAME,FRAME_SHOW_LABEL,TRUE,
			   FRAME_DONE_PROC,text_done_proc,
			   WIN_CLIENT_DATA,&frame,
			   FRAME_NO_CONFIRM,TRUE,
			   0)) == 0)
	return /*CANCEL*/;
    if (frame_label != NULL)
      xv_set(frame,XV_LABEL,frame_label,0);
    else
      xv_set(frame,XV_LABEL,fn,0);

    if ((text_win = xv_create(frame,TEXTSW,0)) == 0)
    {
      xv_destroy_safe(frame);
      frame = 0;
      return /*err*/;
    }
  }

  xv_set( text_win,
	  TEXTSW_FILE, fn,
	  TEXTSW_STATUS, &err,
	  NULL );
  if (err != TEXTSW_STATUS_OKAY)
  {
    xv_set( frame,
	    XV_SHOW, FALSE,
	    NULL );
    concat_strs(&str,2,"  help file = ",fn);
    ShowMsgStrs(owner,0,err,0,3,
		"An error occured when displaying the", str,
		"   The file might be missing !!!",NULL,0);
    c_free(str);
    return /*CANCEL*/;
  }
  window_fit(frame);
  make_2_wins_adjacent(frame,owner,0);
  win_set_mouse_at_center(frame);
  wmgr_top(frame);

  /*  DRW 970301: Turn off editing so files aren't accidentally changed.  */
  xv_set( text_win,
	  TEXTSW_DISABLE_LOAD, TRUE, /* prevent loading another file */
	  TEXTSW_DISABLE_CD, TRUE, /* prevent changedir */
	  TEXTSW_BROWSING, TRUE,	/* prevent edits */
	  TEXTSW_CHECKPOINT_FREQUENCY, 0, /* disable checkpointing */
	  TEXTSW_READ_ONLY, TRUE, /* prevent edits */
	  NULL );

  xv_set( frame,
	  FRAME_CLOSED, FALSE,
	  XV_SHOW, TRUE,
	  NULL );

  return /*NO_ERROR*/;
}
