/****************************************************************************
histo_more_opr.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: histo_more_opr.c,v 1.7 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdlib.h>

#include "mri_format.h"
#include "nria_procedures.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRINT(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define PRINT(x)
#define TRACE(x)
#define PRT(x)



/* Functions defined in this file */
int get_histo_stat(struct pixel_center_rec *center,
    struct buf_2d_rec *occurence, struct buf_2d_rec *mask,
    struct int_bound_rec *affected);
int get_bkg_histo_stat(struct pixel_center_rec *center,
    struct buf_2d_rec *occurence, struct buf_2d_rec *mask,
    struct int_bound_rec *affected, int which_gradient);
int get_histo_roi_stat(struct pixel_center_rec *center,
    struct buf_2d_rec *occurence, struct roi_rec *roi
   , struct draw_mask_rec *mask1);
int find_center(struct pixel_center_rec *centers, unsigned int num_centers,
    struct roi_rec *roi, int set_cur, unsigned int *cur_center);
int sort_centers_proc(const void *center1, const void *center2);
int sort_distance_proc(const void *elem1, const void *elem2);
int Partition1Center(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi);
int Partition2Centers_1(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi, struct pixel_center_rec *center,
    int avex, int avey, int angular_gradient);
int Partition2Centers(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi);
int PartitionManyCenter(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi);
int PartitionCenters(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi);
int HistoPixelTypeOrderType(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi);
int BuildHistoPixelTypeOrder(struct histo_params_rec *histo,
    struct slice_roi_rec *slice_roi);
int thresholding_pixels(struct histo_params_rec *histo);



/*--------------------------------------------------------------------------
get_histo_stat - ???
--------------------------------------------------------------------------*/

int
get_histo_stat(center,occurence,mask,affected)
 struct pixel_center_rec *center;
 struct buf_2d_rec *occurence,*mask;
 struct int_bound_rec *affected;
{
 unsigned int row,col,col_bit,indx,start_bit,end_bit,cnt;
 static unsigned char _1_bit[] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
 unsigned char cur_curve;
 unsigned int start_byte,start_col_bit,stop_byte;
 double sxy,sxx,sx,sy,total_occurences;
 unsigned int left,right,top,bottom;
 struct int_bound_rec bound;

 TRACE(printf("get_histo_stat...BEGIN\n"))
 setup_int_bound_rec(&bound,0,0,occurence->dimx - 1,occurence->dimy - 1);
 if (affected == NULL)
    valid_bound(&bound,&bound,&left,&right,&top,&bottom);
   else valid_bound(affected,&bound,&left,&right,&top,&bottom);

 center->ave_occurences = center->num_xy_pairs = 0;
 center->avex = center->avey = 0;
 center->gradient = 0;

 total_occurences = .0;
 sxy = sxx = sx = sy = .0;
 start_byte = left / 8;
 start_col_bit = start_byte * 8;
 stop_byte = right / 8 + 1;
 for (row = top; row <= bottom; row++)
    {
     start_bit = left - start_col_bit;
     end_bit = 8;
     for (col_bit = start_col_bit,col = start_byte;
             col < stop_byte; col++,col_bit += 8,start_bit = 0)
        {if (mask == NULL) cur_curve = 0xff;
           else cur_curve = mask->buf[row][col];
         if (cur_curve)
            {
             if ((col_bit + 8) > right) end_bit = right - col_bit + 1;
             for (indx = col_bit + start_bit,cnt = start_bit;
                                cnt < end_bit; cnt++,indx++)
                if (cur_curve & _1_bit[cnt])
                   {
                    total_occurences += 1;
                    center->num_xy_pairs += 1;
                    sx += indx;
                    sy += row;
                    sxx += indx * indx;
                    sxy += indx * row;
                   }
            }
        }
    }

 TRACE(printf("sx = %.1f sy = %.1f sxy = %f sxx = %f total = %.1f\n",
      sx,sy,sxy,sxx,total_occurences))
 if (center->num_xy_pairs == 0) return CANCEL;

 center->gradient = (total_occurences * sxy - sx * sy) / 
               (total_occurences * sxx - sx * sx);
 TRACE(printf("center->gradient = %f\n",center->gradient))
 center->avex = (long) ULongRound((double) sx /
                      (double) total_occurences);
 center->avey = (long) ULongRound((double) sy /
                      (double) total_occurences);
 center->ave_occurences = (long) ULongRound((double) total_occurences /
                                (double) center->num_xy_pairs);
 TRACE(printf("get_histo_stat...avex = %d avey = %d num_xy_pairs = %d ave_occurences = %d gradient = %f\n",
            center->avex,center->avey,center->num_xy_pairs,center->ave_occurences,center->gradient))

 TRACE(printf("get_histo_stat...END\n"))
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
get_bkg_histo_stat - ???

  which_gradient = 0  gradient = gx
  which_gradient = 1  gradient = gy
  which_gradient = 2  gradient = (gx + gy) / 2
--------------------------------------------------------------------------*/

int
get_bkg_histo_stat(center,occurence,mask,affected,which_gradient)
 struct pixel_center_rec *center;
 struct buf_2d_rec *occurence,*mask;
 struct int_bound_rec *affected;
 int which_gradient;
{
 unsigned int row,col,col_bit,indx,start_bit,end_bit,cnt;
 static unsigned char _1_bit[] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
 unsigned char cur_curve;
 unsigned int start_byte,start_col_bit,stop_byte;
 double gy,gx,sxy,sxx,syy,sx,sy,x,y,total_occurences,value;
 unsigned int left,right,top,bottom;
 struct int_bound_rec bound;

 PRT(printf("get_histo_stat...BEGIN\n"))
 setup_int_bound_rec(&bound,0,0,occurence->dimx - 1,occurence->dimy - 1);
 if (affected == NULL)
    valid_bound(&bound,&bound,&left,&right,&top,&bottom);
   else valid_bound(affected,&bound,&left,&right,&top,&bottom);

 center->ave_occurences = center->num_xy_pairs = 0;
 center->avex = center->avey = 0;
 center->gradient = 0;

 total_occurences = .0;
 sxy = sxx = syy = sx = sy = .0;
 start_byte = left / 8;
 start_col_bit = start_byte * 8;
 stop_byte = right / 8 + 1;
 for (row = top; row <= bottom; row++)
    {
     start_bit = left - start_col_bit;
     end_bit = 8;
     for (col_bit = start_col_bit,col = start_byte;
             col < stop_byte; col++,col_bit += 8,start_bit = 0)
        {if (mask == NULL) cur_curve = 0xff;
           else cur_curve = mask->buf[row][col];
         if (cur_curve)
            {
             if ((col_bit + 8) > right) end_bit = right - col_bit + 1;
             for (indx = col_bit + start_bit,cnt = start_bit;
                                cnt < end_bit; cnt++,indx++)
                if (cur_curve & _1_bit[cnt])
                   if ((value = occurence->GetVal(occurence,row,indx)) != .0)
                      {center->num_xy_pairs += 1;
                       total_occurences += value;
                       x = value * indx;
                       y = value * row;
                       sxx += x * x;
                       syy += y * y;
                       sx += x;
                       sy += y;
                       sxy += x * y;
                      }
             }
        }
    }

 PRT(printf("sx = %.1f sy = %.1f sxy = %f sxx = %.1f  syy = %.1f total = %.1f\n",
      sx,sy,sxy,sxx,syy,total_occurences))
 if (center->num_xy_pairs == 0) return CANCEL;
 PRT(printf("total_occurences * sxy = %f\n",total_occurences * sxy))
 PRT(printf("sx * sy = %f\n",sx * sy))
 PRT(printf("total_occurences * sxx = %f\n",total_occurences * sxx))
 PRT(printf("sx * sx = %f\n",sx * sx))
 gx = (total_occurences * sxy - sx * sy) / (total_occurences * sxx - sx * sx);
 PRT(printf("gx = %f\n",gx))
 if (which_gradient == 0) center->gradient = gx;
   else
    {gy = (sxy * total_occurences - sx * sy) / (total_occurences * syy - sy * sy);
     PRT(printf("gy = %f\n",gy))
     if (which_gradient == 1)
        {PRT(printf("&&&& gy...\n"))
         center->gradient = 1 / gy;} 
       else
        {PRT(printf("&&&&& gx + gy / 2...\n"))
         center->gradient = (gx + 1/gy) / 2;}
    };

 PRT(printf("center->gradient = %f\n",center->gradient))
 center->avex = (long) ULongRound((double) sx / (double) total_occurences);
 center->avey = (long) ULongRound((double) sy / (double) total_occurences);
 center->ave_occurences = (long) ULongRound((double) total_occurences /
                                (double) center->num_xy_pairs);
 PRT(printf("get_histo_stat...avex = %d avey = %d num_xy_pairs = %d ave_occurences = %d gradient = %f\n",
            center->avex,center->avey,center->num_xy_pairs,
      center->ave_occurences,center->gradient))

 TRACE(printf("get_histo_stat...END\n"))
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
get_histo_roi_stat - ???
--------------------------------------------------------------------------*/

int
get_histo_roi_stat(center,occurence,roi,mask1)
 struct pixel_center_rec *center;
 struct buf_2d_rec *occurence;
 struct roi_rec *roi;
 struct draw_mask_rec *mask1;
{
 struct int_bound_rec affected;
 int err;

 if ((err = cp_str(&(center->roi_label),roi->label)) != NO_ERROR)
    return err;
 FindRoiLongMinMax(roi,NULL,&(affected.left),&(affected.right),
              &(affected.top),&(affected.bottom),0,0,0);

 if ((err = FillDrawRoiRec(mask1,NULL,roi,NULL))
       != NO_ERROR)
    {free_pixel_center_rec(center);
     return err;}
   else if ((err = get_histo_stat(center,occurence,
          &(mask1->mask._2d),&affected)) != NO_ERROR)
    {
     free_pixel_center_rec(center);
     return err;}

 return NO_ERROR;

}


/*--------------------------------------------------------------------------
find_center - ???
--------------------------------------------------------------------------*/

int
find_center(centers,num_centers,roi,set_cur,cur_center)
 struct pixel_center_rec *centers;
 unsigned int num_centers;
 struct roi_rec *roi;
 int set_cur;
 unsigned int *cur_center;
{
 unsigned int lcv;

 for (lcv = 0; lcv < num_centers; lcv++)
    if (str_cmp(centers[lcv].roi_label,roi->label) == 0)
       {if (set_cur) *cur_center = lcv;
        return NO_ERROR;}
 return NOT_FOUND;
}


/*******************
int sort_centers_proc(center1,center2)
 struct pixel_center_rec *center1,*center2;

int sort_centers_proc(ptr1,ptr2)
 const void *ptr1,*ptr2;
{
 struct pixel_center_rec *center1,*center2;

 center1 = (struct pixel_center_rec *) ptr1;
 center2 = (struct pixel_center_rec *) ptr2;
 return (int) ((int) center1->ave_occurences - (int) center2->ave_occurences);
}
****************/


/*--------------------------------------------------------------------------
sort_centers_proc - ???
--------------------------------------------------------------------------*/

int 
sort_centers_proc(center1,center2)
 const void *center1,*center2;
{
 if (((struct pixel_center_rec *) center1)->ave_occurences <
     ((struct pixel_center_rec *) center2)->ave_occurences)
    return -1;
  else if (((struct pixel_center_rec *) center1)->ave_occurences >
     ((struct pixel_center_rec *) center2)->ave_occurences)
    return 1;
 return 0;
}


/******************
int sort_distance_proc(elem1,elem2)
 struct histo_table_elem_rec *elem1,*elem2;
{
 if (elem1->common->distance < elem2->common->distance)
    return (int) -1;
 if (elem1->common->distance == elem2->common->distance)
    return (int) 0;
 if (elem1->common->distance > elem2->common->distance)
    return (int) 1;
}
******************/


/*--------------------------------------------------------------------------
sort_distance_proc - ???
--------------------------------------------------------------------------*/

int 
sort_distance_proc(elem1,elem2)
 const void *elem1,*elem2;
{
 if (((struct histo_table_elem_rec *) elem1)->common->distance < 
     ((struct histo_table_elem_rec *) elem2)->common->distance)
    return (int) -1;
 if (((struct histo_table_elem_rec *) elem1)->common->distance >
     ((struct histo_table_elem_rec *) elem2)->common->distance)
    return (int) 1;
 return 0;
}


/*--------------------------------------------------------------------------
Partition1Center - ???
--------------------------------------------------------------------------*/

int
Partition1Center(histo,slice_roi)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
{
 char *str;
 int err;
 struct roi_rec *roi;

 if ((err = concat_strs(&str,2,histo->centers[0].roi_label,".TYPE"))
                 != NO_ERROR) return err;

 err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,str,
           (unsigned int) DRAW_POLYGON,0,5, .0, .0,
           .0,(float) (histo->_2d.dimy - 1),
           (float) (histo->_2d.dimx - 1),(float) (histo->_2d.dimy - 1),
           (float) (histo->_2d.dimx - 1),.0,.0,.0);
 if (err == NO_ERROR)
    {if ((roi = get_roi(slice_roi,str)) == NULL)
        {c_free(str);
         return CANCEL;}
       else
        {roi->status |= HISTO_SEGMENTED_ROI;
         histo->centers[0].segmented_roi_label = str;}
    }
   else c_free(str);
 return err;
}


/*--------------------------------------------------------------------------
Partition2Centers_1 - ???
--------------------------------------------------------------------------*/

int
Partition2Centers_1(histo,slice_roi,center,avex,avey,angular_gradient)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
 struct pixel_center_rec *center;
 int avex,avey,angular_gradient;
{
 int cnt,row,lcv,pts[2][2],corners[4][2],in[4];
 struct line_rec line;
 int sign,sign1;
 struct roi_rec *roi;
 char *str;
 int err;

 TRACE(printf("Partition2Centers_1 label = %s  cavex = %d  cavey = %d avex = %d avey = %d angular_gradient = %d\n",
     center->roi_label,center->avex,center->avey,avex,avey,angular_gradient))
 for (row = 0; row < 4; row++)
    {in[row] = 0;
     for (lcv = 0; lcv < 2; lcv++)
        corners[row][lcv] = 0;}
       
 corners[1][0] = histo->_2d.dimx - 1;
 corners[2][0] = corners[1][0];
 corners[2][1] = histo->_2d.dimy - 1;
 corners[3][1] = corners[2][1];

 get_2_intersections(avex,avey,angular_gradient,
          (unsigned int) 0,(unsigned int) (histo->_2d.dimx - 1),
          (unsigned int) 0,(unsigned int) (histo->_2d.dimy - 1),&(pts[0][0]),
          &(pts[0][1]),&(pts[1][0]),&(pts[1][1]));

 TRACE(printf("Partition2Centers_1...center->roi_label = %s\n",center->roi_label))
 TRACE(printf("[0]x = %d [0]y = %d   [1]x = %d [1]y = %d\n",pts[0][0],
         pts[0][1],pts[1][0],pts[1][1]))
 if ((err = concat_strs(&str,2,center->roi_label,".TYPE")) != NO_ERROR)
    return err;

 setup_line_rec(&line,(float) pts[1][0],(float) pts[1][1],(float) avex,(float) avey);
 sign = apply_line_equation(&line,(float) center->avex,(float) center->avey);
 TRACE(printf("sign = %d\n",sign))

 for (cnt = 0,row = 0; row < 4; row++)
    {sign1 = apply_line_equation(&line,(float) corners[row][0],
                    (float) corners[row][1]);
     TRACE(printf("%d) [0] = %d  [1] = %d  sign = %d sign1 = %d\n",row,corners[row][0],corners[row][1],sign,sign1))
     if (sign == sign1)
        {in[cnt] = row;
         cnt += 1;
        }
    }
            
 if ((err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,str,
          (unsigned long) DRAW_POLYGON,1,3,
          (float) pts[0][0],(float) pts[0][1],(float) avex,(float) avey,
          (float) pts[1][0],(float) pts[1][1])) != NO_ERROR) 
    {c_free(str);
     return err;}
 TRACE(
    for (lcv = 0; lcv < cnt; lcv++)
       printf("in %d) %d) %d %d\n",lcv,in[lcv],corners[in[lcv]][0],corners[in[lcv]][1]);
   )
 for (lcv = 0; lcv < cnt; )
    {
     setup_line_rec(&line,(float) pts[1][0],(float) pts[1][1],
           (float) corners[in[lcv]][0],(float) corners[in[lcv]][1]);
     sign = apply_line_equation(&line,(float) avex,(float) avey);
     sign1 = apply_line_equation(&line,(float) pts[0][0],(float) pts[0][1]);
     if ((sign != sign1) && (sign != 0) && (sign1 != 0))
        {lcv++;
         continue;}
     for (row = 0; row < cnt; row++)
        {if (row == lcv) continue;
         sign1 = apply_line_equation(&line,(float) corners[in[row]][0],
                  (float) corners[in[row]][1]);
         if ((sign != sign1) && (sign != 0) && (sign1 != 0)) break;
        }
     if (row == cnt)
        {TRACE(printf("enter %d) %d %d\n",
               in[lcv],corners[in[lcv]][0],corners[in[lcv]][1]);)
         if ((err = EnterPtsIntoSliceRoiRec(slice_roi,NULL,str,
              (unsigned long) DRAW_POLYGON,0,1,(float) corners[in[lcv]][0],
                      (float) corners[in[lcv]][1])) != NO_ERROR) 
            {c_free(str);
             return err;}
         cnt -= 1;
         for (row = lcv; row < cnt; row++)
            in[row] = in[row + 1];
         lcv = 0;
        }
       else lcv += 1;
    }
 if ((roi = get_roi(slice_roi,str)) == NULL)
    err = CANCEL;
   else err = EnterRoiCurvePt(roi,
           (unsigned short) DRAW_POLYGON,0,(float) pts[0][0],(float) pts[0][1],
           &(slice_roi->status),NULL);
 if (err != NO_ERROR)
    {c_free(str);
     return err;}
 roi->status |= HISTO_SEGMENTED_ROI;

 center->segmented_roi_label = str;
 TRACE(printf("Partition2Centers_1...end\n\n"))
 return err;
}


/*--------------------------------------------------------------------------
Partition2Centers - ???
--------------------------------------------------------------------------*/

int
Partition2Centers(histo,slice_roi)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
{
 int avex,avey,angular_gradient;
 float gradient;
 int err;


 avex = (long) ULongRound((double) ((double)
              (histo->centers[0].avex + histo->centers[1].avex) / (double) 2));
 avey = (long) ULongRound((double) ((double)
              (histo->centers[0].avey + histo->centers[1].avey) / (double) 2));
 TRACE(printf("Partition2Centers..avex = %x  avey = %d\n",avex,avey))
 gradient = (float) (atan2((double) histo->centers[0].gradient,(double) 1.0) +
             atan2((double) histo->centers[1].gradient,(double) 1.0)) / 2.0;
 gradient = (float) degree(gradient);
 angular_gradient = (int) ULongRound((double) gradient);
 if ((err = Partition2Centers_1(histo,slice_roi,&(histo->centers[0]),
              avex,avey,angular_gradient)) == NO_ERROR)
    err = Partition2Centers_1(histo,slice_roi,&(histo->centers[1]),
              avex,avey,angular_gradient);

 return err;
}


/*--------------------------------------------------------------------------
PartitionManyCenter - ???
--------------------------------------------------------------------------*/

int
PartitionManyCenter(histo,slice_roi)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
{
 unsigned int row,col,num_rows,total,temp1,temp2,lcv;
 struct histo_table_elem_rec **elems,temp_elem;
 struct neighbor_elem_rec *members;
 int err;
 struct histo_common_rec *common;

 num_rows = 0;
 if ((err = get_handle(&elems,&num_rows,histo->num_centers,
     (size_t) (histo->num_centers * sizeof(**elems)))) != NO_ERROR)
    return err;
 for (total = 0, temp1 = histo->num_centers;;)
    {if (temp1 == 0) break;
     total += temp1--;}
 if ((err = get_ptr(&(common),NULL,total,
                  (size_t) sizeof(*common))) != NO_ERROR)
    {free_handle_memory(elems,num_rows);
     return err;}

 if ((err = get_ptr(&members,NULL,num_rows,
                  (size_t) sizeof(*members))) != NO_ERROR)
    {free_handle_memory(elems,num_rows);
     c_free(common);
     return err;}

 qsort(histo->centers,histo->num_centers,
       (size_t) sizeof(struct pixel_center_rec),sort_centers_proc);
 for (col = 0; col < num_rows; col++)
    init_neighbor_elem_rec(&(members[col]));

 for (lcv = 0,row = 0; row < num_rows; row++)
    for (col = 0; col < num_rows; col++)
        {
         if (col >= row)
            {
            elems[row][col].common = &(common[lcv++]);
             elems[row][col].common->status = 0;
             if (col == row) elems[row][col].common->distance = 0;
               else
                {temp1 = histo->centers[row].avex - histo->centers[col].avex;
                 temp2 = histo->centers[row].avey - histo->centers[col].avey;
                 elems[row][col].common->distance = (float) sqrt((double)
                        (temp1 * temp1 + temp2 * temp2));}
            }
           else
            elems[row][col] = elems[col][row];
          elems[row][col].member = &(members[col]);
          elems[row][col].center = &(histo->centers[col]);
          elems[row][col].row_indx = col;
         }
  for (row = 0; row < num_rows; row++)
     {for (col = 0; col < num_rows; col++)
         if (col == row)
            {temp_elem = elems[row][0];
             elems[row][0] = elems[row][col];
             elems[row][col] = temp_elem;
            }
      qsort(&(elems[row][1]),(int) (num_rows - 1),
          (size_t) sizeof(struct histo_table_elem_rec),sort_distance_proc);
      }

 err = PartitionPixelTypes(elems,num_rows,histo,members,slice_roi);

 free_all_neighbor_elem_recs(members,num_rows);
 c_free(members);
 c_free(common);
 free_handle_memory(elems,num_rows);
 TRACE(printf("PartitionManyCenter end err = %d\n",err))
 return err;
}


/*--------------------------------------------------------------------------
PartitionCenters - ???
--------------------------------------------------------------------------*/

int
PartitionCenters(histo,slice_roi)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
{
 TRACE(printf("PartitionCenters...begin histo->num_centers = %d\n",
                histo->num_centers))
 if (histo->num_centers == 0) return NO_ERROR;
 if (histo->num_centers == 1) return Partition1Center(histo,slice_roi);
 if (histo->num_centers == 2) return Partition2Centers(histo,slice_roi);
 return PartitionManyCenter(histo,slice_roi);
}


/*--------------------------------------------------------------------------
HistoPixelTypeOrderType - ???
--------------------------------------------------------------------------*/

int
HistoPixelTypeOrderType(histo,slice_roi)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
{
 unsigned long lcv;
 char *str,*str1;
 int err;
 struct roi_rec *roi;

 TRACE(printf("HistoPixelTypeOrderType...\n"))
 TRACE(pr_strs_rec(&(histo->pixel_type_order));)
 for (lcv = 0; lcv < histo->pixel_type_order.num_strs; lcv++)
    {if ((roi = get_roi(slice_roi,
          histo->pixel_type_order.strs[lcv])) != NULL)
        if (!(roi->status & HISTO_EDITED_ROI))
           {str = histo->pixel_type_order.strs[lcv];
            if ((err = concat_strs(&str1,2,str,".TYPE")) != NO_ERROR)
               return err;
            c_free(str);
            histo->pixel_type_order.strs[lcv] = str1;}
    }
 TRACE(printf("HistoPixelTypeOrderType...1\n"))
 TRACE(pr_strs_rec(&(histo->pixel_type_order));)
 return NO_ERROR;
}


/*--------------------------------------------------------------------------
BuildHistoPixelTypeOrder - ???
--------------------------------------------------------------------------*/

int
BuildHistoPixelTypeOrder(histo,slice_roi)
 struct histo_params_rec *histo;
 struct slice_roi_rec *slice_roi;
{
 int err;

 TRACE(printf("BuildHistoPixelTypeOrder...\n"))
 if ((err = HistoPixelTypeOrderType(histo,slice_roi)) == NO_ERROR)
    if (histo->file != NULL)
       {histo->file->pixel_type.status &= ~HISTO_AND_PIXEL_TYPE_RELATED;
        histo->file->output.status |= FILE_TO_BE_SAVED;}

 TRACE(printf("BuildHistoPixelTypeOrder...err = %d\n",err))
 return err;
}


/*--------------------------------------------------------------------------
thresholding_pixels - ???
--------------------------------------------------------------------------*/

int
thresholding_pixels(histo)
 struct histo_params_rec *histo;
{
 struct roi_rec *roi;
 struct draw_mask_rec mask1;
 struct slice_roi_win_rec *sroi_win;
 unsigned int lcv;
 int err;
 unsigned short histo_type2;

 TRACE(printf("segment_pixels 0\n"))
 histo_type2 = file_get_def_histo_type(histo->file);
 sroi_win = (struct slice_roi_win_rec *)
                  GetHistoSliceRoiWinRec(histo,histo_type2);
 if ((err = SetupHistoPixelTypeOrder(histo,
         &(histo->file->prog->win.pixel_type_priority),0,
        NULL,NULL,histo_type2)) != NO_ERROR) 
    return err;
 TRACE(pr_strs_rec(&(histo->pixel_type_order)))

 free_histo_pixel_center_rec(histo);
 init_draw_mask_rec(&mask1);
 if ((err = get_draw_mask_rec(&mask1,histo->_2d.dimx,histo->_2d.dimy)) 
          != NO_ERROR)
    {free_draw_mask_rec(&mask1);
     return err;}
 TRACE(printf("segment_pixels 1...err = %d  num_centers = %d\n",
      err,histo->num_centers))

 for (lcv = 0; lcv < histo->pixel_type_order.num_strs; lcv++)
    {if (((roi = get_roi(&(sroi_win->slice_roi),
                  histo->pixel_type_order.strs[lcv])) == NULL) ||
         !is_roi_non_empty(roi))
        return CANCEL;
     if (roi->status & HISTO_EDITED_ROI) continue;
     if (find_center(histo->centers,histo->num_centers,
              roi,1,&(histo->cur_center)) != NO_ERROR)
        {if (histo->num_centers >= histo->total_centers)
            {if ((err = get_ptr(&(histo->centers),&(histo->total_centers),
                   (unsigned int) 4,
                    (size_t) sizeof(struct pixel_center_rec))) != NO_ERROR)
                {TRACE(printf("segment_pixels 2...err = %d\n",err))
                 break;}
            }
         histo->cur_center = histo->num_centers;
         histo->num_centers += 1;
         init_pixel_center_rec(&(histo->centers[histo->cur_center]));
         TRACE(printf("centers = %u cur_center = %d roi->label = %s\n",
                 histo->centers,histo->cur_center,roi->label))
         if ((err = get_histo_roi_stat(&(histo->centers[histo->cur_center]),
              &(histo->_2d),roi,&mask1)) != NO_ERROR)
            {TRACE(printf("segment_pixels 3...err = %d\n",err))
             break;}
        }
    }

 free_draw_mask_rec(&mask1);
 TRACE(printf("segment_pixels 4...err = %d  num_centers = %d\n",
      err,histo->num_centers))
 if (err == NO_ERROR)
    if (histo->num_centers > 0) 
       err = PartitionCenters(histo,&(sroi_win->slice_roi));
 TRACE(printf("segment_pixels 5...err = %d\n",err))

 if (err == NO_ERROR)
    {TRACE(printf(" histo->num_centers...%d\n",histo->num_centers))
     if (((roi = get_roi(&(sroi_win->slice_roi),
                  BACKGROUND_PIXEL_TYPE_STR)) == NULL) ||
          !is_roi_non_empty(roi))
        {
         TRACE(printf("BACKGROUND_PIXEL_TYPE_STR == NULL...\n"))
         if (is_roi_non_empty(&(histo->bkg.roi))) {
	     /* FIXME:  Removed an apparent bogus close paren just
		after the argument 0 below and added another close paren
		at the end of the if condition.  -PWH 19990412 */
             if ((err = JoinRoiToSliceRoiRec(&(sroi_win->slice_roi),
                      &(histo->bkg.roi),0,1.0,1.0,NULL,NULL,NULL) != NO_ERROR))
                return err;
            }
          else
           {TRACE(printf("before EnterPtsIntoSliceRoiRec...\n"))
            if ((err = EnterPtsIntoSliceRoiRec(
                &(sroi_win->slice_roi),NULL,BACKGROUND_PIXEL_TYPE_STR,
                 (unsigned int) DRAW_POLYGON,0,7, .0,.0, 
                 .0,(float) (histo->_2d.dimy - 1),
                 (float) HISTO_START,(float) (histo->_2d.dimy - 1), 
                 (float) HISTO_START,(float) HISTO_START,
                 (float) (histo->_2d.dimx - 1),(float) HISTO_START,  
                 (float) (histo->_2d.dimx - 1),.0,
                 .0,.0)) != NO_ERROR)
                return err;
            TRACE(printf("after EnterPtsIntoSliceRoiRec...\n"))
           }
         if ((roi = get_roi(&(sroi_win->slice_roi),
                  BACKGROUND_PIXEL_TYPE_STR)) != NULL)
            {
             roi->status |= HISTO_EDITED_ROI;}
        }
     TRACE(printf("\n\nsegment_pixels 6...err = %d\n",err))
     err = BuildHistoPixelTypeOrder(histo,&(sroi_win->slice_roi));
    }
 TRACE(printf("segment_pixels 8...err = %d\n",err))
 return err;
}
