/****************************************************************************
import_img.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: import_img.c,v 1.11 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";


/* FIXME:  Not needed?
#include <time.h>
#include <sys/timeb.h>
*/

#include <xview/notify.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "ask_file.h"
#include "debug.h"
#include "nria_procedures.h"



#define CLEAN_UP 0
#define SETUP 1
#define GET_ALL_PTR 2
#define TOTAL_BYTES_TO_SKIP "TOTAL BYTES TO SKIP" 
#define FILE_SIZE "CURRENT FILE SIZE"

#define FILE_SET 1
#define FILE_SET_BEFORE 2


struct import_file_rec
{
  char *fn;
  unsigned char set;
  float px,py,pz;
  int num_echoes;
  unsigned long depth,dx,dy,type,
    file_size,total_offset,slice_start,slice_offset;
};


#define CALLED_FROM_MAIN 1
struct import_img_rec
{
  unsigned short status;
  struct image_display_window *win;
  struct input_file_rec *file;
  Frame frame;
  Panel panel;
  Panel_item set,fn,px,py,pz,depth,dx,dy,type,which,psize,tot_slices,
    file_size,total_offset,slice_start,slice_offset,num_echoes;
  char *last_fn;
  unsigned long offset,total_files,num_files,cur;
  struct import_file_rec *files;
};


struct all_import_img_recs
{
  int num_imps;
  struct import_img_rec **imps;
};


/* Functions defined in this file */
void init_import_file_rec(struct import_file_rec *file);
void free_import_file_rec(struct import_file_rec *file);
void init_import_img_rec(struct import_img_rec *imp);
void free_import_img_rec(struct import_img_rec *imp);
void init_all_import_img_recs(struct all_import_img_recs *all_imps);
void import_img_rec_done_proc(Frame frame);
void import_set_file_items(struct import_file_rec *file,
    struct import_img_rec *imp);
void import_select_file_proc(Panel_item item, char *fn, caddr_t client_data,
    Panel_list_op op, Event *event);
int sort_import_file_rec_proc(const void *obj1, const void *obj2);
int import_fns_proc(struct import_img_rec *imp, struct dir_fn_rec *dir);
void import_open_proc(Panel_item item, Event *event);
void import_set_proc(Panel_item item, char *str, caddr_t client_data,
    Panel_list_op op, Event *event);
Panel_setting import_pixel_unit_proc(Panel_item item, Event *event);
void import_offset_proc(Panel_item item, int value, Event *event);
void import_pixel_size_proc(Panel_item item, int value, Event *event);
void import_depth_proc(Panel_item item, int value, Event *event);
void import_type_proc(Panel_item item, int value, Event *event);
void import_set_file_pixel_info(struct import_img_rec *imp, unsigned long lcv,
    int which);
void import_apply_proc(Panel_item item, Event *event);
int import_scan_min_max(struct hdr_rec *hdr, struct import_file_rec *file);
int import_setup_hdr(struct hdr_rec *hdr, struct import_file_rec *file);
int import_open_file(struct import_img_rec *imp);
void import_ready_proc(Panel_item item, Event *event);
void import_cancel_proc(Panel_item item, Event *event);
void import_set_img_defaults(struct import_img_rec *imp);
int import_setup_fns(struct import_img_rec *imp);
int import_set_flags(struct import_img_rec *imp);
int setup_import_img_rec(struct import_img_rec *imp,
    struct image_display_window *win);
struct import_img_rec *find_import_img_rec(
   struct all_import_img_recs *all_imps, struct image_display_window *win);
struct all_import_img_recs *enter_import_img_rec(void *win_or_main_frame,
    int status);



/*-----------------------------------------------------------------------
init_import_file_rec - ???
------------------------------------------------------------------------*/

void
init_import_file_rec(file)
     struct import_file_rec *file;
{
  file->fn = NULL;
  file->set = 0;
}



/*-----------------------------------------------------------------------
free_import_file_rec - ???
------------------------------------------------------------------------*/

void
free_import_file_rec(file)
     struct import_file_rec *file;
{
  c_free(file->fn);
  file->fn = NULL;
  file->set = 0;
}



/*-----------------------------------------------------------------------
init_import_img_rec - ???
------------------------------------------------------------------------*/

void
init_import_img_rec(imp)
     struct import_img_rec *imp;
{
  imp->status = 0;
  imp->win = NULL;
  imp->frame = 0;
  imp->last_fn = NULL;
  imp->total_files = imp->num_files = 0;
  imp->files = NULL;
}



/*-----------------------------------------------------------------------
free_import_img_rec - ???
------------------------------------------------------------------------*/

void
free_import_img_rec(imp)
     struct import_img_rec *imp;
{
  struct all_import_img_recs *all_imps;
  unsigned long lcv;

  TRACE(printf("free_import_img_rec...\n"));
  if (imp->frame != 0)
  {
    destroy_all_subframes(imp->frame);
    xv_destroy_safe(imp->frame);
  }
  for (lcv = 0; lcv < imp->num_files; lcv++)
    free_import_file_rec(&(imp->files[lcv]));
  c_free(imp->files);
  c_free(imp->last_fn);
  c_free(imp);
  TRACE(printf("free_import_img_rec...\n"));
  all_imps = enter_import_img_rec(NULL,GET_ALL_PTR);
  DropHandleAryPtr(&(all_imps->imps),&(all_imps->num_imps),imp);
  TRACE(printf("free_import_img_rec...end\n"));
}



/*-----------------------------------------------------------------------
init_all_import_img_recs - ???
------------------------------------------------------------------------*/

void
init_all_import_img_recs(all_imps)
     struct all_import_img_recs *all_imps;
{
  all_imps->num_imps = 0;
  all_imps->imps = NULL;
}


/*-----------------------------------------------------------------------
import_img_rec_done_proc - ???
------------------------------------------------------------------------*/

void
import_img_rec_done_proc(frame)
 Frame frame;
{
 struct import_img_rec *imp;

 imp = (struct import_img_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_import_img_rec(imp);
}


/*-----------------------------------------------------------------------
import_set_file_items - ???
------------------------------------------------------------------------*/

void
import_set_file_items(file,imp)
 struct import_file_rec *file;
 struct import_img_rec *imp;
{
 char str[80];

 PRT1(printf("import_set_file_items...file = %u\n",file);)
 if (file->set & FILE_SET_BEFORE)
    {
     sprintf(str,"%.4f",file->px);
     xv_set(imp->px,PANEL_VALUE,str,0);
     sprintf(str,"%.4f",file->py);
     xv_set(imp->py,PANEL_VALUE,str,0);
     sprintf(str,"%.4f",file->pz);
     xv_set(imp->pz,PANEL_VALUE,str,0);
     xv_set(imp->depth,PANEL_VALUE,file->depth,0);

     xv_set(imp->dx,PANEL_VALUE,file->dx,0);
     xv_set(imp->dy,PANEL_VALUE,file->dy,0);
     xv_set(imp->type,PANEL_VALUE,file->type,0);
     if ((file->type >= 2) || (file->depth != 8 &&
			       file->depth != 16 &&
			       file->depth != 32))
         xv_set(imp->psize,PANEL_VALUE,3,0);

     xv_set(imp->num_echoes,PANEL_VALUE,file->num_echoes,0);
     PRT(printf("fn = %s...file_size = %u\n",file->fn,file->file_size);)
     sprintf(str,"%s = %lu",FILE_SIZE,file->file_size);
     xv_set(imp->file_size,PANEL_LABEL_STRING,str,0);
     sprintf(str,"%s = %ld",TOTAL_BYTES_TO_SKIP,file->total_offset);
     xv_set(imp->total_offset,PANEL_LABEL_STRING,str,0);
     xv_set(imp->slice_start,PANEL_VALUE,file->slice_start,0);
     xv_set(imp->slice_offset,PANEL_VALUE,file->slice_offset,0);
     import_offset_proc(imp->dx,file->dx,NULL);
    }
   else import_set_img_defaults(imp);
}


/*-----------------------------------------------------------------------
import_select_file_proc - ???
------------------------------------------------------------------------*/

void
import_select_file_proc(item,fn,client_data,op,event)
 Panel_item item;
 char *fn;
 caddr_t client_data;
 Panel_list_op op;
 Event *event;
{
 struct import_img_rec *imp;
 unsigned long posi;

 imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
 posi = (unsigned long) client_data;
 PRT(printf("import_select_file_proc fn = %s op = %d posi = %d\n",fn,op,posi);)
 if (op)
    {PRT(printf("import_select_file_proc fn = %s posi = %d\n",
                imp->files[posi].fn,posi))
     import_set_file_items(&(imp->files[posi]),imp);}
}


/*-----------------------------------------------------------------------
sort_import_file_rec_proc - ???
------------------------------------------------------------------------*/

int
sort_import_file_rec_proc(obj1,obj2)
 const void *obj1,*obj2;
{
 return str_cmp(((struct import_file_rec *) obj1)->fn, 
     ((struct import_file_rec *) obj2)->fn);
}


/*-----------------------------------------------------------------------
import_fns_proc - ???
------------------------------------------------------------------------*/

int
import_fns_proc(imp,dir)
 struct import_img_rec *imp;
 struct dir_fn_rec *dir;
{
 char *fn;
 int err,lcv,cnt,indx,num,change,ok;
 struct import_file_rec file;
 unsigned long len;

 PRT(printf("import_fns_proc dir = %u\n",dir))
 init_import_file_rec(&file);
 err = NO_ERROR;
 change = 0;
 if (dir != NULL)
    {PRT(printf("dir->num_dirs = %d\n",dir->num_dirs))
     for (lcv = 0; lcv < dir->num_dirs; lcv++)
        {if (dir->dirs[lcv]->fns.num_strs == 0) continue;
         PRT(printf("%d) dir = %s  ",lcv,dir->dirs[lcv]->dir))
         PRT(printf("hdr_fn = %s\n",dir->dirs[lcv]->fn))
         PRT(printf("num_strs = %d\n",dir->dirs[lcv]->more_dir_fns.num_strs))
         len = get_strlen(dir->dirs[lcv]->dir);
         for (cnt = 0; cnt < dir->dirs[lcv]->fns.num_strs; cnt++)
            {
             if (len > 0)
                err = concat_strs(&fn,3,dir->dirs[lcv]->dir,"/",
                       dir->dirs[lcv]->fns.strs[cnt]);
               else 
                err = cp_str(&fn,dir->dirs[lcv]->fns.strs[cnt]);
             PRT(printf("...fn = %s\n",fn))
             if (err == NO_ERROR)
                {file.fn = fn;
                 if (imp->num_files > 0)
                    {if (obj_binary_search(imp->files,imp->num_files,
                          sizeof(*(imp->files)),
                          &file,sort_import_file_rec_proc,&indx) == NO_ERROR)
                        {c_free(fn);
                         PRT(printf("...found\n"))
                         continue;}
                    }
                   else indx = 0;
                 if (imp->num_files >= imp->total_files)
                    {num = imp->total_files;
                     if ((err = get_ptr(&(imp->files),&(imp->total_files),10,
                         (size_t) sizeof(*(imp->files)))) != NO_ERROR)
                        break;
                     for (; num < imp->total_files; num++)
                        init_import_file_rec(&(imp->files[num]));
                    }
                 file.file_size = get_file_size_by_fn(fn,&ok);
                 if (ok != NO_ERROR) file.file_size = 0;
                 for (num = imp->num_files; num > indx; num--)
                    imp->files[num] = imp->files[num - 1];
                 imp->files[indx] = file;
                 imp->num_files += 1;
                 change = 1;
                }

             if (err != NO_ERROR) 
                {c_free(fn);
                 break;}
            }
         if (err != NO_ERROR) 
            break;
        }
    }
 if (err != NO_ERROR) 
    notice_prompt(imp->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "Memory Error getting the imported file names",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
 if (change)
    {import_setup_fns(imp);
     import_offset_proc(imp->dx,0,NULL);
     import_set_flags(imp);}
 return err;
}



/*-----------------------------------------------------------------------
import_open_proc - ???
------------------------------------------------------------------------*/

void
import_open_proc(item,event)
     Panel_item item;
     Event *event;
{
  struct import_img_rec *imp;

  imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
  PRT(printf("import_open_proc...1\n"));
  ask_files( SETUP_ASK_FILE,	/* status */
	     imp->frame,	/* parent frame */
	     "GET IMPORTED IMAGE FILES", /* title */
	     NULL,		/* help_fn */
	     1,			/* free_upon_ready_and_cancel */
	     0,			/* is_center_frame */
	     import_fns_proc,	/* ready_proc */
	     imp,		/* ready_proc_arg */
	     NULL,		/* ask_cancel_proc */
	     NULL,		/* ask_cancel_proc_arg */
	     1,			/* is_chdir */
	     "*.img",		/* fn_filter */
	     1,			/* 1=LOAD, 0=SAVE */
	     1,			/* many? */
	     5,			/* num_fn_lines */
	     285,		/* fn_len */
	     NULL,		/* chosen_fn_proc */
	     NULL,		/* chosen_fn_proc_arg */
	     1,			/* num_items */
	     NULL,		/* dirfns[lcv].cur_dir */
	     NULL,		/* dirfns[lcv].open_dir_proc */
	     NULL,		/* dirfns[lcv].open_dir_proc_arg */
	     NULL,		/* dirfns[lcv].fn_msg */
	     NULL,		/* dirfns[lcv].cur_fn */
	     NULL,		/* dirfns[lcv].choose_fn_proc */
	     NULL );		/* dirfns[lcv].choose_fn_proc_arg */
}



/*-----------------------------------------------------------------------
import_set_proc - ???
------------------------------------------------------------------------*/

void
import_set_proc(item,str,client_data,op,event)
     Panel_item item;
     char *str;
     caddr_t client_data;
     Panel_list_op op;
     Event *event;
{
  struct import_img_rec *imp;
  unsigned long posi;

  imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
  posi = (unsigned long) client_data;
  PRT(printf("import_set_proc str = %s op = %d posi = %d\n",str,op,posi););
  if (!op)
    imp->files[posi].set &= ~FILE_SET;
  else
    import_set_file_pixel_info(imp,posi,0);
}



/*-----------------------------------------------------------------------
import_pixel_unit_proc - PANEL_TEXT notification procedure

Debugged, revised for XView v3, DRW 19980223
------------------------------------------------------------------------*/

Panel_setting
import_pixel_unit_proc( item, event )
     Panel_item item;
     Event *event;
{
  struct import_img_rec *imp;
  float num;
  char str[16];	/* This is larger than the PANEL_VALUE_STORED_LENGTHs
		 *  of the individual PANEL_TEXT items which use this
		 *  procedure as their callback.  See
		 *  setup_import_img_rec() routine for details.
		 */

  imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
  strcpy( str, (char *) xv_get( item, PANEL_VALUE ) );
  PRT(printf("import_pixel_unit_proc str = %s\n",str););
  sscanf(str,"%f",&num);
  if (num <= 0.0)
    xv_set(item,PANEL_VALUE,"1.0",0);
  return PANEL_NEXT;
}



/*-----------------------------------------------------------------------
import_offset_proc - ???
------------------------------------------------------------------------*/

void
import_offset_proc(item,value,event)
 Panel_item item;
 Event *event;
{
 struct import_img_rec *imp;
 char str[100];
 unsigned long depth,dimx,total_offset,offset,
               img_size,max_slices,max,lcv,max_size;

 imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
 total_offset = (unsigned long) xv_get(imp->slice_start,PANEL_VALUE);

 offset = (unsigned long) xv_get(imp->slice_offset,PANEL_VALUE);
 dimx = (unsigned long) xv_get(imp->dx,PANEL_VALUE);
 depth = (unsigned long) xv_get(imp->depth,PANEL_VALUE);
 img_size = GetRowBytes(dimx,depth) * 
            (unsigned long) xv_get(imp->dy,PANEL_VALUE);
 PRT1(printf("import_offset_proc...\n"))
 if (next_selected_item(imp->fn,0,&lcv) == NO_ERROR)
    {max_slices = (unsigned long) xv_get(imp->slice_offset,PANEL_MAX_VALUE);
     max = imp->files[lcv].file_size / img_size;
     sprintf(str,"TOTAL SLICES = %ld",max);
     xv_set(imp->tot_slices,PANEL_LABEL_STRING,str,0);
     PRT1(printf("import_offset_proc...str = %s\n",str))
     if (max > 2) max -= 1;
     if (max_slices != max)
        {xv_set(imp->slice_offset,PANEL_MAX_VALUE,max,0);
         if (offset > max) 
            {offset = max;
             xv_set(imp->slice_offset,PANEL_VALUE,offset,0);}
        }
     max_size = (unsigned long) xv_get(imp->slice_start,PANEL_MAX_VALUE);
     if (max_size >= imp->files[lcv].file_size)
        if (max_size > 0)
           {max_size = max_size - 1;
            xv_set(imp->slice_start,PANEL_MAX_VALUE,max_size,0);
            if (total_offset > max_size)
               {xv_set(imp->slice_start,PANEL_VALUE,max_size,0);
                total_offset = max_size;}
           }
    }
 if (offset > 0)
    {total_offset += img_size * offset;}
 sprintf(str,"%s = %ld",TOTAL_BYTES_TO_SKIP,total_offset);
 imp->offset = total_offset;
 xv_set(imp->total_offset,PANEL_LABEL_STRING,str,0);
}



/*-----------------------------------------------------------------------
import_pixel_size_proc - PANEL_CHOICE notification procedure

Make sure DEPTH field matches user's selection of PIXEL DATA SIZE

Debugged, revised for XView v3, DRW 19980223
------------------------------------------------------------------------*/

void
import_pixel_size_proc( item, value, event )
     Panel_item item;
     int value;
     Event *event;
{
  struct import_img_rec *imp;
  int val;

  imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);

  /*  3 means use DEPTH value, so don't change user's setting  */
  if (value == 3)
    return;

  /*
   *  If type is floating point (val is 2 or 3) and the user tries
   *   to change the PIXEL DATA SIZE, reset type to UNSIGNED.
   */
  val = (int) xv_get( imp->type, PANEL_VALUE );
  if (val >= 2) {
    xv_set( imp->type,
	    PANEL_VALUE, 0,
	    0 );
  }

  if (value == 0)
    val = 8;
  else if (value == 1)
    val = 16;
  else if (value == 2)
    val = 32;
  xv_set(imp->depth,PANEL_VALUE,val,0);
  import_offset_proc(imp->dx,xv_get(imp->dx,PANEL_VALUE),NULL);
}



/*-----------------------------------------------------------------------
import_depth_proc - PANEL_SLIDER notification procedure

Debugged, revised for XView v3, DRW 19980223
------------------------------------------------------------------------*/

void
import_depth_proc( item, value, event )
     Panel_item item;
     int value;
     Event *event;
{
  struct import_img_rec *imp;
  unsigned long type;
  long val;

  imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
  PRT(printf("import_depth_proc value = %d\n",value););
  if (value > 48)
  {
    xv_set( item, PANEL_VALUE, 64, 0 );
    xv_set( imp->type, PANEL_VALUE, 3, 0 );
  }
  else if (value > 32)
  {
    xv_set( item, PANEL_VALUE, 32, 0 );
    xv_set( imp->type, PANEL_VALUE, 2, 0 );
  }
  else if (value <= 32)
  {
    type = (unsigned long) xv_get(imp->type,PANEL_VALUE);
    if (type == 3) 
      if (value == 32) 
      {
	xv_set(imp->type,PANEL_VALUE,2,0);
	type = 2;
      }
    if (((type >= 2) && (value < 32)) || (value == 1))
      xv_set(imp->type,PANEL_VALUE,0,0);
  }
  import_offset_proc(item,value,event);
  val = (int) xv_get(item,PANEL_VALUE);
  if (val != 8 && val != 16 && val != 32)
    xv_set(imp->psize,PANEL_VALUE,3,0);
}



/*-----------------------------------------------------------------------
import_type_proc - ???
------------------------------------------------------------------------*/

void
import_type_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct import_img_rec *imp;

 imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT(printf("import_type_proc value = %d\n",value);)
 if (value == 3)
    {xv_set(imp->depth,PANEL_VALUE,64,0);
     xv_set(imp->psize,PANEL_VALUE,3,0);}
   else if (value == 2)
    {xv_set(imp->depth,PANEL_VALUE,32,0);
     xv_set(imp->psize,PANEL_VALUE,3,0);}
 import_offset_proc(item,value,event);
}



/*-----------------------------------------------------------------------
import_set_file_pixel_info - ???
------------------------------------------------------------------------*/

void
import_set_file_pixel_info(imp,lcv,which)
     struct import_img_rec *imp;
     unsigned long lcv;
     int which;
{
  float px,py,pz;
  unsigned long depth,dx,dy,type,slice_start,slice_offset;
  int num_echoes;
  char *str;

  PRT(printf("import_set_file_pixel_info lcv = %d which = %d\n",lcv,which););
  str = (char *) xv_get(imp->px,PANEL_VALUE);
  sscanf(str,"%f",&px);
  str = (char *) xv_get(imp->py,PANEL_VALUE);
  sscanf(str,"%f",&py);
  str = (char *) xv_get(imp->pz,PANEL_VALUE);
  sscanf(str,"%f",&pz);

  PRT(printf("px = %f py = %f pz = %f\n",px,py,pz););
  depth = (unsigned long) xv_get(imp->depth,PANEL_VALUE);
  dx = (unsigned long) xv_get(imp->dx,PANEL_VALUE);
  dy = (unsigned long) xv_get(imp->dy,PANEL_VALUE);
  type = (unsigned long) xv_get(imp->type,PANEL_VALUE);
  num_echoes = (int) xv_get(imp->num_echoes,PANEL_VALUE);

  slice_start = (unsigned long) xv_get(imp->slice_start,PANEL_VALUE);
  slice_offset = (unsigned long) xv_get(imp->slice_offset,PANEL_VALUE);
  PRT(printf("depth = %d dx = %d dy = %d type = %d\n",depth,dx,dy,type););

  for (; lcv < imp->num_files; lcv++) {
    if ((which != 1) || !(imp->files[lcv].set & FILE_SET))
    {
      PRT(printf("== lcv = %d\n",lcv););
      imp->files[lcv].px = px;
      imp->files[lcv].py = py;
      imp->files[lcv].pz = pz;
      imp->files[lcv].depth = depth;
      imp->files[lcv].dx = dx;
      imp->files[lcv].dy = dy;
      imp->files[lcv].type = type;
      imp->files[lcv].num_echoes = num_echoes;
      imp->files[lcv].total_offset = imp->offset;
      imp->files[lcv].slice_start = slice_start;
      imp->files[lcv].slice_offset = slice_offset;
      imp->files[lcv].set = 1;
      xv_set(imp->set,PANEL_LIST_SELECT,lcv,1,0);
      imp->files[lcv].set |= FILE_SET | FILE_SET_BEFORE;
      if (which == 0)
	break;
    }
  }
}



/*-----------------------------------------------------------------------
import_apply_proc - ???
------------------------------------------------------------------------*/

void
import_apply_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct import_img_rec *imp;
 unsigned long lcv;
 int which;

 imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
 which = xv_get(imp->which,PANEL_VALUE);
 PRT(printf("import_apply_proc which = %d\n",which);)
 if (which == 0) next_selected_item(imp->fn,0,&lcv);
   else lcv = 0;
 import_set_file_pixel_info(imp,lcv,which);
}


/*-----------------------------------------------------------------------
import_scan_min_max - ???
------------------------------------------------------------------------*/

int
import_scan_min_max(hdr,file)
 struct hdr_rec *hdr;
 struct import_file_rec *file;
{
 unsigned long type,lcv;
 int err;
 FILE *fid;
 struct buf_2d_rec _2d;

 type = InputDataTypeToBufType(&(hdr->data_info));
 init_buf_2d_rec(&_2d);
 if ((err = SetupBuf2dRec(&_2d,hdr->data_info.dimx,hdr->data_info.dimy,
         hdr->data_info.pixel_depth_bits,type)) != NO_ERROR)
    return err;
 if ((fid = fopen(file->fn,"r")) == NULL)
    {free_buf_2d_rec(&_2d);
     return FILE_ACCESS_ERROR;}

 for (lcv = 0; lcv < hdr->data_info.max_scans; lcv++)
    {if (fseek(fid,(size_t) hdr->scans[lcv].img_addr,0) != 0)
        {err = ILLEGAL_FILE_ADDRESS;
         break;}
     if (fread(_2d.buf[0],(size_t) _2d.total_bytes,(size_t) 1,fid) != 1)
        {err = READ_ERROR;
         break;}
     _2d.status4 &= ~BUF_MIN_MAX_COMPUTED;
     MinMax2d(&_2d,0,(double) .0,(double) .0);
     PRT(printf("%d) img_addr = %u minval = %f max = %f min_pixel = %f max = %f\n",
          lcv,hdr->scans[lcv].img_addr,_2d.minval,_2d.maxval,
          hdr->data_info.min_pixel,hdr->data_info.max_pixel))

     if (lcv == 0)
        {hdr->data_info.min_pixel = _2d.minval;
         hdr->data_info.max_pixel = _2d.maxval;}
       else
        {if (hdr->data_info.min_pixel > _2d.minval)
            hdr->data_info.min_pixel = _2d.minval;
         if (hdr->data_info.max_pixel < _2d.maxval)
            hdr->data_info.max_pixel = _2d.maxval;}
    }
 fclose(fid);
 free_buf_2d_rec(&_2d);
 return err;
}



/*-----------------------------------------------------------------------
import_setup_hdr - ???
------------------------------------------------------------------------*/

int
import_setup_hdr(hdr,file)
     struct hdr_rec *hdr;
     struct import_file_rec *file;
{
  struct data_info_rec *info;
  unsigned long max_scans,img_size;

  img_size = GetRowBytes(file->dx,file->depth) * file->dy;
  max_scans = (file->file_size - file->total_offset) / img_size;
  info = &(hdr->data_info);
  info->xpixel_size = file->px;
  info->ypixel_size = file->py;
  info->pixel_thickness = file->pz;
  info->pixel_depth_bits = file->depth;
  info->dimx = file->dx;
  info->dimy = file->dy;
  if (file->type == 1)
    info->var_use_status |= INPUT_DATA_SIGNED;
  else if (file->type == 2)
    info->var_use_status |= INPUT_DATA_FLOAT;
  else if (file->type == 3)
    info->var_use_status |= INPUT_DATA_DOUBLE;
  info->max_scans = max_scans;
  info->num_echoes = file->num_echoes + 1;
  info->min_pixel = .0;
  info->max_pixel = 250.0;
  return BuildRiaHdr(hdr,file->total_offset);
}



/*-----------------------------------------------------------------------
import_open_file - ???

READY button callback
------------------------------------------------------------------------*/

int
import_open_file(imp)
     struct import_img_rec *imp;
{
  struct hdr_rec hdr;
  struct import_file_rec *file;
  unsigned long max_scans,img_size,lcv;
  FILE *fid;
  char str[200],*str1,*dir,*fn,*suffix,*hdr_fn;
  int err,ans,is_proceed;
  struct prog_rec *prog;

  PRT(printf("import_open_file...1\n"));

 file = &(imp->files[imp->cur]);
 if (file->total_offset >= file->file_size)
    {sprintf(str,"ERROR : total_offset = %ld >= file_size = %ld",
         file->total_offset,file->file_size);
     PRT(printf("import_open_file...1..1\n"))
     notice_prompt(imp->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,str,file->fn,"WILL NOT BE READ !!!",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
     return NO_ERROR;}
 PRT(printf("import_open_file...2\n"))
 img_size = GetRowBytes(file->dx,file->depth) * file->dy;
 max_scans = (file->file_size - file->total_offset) / img_size;
 PRT(printf("import_open_file...3\n"))
 if (max_scans <= 0)
    {sprintf(str,"ERROR : total slices = %ld",max_scans);
     notice_prompt(imp->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,str,file->fn,"WILL NOT BE READ !!!",
              0,
            NOTICE_BUTTON_YES," OK !",
            0);
     return NO_ERROR;}
 PRT(printf("import_open_file...4\n"))

 init_hdr_rec(&hdr);
 is_proceed = 0;
 if ((err = import_setup_hdr(&hdr,file)) != NO_ERROR)
    str1 = "MEMORY ERROR SETTING UP THE HEADER";
   else if ((err = import_scan_min_max(&hdr,file)) != NO_ERROR)
    str1 = "ERROR SCANNING MIN & MAX PIXEL VALUES";
 if (err != NO_ERROR)
    {ans = notice_prompt(imp->frame,(Event *) NULL,
         NOTICE_MESSAGE_STRINGS,str1,
              file->fn,"WILL NOT BE READ !!!",
              0,
            NOTICE_BUTTON_YES," PROCEED WITH NEXT !",
            NOTICE_BUTTON_NO," ABORT THE REST !",
            0);
     if (ans == NOTICE_YES)
        {is_proceed = 1;
         err = NO_ERROR;}
    }
   else
    {
     PRT(printf("import_open_file...5\n"))
     if ((err = break_fn(file->fn,&dir,&fn,&suffix)) == NO_ERROR)
        {if ((err = open_output_file(dir,fn,".nhdr",
             &hdr_fn,&fid,imp->frame,0)) == NO_ERROR)
            {err = wr_hdr(fid,&hdr);
             fclose(fid);
             if (err != NO_ERROR)
                {
                 notice_prompt(imp->frame,(Event *) NULL,
                   NOTICE_MESSAGE_STRINGS,"ERROR WRITING THE *.nhdr FILE",
                     file->fn,"WILL NOT BE SAVED !!!",
                     0,
                   NOTICE_BUTTON_YES," OK !",
                   0);
                 delete_file(hdr_fn);}
               else
                {PRT(printf("open_a_file...\n"))
                 if (imp->status & CALLED_FROM_MAIN) 
                    {PRT(printf("CALLED_FROM_MAIN...\n"))
                     prog = (struct prog_rec *) 
                          xv_get((Frame) imp->win,WIN_CLIENT_DATA);}
                   else 
                    prog = imp->win->owner->root->prog;
                 open_a_file(prog,dir,fn,
                      ".nhdr",suffix,dir,fn,
                      ".ndat",".bin");
                 PRT(printf("open_a_file...\n"))
                 is_proceed = 1;
                }
             err = NO_ERROR;
            }
         c_free(hdr_fn);}

     c_free(dir);
     c_free(fn);
     c_free(suffix);
    }
 free_hdr_rec(&hdr);
 if (is_proceed)
    {for (lcv = imp->cur + 1; lcv < imp->num_files; lcv++)
        if (imp->files[lcv].set & FILE_SET)
           {imp->cur = lcv;
            sprintf(str,"1) Creating the *.nhdr for %s\n",imp->files[lcv].fn);
            ShowMsgStrs(imp->frame,1,NO_ERROR,0,4,
               str,"2) Computing the min & max pixel value",
               "3) loading images",PLEASE_WAIT,import_open_file,1,imp);
            break;}
    }
 PRT(printf("import_open_file...err = %d\n",err))
 return err;
}


/*-----------------------------------------------------------------------
import_ready_proc - ???
------------------------------------------------------------------------*/

void
import_ready_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct import_img_rec *imp;
 char str[200];
 unsigned long lcv;

 imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
 for (lcv = 0; lcv < imp->num_files; lcv++)
    if (imp->files[lcv].set & FILE_SET)
       {imp->cur = lcv;
        sprintf(str,"1) Creating the *.nhdr for %s\n",imp->files[lcv].fn);
        ShowMsgStrs(imp->frame,1,NO_ERROR,0,4,
               str,"2) Computing the min & max pixel value",
               "3) loading images",PLEASE_WAIT,import_open_file,1,imp);
	return;
       }
 free_import_img_rec(imp);
}


/*-----------------------------------------------------------------------
import_cancel_proc - ???
------------------------------------------------------------------------*/

void
import_cancel_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct import_img_rec *imp;

 imp = (struct import_img_rec *) xv_get(item,PANEL_CLIENT_DATA);
 free_import_img_rec(imp);
}


/*-----------------------------------------------------------------------
import_set_img_defaults - ???
------------------------------------------------------------------------*/

void
import_set_img_defaults(imp)
 struct import_img_rec *imp;
{
 unsigned long lcv,size;
 char str[100];

 xv_set(imp->depth,PANEL_VALUE,16,0);
 xv_set(imp->px,PANEL_VALUE,"1.0",0);
 xv_set(imp->py,PANEL_VALUE,"1.0",0);
 xv_set(imp->pz,PANEL_VALUE,"1.0",0);
 xv_set(imp->dx,PANEL_VALUE,256,0);
 xv_set(imp->dy,PANEL_VALUE,256,0);
 if (next_selected_item(imp->fn,0,&lcv) == NO_ERROR)
   size = imp->files[lcv].file_size;
 else
   size = 0;

 sprintf(str,"%s = %ld",FILE_SIZE,size);
 xv_set(imp->file_size,PANEL_LABEL_STRING,str,0);
 sprintf(str,"%s = 0",TOTAL_BYTES_TO_SKIP);
 imp->offset = 0;
 xv_set(imp->total_offset,PANEL_LABEL_STRING,str,0);
 xv_set(imp->slice_start,PANEL_VALUE,0,0);
 xv_set(imp->slice_offset,PANEL_VALUE,0,0);
}


/*-----------------------------------------------------------------------
import_setup_fns - ???
------------------------------------------------------------------------*/

int
import_setup_fns(imp)
 struct import_img_rec *imp;
{
 unsigned long lcv,indx;
 char *fn;
 int ok;

 fn = NULL;
 if ((ok = next_selected_item(imp->fn,0,&indx)) == NO_ERROR)
    if (cp_str(&fn,imp->files[indx].fn) != NO_ERROR)
       ok = ~NO_ERROR;

 delete_all_list_items(imp->fn);
 if (imp->num_files > 0) 
    for (lcv = 0; lcv < imp->num_files; lcv++)
       xv_set(imp->fn,PANEL_LIST_STRING,lcv,imp->files[lcv].fn,
               PANEL_LIST_CLIENT_DATA,lcv,lcv,0);

 if ((ok == NO_ERROR) && (fn != NULL))
    select_list_str_item(fn,imp->fn,1);
 if ((ok = next_selected_item(imp->fn,0,&indx)) == NO_ERROR)
    {PRT(printf("import_setup_fns indx = %d\n",indx))
     import_set_file_items(&(imp->files[indx]),imp);}

 c_free(fn);
 return NO_ERROR;
}


/*-----------------------------------------------------------------------
import_set_flags - ???
------------------------------------------------------------------------*/

int
import_set_flags(imp)
 struct import_img_rec *imp;
{
 unsigned long lcv;

 delete_all_list_items(imp->set);
 if (imp->num_files > 0) 
    for (lcv = 0; lcv < imp->num_files; lcv++)
       xv_set(imp->set,PANEL_LIST_STRING,lcv,"YES",
               PANEL_LIST_CLIENT_DATA,lcv,lcv,PANEL_LIST_SELECT,lcv,0,0);
 return NO_ERROR;
}



/*-----------------------------------------------------------------------
setup_import_img_rec - ???

Import raw image
------------------------------------------------------------------------*/

int
setup_import_img_rec(imp,win)
     struct import_img_rec *imp;
     struct image_display_window *win;
{
  char *str;
  Panel_item item;
  int row,offset,temp;
  Rect *rr,rr1;
  Frame *main_frame_ptr,frame;

  
  PRT(printf("setup_import_img_rec...1\n"));
  main_frame_ptr = main_display(2, NULL, 0, 0);
  PRT(printf("main_frame_ptr = %u win = %u\n",*main_frame_ptr,win));
  if ((Frame) win == *main_frame_ptr)
  {
    imp->status |= CALLED_FROM_MAIN;
    frame = (Frame) win;
  }
  else
    frame = win->dwin->frame;

  imp->frame = xv_create(frame, FRAME,
			 FRAME_INHERIT_COLORS,TRUE,
			 0);
  PRT(printf("setup_import_img_rec...2\n"));
  if (imp->frame == 0) return CANCEL;
  imp->win = win;
  if (imp->status & CALLED_FROM_MAIN)
  {
    PRT(printf("CALLED_FROM_MAIN...\n"));
    concat_strs(&str,2,"MAIN : ",IMPORT_IMG);
  }
  else
  {
    str = win_rec_win_id_title(win->owner,IMPORT_IMG);
  }
  xv_set(imp->frame,
	 FRAME_SHOW_LABEL, TRUE,
	 XV_LABEL,str,
	 FRAME_NO_CONFIRM,TRUE,
	 FRAME_DONE_PROC,import_img_rec_done_proc,
	 WIN_SHOW,FALSE,
	 WIN_CLIENT_DATA,imp,
	 0);
  c_free(str);

  imp->panel = xv_create(imp->frame,PANEL,
                         FRAME_INHERIT_COLORS,TRUE,
                         0);
  if (imp->panel == 0)
  {
    free_import_img_rec(imp);
    return CANCEL;
  }

  PRT(printf("setup_import_img_rec...2\n"));
  row = 5;
  offset = xv_rows(imp->panel,1) - xv_rows(imp->panel,0) - 10;
  PRT(printf("setup_import_img_rec...5\n"));
  xv_create(imp->panel,PANEL_MESSAGE,
	    PANEL_LABEL_STRING,"SET",        
	    XV_X,5,
	    XV_Y,row + 3,
	    0);

  temp = row;
  row += offset - 3;
  imp->fn = xv_create(imp->panel, PANEL_LIST,
		      PANEL_LIST_DISPLAY_ROWS, 4,
		      PANEL_LAYOUT, PANEL_VERTICAL,
		      PANEL_NOTIFY_PROC, import_select_file_proc,
		      PANEL_CLIENT_DATA, imp,
		      0);

  PRT(printf("setup_import_img_rec...3...1\n"));
  imp->set = xv_create(imp->panel,PANEL_LIST,
		       PANEL_CHOOSE_ONE,FALSE,
		       PANEL_LIST_DISPLAY_ROWS,4,
		       PANEL_LAYOUT,PANEL_VERTICAL,
		       PANEL_NOTIFY_PROC,import_set_proc,
		       PANEL_CLIENT_DATA,imp,
		       0);
  import_set_flags(imp);
  xv_set( imp->set,
	  XV_X, 5,
	  XV_Y, row,
	  0 );
  rr = (Rect *) xv_get( imp->set, XV_RECT );
  xv_set( imp->fn,
	  XV_X, rr->r_left + rr->r_width + 25, /* Add space for scrollbar */
	  XV_Y, row,
	  0 );

 xv_create(imp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"OPEN",        
        PANEL_NOTIFY_PROC,import_open_proc,
        XV_X,rr->r_left + rr->r_width + 10,XV_Y,temp,
        PANEL_CLIENT_DATA,imp,0);
 rr = (Rect *) xv_get(imp->fn,XV_RECT);
 row = rr->r_top + rr->r_height + 2;
 item = xv_create(imp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"APPLY TO",        
        PANEL_NOTIFY_PROC,import_apply_proc,
        XV_X,5,XV_Y,row,
        PANEL_CLIENT_DATA,imp,0);
 rr = (Rect *) xv_get(item,XV_RECT);
 imp->which = xv_create(imp->panel,PANEL_CHOICE_STACK,
                  PANEL_LAYOUT,PANEL_VERTICAL,
                  PANEL_CHOICE_STRINGS,"CUR FILE",
                           "CUR + OTHER UNSET FILES","ALL FILES",NULL,
                  XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,PANEL_VALUE,0,
                  0);

 row += offset;
 xv_create(imp->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"THE UNSET FILES WILL NOT BE READ...",        
        XV_X,5,XV_Y,row,0);
 row += offset - 7;
 xv_create(imp->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"AND *.nhdr ARE CREATED FOR THE SET FILES...",
        XV_X,15,XV_Y,row,0);
 row += offset - 4;
 xv_create(imp->panel,PANEL_MESSAGE,
    PANEL_LABEL_STRING,"After changing the parameters, press the APPLY TO button",       
    XV_X,5,XV_Y,row,0);
 row += offset;
 item = xv_create(imp->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"PIXEL INFO...",        
        XV_X,5,XV_Y,row,0);

 row += offset;
 imp->num_echoes = xv_create(imp->panel,PANEL_CHOICE_STACK,
			     PANEL_LABEL_STRING,"# of IMAGE ECHOES =",        
			     PANEL_CHOICE_STRINGS,"1","2",NULL,
			     XV_X,15,XV_Y,row,
			     PANEL_VALUE,0,
			     0);

 row += offset;
 imp->type = xv_create(imp->panel,PANEL_CHOICE_STACK,
	          PANEL_LABEL_STRING,"TYPE",
                  PANEL_CHOICE_STRINGS,
		       "UNSIGNED",
		       "SIGNED",
		       "FLOAT:32-BIT",
		       "DOUBLE:64-BIT",NULL,
                  XV_X,15,XV_Y,row,PANEL_VALUE,0,
                  PANEL_NOTIFY_PROC,import_type_proc,
                  PANEL_CLIENT_DATA,imp,
                  0);
 rr = (Rect *) xv_get(imp->type,XV_RECT);
 item = xv_create(imp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"READY",
        PANEL_NOTIFY_PROC,import_ready_proc,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
        PANEL_CLIENT_DATA,imp,0);
 rr = (Rect *) xv_get(item,XV_RECT);
 xv_create(imp->panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,"CANCEL",
        PANEL_NOTIFY_PROC,import_cancel_proc,
        XV_X,rr->r_left + rr->r_width + 2,XV_Y,row,
        PANEL_CLIENT_DATA,imp,0);

 row += offset;
 imp->psize = xv_create(imp->panel,PANEL_CHOICE_STACK,
	          PANEL_LABEL_STRING,"PIXEL =",
                  PANEL_CHOICE_STRINGS,
			"1 byte,  8  bits", /* 0 */
			"2 bytes, 16 bits", /* 1 */
			"4 bytes, 32 bits", /* 2 */
			"USE DEPTH FIELD...", /* 3 */
			NULL,
                  XV_X,15,XV_Y,row,PANEL_VALUE,1,
                  PANEL_NOTIFY_PROC,import_pixel_size_proc,
                  PANEL_CLIENT_DATA,imp,
                  0);

 row += offset;
 imp->depth = xv_create(imp->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"DEPTH (in bits)",
	PANEL_MIN_VALUE,1,
	PANEL_MAX_VALUE,64,
	PANEL_SLIDER_WIDTH,60,
	PANEL_NOTIFY_PROC,import_depth_proc,
        XV_X,55,XV_Y,row,
        PANEL_CLIENT_DATA,imp,
	0);
 row += offset;
 imp->px = xv_create(imp->panel,PANEL_TEXT,
	PANEL_LABEL_STRING,"SIZE (in mm) X",        
        PANEL_VALUE_DISPLAY_LENGTH,7,
        PANEL_VALUE_STORED_LENGTH,15,
        PANEL_NOTIFY_PROC,import_pixel_unit_proc,
        XV_X,15,XV_Y,row,
        PANEL_CLIENT_DATA,imp,0);
 imp->py = xv_create(imp->panel,PANEL_TEXT,
	PANEL_LABEL_STRING,"Y",  
        PANEL_VALUE_DISPLAY_LENGTH,7,
        PANEL_VALUE_STORED_LENGTH,15,
        PANEL_NOTIFY_PROC,import_pixel_unit_proc,
        XV_Y,row,
        PANEL_CLIENT_DATA,imp,0);
 PRT(printf("setup_import_img_rec...4\n"))
 imp->pz = xv_create(imp->panel,PANEL_TEXT,
	PANEL_LABEL_STRING,"Z",        
        PANEL_VALUE_DISPLAY_LENGTH,7,
        PANEL_VALUE_STORED_LENGTH,15,
        PANEL_NOTIFY_PROC,import_pixel_unit_proc,
        XV_Y,row,
        PANEL_CLIENT_DATA,imp,0);
 row += offset;
 imp->dx = xv_create(imp->panel,PANEL_SLIDER,
        XV_X,15,XV_Y,row,
	PANEL_LABEL_STRING,"DIM X",
	PANEL_MIN_VALUE,1,
	PANEL_MAX_VALUE,512,
	PANEL_SLIDER_WIDTH,100,
        PANEL_NOTIFY_PROC,import_offset_proc,
        PANEL_CLIENT_DATA,imp,0);
 row += offset;
 imp->dy = xv_create(imp->panel,PANEL_SLIDER,
        XV_X,15,XV_Y,row,
	PANEL_LABEL_STRING,"DIM Y",
	PANEL_MIN_VALUE,1,
	PANEL_MAX_VALUE,512,
	PANEL_SLIDER_WIDTH,100,
        PANEL_NOTIFY_PROC,import_offset_proc,
        PANEL_CLIENT_DATA,imp,0);

 row += offset - 4;
 imp->tot_slices = xv_create(imp->panel,PANEL_MESSAGE,
	PANEL_LABEL_STRING,"TOTAL SLICES =",
        XV_X,15,XV_Y,row,0);

 row += offset + 4;
 imp->file_size = xv_create(imp->panel,PANEL_MESSAGE,
        XV_X,5,XV_Y,row,0);
 rr = (Rect *) xv_get(imp->file_size,XV_RECT);
 rr1 = *rr;
 rr1.r_width = 300;
 xv_set(imp->file_size,XV_RECT,&rr1,0);

 row += offset - 4;
 imp->total_offset = xv_create(imp->panel,PANEL_MESSAGE,
        XV_X,5,XV_Y,row,0);
 rr = (Rect *) xv_get(imp->total_offset,XV_RECT);
 rr1 = *rr;
 rr1.r_width = 300;
 xv_set(imp->total_offset,XV_RECT,&rr1,0);
 row += offset - 4;
 imp->slice_start = xv_create(imp->panel,PANEL_NUMERIC_TEXT,
	PANEL_LABEL_STRING,"SKIP SLICE FIRST FEW BYTES =",
           PANEL_VALUE_DISPLAY_LENGTH,10,
           PANEL_MIN_VALUE,0,
           PANEL_MAX_VALUE,10000000,
           XV_X,15,XV_Y,row,
           PANEL_NOTIFY_PROC,import_offset_proc,
           PANEL_CLIENT_DATA,imp,0);
 row += offset;

 imp->slice_offset = xv_create(imp->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"SKIP FIRST FEW SLICES",
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,1000,
	PANEL_SLIDER_WIDTH,40,
        XV_X,15,XV_Y,row,
        PANEL_NOTIFY_PROC,import_offset_proc,
        PANEL_CLIENT_DATA,imp,0);
 import_set_img_defaults(imp);
 window_fit(imp->panel);
 rr = (Rect *) xv_get(imp->panel,XV_RECT);
 xv_set(imp->panel,XV_WIDTH,rr->r_width - 10,XV_HEIGHT,rr->r_height - 10,0);
 PRT(printf("setup_import_img_rec...6\n"))

 window_fit(imp->frame);
 if (imp->status & CALLED_FROM_MAIN) center_frame(imp->frame,1);
   else make_2_wins_adjacent(imp->frame,win->dwin->frame,0);
 win_set_mouse_at_center(imp->frame);
 xv_set(imp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 return NO_ERROR;

}


/*-----------------------------------------------------------------------
find_import_img_rec - ???
------------------------------------------------------------------------*/

struct import_img_rec *
find_import_img_rec(all_imps,win)
 struct all_import_img_recs *all_imps;
 struct image_display_window *win;
{
 int lcv;

 for (lcv = 0; lcv < all_imps->num_imps; lcv++)
    if (all_imps->imps[lcv]->win == win)
       return all_imps->imps[lcv];
 return NULL;
}


/*---------------------------------------------------------------------
enter_import_img_rec - ???

NOTE: The pre-19990223 version of this function could return either
NULL, a pointer to a structure, or an integer error code.  Since the
last two of these cannot be reliably distinguished and since no one
ever seems to check for an error code, I replaced the error code by
NULL.  Of course, it is now impossible to distinguish between NO_ERROR
and NULL (=> error), but I don't think anyone ever looks at the error
code returned.  -PWH

FIXME! This wretched kluge is three separate functions pasted together
with a selection code (misnamed status) and which aren't even consisent
in the type of value return.  Rewrite completely.  -PWH 19990618
-----------------------------------------------------------------------*/

struct all_import_img_recs *
enter_import_img_rec(
    void *win_or_main_frame,
    int status)
{
 static struct all_import_img_recs all_imps;
 struct import_img_rec *imp;
 static char first_time = 1;
 int err;
 Frame *main_frame_ptr,frame;

 err = NO_ERROR;
 if (status == GET_ALL_PTR)
    {if (first_time) return NULL;
       else return &all_imps;}
  else if (status == CLEAN_UP)
   {if (!first_time)
       if ((imp = find_import_img_rec(&all_imps,win_or_main_frame)) != NULL)
          free_import_img_rec(imp);
    return NO_ERROR;}
 
 if (first_time)
    {
     init_all_import_img_recs(&all_imps);
     first_time = 0;}
   else
    {if ((imp = find_import_img_rec(&all_imps,win_or_main_frame)) != NULL)
        {
         xv_set(imp->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         win_set_mouse_at_center(imp->frame);
         wmgr_top(imp->frame);
         return NO_ERROR;}
    }
 get_handle_memory(&(all_imps.imps),&(all_imps.num_imps),1,
     (size_t) sizeof(struct import_img_rec),&err);
 if (err != NO_ERROR) return NULL;        /* Was: return err */
 imp = all_imps.imps[all_imps.num_imps - 1];
 init_import_img_rec(imp);
 if ((err = setup_import_img_rec(imp,win_or_main_frame)) != NO_ERROR)
    {main_frame_ptr = main_display(2, NULL, 0, 0);
     if ((Frame) win_or_main_frame == *main_frame_ptr)
        frame = (Frame) win_or_main_frame;
       else 
        frame = ((struct image_display_window *) win_or_main_frame)->dwin->frame;
     ShowMsgStrs(frame,0,err,0,2,
               "An error occured when setting up",
               "   the SAVE DATA window !!!",NULL,0);}
 return NULL;        /* Was: return err */

}

