/****************************************************************************
mag_win_pixels.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: mag_win_pixels.c,v 1.10 1999/07/07 17:41:17 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define TRACE(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRINT(x)



#define CLOSE_UPON_OWNER_REQUEST 1
struct magnify_pixel_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    Frame frame;
    Panel panel;
    Canvas canvas;
    Panel_item res,win_option,def_mag_fac,res_edit;

    unsigned int total_rects;
    unsigned int num_row_rects;
    unsigned int num_col_rects;
    struct int_rect_rec **rr;
    unsigned char **mask;
    unsigned int **color;
   };


/* Functions defined in this file */
void init_magnify_pixel_rec(struct magnify_pixel_rec *mag_pixel);
void free_magnify_pixel_memory(struct magnify_pixel_rec *mag_pixel);
void free_magnify_pixel_rec(struct magnify_pixel_rec *mag_pixel);
int magnify_pixel_magnify_proc(Panel_item item, Event event);
void magnify_pixel_quit_proc(Panel_item item, Event event);
void magnify_pixel_done_proc(Frame frame);
void magnify_pixel_repaint_proc(Canvas canvas, Pixwin *pw,
    Rectlist *repaint_area);
void magnify_pixel_event_proc(Xv_Window canvas, Event *event);
int define_magnify_pixel_rects(struct magnify_pixel_rec *mag_pixel,
    unsigned int num_rows, unsigned int num_cols);
void draw_magnify_pixel_rects(struct magnify_pixel_rec *mag_pixel);
void window_resolution_text_proc(Panel_item item, Event *event);
void window_resolution_proc(Panel_item item, int value, Event *event);
void notify_magnify_pixel_proc(Panel_item item, int value, Event *event);
int setup_magnify_pixel_proc(struct magnify_pixel_rec *mag_pixel,
    struct image_display_window *win);
int enter_magnify_pixel(struct image_display_window *win, int status);




/*--------------------------------------------------------------------------
init_magnify_pixel_rec - ???
--------------------------------------------------------------------------*/

void
init_magnify_pixel_rec(mag_pixel)
 struct magnify_pixel_rec *mag_pixel;
{
 mag_pixel->status = 0;
 mag_pixel->win = NULL;
 mag_pixel->frame = 0;
 mag_pixel->win = NULL;
 mag_pixel->num_row_rects = 0;
 mag_pixel->num_col_rects = 0;
}


/*--------------------------------------------------------------------------
free_magnify_pixel_memory - ???
--------------------------------------------------------------------------*/

void
free_magnify_pixel_memory(mag_pixel)
 struct magnify_pixel_rec *mag_pixel;
{
 if (mag_pixel->num_col_rects > 0)
    {free_handle_memory(mag_pixel->rr,mag_pixel->num_row_rects);
     free_handle_memory(mag_pixel->mask,mag_pixel->num_row_rects);
     free_handle_memory(mag_pixel->color,mag_pixel->num_row_rects);
     mag_pixel->num_row_rects = mag_pixel->num_col_rects = 0;
    }
}


/*--------------------------------------------------------------------------
free_magnify_pixel_rec - ???
--------------------------------------------------------------------------*/

void
free_magnify_pixel_rec(mag_pixel)
 struct magnify_pixel_rec *mag_pixel;
{
 free_magnify_pixel_memory(mag_pixel);
 xv_destroy_safe(mag_pixel->frame);
 mag_pixel->frame = 0;
 init_magnify_pixel_rec(mag_pixel); 
}


/*--------------------------------------------------------------------------
magnify_pixel_magnify_proc - ???
--------------------------------------------------------------------------*/

int
magnify_pixel_magnify_proc(item,event)
 Panel_item item;
 Event event;
{
 struct magnify_pixel_rec *mag_pixel;
 int row,col;
 struct image_display_window *win;
 int magnify_which_win;
 int err;

 mag_pixel = (struct magnify_pixel_rec *) xv_get(item,PANEL_CLIENT_DATA);
 win = mag_pixel->win;
 win->dwin = mag_pixel->dwin;
 if (mag_pixel->res != 0)
    mag_pixel->dwin->screen.dimension.rr.width = 
        mag_pixel->dwin->screen.dimension.rr.height = 
                 (int) xv_get(mag_pixel->res,PANEL_VALUE);

 for (row = 0; row < mag_pixel->num_row_rects; row++)
    for (col = 0; col < mag_pixel->num_col_rects; col++)
       if (mag_pixel->mask[row][col] == 0) mag_pixel->color[row][col] = 0;

 magnify_which_win = (int) xv_get(mag_pixel->win_option,PANEL_VALUE);
 err = win_get_multiple_win(win,magnify_which_win,
       mag_pixel->num_col_rects,mag_pixel->num_row_rects,
       0,0,0,DIM_FIXED_MAG);
 free_magnify_pixel_rec(mag_pixel);
 return err;
}


/*--------------------------------------------------------------------------
magnify_pixel_quit_proc - ???
--------------------------------------------------------------------------*/

void
magnify_pixel_quit_proc(item,event)
 Panel_item item;
 Event event;
{
 struct magnify_pixel_rec *mag_pixel;

 mag_pixel = (struct magnify_pixel_rec *) xv_get(item,PANEL_CLIENT_DATA);
 free_magnify_pixel_rec(mag_pixel);

}


/*--------------------------------------------------------------------------
magnify_pixel_done_proc( - ???
--------------------------------------------------------------------------*/

void
magnify_pixel_done_proc(frame)
 Frame frame;
{
 struct magnify_pixel_rec *mag_pixel;

 mag_pixel = (struct magnify_pixel_rec *) xv_get(frame,WIN_CLIENT_DATA);
 free_magnify_pixel_rec(mag_pixel);
}


/*--------------------------------------------------------------------------
magnify_pixel_repaint_proc - ???
--------------------------------------------------------------------------*/

void
magnify_pixel_repaint_proc(canvas,pw,repaint_area)
 Canvas canvas;
 Pixwin *pw;
 Rectlist *repaint_area;
{
 struct magnify_pixel_rec *mag_pixel;

 mag_pixel = (struct magnify_pixel_rec *) xv_get(canvas,WIN_CLIENT_DATA);

 draw_magnify_pixel_rects(mag_pixel);

}


/*--------------------------------------------------------------------------
magnify_pixel_event_proc - ???
--------------------------------------------------------------------------*/

void
magnify_pixel_event_proc(canvas,event)
 Xv_Window canvas;
 Event *event;
{
 struct magnify_pixel_rec *mag_pixel;
 int x,y,row,col;
 static int rowindx,colindx;

 mag_pixel = (struct magnify_pixel_rec *) xv_get(canvas,WIN_CLIENT_DATA);
 if (event_is_up(event))
    {colindx = -1;
     return;}
 x = event_x(event);
 y = event_y(event);
 switch (event_action(event))
    {
     case ACTION_SELECT :
     case ACTION_MENU :
     case ACTION_ADJUST :

     case LOC_DRAG :
        for (row = 0; row < mag_pixel->num_row_rects; row++)
           for (col = 0; col < mag_pixel->num_col_rects; col++)
              if (pt_in_int_rect(&(mag_pixel->rr[row][col]),x,y))
                 {if ((row == rowindx) && (col == colindx)) return;
                  if (mag_pixel->mask[row][col] == 0)
                     mag_pixel->mask[row][col] = 1;
                    else
                     {mag_pixel->mask[row][col] = 0;
                      draw_int_rect(mag_pixel->canvas,NULL,1,
                          &(mag_pixel->rr[row][col]),
                          (unsigned int) 0,1);}

                  draw_int_rect(mag_pixel->canvas,NULL,1,
                     &(mag_pixel->rr[row][col]),
                     (unsigned int) mag_pixel->color[row][col],
                     (int) mag_pixel->mask[row][col]);
                  rowindx = row;
                  colindx = col;
                  return;
                 }
         colindx = -1;
     default : colindx = -1;
    }
}


/*--------------------------------------------------------------------------
define_magnify_pixel_rects - ???
--------------------------------------------------------------------------*/

int
define_magnify_pixel_rects(mag_pixel,num_rows,num_cols)
 struct magnify_pixel_rec *mag_pixel;
 unsigned int num_rows,num_cols;
{
 int err;
 int height,width,centerx,centery,x,y,row,col;
 unsigned int total;

 free_magnify_pixel_memory(mag_pixel);
 mag_pixel->num_row_rects = mag_pixel->num_col_rects = 0;
/*
 total = 0;
 get_handle_memory(&(mag_pixel->rr),&total,
      (unsigned int) num_rows,
      (size_t) (sizeof(struct int_rect_rec) * num_cols),&err);
 if (err != NO_ERROR) return err;
*/

 total = 0;
 get_handle_memory(&(mag_pixel->mask),&total,
      (unsigned int) num_rows,
      (size_t) (sizeof(**(mag_pixel->mask)) * num_cols),&err);
 if (err != NO_ERROR)
    {
     return err;}

 get_handle_memory(&(mag_pixel->color),&(mag_pixel->num_row_rects),
      (unsigned int) num_rows,
      (size_t) (sizeof(**(mag_pixel->color)) * num_cols),&err);
 if (err != NO_ERROR)
    {
     free_handle_memory(mag_pixel->mask,num_rows);
     mag_pixel->num_row_rects = mag_pixel->num_col_rects = 0;
     return err;}

 mag_pixel->num_col_rects = num_cols;
 mag_pixel->total_rects = mag_pixel->num_row_rects * mag_pixel->num_col_rects;
 height = (int) xv_get(mag_pixel->canvas,XV_HEIGHT);
 centery = height / 2;
 if ((height - 20) > 0)
    {y = 10;
     height -= 20;}
   else y = 0;

 width = (int) xv_get(mag_pixel->canvas,XV_WIDTH);
 centerx = width / 2;
 if ((width - 20) > 0)
    {x = 10;
     width -= 20;}
   else x = 0;

 err = build_2d_int_rects(&(mag_pixel->rr),mag_pixel->num_row_rects,
             mag_pixel->num_col_rects,mag_pixel->total_rects,x,y,
             width,height,3,3);
 if (err != NO_ERROR)
    {
     free_handle_memory(mag_pixel->color,mag_pixel->num_row_rects);
     free_handle_memory(mag_pixel->mask,num_rows);
     mag_pixel->num_row_rects = mag_pixel->num_col_rects = 0;
     return err;}

 for (row = 0; row < mag_pixel->num_row_rects; row++)
    for (col = 0; col < mag_pixel->num_col_rects; col++)
       {mag_pixel->mask[row][col] = 1;
        mag_pixel->color[row][col] = mag_pixel->win->
                                       intensity->color.total_colors - 1;}

 return NO_ERROR;           
}


/*--------------------------------------------------------------------------
draw_magnify_pixel_rects - ???
--------------------------------------------------------------------------*/

void
draw_magnify_pixel_rects(mag_pixel)
 struct magnify_pixel_rec *mag_pixel;
{
 unsigned int row,col;
/*
 int height,width;
*/
 win_clear(mag_pixel->canvas,NULL,1);

 for (row = 0; row < mag_pixel->num_row_rects; row++)
    for (col = 0; col < mag_pixel->num_col_rects; col++)
       draw_int_rect(mag_pixel->canvas,NULL,1,
           &(mag_pixel->rr[row][col]),
           (unsigned int) mag_pixel->color[row][col],
           (int) mag_pixel->mask[row][col]);

}


/*--------------------------------------------------------------------------
window_resolution_text_proc - ???
--------------------------------------------------------------------------*/

void
window_resolution_text_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct magnify_pixel_rec *mag_pixel;
 int max_val,min_val,value;
 char *str,str1[10];

 mag_pixel = (struct magnify_pixel_rec *) xv_get(item,PANEL_CLIENT_DATA);
 min_val = (int) xv_get(mag_pixel->res,PANEL_MIN_VALUE);
 max_val = (int) xv_get(mag_pixel->res,PANEL_MAX_VALUE);
 str = (char *) xv_get(item,PANEL_VALUE);
 sscanf(str,"%d",&value);
 if ((value < min_val) || (value > max_val))
    {value = (int) xv_get(mag_pixel->res,PANEL_VALUE);
     sprintf(str1,"%d",value);
     xv_set(item,PANEL_VALUE,str1,0);}
   else xv_set(mag_pixel->res,PANEL_VALUE,value,0);
}


/*--------------------------------------------------------------------------
window_resolution_proc - ???
--------------------------------------------------------------------------*/

void
window_resolution_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct magnify_pixel_rec *mag_pixel;
 char str1[10];

 mag_pixel = (struct magnify_pixel_rec *) xv_get(item,PANEL_CLIENT_DATA);
 sprintf(str1,"%d",value);
 xv_set(mag_pixel->res_edit,PANEL_VALUE,str1,0);
     
}


/*--------------------------------------------------------------------------
notify_magnify_pixel_proc - ???
--------------------------------------------------------------------------*/

void
notify_magnify_pixel_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct magnify_pixel_rec *mag_pixel;

 mag_pixel = (struct magnify_pixel_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if (item == mag_pixel->def_mag_fac)
    mag_pixel->dwin->screen.def_mag_fac = (int) xv_get(item,PANEL_VALUE);
   else if (mag_pixel->num_col_rects != value)
    {
     if (define_magnify_pixel_rects(mag_pixel,(unsigned int) value,
                                (unsigned int) value) == NO_ERROR)
         draw_magnify_pixel_rects(mag_pixel);
        else
         free_magnify_pixel_rec(mag_pixel);
    }
}


/*--------------------------------------------------------------------------
setup_magnify_pixel_proc - ???
--------------------------------------------------------------------------*/

int
setup_magnify_pixel_proc(mag_pixel,win)
 struct magnify_pixel_rec *mag_pixel;
 struct image_display_window *win;
{
 int err;
 char *str,str1[10];
 Rect *screen_rr;
 int max_res,row;
/*
 Cms cms;
*/
 int width,width1;
 char *label;

 mag_pixel->frame = xv_create(win->dwin->frame, FRAME,
              /*WIN_DYNAMIC_VISUAL,TRUE,*/
              FRAME_INHERIT_COLORS,TRUE,
              FRAME_SHOW_LABEL, TRUE,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,magnify_pixel_done_proc,
              WIN_CLIENT_DATA,mag_pixel,
              0);
 if (mag_pixel->frame == 0) return CANCEL;
 label = win_rec_win_id_title(win->owner,MAGNIFY_PIXEL);
 xv_set(mag_pixel->frame,XV_LABEL,label,NULL);
 mag_pixel->win = win;
 mag_pixel->dwin = win->dwin;
 mag_pixel->panel = xv_create(mag_pixel->frame,PANEL,
        FRAME_INHERIT_COLORS,TRUE,
        0);
 mag_pixel->canvas = xv_create(mag_pixel->frame,CANVAS,
        /*WIN_DYNAMIC_VISUAL,TRUE,*/
        FRAME_INHERIT_COLORS,TRUE,
        CANVAS_REPAINT_PROC,magnify_pixel_repaint_proc,
        XV_WIDTH,250,
        XV_HEIGHT,250,
        CANVAS_WIDTH,250,
        CANVAS_HEIGHT,250,
        WIN_CLIENT_DATA,mag_pixel,
        0);

 if ((mag_pixel->panel == 0) || (mag_pixel->canvas == 0))
    {free_magnify_pixel_rec(mag_pixel);
     return CANCEL;}

 xv_set(canvas_paint_window(mag_pixel->canvas),
	WIN_EVENT_PROC,magnify_pixel_event_proc,
	WIN_CONSUME_EVENTS,
		MS_LEFT,MS_MIDDLE,MS_RIGHT,WIN_UP_EVENTS,
		LOC_DRAG,
		0,
        WIN_CLIENT_DATA,mag_pixel,
	0);

 TRACE(printf("before create MAG-FACTOR panel button...\n"))
 xv_create(mag_pixel->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"X-Y MAG-FACTOR",
	PANEL_VALUE,(int) mag_pixel->dwin->screen.pixel.mask.dimx,
	PANEL_MIN_VALUE,MIN_MAGNIFY_PIXEL_DIM,
	PANEL_MAX_VALUE,MAX_MAGNIFY_PIXEL_DIM,
	PANEL_SLIDER_WIDTH,150,
	PANEL_NOTIFY_PROC,notify_magnify_pixel_proc,
        PANEL_CLIENT_DATA,mag_pixel,
        PANEL_VALUE_FONT,mag_pixel->dwin->font,
	0);
 TRACE(printf("before create def_mag_fac panel button...\n"))
 mag_pixel->def_mag_fac = xv_create(mag_pixel->panel,PANEL_SLIDER,
	PANEL_LABEL_STRING,"DEFAULT MAG-FACTOR",
        XV_X,xv_cols(mag_pixel->panel,0),
        XV_Y,xv_rows(mag_pixel->panel,1),
	PANEL_VALUE,(int) mag_pixel->dwin->screen.def_mag_fac,
	PANEL_MIN_VALUE,MIN_MAGNIFY_PIXEL_DIM,
	PANEL_MAX_VALUE,MAX_MAGNIFY_PIXEL_DIM,
	PANEL_SLIDER_WIDTH,150,
	PANEL_NOTIFY_PROC,notify_magnify_pixel_proc,
        PANEL_CLIENT_DATA,mag_pixel,
	0);
 screen_rr = (Rect *) xv_get(mag_pixel->frame,WIN_SCREEN_RECT);
 if (screen_rr->r_height < screen_rr->r_width) max_res = screen_rr->r_height;
   else max_res = screen_rr->r_width;
 if (max_res > 256)
    {
     TRACE(printf("before create res_edit panel button...\n"))
     sprintf(str1,"%d",
        (int) mag_pixel->dwin->screen.dimension.rr.width);
     mag_pixel->res_edit = xv_create(mag_pixel->panel,PANEL_TEXT,
	PANEL_LABEL_STRING,"ENTER WINDOW X-Y DIMENSION (IN PIXELS) =",
	PANEL_VALUE,str1,
        PANEL_VALUE_DISPLAY_LENGTH,8,
        PANEL_VALUE_STORED_LENGTH,9,
        XV_X,xv_cols(mag_pixel->panel,0),
        XV_Y,xv_rows(mag_pixel->panel,2),
        PANEL_CLIENT_DATA,mag_pixel,
	PANEL_NOTIFY_PROC,window_resolution_text_proc,
        0);
     if ((max_res - 30) > 256) max_res -= 30;
     mag_pixel->res = xv_create(mag_pixel->panel,PANEL_SLIDER,
        XV_X,xv_cols(mag_pixel->panel,0),
        XV_Y,xv_rows(mag_pixel->panel,3),
	PANEL_LABEL_STRING,"X-Y DIMENSION",
	PANEL_VALUE,(int) mag_pixel->dwin->screen.dimension.rr.width,
	PANEL_MIN_VALUE,256,PANEL_MAX_VALUE,max_res,PANEL_SLIDER_WIDTH,150,
        PANEL_CLIENT_DATA,mag_pixel,
	PANEL_NOTIFY_PROC,window_resolution_proc,
        0);

     row = 4;
    }
   else
    {row = 2;
     mag_pixel->res = mag_pixel->res_edit = 0;
    }

 TRACE(printf("before create magnify panel button...\n"))
 str = "MAGNIFY";
 xv_create(mag_pixel->panel,PANEL_BUTTON,
                   XV_X,xv_cols(mag_pixel->panel,0),
                   XV_Y,xv_rows(mag_pixel->panel,row),
	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,magnify_pixel_magnify_proc,
                   PANEL_CLIENT_DATA,mag_pixel,
                   0);
 str = "CANCEL";
 xv_create(mag_pixel->panel,PANEL_BUTTON,
	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,magnify_pixel_quit_proc,
                   PANEL_CLIENT_DATA,mag_pixel,
                   0);
 str = "MAGNIFY WINDOW :";
 if (mag_pixel->dwin->status2 & MASTER_WINDOW)
    mag_pixel->win_option = xv_create(mag_pixel->panel,PANEL_CYCLE,
 	           PANEL_LABEL_STRING, str,
                   PANEL_CHOICE_STRINGS,"CURRENT","NEW MASTER","NEW SLAVE",0,
                   PANEL_LABEL_FONT,mag_pixel->dwin->font,
                   0);
   else
    mag_pixel->win_option = xv_create(mag_pixel->panel,PANEL_CYCLE,
 	           PANEL_LABEL_STRING, str,
                   PANEL_CHOICE_STRINGS,"CURRENT","NEW MASTER",0,
                   PANEL_LABEL_FONT,mag_pixel->dwin->font,
                   0);

 window_fit(mag_pixel->panel);
 width = (int) xv_get(mag_pixel->panel,XV_WIDTH);
 width1 = (int) xv_get(mag_pixel->canvas,XV_WIDTH);
 if (width < width1) xv_set(mag_pixel->panel,XV_WIDTH,width1,0);

 xv_set(mag_pixel->canvas,WIN_BELOW,mag_pixel->panel,0);
 xv_set(mag_pixel->canvas,XV_X,0,0);

 window_fit(mag_pixel->frame);

 if ((err = define_magnify_pixel_rects(mag_pixel,
      (unsigned int) mag_pixel->dwin->screen.pixel.mask.dimx,
      (unsigned int) mag_pixel->dwin->screen.pixel.mask.dimy)) != NO_ERROR)
    free_magnify_pixel_rec(mag_pixel);

 make_2_wins_adjacent(mag_pixel->frame,win->dwin->frame,0);
 win_set_mouse_at_center(mag_pixel->frame);

 return err;

}


/*--------------------------------------------------------------------------
enter_magnify_pixel - ???
--------------------------------------------------------------------------*/

#define CLEAN_UP_MAGNIFY_PIXEL 0
#define SETUP_MAGNIFY_PIXEL 1
#define CLOSE 10
#define OPEN 11

int
enter_magnify_pixel(win,status)
 struct image_display_window *win;
 int status;
{
 static struct magnify_pixel_rec mag_pixel;
 static char first_time = 1;
 int err;

 err= NO_ERROR;
 if (status == CLEAN_UP_MAGNIFY_PIXEL)
     {if ((first_time != 1) && (mag_pixel.win == win) &&
          (mag_pixel.dwin == win->dwin))
         if (mag_pixel.frame != 0) free_magnify_pixel_rec(&mag_pixel);
     return NO_ERROR;}
 if (status == CLOSE)
    {if ((first_time != 1) && (mag_pixel.win == win) &&
         (mag_pixel.dwin == win->dwin) && (mag_pixel.frame != 0))
        {mag_pixel.status |= CLOSE_UPON_OWNER_REQUEST;
         xv_set(mag_pixel.frame,FRAME_CLOSED,TRUE,0);
        }
     return NO_ERROR;}
 if (status == OPEN)
    {if ((first_time != 1) && (mag_pixel.win == win) &&
         (mag_pixel.dwin == win->dwin) && (mag_pixel.frame != 0))
        {if (mag_pixel.status & CLOSE_UPON_OWNER_REQUEST)
            {mag_pixel.status &= ~CLOSE_UPON_OWNER_REQUEST;
             xv_set(mag_pixel.frame,FRAME_CLOSED,FALSE,0);}
        }
     return NO_ERROR;}

 if (first_time)
    {init_magnify_pixel_rec(&mag_pixel);
     first_time = 0;}
   else if (mag_pixel.frame != 0)
    {mag_pixel.status &= ~CLOSE_UPON_OWNER_REQUEST;
     if ((mag_pixel.win != win) || (mag_pixel.dwin != win->dwin))
        free_magnify_pixel_rec(&mag_pixel);
       else
        {if (xv_get(mag_pixel.frame,FRAME_CLOSED))
            xv_set(mag_pixel.frame,FRAME_CLOSED,FALSE,0);
         if (!xv_get(mag_pixel.frame,XV_SHOW))
            xv_set(mag_pixel.frame,XV_SHOW,TRUE,0);
         wmgr_top(mag_pixel.frame);
         return NO_ERROR;}
    }
 if (mag_pixel.frame == 0)
    err = setup_magnify_pixel_proc(&mag_pixel,win);

 if ((mag_pixel.frame != 0) && (err == NO_ERROR))
    {
     xv_set(mag_pixel.frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
    }
 return err;
}
