/****************************************************************************
pixel_desc_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: pixel_desc_rec.c,v 1.6 1999/07/07 17:41:18 hughett Exp $ Release $Name: nria_1_0_6 $";

#include "mri_format.h"
#include "nria_procedures.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT1(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)
#define PRT1(x)


/* Functions defined in this file */
void init_index_value_rec(struct index_value_rec *index_value);
int setup_index_value_rec(struct index_value_rec *index_value,
    unsigned int num_xys, int is_value);
void free_index_value_rec(struct index_value_rec *index_value);
void init_pixel_desc_rec(struct pixel_desc_rec *pixel, int def_magx,
    int def_magy);
void free_pixel_desc_rec(struct pixel_desc_rec *pixel);
int copy_index_value_rec(struct index_value_rec *dest,
    struct index_value_rec *source);
int copy_pixel_desc_rec(struct pixel_desc_rec *dest,
    struct pixel_desc_rec *source);
int swap_pixel_desc_rec(struct pixel_desc_rec *pixel1,
    struct pixel_desc_rec *pixel2);
int setup_pixel_desc_rec(struct pixel_desc_rec *pixel, unsigned int dimx,
    unsigned int dimy, unsigned char **mask, unsigned int **value);
int is_same_mag(struct pixel_desc_rec *pix1, struct pixel_desc_rec *pix2);
void pr_pixel_desc_rec(struct pixel_desc_rec *pixel);
long screenx_to_bufx(struct screen_data_rec *screen, long x,
    unsigned long dimx);
long screeny_to_bufy(struct screen_data_rec *screen, long y,
    unsigned long dimy);
int bufx_to_screenx(struct screen_data_rec *screen, long x);
int bufy_to_screeny(struct screen_data_rec *screen, int y);
int bufx_to_screenx_center(struct screen_data_rec *screen, int x);
int bufy_to_screeny_center(struct screen_data_rec *screen, int y);



/*---------------------------------------------------------------------------
init_index_value_rec - ???
---------------------------------------------------------------------------*/

void
init_index_value_rec(index_value)
 struct index_value_rec *index_value;
{
 index_value->num_xys = 0;
 index_value->x = NULL;
 index_value->y = NULL;
 index_value->value = NULL;
}


/*---------------------------------------------------------------------------
setup_index_value_rec - ???
---------------------------------------------------------------------------*/

int
setup_index_value_rec(index_value,num_xys,is_value)
 struct index_value_rec *index_value;
 unsigned int num_xys;
 int is_value;
{
 unsigned int total_items;
 int err;

 if (num_xys == 0)
     return ~NO_ERROR;   /* FIXME:  Guessed at right value.  -PWH 19990421 */

 index_value->num_xys = num_xys;
 if (is_value) num_xys *= 3;
   else num_xys *= 2;

 total_items = 0;
 get_ptr_memory(&(index_value->x),&total_items,num_xys,
                    (size_t) sizeof(unsigned int),&err);
 if (err != NO_ERROR) return err;

 index_value->y = index_value->x + index_value->num_xys;

 if (is_value)
    {index_value->value = index_value->y + index_value->num_xys;
     }
   else index_value->value = NULL;
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
free_index_value_rec - ???
---------------------------------------------------------------------------*/

void
free_index_value_rec(index_value)
 struct index_value_rec *index_value;
{
 c_free(index_value->x);
 init_index_value_rec(index_value);
}


/*---------------------------------------------------------------------------
init_pixel_desc_rec - ???
---------------------------------------------------------------------------*/

void
init_pixel_desc_rec(pixel,def_magx,def_magy)
 struct pixel_desc_rec *pixel;
 int def_magx,def_magy;
{
 init_buf_2d_rec(&(pixel->mask));
 init_index_value_rec(&(pixel->fore));
 init_index_value_rec(&(pixel->back));
 pixel->def_magx = def_magx;
 pixel->def_magy = def_magy;
}


/*---------------------------------------------------------------------------
free_pixel_desc_rec - ???
---------------------------------------------------------------------------*/

void
free_pixel_desc_rec(pixel)
 struct pixel_desc_rec *pixel;
{
 free_buf_2d_rec(&(pixel->mask));
 free_index_value_rec(&(pixel->fore));
 free_index_value_rec(&(pixel->back));
 pixel->def_magx = 0;
 pixel->def_magy = 0;
}


/*---------------------------------------------------------------------------
copy_index_value_rec - ???
---------------------------------------------------------------------------*/

int
copy_index_value_rec(dest,source)
 struct index_value_rec *dest,*source;
{
 int err;
 int is_value;

 if (source->value != NULL) is_value = 1;
   else is_value = 0;
 if ((err = setup_index_value_rec(dest,source->num_xys,is_value)) == NO_ERROR)
    {dest->num_xys = source->num_xys;
     memcpy(dest->x,source->x,(size_t) (dest->num_xys * sizeof(*(dest->x))));
     memcpy(dest->y,source->y,(size_t) (dest->num_xys * sizeof(*(dest->y))));
     if (is_value)
        memcpy(dest->value,source->value,
                (size_t) (dest->num_xys * sizeof(*(dest->value))));}
 return err;
}



/*---------------------------------------------------------------------------
copy_pixel_desc_rec - ???
---------------------------------------------------------------------------*/

int
copy_pixel_desc_rec(dest,source)
 struct pixel_desc_rec *dest,*source;
{
 int err;

 free_pixel_desc_rec(dest);
 if (!(source->mask.status4 & BUF_NON_EMPTY)) return NO_ERROR;
 if ((err = copy_buf_2d_rec(&(dest->mask),&(source->mask))) == NO_ERROR)
    if (((err = copy_index_value_rec(&(dest->fore),&(source->fore)))
          != NO_ERROR) ||
        ((err = copy_index_value_rec(&(dest->back),&(source->back)))
          != NO_ERROR))
       free_pixel_desc_rec(dest);
 dest->def_magx = source->def_magx;
 dest->def_magy = source->def_magy;
 return err;
}


/*---------------------------------------------------------------------------
swap_pixel_desc_rec - ???
---------------------------------------------------------------------------*/

int
swap_pixel_desc_rec(pixel1,pixel2)
 struct pixel_desc_rec *pixel1,*pixel2;
{
 struct pixel_desc_rec temp_pixel;
/*
 int err;

 init_pixel_desc_rec(&temp_pixel,0,0);
 if ((err = copy_pixel_desc_rec(&temp_pixel,pixel1)) != NO_ERROR) return err;
 if ((err = copy_pixel_desc_rec(pixel1,pixel2)) == NO_ERROR)
    err = copy_pixel_desc_rec(pixel2,&temp_pixel);
 free_pixel_desc_rec(&temp_pixel);
 return err;
*/
 temp_pixel = *pixel1;
 *pixel1 = *pixel2;
 *pixel2 = temp_pixel;
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
setup_pixel_desc_rec - ???
---------------------------------------------------------------------------*/

int
setup_pixel_desc_rec(pixel,dimx,dimy,mask,value)
 struct pixel_desc_rec *pixel;
 unsigned int dimx,dimy;
 unsigned char **mask;
 unsigned int **value;
{
 int err;
 unsigned int row,col;

 TRACE(printf("setup_pixel_desc_rec...\n"))
 if ((err = SetupBuf2dRec(&(pixel->mask),dimx,dimy,
            (size_t) 8,0)) == NO_ERROR)
    {
     init_index_value_rec(&(pixel->fore));
     init_index_value_rec(&(pixel->back));

     for (pixel->fore.num_xys = 0,pixel->back.num_xys = 0,row = 0;
          row < dimy; row++)
        {
         for (col = 0; col < dimx; col++)
            {pixel->mask.buf[row][col] = mask[row][col];
             if (mask[row][col]) pixel->fore.num_xys++;
               else pixel->back.num_xys++;
            }
        }
     if (pixel->fore.num_xys > 0)
        if ((err = setup_index_value_rec(&(pixel->fore),
                     pixel->fore.num_xys,0)) != NO_ERROR) return err;

     if (pixel->back.num_xys > 0)
        if ((err = setup_index_value_rec(&(pixel->back),
                    pixel->back.num_xys,1)) != NO_ERROR) return err;

     for (pixel->fore.num_xys = 0,pixel->back.num_xys = 0,row = 0;
          row < dimy; row++)
        {for (col = 0; col < dimx; col++)
           if (mask[row][col])
              {pixel->fore.x[pixel->fore.num_xys] = col;
               pixel->fore.y[pixel->fore.num_xys] = row;
               pixel->fore.num_xys++;
              }
             else
              {pixel->back.x[pixel->back.num_xys] = col;
               pixel->back.y[pixel->back.num_xys] = row;
               pixel->back.value[pixel->back.num_xys] = value[row][col];
               pixel->back.num_xys++;
             }
        }

   }
 TRACE(printf("setup_pixel_desc_rec...end\n"))
 return err;
}

/******************
     printf("num_xys = %d x_addr = %d y_addr = %d\n",pixel->fore.num_xys,
      (int)  &(pixel->fore.x[pixel->fore.num_xys]),
      (int)  &(pixel->fore.y[pixel->fore.num_xys]));


      printf("back num_xys = %d x_addr = %d y_addr = %d v_addr = %d %d\n",
       pixel->back.num_xys,
      (int)  &(pixel->back.x[pixel->back.num_xys]),
      (int)  &(pixel->back.y[pixel->back.num_xys]),
      (int)  &(pixel->back.value[pixel->back.num_xys]),
           value[row][col]);
******************/



/*---------------------------------------------------------------------------
is_same_mag - ???
---------------------------------------------------------------------------*/

int
is_same_mag(pix1,pix2)
 struct pixel_desc_rec *pix1,*pix2;
{
PRT(printf("pix1 magx = %d magy = %d pix2 magx = %d magy = %d\n",
        pix1->mask.dimx,pix1->mask.dimy,pix2->mask.dimx,pix2->mask.dimy))
 if ((pix1->mask.dimx != pix2->mask.dimx) ||
     (pix1->mask.dimy != pix2->mask.dimy))
    {PRT(printf("not is_same_mag\n"))
     return 0;}
 return 1;
}


/*---------------------------------------------------------------------------
pr_pixel_desc_rec - ???
---------------------------------------------------------------------------*/

void
pr_pixel_desc_rec(pixel)
 struct pixel_desc_rec *pixel;
{
 unsigned int row,col;

 for (row = 0; row < pixel->mask.dimy; row++)
    for (col = 0; col < pixel->mask.dimx; col++)
       printf("row = %d col = %d value = %d\n",row,col,
               (int) pixel->mask.buf[row][col]);

 printf("ptr addr fore = %p %p\n",pixel->fore.x,pixel->fore.y);
 for (row = 0; row < pixel->fore.num_xys; row++)
    printf("row = %d x = %ld y = %ld\n",row,
           pixel->fore.x[row],pixel->fore.y[row]);
 printf("ptr addr back = %p %p %p\n",pixel->back.x,pixel->back.y,pixel->back.value);
 for (row = 0; row < pixel->back.num_xys; row++)
    printf("row = %d x = %ld y = %ld value = %ld\n",row,
           pixel->back.x[row],pixel->back.y[row],pixel->back.value[row]);
}



/*---------------------------------------------------------------------------
screenx_to_bufx - ???

mapping coordinate x from the display data buffer to the color data
   buffer
---------------------------------------------------------------------------*/

long
screenx_to_bufx( screen, x, dimx )
     struct screen_data_rec *screen;
     long x;
     unsigned long dimx;
{
  long old_x = 0;
     
  if (x <= 0)
    old_x = screen->dimension.bound.left;
  else
  {
    old_x = screen->dimension.bound.left + x / (long) screen->pixel.mask.dimx;
    if (old_x >= (long) dimx)
      old_x = (long) dimx - 1;
  }
  PRT1(printf("dim left = %d coord_x1 newx = %d oldx = %d dimx = %d\n",
	      screen->dimension.bound.left,old_x,x,screen->pixel.mask.dimx));
  return old_x;
}



/*---------------------------------------------------------------------------
screeny_to_bufy - ???

mapping coordinate y from the display data buffer to the color data
   buffer 
---------------------------------------------------------------------------*/

long
screeny_to_bufy( screen, y, dimy )
     struct screen_data_rec *screen;
     long y;
     unsigned long dimy;
{
  long old_y = 0;

  if (y <= 0)
    old_y = screen->dimension.bound.top;
  else
  {
    old_y = screen->dimension.bound.top + y / (long) screen->pixel.mask.dimy;
    if (old_y >= (long) dimy)
      old_y = (long) dimy - 1;
  }
  PRT1(printf("bound.top = %d coord_y1 = %d y = %d dimy = %d\n",
	      screen->dimension.bound.top,old_y,y,screen->pixel.mask.dimy));
  return old_y;
}



/*---------------------------------------------------------------------------
bufx_to_screenx - ???

mapping coordinate x from the color data buffer to the display data buffer
---------------------------------------------------------------------------*/

int
bufx_to_screenx(screen,x)
 struct screen_data_rec *screen;
 long x;
{
 int temp;

 temp = (x - screen->dimension.bound.left) * screen->pixel.mask.dimx;
 return temp;
}


/*---------------------------------------------------------------------------
 - ???

mapping coordinate y from the color data buffer to the display data buffer
---------------------------------------------------------------------------*/

int
bufy_to_screeny(screen,y)
 struct screen_data_rec *screen;
 int y;
{
 int temp;

 temp = (y - screen->dimension.bound.top) * screen->pixel.mask.dimy;

 return temp;
}


/*---------------------------------------------------------------------------
bufx_to_screenx_center - ???

mapping coordinate x from the color data buffer to the display data buffer
---------------------------------------------------------------------------*/

int
bufx_to_screenx_center(screen,x)
 struct screen_data_rec *screen;
 int x;
{
 int temp;

 temp = (x - screen->dimension.bound.left) * screen->pixel.mask.dimx + 
             screen->pixel.mask.dimx / 2;
 return temp;
}


/*---------------------------------------------------------------------------
bufy_to_screeny_center - ???

mapping coordinate y from the color data buffer to the display data buffer
---------------------------------------------------------------------------*/

int
bufy_to_screeny_center(screen,y)
 struct screen_data_rec *screen;
 int y;
{
 int temp;

 temp = (y - screen->dimension.bound.top) * screen->pixel.mask.dimy + 
             screen->pixel.mask.dimy / 2;
 return temp;
}
