/****************************************************************************
prog_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: prog_rec.c,v 1.7 1999/07/07 17:41:18 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <unistd.h>       /* For chdir and getcwd */

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"


/* Functions defined in this file */
void init_defaults_rec(union defaults_rec *dfaults);
void free_defaults_rec(union defaults_rec *defaults);
char *get_prog_def_fn(struct prog_rec *prog, char *fn);
char *get_prog_dir(struct prog_rec *prog);
int assign_defaults(union defaults_rec *defaults, int select, void *arg);
int setup_prog_rec(struct prog_rec *prog, char *defaults_fn,
    int is_defaults_fn_supplied);
void init_prog_rec(struct prog_rec *prog, char *prog_name);
void free_prog_rec(struct prog_rec *prog);



/*--------------------------------------------------------------------------
init_defaults_rec - ???
--------------------------------------------------------------------------*/

void
init_defaults_rec( dfaults )
     union defaults_rec *dfaults;
{
  unsigned int lcv;

  for (lcv = 0; lcv < NUM_DEFAULT_FNS; lcv++)
    dfaults->fns[lcv] = NULL;
}



/*--------------------------------------------------------------------------
free_defaults_rec - ???
--------------------------------------------------------------------------*/

void
free_defaults_rec( defaults )
     union defaults_rec *defaults;
{
  unsigned int lcv;
  for (lcv = 0; lcv < NUM_DEFAULT_FNS; lcv++)
    c_free(defaults->fns[lcv]);
}



/*--------------------------------------------------------------------------
get_prog_def_fn - ???
--------------------------------------------------------------------------*/

char *
get_prog_def_fn(prog,fn)
     struct prog_rec *prog;
     char *fn;
{
  char *dir,*fn1;
  int err;

  if ((dir = (char *) locate_dirfn(fn)) != NULL)
  {
    c_free(dir);
    return NULL;
  }
  dir = (char *) locate_dirfn(prog->prog_name);
  if (get_strlen(dir) > 0)
  {
    if ((err = concat_strs(&fn1,3,dir,"/",fn)) != NO_ERROR)
      return NULL;
  }
  else if ((err = cp_str(&fn1,fn)) != NO_ERROR)
    return NULL;
  c_free(dir);
  return fn1;
}



/*--------------------------------------------------------------------------
get_prog_dir - ???
--------------------------------------------------------------------------*/

char *
get_prog_dir(prog)
 struct prog_rec *prog;
{
 char *dir;

 dir = (char *) locate_dirfn(prog->prog_name);
 return dir;
}


/*--------------------------------------------------------------------------
assign_defaults - ???
--------------------------------------------------------------------------*/

int
assign_defaults(defaults,select,arg)
 union defaults_rec *defaults;
 int select;
 void *arg;
{
 int err;

 if ((select >= 0) && (select < NUM_DEFAULT_FNS))
    if (get_strlen(arg) == 0)
       {if (select == VALUE_DEFAULT_HDR_DIR)
	  defaults->fns[VALUE_DEFAULT_HDR_DIR] = (char *) getcwd(NULL,256);
        else if (select == VALUE_DEFAULT_DAT_DIR)
	  defaults->fns[VALUE_DEFAULT_DAT_DIR] = (char *) getcwd(NULL,256);
	else if (select == VALUE_DEFAULT_RATER)
	  cp_str(&(defaults->fns[VALUE_DEFAULT_RATER]),getenv("USER"));
	else if (select == VALUE_MAG_FACTOR)
	  cp_str(&(defaults->fns[VALUE_MAG_FACTOR]),"2");
       }
      else if ((err = cp_str(&(defaults->fns[select]),(char *) arg))
                != NO_ERROR)
           {printf("Not enough memory...err = %d\n",err);
            exit(0);}
      else
       {if (select == VALUE_DEFAULT_HDR_DIR)
           {if (valid_dir(defaults->fns[VALUE_DEFAULT_HDR_DIR]))
               {c_free(defaults->fns[VALUE_DEFAULT_HDR_DIR]);
                defaults->fns[VALUE_DEFAULT_HDR_DIR] =
                      (char *) getcwd(NULL,256);}
              else chdir(defaults->fns[VALUE_DEFAULT_HDR_DIR]);
           }
          else if (select == VALUE_DEFAULT_DAT_DIR)
           {if (valid_dir(defaults->fns[VALUE_DEFAULT_DAT_DIR]))
               {c_free(defaults->fns[VALUE_DEFAULT_DAT_DIR]);
                defaults->fns[VALUE_DEFAULT_DAT_DIR] = 
                      (char *) getcwd(NULL,256);}
            
           }
       }
 return NO_ERROR;
}



/*--------------------------------------------------------------------------
setup_prog_rec - ???

 DRW  961106  removed function get_defaults(); only used
   in one place, this function...

 Old version:
   get_defaults(prog,defaults,defaults_fn,is_defaults_fn_supplied)
   struct prog_rec *prog;
 	 union defaults_rec *defaults;
 	 char *defaults_fn;
 	 int is_defaults_fn_supplied;
 
 
     id function            status              default value
    
     0  color fn                  r/w file          color.dat
     1  roi labels fn for the     r/w file          pixel_type_roi_labels.main
          pixel type analysis
     2  pixel types fn            r/w file          pixel_types.main
     3  text output format fn     r/w file          text_format.fmt
     4  batch program             execute-only file batch_main
     5  batch log file            write-only file   batch.status
     6  .hdr directory      directory specification user directory
                                                    from where the
                                                    program is run
     7  header file fns            fn specification *.nhdr
     8  .dat directory      directory specification user directory
                                                    from where the
                                                    program is run
     9  data file fns            fn specification   *.ndat
    10  rater's name              text              login user
    11  default image             1 <= factor <= 20 1
        magnifying factor
    12  bin image_ratio/num_rows/num_cols
                     .25 <= ratio <= 1.0            .5/3/3
    13  secondary roi labels fn   r/w file          pixel_type_roi_labels.sub
    14  secondary pixel types fn  r/w file          pixel_types.sub
    15  pixel types priority fn   r/w file          pixel_types.order
    16  save txt automatically    0 or non-zero     0
    17  text output format fn     r/w file          text_format_1_echo.fmt
          for 1 echo
    18  text output format fn     r/w file         text_format_count_value.fmt
        for count value anal.
    19  roi labels fn for         r/w file         count_value_roi_labels.main
        the count value
        analysis
    20  secondary roi labels      r/w file          count_value_roi_labels.sub
        filename for the count
        value analysis
--------------------------------------------------------------------------*/

int
setup_prog_rec( prog, defaults_fn, is_defaults_fn_supplied )
     struct prog_rec *prog;
     char *defaults_fn;
     int is_defaults_fn_supplied;
{
 FILE *fid;
 int err,select;
 unsigned int num,lcv;
 char str[255],*str1;
 union defaults_rec *defaults;
 static char *default_strs[NUM_DEFAULT_FNS] =         
 {
   DEFAULT_COLOR_FN,
   DEFAULT_ROI_LABELS_FN,
   DEFAULT_PIXEL_TYPES_FN,
   DEFAULT_TXT_FORMAT_FN,
   DEFAULT_BATCH_PROG_FN,

   DEFAULT_BATCH_STATUS_FN,
   "",
   "*.nhdr",
   "",
   "*.ndat",

   "",
   "1",
   DEFAULT_BIN_RATIO_ROW_COL,
   DEFAULT_SEC_ROI_LABELS_FN,
   DEFAULT_SEC_PIXEL_TYPES_FN,

   DEFAULT_PIXEL_TYPE_PRIORITY_FN,
   DEFAULT_AUTO_SAVE_TEXT,
   DEFAULT_TXT_FORMAT_FN_1_ECHO,
   DEFAULT_TXT_FORMAT_FN_COUNT_VALUE,
   DEFAULT_CV_ROI_LABELS_FN,

   DEFAULT_CV_SEC_ROI_LABELS_FN,
 };
 static char append_dirfn[NUM_DEFAULT_FNS] = 
          {1,1,1,1,1, 1,0,0,0,0, 0,0,0,1,1, 1,0,1,1,1, 1};


 prog->defaults_fn = defaults_fn;
 defaults = &(prog->defaults);

 /*  Insert default values before checking defaults file.  */
 TRACE( printf( "Default values:\n" ); );
 for (lcv = 0; lcv < NUM_DEFAULT_FNS; lcv++)
 {
   if (defaults->fns[lcv] == NULL)
   {
     if (append_dirfn[lcv])
     {
       str1 = get_prog_def_fn( prog, default_strs[lcv] );
       assign_defaults( defaults, lcv, str1 );
       TRACE( printf( "	%02d: \"%s\"\n", lcv, str1 ); );
       c_free( str1 );
     }
     else
     {
       assign_defaults( defaults, lcv, default_strs[lcv] );
       TRACE( printf( "	%02d: \"%s\"\n", lcv, default_strs[lcv] ); );
     }
   }
 }

 TRACE( printf( "Trying to open defaults file \"%s\".\n", defaults_fn ); );
 if ((fid = fopen( defaults_fn, "r" )) == NULL)
 {
   if (!is_defaults_fn_supplied)
   {
     TRACE( printf( "	Defaults file does not exist or is not readable.\n" ); );
     TRACE( printf( "		Using internal hard-coded defaults listed above.\n" ); );
     return NO_ERROR;
   }
   prog->defaults_fn = (char *) get_prog_def_fn( prog, defaults_fn );
   TRACE( printf( "	Failed.\nTrying to open defaults file \"%s\".\n", prog->defaults_fn ); );
   if ((fid = fopen( prog->defaults_fn, "r" )) == NULL)
   {
     TRACE( printf( "	Defaults file does not exist or is not readable.\n" ); );
     TRACE( printf( "		Using internal hard-coded defaults listed above.\n" ); );
     /*  DRW 961106: moved default code above  */
     return NO_ERROR;
   }
   else {
     TRACE( printf( "	Found defaults file at \"%s\".\n", prog->defaults_fn ); );
   }
 }
 TRACE( printf( "	Opened.\n" ); );
 while (1)
 {
   num = 1;
   err = RdText(fid,&select,&num,RD_INT,0);
   if ((num != 1) || (err != NO_ERROR)) break;

   num = sizeof(str);
   err = RdText(fid,str,&num,RD_TEXT,0);
   if ((num == 0) || (err != NO_ERROR)) break;

   TRACE( printf( "	Replaced %d with \"%s\".\n", select, str ); );

   if (append_dirfn[select])
   {
     str1 = get_prog_def_fn(prog,str);
     TRACE( printf( "		(Full pathname = \"%s\")\n", str1 ); );
     assign_defaults(defaults,select,str1);
     c_free(str1);
   }
   else
   {
     assign_defaults(defaults,select,str);
   }
 }
 fclose(fid);

 return NO_ERROR;
}


/*--------------------------------------------------------------------------
init_prog_rec - ???
--------------------------------------------------------------------------*/

void
init_prog_rec(prog,prog_name)
 struct prog_rec *prog;
 char *prog_name;
{
 int err;

 prog->prog_name = prog_name;
 if (((err = cp_str(&(prog->prog_id),PROG_ID_V1)) != NO_ERROR) ||
     ((err = cp_str(&(prog->hdr_id),HDR_ID_V1)) != NO_ERROR) ||
     ((err = cp_str(&(prog->dat_id),DAT_ID_V1)) != NO_ERROR))
    {printf("Not enough memory...err = %d\n",err);
     exit(0);}

 prog->defaults_fn = NULL;
 init_defaults_rec(&(prog->defaults));
 init_slice_roi_rec(&(prog->cut_slice_roi));
 init_slice_roi_rec(&(prog->delete_slice_roi));
 init_input_rec(&(prog->input),prog);
 init_all_win_rec(&(prog->win),prog);
}


/*--------------------------------------------------------------------------
free_prog_rec - ???
--------------------------------------------------------------------------*/

void
free_prog_rec(prog)
 struct prog_rec *prog;
{
 char *dir;

 if (is_slice_roi_non_empty(&(prog->cut_slice_roi)))
    {dir = get_prog_dir(prog);
     save_slice_roi_rec(dir,"breaker.check",&(prog->cut_slice_roi));
     c_free(dir);}

 c_free(prog->prog_id);
 c_free(prog->hdr_id);
 c_free(prog->dat_id);
 free_defaults_rec(&(prog->defaults));

 free_slice_roi_rec(&(prog->cut_slice_roi));
 free_slice_roi_rec(&(prog->delete_slice_roi));
 free_input_rec(&(prog->input));
 free_all_win_rec(&(prog->win));

 /* FIXME:  We don't know enough to set the second argument below
    properly, so we fake it for now.  -PWH 19990421 */
 init_prog_rec(prog, "nria");

}
