/****************************************************************************
set_image_background.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: set_image_background.c,v 1.12 1999/07/07 17:41:19 hughett Exp $ Release $Name: nria_1_0_6 $";


#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/wmgr.h>

#include "geometry.h"
#include "common.h"
#include "object.h"
#include "mri_format.h"
#include "nria_procedures.h"

/*
#define PRINT(x) printf(x); fflush(stdout);
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#define PRT1(x) x; fflush(stdout);
*/
#define PRINT(x)
#define TRACE(x)
#define PRT(x)
#define PRT1(x)



#define CLEAN_UP_IMAGE_BACKGROUND 0
#define SETUP_IMAGE_BACKGROUND 1
#define GET_ALL_IMAGE_BKG_PTR 2
#define CLOSE 10
#define OPEN 11

#define SET_IMAGE_BACKGROUND "SET IMAGE BACKGROUND"
struct bkg_rec
   {
    unsigned short status;
    struct image_display_window *win;
    struct display_win_rec *dwin;
    struct histo_params_rec *histo;
    Frame frame;
    Panel panel;
    Panel_item gradient,inc_bkg,how,
               x,image_type,whch,xy_gradient_ready,
               set_1st_echo,set_2nd_echo,how_to_segment;
    int last_mid_saved,max_range;
    char *echo1_str,*echo2_str;
   };

struct all_bkg_recs
   {
    int num_bkgs;
    struct bkg_rec **bkgs;
   };


/* Functions defined in this file */
void init_bkg_rec(struct bkg_rec *bkg);
void free_bkg_rec(struct bkg_rec *bkg);
void init_all_bkg_recs(struct all_bkg_recs *all_bkgs);
void default_proc(Panel_item item, Event *event);
void all_0_proc(Panel_item item, Event *event);
void set_only_0_proc(Panel_item item, Event *event);
int get_xy_gradient_roi(struct roi_rec *marker, double x, double y,
    double gradient, int left, int top, int right, int bottom);
int get_2d_bkg_roi_all_0s(struct bkg_rec *bkg,
    struct slice_roi_rec *slice_roi, unsigned long histo_dimx,
    unsigned long histo_dimy);
int get_1d_bkg_roi(struct bkg_rec *bkg, struct slice_roi_rec *slice_roi,
    unsigned long histo_dimx, unsigned long histo_dimy);
int get_bkg_roi(struct bkg_rec *bkg, int show_xy_gradient);
int bkg_ready_proc(Panel_item item, Event *event);
void cancel_proc(Panel_item item, Event *event);
void notify_xy_proc(Panel_item item, int value, Event *event);
void whch_image_type_proc(Panel_item item, int value, Event *event);
void whch_bkg_proc(Panel_item item, int value, Event *event);
void read_me_proc(Panel_item item, Event *event);
void bkg_done_proc(Frame frame);
void update_image_contrast(struct bkg_rec *bkg);
int setup_bkg_rec(struct bkg_rec *bkg, struct image_display_window *win);
struct bkg_rec *find_bkg_rec(struct all_bkg_recs *all_bkgs,
    struct image_display_window *win, struct display_win_rec *dwin);
struct bkg_rec *find_same_input_file_rec(struct all_bkg_recs *all_bkgs,
    struct image_display_window *win);
void setup_enter_bkg(struct image_display_window *win);



/* (Known) local functions */
static struct all_bkg_recs *enter_bkg(
    struct image_display_window *win, int status);


/*---------------------------------------------------------------------------
init_bkg_rec - ???
---------------------------------------------------------------------------*/

void
init_bkg_rec(bkg)
 struct bkg_rec *bkg;
{
 bkg->status = 0;
 bkg->win = NULL;
 bkg->frame = 0;
 bkg->last_mid_saved = 0;
 bkg->x = 0;
 bkg->image_type = 0;
 bkg->echo1_str = NULL;
 bkg->echo2_str = NULL;
}


/*---------------------------------------------------------------------------
free_bkg_rec - ???
---------------------------------------------------------------------------*/

void
free_bkg_rec(bkg)
 struct bkg_rec *bkg;
{
 struct all_bkg_recs *all_bkgs;

 if (bkg->frame != 0) xv_destroy_safe(bkg->frame);
 all_bkgs = enter_bkg(NULL,GET_ALL_IMAGE_BKG_PTR);
 c_free(bkg->echo1_str);
 c_free(bkg->echo2_str);
 c_free(bkg);
 DropHandleAryPtr(&(all_bkgs->bkgs),&(all_bkgs->num_bkgs),bkg);
}


/*---------------------------------------------------------------------------
init_all_bkg_recs - ???
---------------------------------------------------------------------------*/

void
init_all_bkg_recs(all_bkgs)
 struct all_bkg_recs *all_bkgs;
{
 all_bkgs->num_bkgs = 0;
 all_bkgs->bkgs = NULL;
}


/*---------------------------------------------------------------------------
default_proc - ???
---------------------------------------------------------------------------*/

void
default_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bkg_rec *bkg;
 int val;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);

 val = (int) xv_get(bkg->whch,PANEL_VALUE);

 xv_set(bkg->x,PANEL_VALUE,HISTO_START,0);
 notify_xy_proc(bkg->x,HISTO_START,NULL);
}


/*---------------------------------------------------------------------------
all_0_proc - ???
---------------------------------------------------------------------------*/

void
all_0_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bkg_rec *bkg;
 char str2[200];
 struct img_buf_rec *img;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 if ((img = win_rec_cur_img_buf(bkg->win->owner)) == NULL)
     return /*CANCEL*/;
 xv_set(bkg->x,PANEL_VALUE,0,0);
 sprintf(str2,"%s = 0",bkg->echo1_str);
 xv_set(bkg->whch,PANEL_CHOICE_STRING,0,str2,XV_KEY_DATA,0,0,0);

 if (img->file->hdr.data_info.num_echoes == 2)
    {
     sprintf(str2,"%s = 0",bkg->echo2_str);
     xv_set(bkg->whch,PANEL_CHOICE_STRING,1,str2,XV_KEY_DATA,1,0,0);
     sprintf(str2,"%s/%s = 0/0",bkg->echo1_str,bkg->echo2_str);
     xv_set(bkg->whch,PANEL_CHOICE_STRING,2,str2,
                  XV_KEY_DATA,2,0,XV_KEY_DATA,3,0,0);
    }
 if (bkg->last_mid_saved != 0)
    {bkg->last_mid_saved = 0;
     update_image_contrast(bkg);}
}


/*---------------------------------------------------------------------------
set_only_0_proc - ???
---------------------------------------------------------------------------*/

void
set_only_0_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bkg_rec *bkg;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 xv_set(bkg->set_1st_echo,PANEL_VALUE,0,0);
 xv_set(bkg->set_2nd_echo,PANEL_VALUE,0,0);
 all_0_proc(item,event);
}


/*---------------------------------------------------------------------------
get_xy_gradient_roi - ???
---------------------------------------------------------------------------*/

int
get_xy_gradient_roi(marker,x,y,gradient,left,top,right,bottom)
 struct roi_rec *marker;
 float x,y,gradient;
 int left,top,right,bottom;
{
 struct vector_3d_rec center,bounds[4],dire,and1,and2;
 float rad;
 unsigned short status;
 long x1,x2,y1,y2,centerx,centery;
 int err;

 setup_vector_3d_rec(&center,x,y,.0);
 rad = (float) atan2((double) gradient,(double) 1.0);
 TRACE(printf("rad = %f   x = %f  y = %f  gradient = %f\n",rad,x,y,gradient))
 setup_vector_3d_rec(&dire,(float) cos((double) rad),
           (float) sin((double) rad),.0);
 setup_vector_3d_rec(&(bounds[0]),(float) left,(float) top,.0);
 setup_vector_3d_rec(&(bounds[1]),(float) right,(float) top,.0);
 setup_vector_3d_rec(&(bounds[2]),(float) right,(float) bottom,.0);
 setup_vector_3d_rec(&(bounds[3]),(float) left,(float) bottom,.0);
 if (!plane_line_and_bound_intersection(bounds,&center,&dire,
                                   &and1,&and2,0))
    return CANCEL;
 TRACE(printf("and1..."); pr_vector_3d_rec(&and1))
 TRACE(printf("and2..."); pr_vector_3d_rec(&and2))
 x1 = ULongRound((double) and1.x);
 y1 = ULongRound((double) and1.y);
 x2 = ULongRound((double) and2.x);
 y2 = ULongRound((double) and2.y);
 centerx = ULongRound((double) center.x);
 centery = ULongRound((double) center.y);
 TRACE(printf("x1 = %d  y1 = %d  x2 =%d  y2 = %d...",x1,y1,x2,y2))
 if ((err = EnterPtsIntoRoiRec(marker,DRAW_LINE,&status,NULL,1,2,
         (float) x1,(float) y1,(float) x2,(float) y2))
         == NO_ERROR)
    err = EnterPtsIntoRoiRec(marker,DRAW_LINE,&status,NULL,1,5,
        (float) centerx,(float) (centery - 3),(float) centerx,
        (float) (centery + 3),(float) centerx,(float) centery,
        (float) (centerx - 3),(float) centery,
        (float) (centerx + 3),(float) centery);
 return err;
}


/*---------------------------------------------------------------------------
get_2d_bkg_roi_all_0s - ???
---------------------------------------------------------------------------*/

int
get_2d_bkg_roi_all_0s(bkg,slice_roi,histo_dimx,histo_dimy)
 struct bkg_rec *bkg;
 struct slice_roi_rec *slice_roi;
 unsigned long histo_dimx,histo_dimy;
{
 unsigned short status;
 struct roi_rec *roi,*roi3;
 int err;

 PRT(printf("get_2d_bkg_roi_all_0s \n"))
 if ((roi = get_roi(slice_roi,BACKGROUND_PIXEL_TYPE_STR)) != NULL)
    delete_roi(slice_roi,roi);
 bkg->histo->bkg.roi.status |= ROI_USER_MARKED;
 if ((roi3 = get_roi(slice_roi,GRADIENT)) != NULL)
    delete_roi(slice_roi,roi3);
 if ((err = init_roi_rec(&(bkg->histo->bkg.roi),
                      BACKGROUND_PIXEL_TYPE_STR)) == NO_ERROR)
    {if (xv_get(bkg->set_1st_echo,PANEL_VALUE) &&
         xv_get(bkg->set_2nd_echo,PANEL_VALUE))
        err = EnterPtsIntoRoiRec(&(bkg->histo->bkg.roi),
                   DRAW_POLYGON,&status,NULL,1,5,.0,.0,
                       (float) .0,(float) (histo_dimy - 1),.0,.0,
                       (float) (histo_dimx - 1),.0,.0,.0);
       else if (xv_get(bkg->set_1st_echo,PANEL_VALUE))
        err = EnterPtsIntoRoiRec(&(bkg->histo->bkg.roi),
                   DRAW_POLYGON,&status,NULL,1,3,.0,.0,
                    .0,(float) (histo_dimy - 1),.0,.0);
       else if (xv_get(bkg->set_2nd_echo,PANEL_VALUE))
        err = EnterPtsIntoRoiRec(&(bkg->histo->bkg.roi),
                   DRAW_POLYGON,&status,NULL,1,3,.0,.0,
                   (float) (histo_dimx - 1),.0,.0,.0);
       else
        err = EnterPtsIntoRoiRec(&(bkg->histo->bkg.roi),
                   DRAW_POLYGON,&status,NULL,1,2,.0,.0,.0,.0);
     if (err != NO_ERROR) return err;
     if ((err = JoinRoiToSliceRoiRec(slice_roi,
                 &(bkg->histo->bkg.roi),0,1.0,1.0,NULL,NULL,NULL)) != NO_ERROR)
        return err;
     if (xv_get(bkg->how_to_segment,PANEL_VALUE) == 0)
        if ((roi = get_roi(slice_roi,BACKGROUND_PIXEL_TYPE_STR)) != NULL)
           roi->status |= HISTO_EDITED_ROI;
    }
 win_histogram_show(bkg->win);
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
get_1d_bkg_roi - ???
---------------------------------------------------------------------------*/

int
get_1d_bkg_roi(bkg,slice_roi,histo_dimx,histo_dimy)
 struct bkg_rec *bkg;
 struct slice_roi_rec *slice_roi;
 unsigned long histo_dimx,histo_dimy;
{
 unsigned short status;
 struct roi_rec *roi,*roi2;
 int err;

 status = 0;
 PRT(printf("get_1d_bkg_roi 0\n"))
 if (((err = init_roi_rec(&(bkg->histo->bkg.roi),BACKGROUND_PIXEL_TYPE_STR))
        != NO_ERROR) ||
     ((err = EnterPtsIntoRoiRec(&(bkg->histo->bkg.roi),
           (unsigned long) DRAW_POLYGON,&status,NULL,1,5, .0,.0,
           .0,(float) (histo_dimy - 1),
           (float) bkg->histo->bkg.bkgs[0],(float) (histo_dimy - 1),
           (float) bkg->histo->bkg.bkgs[0],.0,.0,.0)) != NO_ERROR))
    {free_roi_rec(&(bkg->histo->bkg.roi));
     return err;}

 PRT(printf("get_1d_bkg_roi 1\n"))
 if ((roi = get_roi(slice_roi,BACKGROUND_PIXEL_TYPE_STR)) != NULL)
    delete_roi(slice_roi,roi);

 PRT(printf("get_1d_bkg_roi 2\n"))
 if ((err = JoinRoiToSliceRoiRec(slice_roi,
             &(bkg->histo->bkg.roi),0,1.0,1.0,NULL,NULL,NULL)) != NO_ERROR)
    return err;

 PRT(printf("get_1d_bkg_roi 3\n"))
 if ((roi = get_roi(slice_roi,BACKGROUND_PIXEL_TYPE_STR)) != NULL)
    {
     roi2 = next_not_cur_roi(slice_roi,roi->label);
     if (roi == roi2)
        setup_more_rois(slice_roi,SCRATCH_STR,1);
     if (xv_get(bkg->how_to_segment,PANEL_VALUE) == 0)
        roi->status |= HISTO_EDITED_ROI;
    }
 PRT(printf("get_1d_bkg_roi 4\n"))

 bkg->histo->bkg.roi.status |= ROI_USER_MARKED;
 win_histogram_show(bkg->win);
 PRT(printf("get_1d_bkg_roi 5\n"))
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
get_bkg_roi - ???
---------------------------------------------------------------------------*/

int
get_bkg_roi(bkg,show_xy_gradient)
 struct bkg_rec *bkg;
 int show_xy_gradient;
{
 struct img_buf_rec *img;
 unsigned int histo_dimx,histo_dimy,pixel_depth;
 struct buf_2d_rec _2d,*histo_2d;
 struct pixel_center_rec center;
 struct int_bound_rec affected;
 struct line_rec line,tangent;
 float x1,y1,x2,y2;
 int parallel,lcv;
 long iy1,ix2,iy2,num_pts;
 float xs[5],ys[5],*pxs,*pys;
 int err,how;
 struct roi_rec *roi,roi1,*roi2,*roi3,marker;
 struct slice_roi_win_rec *slice_roi_win;
 struct curve_rec **curves;
 unsigned int total_curves,num_curves;
 unsigned short status,histo_type2;

 if ((img = win_rec_cur_img_buf(bkg->win->owner)) == NULL) return CANCEL;
 err = NO_ERROR;
 free_roi_rec(&(bkg->histo->bkg.roi));
 histo_type2 = file_get_def_histo_type(img->file);
 get_histo_dim(bkg->histo,&histo_dimx,&histo_dimy,&pixel_depth,histo_type2);
 if ((slice_roi_win = (struct slice_roi_win_rec *)
             GetHistoSliceRoiWinRec(bkg->histo,histo_type2)) == NULL)
    return CANCEL;

 PRT(printf("get_bkg_roi num_bkgs = %d\n",bkg->histo->bkg.num_bkgs))
 PRT(for (lcv = 0; lcv < bkg->histo->bkg.num_bkgs; lcv++)
    printf("get_bkg_roi %d) bkg[] = %d\n",
          lcv,(long) bkg->histo->bkg.bkgs[lcv]))
 err = NO_ERROR;
 free_roi_rec(&(bkg->histo->bkg.roi));
 if (bkg->histo->bkg.num_bkgs == 1)
    return get_1d_bkg_roi(bkg,&(slice_roi_win->slice_roi),
                histo_dimx,histo_dimy);

 PRT(printf("get_bkg_roi ==========\n"))
 if ((bkg->histo->bkg.num_bkgs == 4) &&
     (bkg->histo->bkg.bkgs[0] == 0) && (bkg->histo->bkg.bkgs[1] == 0) &&
     (bkg->histo->bkg.bkgs[2] == 0) && (bkg->histo->bkg.bkgs[3] == 0))
    return get_2d_bkg_roi_all_0s(bkg,&(slice_roi_win->slice_roi),
                histo_dimx,histo_dimy);

 PRT(printf("get_bkg_roi...7\n"))
 if (((err = HistoBufReady(img->file,histo_type2)) != NO_ERROR) ||
     ((histo_2d = GetHistoBuf(bkg->histo,histo_type2)) == NULL))
    {ShowMsgStrs(bkg->win->dwin->frame,0,err,0,2,
               "Unable to get the histogram ready !!!",
               "Probably memory error !!!",NULL,0);
     return err;}

 if ((how = (int) xv_get(bkg->how,PANEL_VALUE)) == 0)
    {PRT(printf("overwrite...\n"))
     InitBuf2dRecData(histo_2d,(double) .0);
    }

 if (how != 2)
    {PRT(printf("changed...\n"))
     if (histo_type2 & HISTO_1D)   
        {if ((err = Get1dHistoBuf(histo_2d,img,NULL,1)) != NO_ERROR)
            return err;
        }
       else if (((err = all_img_echoes_exist(img)) != NO_ERROR) ||
                ((err = get_2d_histo_buf(histo_2d,img,NULL)) != NO_ERROR))
        return err;}

 if ((int) xv_get(bkg->inc_bkg,PANEL_VALUE))
    {PRT(printf(" ...no background...\n"))
     if (xv_get(bkg->set_1st_echo,PANEL_VALUE))
        Set2dByArea(histo_2d,0,0,bkg->histo->bkg.bkgs[0],histo_dimy - 1,
                 (double) .0);
     if (xv_get(bkg->set_2nd_echo,PANEL_VALUE))
        Set2dByArea(histo_2d,0,0,histo_dimx - 1,bkg->histo->bkg.bkgs[1],
                  (double) .0);
     Set2dByArea(histo_2d,0,0,bkg->histo->bkg.bkgs[2],
                    bkg->histo->bkg.bkgs[3],(double) .0);
     histo_2d->status4 &= ~BUF_MIN_MAX_COMPUTED;
     histo_2d->status4 &= ~BUF_MIN_MAX_SET;
    }

 init_pixel_center_rec(&center);
 setup_int_bound_rec(&affected,0,0,histo_2d->dimx - 1,histo_2d->dimy - 1);
 get_bkg_histo_stat(&center,histo_2d,NULL,&affected,
      (int) xv_get(bkg->gradient,PANEL_VALUE));

 init_buf_2d_rec(&_2d);
 PRT(printf("histo_dimx = %d histo_dimy = %d pixel_depth = %d\n",
        histo_dimx,histo_dimy,pixel_depth))

 if ((err = SetupBuf2dRec(&_2d,histo_dimx,histo_dimy,1,1))  != NO_ERROR)
    return err;

 PRT(printf("*****line*****avex = %d avey = %d gradient = %f\n",
       center.avex,center.avey,center.gradient))
 setup_line_rec_by_gradient(&line,(float) center.avex,(float) center.avey,
              center.gradient);
 
 PRT(printf("*****line*****\n"))
 PRT(pr_line_rec(&line))
 get_tangent_line(&tangent,(float) bkg->histo->bkg.bkgs[2],
             (float) bkg->histo->bkg.bkgs[3],&line);
 PRT(printf("*****tangent*****\n"))
 PRT(pr_line_rec(&tangent))

 setup_line_rec(&line,.0,.0,(float) (histo_dimx - 1),.0);
 get_lines_intersection(&tangent,&line,&x1,&y1,&parallel);
 PRT(printf("and x-axis x1 = %f y1 = %f*****parallel = %d\n",x1,y1,
               parallel))
 if (parallel)
    {
     iy1 = ULongRound((double) y1);
     PRT(printf("it is parallel...iy1 = %d\n",iy1))
     Set2dByArea(&_2d,0,0,histo_dimx - 1,iy1,(double) 1.0);
     histo_2d->status4 &= ~BUF_MIN_MAX_COMPUTED;
     histo_2d->status4 &= ~BUF_MIN_MAX_SET;
    }
   else
    {setup_line_rec(&line,.0,.0,.0,(float) (histo_dimy - 1));
     get_lines_intersection(&tangent,&line,&x2,&y2,&parallel);
     PRT(printf("and y-axis x2 = %f y2 = %f*****parallel = %d\n",x2,y2,
               parallel))
     if (parallel)
        {
         ix2 = ULongRound((double) x2);
         PRT(printf("it is parallel1...ix2 = %d\n",ix2))
         Set2dByArea(&_2d,0,0,ix2,histo_dimy - 1,(double) 1.0);
         histo_2d->status4 &= ~BUF_MIN_MAX_COMPUTED;
         histo_2d->status4 &= ~BUF_MIN_MAX_SET;
        }
       else
        {init_roi_rec(&roi1,NULL);
         xs[0] = ys[0] = ys[1] = .0;
         xs[1] = (float) ULongRound((double) x1);
         if (xs[1] >= histo_dimx)
            {setup_line_rec(&line,(float) (histo_dimx - 1),.0,
                        (float) (histo_dimx - 1),(float) (histo_dimy - 1));
             get_lines_intersection(&tangent,&line,&x1,&y1,&parallel);
             xs[1] = xs[2] = histo_dimx - 1;
             ys[2] = y1;
             num_pts = 3;
            }
           else num_pts = 2;
         iy2 = (int) ULongRound((double) y2);
         if (iy2 >= histo_dimy)
            {setup_line_rec(&line,.0,(float) (histo_dimy - 1),
                        (float) (histo_dimx - 1),(float) (histo_dimy - 1));
             get_lines_intersection(&tangent,&line,&x2,&y2,&parallel);
             xs[num_pts] = x2;
             ys[num_pts++] = histo_dimy - 1;
             xs[num_pts] = 0;
             ys[num_pts++] = histo_dimy - 1;
            }
           else 
            {xs[num_pts] = 0;
             ys[num_pts++] = iy2;}
         xs[num_pts] = 0;
         ys[num_pts++] = 0;

         PRT(for (lcv = 0; lcv < num_pts; lcv++)
                printf("%d) y = %f x = %f\n",lcv,ys[lcv],xs[lcv]);)
         pxs = xs;
         pys = ys;
         PRT(printf(" ..._2d = %d\n",CntNonValBuf2dRec(&_2d,(double) 0)))
         if ((err = EnterPtAryIntoRoiRec(&roi1,DRAW_POLYGON,
                     &status,NULL,1,num_pts,pxs,pys)) == NO_ERROR)
            {TRACE(printf(" FillROi...\n"))
             err = FillROi(&roi1,NULL,&_2d,1,1,0);}
         PRT(printf("after FillROi err = %d..._2d = %d\n",
                err,CntNonValBuf2dRec(&_2d,(double) 0)))

         free_roi_rec(&roi1);
        }
    }
 PRT(printf("get_bkg_roi...2...err = %d\n",err))
 if (err == NO_ERROR)
    {
     if (xv_get(bkg->set_1st_echo,PANEL_VALUE))
        {Set2dByArea(&_2d,0,0,bkg->histo->bkg.bkgs[0],histo_dimy - 1,
                      (double) 1.0);
         PRT(printf("after Set2dByArea set_1st_echo..._2d = %d\n",
                CntNonValBuf2dRec(&_2d,(double) 0)))}
     if (xv_get(bkg->set_2nd_echo,PANEL_VALUE))
        {Set2dByArea(&_2d,0,0,histo_dimx - 1,bkg->histo->bkg.bkgs[1],
                      (double) 1.0);
         PRT(printf("after Set2dByArea set_2nd_echo..._2d = %d\n",
                CntNonValBuf2dRec(&_2d,(double) 0)))}
     histo_2d->status4 &= ~BUF_MIN_MAX_COMPUTED;
     histo_2d->status4 &= ~BUF_MIN_MAX_SET;
/*
     PRT(
         init_roi_rec(&roi4,"JUNK");
         printf("get_bkg_roi...2...\n");
         process_buf_2d_rec_by_mask(&_2d,0,0,&_2d,
                 NULL,get_mask_coord_proc,&roi4,0,0,0,0,0,0,0,0);
         printf("get_bkg_roi...2......\n");
         if ((err = JoinRoiToSliceRoiRec(&(slice_roi_win->slice_roi),
               &roi4,0,1.0,1.0,NULL,NULL,NULL)) != NO_ERROR)
            return err;
         free_roi_rec(&roi4);)
*/
     if ((err = scan_objects(&_2d,NULL,
                  &curves,&total_curves,&num_curves,1,1)) == NO_ERROR)
        {status = 0;
         PRT(printf("get_bkg_roi...3...num_curves = %d\n",num_curves))
         if ((err = init_roi_rec(&(bkg->histo->bkg.roi),
                      BACKGROUND_PIXEL_TYPE_STR)) == NO_ERROR)
            for (lcv = 0; lcv < num_curves; lcv++)
               {
                if ((err = JoinCurvesToRoiRec(&(bkg->histo->bkg.roi),
                    &(curves[lcv]),1,&status,1.0,1.0,NULL,NULL,NULL)) != NO_ERROR)
                   break;
               }
         if (err != NO_ERROR) free_roi_rec(&(bkg->histo->bkg.roi));
        }

     free_buf_2d_rec(&_2d);
     PRT(printf("get_bkg_roi...9...err = %d\n",err))

     if (num_curves > 0)
        free_curve_recs(curves,num_curves);
     c_free(curves);
     if (err == NO_ERROR)
        {
         histo_type2 = file_get_def_histo_type(img->file);
         if ((slice_roi_win = (struct slice_roi_win_rec *)
                GetHistoSliceRoiWinRec(bkg->histo,histo_type2)) == NULL)
            return CANCEL;
         if ((roi = get_roi(&(slice_roi_win->slice_roi),
                   BACKGROUND_PIXEL_TYPE_STR)) != NULL)
            delete_roi(&(slice_roi_win->slice_roi),roi);
         if ((err = JoinRoiToSliceRoiRec(&(slice_roi_win->slice_roi),
               &(bkg->histo->bkg.roi),0,1.0,1.0,NULL,NULL,NULL)) != NO_ERROR)
            return err;
         PRT(printf("get_bkg_roi...5...err = %d\n",err))
         if ((roi3 = get_roi(&(slice_roi_win->slice_roi),
                       GRADIENT)) != NULL)
            delete_roi(&(slice_roi_win->slice_roi),roi3);
         if (show_xy_gradient)
            {init_roi_rec(&marker,GRADIENT);
             if (get_xy_gradient_roi(&marker, (float) center.avex,
                  (float) center.avey,center.gradient,0,0,
                  histo_dimx - 1,histo_dimy - 1) == NO_ERROR)
                {if (JoinRoiToSliceRoiRec(&(slice_roi_win->slice_roi),
                     &marker,0,1.0,1.0,NULL,NULL,NULL) == NO_ERROR)
                    if ((roi3 = get_roi(&(slice_roi_win->slice_roi),
                          GRADIENT)) != NULL)
                      {roi3->status |= ROI_MARKER;
                       PRT(printf("ROI_MARKER...\n"))}
                }
             free_roi_rec(&marker);
            }

         if ((roi = get_roi(&(slice_roi_win->slice_roi),
                   BACKGROUND_PIXEL_TYPE_STR)) != NULL)
            {if (xv_get(bkg->how_to_segment,PANEL_VALUE) == 0)
                roi->status |= HISTO_EDITED_ROI;
             roi2 = next_not_cur_roi(&(slice_roi_win->slice_roi),roi->label);
             if (roi == roi2)
                setup_more_rois(&(slice_roi_win->slice_roi),SCRATCH_STR,1);
            }
         bkg->histo->bkg.roi.status |= ROI_USER_MARKED;
         win_histogram_show_it(bkg->win);
        }
    }
   else free_buf_2d_rec(&_2d);
 PRT(printf("get_bkg_roi...10...err = %d\n",err))

 return err;
}


/*---------------------------------------------------------------------------
bkg_ready_proc - ???
---------------------------------------------------------------------------*/

int
bkg_ready_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bkg_rec *bkg;
 int err,show_xy_gradient;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 PRT(printf("bkg_ready_proc\n"));
 update_image_contrast(bkg);
 PRT(printf("bkg_ready_proc...update_image_contrast num_bkgs = %d\n",
        bkg->histo->bkg.num_bkgs));

 if (bkg->histo->bkg.num_bkgs > 0)
    {bkg->histo->bkg.bkgs[0] = (long) xv_get(bkg->whch,XV_KEY_DATA,0);
     if (bkg->histo->bkg.num_bkgs >= 4)
        {bkg->histo->file->output.status |= FILE_CHANGED;
         PRT(printf("bkg_ready_proc...1\n"))
         bkg->histo->bkg.bkgs[1] = (long) xv_get(bkg->whch,XV_KEY_DATA,1);
         bkg->histo->bkg.bkgs[2] = (long) xv_get(bkg->whch,XV_KEY_DATA,2);
         bkg->histo->bkg.bkgs[3] = (long) xv_get(bkg->whch,XV_KEY_DATA,3);
        }
     xv_set(bkg->frame,FRAME_BUSY,TRUE,0);
     if (item == bkg->xy_gradient_ready)
        show_xy_gradient = 1;
       else show_xy_gradient = 0;
     PRT(printf("get_bkg_roi...1\n"))
     err = get_bkg_roi(bkg,show_xy_gradient);
     PRT(printf("get_bkg_roi...out\n"))
     xv_set(bkg->frame,FRAME_BUSY,FALSE,0);
    }
 win_define_color_map(bkg->win,0);
 if (err != NO_ERROR)
    notice_prompt(bkg->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
               "An error occurred when...",
               "setting the image background !!!",
               0,
            NOTICE_BUTTON_YES," OK !",
            0);

 free_bkg_rec(bkg);
 return err;
}


/*---------------------------------------------------------------------------
cancel_proc - ???
---------------------------------------------------------------------------*/

void
cancel_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bkg_rec *bkg;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 win_define_color_map(bkg->win,0);
 free_bkg_rec(bkg);
}


/*---------------------------------------------------------------------------
notify_xy_proc - ???
---------------------------------------------------------------------------*/

void
notify_xy_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct bkg_rec *bkg;
 int val,posi;
 long num,num1;
 char str2[200];

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 val = (int) xv_get(bkg->whch,PANEL_VALUE);
 if (val == 0) posi = 0;
   else if (val == 1) posi = 1;
   else if (xv_get(bkg->image_type,PANEL_VALUE) == 0) posi = 2;
   else posi = 3;
 num = (long) xv_get(bkg->whch,XV_KEY_DATA,posi,0);

 if (num != value)
    {
     if ((posi == 2) || (posi == 3))
        {num = (long) xv_get(bkg->whch,XV_KEY_DATA,2);
         num1 = (long) xv_get(bkg->whch,XV_KEY_DATA,3);
         PRT1(printf("num1 = %d\n",num1))
         if (posi == 2) num = value;
           else num1 = value;
         PRT1(printf("now num1 = %d\n",num1))
         sprintf(str2,"%s/%s = %ld/%ld",
		 bkg->echo1_str,bkg->echo2_str,num,num1);
         PRT1(printf("str2 = %s\n",str2))
        }
       else if (posi == 0)
        {sprintf(str2,"%s = %d",bkg->echo1_str,value);}
       else if (posi == 1)
        {sprintf(str2,"%s = %d",bkg->echo2_str,value);}

     xv_set(bkg->whch,PANEL_CHOICE_STRING,val,str2,XV_KEY_DATA,posi,value,0);
     if (bkg->last_mid_saved != value)
        {bkg->last_mid_saved = value;
         update_image_contrast(bkg);}
    }
}


/*---------------------------------------------------------------------------
whch_image_type_proc - ???
---------------------------------------------------------------------------*/

void
whch_image_type_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 long num;
 struct bkg_rec *bkg;
 int val;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 val = xv_get(bkg->whch,PANEL_VALUE);
 if (val == 0)
    {if (value == 1) xv_set(item,PANEL_VALUE,0,0);
     return;}
   else if (val == 1)
    {if (value == 0) xv_set(item,PANEL_VALUE,1,0);
     return;}
 if (value == 0)
    num = (long) xv_get(bkg->whch,XV_KEY_DATA,2);
   else num = (long) xv_get(bkg->whch,XV_KEY_DATA,3);
 xv_set(bkg->x,PANEL_VALUE,num,0);
 if (bkg->last_mid_saved != num)
    {bkg->last_mid_saved = num;
     update_image_contrast(bkg);}
}


/*---------------------------------------------------------------------------
whch_bkg_proc - ???
---------------------------------------------------------------------------*/

void
whch_bkg_proc(item,value,event)
 Panel_item item;
 int value;
 Event *event;
{
 struct bkg_rec *bkg;
 long num;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 TRACE(printf("whch_bkg_proc value = %d\n",value))
 if (value == 0)
    {xv_set(bkg->image_type,PANEL_VALUE,0,0);
     num = (long) xv_get(item,XV_KEY_DATA,0,0);
     xv_set(bkg->x,PANEL_VALUE,num,0);
    }
   else if (value == 1)
    {xv_set(bkg->image_type,PANEL_VALUE,1,0);
     num = (long) xv_get(item,XV_KEY_DATA,1,0);
     xv_set(bkg->x,PANEL_VALUE,num,0);
    }
   else if (value == 2)
    {if (xv_get(bkg->image_type,PANEL_VALUE) == 0)
        num = (long) xv_get(item,XV_KEY_DATA,2,0);
       else num = (long) xv_get(item,XV_KEY_DATA,3,0);
     xv_set(bkg->x,PANEL_VALUE,num,0);
    }
 bkg->last_mid_saved = num;    
 update_image_contrast(bkg);
 TRACE(printf("whch_bkg_proc value = %d...end\n",value))
}


/*---------------------------------------------------------------------------
read_me_proc - ???
---------------------------------------------------------------------------*/

void
read_me_proc(item,event)
 Panel_item item;
 Event *event;
{
 struct bkg_rec *bkg;

 bkg = (struct bkg_rec *) xv_get(item,PANEL_CLIENT_DATA);
 notice_prompt(bkg->frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,
              "This message apply to the multiple slice echoes only !!!",
              "",
              "Once the background for the echoes is set, the pixel value ",
              "distribution of the current echoes will overwrite or be    ",
              "added to the histogram buffer or be ignored completely,    ",
              "depending on the option.  And based on the values in the   ",
              "buffer, the average X & Y plus the gradient are computed.  ",
              "By adding to the histogram, this means you can set your    ",
              "background based on more than 1 slice by setting the       ",
              "background several times.  By ignoring the current slice,  ",
              "you can set the background based on the pixels sampled.    ",
              NULL,
            NOTICE_BUTTON_YES,OK,
            0); 
}


/*---------------------------------------------------------------------------
bkg_done_proc - ???
---------------------------------------------------------------------------*/

void
bkg_done_proc(frame)
 Frame frame;
{
 struct bkg_rec *bkg;

 bkg = (struct bkg_rec *) xv_get(frame,WIN_CLIENT_DATA);
 win_define_color_map(bkg->win,0);
 free_bkg_rec(bkg);
}


/*---------------------------------------------------------------------------
update_image_contrast - ???
---------------------------------------------------------------------------*/

void
update_image_contrast(bkg)
 struct bkg_rec *bkg;
{
 int mid_saved,range_saved,val,lb,ub;
 struct color_scale_rec *scale;

 scale = win_cur_scale(bkg->win);
 mid_saved = scale->mid_value;
 range_saved = scale->range;
 lb = scale->lb;
 ub = scale->ub;
 val = bkg->last_mid_saved;

 val = (int) ULongRound(
            (double) ((scale->num_colors  - 1) * val) / 
            (double) bkg->max_range) + scale->first;
 scale->mid_value = val;
 scale->range = 0;
 scale->lb = 0;
 scale->ub = 0;

 PRT1(printf("update_image_contrast...mid_value = %d\n",val))
 win_define_color_map(bkg->win,0);

 scale->mid_value = mid_saved;
 scale->range = range_saved;
 scale->lb = lb;
 scale->ub = ub;
}


/*---------------------------------------------------------------------------
setup_bkg_rec - ???
---------------------------------------------------------------------------*/

int
setup_bkg_rec(bkg,win)
 struct bkg_rec *bkg;
 struct image_display_window *win;
{
 int row,offset,xval,yval,err,temp;
 struct img_buf_rec *img;
 char str1[300],*str,str3[200],str4[200];
 unsigned int histo_dimx,histo_dimy,pixel_depth;
 Panel_item item;
 Rect *rr;

 TRACE(printf("setup_bkg_rec...\n"))
 if (((img = win_rec_cur_img_buf(win->owner)) == NULL) ||
     (img->file->hdr.data_info.num_echoes > 2))
    {free_bkg_rec(bkg);
     return CANCEL;}
 bkg->frame = xv_create(win->dwin->frame, FRAME,
              WIN_CLIENT_DATA,bkg,
              FRAME_CLOSED,FALSE,
              0);
 if (bkg->frame == 0)
    {free_bkg_rec(bkg);
     return CANCEL;}
 bkg->win = win;
 bkg->dwin = win->dwin;
 bkg->histo = &(img->file->histo_params);
 str = win_rec_win_id_title(win->owner,SET_IMAGE_BACKGROUND);
 xv_set(bkg->frame,
              FRAME_SHOW_LABEL, TRUE,
              XV_LABEL,str,
              FRAME_NO_CONFIRM,TRUE,
              FRAME_DONE_PROC,bkg_done_proc,
              0);
 c_free(str);
 bkg->panel = xv_create(bkg->frame,PANEL,0);
 if (bkg->panel == 0)
    {free_bkg_rec(bkg);
     return CANCEL;}
 TRACE(printf("setup_bkg_rec...1\n"))

 row = 2;
 offset = xv_rows(bkg->panel,1) - xv_rows(bkg->panel,0);
 bkg->how_to_segment = xv_create(bkg->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"SEGMENT BACKGROUND AS",
        PANEL_CHOICE_STRINGS,"EDITED","PARTITIONED",NULL,
        XV_X,5,XV_Y,row,
        PANEL_VALUE,0,
        0);
 if (img->file->hdr.data_info.num_echoes == 1)
    xv_set(bkg->how_to_segment,PANEL_VALUE,1,0);
 row += offset-10;
 bkg->whch = xv_create(bkg->panel,PANEL_CHOICE,
                    PANEL_CLIENT_DATA,bkg,
                    PANEL_LAYOUT,PANEL_VERTICAL,
                    XV_X,5,XV_Y,row,
                    PANEL_NOTIFY_PROC,whch_bkg_proc,NULL);
 if (img->file->histo_params.bkg.num_bkgs == 0)
    if ((err = init_default_histo_bkgs(&(img->file->histo_params))) != NO_ERROR)
       {free_bkg_rec(bkg);
        return err;}

 sprintf(str3,"%s",img->file->hdr.analysis.img_labels[0]);
 set_str_len(str3,strlen_max(str3,20));
 cp_str(&(bkg->echo1_str),str3);
 sprintf(str1,"%s = %ld",bkg->echo1_str,img->file->histo_params.bkg.bkgs[0]);
 xv_set(bkg->whch,PANEL_CHOICE_STRING,0,str1,
                      XV_KEY_DATA,0,img->file->histo_params.bkg.bkgs[0],0);
 
 if (img->file->hdr.data_info.num_echoes == 2)
    {
     sprintf(str4,"%s",img->file->hdr.analysis.img_labels[1]);
     set_str_len(str4,strlen_max(str4,20));
     cp_str(&(bkg->echo2_str),str4);
     sprintf(str1,"%s = %ld",bkg->echo2_str,img->file->histo_params.bkg.bkgs[1]);
     xv_set(bkg->whch,PANEL_CHOICE_STRING,1,str1,
                      XV_KEY_DATA,1,img->file->histo_params.bkg.bkgs[1],0);

     sprintf(str1,"%s/%s = %ld/%ld",bkg->echo1_str,bkg->echo2_str,
          img->file->histo_params.bkg.bkgs[2],
          img->file->histo_params.bkg.bkgs[3]);
     xv_set(bkg->whch,PANEL_VALUE,2,
                      PANEL_CHOICE_STRING,2,str1,
                      XV_KEY_DATA,2,img->file->histo_params.bkg.bkgs[2],
                      XV_KEY_DATA,3,img->file->histo_params.bkg.bkgs[3],0);
     xval = img->file->histo_params.bkg.bkgs[2];
     yval = img->file->histo_params.bkg.bkgs[3];
     rr = (Rect *) xv_get(bkg->whch,XV_RECT);
     sprintf(str3,"SET X = %s",bkg->echo1_str);
     bkg->set_1st_echo = xv_create(bkg->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,str3,NULL,
        PANEL_VALUE,1,XV_X,rr->r_left + rr->r_width,XV_Y,row,
        PANEL_CLIENT_DATA,bkg,
        0);
     row += offset;
     sprintf(str4,"SET Y = %s",bkg->echo2_str);
     bkg->set_2nd_echo = xv_create(bkg->panel,PANEL_CHECK_BOX,
        PANEL_CHOICE_STRINGS,str4,NULL,
        PANEL_VALUE,1,XV_X,rr->r_left + rr->r_width,XV_Y,row - 7,
        PANEL_CLIENT_DATA,bkg,
        0);
    }
   else
    {xv_set(bkg->whch,PANEL_VALUE,0,0);
     xval = img->file->histo_params.bkg.bkgs[0];
    }
 TRACE(printf("setup_bkg_rec...bkg->whch = %d\n",
           xv_get(bkg->whch,PANEL_VALUE)))
 rr = (Rect *) xv_get(bkg->whch,XV_RECT);
 row = rr->r_top + rr->r_height + 2;
 str = "DEFAULT";
 xv_create(bkg->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,default_proc,
                   PANEL_CLIENT_DATA,bkg,
                   XV_X,5,XV_Y,row,
                   0);
 str = "ALL = 0";
 xv_create(bkg->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,all_0_proc,
                   PANEL_CLIENT_DATA,bkg,
                   XV_Y,row,
                   0);
 if (img->file->hdr.data_info.num_echoes == 2)
    {str = "[X,Y] = [0,0]";
     xv_create(bkg->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,set_only_0_proc,
                   PANEL_CLIENT_DATA,bkg,
                   XV_Y,row,
                   0);
    }
 str = "CANCEL";
 item = xv_create(bkg->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,cancel_proc,
                   PANEL_CLIENT_DATA,bkg,
                   0);

 if (img->file->hdr.data_info.num_echoes == 2)
    {row += offset-10;
     bkg->gradient = xv_create(bkg->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"REGRESSION GRADIENT =",
        PANEL_CHOICE_STRINGS,"GX","GY","(GX + GY) / 2",NULL,
        PANEL_VALUE,0,
        XV_X,5,XV_Y,row,
        0);
     row += offset-10;
     bkg->inc_bkg = xv_create(bkg->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"WITH BACKGROUND",
        PANEL_CHOICE_STRINGS,"INCLUDED","EXCLUDED",NULL,
        PANEL_VALUE,1,
        XV_X,5,XV_Y,row,
        0);

     row += offset-10;
     bkg->how = xv_create(bkg->panel,PANEL_CHOICE_STACK,
	PANEL_LABEL_STRING,"HISTOGRAM =",
        PANEL_CHOICE_STRINGS,"CUR SLICE HISTOGRAM",
                    "+ CUR SLICE HISTOGRAM",
                    "UNCHANGED",NULL,
        PANEL_VALUE,0,
        XV_X,5,XV_Y,row,
        0);
     row += offset-10;
     str = "READY-SHOW X,Y & GRADIENT";
     bkg->xy_gradient_ready = xv_create(bkg->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,bkg_ready_proc,
                   PANEL_CLIENT_DATA,bkg,
                   XV_X,5,XV_Y,row,
                   0);
     rr = (Rect *) xv_get(bkg->xy_gradient_ready,XV_RECT);
     temp = rr->r_left + rr->r_width + 5;
    }
   else
    {
     rr = (Rect *) xv_get(item,XV_RECT);
     temp = rr->r_left + rr->r_width + 5;
     bkg->xy_gradient_ready = 0;}
 str = "READY";
 xv_create(bkg->panel,PANEL_BUTTON,
 	           PANEL_LABEL_STRING, str,
                   PANEL_NOTIFY_PROC,bkg_ready_proc,
                   PANEL_CLIENT_DATA,bkg,
                   XV_X,temp,XV_Y,row,
                   0);

 get_histo_dim(bkg->histo,&histo_dimx,&histo_dimy,&pixel_depth,0);
 row += offset - 5;
 bkg->x = xv_create(bkg->panel,PANEL_SLIDER,
	PANEL_VALUE,xval,
	PANEL_MIN_VALUE,0,
	PANEL_MAX_VALUE,histo_dimx - 1,
	PANEL_SLIDER_WIDTH,130,
        PANEL_CLIENT_DATA,bkg,
	PANEL_NOTIFY_PROC,notify_xy_proc,
        XV_X,5,XV_Y,row,
	0);
 bkg->last_mid_saved = xval;
 bkg->max_range = histo_dimx - 1;
 TRACE(printf("setup_bkg_rec...bkg->whch &&&& = %d\n",
           xv_get(bkg->whch,PANEL_VALUE)))
 if (img->file->hdr.data_info.num_echoes > 1)
    {row += offset - 5;
     bkg->image_type = xv_create(bkg->panel,PANEL_CHOICE,
                    PANEL_CHOICE_STRING,0,str3,
                    PANEL_CHOICE_STRING,1,str4,
                    PANEL_CLIENT_DATA,bkg,
                    XV_X,5,XV_Y,row,
                    PANEL_LAYOUT,PANEL_HORIZONTAL,
                    PANEL_NOTIFY_PROC,whch_image_type_proc,NULL);
     xv_create(bkg->panel,PANEL_BUTTON,
                    PANEL_LABEL_STRING,"READ ME FIRST !!!",
                    PANEL_CLIENT_DATA,bkg,
                    PANEL_NOTIFY_PROC,read_me_proc,NULL);
    }
 window_fit(bkg->panel);
 rr = (Rect *) xv_get(bkg->panel,XV_RECT);
 xv_set(bkg->panel,XV_WIDTH,rr->r_width - 30,XV_HEIGHT,rr->r_height - 10,0);
 window_fit(bkg->frame);
 update_image_contrast(bkg);
 make_2_wins_adjacent(bkg->frame,win->dwin->frame,0);
 win_set_mouse_at_center(bkg->frame);
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
find_bkg_rec - ???
---------------------------------------------------------------------------*/

struct bkg_rec *
find_bkg_rec(all_bkgs,win,dwin)
 struct all_bkg_recs *all_bkgs;
 struct image_display_window *win;
 struct display_win_rec *dwin;
{
 int lcv;

 for (lcv = 0; lcv < all_bkgs->num_bkgs; lcv++)
    if ((all_bkgs->bkgs[lcv]->win == win) &&
        (all_bkgs->bkgs[lcv]->dwin == dwin))
       return all_bkgs->bkgs[lcv];
 return NULL;
}


/*---------------------------------------------------------------------------
find_same_input_file_rec - ???
---------------------------------------------------------------------------*/

struct bkg_rec *
find_same_input_file_rec(all_bkgs,win)
 struct all_bkg_recs *all_bkgs;
 struct image_display_window *win;
{
 int lcv;
 struct input_file_rec *f1;
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NULL;
 f1 = img->file;
 for (lcv = 0; lcv < all_bkgs->num_bkgs; lcv++)
    {if ((img = win_rec_cur_img_buf(all_bkgs->bkgs[lcv]->win->owner)) != NULL)
        if (img->file == f1)
           return all_bkgs->bkgs[lcv];
    }
 return NULL;
}


/*--------------------------------------------------------------------
setup_enter_bkg - ???

Wrapper for those callers who don't need to know about the internal
data structures.
--------------------------------------------------------------------*/

void 
setup_enter_bkg(
    struct image_display_window *win)
{
    enter_bkg(win, 1);
    return;
}


/*---------------------------------------------------------------------
enter_bkg - ???

BEWARE: The pre-19990223 version of this function could return either
NULL, a pointer to a structure, or an integer error code.  Since the
last two of these cannot be reliably distinguished and since no one
ever seems to check for an error code, I replaced the error code by
NULL.  Of course, it is now impossible to distinguish between NO_ERROR
and NULL (=> error), but I don't think anyone ever looks at the error
code returned.  -PWH
-----------------------------------------------------------------------*/

static struct all_bkg_recs *
enter_bkg(
    struct image_display_window *win,
    int status)
{
 static struct all_bkg_recs all_bkgs;
 static char first_time = 1;
 int err;
 struct bkg_rec *bkg;

 err = NO_ERROR;
 if (status == GET_ALL_IMAGE_BKG_PTR)
    {if (first_time) return NULL;
       else return &all_bkgs;}
  else if (status == CLEAN_UP_IMAGE_BACKGROUND)
   {if (!first_time)
       if ((bkg = find_bkg_rec(&all_bkgs,win,win->dwin)) != NULL)
          free_bkg_rec(bkg);
    return NO_ERROR;}

 if (first_time)
    {init_all_bkg_recs(&all_bkgs);
     first_time = 0;}
   else
    {if ((bkg = find_bkg_rec(&all_bkgs,win,win->dwin)) != NULL)
        {update_image_contrast(bkg);
         xv_set(bkg->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         win_set_mouse_at_center(bkg->frame);
         wmgr_top(bkg->frame);
         return NO_ERROR;}
     if ((bkg = find_same_input_file_rec(&all_bkgs,win)) != NULL)
        free_bkg_rec(bkg);
/*
        {bkg->win = win;
         bkg->dwin = win->dwin;
         update_image_contrast(bkg);
         xv_set(bkg->frame,XV_OWNER,win->dwin->frame,
              XV_SHOW,TRUE,FRAME_CLOSED,FALSE,WIN_ALARM,0);
         str = win_rec_win_id_title(win->owner,SET_IMAGE_BACKGROUND);
         xv_set(bkg->frame,XV_LABEL,str,0);
         c_free(str);
         wmgr_top(bkg->frame);
         win_set_mouse_at_center(bkg->frame);

         return NO_ERROR;
        }
*/
    }
 get_handle_memory(&(all_bkgs.bkgs),&(all_bkgs.num_bkgs),1,
     (size_t) sizeof(struct bkg_rec),&err);
 if (err != NO_ERROR) return NULL;    /* Was: return err */
 bkg = all_bkgs.bkgs[all_bkgs.num_bkgs - 1];
 init_bkg_rec(bkg);
 if ((err = setup_bkg_rec(bkg,win)) == NO_ERROR)
    xv_set(bkg->frame,XV_SHOW,TRUE,FRAME_CLOSED,FALSE,0);
 return NULL;    /* Was: return err */
}
