/****************************************************************************
slice_roi_rec_interface.c - Dealing with window slice_roi_rec and breaker???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

Dealing with window slice_roi_rec and breaker...
****************************************************************************/

static const char rcsid[] = "$Id: slice_roi_rec_interface.c,v 1.12 1999/07/07 17:41:19 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/cms.h>

#include "mri_format.h"
#include "common.h"
#include "object.h"
#include "nria_procedures.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT2(x) x; fflush(stdout);
#define PRT3(x) x; fflush(stdout);
#define PRT5(x) x; fflush(stdout);
#define PRT4(x) x; fflush(stdout);
#define PRT7(x) x; fflush(stdout);
#define PRT9(x) x; fflush(stdout);
#define PRT8(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT2(x)
#define PRT3(x)
#define PRT5(x)
#define PRT4(x)
#define PRT7(x)
#define PRT9(x)
#define PRT8(x)




/* Functions defined in this file */
int win_get_cur_slice_roi_rec(struct image_display_window *win,
    struct slice_roi_rec **slice_roi, char **cur_roi_label);
int win_is_non_empty_cur_curve(struct image_display_window *win);
int win_is_non_empty_roi(struct image_display_window *win, int label);
unsigned long win_num_cur_curve_pts(struct image_display_window *win);
struct roi_rec *win_get_roi(struct image_display_window *win, char *label);
int win_set_pt_status(struct image_display_window *win);
int win_get_cur_pt(struct image_display_window *win, long *x, long *y);
int win_get_cur_pt_by_indx(struct image_display_window *win,
    unsigned long indx, long *x, long *y);
int win_is_slice_roi_non_empty(struct image_display_window *win);
int win_is_slice_roi_no_roi(struct image_display_window *win);
int win_is_cur_roi_non_empty(struct image_display_window *win, int label);
int win_is_cur_curve_non_empty(struct image_display_window *win);
char *win_get_cur_roi_label(struct image_display_window *win);
int win_set_control_pen_type(struct image_display_window *win,
    int is_dp_label);
int win_set_pen_type(struct image_display_window *win, int use_this_pen_type,
    int pen_type);
int win_change_cur_curve_pen(struct image_display_window *win, int pen_type);
int win_set_roi_visibility(struct image_display_window *win, char *label,
    int visibility);
int win_set_all_rois_visibility(struct image_display_window *win,
    int visibility, int is_inc_cur);
int XyInCurve(struct curve_rec *curve, struct matrix_2d_rec *localto,
    int curve_type, struct draw_mask_rec *draw_mask, struct coord_rec *coords,
    int x, int y, int use_xybound, struct int_bound_rec *xybound, int *is_set,
    long *closestx, long *closesty);
int FindEnclosingPen(struct curve_2d_rec *pen, struct matrix_2d_rec *localto,
    struct draw_mask_rec *draw_mask, struct coord_rec *coords, int x, int y,
    int use_xybound, struct int_bound_rec *xybound, int *is_set,
    long *closestx, long *closesty);
int FindEnclosingRoi(struct roi_rec *roi, struct matrix_2d_rec *localto,
    struct draw_mask_rec *draw_mask, struct coord_rec *coords, int x, int y,
    int use_xybound, struct int_bound_rec *xybound, int *is_set,
    long *closestx, long *closesty);
int win_find_enclosing_curve(struct image_display_window *win, int x, int y,
    int use_this_xybound, struct int_bound_rec *xybound);
int win_search_enclosing_curve(struct image_display_window *win, int x, int y,
    int use_xybound, struct int_bound_rec *xybound);
void WinGetRoiLabelPtrs(struct image_display_window *win,
    struct strs_rec *roi_labels, int is_inc_empty_roi, int is_sort);
int FileGetAllRoiLabelPtrs(struct input_file_rec *file,
    struct strs_rec *roi_labels, int is_inc_empty_roi, int is_sort);
int win_get_all_pixel_type_ptrs(struct image_display_window *win,
    struct strs_rec *pixel_types);
int WinGetAllRoiLabelPtrs(struct image_display_window *win,
    struct strs_rec *roi_labels, int is_inc_empty_roi, int is_sorted);
int win_delete_cur_roi(struct image_display_window *win);
void win_delete_all_rois(struct image_display_window *win, int whch);
int win_delete_roi_by_label(struct image_display_window *win, char *roi_label);
int win_delete_file_rois_by_label(struct image_display_window *win,
    char *label);
int win_delete_all_file_rois(struct image_display_window *win);
int win_get_next_curve(struct image_display_window *win);
int WinDeleteMarkRoi(struct image_display_window *win, int is_dp_cur);
int win_next_curve(struct image_display_window *win);
int win_make_next_curve_cur(struct image_display_window *win, int is_backward);
int win_next_pen_type(struct image_display_window *win);
int win_next_defined_roi(struct image_display_window *win, int is_backward);
int win_make_roi_no_display(struct image_display_window *win, char *roi_label,
    int set_cur_roi);
int win_make_roi(struct image_display_window *win, char *roi_label,
    int set_cur_roi);
int WinMakeCurRoi(struct image_display_window *win, char *label);
int WinNextCurRoi(struct image_display_window *win, char *label);
int win_highlight_roi_cur(struct image_display_window *win, char *label);
int win_create_cur_roi_in_all_wins(struct image_display_window *win,
    int is_which);
int win_highlight_roi_in_all_wins(struct image_display_window *win,
    int is_which);
int win_join_curve_roi_rec(struct image_display_window *win, char *roi_label);
int win_is_roi_pen_type(struct image_display_window *win);
int win_loop_cur_curve(struct image_display_window *win, int must_loop);
int win_move_show_cur_curve(struct image_display_window *win);
int win_rename_cur_roi_label(struct image_display_window *win,
    char *roi_label);
int MakeRoiConvex(struct roi_rec *roi, struct matrix_2d_rec *localto,
    long dimx, long dimy);
int win_make_cur_roi_convex(struct image_display_window *win);
int win_bring_back_out_of_bound_rois(struct image_display_window *win);
int win_slice_rect_roi(struct image_display_window *win);



/***************************************************************************
   Functions dealing with window slice_roi_rec and breaker...
***************************************************************************/


/*-------------------------------------------------------------------------
win_get_cur_slice_roi_rec - ???
--------------------------------------------------------------------------*/

int
win_get_cur_slice_roi_rec(win,slice_roi,cur_roi_label)
 struct image_display_window *win;
 struct slice_roi_rec **slice_roi;
 char **cur_roi_label;
{
 struct curve_2d_rec *pen;

 if (win->control.display_options & DISPLAY_BREAKER)
    {
     *slice_roi = (struct slice_roi_rec *) win_rec_get_breaker(win->owner);
     if ((pen = get_cur_pen(*slice_roi)) != NULL)
        win->control.pen_type = pen->pen_type;
       else win->control.pen_type = DRAW_LINE;
     if (cur_roi_label != NULL)
        if ((*cur_roi_label = (char *) get_cur_roi_label(*slice_roi)) == NULL)
           *cur_roi_label = BREAKER_LABEL;
    }
   else
    {
     if (win->control.sroi.sroi_win == NULL) return CANCEL;
     *slice_roi = &(win->control.sroi.sroi_win->slice_roi);
     if (cur_roi_label != NULL)
        if ((*cur_roi_label = (char *) get_cur_roi_label(*slice_roi)) == NULL)
           *cur_roi_label = SCRATCH_STR;}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_is_non_empty_cur_curve - ???
--------------------------------------------------------------------------*/

int
win_is_non_empty_cur_curve(win)
 struct image_display_window *win;
{
 struct slice_roi_rec *slice_roi;
 int err;

 if ((err = win_get_cur_slice_roi_rec(win,&slice_roi,NULL)) != NO_ERROR)
    return err;
 return is_cur_curve_non_empty(slice_roi);
}


/*-------------------------------------------------------------------------
win_is_non_empty_roi - ???
--------------------------------------------------------------------------*/

int
win_is_non_empty_roi(win,label)
 struct image_display_window *win;
{
 struct slice_roi_rec *slice_roi;

 if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
    return 0;

 if (label == 0)
    return is_roi_non_empty(get_cur_roi(slice_roi));
   else
     return is_roi_non_empty(get_roi(slice_roi,label));
}


/*-------------------------------------------------------------------------
win_num_cur_curve_pts - ???
--------------------------------------------------------------------------*/

unsigned long 
win_num_cur_curve_pts(win)
 struct image_display_window *win;
{
 struct slice_roi_rec *slice_roi;
 unsigned long cnt;

 if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
    return 0;
 cnt = (unsigned long) num_curve_pts(get_cur_curve(slice_roi));
 return cnt;
}


/*-------------------------------------------------------------------------
win_get_roi - ???
--------------------------------------------------------------------------*/

struct roi_rec *
win_get_roi(win,label)
 struct image_display_window *win;
 char *label;
{
 struct slice_roi_rec *slice_roi;

 if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
    return NULL;
 /* FIXME:  This cast make me really nervous.  -PWH 19990427 */
 return (struct roi_rec *) get_roi(slice_roi,label);
}



/*-------------------------------------------------------------------------
win_set_pt_status - ???
--------------------------------------------------------------------------*/

int
win_set_pt_status(win)
 struct image_display_window *win;
{
 long x,y;
 struct slice_roi_rec *slice_roi;

 if (win_is_non_empty_cur_curve(win))
    {
     win->control.first_pt = EXPECT_SECOND_PT;
     if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
        return CANCEL;
     LongCurPt(slice_roi,&x,&y,win->control.sroi.localto);
     win->control.lastx = win->control.last_cursorx = x;
     win->control.lasty = win->control.last_cursory = y;
    }
   else
    {
     win->control.first_pt = EXPECT_FIRST_PT;
     win->control.lastx = win->control.last_cursorx = 0;
     win->control.lasty = win->control.last_cursory = 0;}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_get_cur_pt - ???
--------------------------------------------------------------------------*/

int
win_get_cur_pt(win,x,y)
 struct image_display_window *win;
 long *x,*y;
{
 struct slice_roi_rec *slice_roi;

 if (!win_is_non_empty_cur_curve(win)) return CANCEL;
 if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
    return CANCEL;
 LongCurPt(slice_roi,x,y,win->control.sroi.localto);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_get_cur_pt_by_indx - ???
--------------------------------------------------------------------------*/

int
win_get_cur_pt_by_indx(win,indx,x,y)
 struct image_display_window *win;
 unsigned long indx;
 long *x,*y; 
{
 struct slice_roi_rec *slice_roi;

 if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
    return CANCEL;
 return LongCurPtByIndx(slice_roi,indx,x,y,win->control.sroi.localto);
}


/*-------------------------------------------------------------------------
win_is_slice_roi_non_empty - ???
--------------------------------------------------------------------------*/

int
win_is_slice_roi_non_empty(win)
 struct image_display_window *win;
{
 if (win->control.sroi.sroi_win == NULL) return 0;
 return is_slice_roi_non_empty(&(win->control.sroi.sroi_win->slice_roi));
}


/*-------------------------------------------------------------------------
win_is_slice_roi_no_roi - ???
--------------------------------------------------------------------------*/

int
win_is_slice_roi_no_roi(win)
 struct image_display_window *win;
{
 if (win->control.sroi.sroi_win == NULL) return 0;
 return is_slice_roi_no_roi(&(win->control.sroi.sroi_win->slice_roi));
}



/***************************************************************************
   Functions dealing only with window slice_roi_rec...

FIXME: I'm not sure where this category is supposed to end.  -PWH 19990427
***************************************************************************/



/*-------------------------------------------------------------------------
win_is_cur_roi_non_empty - ???
--------------------------------------------------------------------------*/

int
win_is_cur_roi_non_empty(win,label)
 struct image_display_window *win;
{
 if (label == 0)
    return is_roi_non_empty(get_cur_roi(
              &(win->control.sroi.sroi_win->slice_roi)));
   else
    return is_roi_non_empty(
       get_roi(&(win->control.sroi.sroi_win->slice_roi),label));
}


/*-------------------------------------------------------------------------
win_is_cur_curve_non_empty - ???
--------------------------------------------------------------------------*/

int
win_is_cur_curve_non_empty(win)
 struct image_display_window *win;
{
 return is_cur_curve_non_empty(&(win->control.sroi.sroi_win->slice_roi));
}


/*-------------------------------------------------------------------------
win_get_cur_roi_label - ???
--------------------------------------------------------------------------*/

char *
win_get_cur_roi_label(win)
 struct image_display_window *win;
{
 if (win->control.sroi.sroi_win != NULL)
    return (char *) get_cur_roi_label(&(win->control.sroi.sroi_win->slice_roi));
   else return NULL;
}


/*-------------------------------------------------------------------------
win_set_control_pen_type - ???
--------------------------------------------------------------------------*/

int
win_set_control_pen_type(win,is_dp_label)
 struct image_display_window *win;
 int is_dp_label;
{
 struct curve_2d_rec *pen;
 int is_clear;

 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 if ((pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi))) == NULL)
    return CANCEL;
 PRT7(printf("win_set_control_pen_type pen_type = %x\n",
     (unsigned long) pen->pen_type))
 win->control.pen_type = pen->pen_type;
 win_set_pt_status(win);
 if (is_dp_label)
    {is_clear = 1;
     win_execute_proc_to_multiple_wins(win,win_draw_enabled_label,&is_clear,
				       0,0,0,0);}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_set_pen_type - ???
--------------------------------------------------------------------------*/

int
win_set_pen_type(win,use_this_pen_type,pen_type)
 struct image_display_window *win;
 int use_this_pen_type,pen_type;
{
 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 win_loop_cur_curve(win,0);

 if (use_this_pen_type)
    {if (win->control.display_options & SET_ALLOWABLE_PEN_TYPE)
        if (!(win->control.allowable_pen_type & pen_type))
           return NO_ERROR;
     win->control.pen_type = pen_type;}
 set_cur_pen(&(win->control.sroi.sroi_win->slice_roi),
                   win->control.pen_type,1);
 win_execute_proc_to_related_wins(win,win_set_control_pen_type,1,0);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_change_cur_curve_pen - ???
--------------------------------------------------------------------------*/

int
win_change_cur_curve_pen(win,pen_type)
 struct image_display_window *win;
 unsigned short pen_type;
{
 int err;
 struct roi_rec *roi;
 struct curve_2d_rec *pen;

 TRACE(printf("win_change_cur_curve_pen...\n"))
 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 if ((pen_type & DRAW_MODE_COMPATIBLE) &&
     (((pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi))) == NULL) ||
      (pen->pen_type & DRAW_MODE_COMPATIBLE)))
    {PRT7(printf("win_change_cur_curve_pen DRAW_MODE_COMPATIBLE...\n"))
     if ((err = change_cur_curve_pen_type(&(win->control.sroi.sroi_win->slice_roi),
           pen_type)) != NO_ERROR)
        return err;
     PRT7(printf("win_change_cur_curve_pen DRAW_MODE_COMPATIBLE...ok\n"))
    }
   else
    {LoopCurCurve(&(win->control.sroi.sroi_win->slice_roi));
     if ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) == NULL)
        if ((err = setup_more_rois(&(win->control.sroi.sroi_win->slice_roi),
                        SCRATCH_STR,1)) != NO_ERROR)
           return err;
     if ((err = set_cur_pen(&(win->control.sroi.sroi_win->slice_roi),
                     pen_type,1)) == NO_ERROR)
        {
         err = next_curve(&(win->control.sroi.sroi_win->slice_roi),1);
         win_display_cur_roi(win);
         PRT7(printf("win_change_cur_curve_pen win_display_cur_roi...\n"))
        }
    }
 if (err == NO_ERROR)
    win_execute_proc_to_related_wins(win,win_set_control_pen_type,1,0);
 TRACE(printf("win_change_cur_curve_pen(...end\n"))

 return err;
}


/*-------------------------------------------------------------------------
win_set_roi_visibility - ???
--------------------------------------------------------------------------*/

int
win_set_roi_visibility(win,label,visibility)
 struct image_display_window *win;
 char *label;
 unsigned short visibility;
{
 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 set_roi_visibility(get_roi(&(win->control.sroi.sroi_win->slice_roi),label),
                    visibility);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_set_all_rois_visibility - ???
--------------------------------------------------------------------------*/

int
win_set_all_rois_visibility(win,visibility,is_inc_cur)
 struct image_display_window *win;
 unsigned short visibility;
 int is_inc_cur;
{
 struct roi_rec *roi;

 if (win->control.sroi.sroi_win == NULL) return CANCEL;

 set_all_rois_visibility(&(win->control.sroi.sroi_win->slice_roi),visibility);
 if (!is_inc_cur)
    if ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi)))
             != NULL)
       set_roi_visibility(roi,(unsigned short) ROI_VISIBLE);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
XyInCurve - ???
--------------------------------------------------------------------------*/

int
XyInCurve(curve,localto,curve_type,draw_mask,coords,x,y,use_xybound,xybound,
            is_set,closestx,closesty)
 struct curve_rec *curve;
 struct matrix_2d_rec *localto;
 unsigned short curve_type;
 struct draw_mask_rec *draw_mask;
 struct coord_rec *coords;
 int x,y;
 int use_xybound;
 struct int_bound_rec *xybound;
 int *is_set;
 long *closestx,*closesty;
{
 long lcv,cnt;
 long minx,maxx,miny,maxy;
 long xdiv8,xmod8;
 unsigned long total_pts_drawn;
 long x1,y1,temp,temp1,left,right,top,bottom;
 int found;
 struct int_bound_rec bound;

 PRT8(printf("XyInCurve...1..curve->status = %x use_xybound = %d\n",
       (unsigned long) curve->status,use_xybound))
 if (curve->num_xys == 0) return 0;
 if (curve->status & COORDINATE_BOUND_COMPUTED)
    {
     if (FindCurveLongMinMax(curve,localto,&minx,&maxx,&miny,&maxy,
                  curve_type,0,0,0) != NO_ERROR)
        {PRT3(printf("XyInCurve...2 != NO_ERROR\n"))
         return 0;}
     PRT3(printf("...x = %d min = %d max = %d\n",x,minx,maxx))
     PRT3(printf("...y = %d min = %d max = %d\n",y,miny,maxy))
     if (use_xybound)
        {if ((xybound->right < minx) || (xybound->left > maxx) ||
             (xybound->bottom < miny) || (xybound->top > maxy))
         return 0;
        }
       else if ((x < minx) || (x > maxx) ||
                (y < miny) || (y > maxy))
        return 0;
    }
 if (coords->num_pts > 0)
    set_buf_by_coord_rec(&(draw_mask->mask._2d),coords,0);
   else clear_draw_masks(draw_mask,1);
 coords->num_pts = 0;
 DrawInBuf(draw_mask->mask._2d.buf,0,0,
          draw_mask->mask._2d.dimx - 1,draw_mask->mask._2d.dimy - 1,
          draw_mask->mask._2d.pixel_depth,curve->xys,
          (unsigned long) curve->num_xys,
          (unsigned int) curve_type,&(curve->matrix),
          (int) (curve->status & CURVE_MATRIX_IN_USE),
          (long) 1,coords,&total_pts_drawn,localto);
 PRT8(printf("XyInCurve...total_pts_drawn = %d\n",total_pts_drawn))
 if (total_pts_drawn > 0) draw_mask->paint_status |= PAINTED;
 xdiv8 = x / 8;
 xmod8 = x % 8;
 if (draw_mask->mask._2d.buf[y][xdiv8] & C_which_bit[xmod8])
    {*is_set = 1;
     *closestx = 0;
     *closesty = 0;
     return 1;}
 if (use_xybound)
    {setup_int_bound_rec(&bound,0,0,draw_mask->mask._2d.dimx - 1,
                     draw_mask->mask._2d.dimy - 1);
     valid_bound(xybound,&bound,&left,&right,&top,&bottom);
     lcv = right;
     for (found = 0; lcv >= 0; lcv--)
        {xdiv8 = lcv / 8;
         xmod8 = lcv % 8;
         for (cnt = top; cnt <= bottom; cnt++)
            if (draw_mask->mask._2d.buf[cnt][xdiv8] & C_which_bit[xmod8])
               {found = 1;
                break;}
         if (found) break;
        }
     if (lcv < 0) return 0;
     PRT3(printf("=xy_in_..lcv = %d\n",lcv))
     x1 = x - lcv;
     temp1 = cnt - top;
     x1 = x1 * x1 + temp1 * temp1;
     x1 = (long) sqrt((double) x1);
     lcv = left;
     PRT3(printf("=XyInCurve...3..lcv = %d\n",lcv))
     for (found = 0; ;lcv++)
        {if (lcv >= draw_mask->mask._2d.dimx)
            {PRT3(printf("=return...3..lcv = %d top = %d bot = %d\n",lcv,
                 top,bottom))
             return 0;}
         xdiv8 = lcv / 8;
         xmod8 = lcv % 8;
         for (cnt = top; cnt <= bottom; cnt++)
            if (draw_mask->mask._2d.buf[cnt][xdiv8] & C_which_bit[xmod8])
               {found = 1;
                break;}
         if (found) break;
        }
     temp = lcv - x;
     temp1 = cnt - top;
     temp = temp * temp + temp1 * temp1;
     temp = (long) sqrt((double) temp);
     if (temp < x1) x1 = temp;
     PRT3(printf("=XyInCurve...4\n"))
     lcv = bottom;
     for (; lcv >= 0; lcv--)
        {for (cnt = left; cnt <= right; cnt++)
            if (draw_mask->mask._2d.buf[lcv][cnt / 8] & C_which_bit[cnt % 8])
               {found = 1;
                break;}
         if (found) break;}
     if (lcv < 0) return 0;
     y1 = y - lcv;
     temp1 = cnt - left;
     y1 = y1 * y1 + temp1 * temp1;
     y1 = (long) sqrt((double) y1);

     lcv = top;
     PRT3(printf("=XyInCurve...lcv = %d\n",lcv))
     for (found = 0;lcv < draw_mask->mask._2d.dimy;lcv++)
        {for (cnt = left; cnt <= right; cnt++)
            if (draw_mask->mask._2d.buf[lcv][cnt / 8] & C_which_bit[cnt % 8])
               {found = 1;
                break;}
         if (found) break;}
     if (lcv >= draw_mask->mask._2d.dimy) return 0;
     temp = lcv - y;
     temp1 = cnt - left;
     temp = temp * temp + temp1 * temp1;
     temp = (long) sqrt((double) temp);
     if (temp < y1) y1 = temp;

     PRT3(printf("=XyInCurve...5\n"))
    }
   else
    {lcv = x - 1;
     for (; lcv >= 0; lcv--)
        if (draw_mask->mask._2d.buf[y][lcv / 8] & C_which_bit[lcv % 8]) break;
     if (lcv < 0) return 0;
     x1 = x - lcv;
     if (x1 < 0) x1 = -x1;
     PRT3(printf("XyInCurve...3\n"))
     lcv = x + 1;
     for (; ;lcv++)
        {if (lcv >= draw_mask->mask._2d.dimx) return 0;
         if (draw_mask->mask._2d.buf[y][lcv / 8] & C_which_bit[lcv % 8]) break;      
        }
     temp = lcv - x;
     if (temp < 0) temp = -temp;
     if (temp < x1) x1 = temp;
     PRT3(printf("XyInCurve...4\n"))
     lcv = y - 1;
     for (; lcv >= 0; lcv--)
        if (draw_mask->mask._2d.buf[lcv][xdiv8] & C_which_bit[xmod8]) break;
     if (lcv < 0) return 0;
     y1 = y - lcv;
     if (y1 < 0) y1 = -y1;
     lcv = y + 1;
     for (;lcv < draw_mask->mask._2d.dimy;lcv++)
        if (draw_mask->mask._2d.buf[lcv][xdiv8] & C_which_bit[xmod8]) break;
     if (lcv >= draw_mask->mask._2d.dimy) return 0;
     temp = lcv - y;
     if (temp < 0) temp = -temp;
     if (temp < y1) y1 = temp;
     PRT3(printf("XyInCurve...5\n"))
    }

 if (*is_set == 0)
    {*closestx = x1;
     *closesty = y1;}
   else
    {if ((x1 < *closestx) || (y1 < *closesty))
        {*closestx = x1;
         *closesty = y1;}
       else return 0;
    }
 *is_set = 1;
 PRT3(printf("XyInCurve...6...1\n"))

 return 1;
}


/*-------------------------------------------------------------------------
FindEnclosingPen - ???
--------------------------------------------------------------------------*/

int
FindEnclosingPen(pen,localto,draw_mask,coords,x,y,use_xybound,xybound,
                 is_set,closestx,closesty)
 struct curve_2d_rec *pen;
 struct matrix_2d_rec *localto;
 struct draw_mask_rec *draw_mask;
 struct coord_rec *coords;
 int x,y;
 int use_xybound;
 struct int_bound_rec *xybound;
 int *is_set;
 long *closestx,*closesty;
{
 unsigned long lcv;
 int found;
 unsigned long cur_curve;

 PRT8(printf("FindEnclosingPen...num_curves = %d\n",pen->num_curves))
 if (pen->num_curves == 0) return 0;
 if (pen->cur_curve >= pen->num_curves) pen->cur_curve = 0;
 cur_curve = pen->cur_curve;
 for (found = 0,lcv = (cur_curve + 1) % pen->num_curves; ;
             lcv = (lcv + 1) % pen->num_curves)
    {
     if (XyInCurve(pen->curves[lcv],localto,pen->pen_type,draw_mask,coords,x,y,
                 use_xybound,xybound,is_set,closestx,closesty))
        {
         pen->cur_curve = lcv;
         if ((*closestx == 0) && (*closesty == 0)) return 1;
         found = 1;
        }
     if (lcv == cur_curve) break;}

 return found;
}


/*-------------------------------------------------------------------------
FindEnclosingRoi - ???
--------------------------------------------------------------------------*/

int
FindEnclosingRoi(roi,localto,draw_mask,coords,x,y,use_xybound,xybound,
                 is_set,closestx,closesty)
 struct roi_rec *roi;
 struct matrix_2d_rec *localto;
 struct draw_mask_rec *draw_mask;
 struct coord_rec *coords;
 int x,y;
 int use_xybound;
 struct int_bound_rec *xybound;
 int *is_set;
 long *closestx,*closesty;
{
 unsigned long lcv;
 int found;
 unsigned long cur_pen;

 if (roi->num_pens == 0) return 0;
 if (roi->cur_pen >= roi->num_pens) roi->cur_pen = 0;
 cur_pen = roi->cur_pen;
 for (found = 0,lcv = (cur_pen + 1) % roi->num_pens; ;
             lcv = (lcv + 1) % roi->num_pens)
    {
     if (FindEnclosingPen(roi->pens[lcv],localto,draw_mask,coords,x,y,
             use_xybound,xybound,is_set,closestx,closesty))
        {
         roi->cur_pen = lcv;
         if ((*closestx == 0) && (*closesty == 0)) return 1;
         found = 1;
        }
     if (lcv == cur_pen) break;}

 return found;
}



/*-------------------------------------------------------------------------
win_find_enclosing_curve - ???

FIXME! DRW 970309  crashes in delete ROI cursor mode when user selects
outside any regions.
--------------------------------------------------------------------------*/

int
win_find_enclosing_curve( win, x, y, use_this_xybound, xybound )
     struct image_display_window *win;
     int x,y;
     int use_this_xybound;
     struct int_bound_rec *xybound;
{
  unsigned long lcv;
  struct slice_roi_rec *slice_roi;
  struct curve_rec *cur_curve;
  struct curve_2d_rec *pen;
  int is_set;
  struct coord_rec coords;
  long closestx,closesty;
  unsigned long cur_roi_indx,cur_pen_indx,cur_curve_indx;

  PRT8(printf("win_find_enclosing_curve...x = %d y = %d %d %d\n",x,y,
	      win->data.draw_masks[DRAW_MASK_CUR_CURVE].mask._2d.dimx,
	      win->data.draw_masks[DRAW_MASK_CUR_CURVE].mask._2d.dimy));
  if ((x < 0) || (y < 0) ||
      (x >= win->data.draw_masks[DRAW_MASK_CUR_CURVE].mask._2d.dimx) ||
      (y >= win->data.draw_masks[DRAW_MASK_CUR_CURVE].mask._2d.dimy))
    return 0;

  slice_roi = &(win->control.sroi.sroi_win->slice_roi);
  if (slice_roi->num_rois == 0)
    return 0;
  if (slice_roi->cur_roi >= slice_roi->num_rois)
    slice_roi->cur_roi = 0;
  if ((pen = (struct curve_2d_rec *) get_cur_pen(slice_roi)) != NULL)
    cur_curve_indx = pen->cur_curve;
  else
    cur_curve_indx = 0;

  cur_curve = (struct curve_rec *) get_cur_curve(slice_roi);
  init_coord_rec(&coords);
  cur_roi_indx = slice_roi->cur_roi;
  if (slice_roi->rois[cur_roi_indx] != NULL)
    cur_pen_indx = slice_roi->rois[cur_roi_indx]->cur_pen;
  else
    cur_pen_indx = 0;

  for (is_set = 0, lcv = (cur_roi_indx + 1) % slice_roi->num_rois;
       ;
       lcv = (lcv + 1) % slice_roi->num_rois)
  {
    PRT8(printf("[%d]->status = %x\n",lcv,(unsigned long) slice_roi->rois[lcv]->status));
    if (slice_roi->rois[lcv]->status & ROI_VISIBLE)
      if (FindEnclosingRoi(slice_roi->rois[lcv],win->control.sroi.localto,
			   &(win->data.draw_masks[DRAW_MASK_CUR_CURVE]),&coords,x,y,
			   use_this_xybound,xybound,&is_set,&closestx,&closesty))
      {
	slice_roi->cur_roi = lcv;
	if ((closestx == 0) && (closesty == 0))
	{
	  break;
	}
      }
    if (lcv == cur_roi_indx)
      break;
  }
  free_coord_rec(&coords);
  PRT8(printf("win_find_enclosing_curve...1\n"));
  if (is_set == 1)
  {
    CloseCurCurve(pen,&(slice_roi->status),1);
    win_execute_proc_to_related_wins(win,win_set_control_pen_type,1,0);
    PRT8(printf("win_find_enclosing_curve...3\n"));
    win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
    return 1;
  }

  PRT8(printf("win_find_enclosing_curve...2\n"));
  if (cur_curve != NULL)
  {
    slice_roi->cur_roi = cur_roi_indx;
    slice_roi->rois[cur_roi_indx]->cur_pen = cur_pen_indx;
    pen->cur_curve = cur_curve_indx;
    if ((cur_curve->num_xys > 0) &&
	(win->data.draw_masks[DRAW_MASK_CUR_CURVE].paint_status & PAINTED))
    {
      PRT8(printf("win_find_enclosing_curve...4\n"));
      win_draw_cur_curve_rec(win);
    }
  }
  PRT8(printf("win_find_enclosing_curve...end\n"));
  return 0;
}



/*-------------------------------------------------------------------------
win_search_enclosing_curve - ???
--------------------------------------------------------------------------*/

int
win_search_enclosing_curve(win,x,y,use_xybound,xybound)
 struct image_display_window *win;
 int x,y;
 int use_xybound;
 struct int_bound_rec *xybound;
{
 int found;

 if ((found = win_find_enclosing_curve(win,x,y,use_xybound,xybound)) == 1)
    {
     if (win->type2 & DISPLAY_WINDOW_3D)
        _3d_after_next_roi_proc(win);
    }
 return found;
}


/*-------------------------------------------------------------------------
WinGetRoiLabelPtrs - ???
--------------------------------------------------------------------------*/

void
WinGetRoiLabelPtrs(win,roi_labels,is_inc_empty_roi,is_sort)
 struct image_display_window *win;
 struct strs_rec *roi_labels;
 int is_inc_empty_roi,is_sort;
{
 init_strs_rec(roi_labels);
 if (win->control.sroi.sroi_win != NULL) 
    GetRoiLabelPtrs(&(win->control.sroi.sroi_win->slice_roi),roi_labels,
           is_inc_empty_roi,is_sort);
}


/*-------------------------------------------------------------------------
FileGetAllRoiLabelPtrs - ???
--------------------------------------------------------------------------*/

int
FileGetAllRoiLabelPtrs(file,roi_labels,is_inc_empty_roi,is_sort)
 struct input_file_rec *file;
 struct strs_rec *roi_labels;
 int is_inc_empty_roi,is_sort;
{
 unsigned long lcv;
 int err;

 init_strs_rec(roi_labels);
 for (lcv = 0; lcv < file->slice_info.num_sroi_wins; lcv++)
    if ((err = GetRoiLabelPtrs(
           &(file->slice_info.sroi_wins[lcv]->slice_roi),roi_labels,
             is_inc_empty_roi,is_sort)) != NO_ERROR)
       return err;
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_get_all_pixel_type_ptrs - ???
--------------------------------------------------------------------------*/

int
win_get_all_pixel_type_ptrs(win,pixel_types)
 struct image_display_window *win;
 struct strs_rec *pixel_types;
{
 struct img_buf_rec *img;

 init_strs_rec(pixel_types);
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NO_ERROR;
 return cp_strs_rec_ptrs(pixel_types,&(img->file->pixel_type.order));
}


/*-------------------------------------------------------------------------
WinGetAllRoiLabelPtrs - ???
--------------------------------------------------------------------------*/

int
WinGetAllRoiLabelPtrs(win,roi_labels,is_inc_empty_roi,is_sorted)
 struct image_display_window *win;
 struct strs_rec *roi_labels;
 int is_inc_empty_roi,is_sorted;
{
 struct img_buf_rec *img;

 if (win->type2 & HISTOGRAM_WINDOW) 
    return win_get_all_pixel_type_ptrs(win,roi_labels);
   else if ((img = win_rec_cur_img_buf(win->owner)) == NULL)
     {init_strs_rec(roi_labels);
      return NO_ERROR;}
   else
     return FileGetAllRoiLabelPtrs(img->file,roi_labels,
                is_inc_empty_roi,is_sorted);
}


/*-------------------------------------------------------------------------
win_delete_cur_roi - ???
--------------------------------------------------------------------------*/

int
win_delete_cur_roi(win)
 struct image_display_window *win;
{
 struct slice_roi_rec *slice_roi;

 if (win->control.display_options & DISPLAY_BREAKER)
    win_delete_breaker(win);
   else
    {if (!win_is_non_empty_roi(win,0)) return CANCEL;
     if (win_get_cur_slice_roi_rec(win,&slice_roi,NULL) != NO_ERROR)
        return CANCEL;
     delete_cur_roi(slice_roi);
     init_input_pt_state(&(win->control));

     win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
    }

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_delete_all_rois - ???

FIXME:  A switch statement with only one case seems a bit odd.
       -PWH 19990427
--------------------------------------------------------------------------*/

void
win_delete_all_rois(win,whch)
 struct image_display_window *win;
 int whch;
{
 switch (whch)
    {
     case (int) DELETE_EVERY_ROI :
        if (!win_is_slice_roi_non_empty(win))
	    return /*CANCEL*/;
        if (win_is_slice_roi_non_empty(win))
            delete_all_rois(&(win->control.sroi.sroi_win->slice_roi));
    }
 win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
}


/*-------------------------------------------------------------------------
win_delete_roi_by_label - ???
--------------------------------------------------------------------------*/

int
win_delete_roi_by_label(win,roi_label)
 struct image_display_window *win;
 char *roi_label;
{
 struct roi_rec *roi;

 if ((roi = get_roi(&(win->control.sroi.sroi_win->slice_roi),
                       roi_label)) != NULL) 
    {delete_roi(&(win->control.sroi.sroi_win->slice_roi),roi);
     win_run_proc_on_slice_roi_wins(win->control.sroi.sroi_win,
				    win_redisplay_all_curves,0,0);
     return NO_ERROR;
    }
 return CANCEL;
}


/*-------------------------------------------------------------------------
win_delete_file_rois_by_label - ???
--------------------------------------------------------------------------*/

int
win_delete_file_rois_by_label(win,label)
 struct image_display_window *win;
 char *label;
{
 struct img_buf_rec *img;
 struct roi_rec *roi;
 unsigned long lcv;
 struct slice_roi_win_rec *slice_roi_win;
 struct slice_roi_rec *slice_roi;
 unsigned short histo_type2;
 int yes;

 if ((img = (struct img_buf_rec *) win_rec_cur_img_buf(win->owner)) == NULL) 
    return NO_ERROR;
 if (win->type2 & HISTOGRAM_WINDOW)
    {if (win->type2 & HISTOGRAM_1D_WINDOWS)
        histo_type2 = HISTO_1D;
       else histo_type2 = HISTO_MULTI_D;
     slice_roi_win = GetHistoSliceRoiWinRec(
               &(img->file->histo_params),histo_type2);
     if ((roi = get_roi(&(slice_roi_win->slice_roi),label)) == NULL)
        return NO_ERROR;
     delete_roi(&(slice_roi_win->slice_roi),roi);
     win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,2,0,0,0);}
   else
    {slice_roi = &(img->file->slice_info.sroi_wins[lcv]->slice_roi);
     for (yes = 0,lcv = 0; lcv <img->file->slice_info.max_sroi_wins; lcv++)
        if ((roi = get_roi(slice_roi,label))
            != NULL)
           {yes = 1;
            delete_roi(slice_roi,roi);
           }
     if (yes)
	 win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,1,0,0,0);}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_delete_all_file_rois - ???
--------------------------------------------------------------------------*/

int
win_delete_all_file_rois(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;
 long lcv;
 struct slice_roi_win_rec *slice_roi_win;
 unsigned short histo_type2;

 PRT9(printf("win_delete_all_file_rois...\n"))
 if ((img = (struct img_buf_rec *) win_rec_cur_img_buf(win->owner)) == NULL) 
    return NO_ERROR;
 if (win->type2 & HISTOGRAM_WINDOW)
    {if (win->type2 & HISTOGRAM_1D_WINDOWS)
        histo_type2 = HISTO_1D;
       else histo_type2 = HISTO_MULTI_D;
     slice_roi_win = GetHistoSliceRoiWinRec(&(img->file->histo_params),
                           histo_type2);
     free_slice_roi_rec(&(slice_roi_win->slice_roi));
     win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,2,0,0,0);}
   else
    {PRT9(printf("win_delete_all_file_rois...max_sroi_wins = %d\n",
              img->file->slice_info.max_sroi_wins))
     for (lcv = 0; lcv < img->file->slice_info.max_sroi_wins; lcv++)
        free_slice_roi_rec(
              &(img->file->slice_info.sroi_wins[lcv]->slice_roi));
     win_run_proc_to_all_file_wins(win,win_redisplay_all_curves,1,0,0,0);}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_get_next_curve - ???
--------------------------------------------------------------------------*/

int
win_get_next_curve(win)
 struct image_display_window *win;
{
 struct curve_rec *cur,*next;
 struct slice_roi_rec *slice_roi;
 int err;

 if ((err = win_get_cur_slice_roi_rec(win,&slice_roi,NULL)) != NO_ERROR)
    return err;
 cur = get_cur_curve(slice_roi);
 if (!is_curve_non_empty(cur)) return NO_ERROR;

 next_curve(slice_roi,1);
 next = get_cur_curve(slice_roi);
 TRACE(printf("win_get_next_curve  cur = %u  next = %u\n",cur,next))
 if ((next != cur) && (next != NULL))
    {
     win_display_cur_roi(win);
     win_set_pt_status(win);
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
WinDeleteMarkRoi - ???
--------------------------------------------------------------------------*/

int
WinDeleteMarkRoi(win,is_dp_cur)
 struct image_display_window *win;
 int is_dp_cur;
{
 int is_some_deleted;

 if (win->control.sroi.sroi_win != NULL)
    {if ((win->control.sroi.sroi_win->slice_roi.status &
             SLICE_ROI_MARK_DELETE) ||
         (win->status4 & WIN_ROI_MARK_DELETE))
        {delete_roi_marked_delete(&(win->control.sroi.sroi_win->slice_roi),
                    &is_some_deleted);
         win->status4 &= ~WIN_ROI_MARK_DELETE;
         if (is_some_deleted)
            {if (win->control.sroi.sroi_win->num_wins == 1) 
                if (!is_dp_cur && (win->num_display_wins == 1) &&
                    (win->control.sroi.sroi_win->wins[0] == win))
                   return NO_ERROR;
             win_execute_proc_to_related_wins(win,win_redisplay_all_curves,
					      0,0);
            }
        }
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_next_curve - ???
--------------------------------------------------------------------------*/

int
win_next_curve(win)
 struct image_display_window *win;
{
 struct curve_rec *cur,*next;
 int err;
 struct slice_roi_rec *slice_roi;

 if ((err = win_get_cur_slice_roi_rec(win,&slice_roi,NULL)) != NO_ERROR)
    return err;
 cur = get_cur_curve(slice_roi);
 if (!is_curve_non_empty(cur)) return NO_ERROR;
 win_loop_cur_curve(win,0);

 next_curve(slice_roi,1);
 next = get_cur_curve(slice_roi);
 TRACE(printf("win_get_next_curve  cur = %u  next = %u\n",cur,next))
 if ((next != cur) && (next != NULL))
    {
     win_display_cur_roi(win);
     win_set_pt_status(win);
    }
 if (win->draw_list.item != NULL)
    err = WinRunProcInList(WIN_DRAW_LIST,win,win_get_next_curve,0,0,0,0,0,0);

 return err;
}


/*-------------------------------------------------------------------------
win_make_next_curve_cur - ???
--------------------------------------------------------------------------*/

int
win_make_next_curve_cur(win,is_backward)
 struct image_display_window *win;
 int is_backward;
{
 struct curve_2d_rec *pen;
 struct curve_rec *curve;
 int err;

 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi));
 if (pen == NULL) return CANCEL;
   else win_loop_cur_curve(win,0);
 curve = get_cur_curve(&(win->control.sroi.sroi_win->slice_roi));

 if ((err = make_next_curve_cur(&(win->control.sroi.sroi_win->slice_roi),
           is_backward)) != NO_ERROR)
    return err;
 PRT3(printf("after make_next_curve_cur...\n"))
 if (get_cur_curve(&(win->control.sroi.sroi_win->slice_roi)) != curve)
    {win_display_cur_roi(win);}
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_next_pen_type - ???
--------------------------------------------------------------------------*/

int
win_next_pen_type(win)
 struct image_display_window *win;
{
 struct curve_2d_rec *pen;

 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi));
 if (pen == NULL) return CANCEL;
   else win_loop_cur_curve(win,0);
 next_pen_type(&(win->control.sroi.sroi_win->slice_roi));
 if (get_cur_pen(&(win->control.sroi.sroi_win->slice_roi)) != pen)
    {
     win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
    }

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_next_defined_roi - ???
--------------------------------------------------------------------------*/

int
win_next_defined_roi(win,is_backward)
 struct image_display_window *win;
 int is_backward;
{
 struct curve_2d_rec *pen;
 struct roi_rec *roi,*next;

 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi));
 next_roi(&(win->control.sroi.sroi_win->slice_roi),is_backward);
/*
 next_non_empty_roi(&(win->control.sroi.sroi_win->slice_roi));
*/
 pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi));
 if (pen != NULL)
    {win_loop_cur_curve(win,0);
    }
 next = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi));
 if (next != roi)
    win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_make_roi_no_display - ???
--------------------------------------------------------------------------*/

int
win_make_roi_no_display(win,roi_label,set_cur_roi)
 struct image_display_window *win;
 char *roi_label;
 int set_cur_roi;
{
 struct curve_2d_rec *pen;
 struct roi_rec *roi1,*roi;
 int err;

 if (win->control.sroi.sroi_win == NULL) return CANCEL;
 roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi));
 pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi));

 TRACE(printf("*** win_make_roi = %s   cur_roi = %d\n",roi_label,
                  win->control.sroi.sroi_win->slice_roi.cur_roi))
 if ((err = setup_more_rois(&(win->control.sroi.sroi_win->slice_roi),
                        roi_label,set_cur_roi)) == NO_ERROR)
    if ((roi1 = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != roi)
       {
        win_loop_cur_curve(win,0);
       }
      else return CANCEL;
 return err;
}


/*-------------------------------------------------------------------------
win_make_roi - ???
--------------------------------------------------------------------------*/

int
win_make_roi(win,roi_label,set_cur_roi)
 struct image_display_window *win;
 char *roi_label;
 int set_cur_roi;
{
 int err;

 if ((err = win_make_roi_no_display(win,roi_label,set_cur_roi)) == NO_ERROR)
    win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
   else if (err == CANCEL) return NO_ERROR;
 return err;
}


/*-------------------------------------------------------------------------
WinMakeCurRoi - ???
--------------------------------------------------------------------------*/

int
WinMakeCurRoi(win,label)
 struct image_display_window *win;
 char *label;
{
 struct strs_rec *roi_labels;
 int err;

 win_loop_cur_curve(win,0);
 roi_labels = win_rec_get_roi_labels_PTR(win->owner,win->type2);
 if ((err = AddStrToSortedStrsRec(roi_labels,label,1,1)) == NO_ERROR)
    {err = win_make_roi(win,label,1);
    }
 return err;
}


/*-------------------------------------------------------------------------
WinNextCurRoi - ???
--------------------------------------------------------------------------*/

int
WinNextCurRoi(win,label)
 struct image_display_window *win;
 char *label;
{
 struct roi_rec *roi,*roi1;

 if (win->control.sroi.sroi_win == NULL) return NO_ERROR;
 if ((roi1 = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != NULL)
    if (str_cmp(roi1->label,label) == 0)
       {PRT2(printf("WinNextCurRoi same...cur = %s\n",roi1->label))
        return NO_ERROR;}

 if (((roi = next_cur_roi(&(win->control.sroi.sroi_win->slice_roi),
           label)) != NULL) &&
     (roi != roi1))
    {PRT2(printf("WinNextCurRoi...cur = %s  next = %s\n",roi1->label,label))
     WinMakeCurRoi(win,roi->label);
     win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_highlight_roi_cur - ???
--------------------------------------------------------------------------*/

int
win_highlight_roi_cur(win,label)
 struct image_display_window *win;
 char *label;
{
 struct roi_rec *roi;

 if ((label != NULL) &&
     ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != NULL) &&
     ((str_cmp(roi->label,label) != 0) || !(roi->status & ROI_VISIBLE)))
    {win_set_roi_visibility(win,label,ROI_VISIBLE);
     WinNextCurRoi(win,label);
     if (win->type2 & DISPLAY_WINDOW_3D)
        _3d_after_next_roi_proc(win);
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_create_cur_roi_in_all_wins - ???

 if is_which & 1 then apply to the image_display window
 if is_which & 2 then apply to the histogram window
 if is_which & 4 then do not apply to the current window
 if is_which & 8 then do not apply to the windows that use the same slice roi
 if is_which & 16 then apply to the windows with the same window type
--------------------------------------------------------------------------*/

int
win_create_cur_roi_in_all_wins(win,is_which)
 struct image_display_window *win;
 int is_which;
{
 struct roi_rec *roi;

 if ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != NULL)
    win_run_proc_to_all_file_wins(win,WinNextCurRoi,is_which,roi->label,0,0);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_highlight_roi_in_all_wins - ???
--------------------------------------------------------------------------*/

int
win_highlight_roi_in_all_wins(win,is_which)
 struct image_display_window *win;
 int is_which;
{
 struct roi_rec *roi;

 if ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != NULL)
    win_run_proc_to_all_file_wins(win,win_highlight_roi_cur,is_which,
				  roi->label,0,0);

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_join_curve_roi_rec - ???
--------------------------------------------------------------------------*/

int
win_join_curve_roi_rec(win,roi_label)
 struct image_display_window *win;
 char *roi_label;
{
 struct curve_rec *cur_curve;
 struct roi_rec *roi;
 struct curve_2d_rec *pen;
 int err;

 if (((cur_curve = get_cur_curve(&(win->control.sroi.sroi_win->slice_roi))) ==
       NULL) ||
     ((err = setup_more_rois(&(win->control.sroi.sroi_win->slice_roi),
                 roi_label,0)) != NO_ERROR) ||
     ((pen = get_cur_pen(&(win->control.sroi.sroi_win->slice_roi))) == NULL) ||
     ((roi = get_roi(&(win->control.sroi.sroi_win->slice_roi),roi_label)) == 
         NULL))
    return CANCEL;
 if ((err = JoinCurveToRoiRec(roi,cur_curve,pen->pen_type,1,
              &(win->control.sroi.sroi_win->slice_roi.status),
              1.0,1.0,NULL,NULL,NULL)) != NO_ERROR)
    return err;
 delete_cur_curve(pen,&(win->control.sroi.sroi_win->slice_roi.status),1);
 win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
 return NO_ERROR;

}


/*-------------------------------------------------------------------------
win_is_roi_pen_type - ???
--------------------------------------------------------------------------*/

int
win_is_roi_pen_type(win)
 struct image_display_window *win;
{
 unsigned short ary[2];

 ary[0] = DRAW_FREE;
 ary[1] = DRAW_LINE;
 return !is_cur_roi_pen_type(&(win->control.sroi.sroi_win->slice_roi),ary,2);

}


/*-------------------------------------------------------------------------
win_loop_cur_curve - ???
--------------------------------------------------------------------------*/

int
win_loop_cur_curve(win,must_loop)
 struct image_display_window *win;
 int must_loop;
{
 unsigned long num_pts;
 struct slice_roi_rec *slice_roi;
 int err;

 if ((err = win_get_cur_slice_roi_rec(win,&slice_roi,NULL)) != NO_ERROR)
    return err;
 num_pts = num_curve_pts(get_cur_curve(slice_roi));
 if (num_pts == 0) return NO_ERROR;

 if (must_loop) MustLoopCurCurve(slice_roi);
   else LoopCurCurve(slice_roi);
 if (num_pts != num_curve_pts(get_cur_curve(slice_roi)))
    {win->control.must_display = 1;
     if (win->draw_list.item != NULL)
        err = WinRunProcInList(WIN_DRAW_LIST,win,WinListProcessCurLine,
                win,0,0,0,0,2);
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_move_show_cur_curve - ???
--------------------------------------------------------------------------*/

int
win_move_show_cur_curve(win)
 struct image_display_window *win;
{
 int err;

 set_paint_status(&(win->data.draw_masks[DRAW_MASK_CUR_CURVE]),1,PAINT_ERASE);
 err = paint_cur_curve_to_multiple_win(win,win->data.draw_masks,
         win->data.num_draw_masks,0);
 set_paint_status(&(win->data.draw_masks[DRAW_MASK_CUR_CURVE]),1,
               PAINT_SUPERIMPOSE);
 if (err == NO_ERROR)
    {
     if ((win->control.sroi.sroi_win != NULL) &&
         (get_cur_curve(&(win->control.sroi.sroi_win->slice_roi)) != NULL))
        {win_draw_cur_curve_rec(win);
         paint_cur_curve_to_multiple_win(win,win->data.draw_masks,
             win->data.num_draw_masks,0);
         WinMultiCanvas(win);
        }
       else return CANCEL;
    }
 return err;
}


/*-------------------------------------------------------------------------
win_rename_cur_roi_label - ???
--------------------------------------------------------------------------*/

int
win_rename_cur_roi_label(win,roi_label)
 struct image_display_window *win;
 char *roi_label;
{
 struct roi_rec *cur_roi,*roi;
 struct slice_roi_rec *slice_roi;
 char *str;
 int answer,err;
 Frame frame;

 slice_roi = &(win->control.sroi.sroi_win->slice_roi);
 cur_roi = get_cur_roi(slice_roi);
 if ((cur_roi != NULL) &&
     ((roi = find_roi_rec(slice_roi,roi_label,0)) != NULL))
    {TRACE(printf("win_rename_cur_roi_label...cur_roi = %u roi = %u\n",
              cur_roi,roi))
     if (cur_roi == roi) return NO_ERROR;
     TRACE(printf("win_rename_cur_roi_label...1\n"))
     if (is_roi_non_empty(roi))
        {concat_strs(&str,3,"THE ROI LABEL = ",roi_label,"...");
         if (win->dwin != NULL) frame = win->dwin->frame;
           else frame = win->ancestor;
         answer = (int) notice_prompt(frame,(Event *) NULL,
            NOTICE_MESSAGE_STRINGS,str," has been used !!!",0,
            NOTICE_BUTTON," CANCEL ?",1,
            NOTICE_BUTTON," REPLACE !!",2,
            NOTICE_BUTTON," ADD TO !!",3,
            0);

         c_free(str);
        }
       else answer = 2;

     if (answer == 2)
        {if (roi != NULL) delete_roi(slice_roi,roi);

         err = rename_roi_label(slice_roi,cur_roi,roi_label);
         win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
         return err;
        }
       else if (answer == 3)
        {if ((err = JoinRoiRecs(roi,cur_roi,0,
                     &(slice_roi->status),1.0,1.0,NULL,NULL,NULL)) ==
                NO_ERROR)
            {delete_roi(slice_roi,cur_roi);
             roi = find_roi_rec(slice_roi,roi_label,1);}
         win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
         return err;
        }
       else return NO_ERROR;
    }
   else
    {
     if (cur_roi == NULL)
        if ((err = setup_more_rois(slice_roi,roi_label,1)) == NO_ERROR)
           {win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
            return NO_ERROR;}
	else return err;  /* FIXME:  Was just "else err;"  -PWH 19990427 */
     err = rename_roi_label(slice_roi,cur_roi,roi_label);
     win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
     return err;
    }
}


/*-------------------------------------------------------------------------
MakeRoiConvex - ???
--------------------------------------------------------------------------*/

int
MakeRoiConvex(roi,localto,dimx,dimy)
 struct roi_rec *roi;
 struct matrix_2d_rec *localto;
 long dimx,dimy;
{
 struct draw_mask_rec mask;
 struct curve_rec **curves;
 unsigned long num_curves,total_curves;
 int err;
 struct roi_rec roi1;
 unsigned short status;

 init_draw_mask_rec(&mask);
 if ((err = get_draw_mask_rec(&mask,dimx,dimy)) != NO_ERROR)
    return err;
 DrawRoiRec(&mask,&mask,roi,localto);
 if ((err = fill_2d_convex_area(&(mask.mask._2d),1)) == NO_ERROR)
    if ((err = scan_objects(&(mask.mask._2d),NULL,
               &curves,&total_curves,&num_curves,1,1)) == NO_ERROR)
       {init_roi_rec(&roi1,roi->label);
        err = append_curves_to_roi_rec(&roi1,curves,num_curves,&status);
        if (err == NO_ERROR)
           {free_roi_rec(roi);
            *roi = roi1;}
          else if (num_curves > 0)
           {free_roi_rec(&roi1);
            free_curve_recs(curves,num_curves);
           }
        c_free(curves);
       }
 free_draw_mask_rec(&mask);
 return err;
}


/*-------------------------------------------------------------------------
win_make_cur_roi_convex - ???
--------------------------------------------------------------------------*/

int
win_make_cur_roi_convex(win)
 struct image_display_window *win;
{
 struct roi_rec *roi;
 int err;
 char str[40];
 struct img_buf_rec *img;

 img = win_rec_cur_img_buf(win->owner);
 if (img == NULL) return NO_ERROR;
 if ((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != NULL)
    {if ((err = MakeRoiConvex(roi,win->control.sroi.localto,
                img->_2d.dimx,img->_2d.dimy)) != NO_ERROR)
        {sprintf(str,"err id = %d",err);
         notice_prompt(win->dwin->frame,(Event *) NULL,
                NOTICE_MESSAGE_STRINGS,
               "An error occured when making the ROI convex !!!",str,
                "Operation aborted !!!",0,
                NOTICE_BUTTON_YES," OK !",
                0);
         return err;}
     win->control.sroi.sroi_win->slice_roi.status |= 
                  SLICE_ROI_NON_EMPTY | SLICE_ROI_TO_BE_SAVED;
     win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);
    }
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_bring_back_out_of_bound_rois - ???
--------------------------------------------------------------------------*/

int
win_bring_back_out_of_bound_rois(win)
 struct image_display_window *win;
{
 unsigned long dimx,dimy;
 int err;

 if ((err = WinColorDataDims(win,&dimx,&dimy,NULL)) != NO_ERROR)
    return err;
 if (MoveOutOfBoundRois(&(win->control.sroi.sroi_win->slice_roi),
          win->control.sroi.localto,
          .0,.0,(float) (dimx - 1),(float) (dimy - 1),
          (float) (dimx / 2.0),(float) (dimy / 2.0)))
    win_run_proc_on_slice_roi_wins(win->control.sroi.sroi_win,
				   win_redisplay_all_curves,0,0);
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
win_slice_rect_roi - ???
--------------------------------------------------------------------------*/

int
win_slice_rect_roi(win)
 struct image_display_window *win;
{
 struct roi_rec *roi;
 unsigned long dimx,dimy;
 int err;

 if ((err = WinColorDataDims(win,&dimx,&dimy,NULL)) != NO_ERROR)
    return err;
 if ((roi = get_roi(&(win->control.sroi.sroi_win->slice_roi),
       SLICE_RECT_ROI)) != NULL)
    delete_roi(&(win->control.sroi.sroi_win->slice_roi),roi);

 if ((err = setup_more_rois(&(win->control.sroi.sroi_win->slice_roi),
         SLICE_RECT_ROI,1)) == NO_ERROR)
    if (((roi = get_cur_roi(&(win->control.sroi.sroi_win->slice_roi))) != NULL) &&
        ((err = EnterPtsIntoRoiRec(roi,
           (unsigned long) DRAW_RECT,
           &(win->control.sroi.sroi_win->slice_roi.status),
           win->control.sroi.tolocal,1,2, .0,.0,
           (float) dimx - 1,(float) dimy - 1)) == NO_ERROR))
       win_execute_proc_to_related_wins(win,win_redisplay_all_curves,0,0);

 return err;
}
