/****************************************************************************
slice_roi_win_rec.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: slice_roi_win_rec.c,v 1.7 1999/07/07 17:41:19 hughett Exp $ Release $Name: nria_1_0_6 $";

#include "mri_format.h"
#include "nria_procedures.h"


/*
#define TRACE(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
*/
#define TRACE(x)
#define PRT(x)



/* Functions defined in this file */
void AssociateSlicesWithSliceRoiWinRecs(struct hdr_rec *hdr,
    struct hdr_scan_entry_rec *scans, unsigned long num_scans,
    struct slice_roi_win_rec **sroi_wins, unsigned long num_sroi_wins);
void init_slice_roi_win_rec(struct slice_roi_win_rec *slice_roi_win,
    unsigned int dimx, unsigned int dimy);
void InitSliceRoiWinRecs(struct slice_roi_win_rec **sroi_wins,
    unsigned int num_sroi_wins, unsigned int dimx, unsigned int dimy);
void free_slice_roi_win_rec(struct slice_roi_win_rec *slice_roi_win);
void FreeSliceRoiWinRecs(struct slice_roi_win_rec **sroi_wins,
    unsigned int num_sroi_wins);
int AssociateWinWithSliceRoiWinRec(struct slice_roi_win_rec *slice_roi_win,
    struct image_display_window *win);
int DisassociateWinWithSliceRoiWinRec(struct slice_roi_win_rec *slice_roi_win,
    struct image_display_window *win);
int build_slice_roi_win_rec(struct input_file_rec *file);
struct slice_roi_win_rec *file_scan_slice_roi_win(struct input_file_rec *file,
    struct hdr_scan_entry_rec *scan_entry);



/*-------------------------------------------------------------------------
AssociateSlicesWithSliceRoiWinRecs - ???
--------------------------------------------------------------------------*/

void
AssociateSlicesWithSliceRoiWinRecs(hdr,scans,num_scans,sroi_wins,num_sroi_wins)
 struct hdr_rec *hdr;
 struct hdr_scan_entry_rec *scans;
 unsigned long num_scans,num_sroi_wins;
 struct slice_roi_win_rec **sroi_wins;
{
 unsigned int lcv,cnt,num_cuts,num_echoes,num,total,max_scans;

 PRT(printf("AssociateSlicesWithSliceRoiWinRecs num_scans = %d\n",num_scans))
 num_cuts = NumberOfSliceCuts(&(hdr->data_info));
 max_scans = NumberOfSliceScans(&(hdr->data_info));
 if (hdr->data_info.num_echoes == 0)
    num_echoes = 1;
   else num_echoes = hdr->data_info.num_echoes;
 total = num_cuts * num_echoes * max_scans;
 PRT(printf("AssociateSlicesWithSliceRoiWinRecs total = %d num_scans = %d\n",
          total,num_scans))

 if (num_scans < total) total = num_scans;
 for (cnt = 0,lcv = 0; lcv < total; )
    {for (num = 0; num < num_echoes; num += 1,lcv++)
        {
         scans[lcv].slice->slice_roi_win = sroi_wins[cnt];
         PRT(printf("%d) %d %u %d\n",cnt,lcv,sroi_wins[cnt],
                sroi_wins[cnt]->num_wins);)
        }
     cnt = (cnt + 1) % num_sroi_wins;
    }
 PRT(printf("AssociateSlicesWithSliceRoiWinRecs end\n"))
}


/*-------------------------------------------------------------------------
init_slice_roi_win_rec - ???
--------------------------------------------------------------------------*/

void
init_slice_roi_win_rec(slice_roi_win,dimx,dimy)
 struct slice_roi_win_rec *slice_roi_win;
 unsigned int dimx,dimy;
{
 init_slice_roi_rec(&(slice_roi_win->slice_roi));
 setup_slice_roi_rec_dim(&(slice_roi_win->slice_roi),dimx,dimy);
 slice_roi_win->num_wins = 0;
 slice_roi_win->wins = NULL;
}


/*-------------------------------------------------------------------------
InitSliceRoiWinRecs - ???
--------------------------------------------------------------------------*/

void
InitSliceRoiWinRecs(sroi_wins,num_sroi_wins,dimx,dimy)
 struct slice_roi_win_rec **sroi_wins;
 unsigned int num_sroi_wins;
 unsigned int dimx,dimy;
{
 unsigned int lcv;

 for (lcv = 0; lcv < num_sroi_wins; lcv += 1)
    init_slice_roi_win_rec(sroi_wins[lcv],dimx,dimy);
}


/*-------------------------------------------------------------------------
free_slice_roi_win_rec - ???
--------------------------------------------------------------------------*/

void
free_slice_roi_win_rec(slice_roi_win)
 struct slice_roi_win_rec *slice_roi_win;
{
 free_slice_roi_rec(&(slice_roi_win->slice_roi));
 if (slice_roi_win->num_wins > 0)
    free_handle_memory(slice_roi_win->wins,slice_roi_win->num_wins);
 init_slice_roi_win_rec(slice_roi_win,0,0);
}


/*-------------------------------------------------------------------------
FreeSliceRoiWinRecs - ???
--------------------------------------------------------------------------*/

void
FreeSliceRoiWinRecs(sroi_wins,num_sroi_wins)
 struct slice_roi_win_rec **sroi_wins;
 unsigned int num_sroi_wins;
{
 unsigned int lcv;

 PRT(printf("FreeSliceRoiWinRecs...num_sroi_wins = %d\n",num_sroi_wins));
 for (lcv = 0; lcv < num_sroi_wins; lcv += 1)
    {
     PRT(printf("sroi_wins[%d] = %u\n",lcv,sroi_wins[lcv]));
     if (sroi_wins[lcv] != NULL)
        free_slice_roi_win_rec(sroi_wins[lcv]);}
}


/*-------------------------------------------------------------------------
AssociateWinWithSliceRoiWinRec - ???
--------------------------------------------------------------------------*/

int
AssociateWinWithSliceRoiWinRec(slice_roi_win,win)
 struct slice_roi_win_rec *slice_roi_win;
 struct image_display_window *win;
{
 return add_handle_element(&(slice_roi_win->wins),
           &(slice_roi_win->num_wins),win);
}


/*-------------------------------------------------------------------------
DisassociateWinWithSliceRoiWinRec - ???
--------------------------------------------------------------------------*/

int
DisassociateWinWithSliceRoiWinRec(slice_roi_win,win)
 struct slice_roi_win_rec *slice_roi_win;
 struct image_display_window *win;
{
 int err;

 if ((err = DropHandleAryPtr(&(slice_roi_win->wins),
           &(slice_roi_win->num_wins),win)) == NOT_FOUND)
    {
     return NO_ERROR;}
 return err;
}


/*-------------------------------------------------------------------------
build_slice_roi_win_rec - ???

build up a slice_roi_win_rec for each individual image and the
associate scans with their corresponding slice_roi_win_recs.
--------------------------------------------------------------------------*/

int
build_slice_roi_win_rec(file)
 struct input_file_rec *file;
{
 unsigned int cnt,lcv;
 struct slice_info_rec *slice_info;
 int err;

 TRACE(printf("build_slice_roi_win_rec...file->hdr.num_scans = %d\n",
        file->hdr.num_scans))
 cnt = NumberOfSliceCuts(&(file->hdr.data_info));
 slice_info = &(file->slice_info);
 TRACE(printf("build_slice_roi_win_rec num_slices = %d max_sroi_wins = %d\n",
          cnt,slice_info->max_sroi_wins))
 if (slice_info->max_sroi_wins > 0)
    {TRACE(printf("build_slice_roi_win_rec FreeSliceRoiWinRecs\n"))
     FreeSliceRoiWinRecs(slice_info->sroi_wins,slice_info->max_sroi_wins);
    }
 slice_info->num_sroi_wins = 0;
 if (cnt > 0)
    {
     if (cnt > slice_info->max_sroi_wins)
        {
         cnt = cnt - slice_info->max_sroi_wins;
         TRACE(printf("build_slice_roi_win_rec init cnt = %d\n",cnt))
         lcv = slice_info->max_sroi_wins;
         if ((err = get_handle(&(slice_info->sroi_wins),
               &(slice_info->max_sroi_wins),cnt,
               (size_t) sizeof(**(slice_info->sroi_wins)))) != NO_ERROR)
            return err;
             
         TRACE(printf(" cnt = %d max_sroi_wins = %d num = %d\n",cnt,
                   slice_info->max_sroi_wins,
                   slice_info->num_sroi_wins))
         slice_info->num_sroi_wins = slice_info->max_sroi_wins;
        }
       else slice_info->num_sroi_wins = cnt;
     InitSliceRoiWinRecs(slice_info->sroi_wins,
              slice_info->max_sroi_wins,
              file->hdr.data_info.dimx,file->hdr.data_info.dimy);
     PRT(printf("max_sroi_wins = %d\n",slice_info->max_sroi_wins))
     PRT(printf("num_sroi_wins = %d\n",slice_info->num_sroi_wins))
     AssociateSlicesWithSliceRoiWinRecs(&(file->hdr),
        file->hdr.scans,file->hdr.num_scans,slice_info->sroi_wins,
        slice_info->num_sroi_wins);
     return NO_ERROR;
    }
 return  CANCEL;
}


/*-------------------------------------------------------------------------
file_scan_slice_roi_win - ???
--------------------------------------------------------------------------*/

struct slice_roi_win_rec *
file_scan_slice_roi_win(file,scan_entry)
 struct input_file_rec *file;
 struct hdr_scan_entry_rec *scan_entry;
{
 if (scan_entry->slice_num >= file->slice_info.num_sroi_wins)
    return NULL;
 return file->slice_info.sroi_wins[scan_entry->slice_num];
}
