/****************************************************************************
win_init_free.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: win_init_free.c,v 1.8 1999/07/07 17:41:20 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/cms.h>

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"



/* Functions defined in this file */
int init_win_rec(struct win_rec *win, struct all_win_rec *win_root);
void all_win_rec_get_default_color_values(struct all_win_rec *win);
void init_all_win_rec(struct all_win_rec *win, struct prog_rec *prog);
int setup_roi_labels(char **roi_label_fn, struct strs_rec *roi_labels,
    char *label_fn, unsigned short *status, int flag);
void setup_all_win_rec(struct all_win_rec *win, struct prog_rec *prog,
    char *color_fn, char *roi_label_fn, char *sec_roi_label_fn,
    char *cv_roi_label_fn, char *cv_sec_roi_label_fn, char *pixel_type_fn,
    char *sec_pixel_type_fn, int pixel_type_priority_fn,
    Xv_singlecolor *system_colors, unsigned int num_system_colors);
int free_histogram_window(struct win_rec *win);
void free_display_window(struct win_rec *win);
void free_win_rec(struct win_rec *win, int free_img_bufs);
int delete_win_rec(struct win_rec *win, int window_type, int free_img_bufs);
int write_labels(char *fn, char **strs, unsigned long num_strs);
void auto_save_labels(struct all_win_rec *win);
void free_all_win_rec(struct all_win_rec *win);
struct win_rec *get_new_win(struct all_win_rec *win, int *err);
int associate_win_rec_and_img(struct win_rec *win, struct img_buf_rec *img);
struct win_rec *get_new_img_win(struct img_buf_rec *img, int *err);



/*-------------------------------------------------------------------------
init_win_rec - ???
-------------------------------------------------------------------------*/

int
init_win_rec(win,win_root)
     struct win_rec *win;
     struct all_win_rec *win_root;
{
  win->root = win_root;
  win->status = 0;
  win->win_id = 0;
  win->wid = NULL;
  win->cur_img = 0;
  win->num_img_bufs = 0;
  win->img_bufs = NULL;
  win->image_display = NULL;
  win->histogram = NULL;

  return NO_ERROR;
}



/*-------------------------------------------------------------------------
all_win_rec_get_default_color_values - ???
-------------------------------------------------------------------------*/

void
all_win_rec_get_default_color_values(win)
     struct all_win_rec *win;
{
  if (win->def_color.pallettes[win->def_color.cur_pallette]->
      color.total_colors >= BREAKER_COLOR)
    win->breaker_mask.pixel_value =
      win->def_color.pallettes[win->def_color.cur_pallette]->
      color.total_colors - BREAKER_COLOR;
  else
    win->breaker_mask.pixel_value = 0;

  if (win->def_color.pallettes[win->def_color.cur_pallette]->
      color.total_colors >= CLIPBOARD_COLOR)
    win->clipboard_mask.pixel_value =
      win->def_color.pallettes[win->def_color.cur_pallette]->
      color.total_colors - CLIPBOARD_COLOR;
  else
    win->clipboard_mask.pixel_value = 0;
}



/*-------------------------------------------------------------------------
init_all_win_rec - ???
-------------------------------------------------------------------------*/

void
init_all_win_rec(win,prog)
     struct all_win_rec *win;
     struct prog_rec *prog;
{
  win->status = 0;
  win->prog = prog;
  init_pallette_rec(&(win->def_color));
  win->roi_label_fn = NULL;
  win->sec_roi_label_fn = NULL;
  win->cv_roi_label_fn = NULL;
  win->cv_sec_roi_label_fn = NULL;

  win->pixel_type_fn = NULL;
  win->sec_pixel_type_fn = NULL;
  win->pixel_type_priority_fn = NULL;
  init_strs_rec(&(win->roi_labels));
  init_strs_rec(&(win->sec_roi_labels));
  init_strs_rec(&(win->cv_roi_labels));
  init_strs_rec(&(win->cv_sec_roi_labels));
  
  init_strs_rec(&(win->pixel_types));
  init_strs_rec(&(win->sec_pixel_types));
  
  init_strs_rec(&(win->pixel_type_priority));
  
  init_slice_roi_rec(&(win->breaker));
  win->breaker.prog_status |= BREAKER_LABEL_BREAKER_REGION;
  init_slice_roi_rec(&(win->clipboard));
  init_draw_mask_rec(&(win->breaker_mask));
  init_draw_mask_rec(&(win->clipboard_mask));
  win->breaker_win = NULL;
  win->clipboard_win = NULL;
  win->total_wins = 0;
  win->wins = NULL;
}




/*-------------------------------------------------------------------------
setup_roi_labels - ???
-------------------------------------------------------------------------*/

int
setup_roi_labels( roi_label_fn, roi_labels, label_fn, status, flag )
     char **roi_label_fn;	/* storage space for filename */
     struct strs_rec *roi_labels; /* storage space for text items in file */
     char *label_fn;		/* original filename */
     unsigned short *status;	/* storage space for file status */
     unsigned short flag;	/* if successful, turn this flag on in
				   file status */
{
  int err;
  char *reason;
  Frame *frame;

  TRACE(printf("Reading in strings from \"%s\"...\n",label_fn););
  err = NO_ERROR;

  if (label_fn == NULL)
  {
    *roi_label_fn = NULL;
    TRACE(printf("\n"););
    return NO_ERROR;
  }

  if ((err = cp_str(roi_label_fn,label_fn)) == NO_ERROR)
  {
    if (file_access(*roi_label_fn,12,NULL,NULL) == NO_ERROR)
    {
      if ((err = rd_ascii_file_strs_to_strs_rec(*roi_label_fn,roi_labels,0)) != NO_ERROR) 
      {
	if (((frame = (Frame *) main_display(2,NULL,0,0)) != NULL) && (*frame != 0))
	{
	  ShowMsgStrs(*frame,0,err,0,2,
		      "An error occured when reading from",
		      *roi_label_fn,NULL,0);
	}
      }
      else
	drop_dup_str_from_strs_rec(roi_labels);
    }
    else
    {
      if (((frame = (Frame *) main_display(2,NULL,0,0)) != NULL) && (*frame != 0))
      {
	ShowMsgStrs(*frame,0,err,0,3,
		    "An error occured when reading from",
		    *roi_label_fn,reason,NULL,0);
      }
      c_free(reason);
    }
  }

  if (flag && (err == NO_ERROR))
    if (AddStrToSortedStrsRec(roi_labels,SCRATCH_STR,0,0) == NO_ERROR)
      *status |= flag;

#if 0
  TRACE(pr_strs_rec(roi_labels););
  TRACE(printf("\n"););
#endif

  return err;
}



/*-------------------------------------------------------------------------
setup_all_win_rec - ???
-------------------------------------------------------------------------*/

void
setup_all_win_rec( win, prog, color_fn, roi_label_fn, sec_roi_label_fn,
		   cv_roi_label_fn, cv_sec_roi_label_fn, pixel_type_fn,
		   sec_pixel_type_fn, pixel_type_priority_fn,
		   system_colors, num_system_colors )
     struct all_win_rec *win;
     struct prog_rec *prog;
     char *color_fn, *roi_label_fn, *pixel_type_fn, *cv_roi_label_fn;
     char *sec_roi_label_fn, *sec_pixel_type_fn, *cv_sec_roi_label_fn;
     Xv_singlecolor *system_colors;
     unsigned int num_system_colors;
{
  TRACE(printf("setup_all_win_rec()\n"););

  if (setup_pallette_rec(color_fn,&(win->def_color),
			 system_colors,num_system_colors) != NO_ERROR)
  {
    abort_program(prog,1);
  }

  if (win->def_color.color_fn == NULL)
    cp_str(&(win->def_color.color_fn),DEFAULT_COLOR_FN);

  setup_roi_labels(&(win->roi_label_fn),
		   &(win->roi_labels),
		   roi_label_fn,
		   &(win->status),
		   WIN_REC_SAVE_ROI_LABELS);

  setup_roi_labels(&(win->cv_roi_label_fn),
		   &(win->cv_roi_labels),
		   cv_roi_label_fn,
		   &(win->status),
		   WIN_REC_SAVE_CV_ROI_LABELS);

  setup_roi_labels(&(win->sec_roi_label_fn),
		   &(win->sec_roi_labels),
		   sec_roi_label_fn,
		   &(win->status),
		   0);

  setup_roi_labels(&(win->cv_sec_roi_label_fn),
		   &(win->cv_sec_roi_labels),
		   cv_sec_roi_label_fn,
		   &(win->status),
		   0);

  setup_roi_labels(&(win->pixel_type_fn),
		   &(win->pixel_types),
		   pixel_type_fn,
		   &(win->status),
		   WIN_REC_SAVE_PIXEL_TYPES);

  setup_roi_labels(&(win->sec_pixel_type_fn),
		   &(win->sec_pixel_types),
		   sec_pixel_type_fn,
		   &(win->status),
		   0);

  setup_roi_labels(&(win->pixel_type_priority_fn),
		   &(win->pixel_type_priority),
		   pixel_type_priority_fn,
		   &(win->status),
		   0);

  all_win_rec_get_default_color_values(win);

  win->prog = prog;
}




/*-------------------------------------------------------------------------
free_histogram_window - ???
-------------------------------------------------------------------------*/

int
free_histogram_window(win)
 struct win_rec *win;
{
 unsigned long lcv;
 struct image_display_window *histogram;

 if (win->histogram != NULL)
    {
     PRT3(printf("free_histogram_window = %u\n",win->histogram))
     histogram = win->histogram;
     win->histogram = NULL;
     for (lcv = 0; lcv < win->root->total_wins; lcv++)
        if (win->root->wins[lcv] != NULL)
           {if (win->root->wins[lcv]->histogram == histogram)
               {win->root->wins[lcv]->histogram = NULL;
                PRT3(printf("%d) free_histogram_window in use\n",lcv));}
           }

     free_image_display_window(histogram);
     c_free(histogram);
     PRT3(printf("free_histogram_window = end\n"))
/*
     histogram = win->histogram;
     win->histogram = NULL;
     win_rec_get_file_histogram(win,&histogram1);
     PRT3(printf("free_histogram_window histogram = %u histogram1 = %u\n",
                histogram,histogram1))
     if (histogram1 == NULL)
        {
         free_image_display_window(histogram);
         c_free(histogram);}
       else
        {find_handle_element(histogram->owner->root->wins,
                  histogram->owner->root->total_wins,histogram->owner,&err);
         if (err == NOT_FOUND)
            {
             TRACE(printf("free_histogram_window NOT_FOUND = %u\n",histogram1))
             if ((img = win_rec_cur_img_buf(histogram->owner)) == NULL)
                return CANCEL;
             for (lcv = 0; lcv < img->file->img.num_data; lcv++)
                if ((img->file->img.data[lcv] != NULL) &&
                    (img->file->img.data[lcv]->win != NULL))
                   {histogram->owner = img->file->img.data[lcv]->win;
                    histogram->owner->histogram = histogram;
                    break;}
            }
        }
     PRT3(printf("free_histogram_window 2 = %u end\n\n",histogram1))
*/
    }

 return NO_ERROR;
}


/*-------------------------------------------------------------------------
free_display_window - ???
-------------------------------------------------------------------------*/

void
free_display_window(win)
 struct win_rec *win;
{
 if (win->image_display != NULL) 
    {PRT3(printf("free_display_window win->image_display = %u\n",win->image_display))
     free_image_display_window(win->image_display);
     PRT3(printf("free_display_window win->image_display = %u\n",
             win->image_display))
     c_free(win->image_display);
     win->image_display = NULL;
    }
}



/*-------------------------------------------------------------------------
free_win_rec - ???
-------------------------------------------------------------------------*/

void
free_win_rec( win, free_img_bufs )
     struct win_rec *win;
     int free_img_bufs;
{
  unsigned int lcv;
  struct input_file_rec *file;


  NOPRT(printf("\nfree_win_rec...win = %u %s num = %d free = %d\n",
	      win,win->wid,win->num_img_bufs,free_img_bufs));

  for (lcv = 0; lcv < win->num_img_bufs; lcv++)
  {
    win->img_bufs[lcv]->win = NULL;
    NOPRT(printf("img_bufs[%d] = %u...win = NULL\n",lcv,win->img_bufs[lcv]));
  }

  NOPRT(printf("win->histogram = %u  win->image_display = %u\n",
	      win->histogram,win->image_display));

  if (win->histogram != NULL)
    free_histogram_window(win);
  if (win->image_display != NULL)
    free_display_window(win);

  NOPRT(printf("free_win_rec+...win->num_img_bufs = %d  free_img_bufs = %d\n",
	      win->num_img_bufs,free_img_bufs));

  if (free_img_bufs && (win->num_img_bufs > 0))
  {
    file = win->img_bufs[0]->file;
    for (lcv = 0; lcv < win->num_img_bufs; lcv++)
      delete_img(win->img_bufs[lcv]);
    if (file->img.num_data == 0)
      close_and_delete_file(file,0);
    NOPRT(for (cnt = 0; cnt < file->img.num_data; cnt++)
	  printf("FILE %d) %u\n",cnt,file->img.data[cnt]);
	  );
  }
  
  NOPRT(printf("\nfree_win_rec...win->wid = %s\n",win->wid));

  c_free(win->wid);
  if (win->num_img_bufs > 0)
    c_free(win->img_bufs);
  init_win_rec(win,NULL);
  NOPRT(printf("after free_win_rec...end ...\n\n"));
}




/*-------------------------------------------------------------------------
delete_win_rec - ???
-------------------------------------------------------------------------*/

int
delete_win_rec(win,window_type,free_img_bufs)
 struct win_rec *win;
 unsigned short window_type;
 int free_img_bufs;
{
 PRT3(printf("\ndelete_win_rec win = %u wid = %s total_wins = %d num_img_bufs = %d\n",win,win->wid,
             win->root->total_wins,win->num_img_bufs))
 if (window_type & MANY_SLICES_WINDOWS)
    {if (win->root->total_wins > 0)
        DropHandleAryPtr(&(win->root->wins),
                        &(win->root->total_wins),win);
     free_win_rec(win,free_img_bufs);
     c_free(win);
    }
   else if (window_type & HISTOGRAM_WINDOW)
    {TRACE(printf("HISTOGRAM_WINDOW free_histogram_window\n"))
     free_histogram_window(win);
    }
 PRT3(printf("delete_win_rec end\n"))
 return NO_ERROR;
}


/*-------------------------------------------------------------------------
write_labels - ???
-------------------------------------------------------------------------*/

int
write_labels(fn,strs,num_strs)
 char *fn,**strs;
 unsigned long num_strs;
{
 int err;
 char *reason;
 Frame *frame;

 if ((err = file_access(fn,2,NULL,&reason)) == NO_ERROR)
    {if ((err = wr_all_file_texts(fn,strs,num_strs)) != NO_ERROR)
        {
         if (((frame = (Frame *) main_display(2,NULL,0,0)) != NULL) &&
              (*frame != 0))
            ShowMsgStrs(*frame,0,err,0,2,
               "An error occured when writing to",fn,NULL,0);
        }
    }
   else
    {
     if (((frame = (Frame *) main_display(2,NULL,0,0)) != NULL) &&
         (*frame != 0))
        ShowMsgStrs(*frame,0,err,0,3,
            "An error occured when writing to",fn,reason,NULL,0);
     c_free(reason);}
 return err;
}


/*-------------------------------------------------------------------------
auto_save_labels - ???
-------------------------------------------------------------------------*/

void
auto_save_labels(win)
 struct all_win_rec *win;
{
 int err;

 if (win->status & WIN_REC_SAVE_ROI_LABELS)
    {if ((err = write_labels(win->roi_label_fn,win->roi_labels.strs,
               win->roi_labels.num_strs)) == NO_ERROR)
        win->status &= ~WIN_REC_SAVE_ROI_LABELS;}
 if (win->status & WIN_REC_SAVE_SEC_ROI_LABELS)
    {if ((err = write_labels(win->sec_roi_label_fn,win->sec_roi_labels.strs,
               win->sec_roi_labels.num_strs)) == NO_ERROR)
        win->status &= ~WIN_REC_SAVE_SEC_ROI_LABELS;}

 if (win->status & WIN_REC_SAVE_CV_ROI_LABELS)
    {if ((err = write_labels(win->cv_roi_label_fn,win->cv_roi_labels.strs,
               win->cv_roi_labels.num_strs)) == NO_ERROR)
        win->status &= ~WIN_REC_SAVE_CV_ROI_LABELS;}

 if (win->status & WIN_REC_SAVE_CV_SEC_ROI_LABELS)
    {if ((err = write_labels(win->cv_sec_roi_label_fn,win->cv_sec_roi_labels.strs,
               win->cv_sec_roi_labels.num_strs)) == NO_ERROR)
        win->status &= ~WIN_REC_SAVE_CV_SEC_ROI_LABELS;}

 if (win->status & WIN_REC_SAVE_PIXEL_TYPES)
    {if ((err = write_labels(win->pixel_type_fn,win->pixel_types.strs,
               win->pixel_types.num_strs)) == NO_ERROR)
         win->status &= ~WIN_REC_SAVE_PIXEL_TYPES;}

 if (win->status & WIN_REC_SAVE_SEC_PIXEL_TYPES)
    {if ((err = write_labels(win->sec_pixel_type_fn,win->sec_pixel_types.strs,
               win->sec_pixel_types.num_strs)) == NO_ERROR)
        win->status &= ~WIN_REC_SAVE_SEC_PIXEL_TYPES;}
}



/*-------------------------------------------------------------------------
free_all_win_rec - ???
-------------------------------------------------------------------------*/

void
free_all_win_rec(win)
     struct all_win_rec *win;
{
  unsigned int lcv;

  if (check_save_color(&(win->def_color)))
    wr_pallette_rec(win->def_color.color_fn,&(win->def_color));
  
  free_pallette_rec(&(win->def_color));
  auto_save_labels(win);
  if (win->pixel_type_priority.status & SAVE_STRS_REC)
  {
    wr_all_file_texts(win->pixel_type_priority_fn,
		      win->pixel_type_priority.strs,
		      win->pixel_type_priority.num_strs);
    win->status &= ~SAVE_STRS_REC;
  }

  c_free(win->roi_label_fn);
  c_free(win->sec_roi_label_fn);
  c_free(win->cv_roi_label_fn);
  c_free(win->cv_sec_roi_label_fn);
  c_free(win->pixel_type_fn);
  c_free(win->sec_pixel_type_fn);
  c_free(win->pixel_type_priority_fn);

  NOPRT(printf("free_strs_rec...\n"));

  free_strs_rec(&(win->roi_labels));
  free_strs_rec(&(win->sec_roi_labels));
  free_strs_rec(&(win->cv_roi_labels));
  free_strs_rec(&(win->cv_sec_roi_labels));
  free_strs_rec(&(win->pixel_types));
  free_strs_rec(&(win->sec_pixel_types));
  free_strs_rec(&(win->pixel_type_priority));
  free_slice_roi_rec(&(win->breaker));
  free_slice_roi_rec(&(win->clipboard));
  free_draw_mask_rec(&(win->breaker_mask));
  free_draw_mask_rec(&(win->clipboard_mask));

  NOPRT(printf("AFTER free_strs_rec, win->total_wins = %d\n",win->total_wins));
  for (lcv = 0; lcv < win->total_wins; lcv++)
    if (win->wins[lcv] != NULL)
    {
      NOPRT(printf("win->wins[%d]  = %u\n",lcv,win->wins[lcv]));
      free_win_rec(win->wins[lcv],0);
      c_free(win->wins[lcv]);
      NOPRT(printf("after c_free(win->wins[lcv])\n"));
    }

  NOPRT(printf("AFTER free_win_rec...total_wins = %d\n",win->total_wins));
  if (win->total_wins > 0)
    c_free(win->wins);
  init_all_win_rec(win,NULL);
}



/*-------------------------------------------------------------------------
get_new_win - ???
-------------------------------------------------------------------------*/

struct win_rec *
get_new_win( win, err )
     struct all_win_rec *win;
     int *err;
{
  struct win_rec *window;

  if ((*err = get_ptr(&window,NULL,(unsigned int) 1,
		      (size_t) sizeof(*window))) != NO_ERROR)
    return NULL;

  if ((*err = init_win_rec(window,win)) == NO_ERROR)
  {
    *err = add_handle_element(&(win->wins),&(win->total_wins),window);
  }
  TRACE(printf("get_new_win##### win->total_wins = %d\n",win->total_wins));
  if (*err != NO_ERROR)
  {
    free_win_rec(window,1);
    return NULL;
  }
  TRACE(printf("get_new_win ok...err = %d\n",*err));

  return window;
}


/*-------------------------------------------------------------------------
associate_win_rec_and_img - ???
-------------------------------------------------------------------------*/

int
associate_win_rec_and_img(win,img)
 struct win_rec *win;
 struct img_buf_rec *img;
{
 return add_handle_element(&(win->img_bufs),&(win->num_img_bufs),img);

}



/*-------------------------------------------------------------------------
get_new_img_win - ???
-------------------------------------------------------------------------*/

struct win_rec *
get_new_img_win(img,err)
     struct img_buf_rec *img;
     int *err;
{
  struct win_rec *window;

  if ((window = get_new_win(&(img->file->prog->win),err)) != NULL)
    if ((*err = associate_win_rec_and_img(window,img)) != NO_ERROR)
    {
      DropHandleAryPtr(&(img->file->prog->win.wins),
		       &(img->file->prog->win.total_wins),window);
      free_win_rec(window,1);
      window = NULL;
    }

  return window;
}




#if 0   /***********************/

/*
 *  This batch of code was originally commented out and floating next to
 *  free_win_rec().
 */

for (lcv = 0; lcv < win->num_img_bufs; lcv++)
{
  file = win->img_bufs[lcv]->file;

  if (is_all_img_partner_wins_closed(win->img_bufs[lcv]))
  {
    PRT3(printf("free_win_rec deleting buf...\n"));
            
    delete_img_and_partner_bufs(win->img_bufs[lcv]);
    PRT3(printf("free_win_rec num_data = %d...\n",file->img.num_data));

    if (file->img.num_data == 0)
    {
      close_and_delete_file(file,0);
    }
  }

  PRT3({
    for (cnt = 0; cnt < file->img.num_data; cnt++)
      printf("FILE %d) %u\n",cnt,file->img.data[cnt]);
  }
       );

}

#endif  /*************************/

/********************
win_rec_delete_file_histogram(win)
 struct win_rec *win;
{
 struct img_buf_rec *img;
 unsigned int lcv;

 TRACE(printf("win_rec_delete_file_histogram...\n"))
 if (win->histogram == NULL)
    win_rec_get_file_histogram(win,&(win->histogram));
 TRACE(printf("win_rec_delete_file_histogram...1\n"))
 if (win->histogram != NULL)
    {
     free_image_display_window(win->histogram);
     c_free(win->histogram);}
 if ((img = win_rec_cur_img_buf(win)) == NULL) return CANCEL;
 TRACE(printf("win_rec_delete_file_histogram...2\n"))
 for (lcv = 0; lcv < img->file->img.num_data; lcv++)
    if ((img->file->img.data[lcv] != NULL) &&
        (img->file->img.data[lcv]->win != NULL))
       img->file->img.data[lcv]->win->histogram = NULL;
 TRACE(printf("win_rec_delete_file_histogram...end\n"))
 return NO_ERROR;
}
*****************************/
