/****************************************************************************
window.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: window.c,v 1.10 1999/07/07 17:41:20 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <xview/xview.h>
#include <xview/cms.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/scrollbar.h>
#include <xview/notify.h>
#include <xview/font.h>
#include <xview/wmgr.h>

/* FIXME:  No longer necessary?
#include <sys/timeb.h>
*/

#include "mri_format.h"
#include "debug.h"
#include "nria_procedures.h"


#define MAG_CURRENT_WINDOW 0
#define MAG_NEW_MASTER_WINDOW 1
#define MAG_NEW_SLAVE_WINDOW 2



struct intermediate_rec
{
  struct image_display_window *win;
  struct display_win_rec *dwin;
  int display_cur_win,center_win,center_x,center_y;
};


/* Functions defined in this file */
struct display_win_rec *get_valid_win_dwin(struct image_display_window *win);
void drop_dwin_from_master_slave_relationship(struct display_win_rec *dwin);
void add_master_slave_relationship(struct display_win_rec *master,
    struct display_win_rec *slave);
int win_master_slave_mouse_xy(struct image_display_window *win, int x, int y);
void save_window_attributes1(struct win_rec *win, int win_type);
void save_all_window_attributes(struct all_win_rec *win);
int win_delete_multiple_wins(struct image_display_window *win);
int delete_dwin(struct display_win_rec *dwin);
int SetupImageDisplayWindow(struct image_display_window *win,
    struct win_rec *owner, Frame ancestor,
    struct image_display_window *def_win);
int win_setup_multiple_win(struct image_display_window *win,
    struct display_win_rec *master);
int win_get_multiple_win(struct image_display_window *win,
    int magnify_which_win, int magx, int magy, int center_win,
    int center_x, int center_y, int scale_option);
int win_display_win_proc(struct intermediate_rec *inter, int which);
int redisplay_cur_win(struct image_display_window *win, int center_win,
    int center_x, int center_y, int is_display_cur_win);
int WinSetupMagnifyWindow(struct image_display_window *win, int mag_factorx,
    int mag_factory, int magnify_which_win, int center_win, int center_x,
    int center_y);
int WinImageType(struct image_display_window *win);
int SetupDisplayWindow(struct img_buf_rec *next_img, Frame owner_frame,
    int win_type2, struct image_display_window *def_win);
int build_all_echo_wins(struct img_buf_rec *img, int win_type2);
int win_setup_windows(struct win_rec *owner, Frame setup_fr_win,
    int direction, int status);
int win_rec_setup_histogram(struct win_rec *win, int win_type2);
int WinShowNewWin(struct image_display_window *newwin, Frame next_to_frame,
    struct image_display_window *def_win);
int GetNewDisplayWin(struct image_display_window *win,
    struct img_buf_rec *img1, int win_type2,
    struct image_display_window **newwin);
int NewWin(struct image_display_window *win, int win_type2,
    int use_this_buf_dims, unsigned long dimx, unsigned long dimy,
    unsigned long pixel_depth_bits, unsigned long data_type,
    unsigned long img_type4, struct image_display_window **newwin);
int NewWinAllSlicesHisto(struct image_display_window *win, int win_type2,
    int use_this_buf_dims, unsigned long dimx, unsigned long dimy,
    unsigned long pixel_depth_bits, unsigned long data_type,
    unsigned long img_type4, int histo_option, int histo_kind,
    struct image_display_window **newwin);
int win_all_win_echoes_exist(struct image_display_window *win);
int GetNumDwins(struct image_display_window *win, int num_display_wins,
    struct display_win_rec ***dwins, int *total_dwins, int *num_dwins,
    int num_echoes, int is_mag_win, int is_same_slice, int win_type2);
int project_num_dwins(struct display_win_rec **dwins, int num_dwins,
    struct int_rect_rec *projected_rects, int num_echoes, int is_mag_win);
int ProjectWins(struct image_display_window *win,
    struct int_rect_rec *projected_rect, int num_projected_wins,
    int num_echoes, int win_type2);
int win_show_tissue_wins(struct image_display_window *win);



/*---------------------------------------------------------------------------
get_valid_win_dwin - ???
---------------------------------------------------------------------------*/

struct display_win_rec *
get_valid_win_dwin(win)
     struct image_display_window *win;
{
  if (win->num_display_wins > 0)
    return win->dwins[0];
  return NULL;
}



/*---------------------------------------------------------------------------
drop_dwin_from_master_slave_relationship - ???
---------------------------------------------------------------------------*/

void
drop_dwin_from_master_slave_relationship(dwin)
     struct display_win_rec *dwin;
{
  if (dwin->master != NULL)
    dwin->master->slave = dwin->slave;
  if (dwin->slave != NULL)
  {
    if (dwin->status2 & MASTER_WINDOW)
      dwin->slave->status2 |= MASTER_WINDOW;
    dwin->slave->master = dwin->master;
  }
  if (dwin->owner->dwin == dwin)
  {
    dwin->owner->dwin = get_valid_win_dwin(dwin->owner);
  }
  dwin->master = NULL;
  dwin->slave = NULL;
}



/*---------------------------------------------------------------------------
add_master_slave_relationship - ???
---------------------------------------------------------------------------*/

void
add_master_slave_relationship(master,slave)
     struct display_win_rec *master,*slave;
{
  if (master->slave != NULL)
  {
    slave->slave = master->slave;
    master->slave->master = slave;
  }
  master->slave = slave;
  slave->status2 &= ~MASTER_WINDOW;
  slave->master = master;
}



/*---------------------------------------------------------------------------
win_master_slave_mouse_xy - ???
---------------------------------------------------------------------------*/

int
win_master_slave_mouse_xy(win,x,y)
     struct image_display_window *win;
     int x,y;
{
  int err;
  struct display_win_rec *dwin;
  
  err = NO_ERROR;
  if (win->dwin->slave != NULL)
  {
    win->dwin = win->dwin->slave;
    win_move_mouse_to(win,x,y);
  }
  else if (win->dwin->master != NULL)
  {
    dwin = win->dwin->master;
    while (dwin->master != NULL) dwin = win->dwin->master;
    win->dwin = dwin;
    win_move_mouse_to(win,x,y);
  }
  return err;
}



/*---------------------------------------------------------------------------
save_window_attributes1 - ???
---------------------------------------------------------------------------*/

void
save_window_attributes1(win,win_type)
     struct win_rec *win;
     unsigned short win_type;
{
  TRACE(printf("save_window_attributes1 begin...\n"));
  if ((win_type & MANY_SLICES_WINDOWS) && (win->image_display != NULL))
  {
    win_release_slice_roi_win(win->image_display);
    if (win->histogram != NULL)
      win_release_slice_roi_win(win->histogram);
  }
  else if ((win_type & HISTOGRAM_WINDOW) && (win->histogram != NULL))
    win_release_slice_roi_win(win->histogram);
  TRACE(printf("save_window_attributes1 end\n"));
}



/*---------------------------------------------------------------------------
save_all_window_attributes - ???
---------------------------------------------------------------------------*/

void
save_all_window_attributes(win)
     struct all_win_rec *win;
{
  unsigned int lcv;
  
  TRACE(printf("save_all_window_attributes\n"));
  for (lcv = 0; lcv < win->total_wins; lcv++)
    save_window_attributes1(win->wins[lcv],0xFFFF);
  TRACE(printf("save_all_window_attributes...end\n"));

}



/*---------------------------------------------------------------------------
win_delete_multiple_wins - ???
---------------------------------------------------------------------------*/

int
win_delete_multiple_wins(win)
 struct image_display_window *win;
{
 if (win->type2 & DISPLAY_WINDOW_3D) exec_3d_win_exit_proc(win);
 return delete_win_rec(win->owner,win->type2,1);
}



/*---------------------------------------------------------------------------
delete_dwin - ???
---------------------------------------------------------------------------*/

int
delete_dwin(dwin)
 struct display_win_rec *dwin;
{
 PRT3(printf("delete_dwin num_display_wins = %d\n",dwin->owner->num_display_wins))
 WinDeleteMarkRoi(dwin->owner,0);
 if (dwin->owner->num_display_wins == 1)
    {win_delete_multiple_wins(dwin->owner);
    }
   else
    {DropHandleAryPtr(&(dwin->owner->dwins),
              &(dwin->owner->num_display_wins),dwin);
     free_display_win_rec(dwin);
     c_free(dwin);}
 PRT3(printf("delete_dwin end\n"))
 return NO_ERROR;
}



/*---------------------------------------------------------------------------
SetupImageDisplayWindow - ???
---------------------------------------------------------------------------*/

int
SetupImageDisplayWindow(win,owner,ancestor,def_win)
     struct image_display_window *win;
     struct win_rec *owner;
     Frame ancestor;
     struct image_display_window *def_win;
{
  int err;
  struct display_win_rec *dwin;
  struct img_buf_rec *img;

  if ((img = win_rec_cur_img_buf(owner)) == NULL)
    return CANCEL;
  PRT1(printf("SetupImageDisplayWindow(): def_win = %p\n",
	      (void *) def_win));

  if (win->num_display_wins == 0)
  {
    win->owner = owner;
    win->ancestor = ancestor;
    if (def_win != NULL)
    {
      win->intensity = def_win->intensity;
    }
    else
    {
      win->intensity = img->file->color_guide;
    }
  }

  if ((err = get_ptr(&dwin,NULL,(unsigned int) 1,
		     (size_t) sizeof(struct display_win_rec))) != NO_ERROR)
    return err;

  init_display_win_rec(dwin);

  PRT3(printf("	win = %p, dwin = %p\n", (void *)win,(void *)dwin););

  if ((err = SetupDisplayWinRec(dwin,win,ancestor,NULL)) != NO_ERROR)
    c_free(dwin);
  else
  {
    TRACE(printf("SetupImageDisplayWindow() ok.\n"));
    win->dwin = dwin;
    add_handle_element(&(win->dwins),&(win->num_display_wins),dwin);
    if (win->control.sroi.sroi_win == NULL)
      win_before_enter_next_slice(win);
  }
  TRACE(printf("SetupImageDisplayWindow() end, err = %d\n",err));
  return err;
}



/*---------------------------------------------------------------------------
win_setup_multiple_win - ???
---------------------------------------------------------------------------*/

int
win_setup_multiple_win(win,master)
 struct image_display_window *win;
 struct display_win_rec *master;
{
 int err;
 unsigned int total;
 struct display_win_rec *dwin;

 total = 0;
 TRACE(printf("win_setup_multiple_win...begin\n"))
 if ((err = get_ptr(&dwin,&total,(unsigned int) 1,
                  (size_t) sizeof(struct display_win_rec))) != NO_ERROR)
    return err;

 init_display_win_rec(dwin);
 if ((err = SetupDisplayWinRec(dwin,win,win->ancestor,master)) != NO_ERROR)
    c_free(dwin);
   else
    {
     dwin->display_status |= (win->dwin->display_status & DISPLAY_PIXEL_TYPE);
     win->dwin = dwin;
     add_handle_element(&(win->dwins),&(win->num_display_wins),dwin);
     if (win->control.sroi.sroi_win == NULL)
        win_before_enter_next_slice(win);
    }
 TRACE(printf("win_setup_multiple_win...end...\n"))
 return err;
}



/*---------------------------------------------------------------------------
win_get_multiple_win - ???
---------------------------------------------------------------------------*/

int
win_get_multiple_win(win,magnify_which_win,magx,magy,
                     center_win,center_x,center_y,scale_option)
 struct image_display_window *win;
 int magnify_which_win;
 int magx,magy;
 int center_win,center_x,center_y,scale_option;
{
 int err;
 struct int_region_rect dimension;

 if (magnify_which_win == MAG_NEW_MASTER_WINDOW)
    {TRACE(printf("MAG_NEW_MASTER_WINDOW...\n"))
    if ((err = win_setup_multiple_win(win,NULL)) != NO_ERROR) return err;
    }
   else if (magnify_which_win == MAG_NEW_SLAVE_WINDOW)
    {
     dimension = win->dwin->screen.dimension;
     if ((err = win_setup_multiple_win(win,win->dwin)) != NO_ERROR) return err;
     win->dwin->screen.dimension = dimension;
    }
 win->dwin->screen.dimension.magrect.left = magx;
 win->dwin->screen.dimension.magrect.top = magy;
 if (center_win) center_win = DIM_CENTER_AT_POINT;

 if ((err = setup_screen_pixel_desc_rec1(&(win->dwin->screen.last_pixel),NULL,
              (unsigned int) magx,(unsigned int) magy)) != NO_ERROR)
    return err;
 if (magnify_which_win == MAG_NEW_SLAVE_WINDOW)
    {
     win->dwin->screen.dimension.rr = win->dwin->screen.dimension.limitboundrect;
     win->dwin->screen.dimension.rr.width = 256;
     win->dwin->screen.dimension.rr.height = 256;
/*
     setup_int_rect_rec(&(dim->rr),dim->rr.left,dim->rr.top,
         dim->magrect.left * (dim->bound.right - dim->bound.left + 1),
         dim->magrect.top * (dim->bound.bottom - dim->bound.top + 1));
*/
    }
/*
 printf("before************\n");
 pr_int_region_rect(&(win->dwin->screen.dimension));
*/
 setup_screen_dimension(&(win->dwin->screen.dimension),
      &(win->dwin->screen.last_dimension),
      &(win->dwin->screen.pixel),NULL,
      scale_option,center_win | DIM_OPR_SAME_DIM_MAG_FACTOR,center_x,center_y);
/*
 printf("\nafter************\n");
 pr_int_region_rect(&(win->dwin->screen.dimension));
*/
 win->dwin->screen.last_dimension = win->dwin->screen.dimension; 
 win_set_frame_label(win);

 if ((err = WinColorCanvas(win)) != NO_ERROR) return err;
 return NO_ERROR;
}



/*---------------------------------------------------------------------------
win_display_win_proc - ???
---------------------------------------------------------------------------*/

int
win_display_win_proc(inter,which)
 struct intermediate_rec *inter;
 int which;
{
 (void) notify_set_itimer_func((Notify_client) inter,
             NOTIFY_FUNC_NULL,ITIMER_REAL,
             (struct itimerval *) 0,(struct itimerval *) 0);
 inter->win->dwin = inter->dwin;
 if (inter->display_cur_win)
    {
     WinColorCanvas(inter->win);
    }
 xv_set(inter->win->dwin->frame,XV_SHOW,TRUE,0);
 wmgr_top(inter->win->dwin->frame);

 if (inter->center_win) 
    win_move_mouse_to(inter->win,inter->center_x,inter->center_y);
   else 
    win_set_mouse_at_center(inter->win->dwin->frame);
 return NO_ERROR;
}



/*---------------------------------------------------------------------------
redisplay_cur_win - ???
---------------------------------------------------------------------------*/

int
redisplay_cur_win(win,center_win,center_x,center_y,is_display_cur_win)
 struct image_display_window *win;
 int center_win,center_x,center_y,is_display_cur_win;
{
 static struct intermediate_rec inter;

 inter.win = win;
 inter.dwin = win->dwin;
 inter.center_win = center_win;
 inter.center_x = center_x;
 inter.center_y = center_y;
 inter.display_cur_win = is_display_cur_win;
 (void) notify_set_itimer_func((Notify_client) &inter,
        (Notify_func) win_display_win_proc,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);

 return NO_ERROR;
}



/*---------------------------------------------------------------------------
WinSetupMagnifyWindow - ???
---------------------------------------------------------------------------*/

int
WinSetupMagnifyWindow(win,mag_factorx,mag_factory,magnify_which_win,
                     center_win,center_x,center_y)
 struct image_display_window *win;
 int mag_factorx,mag_factory;
 int magnify_which_win;
 int center_win,center_x,center_y;
{
 int get_win,err,is_display_cur_win;
 Frame setup_fr_win;

 PRT3(printf("WinSetupMagnifyWindow...win->dwin = %u\n",win->dwin))
 setup_fr_win = win->dwin->frame;
 get_win = 1;
 err = NO_ERROR;
 if (magnify_which_win == MAG_NEW_SLAVE_WINDOW)
    {
     if (win->dwin->slave != NULL) return NO_ERROR;
     TRACE(printf("magnify_which_win == MAG_NEW_SLAVE_WINDOW...\n"))
    }
   else if (magnify_which_win == MAG_CURRENT_WINDOW)
    {
     if ((win->dwin->screen.dimension.magrect.left != mag_factorx) ||
         (win->dwin->screen.dimension.magrect.top != mag_factory))
        {
         win->dwin->screen.dimension.magrect.left = mag_factorx;
         win->dwin->screen.dimension.magrect.top = mag_factory;
         if (center_win) center_win = DIM_CENTER_AT_POINT;
         setup_screen_dimension(&(win->dwin->screen.dimension),
            &(win->dwin->screen.last_dimension),
            &(win->dwin->screen.pixel),&(win->dwin->screen.last_pixel),
            DIM_FIXED_MAG,center_win | DIM_OPR_SAME_DIM_MAG_FACTOR,
            center_x,center_y);
        }
       else return NO_ERROR;
     get_win = 0;
   }
 PRT3(printf("WinSetupMagnifyWindow...2\n"))
 if (get_win)
    {
     err = win_get_multiple_win(win,magnify_which_win,
           mag_factorx,mag_factory,
          center_win,center_x,center_y,DIM_FIXED_MAG);
     if (err != NO_ERROR) return err;
     make_2_wins_adjacent(win->dwin->frame,setup_fr_win,1);
     is_display_cur_win = 0;
    }
   else is_display_cur_win = 1;
 redisplay_cur_win(win,center_win,center_x,center_y,is_display_cur_win);
 return NO_ERROR;
}



/*---------------------------------------------------------------------------
WinImageType - ???
---------------------------------------------------------------------------*/

int
WinImageType(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 PRT2(printf("WinImageType img->status4 = %x\n",img->status4))
 if ((img->status4 & IMAGE_POS_NEG_ACTIVATION) == IMAGE_POS_NEG_ACTIVATION)
    {PRT2(printf("WinImageType IMAGE_POS_NEG_ACTIVATION\n"))
     win->status4 |= WIN_POS_NEG_ACTIVATION;}
   else if (img->status4 & IMAGE_POS_ACTIVATION)
    {PRT2(printf("WinImageType IMAGE_POS_ACTIVATION\n"))
     win->status4 |= WIN_POS_ACTIVATION;}
   else 
    {PRT2(printf("WinImageType WIN_NEG_ACTIVATION\n"))
     win->status4 |= WIN_NEG_ACTIVATION;}
 return NO_ERROR;
}



/*---------------------------------------------------------------------------
SetupDisplayWindow - ???
---------------------------------------------------------------------------*/

int
SetupDisplayWindow(next_img,owner_frame,win_type2,def_win)
     struct img_buf_rec *next_img;
     Frame owner_frame;
     unsigned short win_type2;
     struct image_display_window *def_win;
{
  struct win_rec *the_owner;
  int err;


  if (next_img == NULL)
    return NO_ERROR;
  the_owner = next_img->win;
  if  (the_owner == NULL)
    return NO_ERROR;
  if (next_img->win->image_display == NULL)
  {
    if ((err = get_ptr(&(next_img->win->image_display),NULL,1,
		       (size_t) sizeof(*(next_img->win->image_display))))
	!= NO_ERROR)
      return err;
    init_image_display_window(next_img->win->image_display);
    next_img->win->image_display->type2 = win_type2;
    if (win_type2 & HISTO_1D_WINDOW)
      next_img->status4 |= IMG_HISTO_1D;
    else if (win_type2 & HISTO_2D_WINDOW)
      next_img->status4 |= IMG_HISTO_2D;
  }

  TRACE(printf("\nSetupDisplayWindow()\n"));

  if ((err = SetupImageDisplayWindow(next_img->win->image_display,
				     the_owner,owner_frame,def_win))
      != NO_ERROR)
    return err;
  else if (next_img->file->hdr.data_info.var_use_status & CHANGES_ACTIVATION)
    WinImageType(next_img->win->image_display);
  if (def_win == NULL)
    WinDefaultColorScaleLabels(next_img->win->image_display);
  else
    WinSetDefaultScaleLabels(next_img->win->image_display,def_win);

  TRACE(printf("SetupDisplayWindow() end\n\n"));
  return NO_ERROR;
}



/*---------------------------------------------------------------------------
build_all_echo_wins - ???
---------------------------------------------------------------------------*/

int
build_all_echo_wins(img,win_type2)
     struct img_buf_rec *img;
     unsigned short win_type2;
{
  struct img_buf_rec *next_img;
  Frame *frame;
  struct list_rec *cur;
  int err;


  next_img = img = pick_first_img_buf_rec_partner(img);

  if ((err = RdCurSliceEchoes(img,0,0)) != NO_ERROR)
    return err;
  frame = (Frame *) main_display(2,NULL,0,0);
  cur = NULL;
  do
  {
    if ((err = SetupDisplayWindow(next_img,*frame,win_type2, NULL))
	!= NO_ERROR)
      return err;

    TRACE(printf("\nbuild_all_echo_wins(), next_img = %p\n",
		 (void *)next_img);
	  printf("	next_img->win = %p\n",(void *)next_img->win);
	  if (next_img->win != NULL)
	  {
	    printf("	image_display = %p\n",
		   (void *) next_img->win->image_display);
	  });
    
    if ((next_img->win != NULL) &&
	(next_img->win->image_display != NULL))
    {
      if (cur != NULL)
	add_item_after_cur_list(cur,&(next_img->win->image_display->partner));
      cur = &(next_img->win->image_display->partner);
    }
    next_img = next_img->partner;
  }
  while (next_img != img);

  TRACE(printf("build_all_echo_wins() end\n"));
  return NO_ERROR;
}


/*---------------------------------------------------------------------------
win_setup_windows - ???
---------------------------------------------------------------------------*/

int
win_setup_windows( owner, setup_fr_win, direction, status )
     struct win_rec *owner;
     Frame setup_fr_win;
     int direction;
     unsigned short status;
{
  struct img_buf_rec *img;
  int err;
  Frame setup_win;

  setup_win = setup_fr_win;
  TRACE(printf("win_setup_windows(owner)\n"));
  if (owner == NULL)
    return CANCEL;
  if ((img = win_rec_cur_img_buf(owner)) == NULL)
    return CANCEL;
  TRACE(printf("win_setup_windows(owner)...1\n"));


  build_all_echo_wins(img,status);
  TRACE(printf("win_setup_windows(owner)...2\n"));
  if ((img != NULL) && (img->win != NULL) &&
      (img->win->image_display != NULL))
  {
    PRT2(printf("img->win->image_display->status = %x\n",
		(unsigned long) img->win->image_display->status4));
    if (img->win->image_display->dwin->frame != 0)
    {
      TRACE(printf("win_setup_windows(owner)...3\n"));
      err = WinMultiEchoNewColorMultiCanvas(img->win->image_display);
      TRACE(printf("win_setup_windows(owner)...4..err = %d\n",err));
      if (err != NO_ERROR)
	return err;
      WinMultiEchoRunProc(img->win->image_display,
			  position_partner_wins,&setup_win,&direction);
      WinMultiEchoRunProc(img->win->image_display,
			  win_multiple_set_frame_label,0,0);

      TRACE(printf("win_setup_windows(owner)...5\n"));
      (void) notify_set_itimer_func((Notify_client) img->win->image_display,
				    (Notify_func) win_show_partner_windows_proc,ITIMER_REAL,
				    &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
      TRACE(printf("win_setup_windows.. 6\n"));
    }
  }
  return NO_ERROR;
}



/*---------------------------------------------------------------------------
win_rec_setup_histogram - ???
---------------------------------------------------------------------------*/

int
win_rec_setup_histogram(win,win_type2)
 struct win_rec *win;
 unsigned short win_type2;
{
 int err,yes;
 Frame *frame;

 err = CANCEL;
 if (win->image_display != NULL)
    {
     if (win->histogram == NULL)
        {TRACE(printf("create win_rec_setup_histogram...\n"))
         if ((err = get_ptr(&(win->histogram),NULL,
               1,(size_t) sizeof(*(win->histogram)))) != NO_ERROR)
            return err;
         init_image_display_window(win->histogram);
         win->histogram->type2 = win_type2;
         frame = (Frame *) main_display(2,NULL,0,0); /* return main frame ptr */

         TRACE(printf("win_rec_setup_histogram...1\n"))
         if ((err = SetupImageDisplayWindow(win->histogram,
             win,*frame,NULL)) != NO_ERROR)
            {free_image_display_window(win->histogram);
             free_ptr(&(win->histogram));
             return err;}
         TRACE(printf("win_rec_setup_histogram.1..type2 = %x dp = %x anal = %x  resize = %d\n",
                 (unsigned long) win->histogram->type2,
                 (unsigned long) win->histogram->dwin->display_status,
                 (unsigned long) win->histogram->dwin->analysis_status,
                 (unsigned long) win->histogram->dwin->resize_status))

         TRACE(printf("win_rec_setup_histogram...2\n"))
         win->histogram->dwin->screen.dimension.magrect.top = 1;
         win->histogram->dwin->screen.dimension.magrect.left = 1;
         setup_screen_dimension(&(win->histogram->dwin->screen.dimension),
            &(win->histogram->dwin->screen.last_dimension),
            &(win->histogram->dwin->screen.pixel),
            &(win->histogram->dwin->screen.last_pixel),
             0,0,0,0);
         TRACE(printf("win_rec_setup_histogram...3\n"))

         yes = 1;
         TRACE(printf("win_rec_setup_histogram...4\n"))
        }
       else yes = 0;
     if ((win->histogram != NULL) && (win->histogram->num_display_wins > 0) &&
         (win->histogram->dwin->frame != 0))
        {
         TRACE(printf("win_rec_setup_histogram.1..type2 = %x dp = %x anal = %x  resize = %d\n",
                 (unsigned long) win->histogram->type2,
                 (unsigned long) win->histogram->dwin->display_status,
                 (unsigned long) win->histogram->dwin->analysis_status,
                 (unsigned long) win->histogram->dwin->resize_status))
         err = WinHistoNewColorMultiCanvas(win->histogram);
         TRACE(printf("win_rec_setup_histogram...6..err = %d\n",err))
         if (err != NO_ERROR) return err;
         if (yes) make_2_wins_adjacent(win->histogram->dwin->frame,
                              win->image_display->dwin->frame,1);
         
         win_multiple_set_frame_label(win->histogram);
         xv_set(win->histogram->dwin->frame,XV_SHOW,TRUE,0);
         TRACE(printf("win_rec_setup_histogram...8\n"))
        }
    }
 return err;

}



/*---------------------------------------------------------------------------
WinShowNewWin - ???
---------------------------------------------------------------------------*/

int
WinShowNewWin(newwin,next_to_frame,def_win)
 struct image_display_window *newwin,*def_win;
 Frame next_to_frame;
{
 int err,direction;
 Frame next_to_frame1;

 PRT1(printf("WinShowNewWin...\n"))
 if ((err = WinNewColorData(newwin)) != NO_ERROR)
    return err;
 if (def_win != NULL)
    if (def_win->intensity == newwin->intensity)
       {PRT3(printf("WinShowNewWin...1\n"))
        win_setup_default_local_color_indxs(newwin,def_win);}

 PRT1(printf("WinShowNewWin...2\n"))
 WinMultiScreenCanvas(newwin);

 direction = EAST;
 next_to_frame1 = next_to_frame;
 win_execute_proc_to_partner_wins(newwin,
                      position_partner_wins,&next_to_frame1,&direction);
 PRT1(printf("WinShowNewWin...5\n"))
 WinMultiEchoRunProc(newwin,win_multiple_set_frame_label,0,0);
 (void) notify_set_itimer_func((Notify_client) newwin,
             (Notify_func) win_show_partner_windows_proc,ITIMER_REAL,
             &NOTIFY_POLLING_ITIMER,(struct itimerval *) 0);
 PRT1(printf("WinShowNewWin...6\n"))
 return NO_ERROR;
}



/*---------------------------------------------------------------------------
GetNewDisplayWin - ???
---------------------------------------------------------------------------*/

int
GetNewDisplayWin(win,img1,win_type2,newwin)
 struct image_display_window *win,**newwin;
 struct img_buf_rec *img1;
 unsigned short win_type2;
{
 Frame *frame;
 int err;

 TRACE(printf("GetNewDisplayWin...1\n"))
 frame = (Frame *) main_display(2,NULL,0,0);
 if (((err = SetupDisplayWindow(img1,*frame,win_type2,win)) != NO_ERROR) ||
     ((err = RdSlice(img1,0,NULL,0)) != NO_ERROR))
    {if (newwin != NULL) *newwin = NULL;
     return err;
    }
   else
    {
     if ((err = WinShowNewWin(img1->win->image_display,win->dwin->frame,win)) !=
               NO_ERROR)
        return err;
     if (newwin != NULL) *newwin = img1->win->image_display;
    }
 return err;
}



/*---------------------------------------------------------------------------
NewWin - ???
---------------------------------------------------------------------------*/

int
NewWin(win,win_type2,use_this_buf_dims,dimx,dimy,pixel_depth_bits,data_type,
       img_type4,newwin)
 struct image_display_window *win,**newwin;
 unsigned short win_type2;
 int use_this_buf_dims;
 unsigned long dimx,dimy,pixel_depth_bits,data_type,img_type4;
{
 struct img_buf_rec *img,*img1;
 int err;

 PRT1(printf("NewWin...\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (win_type2 & DISPLAY_WINDOW)
    {img1 = NewImgBuf(&(img->file->img),img->cur_slice,&err,1,0,0,0,0,0,0);
     if (err != NO_ERROR) return err;
     img1->status4 |= img->status4 & IMAGE_POS_NEG_ACTIVATION;
    }
   else
    {if (((img1 = NewImgBuf(&(img->file->img),img->cur_slice,&err,1,
                 use_this_buf_dims,dimx,dimy,pixel_depth_bits,
                 data_type,0)) == NULL) ||
         (err != NO_ERROR))
        return err;
    }
 img1->status4 |= img_type4;
 PRT1(printf("new_win...1\n"))
 if ((err = GetNewDisplayWin(win,img1,win_type2,newwin)) != NO_ERROR)
    delete_img_and_partner_bufs(img1);
 PRT1(printf("new_win...err = %d\n",err))
 return err;
}



/*---------------------------------------------------------------------------
NewWinAllSlicesHisto - ???
---------------------------------------------------------------------------*/

int
NewWinAllSlicesHisto(win,win_type2,
		     use_this_buf_dims,dimx,dimy,
		     pixel_depth_bits,
                     data_type,img_type4,
		     histo_option,histo_kind,newwin)
     struct image_display_window *win,**newwin;
     unsigned short win_type2;
     int use_this_buf_dims,histo_option,histo_kind;
     unsigned long dimx,dimy,pixel_depth_bits,data_type,img_type4;
{
 struct img_buf_rec *img,*img1;
 Frame *frame;
 int err;

 PRT3(printf("NewWinAllSlicesHisto...1\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if (((img1 = NewImgBuf(&(img->file->img),img->cur_slice,&err,1,
             use_this_buf_dims,dimx,dimy,pixel_depth_bits,data_type,0)) == NULL) ||
     (err != NO_ERROR))
    return err;
 PRT3(printf("NewWinAllSlicesHisto...2\n"))
 img1->status4 |= img_type4;
 frame = (Frame *) main_display(2,NULL,0,0);
 if (((err = SetupDisplayWindow(img1,*frame,win_type2,win)) != NO_ERROR) ||
     ((err = SetupAllSlicesHisto(img1,0,NULL,0,histo_option,histo_kind)) != NO_ERROR))
    {if (newwin != NULL) *newwin = NULL;
     delete_img_and_partner_bufs(img1);
     PRT3(printf("NewWinAllSlicesHisto...err = %d\n",err))
    }
   else
    {
     PRT3(printf("NewWinAllSlicesHisto.3..err = %d\n",err))
     if ((err = WinShowNewWin(img1->win->image_display,win->dwin->frame,win)) !=
               NO_ERROR)
        return err;
     if (newwin != NULL) *newwin = img1->win->image_display;
    }
 return err;
}



/*---------------------------------------------------------------------------
win_all_win_echoes_exist - ???
---------------------------------------------------------------------------*/

int
win_all_win_echoes_exist(win)
 struct image_display_window *win;
{
 struct img_buf_rec *img,*img1;
 struct image_display_window *prior_win;
 Frame *frame,setup_win;
 int err,direction;

 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 if ((err = all_img_echoes_exist(img)) != NO_ERROR) return err;

 img1 = img->partner;
 prior_win =  win;
 frame = (Frame *) main_display(2,NULL,0,0);
 setup_win = win->dwin->frame;
 direction = EAST;
 do 
    {if (img1->win == NULL)
        {img1->win = get_new_img_win(img1,&err);
         if (err != NO_ERROR) return err;}

     if (img1->win->image_display == NULL)
        {if ((err = GetNewDisplayWin(win,img1,DISPLAY_WINDOW,NULL)) != NO_ERROR)
            {delete_img_and_partner_bufs(img1);
             return err;}
         add_item_after_cur_list(&(prior_win->partner),
                 &(img1->win->image_display->partner));
        }
     prior_win = img1->win->image_display;
     img1 = img1->partner;
    }
 while (img != img1);

 return NO_ERROR;
}



/*---------------------------------------------------------------------------
GetNumDwins - ???
---------------------------------------------------------------------------*/

int
GetNumDwins(win,num_display_wins,dwins,total_dwins,num_dwins,num_echoes,
              is_mag_win,is_same_slice,win_type2)
 struct image_display_window *win;
 int num_display_wins;
 struct display_win_rec ***dwins;
 int *total_dwins,*num_dwins,num_echoes,is_mag_win,is_same_slice;
 unsigned short win_type2;
{
 struct img_buf_rec *new_img,*img;
 Frame *frame;
 struct list_rec *cur;
 int err,lcv,is_all_echoes;
 struct hdr_scan_entry_rec *cur_slice,*next_slice;

 PRT2(printf("GetNumDwins(  wid = %s\n",win->owner->wid))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return CANCEL;
 PRT2(printf("GetNumDwins(  img->status4 = %x\n",img->status4))
 if (!is_same_slice)
    if (num_echoes > 1) cur_slice = pick_first_partner_slice(img->cur_slice);
      else cur_slice = img->cur_slice;
   else cur_slice = img->cur_slice;

 *total_dwins = 0;
 *num_dwins = 0;
 if ((err = get_ptr(dwins,total_dwins,num_display_wins,
     (size_t) sizeof(struct image_display_window *))) != NO_ERROR)
    return err;
 frame = (Frame *) main_display(2,NULL,0,0);
 lcv = 0;
 if (is_mag_win) num_display_wins -= 1;
 if (win_type2 & HISTOGRAM_WINDOW) is_all_echoes = 0;
   else if (num_echoes == 1)
    is_all_echoes = 0;
   else 
    is_all_echoes = 1;
 while ((lcv < num_display_wins) && (err == NO_ERROR))
    {
     next_slice = cur_slice;
     cur = NULL;
     if ((new_img = NewImgBuf(&(img->file->img),
                     next_slice,&err,1,0,0,0,0,0,is_all_echoes)) != NULL)
        do
           {PRT2(printf("new_img->status4 = %x is_all_echoes = %d\n",
                      new_img->status4,is_all_echoes))

            if (img->file->hdr.data_info.var_use_status & CHANGES_ACTIVATION)
               if (!(new_img->status4 & IMAGE_POS_NEG_ACTIVATION))
                  new_img->status4 |= (img->status4 & IMAGE_POS_NEG_ACTIVATION);
            if ((err = SetupDisplayWindow(new_img,*frame,
                         win_type2,win)) != NO_ERROR)
               break;
            if (cur != NULL) 
               add_item_after_cur_list(cur,
                      &(new_img->win->image_display->partner));
            TRACE(printf("%d)  new_img = %u cur_slice->img_label = %s num = %d\n",
             lcv,new_img,new_img->cur_slice->img_label,(int) new_img->cur_slice->num))

            if (new_img->win->image_display->dwin == NULL)
               (*dwins)[lcv++] = get_valid_win_dwin(new_img->win->image_display);
              else
               (*dwins)[lcv++] = new_img->win->image_display->dwin;

            if (lcv == num_display_wins) break;
            if ((num_echoes > 1) && !is_same_slice)
               {cur = &(new_img->win->image_display->partner);
                new_img = new_img->partner;
                next_slice = next_slice->partner_slice;}
              else break;
           }
        while (next_slice != cur_slice);
     if (!is_same_slice) cur_slice = cur_slice->next_slice;
    }

 if (err == NO_ERROR)
    if (is_mag_win) 
       if ((err = win_setup_multiple_win((*dwins)[0]->owner,(*dwins)[0])) == NO_ERROR)
          {(*dwins)[lcv++] = (*dwins)[0]->owner->dwin;}

 *num_dwins = lcv;
 return err;
}



/*---------------------------------------------------------------------------
project_num_dwins - ???
---------------------------------------------------------------------------*/

int
project_num_dwins(dwins,num_dwins,projected_rects,num_echoes,is_mag_win)
 struct display_win_rec **dwins;
 int num_dwins;
 struct int_rect_rec *projected_rects;
 int num_echoes,is_mag_win;
{
 struct img_buf_rec *img,*cur_img;
 int lcv,err,cnt;
 struct list_rec *list;
 int scalex,scaley;
 struct int_region_rect *dim;

 TRACE(printf("project_num_dwins num_dwins = %d\n",num_dwins);)
 for (lcv = 0; lcv < num_dwins; lcv++)
    {
     list = &(dwins[lcv]->owner->list);
     if (list->before != list->next) continue;

     cur_img = win_rec_cur_img_buf(dwins[lcv]->owner->owner);
     for (cnt = lcv + 1; cnt < num_dwins; cnt++)
        {if (dwins[cnt]->owner->list.before != 
             dwins[cnt]->owner->list.next) continue;
         if ((img = win_rec_cur_img_buf(dwins[cnt]->owner->owner)) != NULL)
            if (str_cmp(cur_img->cur_slice->img_label,
                       img->cur_slice->img_label) == 0)
               {add_item_after_cur_list(list,&(dwins[cnt]->owner->list));
                list = &(dwins[cnt]->owner->list);}
        }
    }
 /*
 for (lcv = 0; lcv < num_dwins; lcv++)
    {pr_list_rec(&(dwins[lcv]->owner->partner),NULL);
     printf("\n");}
*/
 for (lcv = 0; lcv < num_dwins; lcv++)
    {dim = &(dwins[lcv]->screen.dimension);
     set_bound_by_int_rect(&(dim->bound),&(dim->limitboundrect));
     dim->rr = projected_rects[lcv];

     if ((img = win_rec_cur_img_buf(dwins[lcv]->owner->owner)) != NULL)
        {if (img->file->hdr.data_info.dimx < 256)
            scalex = 256 / img->file->hdr.data_info.dimx;
           else scalex = 1;
         if (img->file->hdr.data_info.dimy < 256)
            scaley = 256 / img->file->hdr.data_info.dimy;
           else scaley = 1;
         if (scalex > scaley) scalex = scaley;
        }
       else
        {scalex = 1;
         scaley = 1;}
     if ((lcv < (num_dwins - 1)) || (is_mag_win == 0))
        {dim->magrect.top = 1 * scalex;
         dim->magrect.left = 1 * scalex;
         setup_screen_dimension(&(dwins[lcv]->screen.dimension),
            &(dwins[lcv]->screen.last_dimension),
            &(dwins[lcv]->screen.pixel),&(dwins[lcv]->screen.last_pixel),
             0,4 | DIM_SHRINK,0,0);
         TRACE(printf("lcv = %d\n",lcv);
         pr_int_rect_rec(&(projected_rects[lcv]));
         pr_int_region_rect(&(dwins[lcv]->screen.dimension));)}
        else
        {scalex = 3 * scalex;
         if (scalex > 4) scalex = 4;
         dim->magrect.top = scalex;
         dim->magrect.left = scalex;
/*
         pr_int_region_rect(dim);
         if (dim->limitboundrect.width >= 50)
            {dim->bound.left += 10;
             dim->bound.right -= 10;}
         if (dim->limitboundrect.height >= 50)
            {dim->bound.top += 10;
             dim->bound.bottom -= 10;}
         pr_int_region_rect(dim);
*/
         setup_screen_dimension(dim,
            &(dwins[lcv]->screen.last_dimension),
            &(dwins[lcv]->screen.pixel),&(dwins[lcv]->screen.last_pixel),
             /*DIM_FIXED_MAG,DIM_MAX_AREA,DIM_FIXED_BOUND*/
             DIM_FIXED_MAG,DIM_OPR_SAME_DIM_MAG_FACTOR |
             DIM_CENTER_AT_POINT | DIM_SHRINK | DIM_CENTER_AT_LIMITBOUNDRECT,
             0,0);

        }
    }

 for (lcv = num_dwins - 1; lcv >= 0; lcv--)
    {

     TRACE(printf("dwins[%d] = %u win = %u  num = %d",lcv,dwins[lcv],
               dwins[lcv]->owner,dwins[lcv]->owner->num_display_wins))

     dwins[lcv]->owner->dwin = dwins[lcv];
     if (dwins[lcv]->frame == 0) continue;
     img = win_rec_cur_img_buf(dwins[lcv]->owner->owner);
     RdSlice(img,0,NULL,0);
     err = WinNewColorScreen(dwins[lcv]->owner);
     xv_set(dwins[lcv]->frame,XV_SHOW,FALSE,0);
     display_data_to_canvas1(dwins[lcv]->owner,0);
    }

 for (lcv = 0; lcv < num_echoes; lcv++)
    {WinRunProcInList(WIN_LIST,dwins[lcv]->owner,
                      win_multiple_set_frame_label,0,0,0,0,0,0);
     WinRunProcInList(WIN_LIST,dwins[lcv]->owner,win_show_multiple_windows,
		      0,0,0,0,0,0);}

 return err;
}



/*---------------------------------------------------------------------------
ProjectWins - ???

num_projected_wins = 9 or 12 
---------------------------------------------------------------------------*/

int
ProjectWins(win,projected_rect,num_projected_wins,num_echoes,win_type2)
 struct image_display_window *win;
 struct int_rect_rec *projected_rect;
 int num_projected_wins;
 int num_echoes;
 unsigned short win_type2;
{
 int num_rows,total_dwins,num_dwins;
 struct display_win_rec **dwins;
 struct int_rect_rec **rr,*ptr;
 int err,lcv;
 int is_mag_win;

 PRT3(printf("ProjectWins...win_type2 = %x\n",(unsigned long) win_type2))
 num_rows = num_projected_wins / 4;
 if (num_projected_wins % 4) num_rows += 1;
 if ((err = build_2d_int_rects(&rr,num_rows,4,12,
           projected_rect->left,projected_rect->top,
           projected_rect->width,projected_rect->height,1,1)) != NO_ERROR)
    return err;

 if (num_projected_wins == 9)
    {is_mag_win = 1;
     ptr = rr[0];
     ptr[6] = ptr[8];

     ptr[num_projected_wins - 1].top = ptr[5].top + 10;
     ptr[num_projected_wins - 1].left = ptr[5].left - 10;
     ptr[num_projected_wins - 1].height = ptr[5].height + ptr[5].height - 70;
     ptr[num_projected_wins - 1].width = ptr[5].width + ptr[5].width - 20;
     ptr[5] = ptr[7];
     ptr[7] = ptr[11];
    }
   else is_mag_win = 0;

 err = GetNumDwins(win,num_projected_wins,&dwins,
              &total_dwins,&num_dwins,num_echoes,is_mag_win,0,win_type2);

 if ((err == NO_ERROR) && (num_dwins > 0))
    {
     if ((err = project_num_dwins(dwins,num_dwins,rr[0],num_echoes,is_mag_win)) ==
           NO_ERROR)
        for (lcv = 0; lcv < num_dwins; lcv++)
           dwins[lcv]->owner->status4 |= STACK_WIN_COSECUTIVE;
    }

 if (total_dwins > 0)
    {
     c_free(dwins);
    }
 free_handle_memory(rr,num_rows);
 TRACE(printf("ProjectWins...end\n"))
 return err;
}



/*---------------------------------------------------------------------------
win_show_tissue_wins - ???
---------------------------------------------------------------------------*/

int
win_show_tissue_wins(win)
 struct image_display_window *win;
{
 int num_rows,total_dwins,num_dwins,num_projected_wins;
 struct display_win_rec **dwins;
 struct int_rect_rec **rr;
 int err,lcv;
 struct img_buf_rec *img;
 unsigned long indx,which;
 int is_bkg;
 struct int_rect_rec projected_rect;
 Rect *rect;

 TRACE(printf("win_show_tissue_wins...\n"))
 if ((img = win_rec_cur_img_buf(win->owner)) == NULL) return NO_ERROR;
 if (img->file->pixel_type.order.num_strs == 0) return NO_ERROR;
 num_projected_wins = img->file->pixel_type.order.num_strs;
 if ((find_str_in_strs_rec(&(img->file->pixel_type.order),
         BACKGROUND_PIXEL_TYPE_STR,&indx) == NO_ERROR) ||
     (find_str_in_strs_rec(&(img->file->pixel_type.order),"BACKGROUND.TYPE",
        &indx) == NO_ERROR))
    {is_bkg = 1;
     num_projected_wins -= 1;}
   else is_bkg = 0;
 rect = (Rect *) xv_get(win->dwin->frame,WIN_SCREEN_RECT);
 Rect_to_int_rect_rec(rect,&projected_rect);
 num_projected_wins += 2;
 if (num_projected_wins > 12) num_projected_wins = 12;

 TRACE(printf("win_show_tissue_wins...num_projected_wins = %d\n",num_projected_wins))
 num_rows = num_projected_wins / 4;
 if (num_projected_wins % 4) num_rows += 1;
 if ((err = build_2d_int_rects(&rr,3,4,12,
           projected_rect.left,projected_rect.top,
           projected_rect.width,projected_rect.height,1,1)) != NO_ERROR)
    return err;
 err = GetNumDwins(win,num_projected_wins,&dwins,
              &total_dwins,&num_dwins,1,0,1,DISPLAY_WINDOW);
 TRACE(printf("win_show_tissue_wins...err = %d\n",err);)
 if ((err == NO_ERROR) && (num_dwins > 0))
    {for (which = 0,lcv = 2; lcv < num_dwins; lcv++)
        {TRACE(printf("lcv = %d\n",lcv);pr_int_rect_rec(&(rr[0][lcv]));)
         if (is_bkg && (which == indx)) which += 1;
         TRACE(printf("which = %d add = %s\n",which,
                 img->file->pixel_type.order.strs[which]);)
         if ((err = add_str_to_strs_rec(&(dwins[lcv]->owner->data_filter_types),
                      img->file->pixel_type.order.strs[which])) == NO_ERROR)
            dwins[lcv]->owner->status4 |= DISPLAY_SLICE_TISSUE_TYPE;
         which += 1;
        }
     for (lcv = 0; lcv < num_projected_wins; lcv++)
        dwins[lcv]->owner->status4 |= DONT_APPLY_SAME_TISSUE_TO_STACK_WINS;
     TRACE(printf("win_show_tissue_wins.num_dwins = %d..err = %d\n",num_dwins,
       err))
     if ((err = project_num_dwins(dwins,num_dwins,rr[0],1,0)) == NO_ERROR)
        {WinShowPixelTypes(dwins[1]->owner,
                      (int) VALUE_SHOW_SLICE_PIXEL_TYPE);
         for (lcv = 2; lcv < num_dwins; lcv++)
            win_image_tissue_type_to_screen(dwins[lcv]->owner);}
    }
 TRACE(printf("win_show_tissue_wins.2..err = %d\n",err))

 if (total_dwins > 0)
    {
     c_free(dwins);
    }
 free_handle_memory(rr,num_rows);
 TRACE(printf("win_show_tissue_wins...end\n"))
 return err;
}
