/****************************************************************************
bbl_dicom_test.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

FIXME:  Does the "test" in the source filename mean that this tests
bbl_dicom or that this is a test version of bbl_dicom?  The function
prog_usage may hold some clues.  -PWH 19990430

FIXME:  This source file is not included in the Linux makefile.  Perhaps
for good reason?  -PWH 19990430

FIXME:  Does the following text really apply to this file, or to some
other?  -PWH 19990430

 opal2nria

 Usage:
   opal2nria DICOM.tar.Z mriMHCRCsq# [new_size min max]

 Purpose:
   Convert DICOM-format images retrieved from OPAL system to
   NRIA-format image and header file (.img and .nhdr).

 History:
 961003 DRW  First version.  Uses my bbl_dicom package
   (bbl_dicom.c, dicom_dict.c, and bbl_dicom.h).
   Favors a lot of code from Teong Thew's hup.c, with
   some bug fixes and new code to work with DICOM v3.0.
****************************************************************************/

static const char rcsid[] = "$Id: bbl_dicom_test.c,v 1.4 1999/07/07 18:07:13 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <string.h>

#include <sys/types.h>

#include "error_constants.h"
#include "typedef.h"
#include "str_def.h"
#include "ia_hdr.h"
#include "mark_hdr.h"
#include "nlib.h"
#include "bbl_dicom.h"



#if 0
#define TRACE(x) x; fflush(stdout);
#define TRACE1(x) x; fflush(stdout);
#define TRACE2(x) x; fflush(stdout);
#define PRT(x) x; fflush(stdout);
#else
#define TRACE(x)
#define TRACE1(x)
#define TRACE2(x)
#define PRT(x)
#endif

#define USE_SCALE 1
#define USE_ORIGINAL_BYTES_PER_PIXEL 2
#define USE_WINDOW_MIN_MAX 4


struct slice_rec {
  char			*inpfn, *echo_time;
  unsigned long		in_img_addr;
  unsigned long		num, echo_cnt, offset;
  unsigned long		min, max;
};

struct file_rec {
  struct the_file_rec	*root;
  char			*fn;
  FILE			*fid;
  struct hdr_rec	hdr;
  unsigned long		offset,insize,outsize,in_pixel_bytes;
  char			*image_orientation,*series;
  unsigned long		total_slices, num_slices;
  struct slice_rec	*slices;
  int			which_format;

  /*  DRW 961007  Added this for dynamic calculation of slice thickness  */
  float			sliceThickness;
};

struct the_file_rec {
  unsigned status;
  unsigned long scale_bytes,scale_min,scale_max,win_min,win_max;
  unsigned long total,num;
  struct file_rec **files;
};


/*---------------------------------------------------------------------------
init_slice_rec - ???
---------------------------------------------------------------------------*/

void
init_slice_rec(slice)
     struct slice_rec *slice;
{
  slice->inpfn = NULL;
  slice->echo_time = NULL;
}



/*---------------------------------------------------------------------------
free_slice_rec - ???
---------------------------------------------------------------------------*/

void
free_slice_rec(slice)
     struct slice_rec *slice;
{
  c_free(slice->inpfn);
  c_free(slice->echo_time);
  init_slice_rec(slice);
}



/*---------------------------------------------------------------------------
init_file_rec - ???
---------------------------------------------------------------------------*/

void
init_file_rec( file, out, transferSyntax )
     struct file_rec *file;
     struct the_file_rec *out;
     int transferSyntax;
{
  file->root = out;
  file->fn = NULL;
  file->fid = NULL;
  file->offset = file->insize = file->outsize = 0;
  file->image_orientation = NULL;
  file->series = NULL;
  file->in_pixel_bytes = 0;
  init_hdr_rec(&(file->hdr));
  file->total_slices = file->num_slices = 0;
  file->slices = NULL;
  file->which_format = transferSyntax;
  file->sliceThickness = 0.0;
}



/*---------------------------------------------------------------------------
free_file_rec - ???
---------------------------------------------------------------------------*/

void
free_file_rec(file)
     struct file_rec *file;
{
  unsigned long lcv;

  c_free(file->fn);
  if (file->fid != NULL) fclose(file->fid);
  c_free(file->image_orientation);
  c_free(file->series);
  free_hdr_rec(&(file->hdr));
  if (file->total_slices > 0) {
    for (lcv = 0; lcv < file->num_slices; lcv++)
      free_slice_rec(&(file->slices[lcv]));
    c_free(file->slices);
  }
  init_file_rec( file, NULL, 0 );
}



/*---------------------------------------------------------------------------
init_the_file_rec - ???
---------------------------------------------------------------------------*/

void
init_the_file_rec(out)
     struct the_file_rec *out;
{
  out->status = 0;
  out->scale_bytes = 0;
  out->total = out->num = 0;
  out->files = NULL;
}



/*---------------------------------------------------------------------------
free_the_file_rec - ???
---------------------------------------------------------------------------*/

void
free_the_file_rec(out)
     struct the_file_rec *out;
{
  unsigned long lcv;

  if (out->total > 0) {
    for (lcv = 0; lcv < out->num; lcv++) {
      free_file_rec(out->files[lcv]);
      c_free(out->files[lcv]);
    }
    c_free(out->files);
  }
  init_the_file_rec(out);
}



/*---------------------------------------------------------------------------
pr_header - ???
---------------------------------------------------------------------------*/

static void
pr_header( hdr, is_pr_all )
     struct hdr_rec *hdr;
     int is_pr_all;
{
  struct data_info_rec *info;

  info = &(hdr->data_info);
  printf("\n");
  printf(" SUBJECT   Name: \"%s\"\n", info->patient.name);
  printf("\n");
  printf(" STUDY     ID: %s\n",info->patient.study_num);
  printf("           Date: %s, Time: %.4s\n",
	 info->date_of_study,info->time_of_study);
  printf("\n");
  printf(" IMAGE     Dimensions: %d x %d,  %d-byte data\n",
	 info->dimx,info->dimy,info->pixel_depth_bits / 8);
  printf("           Pixel Size: %.6f mm x %.6f mm\n",
	 info->xpixel_size,info->ypixel_size);
  if (is_pr_all)
    printf("           Pixel Minimum and Maximum values: %.0f, %.0f\n",
	   info->min_pixel,info->max_pixel);
  printf("           Slice thickness: %.2f mm, spacing: %.2f mm\n",
	 info->pixel_thickness,info->spacing);
  if (is_pr_all)
    printf("           Total slices: %d\n",info->max_scans);
  printf("\n");
  printf(" PHYSICAL  Rep. time (tr): %.0f (ms), Echo time (te): %s (ms)\n",
	 info->tr,info->te);
  printf("           Echoes per slice: %d\n",info->num_echoes);
  printf("\n\n");
}



/*---------------------------------------------------------------------------
enter_slice - ???

Enter a slice for writing into the output file pointed to by the file
structure (arg 1).  infn is the input data file for this particular
slice.  in_img_addr is the data offset within that file of the pixel
data.
---------------------------------------------------------------------------*/

int
enter_slice(file,slice_num,infn,in_img_addr,hdr)
     struct file_rec *file;
     unsigned int slice_num;
     char *infn;
     unsigned long in_img_addr;
     struct hdr_rec *hdr;
{
  int err;
  unsigned long lcv,cnt,offset,temp;

  offset = file->offset;

  if (file->total_slices == file->num_slices) {
    temp = file->total_slices;
    get_ptr_memory(&(file->slices),&(file->total_slices),100,
		   (size_t) sizeof(*(file->slices)),&err);
    if (err != NO_ERROR)
      return err;
    for (lcv = temp; lcv < file->total_slices; lcv++)
      init_slice_rec(&(file->slices[lcv]));
  }

  if ((err = cp_str(&(file->slices[file->num_slices].inpfn),infn))
      != NO_ERROR)
    return err;
  trim_str_blanks(file->slices[file->num_slices].inpfn);

  if ((err = cp_str(&(file->slices[file->num_slices].echo_time),
		    hdr->data_info.te))
      != NO_ERROR)
    return err;
  trim_str_blanks(file->slices[file->num_slices].echo_time);

  file->slices[file->num_slices].in_img_addr = in_img_addr;
  file->slices[file->num_slices].num = slice_num;
  file->slices[file->num_slices].echo_cnt = hdr->data_info.num_echoes;
  file->slices[file->num_slices].offset = offset;
  file->slices[file->num_slices].min =
    (unsigned long) hdr->data_info.min_pixel;
  file->slices[file->num_slices].max =
    (unsigned long) hdr->data_info.max_pixel;
  offset += file->outsize;
  (file->num_slices)++;
  file->offset = offset;
  return NO_ERROR;
}



/*---------------------------------------------------------------------------
same_hdr - ???
---------------------------------------------------------------------------*/

int
same_hdr(file,hdr2,image_orientation,series)
     struct file_rec *file;
     struct hdr_rec *hdr2;
     char *image_orientation,*series;
{
  struct data_info_rec *info1,*info2;
  struct hdr_rec *hdr1;

  hdr1 = &(file->hdr);
  info1 = &(hdr1->data_info);
  info2 = &(hdr2->data_info);
  if ((str_cmp(file->image_orientation,image_orientation) != 0) ||
      (str_cmp(file->series,series) != 0) ||
      (str_cmp(info1->patient.name,info2->patient.name) != 0) ||
      (str_cmp(info1->patient.patient_id,info2->patient.patient_id) != 0) ||
      (str_cmp(info1->patient.study_num,info2->patient.study_num) != 0) ||
      (info2->patient.sex != info2->patient.sex) ||
      (str_cmp(info1->date_of_study,info2->date_of_study) != 0) ||
      (str_cmp(info1->time_of_study,info2->time_of_study) != 0) ||
      (info1->pixel_thickness != info2->pixel_thickness) ||
      (info1->spacing != info2->spacing) ||
      (info1->dimx != info2->dimx) ||
      (info1->dimy != info2->dimy) ||
      (info1->pixel_depth_bits != info2->pixel_depth_bits))
    return 0;

  return 1;

#if 0
  (info1->pixel_depth_bits != info2->pixel_depth_bits) ||
    (info1->tr != info2->tr) ||
      (str_cmp(info1->te,info2->te) != 0))
(info1->pixel_depth_bits != info2->pixel_depth_bits) ||
(info1->num_echoes != info2->num_echoes))
#endif
}



/*---------------------------------------------------------------------------
enter_file - ???
---------------------------------------------------------------------------*/

int
enter_file( message,
	   out, hdr,
	   slice_num, image_orientation, series,
           outfn, infn,
	   in_img_addr, in_img_len )
     MessagePtr message;
     struct the_file_rec *out;
     struct hdr_rec *hdr;
     unsigned int slice_num;
     char *image_orientation,*series;
     char *outfn,*infn;
     unsigned long in_img_addr,in_img_len;
{
  unsigned long		lcv,temp;
  char			found;
  int			err;
  char			str[250];
  struct file_rec	*file;
  static float		sliceLocation[2];
  ElementPtr		currElement;
  static int		numSlice;


  /*
   *  Match this file's header with previously created output headers.
   *    If they match (which means we're still working on the same image
   *    set), but the image length is different (which means different
   *    slices of the same set had different sizes), return an error.
   *
   *  If they match then add this slice to the matching output file's
   *    list of slices to process.
   */
  for (found = 0,lcv = 0; lcv < out->num; lcv++) {

    /*
     *  The out->files[] array contains a list of output files.
     *    This is usually only one, but may be more if the headers
     *    in the various input messages differ.
     */
    if (same_hdr(out->files[lcv],hdr,image_orientation,series)) {

      if (out->files[lcv]->insize > in_img_len) {
	TRACE(printf("enter_file(1)\n"););
	return ~NO_ERROR;
      }

      /*
       *  DRW 961007: Check if slice thickness exists.  If not, we need
       *    to `guestimate' it, either by subtracting slice locations, or
       *    if that fails, then we need to use a proprietary method (e.g.
       *    check for the specific protocol that was run).
       */
      if (out->files[lcv]->sliceThickness == 0.0) {

	/*  If slice thickness already read in, just use that value.  */
	if (hdr->data_info.pixel_thickness != 0)
	  out->files[lcv]->sliceThickness = hdr->data_info.pixel_thickness;

	else {

	  /*
	   *  If not, we need to collect two slice location values, and
	   *  subtract.  This is really giving us slice gap; any other
	   *  ideas?
	   */
	  currElement = findElement( message, GstorageInfo, EsliceLocation );
	  if (!currElement ||
	      (sscanf( (CONST char *) currElement->value.ptr, "%f",
		      &sliceLocation[numSlice++] ) != 1)) {
	    /*
	     *  Is there any other way of getting slice thickness?
	     *    Put that algorithm to use right here!
	     *
	     *  The following is VERY implementation dependent.  3D
	     *    SPGR is used for 1mm acquisition only, as of
	     *    October 1996.  A sequence name of "fse" signifies
	     *    a fast-spin-echo protocol which was acquired at
	     *    3mm thickness, as opposed to the standard spin-echo
	     *    acquisition at 5mm, again as of October 1996.
	     */

	    currElement = findElement( message, GphysicalInfo, Emr_acq_type );
	    if (currElement && !(strcmp(currElement->value.ptr,"3D")))
	      out->files[lcv]->sliceThickness = 1.0;
	    else {
	      currElement = findElement( message, GphysicalInfo,
					Esequence_name );
	      if (currElement &&
		  !(strncasecmp(currElement->value.ptr,"fse",3)))
		out->files[lcv]->sliceThickness = 3.0;
	      else
		out->files[lcv]->sliceThickness = 5.0;
	    }
	  }
	  else if (numSlice == 2) {
	    out->files[lcv]->sliceThickness =
	      sliceLocation[1] - sliceLocation[0];
	  }
	}

      }  /*  If we have not yet retrieved a value for slice thickness...  */

      if ((err = enter_slice(out->files[lcv],slice_num,
			     infn,in_img_addr,hdr)) != NO_ERROR) {
	TRACE(printf("enter_file(2)\n"););
	return err;
      }
      return NO_ERROR;
    }
  }

  /*
   *  If we haven't matched any previously created output headers, then
   *    start a new one (this also happens when we get called with the
   *    very first slice, as a side effect).
   */
  if (out->total == out->num) {
    get_handle_memory(&(out->files),&(out->total),1,
		      (size_t) sizeof(struct file_rec),&err);
    if (err != NO_ERROR) {
      TRACE(printf("enter_file(3)\n"););
      return err;
    }
  }
  init_file_rec( out->files[out->num], out, message->transferSyntax );

  /*  insize = dimx * dimy * (pixel_depth_bits / 8)  */
  out->files[out->num]->insize = (unsigned long) slice_size(hdr);
  if (out->files[out->num]->insize > in_img_len) {
    TRACE(printf("enter_file(4): %lu > %lu\n", out->files[out->num]->insize,
		 in_img_len););
    free_file_rec(out->files[out->num]);
    return ~NO_ERROR;
  }

  /*  in_pixel_bytes = (pixel_depth_bits / 8)  */
  out->files[out->num]->in_pixel_bytes = 
    GetPixelBytes(hdr->data_info.pixel_depth_bits);
  if (out->status & USE_ORIGINAL_BYTES_PER_PIXEL) {
    out->scale_bytes = out->files[out->num]->in_pixel_bytes;
    if (check_minmax_range(out) != NO_ERROR) {
      free_file_rec(out->files[out->num]);
      TRACE(printf("enter_file(5)\n"););
      return ~NO_ERROR;
    }
    out->scale_bytes = 0;
  }

  if ((err = cp_str(&(out->files[lcv]->image_orientation),image_orientation))
      != NO_ERROR) {
    TRACE(printf("enter_file(6)\n"););
    return err;
  }
  if ((err = cp_str(&(out->files[lcv]->series),series))
      != NO_ERROR) {
    TRACE(printf("enter_file(7)\n"););
    return err;
  }

  out->files[out->num]->hdr = *hdr;
  init_hdr_rec(hdr);

  if (out->files[out->num]->root->scale_bytes) {
    temp = out->files[out->num]->hdr.data_info.pixel_depth_bits;
    out->files[out->num]->hdr.data_info.pixel_depth_bits = 
      out->files[out->num]->root->scale_bytes * 8;
    out->files[out->num]->outsize = 
      (unsigned long) slice_size(&(out->files[out->num]->hdr));
    out->files[out->num]->hdr.data_info.pixel_depth_bits = temp;
  }
  else
    out->files[out->num]->outsize = out->files[out->num]->insize;

  if (out->num == 0)
    sprintf(str,"%s.img",outfn);
  else
    sprintf(str,"%s_%-d.img",outfn,out->num);

  if (((err = enter_slice(out->files[out->num],slice_num,
			  infn,in_img_addr,&(out->files[out->num]->hdr)))
       != NO_ERROR) ||
      ((err = cp_str(&(out->files[out->num]->fn),str)) != NO_ERROR)) {
    free_file_rec(out->files[out->num]);
    TRACE(printf("enter_file(8)\n"););
    return err;
  }

  out->num += 1;
  return NO_ERROR;
}



/*---------------------------------------------------------------------------
wr_acr_img_data - ???
---------------------------------------------------------------------------*/

int
wr_acr_img_data(file)
 struct file_rec *file;
{
 FILE *fid;
 unsigned char *data,*outdata;
 unsigned long num,lcv,total;
 unsigned long min,max;
 char *reason;
 int err;
 struct data_info_rec *info;

 if (file->fid == NULL)
    {
     if ((file->fid = fopen(file->fn,"w")) == NULL)
        {printf("....Open output file error...%s\n",file->fn);
         file_access(file->fn,2,NULL,&reason);
         printf("....Error reason : %s\n",reason);
         c_free(reason);
         return FILE_ACCESS_ERROR;
        }
    }
 total = 0;
 get_ptr_memory(&data,&total,file->insize,(size_t) 1,&err);
 if (err != NO_ERROR) return err;

 if ((file->root->scale_bytes) && (file->outsize > file->insize))
    {total = 0;
     get_ptr_memory(&outdata,&total,file->outsize,(size_t) 1,&err);
     if (err != NO_ERROR)
        {c_free(outdata);
         return err;}
    }
   else outdata = data;
 info = &(file->hdr.data_info);

 for (lcv = 0; lcv < file->num_slices; lcv++)
    {
     if ((fid = fopen(file->slices[lcv].inpfn,"r")) == NULL)
        {printf("input file open error...fn = %s\n",file->slices[lcv].inpfn);
         err = FILE_ACCESS_ERROR;
         break;}

#if 0
     printf("...%d) reading slice# %d - %s...SLICE Min = %d  max = %d\n",
            lcv + 1,file->slices[lcv].num,file->slices[lcv].inpfn,
            file->slices[lcv].min,file->slices[lcv].max);
#else
     printf("%d) Slice #%03d, \"%s\"\r",
	    lcv + 1,
            file->slices[lcv].num,
	    file->slices[lcv].inpfn);
     fflush(stdout);
#endif

     fseek(fid,file->slices[lcv].in_img_addr,0);
     if (fread(data,file->insize,1,fid) != 1)
        {printf("read error...fn = %s\n",file->slices[lcv].inpfn);
          fclose(fid);
         err = READ_ERROR;
         break;}
     fclose(fid);

     num = file->hdr.data_info.dimx * file->hdr.data_info.dimy;

     if (file->which_format == LITTLE_ENDIAN)
        if (file->in_pixel_bytes == 2)
           {dec2sun_word(data,num);}
          else if (file->in_pixel_bytes == 4)
           {dec2sun_long(data,num);}

     if (file->root->scale_bytes)
        {
         if (file->root->scale_bytes <= file->in_pixel_bytes)
            {
             scale_values(outdata,outdata,info->dimx,info->dimy,
                 (unsigned long) info->min_pixel,
                 (unsigned long) info->max_pixel,
                 file->root->scale_min,file->root->scale_max,
                 file->in_pixel_bytes);
             get_1d_ary_min_max(outdata,num,&min,&max,file->in_pixel_bytes);
             if (file->root->scale_bytes < file->in_pixel_bytes)
                assign_values(outdata,file->root->scale_bytes,
                          data,file->in_pixel_bytes,num);
             get_1d_ary_min_max(outdata,num,&min,&max,file->root->scale_bytes);

            }
           else
            {assign_values(outdata,file->root->scale_bytes,
                          data,file->in_pixel_bytes,num);
             scale_values(outdata,outdata,info->dimx,info->dimy,
                 (unsigned long) info->min_pixel,
                 (unsigned long) info->max_pixel,
                 file->root->scale_min,file->root->scale_max,
                 file->root->scale_bytes);
            }
        }

     if (fwrite(outdata,file->outsize,1,file->fid) != 1)
        {err = ferror(file->fid);
         printf("...err = %d occurred when writing to %s\n",err,file->fn);
         err = WRITE_ERROR;
         break;}
    }

 printf ("\n");

 fclose(file->fid);
 if (outdata != data) c_free(outdata);
 c_free(data);
 return err;
}


/*---------------------------------------------------------------------------
sort_echo_cnt_proc - ???
---------------------------------------------------------------------------*/

int
sort_echo_cnt_proc(s1,s2)
 struct slice_rec **s1,**s2;
{
 if ((*s1)->echo_cnt > (*s2)->echo_cnt) return 1;
 if ((*s1)->echo_cnt < (*s2)->echo_cnt) return -1;
 return 0;
}


/*---------------------------------------------------------------------------
sort_slice_rec_proc - ???
---------------------------------------------------------------------------*/

int
sort_slice_rec_proc(s1,s2)
 struct slice_rec *s1,*s2;
{
 if (s1->num > s2->num) return 1;
 if (s1->num < s2->num) return -1;
 if (s1->echo_cnt > s2->echo_cnt) return 1;
 if (s1->echo_cnt < s2->echo_cnt) return -1;
 return 0;
}


/*---------------------------------------------------------------------------
sort_slice_rec_max_proc - ???
---------------------------------------------------------------------------*/

int
sort_slice_rec_max_proc(s1,s2)
 struct slice_rec *s1,*s2;
{
 if (s1->max < s2->max) return 1;
 if (s1->max > s2->max) return -1;
 return 0;
}


/*---------------------------------------------------------------------------
rearrange_slice_num - ???
---------------------------------------------------------------------------*/

int
rearrange_slice_num(file)
 struct file_rec *file;
{
 unsigned long lcv,num_slices,total,cnt;
 struct slice_rec *slices;
 int err;

 num_slices = file->num_slices / file->hdr.data_info.num_echoes;
 file->num_slices = num_slices * file->hdr.data_info.num_echoes;
 if (file->num_slices == 0) return CANCEL;
 total = 0;
 get_ptr_memory(&slices,&total,file->num_slices,
   (size_t) sizeof(*slices),&err);
 if (err != NO_ERROR) return err;
 for (cnt = 0,lcv = 0; lcv < num_slices; lcv++)
    {slices[cnt++] = file->slices[lcv];
     slices[cnt++] = file->slices[num_slices + lcv];
    }
 for (cnt = 0,lcv = 0; lcv < file->num_slices; lcv++)
    file->slices[lcv] = slices[lcv];
 c_free(slices);
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
find_num_echoes_and_echo_time - ???
---------------------------------------------------------------------------*/

int
find_num_echoes_and_echo_time(file)
 struct file_rec *file;
{
 unsigned long lcv,total,cnt;
 struct slice_rec **slices;
 int err;
 char *te,*str;

 total = 0;
 get_ptr_memory(&slices,&total,file->num_slices,
   (size_t) sizeof(*slices),&err);
 if (err != NO_ERROR) return err;
 for (lcv = 0; lcv < file->num_slices; lcv++)
    {slices[lcv] = &(file->slices[lcv]);
    }
 qsort(slices,file->num_slices,
        (size_t) sizeof(*slices),sort_echo_cnt_proc);

 file->hdr.data_info.num_echoes = 0;
 te = NULL;
 for (lcv = 0; lcv < file->num_slices; lcv++)
    if (slices[lcv]->echo_cnt > file->hdr.data_info.num_echoes)
       {file->hdr.data_info.num_echoes = slices[lcv]->echo_cnt;
        if (te == NULL)
           {if ((err = cp_str(&te,slices[lcv]->echo_time)) != NO_ERROR)
               return err;
           }
          else
           {str = te;
            if ((err = concat_strs(&te,3,str,"/",slices[lcv]->echo_time)) != NO_ERROR)
               {c_free(str);
                return err;}
            c_free(str);}
        }
 c_free(file->hdr.data_info.te);
 file->hdr.data_info.te = te;

 c_free(slices);

 return NO_ERROR;
}



/*---------------------------------------------------------------------------
save_file_rec - ???

This gets called from save_the_file_rec() for each output file to
be created.
---------------------------------------------------------------------------*/

int
save_file_rec( file )
     struct file_rec *file;
{
  unsigned long lcv;
  int err;
  char fn[250],hdrfn[250];
  struct mark_hdr mark;
  unsigned long min,max,num;


  if (file->num_slices == 0)
    return CANCEL;

  /*  DRW 960323: Removed display of min/max for first few slices  */

  /*
   *  Sort slices by internal slice numbering.
   */
  if (file->num_slices > 1) {
    qsort( file->slices, file->num_slices,
	  (size_t) sizeof(*(file->slices)),
	  sort_slice_rec_proc );
#if 0
    if (file->hdr.data_info.num_echoes > 1)
      if ((err = rearrange_slice_num(file)) != NO_ERROR)
	return err;
#endif
  }

  sprintf(fn,"%s",file->fn);
  drop_suffix(fn);
  sprintf(fn,"%s.nhdr",fn);

  file->hdr.data_info.max_scans = file->num_slices;
  if (file->root->status & USE_ORIGINAL_BYTES_PER_PIXEL)
    file->root->scale_bytes = file->in_pixel_bytes;
  if (file->root->scale_bytes)
    file->hdr.data_info.pixel_depth_bits = file->root->scale_bytes * 8;

  /*
   *  DRW 961007
   *  Note: NRIA package doesn't support varying slice thickness,
   *    therefore this variable lies within the global header structure
   *    (as opposed to within the individual slice structures).
   */
  if (file->hdr.data_info.pixel_thickness == 0.0)
    file->hdr.data_info.pixel_thickness = file->sliceThickness;

  if (((err = build_extra_file_info_rec(&(file->hdr.extra_info)))
       != NO_ERROR) ||
      ((err = BuildRiaHdr(&(file->hdr),0)) != NO_ERROR)) {
    printf("ERROR generating the RIA header file = %s ... err = %d\n",
	   fn,err);
    return err;
  }


  min = 0xffffffff;
  max = 0;
  for (lcv = 0; lcv < file->num_slices; lcv++) {
    if (file->root->status & USE_WINDOW_MIN_MAX) {
      if (file->slices[lcv].min < file->root->win_min)
	file->slices[lcv].min = file->root->win_min;
      if (file->slices[lcv].max > file->root->win_max)
	file->slices[lcv].max = file->root->win_max;
    }
    if (file->slices[lcv].min < min)
      min = file->slices[lcv].min;
    if (file->slices[lcv].max > max)
      max = file->slices[lcv].max;
  }
  file->hdr.data_info.min_pixel = (double) min;
  file->hdr.data_info.max_pixel = (double) max;


  for (lcv = 0; lcv < file->num_slices; lcv++) {
    file->hdr.scans[lcv].num = file->slices[lcv].num;
    file->slices[lcv].offset = file->hdr.scans[lcv].img_addr;
  }

  if ((err = wr_acr_img_data(file)) != NO_ERROR) {
    if (file->root->status & USE_ORIGINAL_BYTES_PER_PIXEL) 
      file->root->scale_bytes = 0;
    return err;
  }

  printf("\nCreated image file \"%s\".\n",file->fn);

  if (file->root->scale_bytes) {
    file->hdr.data_info.min_pixel = (double) file->root->scale_min;
    file->hdr.data_info.max_pixel =  (double) file->root->scale_max;
  }
  if (file->root->status & USE_ORIGINAL_BYTES_PER_PIXEL) 
    file->root->scale_bytes = 0;

  if ((err = wr_the_hdr(fn,&(file->hdr))) != NO_ERROR) {
    printf("ERROR writing the RIA header file = %s ... err = %d\n",fn,err);
    return err;
  }

  printf("Created NRIA header file \"%s\".\n",fn);

  drop_suffix(fn);
  sprintf(hdrfn,"%s.hdr",fn);
  init_mark_hdr(&mark);
  if ((err = assign_nhdr_to_hdr(&mark,&(file->hdr))) != NO_ERROR) {
    printf("ERROR generating the MRVBR header file = %s ... err = %d\n",
	   hdrfn,err);
    return err;
  }
  if (wr_hdr_file(hdrfn,&mark) != NO_ERROR) {
    printf("ERROR writing the MRVBR header file = %s ... err = %d\n",
	   hdrfn,err);
    return err;
  }

  printf("Created ANALYSE header file \"%s\".\n",hdrfn);

  if (file->fid) {
    fclose(file->fid);
    file->fid = NULL;
  }

  printf("Image header contents:\n");
  pr_header(&(file->hdr),1);

  return NO_ERROR;
}


/*---------------------------------------------------------------------------
save_the_file_rec - ???
---------------------------------------------------------------------------*/

int
save_the_file_rec(out)
 struct the_file_rec *out;
{
  unsigned long lcv;
  int err;

  for (lcv = 0; lcv < out->num; lcv++) {
    printf("Writing output file #%d, \"%s\"...\n", lcv + 1,
	   out->files[lcv]->fn);
    if ((err = find_num_echoes_and_echo_time(out->files[lcv])) != NO_ERROR)
      return err;
    save_file_rec(out->files[lcv]);
  }

  return NO_ERROR;
}


/*---------------------------------------------------------------------------
cp_str_data - ???
---------------------------------------------------------------------------*/

void
cp_str_data(dest,src,len)
 unsigned char *dest,*src;
 unsigned long len;
{
 memcpy(dest,src,len);
 dest[len] = 0;
}


/*---------------------------------------------------------------------------
cp_data - ???
---------------------------------------------------------------------------*/

#define SHORT_DATA 2
#define LONG_DATA 3
#define FLOAT_DATA 4

void
cp_data(dest,src,select,which_format)
 unsigned char *dest,*src;
 unsigned long select;
 int which_format;
{
 unsigned short *sh_num;
 unsigned long *num;
 float fl;

 if (select == SHORT_DATA)
    {sh_num = (unsigned short *) dest;
     memcpy(dest,src,2);
     if (which_format == LITTLE_ENDIAN)
        *sh_num = to_dec_short(*sh_num);
    }
   else if (select == LONG_DATA)
    {num = (unsigned long *) dest;
     memcpy(dest,src,4);
     if (which_format == LITTLE_ENDIAN)
        *num = to_dec_long(*num);
    }
   else if (select == FLOAT_DATA)
    {
     memcpy(dest,src,4);
     if (which_format == LITTLE_ENDIAN)
        dec2sun_float(src,dest);
    }
}



/*---------------------------------------------------------------------------
message2ia_hdr - ???
---------------------------------------------------------------------------*/

int
message2ia_hdr( MessagePtr message,
	       struct hdr_rec *hdr,
	       unsigned int *slice_num,
	       char **image_orientation,
	       char **series,
	       unsigned long *in_img_addr,
	       unsigned long *in_img_len )
{
  unsigned char		*data;
  unsigned long		lcv,num,len,num1;
  unsigned short	sh_num;
  unsigned short	*sh_ptr;
  char			str[250];
  float			fl;
  struct data_info_rec	*info;
  int			err;
  unsigned long		min,max;
  ElementPtr		element;


  if (!message) {
    fprintf( stderr, "message2ia_hdr() called with NULL message.\n" );
    return -1;
  }

  TRACE2(printf("message2ia_hdr()\n"));
  info = (struct data_info_rec *) &(hdr->data_info);
  *image_orientation = NULL;
  *series = NULL;


#define getElement( gr, el ) \
  if (!(element = findElement( message, gr, el ))) \
    if (message->error && message->error != ELEMENT_NOTFOUND) \
      return message->error; \
    else if (message->error == ELEMENT_NOTFOUND) \
      printf( "Could not find element %04X.%04X, \"%s\"\n", \
	     gr, el, (findElementDefn( gr, el )->elementName) );
  
  getElement( GstudyInfo, EstudyDate );
  if (element) {
    info->date_of_study = strdup( (const char *) element->value.ptr );
    TRACE2(printf("date_of_study = %s\n",info->date_of_study ));
  }
  getElement( GstudyInfo, EstudyTime );
  if (element) {
    info->time_of_study = strdup( (const char *) element->value.ptr );
    TRACE2(printf("time_of_study = %s\n",info->time_of_study ));
  }
  getElement( GstudyInfo, Emodality );
  if (element) {
    info->image_id = strdup( (const char *) element->value.ptr );
    TRACE2(printf("Eimage_id data = %s\n", info->image_id ));
  }
  getElement( GstudyInfo, EinstitutionName );
  if (element) {
    info->hosp_name = strdup( (const char *) element->value.ptr );
    TRACE2(printf("hosp_name = %s\n", info->hosp_name ));
  }
  getElement( GstudyInfo, EreferringPhysiciansName );
  if (element) {
    info->refered_by = strdup( (const char *) element->value.ptr );
    TRACE2(printf("referred_by = %s\n", info->refered_by ));
  }

  getElement( GpatientInfo, Epatient_name );
  if (element) {
    info->patient.name = strdup( (const char *) element->value.ptr );
    TRACE2(printf("name = %s\n", info->patient.name ));
  }
  getElement( GpatientInfo, Epatient_sex );
  if (element) {
    data = (unsigned char *)
      strpbrk( (const char *) element->value.ptr, "FfMmOo" );
    if (data)
      info->patient.sex = *data;
    else
      info->patient.sex = 'U';
    TRACE2(printf("Epatient_sex = %c\n", info->patient.sex ));
  }

  getElement( GstorageInfo, Estudy_id );
  if (element) {
    info->patient.study_num = strdup( (const char *) element->value.ptr );
    TRACE2(printf("study_num = %s\n", info->patient.study_num ));
  }
  getElement( GstorageInfo, EseriesNumber );
  if (element) {
    *series = strdup( (const char *) element->value.ptr );
    TRACE2(printf("series = %s\n", *series ));
  }

#if 0
  /*  comment is a LT (long text), stored on disk.  */
  getElement( GstorageInfo, Ecomment );
  if (element) {
    info->description = strdup( (const char *) element->value.ptr );
    TRACE2(printf("Ecomment = %s\n", info->description ));
  }
  else {
    TRACE2(printf("Ecomment = <MISSING>\n"));
  }
#endif

  getElement( GphysicalInfo, Eslice_thickness );
  if (element) {
    if (sscanf( (CONST char *) element->value.ptr, "%f",
	       &(info->pixel_thickness) ) == 1) {
      TRACE2(printf("slice thickness = %f\n",info->pixel_thickness));
    }
  }
  getElement( GphysicalInfo, Etr );
  if (element) {
    if (sscanf( (CONST char *) element->value.ptr, "%f",
	       &(info->tr) ) == 1) {
      TRACE2(printf("tr = %f\n",info->tr));
    }
  }
  getElement( GphysicalInfo, Ete );
  if (element) {
    info->te = strdup( (const char *) element->value.ptr );
    TRACE2(printf("te = %s\n", info->te ))
  };
  getElement( GphysicalInfo, Eecho_num );
  if (element &&
      (sscanf( (CONST char *) element->value.ptr, "%d", &num ) == 1)) {
    info->num_echoes = num;
    TRACE2(printf("echo_num = %d\n", info->num_echoes));
  }
  else if (info->description != NULL) {
    if (sscanf( info->description, "NECHO=%d ECHON=%d", &num1, &num ) == 2) {
      info->num_echoes = num;
      TRACE2(printf("Eecho_num = %d\n",info->num_echoes));
    }
  }
  else {
    printf("Error : echo# not found !!!\n");
    return CANCEL;
  }
  getElement( GphysicalInfo, EsliceSpacing );
  if (element) {
    if (sscanf( (CONST char *) element->value.ptr, "%f",
	       &(info->spacing) ) != 1) {
      TRACE2(printf( "Error reading the inter-slice spacing,\n" ););
      TRACE2(printf( "  data = %s, float = %f\n",
		     element->value.ptr, info->spacing ););
    }
    else
      TRACE2(printf("slice_spacing = %f\n",info->spacing ););
  }

  getElement( GimagingInfo, Erows );
  if (element) {
    info->dimy = *((unsigned short *) element->value.ptr);
    TRACE2(printf("dimy = %d\n",info->dimy));
  }
  else
    return CANCEL;
  getElement( GimagingInfo, Ecolumns );
  if (element) {
    info->dimx = *((unsigned short *) element->value.ptr);
    TRACE2(printf("dimx = %d\n",info->dimx));
  }
  else
    return CANCEL;
  getElement( GimagingInfo, EpixelSpacing );
  if (element) {
    if (sscanf( (CONST char *) element->value.ptr, "%f",
	       &(info->xpixel_size) ) != 1) {
      printf("Error reading the xy pixel size...\n");
      return CANCEL;
    }
    info->ypixel_size = info->xpixel_size;
    TRACE2(printf("pixel_spacing = %f\n",info->xpixel_size ););
  }
  else
    return CANCEL;
  getElement( GimagingInfo, EbitsAllocated );
  if (element) {
    sh_num = *((unsigned short *) element->value.ptr);
    TRACE2(printf("Epixel_depth_bits data = %u\n",(unsigned long) sh_num));
    if (sh_num % 8) {
      printf("Sorry...bits per pixel = %d is not a multiple of 8...\n",
	     (int) sh_num);
      printf("Sorry...data cannot be handled !!!\n");
      return CANCEL;
    }
    info->pixel_depth_bits = sh_num;
  }
  else
    return CANCEL;
  getElement( GstorageInfo, Eslice_num );
  if (element) {
    if (sscanf( (CONST char *) element->value.ptr, "%d", &num ) == 1) {
      *slice_num = num;
      TRACE2(printf("Eslice_num = %d\n", *slice_num));
    }
  }
  getElement( GpixelData, Edata );
  if (element) {
    data = (unsigned char *) getValue( message, element );
    get_1d_ary_min_max(data, info->dimx * info->dimy,
		       &min, &max, info->pixel_depth_bits / 8);
    info->min_pixel = (double) min;
    info->max_pixel = (double) max;
    TRACE2(printf("info->min_pixel = %.0f  info->max_pixel = %.0f\n",
		  info->min_pixel,info->max_pixel));
    *in_img_addr = element->value.offset;
    *in_img_len = element->length;
  }
  else
    return CANCEL;

  return err;

  /*
   *  Obsoleted:
   *
   *  Epatient_age
   *  Epatient_weight
   *  Estudy_series
   *  Eorientation
   *    *image_orientation = strdup( (const char *) element->value.ptr );
   *  Eserial_num
   *  Enecho
   */

} /* message2ia_hdr() */



/*---------------------------------------------------------------------------
prog_usage - ???
---------------------------------------------------------------------------*/

void
prog_usage(argc,argv)
     int	argc;
     char	*argv[];
{
  if (argc == 3 || argc == 6)
    return;

  printf( "Usage 1: %s <DICOM.tar.Z> mri\n",argv[0] );
  printf( "Usage 2: %s <DICOM.tar.Z> mri new_size min max\n", argv[0] );
  printf( "\n" );
  printf( "Where:\n" );
  printf( "	<DICOM.tar.Z>	is the DICOM-format image set\n" );
  printf( "			(see data or system administrator\n" );
  printf( "			 for more information)\n" );
  printf( "	new_size	is the desired number of bytes per pixel:\n" );
  printf( "			(1, 2, or 4 bytes per pixel, or\n" );
  printf( "			 0 to retain original pixel width)\n" );
  printf( "	min & max	are the minimum and maximum values\n" );
  printf( "			to which to scale the image data.\n" );
  printf( "\n" );
  printf( "For MRVBR, use new_size = 1, min = 0, max = 250.\n" );
  printf( "\n" );
  printf( "Example: %s \"MR01OC0.30901.002.tar.Z\" mri10903am0\n", argv[0] );
  printf( "   1) input  = MR01OC0.30901.002.tar.Z\n" );
  printf( "               This file is the DICOM-format hup image set\n" );
  printf( "   2) output = mri10903am0.hdr\n" );
  printf( "               ANALYSE-format header file\n" );
  printf( "   3) output = mri10903am0.nhdr\n" );
  printf( "               NRIA-format header\n" );
  printf( "   4) output = mri10903am0.img\n" );
  printf( "               Raw image file\n" );
  printf( "\n\n" );
  exit( 1 );
}


/*---------------------------------------------------------------------------
dp_minmax_range - ???
---------------------------------------------------------------------------*/

void
dp_minmax_range()
{
 printf("\n.... Listing of Range of values based on # of bytes/pixel\n");
 printf("....   0 <= 1 byte/pixel <= 255\n");
 printf("....   0 <= 2 byte/pixel <= 65535\n");
 printf("....   0 <= 4 byte/pixel <= %u\n\n",0xffffffff);

}


/*---------------------------------------------------------------------------
check_minmax_range - ???
---------------------------------------------------------------------------*/

int
check_minmax_range(out)
 struct the_file_rec *out;
{
 unsigned long max;

 if (out->scale_bytes == 1) max = 255;
   else if (out->scale_bytes == 2) max = 65535;
   else if (out->scale_bytes == 4) max = 0xffffffff;

 if (out->scale_min > max)
    {printf("Invalid input : scale_min = %d\n",out->scale_min);
     printf("0 <= scale_min <= %u for %d bytes output\n\n",
                     out->scale_bytes,(unsigned long) max);
     dp_minmax_range();
     return CANCEL;}
 if (out->scale_max > max)
    {printf("Invalid input : scale_max = %d\n",out->scale_max);
     printf("0 <= scale_max <= %u for %d bytes output\n\n",
                     out->scale_bytes,(unsigned long) max);
     dp_minmax_range();
     return CANCEL;}
 if (out->scale_min >= out->scale_max)
    {printf("Invalid input : scale_min = %d >= scale_max = %d\n\n",
            out->scale_min,out->scale_max);
     dp_minmax_range();
     return CANCEL;}
 return NO_ERROR;

}


/*---------------------------------------------------------------------------
get_scale_info - ???
---------------------------------------------------------------------------*/

void
get_scale_info(out,argc,argv)
 struct the_file_rec *out;
 int argc;
 char *argv[];
{
 if (argc > 3)
    {sscanf(argv[3],"%d",&(out->scale_bytes));
     if ((out->scale_bytes != 0) && (out->scale_bytes != 1)
             && (out->scale_bytes != 2) && (out->scale_bytes != 4))
        {printf("Invalid input : scale_to_num_bytes = %d\n",out->scale_bytes);
         printf("   scale_to_num_bytes = 1 or 2 or 4 bytes/pixel!!!\n");
         printf("                      = 0 use same # of bytes/pixel\n");
         exit(1);
        }
     
     sscanf(argv[4],"%d",&(out->scale_min));
     sscanf(argv[5],"%d",&(out->scale_max));
     out->status |= USE_SCALE;
     if (out->scale_bytes != 0)
        {if (check_minmax_range(out) != NO_ERROR) exit(1);
         }
       else out->status |= USE_ORIGINAL_BYTES_PER_PIXEL;
     if (argc == 8)
        {sscanf(argv[6],"%d",&(out->win_min));
         sscanf(argv[7],"%d",&(out->win_max));
         if (out->win_min < out->win_max)
            out->status |= USE_WINDOW_MIN_MAX;
           else
            {printf("...Invalid win_min & win_max values\n");
             printf("...Input : win_min = %d   win_max = %d\n",
                    out->win_min,out->win_max);
             printf("...The values will not be used...\n\n");
            }
        }
    }
}


/*---------------------------------------------------------------------------
dp_errors - ???
---------------------------------------------------------------------------*/

void
dp_errors(log)
 struct strs_rec *log;
{
 unsigned long lcv;

 printf("\n");
 printf("The following list shows the files with processing error...\n");
 printf("These error files are not included in any of the .img files...\n");
 for (lcv = 0; lcv < log->num_strs; lcv++)
    printf("%d) %s",lcv + 1,log->strs[lcv]);
 printf("\n");
}


/*---------------------------------------------------------------------------
main - ???
---------------------------------------------------------------------------*/

void
main(argc,argv)
 int argc;
 char *argv[];
{
  FILE *fid;
  struct hdr_rec hdr;
  unsigned int slice_num;
  int err,err1;
  char **fns;
  unsigned int num_fns;
  unsigned long lcv;

  char *dirfn,fn[200],str[250],spec[250],tapedir[250],
  tar[50],mk_tapedir[50],cd_to_tapedir[50],*temp;

  struct the_file_rec out;
  unsigned long in_img_addr,in_img_len;
  struct strs_rec log;
  unsigned long start_indx;
  int which_format;
  char *image_orientation,*series;

  MessagePtr		message;


  prog_usage(argc,argv);
  init_strs_rec(&log);
  init_the_file_rec(&out);
  get_scale_info(&out,argc,argv);

  if (find_sub_str(argv[1],"/dev/rst",&start_indx) == 0) {
    strcpy(tapedir, (const char *) mktemp("namXXXXXX"));
    if ((temp = (char *) complete_fn(tapedir)) == NULL) {
      printf("Not enough memory...\n");
      exit(~NO_ERROR);
    }
    strcpy(tapedir,temp);
    c_free(temp);
    sprintf(mk_tapedir,"mkdir %s\n",tapedir);
    sprintf(cd_to_tapedir,"cd %s\n",tapedir);
    sprintf(tar,"tar xvf %s\n",argv[1]);
    if ((err = setup_batch_file("unalias rm",mk_tapedir,cd_to_tapedir,tar,0))
	!= NO_ERROR) {
      printf("Failure to extract data from the tape...\n");
      exit(err);
    }
    sprintf(spec,"%s/*",tapedir);
  }
  else {
    tapedir[0] = '\0';
    sprintf(spec,"%s",argv[1]);
  }

  dirfn = (char *) locate_dirfn(spec);
  drop_dirfn(spec);
  if (dirfn == NULL) dirfn = ".";
  if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,spec,
		     NULL,0,0,NULL)) != NO_ERROR) {
    printf("Sorry...get file name error...err = %d\n",err);
    exit(err);
  }
  if (num_fns == 0) {
    printf("Sorry...No file names...\n");
    sprintf(fn,"%s/%s",dirfn,spec);
    printf("...Fn specification processed = %s\n",fn);
    if (tapedir[0] != '\0') {
      printf("...Error !!! Unable to process the data...\n");
      printf("...If the data has beed stored on the tape with a\n");
      printf("directory file name specification, please tar\n");
      printf("your data to the disk first, cd to that directory, and\n");
      printf("then use argv[0] again to process the data !!!\n");
    }
    free_the_file_rec(&out);
    free_strs_rec(&log);
    exit(1);
  }

  printf("# of files found: %d\n",num_fns);
  for (lcv = 0; lcv < num_fns; lcv++) {

    sprintf(fn,"%s/%s",dirfn,fns[lcv]);
    printf("Reading in \"%s\"      \r",fn);
    fflush(stdout);

    message = msgOpen( fn, MSG_READ );
    if (getElements( message )) {
      sprintf( str, "Input file header error (%s)\n", fn );
      add_str_to_strs_rec(&log,str);
      msgClose( message );
      continue;
    }

    /*
     *  Convert file header
     */
    init_hdr_rec(&hdr);
    printf("Converting \"%s\"      \r",fn);
    fflush(stdout);
    if (message2ia_hdr( message, &hdr,
		       &slice_num, &image_orientation, &series,
		       &in_img_addr, &in_img_len )
	!= NO_ERROR) {
      sprintf( str, "Invalid header: %s is excluded.\n", fn );
      add_str_to_strs_rec(&log,str);
      free_hdr_rec(&hdr);
      msgClose( message );
      continue;		/* Attempt next file */
    }
    if ((in_img_addr + in_img_len) > message->size) {
      sprintf( str, "...Invalid file size : %s is excluded...\n", fn );
      add_str_to_strs_rec(&log,str);
      free_hdr_rec(&hdr);
      msgClose( message );
      continue;		/* Attempt next file */
    }

    err1 = enter_file( message,
		      &out, &hdr,
		      slice_num, image_orientation, series,
		      argv[2], fn,
		      in_img_addr, in_img_len );
    c_free(image_orientation);
    image_orientation = NULL;
    c_free(series);
    series = NULL;
    if (err1) {
      if (err1 != ~NO_ERROR) {
	err = err1;
	sprintf( str, "...err = %d : %s is excluded...\n", err1, fn );
	add_str_to_strs_rec(&log,str);
      }
      else {
	err = err1;
	sprintf( str, "...err = %d\n", err1 );
	add_str_to_strs_rec(&log,str);
      }
    }

    free_hdr_rec(&hdr);
    msgClose( message );
    if (err != NO_ERROR)
      break;
  }  /*  end of for loop through input files */

  printf("\n");
  if (err == NO_ERROR) {
    printf("Writing                      \r");
    fflush(stdout);
    err = save_the_file_rec(&out);
  }
  if (log.num_strs > 0)
    dp_errors(&log);

  free_the_file_rec(&out);
  free_strs_rec(&log);
  if (tapedir[0] != '\0') {
    sprintf(spec,"rm -R %s",tapedir);
    setup_batch_file("unalias rm",spec,0);
  }
  printf("Done                      \r");
  fflush(stdout);
  exit(0);
}
