/****************************************************************************
dumpndat.c - Dump the contents of an ndat file

Author:  Paul Hughett

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.


   dumpndat  file.ndat

Dumps selected contents from an ndat file in human-readable form,
for debugging.  The selected contents consist of those that I have
needed to dump; you may extend the program as necessary to dump
the components that you need to see, or comment out the ones you
don't care about.
****************************************************************************/

static const char rcsid[] = "$Id: dumpndat.c,v 1.6 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <stdlib.h>

#include "input.h"
#include "output.h"

/* FIXME:  It shouldn't really be necessary to read both major header files
   for nria, but that's the easy way for now.  -PWH 19990706 */
#include "mri_format.h"
#include "nria_procedures.h"


/***
extern int init_output_file_rec(
     struct output_file_rec *output,
     struct input_file_rec *file);
extern int rd_dat_file(char *fn, struct output_file_rec *output);
***/

/* Required by NRIA library if DEBUG defined; controls debugging output */
int debugFlag = 0;


/* dumpndat.c */
int main(int argc, char *argv[]);
void dump_selected_parts(struct output_file_rec *output);
void free_img_rec(struct img_rec *img);
void init_img_rec(struct img_rec *img, struct input_file_rec *file);



/*-------------------------------------------------------------------------
main - ???
-------------------------------------------------------------------------*/

int
main (int argc, char *argv[])
{  struct output_file_rec output;
   struct input_file_rec input;
   int stat;

   /* Initialize the data structures */
   init_output_file_rec(&output, &input);
   init_input_file_rec(&input, NULL);    /* FIXME:  Is NULL valid here? */

   /* Parse arguments to get the input file name */
   if (argc < 2) {
      fprintf(stderr, "Not enough arguments.\n");
      exit(EXIT_FAILURE); }

   /* Read the ndat file into core */
   stat = rd_dat_file(argv[1], &output);
   if (stat != 0) {
      fprintf(stderr, "Error reading the ndat file\n");
      exit(EXIT_FAILURE); }

   /* Look at selected contents from the ndat file */
   dump_selected_parts(&output);

   return 0;
}




/*-------------------------------------------------------------------------
dump_selected_parts - ???

A routine to dump some selected parts of the ndat file read.  Intended
primarily for debugging, to verify that the data read actually makes
sense and that I understand how to interpret it correctly.
-------------------------------------------------------------------------*/

void
dump_selected_parts(struct output_file_rec *output)
{  FILE *out = stdout;
   struct input_file_rec *pifr;
   struct hdr_rec *phr;
   struct slice_info_rec *psir;
   struct histo_params_rec *phpr;
   struct slice_roi_win_rec *srwr1, *srwr2;

   dump_output_file_rec(out, output);

   pifr = output->file;          /* input_file_rec */
   dump_input_file_rec(out, pifr);

   phr = &pifr->hdr;
   dump_hdr_rec(out, phr);

   psir = &pifr->slice_info;
   dump_slice_info_rec_tree(out, psir);

   /*
   for (iroi = 0; iroi < psir->num_sroi_wins; iroi++) {
     dump_slice_roi_win_rec_tree(out, psir->sroi_wins[iroi]);
   }
   */

   phpr = (struct histo_params_rec *)&pifr->histo_params;
   dump_histo_params_rec(out, phpr);

   srwr1 = &phpr->slice_roi_win_1d;
   dump_slice_roi_win_rec_tree(out, srwr1);

   srwr2 = &phpr->slice_roi_win_2d;
   dump_slice_roi_win_rec_tree(out, srwr2);

   return;
}



/*********************************************************************
The following two routines are copied from the NRIA file img_buf_opr.c
with sufficient modifications so that they do not require XView or the
GUI to serve the present purpose.  In particular, free_img_rec is not
needed here, so it is just converted to a stub that will complain if
called; it is called from the NRIA source file input_file_rec.c, which
we need for other functions.  On the other hand, init_img_rec needs no
changes at all.
**********************************************************************/

/*-------------------------------------------------------------------------
free_img_rec - ???
-------------------------------------------------------------------------*/

void
free_img_rec(
   struct img_rec *img)
{  
   fprintf(stderr, "The function free_img_rec is not yet implemented.\n");
   exit(EXIT_FAILURE);
}


/*-------------------------------------------------------------------------
init_img_rec - ???
-------------------------------------------------------------------------*/

void
init_img_rec(
   struct img_rec *img,
   struct input_file_rec *file)
{
   img->file = file;
   img->data = NULL;
   img->num_data = 0;
}
