/****************************************************************************
mv_replace_str.c - Run mv on multiple files.

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

#ifndef lint
static const char rcsid[] = "$Id: mv_replace_str.c,v 1.5 1999/04/30 21:29:04 hughett Exp $ Release $Name: nria_1_0_6 $";
#endif /* lint */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "error_constants.h"
#include "debug.h"
#include "nria_lib.h"



int debugFlag = 0;


/* mv_replace_str.c */
int replace_str_by_str(char **result, char *str, char *replace_str,
    char *replacement_str);
void prog_usage(int argc, char *argv[]);
void main(int argc, char *argv[]);



/*---------------------------------------------------------------------------
replace_str_by_str - ???
---------------------------------------------------------------------------*/

int
replace_str_by_str(result,str,replace_str,replacement_str)
 char **result,*str,*replace_str,*replacement_str;
{
 unsigned int cnt,lcv,len1,len2,posi,indx;
 char ch;
 int err;
 char *str1,*str2;

 *result = NULL;
 len1 = get_strlen(str);
 if (len1 == 0) return NO_ERROR;
 len2 = get_strlen(replace_str);
 if (len2 == 0) return NO_ERROR;
 str1 = NULL;
 for (posi = lcv = 0; lcv < len1; )
    {
     for (cnt = lcv,indx = 0; indx < len2; indx++,cnt++)
        if (str[cnt] != replace_str[indx]) break;
     if (indx == len2)
        {
         if (lcv != posi)
            {str2 = &(str[posi]);
             ch = str[lcv];
             str[lcv] = '\0';}
           else str2 = NULL;
         if ((err = concat_strs(&str1,3,str1,str2,replacement_str)) != NO_ERROR)
            {c_free(str1);
             return err;}
         if (str2 != NULL) str[lcv] = ch;
         posi = lcv = cnt;
        }
       else lcv++;
   }
 if (lcv != posi)
    {str2 = &(str[posi]);
     ch = str[lcv];
     str[lcv] = '\0';
     if ((err = concat_strs(&str1,2,str1,str2)) != NO_ERROR)
       {c_free(str1);
        return err;}
     str[lcv] = ch;
    }
 *result = str1;
 return NO_ERROR;
}


/*---------------------------------------------------------------------------
prog_usage - ???
---------------------------------------------------------------------------*/

void
prog_usage(argc,argv)
 int argc;
 char *argv[];
{
 if ((argc != 4) && (argc != 5))
    {printf("Usage : %s str replace_str replacement_str [perform_mv]\n",
            argv[0]);
     exit(1);}
}



/*---------------------------------------------------------------------------
main - ???
---------------------------------------------------------------------------*/

void
main(argc,argv)
 int argc;
 char *argv[];
{
 int err;
 char **fns,**dirs;
 unsigned int num_fns,num_dirs;
 unsigned long lcv;
 char *dirfn,fn[200];
 char *result;
 int is_mv;
 struct stat statbuf;

 prog_usage(argc,argv);
 if (argc == 5) sscanf(argv[4],"%d",&is_mv);
   else is_mv = 0;
 dirfn = (char *) locate_dirfn(argv[1]);
 drop_dirfn(argv[1]);
 if (dirfn == NULL) dirfn = ".";
 if ((err = get_fns(dirfn,&dirs,&num_dirs,1,argv[1],&fns,&num_fns,1,argv[1],
         NULL,0,0,NULL)) != NO_ERROR) exit(err);

 err = CANCEL;
 if ((num_fns == 0) && (num_dirs == 0))
    {printf("Sorry...No file names...\n");
     sprintf(fn,"%s/%s",dirfn,argv[1]);
     printf("...Fn specification processed = %s\n",fn);
    }
   else
    {printf("\n");
     for (lcv = 0; lcv < num_fns; lcv++)
        {sprintf(fn,"%s/%s",dirfn,fns[lcv]);
/*         printf("%d) reading %s\n",lcv,fn); */
         if ((err = replace_str_by_str(&result,fn,argv[2],argv[3]))
              != NO_ERROR)
            exit(err);
         
         if (strcmp( result, fn ) == 0)
	   continue;

         if (is_mv) {
	   /* Check to see if the new filename exists yet.  If so,
	      do not overwrite it! DRW 960402 */
	   if (stat(result,&statbuf)) {
	     if (errno != ENOENT) {
	       printf("%s not renamed; could not stat %s.\n",fn,result);
	     }
	     else {
	       rename(fn,result);
	       printf("%s renamed to %s\n",fn,result);
	     }
	   }
	   else {
	     printf("%s not renamed; %s already exists.\n",fn,result);
	   }
	 }
	 else
	   printf("mv %s %s\n",fn,result);

         c_free(result);
        }
     /*     printf("\nnum_dirs = %d\n",num_dirs); */
     for (lcv = 0; lcv < num_dirs; lcv++)
        {
	 if (strcmp(dirs[lcv],".") == 0)
	   continue;
	 if (strcmp(dirs[lcv],"..") == 0)
	   continue;
	 sprintf(fn,"%s/%s",dirfn,dirs[lcv]);
	/*         printf("%d) reading %s\n",lcv,fn); */
         if ((err = replace_str_by_str(&result,fn,argv[2],argv[3]))
              != NO_ERROR)
            exit(err);

         if (strcmp( result, fn ) == 0)
	   continue;

         if (is_mv)
            {
	   /* Check to see if the new filename exists yet.  If so,
	      do not overwrite it! DRW 960402 */
	   if (stat(result,&statbuf)) {
	     if (errno != ENOENT) {
	       printf("%s not renamed; could not stat %s.\n",fn,result);
	     }
	     else {
	       rename(fn,result);
	       printf("%s renamed to %s\n",fn,result);
	     }
	   }
	   else {
	     printf("%s not renamed; %s already exists.\n",fn,result);
	   }
            }
           else printf("mv %s %s\n",fn,result);
         c_free(result);
        }
    }
 exit(0);
}
