/****************************************************************************
ntxt2sas.c - ???

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

static const char rcsid[] = "$Id: ntxt2sas.c,v 1.6 1999/07/07 18:07:14 hughett Exp $ Release $Name: nria_1_0_6 $";

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>

#include "error_constants.h"
#include "menu_items.h"
#include "nria_lib.h"


#define NO_ERROR 0

/* ntxt2sas.c */
int process_and_output(FILE *fid, FILE *outfid, int rater, char *mhcrc_num);
static int get_mhcrc_num(char *fn, char *mhcrc_num);
int process_txt_and_output(char *fn, int rater);
void main(int argc, char *argv[]);



/* Global variables */
int debugFlag = 0;          /* Controls debugging output from library */


/*------------------------------------------------------------------------
process_and_output - ???
------------------------------------------------------------------------*/

#define STRLEN 250

int
process_and_output(fid,outfid,rater,mhcrc_num)
 FILE *fid,*outfid;
 int rater;
 char *mhcrc_num;
{
 float num;
 int num_slices,cnt;
 int err;
 char line[STRLEN];
 unsigned long pat_start,pat_stop,token_num;
 char *next;
 float xpix,ypix,thickness,voxel;
 int time;

 if ((err = rd_until(fid,line,STRLEN,"(X,Y)",&pat_start,&pat_stop,&token_num))
        != NO_ERROR)
    return err;
 if ((next = (char *) next_token(line,0,2)) == NULL) return CANCEL;
 sscanf(next,"%fmm,%f",&xpix,&ypix);
 if ((next = (char *) next_token(line,0,3)) == NULL) return CANCEL;
 sscanf(next,"%f",&thickness);
 if ((err = rd_until(fid,line,STRLEN,"VOXEL",&pat_start,&pat_stop,&token_num))
        != NO_ERROR)
    return err;

 if ((next = (char *) next_token(line,0,2)) == NULL) return CANCEL;
 sscanf(next,"%f",&voxel);

 if ((err = rd_until(fid,line,STRLEN,"type",&pat_start,&pat_stop,&token_num))
        != NO_ERROR)
    return err;
 if ((next = (char *) next_token(line,0,3)) == NULL) return CANCEL;
 sscanf(next,"%d",&num_slices);

 time = 0;
 cnt = 0;
 do 
    {if ((err = rd_until(fid,line,STRLEN,"VOL(cc)",
          &pat_start,&pat_stop,&token_num))
           != NO_ERROR)
        {if (time > 0)
            {fprintf(outfid,"\n");
             return NO_ERROR;}
         return err;}
     if (time == 0)
        {fprintf(outfid,"%d %s %d\n",rater,mhcrc_num,num_slices);
         fprintf(outfid,"%.4f %.4f %.4f %.4f\n",xpix,ypix,thickness,voxel);}

     if ((next = (char *) next_token(line,1,0)) != NULL)
        {for (; ;)
            {
             if ((next = (char *) next_token(line,0,0)) == NULL) break;

             sscanf(next,"%f",&num);
             fprintf(outfid,"%.2f ",num);
             if (cnt == 5)
                {fprintf(outfid,"\n");
                 cnt = 0;}
               else cnt += 1;
            }
         time += 1;
        }
    }
 while (1);    /* FIXME:  This sure looks like a bug of some kind */
 fprintf(outfid,"\n");
 return NO_ERROR;
}



/*------------------------------------------------------------------------
get_mhcrc_num - ???

FIXME!  There is a routine of the same name but different calling
sequence and contents defined in fn.c
------------------------------------------------------------------------*/

static int
get_mhcrc_num(fn,mhcrc_num)
 char *fn;
 char *mhcrc_num;
{
 unsigned int first,last,indx,lcv;

 mhcrc_num[0] = '\0';
 if (locate_next_str_num(fn,&first,1) == NO_ERROR)
    {last = first;
     while (locate_next_str_num(&(fn[last + 1]),&indx,0) == NO_ERROR)
        last = last + 1 + indx;
     for (lcv = 0,indx = first; indx <= last; indx++,lcv++)
        mhcrc_num[lcv] = fn[indx];
     mhcrc_num[lcv] = '\0';
     return NO_ERROR;
    }

 return NOT_FOUND;
}


/*------------------------------------------------------------------------
process_txt_and_output - ???
------------------------------------------------------------------------*/

int
process_txt_and_output(fn,rater)
 char *fn;
 int rater;
{
 FILE *fid,*outfid;
 char outfn[250];
 int err;
 char mhcrc_num[100];

 if ((fid = fopen(fn,"r")) == NULL)
    {printf("Error !!! : Unable to open up the input file %s\n",fn);
     return ~NO_ERROR;}

 strcpy(outfn,fn);
 drop_suffix(outfn);
 drop_dirfn(outfn);
 strcpy(&(outfn[strlen(outfn)]),".out");
 if ((outfid = fopen(outfn,"w")) == NULL)
    {printf("Error !!! : Unable to open up the output file %s\n",outfn);
     fclose(fid);
     return ~NO_ERROR;}
 if (get_mhcrc_num(fn,mhcrc_num) != NO_ERROR)
    printf("   *** warning : the mhcrc# was not found...\n");

 err = process_and_output(fid,outfid,rater,mhcrc_num);
 fclose(fid);
 fclose(outfid);
 if (err != NO_ERROR)
    {unlink(outfn);
     printf("Error = %d...Operation aborted !!!\n",err);}
   else printf("output fn = %s\n",outfn);
/*
 printf("err = %d\n",err);
*/
 return err;

}


/*------------------------------------------------------------------------
main - ???
------------------------------------------------------------------------*/

void
main(argc,argv)
 int argc;
 char *argv[];
{
 int rater;
 int err;
 char **fns;
 unsigned int num_fns;
 unsigned long lcv;
 char *dirfn,fn[200];
         
 if (argc != 3)
    {printf("\nUsage : %s fn rater\n",argv[0]);
     printf("Example 1 : %s srmri10072a.txt 1\n",argv[0]);
     printf("   Output ==> srmri10072a.out\n\n");
     printf("\n");
     printf("Example 2 : %s \"*.txt\" 1\n",argv[0]);
     printf("   Output ==> *.out\n\n");
     printf("\n");
     printf("Note : if you use '*' in the file specification, please\n");
     printf("       enclose that file specification by double quotes.\n");
     exit(0);}
 sscanf(argv[2],"%d",&rater);
 dirfn = (char *) locate_dirfn(argv[1]);
 drop_dirfn(argv[1]);
 if (dirfn == NULL) dirfn = ".";
 if ((err = get_fns(dirfn,NULL,0,0,NULL,&fns,&num_fns,1,argv[1],
         NULL,0,0,NULL)) != NO_ERROR) exit(err);
/*
 printf("the_fn = %s dirfn = %s fn = %s argv[1] = %s\n",
      the_fn,dirfn,fn,argv[1]);
*/
 if (num_fns == 0)
    {printf("Sorry...No file names...\n");
     sprintf(fn,"%s/%s",dirfn,argv[1]);
     printf("...Fn specification processed = %s\n",fn);
    }
   else
    {printf("\n");
     for (lcv = 0; lcv < num_fns; lcv++)
        {sprintf(fn,"%s/%s",dirfn,fns[lcv]);
         printf("%ld) reading %s\n",lcv,fn);
         if ((err = process_txt_and_output(fn,rater)) != NO_ERROR)
            exit(err);
        }
    }
}
