/****************************************************************************
pr_ia_hdr.c - Print out NRIA header information

Copyright (c) 1998 Trustees of the University of Pennsylvania
See the file COPYING for license terms.
****************************************************************************/

#ifndef lint
static const char rcsid[] = "$Id: pr_ia_hdr.c,v 1.9 1999/04/30 21:29:04 hughett Exp $ Release $Name: nria_1_0_6 $";
#endif


#include <stdio.h>
#include <stdlib.h>
#include <grp.h>
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>

#include "mri_format.h"
#include "typedef.h"
#include "ia_hdr.h"
#include "error_constants.h"
#include "dates.h"
#include "nria_lib.h"

extern unsigned long NumberOfSliceScans();
extern unsigned long NumberOfSliceCuts();


/* pr_ia_hdr.c */
void rotate(char *ch);
void pr_data_info_rec(struct hdr_rec *hdr, int allinfo);
void pr_scan_entry_info(struct hdr_scan_entry_rec *scans,
    unsigned int num_scans);
void pr_hdr_rec(struct hdr_rec *hdr, int allinfo);
int sysadminuser(void);
void showusage(char *argv0);
int main(int argc, char *argv[]);


#define CARRET printf("\n")
#define COMMA  printf(", ")
#define FORCESTRING(x) (strlen(x) ? (x) : "(empty)")

static int restricted = 1;



/*--------------------------------------------------------------------------
rotate - ???

FIXME: The rotate function appears in both pr_ia_hdr.c and 
examine_hdrs.c; it really ought to be a common library routine.
--------------------------------------------------------------------------*/

void
rotate( char *ch )
{
  char nums[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789nopqrstuvwxyzabcdefghijklmNOPQRSTUVWXYZABCDEFGHIJKLM!@#{}&()[]";
  char *index;
  int numchars = sizeof(nums) - 1;
  int rotateby = numchars / 2;


  while (*ch) {
    if (index = strchr( nums, *ch ))
      *ch = nums[((index - nums) + rotateby) % numchars];
    ch++;
  }
}


/*--------------------------------------------------------------------------
pr_data_info_rec - ???
---------------------------------------------------------------------------*/

void
pr_data_info_rec( struct hdr_rec * hdr, int allinfo )
{
  int tmpint;
  struct data_info_rec * info = &(hdr->data_info);

  if (sscanf( info->patient.patient_num, "%d", &tmpint ) != 1)
    printf( "BBLID  = 00000" );
  else
    printf( "BBLID  = %0.5d", tmpint );
  COMMA;
  if (sscanf( info->patient.study_num, "%d", &tmpint ) != 1)
    printf( "Study number  = 00000000" );
  else
    printf( "Study number  = %0.8d", tmpint );
  COMMA;
  printf("DOMRI  = %s", canonizeDate(info->date_of_study, NULL));
  CARRET;

  if (allinfo) {
    if (hdr->comment_size == 24 && hdr->comment &&
	*(hdr->comment) == '$' && *(hdr->comment + 1) == '$') {
      rotate( hdr->comment );
      printf( "*BBLID = %.5s", (hdr->comment + 2) );
      COMMA;
      printf( "*Study number = %.8s", (hdr->comment + 7) );
      COMMA;
      printf( "*DOMRI = %.8s", (hdr->comment + 15) );
      CARRET;
    }
  }

  CARRET;

  printf("Pixel size (mm):  ");
  printf("x = %.4f", info->xpixel_size);
  COMMA;
  printf("y = %.4f", info->ypixel_size);
  COMMA;
  printf("z = %.4f", info->pixel_thickness);
  CARRET;
  printf("# echoes = %d", info->num_echoes);
  COMMA;
  printf("# slices = %ld", NumberOfSliceCuts(info));
  COMMA;
  printf("Total # slices = %d", (int) info->max_scans);
  CARRET;

  CARRET;

  printf("Image size: %d x %d pixels", (int) info->dimx, (int) info->dimy);
  COMMA;
  printf("\"%d-bit data\" or # Bytes/Pixel: %d",
	 (int) info->pixel_depth_bits, (int) (info->pixel_depth_bits / 8));
  CARRET;
  printf("Pixel values:  ");
  printf("Min = %.0f", info->min_pixel);
  COMMA;
  printf("Max = %.0f", info->max_pixel);
  CARRET;
  printf("Data type:  ");
  switch (info->var_use_status & INPUT_DATA_TYPE) {
  case INPUT_DATA_SIGNED:
    printf("Signed"); break;
  case INPUT_DATA_FLOAT:
    printf("Float"); break;
  case INPUT_DATA_DOUBLE:
    printf("Double Float"); break;
  default:
    printf("Unsigned Integer");
  }
  CARRET;

  CARRET;

  printf("Slice gap = %.4f", info->spacing);
  COMMA;
  printf("T(relax.) = %.0f ms", info->tr);
  COMMA;
  printf("T(echo) = %s ms", FORCESTRING(info->te));
  CARRET;
}




/*--------------------------------------------------------------------------
pr_scan_entry_info - ???
---------------------------------------------------------------------------*/

void
pr_scan_entry_info( struct hdr_scan_entry_rec *scans,
		    unsigned int num_scans )
{
  unsigned int lcv;
  struct hdr_scan_entry_rec *slice;

  printf("\n");
  for (lcv = 0; lcv < num_scans; lcv += 1)
  {
    printf("%d) ",lcv + 1);
    slice = &scans[lcv];
    printf( "status: %lx label: %s num: %ld img_addr: %lu ",
	    (unsigned long) slice->status,
	    slice->img_label,
	    slice->num,
	    slice->img_addr );
    printf( "prior = %p next = %p par = %p\n",
	    slice->prior_slice,
	    slice->next_slice,
	    slice->partner_slice );
  }
}



/*--------------------------------------------------------------------------
pr_hdr_rec - ???
---------------------------------------------------------------------------*/

void
pr_hdr_rec( struct hdr_rec *hdr, int allinfo )
{
  int lcv;

  printf("\n");
  pr_data_info_rec( hdr, allinfo );

  printf("Echo labels:  ");
  for (lcv = 0; lcv < hdr->analysis.num_img_labels; lcv++) {
    if (lcv > 0) {
      printf(", ");
    }
    printf("\"%s\"", hdr->analysis.img_labels[lcv]);
  }
  printf("\n");
}



/*--------------------------------------------------------------------------
sysadminuser - ???

If user is in sysadmin group, return 1, otherwise return 0.

FIXME:  I think this is a duplicated function that ought to move into
a shared library.  -PWH 19990430
---------------------------------------------------------------------------*/

int
sysadminuser( void )
{
  struct group *groupptr;
  struct passwd *passwdptr;
  char **groupmem;
  char *username;
  uid_t myuid;


  myuid = getuid();

  setpwent();
  passwdptr = getpwuid( myuid );
  username = passwdptr->pw_name;
  endpwent();

  setgrent();
  groupptr = getgrnam( "sysadmin" );
  groupmem = groupptr->gr_mem;
  endgrent();

  while (*groupmem && (strcmp(*groupmem,username) != 0)) {
    groupmem++;
  }

  if (!(*groupmem)) {
    return 0;
  }

  return 1;
}



/*--------------------------------------------------------------------------
showusage - ???
---------------------------------------------------------------------------*/

void
showusage( char *argv0 )
{
  if (restricted) {
    printf( "Usage: %s [ -l ] nria_header [...]\n",argv0);
    printf( "\n-l = long format (show all slice information)\n" );
  }
  else {
    printf( "Usage: %s [ -l ] [ -h ] nria_header [...]\n", argv0 );
    printf( "\n-l = long format (show all slice information)\n" );
    printf( "\n-h = show hidden information\n" );
  }
  printf("Example : %s mri10010a.nhdr\n", argv0);
}



/*--------------------------------------------------------------------------
main - ???
---------------------------------------------------------------------------*/

int
main( int argc, char *argv[] )
{
  int err;
  struct hdr_rec hdr;
  int c;
  int longformat = 0;
  int allinfo = 0;
  const char resoptions[] = "l";
  const char alloptions[] = "lh";
  const char *options = resoptions;
  int retval = 0;


  if (sysadminuser())
    restricted = 0;

  if (!restricted) {
    options = alloptions;
  }

  while ((c = getopt( argc, argv, options )) != EOF) {
    switch (c) {

    case 'l':
      longformat = 1;
      break;

    case 'h':
      allinfo = 1;
      break;

    case '?':
      showusage( argv[0] );
      exit( 1 );

    }
  }

  if (optind == argc) {
    showusage( argv[0] );
    exit( 0 );
  }

  while (optind < argc) {

    if ((err = rd_the_hdr(argv[optind],&hdr)) != NO_ERROR) {
      fprintf( stderr, "Error reading \"%s\".\n", argv[optind] );
      optind++;
      retval = 1;
      continue;
    }

    printf( "\n%s:\n", argv[optind] );
    pr_hdr_rec( &hdr, allinfo );

    if (longformat) {
      slice_entry_addr_to_index( hdr.scans, hdr.num_scans );
      pr_scan_entry_info( hdr.scans, hdr.num_scans );
    }
    printf( "\n" );

    optind++;
  }

  exit(0);
}
